@@ -7,11 +7,11 @@
 #ifndef mozilla_a11y_Accessible_inl_h_
 #define mozilla_a11y_Accessible_inl_h_
 
-#include "nsAccessible.h"
+#include "Accessible.h"
 #include "nsARIAMap.h"
 
 inline mozilla::a11y::role
-nsAccessible::Role()
+Accessible::Role()
 {
   if (!mRoleMapEntry || mRoleMapEntry->roleRule != kUseMapRole)
     return ARIATransformRole(NativeRole());
@@ -20,7 +20,7 @@ nsAccessible::Role()
 }
 
 inline mozilla::a11y::role
-nsAccessible::ARIARole()
+Accessible::ARIARole()
 {
   if (!mRoleMapEntry || mRoleMapEntry->roleRule != kUseMapRole)
     return mozilla::a11y::roles::NOTHING;
