@@ -1,53 +1,21 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Author: Eric Vaughan (evaughan@netscape.com)
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "HTMLFormControlAccessible.h"
 
 #include "Accessible-inl.h"
 #include "nsAccUtils.h"
 #include "nsARIAMap.h"
+#include "nsEventShell.h"
 #include "nsTextEquivUtils.h"
 #include "Relation.h"
 #include "Role.h"
 #include "States.h"
 
+#include "nsContentList.h"
 #include "nsIAccessibleRelation.h"
-#include "nsIDOMDocument.h"
 #include "nsIDOMHTMLInputElement.h"
 #include "nsIDOMNSEditableElement.h"
 #include "nsIDOMHTMLFormElement.h"
@@ -55,6 +23,7 @@
 #include "nsIDOMHTMLTextAreaElement.h"
 #include "nsIDOMNodeList.h"
 #include "nsIEditor.h"
+#include "nsIFormControl.h"
 #include "nsIFrame.h"
 #include "nsINameSpaceManager.h"
 #include "nsISelectionController.h"
@@ -63,6 +32,9 @@
 #include "nsIServiceManager.h"
 #include "nsITextControlFrame.h"
 
+#include "mozilla/Preferences.h"
+
+using namespace mozilla;
 using namespace mozilla::a11y;
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -70,8 +42,8 @@ using namespace mozilla::a11y;
 ////////////////////////////////////////////////////////////////////////////////
 
 HTMLCheckboxAccessible::
-  HTMLCheckboxAccessible(nsIContent* aContent, nsDocAccessible* aDoc) :
-  nsLeafAccessible(aContent, aDoc)
+  HTMLCheckboxAccessible(nsIContent* aContent, DocAccessible* aDoc) :
+  LeafAccessible(aContent, aDoc)
 {
 }
 
@@ -81,18 +53,18 @@ HTMLCheckboxAccessible::NativeRole()
   return roles::CHECKBUTTON;
 }
 
-PRUint8
+uint8_t
 HTMLCheckboxAccessible::ActionCount()
 {
   return 1;
 }
 
 NS_IMETHODIMP
-HTMLCheckboxAccessible::GetActionName(PRUint8 aIndex, nsAString& aName)
+HTMLCheckboxAccessible::GetActionName(uint8_t aIndex, nsAString& aName)
 {
   if (aIndex == eAction_Click) {    // 0 is the magic value for default action
     // cycle, check or uncheck
-    PRUint64 state = NativeState();
+    uint64_t state = NativeState();
 
     if (state & states::CHECKED)
       aName.AssignLiteral("uncheck"); 
@@ -107,7 +79,7 @@ HTMLCheckboxAccessible::GetActionName(PRUint8 aIndex, nsAString& aName)
 }
 
 NS_IMETHODIMP
-HTMLCheckboxAccessible::DoAction(PRUint8 aIndex)
+HTMLCheckboxAccessible::DoAction(uint8_t aIndex)
 {
   if (aIndex != 0)
     return NS_ERROR_INVALID_ARG;
@@ -116,10 +88,10 @@ HTMLCheckboxAccessible::DoAction(PRUint8 aIndex)
   return NS_OK;
 }
 
-PRUint64
+uint64_t
 HTMLCheckboxAccessible::NativeState()
 {
-  PRUint64 state = nsLeafAccessible::NativeState();
+  uint64_t state = LeafAccessible::NativeState();
 
   state |= states::CHECKABLE;
   bool checkState = false;   // Radio buttons and check boxes can be checked or mixed
@@ -157,20 +129,19 @@ HTMLCheckboxAccessible::IsWidget() const
 ////////////////////////////////////////////////////////////////////////////////
 
 HTMLRadioButtonAccessible::
-  HTMLRadioButtonAccessible(nsIContent* aContent, nsDocAccessible* aDoc) :
+  HTMLRadioButtonAccessible(nsIContent* aContent, DocAccessible* aDoc) :
   RadioButtonAccessible(aContent, aDoc)
 {
 }
 
-PRUint64
+uint64_t
 HTMLRadioButtonAccessible::NativeState()
 {
-  PRUint64 state = nsAccessibleWrap::NativeState();
+  uint64_t state = AccessibleWrap::NativeState();
 
   state |= states::CHECKABLE;
-  
-  bool checked = false;   // Radio buttons and check boxes can be checked
 
+  bool checked = false;   // Radio buttons and check boxes can be checked
   nsCOMPtr<nsIDOMHTMLInputElement> htmlRadioElement =
     do_QueryInterface(mContent);
   if (htmlRadioElement)
@@ -183,11 +154,10 @@ HTMLRadioButtonAccessible::NativeState()
 }
 
 void
-HTMLRadioButtonAccessible::GetPositionAndSizeInternal(PRInt32* aPosInSet,
-                                                      PRInt32* aSetSize)
+HTMLRadioButtonAccessible::GetPositionAndSizeInternal(int32_t* aPosInSet,
+                                                      int32_t* aSetSize)
 {
-  nsAutoString nsURI;
-  mContent->NodeInfo()->GetNamespaceURI(nsURI);
+  int32_t namespaceId = mContent->NodeInfo()->NamespaceID();
   nsAutoString tagName;
   mContent->NodeInfo()->GetName(tagName);
 
@@ -196,43 +166,30 @@ HTMLRadioButtonAccessible::GetPositionAndSizeInternal(PRInt32* aPosInSet,
   nsAutoString name;
   mContent->GetAttr(kNameSpaceID_None, nsGkAtoms::name, name);
 
-  nsCOMPtr<nsIDOMNodeList> inputs;
+  nsRefPtr<nsContentList> inputElms;
 
-  nsCOMPtr<nsIDOMHTMLInputElement> radio(do_QueryInterface(mContent));
-  nsCOMPtr<nsIDOMHTMLFormElement> form;
-  radio->GetForm(getter_AddRefs(form));
-  if (form) {
-    form->GetElementsByTagNameNS(nsURI, tagName, getter_AddRefs(inputs));
-  } else {
-    nsIDocument* doc = mContent->OwnerDoc();
-    nsCOMPtr<nsIDOMDocument> document(do_QueryInterface(doc));
-    if (document)
-      document->GetElementsByTagNameNS(nsURI, tagName, getter_AddRefs(inputs));
-  }
+  nsCOMPtr<nsIFormControl> formControlNode(do_QueryInterface(mContent));
+  dom::Element* formElm = formControlNode->GetFormElement();
+  if (formElm)
+    inputElms = NS_GetContentList(formElm, namespaceId, tagName);
+  else
+    inputElms = NS_GetContentList(mContent->OwnerDoc(), namespaceId, tagName);
+  NS_ENSURE_TRUE(inputElms, );
 
-  NS_ENSURE_TRUE(inputs, );
-
-  PRUint32 inputsCount = 0;
-  inputs->GetLength(&inputsCount);
+  uint32_t inputCount = inputElms->Length(false);
 
   // Compute posinset and setsize.
-  PRInt32 indexOf = 0;
-  PRInt32 count = 0;
-
-  for (PRUint32 index = 0; index < inputsCount; index++) {
-    nsCOMPtr<nsIDOMNode> itemNode;
-    inputs->Item(index, getter_AddRefs(itemNode));
-
-    nsCOMPtr<nsIContent> item(do_QueryInterface(itemNode));
-    if (item &&
-        item->AttrValueIs(kNameSpaceID_None, nsGkAtoms::type,
-                          type, eCaseMatters) &&
-        item->AttrValueIs(kNameSpaceID_None, nsGkAtoms::name,
-                          name, eCaseMatters)) {
+  int32_t indexOf = 0;
+  int32_t count = 0;
 
+  for (uint32_t index = 0; index < inputCount; index++) {
+    nsIContent* inputElm = inputElms->Item(index, false);
+    if (inputElm->AttrValueIs(kNameSpaceID_None, nsGkAtoms::type,
+                              type, eCaseMatters) &&
+        inputElm->AttrValueIs(kNameSpaceID_None, nsGkAtoms::name,
+                              name, eCaseMatters)) {
       count++;
-
-      if (item == mContent)
+      if (inputElm == mContent)
         indexOf = count;
     }
   }
@@ -246,19 +203,19 @@ HTMLRadioButtonAccessible::GetPositionAndSizeInternal(PRInt32* aPosInSet,
 ////////////////////////////////////////////////////////////////////////////////
 
 HTMLButtonAccessible::
-  HTMLButtonAccessible(nsIContent* aContent, nsDocAccessible* aDoc) :
-  nsHyperTextAccessibleWrap(aContent, aDoc)
+  HTMLButtonAccessible(nsIContent* aContent, DocAccessible* aDoc) :
+  HyperTextAccessibleWrap(aContent, aDoc)
 {
 }
 
-PRUint8
+uint8_t
 HTMLButtonAccessible::ActionCount()
 {
   return 1;
 }
 
 NS_IMETHODIMP
-HTMLButtonAccessible::GetActionName(PRUint8 aIndex, nsAString& aName)
+HTMLButtonAccessible::GetActionName(uint8_t aIndex, nsAString& aName)
 {
   if (aIndex == eAction_Click) {
     aName.AssignLiteral("press"); 
@@ -268,7 +225,7 @@ HTMLButtonAccessible::GetActionName(PRUint8 aIndex, nsAString& aName)
 }
 
 NS_IMETHODIMP
-HTMLButtonAccessible::DoAction(PRUint8 aIndex)
+HTMLButtonAccessible::DoAction(uint8_t aIndex)
 {
   if (aIndex != eAction_Click)
     return NS_ERROR_INVALID_ARG;
@@ -277,10 +234,10 @@ HTMLButtonAccessible::DoAction(PRUint8 aIndex)
   return NS_OK;
 }
 
-PRUint64
+uint64_t
 HTMLButtonAccessible::State()
 {
-  PRUint64 state = nsHyperTextAccessibleWrap::State();
+  uint64_t state = HyperTextAccessibleWrap::State();
   if (state == states::DEFUNCT)
     return state;
 
@@ -288,7 +245,7 @@ HTMLButtonAccessible::State()
   // no special processing for unavailable state since inheritance is supplied
   // other code paths.
   if (mParent && mParent->IsHTMLFileInput()) {
-    PRUint64 parentState = mParent->State();
+    uint64_t parentState = mParent->State();
     state |= parentState & (states::BUSY | states::REQUIRED |
                             states::HASPOPUP | states::INVALID);
   }
@@ -296,10 +253,10 @@ HTMLButtonAccessible::State()
   return state;
 }
 
-PRUint64
+uint64_t
 HTMLButtonAccessible::NativeState()
 {
-  PRUint64 state = nsHyperTextAccessibleWrap::NativeState();
+  uint64_t state = HyperTextAccessibleWrap::NativeState();
 
   nsEventStates elmState = mContent->AsElement()->State();
   if (elmState.HasState(NS_EVENT_STATE_DEFAULT))
@@ -317,7 +274,7 @@ HTMLButtonAccessible::NativeRole()
 nsresult
 HTMLButtonAccessible::GetNameInternal(nsAString& aName)
 {
-  nsAccessible::GetNameInternal(aName);
+  Accessible::GetNameInternal(aName);
   if (!aName.IsEmpty() || mContent->Tag() != nsGkAtoms::input)
     return NS_OK;
 
@@ -362,14 +319,13 @@ HTMLButtonAccessible::IsWidget() const
 ////////////////////////////////////////////////////////////////////////////////
 
 HTMLTextFieldAccessible::
-  HTMLTextFieldAccessible(nsIContent* aContent, nsDocAccessible* aDoc) :
-  nsHyperTextAccessibleWrap(aContent, aDoc)
+  HTMLTextFieldAccessible(nsIContent* aContent, DocAccessible* aDoc) :
+  HyperTextAccessibleWrap(aContent, aDoc)
 {
 }
 
-NS_IMPL_ISUPPORTS_INHERITED3(HTMLTextFieldAccessible,
-                             nsAccessible,
-                             nsHyperTextAccessible,
+NS_IMPL_ISUPPORTS_INHERITED2(HTMLTextFieldAccessible,
+                             Accessible,                             
                              nsIAccessibleText,
                              nsIAccessibleEditableText)
 
@@ -387,7 +343,7 @@ HTMLTextFieldAccessible::NativeRole()
 nsresult
 HTMLTextFieldAccessible::GetNameInternal(nsAString& aName)
 {
-  nsresult rv = nsAccessible::GetNameInternal(aName);
+  nsresult rv = Accessible::GetNameInternal(aName);
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (!aName.IsEmpty())
@@ -400,7 +356,7 @@ HTMLTextFieldAccessible::GetNameInternal(nsAString& aName)
     // This means we're part of another control, so use parent accessible for name.
     // This ensures that a textbox inside of a XUL widget gets
     // an accessible name.
-    nsAccessible* parent = Parent();
+    Accessible* parent = Parent();
     if (parent)
       parent->GetName(aName);
   }
@@ -434,17 +390,17 @@ HTMLTextFieldAccessible::Value(nsString& aValue)
 }
 
 void
-HTMLTextFieldAccessible::ApplyARIAState(PRUint64* aState)
+HTMLTextFieldAccessible::ApplyARIAState(uint64_t* aState) const
 {
-  nsHyperTextAccessibleWrap::ApplyARIAState(aState);
+  HyperTextAccessibleWrap::ApplyARIAState(aState);
 
   aria::MapToState(aria::eARIAAutoComplete, mContent->AsElement(), aState);
 }
 
-PRUint64
+uint64_t
 HTMLTextFieldAccessible::State()
 {
-  PRUint64 state = nsHyperTextAccessibleWrap::State();
+  uint64_t state = HyperTextAccessibleWrap::State();
   if (state & states::DEFUNCT)
     return state;
 
@@ -452,7 +408,7 @@ HTMLTextFieldAccessible::State()
   // no special processing for unavailable state since inheritance is supplied
   // by other code paths.
   if (mParent && mParent->IsHTMLFileInput()) {
-    PRUint64 parentState = mParent->State();
+    uint64_t parentState = mParent->State();
     state |= parentState & (states::BUSY | states::REQUIRED |
       states::HASPOPUP | states::INVALID);
   }
@@ -460,10 +416,10 @@ HTMLTextFieldAccessible::State()
   return state;
 }
 
-PRUint64
+uint64_t
 HTMLTextFieldAccessible::NativeState()
 {
-  PRUint64 state = nsHyperTextAccessibleWrap::NativeState();
+  uint64_t state = HyperTextAccessibleWrap::NativeState();
 
   // can be focusable, focused, protected. readonly, unavailable, selected
   if (mContent->AttrValueIs(kNameSpaceID_None, nsGkAtoms::type,
@@ -484,7 +440,7 @@ HTMLTextFieldAccessible::NativeState()
     return state;
 
   // Expose autocomplete states if this input is part of autocomplete widget.
-  nsAccessible* widget = ContainerWidget();
+  Accessible* widget = ContainerWidget();
   if (widget && widget-IsAutoComplete()) {
     state |= states::HASPOPUP | states::SUPPORTS_AUTOCOMPLETION;
     return state;
@@ -496,7 +452,7 @@ HTMLTextFieldAccessible::NativeState()
 
   // No parent can mean a fake widget created for XUL textbox. If accessible
   // is unattached from tree then we don't care.
-  if (mParent && gIsFormFillEnabled) {
+  if (mParent && Preferences::GetBool("browser.formfill.enable")) {
     // Check to see if autocompletion is allowed on this input. We don't expose
     // it for password fields even though the entire password can be remembered
     // for a page if the user asks it to be. However, the kind of autocomplete
@@ -523,14 +479,14 @@ HTMLTextFieldAccessible::NativeState()
   return state;
 }
 
-PRUint8
+uint8_t
 HTMLTextFieldAccessible::ActionCount()
 {
   return 1;
 }
 
 NS_IMETHODIMP
-HTMLTextFieldAccessible::GetActionName(PRUint8 aIndex, nsAString& aName)
+HTMLTextFieldAccessible::GetActionName(uint8_t aIndex, nsAString& aName)
 {
   if (aIndex == eAction_Click) {
     aName.AssignLiteral("activate");
@@ -540,7 +496,7 @@ HTMLTextFieldAccessible::GetActionName(PRUint8 aIndex, nsAString& aName)
 }
 
 NS_IMETHODIMP
-HTMLTextFieldAccessible::DoAction(PRUint8 aIndex)
+HTMLTextFieldAccessible::DoAction(uint8_t aIndex)
 {
   if (aIndex == 0) {
     nsCOMPtr<nsIDOMHTMLElement> element(do_QueryInterface(mContent));
@@ -557,14 +513,14 @@ HTMLTextFieldAccessible::GetEditor() const
 {
   nsCOMPtr<nsIDOMNSEditableElement> editableElt(do_QueryInterface(mContent));
   if (!editableElt)
-    return nsnull;
+    return nullptr;
 
   // nsGenericHTMLElement::GetEditor has a security check.
   // Make sure we're not restricted by the permissions of
   // whatever script is currently running.
   nsCOMPtr<nsIJSContextStack> stack =
     do_GetService("@mozilla.org/js/xpc/ContextStack;1");
-  bool pushed = stack && NS_SUCCEEDED(stack->Push(nsnull));
+  bool pushed = stack && NS_SUCCEEDED(stack->Push(nullptr));
 
   nsCOMPtr<nsIEditor> editor;
   editableElt->GetEditor(getter_AddRefs(editor));
@@ -587,10 +543,10 @@ HTMLTextFieldAccessible::IsWidget() const
   return true;
 }
 
-nsAccessible*
+Accessible*
 HTMLTextFieldAccessible::ContainerWidget() const
 {
-  return mParent && mParent->Role() == roles::AUTOCOMPLETE ? mParent : nsnull;
+  return mParent && mParent->Role() == roles::AUTOCOMPLETE ? mParent : nullptr;
 }
 
 
@@ -599,8 +555,8 @@ HTMLTextFieldAccessible::ContainerWidget() const
 ////////////////////////////////////////////////////////////////////////////////
 
 HTMLFileInputAccessible::
-HTMLFileInputAccessible(nsIContent* aContent, nsDocAccessible* aDoc) :
-  nsHyperTextAccessibleWrap(aContent, aDoc)
+HTMLFileInputAccessible(nsIContent* aContent, DocAccessible* aDoc) :
+  HyperTextAccessibleWrap(aContent, aDoc)
 {
   mFlags |= eHTMLFileInputAccessible;
 }
@@ -616,7 +572,7 @@ HTMLFileInputAccessible::NativeRole()
 nsresult
 HTMLFileInputAccessible::HandleAccEvent(AccEvent* aEvent)
 {
-  nsresult rv = nsHyperTextAccessibleWrap::HandleAccEvent(aEvent);
+  nsresult rv = HyperTextAccessibleWrap::HandleAccEvent(aEvent);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Redirect state change events for inherited states to child controls. Note,
@@ -628,7 +584,7 @@ HTMLFileInputAccessible::HandleAccEvent(AccEvent* aEvent)
        event->GetState() == states::REQUIRED ||
        event->GetState() == states::HASPOPUP ||
        event->GetState() == states::INVALID)) {
-    nsAccessible* input = GetChildAt(0);
+    Accessible* input = GetChildAt(0);
     if (input && input->Role() == roles::ENTRY) {
       nsRefPtr<AccStateChangeEvent> childEvent =
         new AccStateChangeEvent(input, event->GetState(),
@@ -637,7 +593,7 @@ HTMLFileInputAccessible::HandleAccEvent(AccEvent* aEvent)
       nsEventShell::FireEvent(childEvent);
     }
 
-    nsAccessible* button = GetChildAt(1);
+    Accessible* button = GetChildAt(1);
     if (button && button->Role() == roles::PUSHBUTTON) {
       nsRefPtr<AccStateChangeEvent> childEvent =
         new AccStateChangeEvent(button, event->GetState(),
@@ -654,8 +610,8 @@ HTMLFileInputAccessible::HandleAccEvent(AccEvent* aEvent)
 ////////////////////////////////////////////////////////////////////////////////
 
 HTMLGroupboxAccessible::
-  HTMLGroupboxAccessible(nsIContent* aContent, nsDocAccessible* aDoc) :
-  nsHyperTextAccessibleWrap(aContent, aDoc)
+  HTMLGroupboxAccessible(nsIContent* aContent, DocAccessible* aDoc) :
+  HyperTextAccessibleWrap(aContent, aDoc)
 {
 }
 
@@ -677,13 +633,13 @@ HTMLGroupboxAccessible::GetLegend()
     }
   }
 
-  return nsnull;
+  return nullptr;
 }
 
 nsresult
 HTMLGroupboxAccessible::GetNameInternal(nsAString& aName)
 {
-  nsresult rv = nsAccessible::GetNameInternal(aName);
+  nsresult rv = Accessible::GetNameInternal(aName);
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (!aName.IsEmpty())
@@ -699,12 +655,12 @@ HTMLGroupboxAccessible::GetNameInternal(nsAString& aName)
 }
 
 Relation
-HTMLGroupboxAccessible::RelationByType(PRUint32 aType)
+HTMLGroupboxAccessible::RelationByType(uint32_t aType)
 {
-  Relation rel = nsHyperTextAccessibleWrap::RelationByType(aType);
+  Relation rel = HyperTextAccessibleWrap::RelationByType(aType);
     // No override for label, so use <legend> for this <fieldset>
   if (aType == nsIAccessibleRelation::RELATION_LABELLED_BY)
-    rel.AppendTarget(GetLegend());
+    rel.AppendTarget(mDoc, GetLegend());
 
   return rel;
 }
@@ -714,19 +670,19 @@ HTMLGroupboxAccessible::RelationByType(PRUint32 aType)
 ////////////////////////////////////////////////////////////////////////////////
 
 HTMLLegendAccessible::
-  HTMLLegendAccessible(nsIContent* aContent, nsDocAccessible* aDoc) :
-  nsHyperTextAccessibleWrap(aContent, aDoc)
+  HTMLLegendAccessible(nsIContent* aContent, DocAccessible* aDoc) :
+  HyperTextAccessibleWrap(aContent, aDoc)
 {
 }
 
 Relation
-HTMLLegendAccessible::RelationByType(PRUint32 aType)
+HTMLLegendAccessible::RelationByType(uint32_t aType)
 {
-  Relation rel = nsHyperTextAccessibleWrap::RelationByType(aType);
+  Relation rel = HyperTextAccessibleWrap::RelationByType(aType);
   if (aType != nsIAccessibleRelation::RELATION_LABEL_FOR)
     return rel;
 
-  nsAccessible* groupbox = Parent();
+  Accessible* groupbox = Parent();
   if (groupbox && groupbox->Role() == roles::GROUPING)
     rel.AppendTarget(groupbox);
 
@@ -744,15 +700,15 @@ HTMLLegendAccessible::NativeRole()
 ////////////////////////////////////////////////////////////////////////////////
 
 HTMLFigureAccessible::
-  HTMLFigureAccessible(nsIContent* aContent, nsDocAccessible* aDoc) :
-  nsHyperTextAccessibleWrap(aContent, aDoc)
+  HTMLFigureAccessible(nsIContent* aContent, DocAccessible* aDoc) :
+  HyperTextAccessibleWrap(aContent, aDoc)
 {
 }
 
 nsresult
 HTMLFigureAccessible::GetAttributesInternal(nsIPersistentProperties* aAttributes)
 {
-  nsresult rv = nsHyperTextAccessibleWrap::GetAttributesInternal(aAttributes);
+  nsresult rv = HyperTextAccessibleWrap::GetAttributesInternal(aAttributes);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Expose figure xml-role.
@@ -770,7 +726,7 @@ HTMLFigureAccessible::NativeRole()
 nsresult
 HTMLFigureAccessible::GetNameInternal(nsAString& aName)
 {
-  nsresult rv = nsHyperTextAccessibleWrap::GetNameInternal(aName);
+  nsresult rv = HyperTextAccessibleWrap::GetNameInternal(aName);
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (!aName.IsEmpty())
@@ -786,11 +742,11 @@ HTMLFigureAccessible::GetNameInternal(nsAString& aName)
 }
 
 Relation
-HTMLFigureAccessible::RelationByType(PRUint32 aType)
+HTMLFigureAccessible::RelationByType(uint32_t aType)
 {
-  Relation rel = nsHyperTextAccessibleWrap::RelationByType(aType);
+  Relation rel = HyperTextAccessibleWrap::RelationByType(aType);
   if (aType == nsIAccessibleRelation::RELATION_LABELLED_BY)
-    rel.AppendTarget(Caption());
+    rel.AppendTarget(mDoc, Caption());
 
   return rel;
 }
@@ -806,7 +762,7 @@ HTMLFigureAccessible::Caption() const
     }
   }
 
-  return nsnull;
+  return nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -814,8 +770,8 @@ HTMLFigureAccessible::Caption() const
 ////////////////////////////////////////////////////////////////////////////////
 
 HTMLFigcaptionAccessible::
-  HTMLFigcaptionAccessible(nsIContent* aContent, nsDocAccessible* aDoc) :
-  nsHyperTextAccessibleWrap(aContent, aDoc)
+  HTMLFigcaptionAccessible(nsIContent* aContent, DocAccessible* aDoc) :
+  HyperTextAccessibleWrap(aContent, aDoc)
 {
 }
 
@@ -826,13 +782,13 @@ HTMLFigcaptionAccessible::NativeRole()
 }
 
 Relation
-HTMLFigcaptionAccessible::RelationByType(PRUint32 aType)
+HTMLFigcaptionAccessible::RelationByType(uint32_t aType)
 {
-  Relation rel = nsHyperTextAccessibleWrap::RelationByType(aType);
+  Relation rel = HyperTextAccessibleWrap::RelationByType(aType);
   if (aType != nsIAccessibleRelation::RELATION_LABEL_FOR)
     return rel;
 
-  nsAccessible* figure = Parent();
+  Accessible* figure = Parent();
   if (figure &&
       figure->GetContent()->NodeInfo()->Equals(nsGkAtoms::figure,
                                                mContent->GetNameSpaceID())) {
