@@ -4,9 +4,6 @@
 const URL = "http://mochi.test:8888/browser/browser/components/thumbnails/" +
             "test/background_red_redirect.sjs";
 
-let cacheService = Cc["@mozilla.org/network/cache-service;1"]
-                   .getService(Ci.nsICacheService);
-
 /**
  * These tests ensure that we save and provide thumbnails for redirecting sites.
  */
@@ -19,33 +16,17 @@ function runTests() {
   yield addTab(URL);
   yield captureAndCheckColor(255, 0, 0, "we have a red thumbnail");
 
-  // Wait until the referrer's thumbnail's cache entry has been written.
-  yield whenCacheEntryExists(URL);
+  // Wait until the referrer's thumbnail's file has been written.
+  yield whenFileExists(URL);
   yield checkThumbnailColor(URL, 255, 0, 0, "referrer has a red thumbnail");
 }
 
-function whenCacheEntryExists(aKey) {
+function whenFileExists(aURL) {
   let callback = next;
 
-  checkCacheEntryExists(aKey, function (aExists) {
-    if (!aExists)
-      callback = function () whenCacheEntryExists(aKey);
+  let file = PageThumbsStorage.getFileForURL(aURL);
+  if (!file.exists())
+    callback = function () whenFileExists(aURL);
 
-    executeSoon(callback);
-  });
-}
-
-function checkCacheEntryExists(aKey, aCallback) {
-  PageThumbsCache.getReadEntry(aKey, function (aEntry) {
-    let inputStream = aEntry && aEntry.openInputStream(0);
-    let exists = inputStream && inputStream.available();
-
-    if (inputStream)
-      inputStream.close();
-
-    if (aEntry)
-      aEntry.close();
-
-    aCallback(exists);
-  });
+  executeSoon(callback);
 }
