@@ -14,32 +14,26 @@ const TEST_URI = "http://example.com/browser/browser/devtools/" +
 
 function test() {
   addTab(TEST_URI);
-  browser.addEventListener("load", tabLoaded, true);
+  browser.addEventListener("load", function onLoad() {
+    browser.removeEventListener("load", onLoad, true);
+    openConsole(null, consoleOpened);
+  }, true);
 }
 
-function tabLoaded(aEvent) {
-  browser.removeEventListener("load", tabLoaded, true);
-  openConsole();
-
-  browser.addEventListener("load", contentLoaded, true);
-  content.location.reload();
-}
-
-function contentLoaded(aEvent) {
-  browser.removeEventListener("load", contentLoaded, true);
-
+function consoleOpened(hud) {
   let button = content.document.querySelector("button");
+  let outputNode = hud.outputNode;
+
   expectUncaughtException();
   EventUtils.sendMouseEvent({ type: "click" }, button, content);
-  executeSoon(buttonClicked);
+
+  waitForSuccess({
+    name: "external script error message",
+    validatorFn: function()
+    {
+      return outputNode.textContent.indexOf("bogus is not defined") > -1;
+    },
+    successFn: finishTest,
+    failureFn: finishTest,
+  });
 }
-
-function buttonClicked() {
-  let outputNode = HUDService.getHudByWindow(content).outputNode;
-
-  let msg = "the error from the external script was logged";
-  testLogEntry(outputNode, "bogus", msg);
-
-  finishTest();
-}
-
