@@ -1,40 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * IBM Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2003
- * IBM Corporation. All Rights Reserved.
- *
- * Contributor(s):
- *   IBM Corporation
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /*
  * Storage of the children and attributes of a DOM node; storage for
@@ -78,7 +45,7 @@ any index will get used.
 struct IndexCacheSlot
 {
   const nsAttrAndChildArray* array;
-  PRInt32 index;
+  int32_t index;
 };
 
 // This is inited to all zeroes since it's static. Though even if it wasn't
@@ -89,19 +56,19 @@ static IndexCacheSlot indexCache[CACHE_NUM_SLOTS];
 static
 inline
 void
-AddIndexToCache(const nsAttrAndChildArray* aArray, PRInt32 aIndex)
+AddIndexToCache(const nsAttrAndChildArray* aArray, int32_t aIndex)
 {
-  PRUint32 ix = CACHE_GET_INDEX(aArray);
+  uint32_t ix = CACHE_GET_INDEX(aArray);
   indexCache[ix].array = aArray;
   indexCache[ix].index = aIndex;
 }
 
 static
 inline
-PRInt32
+int32_t
 GetIndexFromCache(const nsAttrAndChildArray* aArray)
 {
-  PRUint32 ix = CACHE_GET_INDEX(aArray);
+  uint32_t ix = CACHE_GET_INDEX(aArray);
   return indexCache[ix].array == aArray ? indexCache[ix].index : -1;
 }
 
@@ -121,7 +88,7 @@ GetIndexFromCache(const nsAttrAndChildArray* aArray)
   ((sizeof(Impl) - sizeof(mImpl->mBuffer)) / sizeof(void*))
 
 nsAttrAndChildArray::nsAttrAndChildArray()
-  : mImpl(nsnull)
+  : mImpl(nullptr)
 {
 }
 
@@ -137,35 +104,35 @@ nsAttrAndChildArray::~nsAttrAndChildArray()
 }
 
 nsIContent*
-nsAttrAndChildArray::GetSafeChildAt(PRUint32 aPos) const
+nsAttrAndChildArray::GetSafeChildAt(uint32_t aPos) const
 {
   if (aPos < ChildCount()) {
     return ChildAt(aPos);
   }
   
-  return nsnull;
+  return nullptr;
 }
 
 nsIContent * const *
-nsAttrAndChildArray::GetChildArray(PRUint32* aChildCount) const
+nsAttrAndChildArray::GetChildArray(uint32_t* aChildCount) const
 {
   *aChildCount = ChildCount();
   
   if (!*aChildCount) {
-    return nsnull;
+    return nullptr;
   }
   
   return reinterpret_cast<nsIContent**>(mImpl->mBuffer + AttrSlotsSize());
 }
 
 nsresult
-nsAttrAndChildArray::InsertChildAt(nsIContent* aChild, PRUint32 aPos)
+nsAttrAndChildArray::InsertChildAt(nsIContent* aChild, uint32_t aPos)
 {
   NS_ASSERTION(aChild, "nullchild");
   NS_ASSERTION(aPos <= ChildCount(), "out-of-bounds");
 
-  PRUint32 offset = AttrSlotsSize();
-  PRUint32 childCount = ChildCount();
+  uint32_t offset = AttrSlotsSize();
+  uint32_t childCount = ChildCount();
 
   NS_ENSURE_TRUE(childCount < ATTRCHILD_ARRAY_MAX_CHILD_COUNT,
                  NS_ERROR_FAILURE);
@@ -187,7 +154,7 @@ nsAttrAndChildArray::InsertChildAt(nsIContent* aChild, PRUint32 aPos)
   if (offset && !mImpl->mBuffer[offset - ATTRSIZE]) {
     // Compress away all empty slots while we're at it. This might not be the
     // optimal thing to do.
-    PRUint32 attrCount = NonMappedAttrCount();
+    uint32_t attrCount = NonMappedAttrCount();
     void** newStart = mImpl->mBuffer + attrCount * ATTRSIZE;
     void** oldStart = mImpl->mBuffer + offset;
     memmove(newStart, oldStart, aPos * sizeof(nsIContent*));
@@ -217,7 +184,7 @@ nsAttrAndChildArray::InsertChildAt(nsIContent* aChild, PRUint32 aPos)
 }
 
 void
-nsAttrAndChildArray::RemoveChildAt(PRUint32 aPos)
+nsAttrAndChildArray::RemoveChildAt(uint32_t aPos)
 {
   // Just store the return value of TakeChildAt in an nsCOMPtr to
   // trigger a release.
@@ -225,11 +192,11 @@ nsAttrAndChildArray::RemoveChildAt(PRUint32 aPos)
 }
 
 already_AddRefed<nsIContent>
-nsAttrAndChildArray::TakeChildAt(PRUint32 aPos)
+nsAttrAndChildArray::TakeChildAt(uint32_t aPos)
 {
   NS_ASSERTION(aPos < ChildCount(), "out-of-bounds");
 
-  PRUint32 childCount = ChildCount();
+  uint32_t childCount = ChildCount();
   void** pos = mImpl->mBuffer + AttrSlotsSize() + aPos;
   nsIContent* child = static_cast<nsIContent*>(*pos);
   if (child->mPreviousSibling) {
@@ -238,7 +205,7 @@ nsAttrAndChildArray::TakeChildAt(PRUint32 aPos)
   if (child->mNextSibling) {
     child->mNextSibling->mPreviousSibling = child->mPreviousSibling;
   }
-  child->mPreviousSibling = child->mNextSibling = nsnull;
+  child->mPreviousSibling = child->mNextSibling = nullptr;
 
   memmove(pos, pos + 1, (childCount - aPos - 1) * sizeof(nsIContent*));
   SetChildCount(childCount - 1);
@@ -246,7 +213,7 @@ nsAttrAndChildArray::TakeChildAt(PRUint32 aPos)
   return child;
 }
 
-PRInt32
+int32_t
 nsAttrAndChildArray::IndexOfChild(nsINode* aPossibleChild) const
 {
   if (!mImpl) {
@@ -254,10 +221,10 @@ nsAttrAndChildArray::IndexOfChild(nsINode* aPossibleChild) const
   }
   void** children = mImpl->mBuffer + AttrSlotsSize();
   // Use signed here since we compare count to cursor which has to be signed
-  PRInt32 i, count = ChildCount();
+  int32_t i, count = ChildCount();
 
   if (count >= CACHE_CHILD_LIMIT) {
-    PRInt32 cursor = GetIndexFromCache(this);
+    int32_t cursor = GetIndexFromCache(this);
     // Need to compare to count here since we may have removed children since
     // the index was added to the cache.
     // We're also relying on that GetIndexFromCache returns -1 if no cached
@@ -269,7 +236,7 @@ nsAttrAndChildArray::IndexOfChild(nsINode* aPossibleChild) const
     // Seek outward from the last found index. |inc| will change sign every
     // run through the loop. |sign| just exists to make sure the absolute
     // value of |inc| increases each time through.
-    PRInt32 inc = 1, sign = 1;
+    int32_t inc = 1, sign = 1;
     while (cursor >= 0 && cursor < count) {
       if (children[cursor] == aPossibleChild) {
         AddIndexToCache(this, cursor);
@@ -292,7 +259,7 @@ nsAttrAndChildArray::IndexOfChild(nsINode* aPossibleChild) const
         if (children[cursor] == aPossibleChild) {
           AddIndexToCache(this, cursor);
 
-          return static_cast<PRInt32>(cursor);
+          return static_cast<int32_t>(cursor);
         }
       }
     }
@@ -301,7 +268,7 @@ nsAttrAndChildArray::IndexOfChild(nsINode* aPossibleChild) const
         if (children[cursor] == aPossibleChild) {
           AddIndexToCache(this, cursor);
 
-          return static_cast<PRInt32>(cursor);
+          return static_cast<int32_t>(cursor);
         }
       }
     }
@@ -312,23 +279,23 @@ nsAttrAndChildArray::IndexOfChild(nsINode* aPossibleChild) const
 
   for (i = 0; i < count; ++i) {
     if (children[i] == aPossibleChild) {
-      return static_cast<PRInt32>(i);
+      return static_cast<int32_t>(i);
     }
   }
 
   return -1;
 }
 
-PRUint32
+uint32_t
 nsAttrAndChildArray::AttrCount() const
 {
   return NonMappedAttrCount() + MappedAttrCount();
 }
 
 const nsAttrValue*
-nsAttrAndChildArray::GetAttr(nsIAtom* aLocalName, PRInt32 aNamespaceID) const
+nsAttrAndChildArray::GetAttr(nsIAtom* aLocalName, int32_t aNamespaceID) const
 {
-  PRUint32 i, slotCount = AttrSlotCount();
+  uint32_t i, slotCount = AttrSlotCount();
   if (aNamespaceID == kNameSpaceID_None) {
     // This should be the common case so lets make an optimized loop
     for (i = 0; i < slotCount && AttrSlotIsTaken(i); ++i) {
@@ -349,16 +316,49 @@ nsAttrAndChildArray::GetAttr(nsIAtom* aLocalName, PRInt32 aNamespaceID) const
     }
   }
 
-  return nsnull;
+  return nullptr;
 }
 
 const nsAttrValue*
-nsAttrAndChildArray::AttrAt(PRUint32 aPos) const
+nsAttrAndChildArray::GetAttr(const nsAString& aName,
+                             nsCaseTreatment aCaseSensitive) const
+{
+  // Check whether someone is being silly and passing non-lowercase
+  // attr names.
+  if (aCaseSensitive == eIgnoreCase &&
+      nsContentUtils::StringContainsASCIIUpper(aName)) {
+    // Try again with a lowercased name, but make sure we can't reenter this
+    // block by passing eCaseSensitive for aCaseSensitive.
+    nsAutoString lowercase;
+    nsContentUtils::ASCIIToLower(aName, lowercase);
+    return GetAttr(lowercase, eCaseMatters);
+  }
+
+  uint32_t i, slotCount = AttrSlotCount();
+  for (i = 0; i < slotCount && AttrSlotIsTaken(i); ++i) {
+    if (ATTRS(mImpl)[i].mName.QualifiedNameEquals(aName)) {
+      return &ATTRS(mImpl)[i].mValue;
+    }
+  }
+
+  if (mImpl && mImpl->mMappedAttrs) {
+    const nsAttrValue* val =
+      mImpl->mMappedAttrs->GetAttr(aName);
+    if (val) {
+      return val;
+    }
+  }
+
+  return nullptr;
+}
+
+const nsAttrValue*
+nsAttrAndChildArray::AttrAt(uint32_t aPos) const
 {
   NS_ASSERTION(aPos < AttrCount(),
                "out-of-bounds access in nsAttrAndChildArray");
 
-  PRUint32 mapped = MappedAttrCount();
+  uint32_t mapped = MappedAttrCount();
   if (aPos < mapped) {
     return mImpl->mMappedAttrs->AttrAt(aPos);
   }
@@ -369,7 +369,7 @@ nsAttrAndChildArray::AttrAt(PRUint32 aPos) const
 nsresult
 nsAttrAndChildArray::SetAndTakeAttr(nsIAtom* aLocalName, nsAttrValue& aValue)
 {
-  PRUint32 i, slotCount = AttrSlotCount();
+  uint32_t i, slotCount = AttrSlotCount();
   for (i = 0; i < slotCount && AttrSlotIsTaken(i); ++i) {
     if (ATTRS(mImpl)[i].mName.Equals(aLocalName)) {
       ATTRS(mImpl)[i].mValue.Reset();
@@ -396,13 +396,13 @@ nsAttrAndChildArray::SetAndTakeAttr(nsIAtom* aLocalName, nsAttrValue& aValue)
 nsresult
 nsAttrAndChildArray::SetAndTakeAttr(nsINodeInfo* aName, nsAttrValue& aValue)
 {
-  PRInt32 namespaceID = aName->NamespaceID();
+  int32_t namespaceID = aName->NamespaceID();
   nsIAtom* localName = aName->NameAtom();
   if (namespaceID == kNameSpaceID_None) {
     return SetAndTakeAttr(localName, aValue);
   }
 
-  PRUint32 i, slotCount = AttrSlotCount();
+  uint32_t i, slotCount = AttrSlotCount();
   for (i = 0; i < slotCount && AttrSlotIsTaken(i); ++i) {
     if (ATTRS(mImpl)[i].mName.Equals(localName, namespaceID)) {
       ATTRS(mImpl)[i].mName.SetTo(aName);
@@ -429,11 +429,11 @@ nsAttrAndChildArray::SetAndTakeAttr(nsINodeInfo* aName, nsAttrValue& aValue)
 
 
 nsresult
-nsAttrAndChildArray::RemoveAttrAt(PRUint32 aPos, nsAttrValue& aValue)
+nsAttrAndChildArray::RemoveAttrAt(uint32_t aPos, nsAttrValue& aValue)
 {
   NS_ASSERTION(aPos < AttrCount(), "out-of-bounds");
 
-  PRUint32 mapped = MappedAttrCount();
+  uint32_t mapped = MappedAttrCount();
   if (aPos < mapped) {
     if (mapped == 1) {
       // We're removing the last mapped attribute.  Can't swap in this
@@ -444,10 +444,8 @@ nsAttrAndChildArray::RemoveAttrAt(PRUint32 aPos, nsAttrValue& aValue)
       return NS_OK;
     }
 
-    nsRefPtr<nsMappedAttributes> mapped;
-    nsresult rv = GetModifiableMapped(nsnull, nsnull, false,
-                                      getter_AddRefs(mapped));
-    NS_ENSURE_SUCCESS(rv, rv);
+    nsRefPtr<nsMappedAttributes> mapped =
+      GetModifiableMapped(nullptr, nullptr, false);
 
     mapped->RemoveAttrAt(aPos, aValue);
 
@@ -458,22 +456,22 @@ nsAttrAndChildArray::RemoveAttrAt(PRUint32 aPos, nsAttrValue& aValue)
   ATTRS(mImpl)[aPos].mValue.SwapValueWith(aValue);
   ATTRS(mImpl)[aPos].~InternalAttr();
 
-  PRUint32 slotCount = AttrSlotCount();
+  uint32_t slotCount = AttrSlotCount();
   memmove(&ATTRS(mImpl)[aPos],
           &ATTRS(mImpl)[aPos + 1],
           (slotCount - aPos - 1) * sizeof(InternalAttr));
-  memset(&ATTRS(mImpl)[slotCount - 1], nsnull, sizeof(InternalAttr));
+  memset(&ATTRS(mImpl)[slotCount - 1], 0, sizeof(InternalAttr));
 
   return NS_OK;
 }
 
 const nsAttrName*
-nsAttrAndChildArray::AttrNameAt(PRUint32 aPos) const
+nsAttrAndChildArray::AttrNameAt(uint32_t aPos) const
 {
   NS_ASSERTION(aPos < AttrCount(),
                "out-of-bounds access in nsAttrAndChildArray");
 
-  PRUint32 mapped = MappedAttrCount();
+  uint32_t mapped = MappedAttrCount();
   if (aPos < mapped) {
     return mImpl->mMappedAttrs->NameAt(aPos);
   }
@@ -482,21 +480,21 @@ nsAttrAndChildArray::AttrNameAt(PRUint32 aPos) const
 }
 
 const nsAttrName*
-nsAttrAndChildArray::GetSafeAttrNameAt(PRUint32 aPos) const
+nsAttrAndChildArray::GetSafeAttrNameAt(uint32_t aPos) const
 {
-  PRUint32 mapped = MappedAttrCount();
+  uint32_t mapped = MappedAttrCount();
   if (aPos < mapped) {
     return mImpl->mMappedAttrs->NameAt(aPos);
   }
 
   aPos -= mapped;
   if (aPos >= AttrSlotCount()) {
-    return nsnull;
+    return nullptr;
   }
 
   void** pos = mImpl->mBuffer + aPos * ATTRSIZE;
   if (!*pos) {
-    return nsnull;
+    return nullptr;
   }
 
   return &reinterpret_cast<InternalAttr*>(pos)->mName;
@@ -505,7 +503,7 @@ nsAttrAndChildArray::GetSafeAttrNameAt(PRUint32 aPos) const
 const nsAttrName*
 nsAttrAndChildArray::GetExistingAttrNameFromQName(const nsAString& aName) const
 {
-  PRUint32 i, slotCount = AttrSlotCount();
+  uint32_t i, slotCount = AttrSlotCount();
   for (i = 0; i < slotCount && AttrSlotIsTaken(i); ++i) {
     if (ATTRS(mImpl)[i].mName.QualifiedNameEquals(aName)) {
       return &ATTRS(mImpl)[i].mName;
@@ -516,23 +514,23 @@ nsAttrAndChildArray::GetExistingAttrNameFromQName(const nsAString& aName) const
     return mImpl->mMappedAttrs->GetExistingAttrNameFromQName(aName);
   }
 
-  return nsnull;
+  return nullptr;
 }
 
-PRInt32
-nsAttrAndChildArray::IndexOfAttr(nsIAtom* aLocalName, PRInt32 aNamespaceID) const
+int32_t
+nsAttrAndChildArray::IndexOfAttr(nsIAtom* aLocalName, int32_t aNamespaceID) const
 {
-  PRInt32 idx;
-  if (mImpl && mImpl->mMappedAttrs) {
-    idx = mImpl->mMappedAttrs->IndexOfAttr(aLocalName, aNamespaceID);
+  int32_t idx;
+  if (mImpl && mImpl->mMappedAttrs && aNamespaceID == kNameSpaceID_None) {
+    idx = mImpl->mMappedAttrs->IndexOfAttr(aLocalName);
     if (idx >= 0) {
       return idx;
     }
   }
 
-  PRUint32 i;
-  PRUint32 mapped = MappedAttrCount();
-  PRUint32 slotCount = AttrSlotCount();
+  uint32_t i;
+  uint32_t mapped = MappedAttrCount();
+  uint32_t slotCount = AttrSlotCount();
   if (aNamespaceID == kNameSpaceID_None) {
     // This should be the common case so lets make an optimized loop
     for (i = 0; i < slotCount && AttrSlotIsTaken(i); ++i) {
@@ -558,18 +556,15 @@ nsAttrAndChildArray::SetAndTakeMappedAttr(nsIAtom* aLocalName,
                                           nsMappedAttributeElement* aContent,
                                           nsHTMLStyleSheet* aSheet)
 {
-  nsRefPtr<nsMappedAttributes> mapped;
-
   bool willAdd = true;
   if (mImpl && mImpl->mMappedAttrs) {
-    willAdd = mImpl->mMappedAttrs->GetAttr(aLocalName) == nsnull;
+    willAdd = !mImpl->mMappedAttrs->GetAttr(aLocalName);
   }
 
-  nsresult rv = GetModifiableMapped(aContent, aSheet, willAdd,
-                                    getter_AddRefs(mapped));
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsRefPtr<nsMappedAttributes> mapped =
+    GetModifiableMapped(aContent, aSheet, willAdd);
 
-  rv = mapped->SetAndTakeAttr(aLocalName, aValue);
+  nsresult rv = mapped->SetAndTakeAttr(aLocalName, aValue);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return MakeMappedUnique(mapped);
@@ -584,10 +579,8 @@ nsAttrAndChildArray::DoSetMappedAttrStyleSheet(nsHTMLStyleSheet* aSheet)
     return NS_OK;
   }
 
-  nsRefPtr<nsMappedAttributes> mapped;
-  nsresult rv = GetModifiableMapped(nsnull, nsnull, false, 
-                                    getter_AddRefs(mapped));
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsRefPtr<nsMappedAttributes> mapped =
+    GetModifiableMapped(nullptr, nullptr, false);
 
   mapped->SetStyleSheet(aSheet);
 
@@ -610,9 +603,9 @@ nsAttrAndChildArray::Compact()
   }
 
   // First compress away empty attrslots
-  PRUint32 slotCount = AttrSlotCount();
-  PRUint32 attrCount = NonMappedAttrCount();
-  PRUint32 childCount = ChildCount();
+  uint32_t slotCount = AttrSlotCount();
+  uint32_t attrCount = NonMappedAttrCount();
+  uint32_t childCount = ChildCount();
 
   if (attrCount < slotCount) {
     memmove(mImpl->mBuffer + attrCount * ATTRSIZE,
@@ -622,10 +615,10 @@ nsAttrAndChildArray::Compact()
   }
 
   // Then resize or free buffer
-  PRUint32 newSize = attrCount * ATTRSIZE + childCount;
+  uint32_t newSize = attrCount * ATTRSIZE + childCount;
   if (!newSize && !mImpl->mMappedAttrs) {
     PR_Free(mImpl);
-    mImpl = nsnull;
+    mImpl = nullptr;
   }
   else if (newSize < mImpl->mBufferSize) {
     mImpl = static_cast<Impl*>(PR_Realloc(mImpl, (newSize + NS_IMPL_EXTRA_SIZE) * sizeof(nsIContent*)));
@@ -646,13 +639,13 @@ nsAttrAndChildArray::Clear()
     NS_RELEASE(mImpl->mMappedAttrs);
   }
 
-  PRUint32 i, slotCount = AttrSlotCount();
+  uint32_t i, slotCount = AttrSlotCount();
   for (i = 0; i < slotCount && AttrSlotIsTaken(i); ++i) {
     ATTRS(mImpl)[i].~InternalAttr();
   }
 
   nsAutoScriptBlocker scriptBlocker;
-  PRUint32 end = slotCount * ATTRSIZE + ChildCount();
+  uint32_t end = slotCount * ATTRSIZE + ChildCount();
   for (i = slotCount * ATTRSIZE; i < end; ++i) {
     nsIContent* child = static_cast<nsIContent*>(mImpl->mBuffer[i]);
     // making this false so tree teardown doesn't end up being
@@ -669,21 +662,21 @@ nsAttrAndChildArray::Clear()
     // to point to each other but keep the kid being removed pointing to them
     // through ContentRemoved so consumers can find where it used to be in the
     // list?
-    child->mPreviousSibling = child->mNextSibling = nsnull;
+    child->mPreviousSibling = child->mNextSibling = nullptr;
     NS_RELEASE(child);
   }
 
   SetAttrSlotAndChildCount(0, 0);
 }
 
-PRUint32
+uint32_t
 nsAttrAndChildArray::NonMappedAttrCount() const
 {
   if (!mImpl) {
     return 0;
   }
 
-  PRUint32 count = AttrSlotCount();
+  uint32_t count = AttrSlotCount();
   while (count > 0 && !mImpl->mBuffer[(count - 1) * ATTRSIZE]) {
     --count;
   }
@@ -691,39 +684,26 @@ nsAttrAndChildArray::NonMappedAttrCount() const
   return count;
 }
 
-PRUint32
+uint32_t
 nsAttrAndChildArray::MappedAttrCount() const
 {
-  return mImpl && mImpl->mMappedAttrs ? (PRUint32)mImpl->mMappedAttrs->Count() : 0;
+  return mImpl && mImpl->mMappedAttrs ? (uint32_t)mImpl->mMappedAttrs->Count() : 0;
 }
 
-nsresult
+nsMappedAttributes*
 nsAttrAndChildArray::GetModifiableMapped(nsMappedAttributeElement* aContent,
                                          nsHTMLStyleSheet* aSheet,
-                                         bool aWillAddAttr,
-                                         nsMappedAttributes** aModifiable)
+                                         bool aWillAddAttr)
 {
-  *aModifiable = nsnull;
-
   if (mImpl && mImpl->mMappedAttrs) {
-    *aModifiable = mImpl->mMappedAttrs->Clone(aWillAddAttr);
-    NS_ENSURE_TRUE(*aModifiable, NS_ERROR_OUT_OF_MEMORY);
-
-    NS_ADDREF(*aModifiable);
-    
-    return NS_OK;
+    return mImpl->mMappedAttrs->Clone(aWillAddAttr);
   }
 
-  NS_ASSERTION(aContent, "Trying to create modifiable without content");
+  MOZ_ASSERT(aContent, "Trying to create modifiable without content");
 
   nsMapRuleToAttributesFunc mapRuleFunc =
     aContent->GetAttributeMappingFunction();
-  *aModifiable = new nsMappedAttributes(aSheet, mapRuleFunc);
-  NS_ENSURE_TRUE(*aModifiable, NS_ERROR_OUT_OF_MEMORY);
-
-  NS_ADDREF(*aModifiable);
-
-  return NS_OK;
+  return new nsMappedAttributes(aSheet, mapRuleFunc);
 }
 
 nsresult
@@ -763,10 +743,10 @@ nsAttrAndChildArray::MakeMappedUnique(nsMappedAttributes* aAttributes)
 
 
 bool
-nsAttrAndChildArray::GrowBy(PRUint32 aGrowSize)
+nsAttrAndChildArray::GrowBy(uint32_t aGrowSize)
 {
-  PRUint32 size = mImpl ? mImpl->mBufferSize + NS_IMPL_EXTRA_SIZE : 0;
-  PRUint32 minSize = size + aGrowSize;
+  uint32_t size = mImpl ? mImpl->mBufferSize + NS_IMPL_EXTRA_SIZE : 0;
+  uint32_t minSize = size + aGrowSize;
 
   if (minSize <= ATTRCHILD_ARRAY_LINEAR_THRESHOLD) {
     do {
@@ -785,7 +765,7 @@ nsAttrAndChildArray::GrowBy(PRUint32 aGrowSize)
 
   // Set initial counts if we didn't have a buffer before
   if (needToInitialize) {
-    mImpl->mMappedAttrs = nsnull;
+    mImpl->mMappedAttrs = nullptr;
     SetAttrSlotAndChildCount(0, 0);
   }
 
@@ -797,8 +777,8 @@ nsAttrAndChildArray::GrowBy(PRUint32 aGrowSize)
 bool
 nsAttrAndChildArray::AddAttrSlot()
 {
-  PRUint32 slotCount = AttrSlotCount();
-  PRUint32 childCount = ChildCount();
+  uint32_t slotCount = AttrSlotCount();
+  uint32_t childCount = ChildCount();
 
   // Grow buffer if needed
   if (!(mImpl && mImpl->mBufferSize >= (slotCount + 1) * ATTRSIZE + childCount) &&
@@ -813,15 +793,15 @@ nsAttrAndChildArray::AddAttrSlot()
   }
 
   SetAttrSlotCount(slotCount + 1);
-  offset[0] = nsnull;
-  offset[1] = nsnull;
+  offset[0] = nullptr;
+  offset[1] = nullptr;
 
   return true;
 }
 
 inline void
 nsAttrAndChildArray::SetChildAtPos(void** aPos, nsIContent* aChild,
-                                   PRUint32 aIndex, PRUint32 aChildCount)
+                                   uint32_t aIndex, uint32_t aChildCount)
 {
   NS_PRECONDITION(!aChild->GetNextSibling(), "aChild with next sibling?");
   NS_PRECONDITION(!aChild->GetPreviousSibling(), "aChild with prev sibling?");
@@ -849,8 +829,8 @@ nsAttrAndChildArray::SizeOfExcludingThis(nsMallocSizeOfFun aMallocSizeOf) const
 
     n += aMallocSizeOf(mImpl);
 
-    PRUint32 slotCount = AttrSlotCount();
-    for (PRUint32 i = 0; i < slotCount && AttrSlotIsTaken(i); ++i) {
+    uint32_t slotCount = AttrSlotCount();
+    for (uint32_t i = 0; i < slotCount && AttrSlotIsTaken(i); ++i) {
       nsAttrValue* value = &ATTRS(mImpl)[i].mValue;
       n += value->SizeOfExcludingThis(aMallocSizeOf);
     }
