@@ -1,41 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Pierre Phaneuf <pp@ludusdesign.com>
- *   Mats Palmgren <matpal@gmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /*
  * Object that can be used to serialize selections, ranges, or nodes
@@ -110,9 +76,9 @@ public:
 
 protected:
   void Initialize(bool aClearCachedSerializer = true);
-  nsresult SerializeNodeStart(nsINode* aNode, PRInt32 aStartOffset,
-                              PRInt32 aEndOffset, nsAString& aStr,
-                              nsINode* aOriginalNode = nsnull);
+  nsresult SerializeNodeStart(nsINode* aNode, int32_t aStartOffset,
+                              int32_t aEndOffset, nsAString& aStr,
+                              nsINode* aOriginalNode = nullptr);
   nsresult SerializeToStringRecursive(nsINode* aNode,
                                       nsAString& aStr,
                                       bool aDontSerializeRoot);
@@ -125,11 +91,16 @@ protected:
   nsresult SerializeRangeNodes(nsRange* aRange, 
                                nsINode* aNode, 
                                nsAString& aString,
-                               PRInt32 aDepth);
+                               int32_t aDepth);
   nsresult SerializeRangeContextStart(const nsTArray<nsINode*>& aAncestorArray,
                                       nsAString& aString);
   nsresult SerializeRangeContextEnd(const nsTArray<nsINode*>& aAncestorArray,
                                     nsAString& aString);
+  virtual int32_t
+  GetImmediateContextCount(const nsTArray<nsINode*>& aAncestorArray)
+  {
+    return -1;
+  }
 
   nsresult FlushText(nsAString& aString, bool aForce);
 
@@ -173,18 +144,21 @@ protected:
 
   nsString          mMimeType;
   nsCString         mCharset;
-  PRUint32          mFlags;
-  PRUint32          mWrapColumn;
-  PRUint32          mStartDepth;
-  PRUint32          mEndDepth;
-  PRInt32           mStartRootIndex;
-  PRInt32           mEndRootIndex;
+  uint32_t          mFlags;
+  uint32_t          mWrapColumn;
+  uint32_t          mStartDepth;
+  uint32_t          mEndDepth;
+  int32_t           mStartRootIndex;
+  int32_t           mEndRootIndex;
   nsAutoTArray<nsINode*, 8>    mCommonAncestors;
   nsAutoTArray<nsIContent*, 8> mStartNodes;
-  nsAutoTArray<PRInt32, 8>     mStartOffsets;
+  nsAutoTArray<int32_t, 8>     mStartOffsets;
   nsAutoTArray<nsIContent*, 8> mEndNodes;
-  nsAutoTArray<PRInt32, 8>     mEndOffsets;
+  nsAutoTArray<int32_t, 8>     mEndOffsets;
   bool              mHaltRangeHint;  
+  // Used when context has already been serialized for
+  // table cell selections (where parent is <tr>)
+  bool              mDisableContextSerialize;
   bool              mIsCopying;  // Set to true only while copying
   bool              mNodeIsContainer;
   nsStringBuffer*   mCachedBuffer;
@@ -216,7 +190,7 @@ NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsDocumentEncoder)
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mCommonParent)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
-nsDocumentEncoder::nsDocumentEncoder() : mCachedBuffer(nsnull)
+nsDocumentEncoder::nsDocumentEncoder() : mCachedBuffer(nullptr)
 {
   Initialize();
   mMimeType.AssignLiteral("text/plain");
@@ -232,9 +206,10 @@ void nsDocumentEncoder::Initialize(bool aClearCachedSerializer)
   mStartRootIndex = 0;
   mEndRootIndex = 0;
   mHaltRangeHint = false;
+  mDisableContextSerialize = false;
   mNodeIsContainer = false;
   if (aClearCachedSerializer) {
-    mSerializer = nsnull;
+    mSerializer = nullptr;
   }
 }
 
@@ -248,7 +223,7 @@ nsDocumentEncoder::~nsDocumentEncoder()
 NS_IMETHODIMP
 nsDocumentEncoder::Init(nsIDOMDocument* aDocument,
                         const nsAString& aMimeType,
-                        PRUint32 aFlags)
+                        uint32_t aFlags)
 {
   if (!aDocument)
     return NS_ERROR_INVALID_ARG;
@@ -262,7 +237,7 @@ nsDocumentEncoder::Init(nsIDOMDocument* aDocument,
 NS_IMETHODIMP
 nsDocumentEncoder::NativeInit(nsIDocument* aDocument,
                               const nsAString& aMimeType,
-                              PRUint32 aFlags)
+                              uint32_t aFlags)
 {
   if (!aDocument)
     return NS_ERROR_INVALID_ARG;
@@ -280,7 +255,7 @@ nsDocumentEncoder::NativeInit(nsIDocument* aDocument,
 }
 
 NS_IMETHODIMP
-nsDocumentEncoder::SetWrapColumn(PRUint32 aWC)
+nsDocumentEncoder::SetWrapColumn(uint32_t aWC)
 {
   mWrapColumn = aWC;
   return NS_OK;
@@ -391,15 +366,15 @@ IsInvisibleBreak(nsINode *aNode) {
 
 nsresult
 nsDocumentEncoder::SerializeNodeStart(nsINode* aNode,
-                                      PRInt32 aStartOffset,
-                                      PRInt32 aEndOffset,
+                                      int32_t aStartOffset,
+                                      int32_t aEndOffset,
                                       nsAString& aStr,
                                       nsINode* aOriginalNode)
 {
   if (!IsVisibleNode(aNode))
     return NS_OK;
   
-  nsINode* node = nsnull;
+  nsINode* node = nullptr;
   nsCOMPtr<nsINode> fixedNodeKungfuDeathGrip;
 
   // Caller didn't do fixup, so we'll do it ourselves
@@ -428,7 +403,7 @@ nsDocumentEncoder::SerializeNodeStart(nsINode* aNode,
     }
     Element* originalElement =
       aOriginalNode && aOriginalNode->IsElement() ?
-        aOriginalNode->AsElement() : nsnull;
+        aOriginalNode->AsElement() : nullptr;
     mSerializer->AppendElementStart(node->AsElement(),
                                     originalElement, aStr);
     return NS_OK;
@@ -492,7 +467,7 @@ nsDocumentEncoder::SerializeToStringRecursive(nsINode* aNode,
 
   nsresult rv = NS_OK;
   bool serializeClonedChildren = false;
-  nsINode* maybeFixedNode = nsnull;
+  nsINode* maybeFixedNode = nullptr;
 
   // Keep the node from FixupNode alive.
   nsCOMPtr<nsINode> fixedNodeKungfuDeathGrip;
@@ -512,7 +487,7 @@ nsDocumentEncoder::SerializeToStringRecursive(nsINode* aNode,
       nsIFrame* frame = static_cast<nsIContent*>(aNode)->GetPrimaryFrame();
       if (frame) {
         bool isSelectable;
-        frame->IsSelectable(&isSelectable, nsnull);
+        frame->IsSelectable(&isSelectable, nullptr);
         if (!isSelectable){
           aDontSerializeRoot = true;
         }
@@ -582,11 +557,11 @@ ConvertAndWrite(const nsAString& aString,
   NS_ENSURE_ARG_POINTER(aStream);
   NS_ENSURE_ARG_POINTER(aEncoder);
   nsresult rv;
-  PRInt32 charLength, startCharLength;
+  int32_t charLength, startCharLength;
   const nsPromiseFlatString& flat = PromiseFlatString(aString);
   const PRUnichar* unicodeBuf = flat.get();
-  PRInt32 unicodeLength = aString.Length();
-  PRInt32 startLength = unicodeLength;
+  int32_t unicodeLength = aString.Length();
+  int32_t startLength = unicodeLength;
 
   rv = aEncoder->GetMaxLength(unicodeBuf, unicodeLength, &charLength);
   startCharLength = charLength;
@@ -611,7 +586,7 @@ ConvertAndWrite(const nsAString& aString,
 
     charXferBuf[charLength] = '\0';
 
-    PRUint32 written;
+    uint32_t written;
     rv = aStream->Write(charXferBuf, charLength, &written);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -678,11 +653,11 @@ nsDocumentEncoder::FlushText(nsAString& aString, bool aForce)
 
 #if 0 // This code is really fast at serializing a range, but unfortunately
       // there are problems with it so we don't use it now, maybe later...
-static nsresult ChildAt(nsIDOMNode* aNode, PRInt32 aIndex, nsIDOMNode*& aChild)
+static nsresult ChildAt(nsIDOMNode* aNode, int32_t aIndex, nsIDOMNode*& aChild)
 {
   nsCOMPtr<nsIContent> content(do_QueryInterface(aNode));
 
-  aChild = nsnull;
+  aChild = nullptr;
 
   NS_ENSURE_TRUE(content, NS_ERROR_FAILURE);
 
@@ -694,7 +669,7 @@ static nsresult ChildAt(nsIDOMNode* aNode, PRInt32 aIndex, nsIDOMNode*& aChild)
   return NS_OK;
 }
 
-static PRInt32 IndexOf(nsIDOMNode* aParent, nsIDOMNode* aChild)
+static int32_t IndexOf(nsIDOMNode* aParent, nsIDOMNode* aChild)
 {
   nsCOMPtr<nsIContent> parent(do_QueryInterface(aParent));
   nsCOMPtr<nsIContent> child(do_QueryInterface(aChild));
@@ -705,9 +680,9 @@ static PRInt32 IndexOf(nsIDOMNode* aParent, nsIDOMNode* aChild)
   return parent->IndexOf(child);
 }
 
-static inline PRInt32 GetIndex(nsTArray<PRInt32>& aIndexArray)
+static inline int32_t GetIndex(nsTArray<int32_t>& aIndexArray)
 {
-  PRInt32 count = aIndexArray.Length();
+  int32_t count = aIndexArray.Length();
 
   if (count) {
     return aIndexArray.ElementAt(count - 1);
@@ -716,13 +691,13 @@ static inline PRInt32 GetIndex(nsTArray<PRInt32>& aIndexArray)
   return 0;
 }
 
-static nsresult GetNextNode(nsIDOMNode* aNode, nsTArray<PRInt32>& aIndexArray,
+static nsresult GetNextNode(nsIDOMNode* aNode, nsTArray<int32_t>& aIndexArray,
                             nsIDOMNode*& aNextNode,
                             nsRangeIterationDirection& aDirection)
 {
   bool hasChildren;
 
-  aNextNode = nsnull;
+  aNextNode = nullptr;
 
   aNode->HasChildNodes(&hasChildren);
 
@@ -745,10 +720,10 @@ static nsresult GetNextNode(nsIDOMNode* aNode, nsTArray<PRInt32>& aIndexArray,
     aNode->GetParentNode(getter_AddRefs(parent));
     NS_ENSURE_TRUE(parent, NS_ERROR_FAILURE);
 
-    PRInt32 count = aIndexArray.Length();
+    int32_t count = aIndexArray.Length();
 
     if (count) {
-      PRInt32 indx = aIndexArray.ElementAt(count - 1);
+      int32_t indx = aIndexArray.ElementAt(count - 1);
 
       ChildAt(parent, indx + 1, aNextNode);
 
@@ -757,7 +732,7 @@ static nsresult GetNextNode(nsIDOMNode* aNode, nsTArray<PRInt32>& aIndexArray,
       else
         aIndexArray.RemoveElementAt(count - 1);
     } else {
-      PRInt32 indx = IndexOf(parent, aNode);
+      int32_t indx = IndexOf(parent, aNode);
 
       if (indx >= 0) {
         ChildAt(parent, indx + 1, aNextNode);
@@ -791,7 +766,7 @@ nsresult
 nsDocumentEncoder::SerializeRangeNodes(nsRange* aRange,
                                        nsINode* aNode,
                                        nsAString& aString,
-                                       PRInt32 aDepth)
+                                       int32_t aDepth)
 {
   nsCOMPtr<nsIContent> content = do_QueryInterface(aNode);
   NS_ENSURE_TRUE(content, NS_ERROR_FAILURE);
@@ -804,12 +779,12 @@ nsDocumentEncoder::SerializeRangeNodes(nsRange* aRange,
   // get start and end nodes for this recursion level
   nsCOMPtr<nsIContent> startNode, endNode;
   {
-    PRInt32 start = mStartRootIndex - aDepth;
-    if (start >= 0 && (PRUint32)start <= mStartNodes.Length())
+    int32_t start = mStartRootIndex - aDepth;
+    if (start >= 0 && (uint32_t)start <= mStartNodes.Length())
       startNode = mStartNodes[start];
 
-    PRInt32 end = mEndRootIndex - aDepth;
-    if (end >= 0 && (PRUint32)end <= mEndNodes.Length())
+    int32_t end = mEndRootIndex - aDepth;
+    if (end >= 0 && (uint32_t)end <= mEndNodes.Length())
       endNode = mEndNodes[end];
   }
 
@@ -829,13 +804,13 @@ nsDocumentEncoder::SerializeRangeNodes(nsRange* aRange,
     {
       if (startNode == content)
       {
-        PRInt32 startOffset = aRange->StartOffset();
+        int32_t startOffset = aRange->StartOffset();
         rv = SerializeNodeStart(aNode, startOffset, -1, aString);
         NS_ENSURE_SUCCESS(rv, rv);
       }
       else
       {
-        PRInt32 endOffset = aRange->EndOffset();
+        int32_t endOffset = aRange->EndOffset();
         rv = SerializeNodeStart(aNode, 0, endOffset, aString);
         NS_ENSURE_SUCCESS(rv, rv);
       }
@@ -860,15 +835,15 @@ nsDocumentEncoder::SerializeRangeNodes(nsRange* aRange,
       
       // do some calculations that will tell us which children of this
       // node are in the range.
-      nsIContent* childAsNode = nsnull;
-      PRInt32 startOffset = 0, endOffset = -1;
+      nsIContent* childAsNode = nullptr;
+      int32_t startOffset = 0, endOffset = -1;
       if (startNode == content && mStartRootIndex >= aDepth)
         startOffset = mStartOffsets[mStartRootIndex - aDepth];
       if (endNode == content && mEndRootIndex >= aDepth)
         endOffset = mEndOffsets[mEndRootIndex - aDepth];
       // generated content will cause offset values of -1 to be returned.  
-      PRInt32 j;
-      PRUint32 childCount = content->GetChildCount();
+      int32_t j;
+      uint32_t childCount = content->GetChildCount();
 
       if (startOffset == -1) startOffset = 0;
       if (endOffset == -1) endOffset = childCount;
@@ -913,16 +888,23 @@ nsresult
 nsDocumentEncoder::SerializeRangeContextStart(const nsTArray<nsINode*>& aAncestorArray,
                                               nsAString& aString)
 {
-  PRInt32 i = aAncestorArray.Length();
+  if (mDisableContextSerialize) {
+    return NS_OK;
+  }
+  int32_t i = aAncestorArray.Length(), j;
   nsresult rv = NS_OK;
 
+  // currently only for table-related elements; see Bug 137450
+  j = GetImmediateContextCount(aAncestorArray);
+
   while (i > 0) {
     nsINode *node = aAncestorArray.ElementAt(--i);
 
     if (!node)
       break;
 
-    if (IncludeInContext(node)) {
+    // Either a general inclusion or as immediate context
+    if (IncludeInContext(node) || i < j) {
       rv = SerializeNodeStart(node, 0, -1, aString);
 
       if (NS_FAILED(rv))
@@ -937,17 +919,24 @@ nsresult
 nsDocumentEncoder::SerializeRangeContextEnd(const nsTArray<nsINode*>& aAncestorArray,
                                             nsAString& aString)
 {
-  PRInt32 i = 0;
-  PRInt32 count = aAncestorArray.Length();
+  if (mDisableContextSerialize) {
+    return NS_OK;
+  }
+  int32_t i = 0, j;
+  int32_t count = aAncestorArray.Length();
   nsresult rv = NS_OK;
 
+  // currently only for table-related elements
+  j = GetImmediateContextCount(aAncestorArray);
+
   while (i < count) {
     nsINode *node = aAncestorArray.ElementAt(i++);
 
     if (!node)
       break;
 
-    if (IncludeInContext(node)) {
+    // Either a general inclusion or as immediate context
+    if (IncludeInContext(node) || i - 1 < j) {
       rv = SerializeNodeEnd(node, aString);
 
       if (NS_FAILED(rv))
@@ -972,11 +961,11 @@ nsDocumentEncoder::SerializeRangeToString(nsRange *aRange,
   
   nsINode* startParent = aRange->GetStartParent();
   NS_ENSURE_TRUE(startParent, NS_ERROR_FAILURE);
-  PRInt32 startOffset = aRange->StartOffset();
+  int32_t startOffset = aRange->StartOffset();
 
   nsINode* endParent = aRange->GetEndParent();
   NS_ENSURE_TRUE(endParent, NS_ERROR_FAILURE);
-  PRInt32 endOffset = aRange->EndOffset();
+  int32_t endOffset = aRange->EndOffset();
 
   mCommonAncestors.Clear();
   mStartNodes.Clear();
@@ -1045,7 +1034,7 @@ nsDocumentEncoder::EncodeToString(nsAString& aOutputString)
   static_cast<PRUnichar*>(mCachedBuffer->Data())[0] = PRUnichar(0);
   mCachedBuffer->ToString(0, output, true);
   // output owns the buffer now!
-  mCachedBuffer = nsnull;
+  mCachedBuffer = nullptr;
   
 
   if (!mSerializer) {
@@ -1071,7 +1060,7 @@ nsDocumentEncoder::EncodeToString(nsAString& aOutputString)
 
   if (mSelection) {
     nsCOMPtr<nsIDOMRange> range;
-    PRInt32 i, count = 0;
+    int32_t i, count = 0;
 
     rv = mSelection->GetRangeCount(&count);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -1083,21 +1072,43 @@ nsDocumentEncoder::EncodeToString(nsAString& aOutputString)
       // Bug 236546: newlines not added when copying table cells into clipboard
       // Each selected cell shows up as a range containing a row with a single cell
       // get the row, compare it to previous row and emit </tr><tr> as needed
+      // Bug 137450: Problem copying/pasting a table from a web page to Excel.
+      // Each separate block of <tr></tr> produced above will be wrapped by the
+      // immediate context. This assumes that you can't select cells that are
+      // multiple selections from two tables simultaneously.
       range->GetStartContainer(getter_AddRefs(node));
       NS_ENSURE_TRUE(node, NS_ERROR_FAILURE);
       if (node != prevNode) {
+        nsCOMPtr<nsINode> p;
         if (prevNode) {
-          nsCOMPtr<nsINode> p = do_QueryInterface(prevNode);
+          p = do_QueryInterface(prevNode);
           rv = SerializeNodeEnd(p, output);
           NS_ENSURE_SUCCESS(rv, rv);
-          prevNode = nsnull;
         }
         nsCOMPtr<nsIContent> content = do_QueryInterface(node);
-        if (content && content->Tag() == nsGkAtoms::tr) {
-          nsCOMPtr<nsINode> n = do_QueryInterface(node);
+        if (content && content->IsHTML(nsGkAtoms::tr)) {
+          nsINode* n = content;
+          if (!prevNode) {
+            // Went from a non-<tr> to a <tr>
+            mCommonAncestors.Clear();
+            nsContentUtils::GetAncestors(n->GetNodeParent(), mCommonAncestors);
+            rv = SerializeRangeContextStart(mCommonAncestors, output);
+            NS_ENSURE_SUCCESS(rv, rv);
+            // Don't let SerializeRangeToString serialize the context again
+            mDisableContextSerialize = true;
+          }
+
           rv = SerializeNodeStart(n, 0, -1, output);
           NS_ENSURE_SUCCESS(rv, rv);
           prevNode = node;
+        } else if (prevNode) {
+          // Went from a <tr> to a non-<tr>
+          mCommonAncestors.Clear();
+          nsContentUtils::GetAncestors(p->GetNodeParent(), mCommonAncestors);
+          mDisableContextSerialize = false;
+          rv = SerializeRangeContextEnd(mCommonAncestors, output);
+          NS_ENSURE_SUCCESS(rv, rv);
+          prevNode = nullptr;
         }
       }
 
@@ -1105,17 +1116,26 @@ nsDocumentEncoder::EncodeToString(nsAString& aOutputString)
       rv = SerializeRangeToString(r, output);
       NS_ENSURE_SUCCESS(rv, rv);
     }
+
     if (prevNode) {
       nsCOMPtr<nsINode> p = do_QueryInterface(prevNode);
       rv = SerializeNodeEnd(p, output);
       NS_ENSURE_SUCCESS(rv, rv);
+      mCommonAncestors.Clear();
+      nsContentUtils::GetAncestors(p->GetNodeParent(), mCommonAncestors);
+      mDisableContextSerialize = false; 
+      rv = SerializeRangeContextEnd(mCommonAncestors, output);
+      NS_ENSURE_SUCCESS(rv, rv);
     }
 
-    mSelection = nsnull;
+    // Just to be safe
+    mDisableContextSerialize = false; 
+
+    mSelection = nullptr;
   } else if (mRange) {
       rv = SerializeRangeToString(mRange, output);
 
-      mRange = nsnull;
+      mRange = nullptr;
   } else if (mNode) {
     if (!mNodeFixup && !(mFlags & SkipInvisibleContent) && !mStream &&
         mNodeIsContainer) {
@@ -1123,7 +1143,7 @@ nsDocumentEncoder::EncodeToString(nsAString& aOutputString)
     } else {
       rv = SerializeToStringRecursive(mNode, output, mNodeIsContainer);
     }
-    mNode = nsnull;
+    mNode = nullptr;
   } else {
     rv = mSerializer->AppendDocumentStart(mDocument, output);
 
@@ -1149,7 +1169,7 @@ nsDocumentEncoder::EncodeToString(nsAString& aOutputString)
         mCachedBuffer->ToString(output.Length(), aOutputString);
         setOutput = true;
       }
-      mCachedBuffer = nsnull;
+      mCachedBuffer = nullptr;
     }
   }
 
@@ -1178,7 +1198,7 @@ nsDocumentEncoder::EncodeToStream(nsIOutputStream* aStream)
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (mMimeType.LowerCaseEqualsLiteral("text/plain")) {
-    rv = mUnicodeEncoder->SetOutputErrorBehavior(nsIUnicodeEncoder::kOnError_Replace, nsnull, '?');
+    rv = mUnicodeEncoder->SetOutputErrorBehavior(nsIUnicodeEncoder::kOnError_Replace, nullptr, '?');
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
@@ -1191,8 +1211,8 @@ nsDocumentEncoder::EncodeToStream(nsIOutputStream* aStream)
   // Force a flush of the last chunk of data.
   FlushText(buf, true);
 
-  mStream = nsnull;
-  mUnicodeEncoder = nsnull;
+  mStream = nullptr;
+  mUnicodeEncoder = nullptr;
 
   return rv;
 }
@@ -1232,7 +1252,7 @@ public:
   nsHTMLCopyEncoder();
   virtual ~nsHTMLCopyEncoder();
 
-  NS_IMETHOD Init(nsIDOMDocument* aDocument, const nsAString& aMimeType, PRUint32 aFlags);
+  NS_IMETHOD Init(nsIDOMDocument* aDocument, const nsAString& aMimeType, uint32_t aFlags);
 
   // overridden methods from nsDocumentEncoder
   NS_IMETHOD SetSelection(nsISelection* aSelection);
@@ -1250,18 +1270,20 @@ protected:
   
   nsresult PromoteRange(nsIDOMRange *inRange);
   nsresult PromoteAncestorChain(nsCOMPtr<nsIDOMNode> *ioNode, 
-                                PRInt32 *ioStartOffset, 
-                                PRInt32 *ioEndOffset);
-  nsresult GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32 aOffset, 
-                            nsCOMPtr<nsIDOMNode> *outNode, PRInt32 *outOffset, nsIDOMNode *aCommon);
-  nsCOMPtr<nsIDOMNode> GetChildAt(nsIDOMNode *aParent, PRInt32 aOffset);
+                                int32_t *ioStartOffset, 
+                                int32_t *ioEndOffset);
+  nsresult GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, int32_t aOffset, 
+                            nsCOMPtr<nsIDOMNode> *outNode, int32_t *outOffset, nsIDOMNode *aCommon);
+  nsCOMPtr<nsIDOMNode> GetChildAt(nsIDOMNode *aParent, int32_t aOffset);
   bool IsMozBR(nsIDOMNode* aNode);
-  nsresult GetNodeLocation(nsIDOMNode *inChild, nsCOMPtr<nsIDOMNode> *outParent, PRInt32 *outOffset);
+  nsresult GetNodeLocation(nsIDOMNode *inChild, nsCOMPtr<nsIDOMNode> *outParent, int32_t *outOffset);
   bool IsRoot(nsIDOMNode* aNode);
   bool IsFirstNode(nsIDOMNode *aNode);
   bool IsLastNode(nsIDOMNode *aNode);
   bool IsEmptyTextContent(nsIDOMNode* aNode);
   virtual bool IncludeInContext(nsINode *aNode);
+  virtual int32_t
+  GetImmediateContextCount(const nsTArray<nsINode*>& aAncestorArray);
 
   bool mIsTextWidget;
 };
@@ -1278,7 +1300,7 @@ nsHTMLCopyEncoder::~nsHTMLCopyEncoder()
 NS_IMETHODIMP
 nsHTMLCopyEncoder::Init(nsIDOMDocument* aDocument,
                         const nsAString& aMimeType,
-                        PRUint32 aFlags)
+                        uint32_t aFlags)
 {
   if (!aDocument)
     return NS_ERROR_INVALID_ARG;
@@ -1322,7 +1344,7 @@ nsHTMLCopyEncoder::SetSelection(nsISelection* aSelection)
   
   nsCOMPtr<nsIDOMRange> range;
   nsCOMPtr<nsIDOMNode> commonParent;
-  PRInt32 count = 0;
+  int32_t count = 0;
 
   nsresult rv = aSelection->GetRangeCount(&count);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -1398,7 +1420,7 @@ nsHTMLCopyEncoder::SetSelection(nsISelection* aSelection)
   // loop thru the ranges in the selection
   enumerator->First(); 
   nsCOMPtr<nsISupports> currentItem;
-  while ((NS_ENUMERATOR_FALSE == enumerator->IsDone()))
+  while (static_cast<nsresult>(NS_ENUMERATOR_FALSE) == enumerator->IsDone())
   {
     rv = enumerator->CurrentItem(getter_AddRefs(currentItem));
     NS_ENSURE_SUCCESS(rv, rv);
@@ -1442,8 +1464,8 @@ nsHTMLCopyEncoder::EncodeToStringWithContext(nsAString& aContextString,
   // where all the cells are in the same table.
 
   // leaf of ancestors might be text node.  If so discard it.
-  PRInt32 count = mCommonAncestors.Length();
-  PRInt32 i;
+  int32_t count = mCommonAncestors.Length();
+  int32_t i;
   nsCOMPtr<nsINode> node;
   if (count > 0)
     node = mCommonAncestors.ElementAt(0);
@@ -1529,7 +1551,7 @@ nsHTMLCopyEncoder::PromoteRange(nsIDOMRange *inRange)
   if (!inRange) return NS_ERROR_NULL_POINTER;
   nsresult rv;
   nsCOMPtr<nsIDOMNode> startNode, endNode, common;
-  PRInt32 startOffset, endOffset;
+  int32_t startOffset, endOffset;
   
   rv = inRange->GetCommonAncestorContainer(getter_AddRefs(common));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -1544,7 +1566,7 @@ nsHTMLCopyEncoder::PromoteRange(nsIDOMRange *inRange)
   
   nsCOMPtr<nsIDOMNode> opStartNode;
   nsCOMPtr<nsIDOMNode> opEndNode;
-  PRInt32 opStartOffset, opEndOffset;
+  int32_t opStartOffset, opEndOffset;
   nsCOMPtr<nsIDOMRange> opRange;
   
   // examine range endpoints.  
@@ -1574,8 +1596,8 @@ nsHTMLCopyEncoder::PromoteRange(nsIDOMRange *inRange)
 // promote the other.  Thus, instead of having a startnode/endNode, there is just the one ioNode.
 nsresult
 nsHTMLCopyEncoder::PromoteAncestorChain(nsCOMPtr<nsIDOMNode> *ioNode, 
-                                        PRInt32 *ioStartOffset, 
-                                        PRInt32 *ioEndOffset) 
+                                        int32_t *ioStartOffset, 
+                                        int32_t *ioEndOffset) 
 {
   if (!ioNode || !ioStartOffset || !ioEndOffset) return NS_ERROR_NULL_POINTER;
 
@@ -1583,7 +1605,7 @@ nsHTMLCopyEncoder::PromoteAncestorChain(nsCOMPtr<nsIDOMNode> *ioNode,
   bool done = false;
 
   nsCOMPtr<nsIDOMNode> frontNode, endNode, parent;
-  PRInt32 frontOffset, endOffset;
+  int32_t frontOffset, endOffset;
 
   //save the editable state of the ioNode, so we don't promote an ancestor if it has different editable state
   nsCOMPtr<nsINode> node = do_QueryInterface(*ioNode);
@@ -1622,13 +1644,13 @@ nsHTMLCopyEncoder::PromoteAncestorChain(nsCOMPtr<nsIDOMNode> *ioNode,
 }
 
 nsresult
-nsHTMLCopyEncoder::GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32 aOffset, 
-                                  nsCOMPtr<nsIDOMNode> *outNode, PRInt32 *outOffset, nsIDOMNode *common)
+nsHTMLCopyEncoder::GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, int32_t aOffset, 
+                                  nsCOMPtr<nsIDOMNode> *outNode, int32_t *outOffset, nsIDOMNode *common)
 {
   nsresult rv = NS_OK;
   nsCOMPtr<nsIDOMNode> node = aNode;
   nsCOMPtr<nsIDOMNode> parent = aNode;
-  PRInt32 offset = aOffset;
+  int32_t offset = aOffset;
   bool    bResetPromotion = false;
   
   // default values
@@ -1726,8 +1748,8 @@ nsHTMLCopyEncoder::GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32
     if (IsTextNode(n))
     {
       // if not at end of text node, we are done
-      PRUint32 len = n->Length();
-      if (offset < (PRInt32)len)
+      uint32_t len = n->Length();
+      if (offset < (int32_t)len)
       {
         // unless everything after us in just whitespace.  NOTE: we need a more
         // general solution that truly detects all cases of non-significant
@@ -1806,7 +1828,7 @@ nsHTMLCopyEncoder::GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32
 }
 
 nsCOMPtr<nsIDOMNode> 
-nsHTMLCopyEncoder::GetChildAt(nsIDOMNode *aParent, PRInt32 aOffset)
+nsHTMLCopyEncoder::GetChildAt(nsIDOMNode *aParent, int32_t aOffset)
 {
   nsCOMPtr<nsIDOMNode> resultNode;
   
@@ -1835,7 +1857,7 @@ nsHTMLCopyEncoder::IsMozBR(nsIDOMNode* aNode)
 nsresult 
 nsHTMLCopyEncoder::GetNodeLocation(nsIDOMNode *inChild,
                                    nsCOMPtr<nsIDOMNode> *outParent,
-                                   PRInt32 *outOffset)
+                                   int32_t *outOffset)
 {
   NS_ASSERTION((inChild && outParent && outOffset), "bad args");
   nsresult result = NS_ERROR_NULL_POINTER;
@@ -1875,7 +1897,7 @@ bool
 nsHTMLCopyEncoder::IsFirstNode(nsIDOMNode *aNode)
 {
   nsCOMPtr<nsIDOMNode> parent;
-  PRInt32 offset, j=0;
+  int32_t offset, j=0;
   nsresult rv = GetNodeLocation(aNode, address_of(parent), &offset);
   if (NS_FAILED(rv)) 
   {
@@ -1916,7 +1938,7 @@ bool
 nsHTMLCopyEncoder::IsLastNode(nsIDOMNode *aNode)
 {
   nsCOMPtr<nsIDOMNode> parent;
-  PRInt32 offset,j;
+  int32_t offset,j;
   nsresult rv = GetNodeLocation(aNode, address_of(parent), &offset);
   if (NS_FAILED(rv)) 
   {
@@ -1928,15 +1950,15 @@ nsHTMLCopyEncoder::IsLastNode(nsIDOMNode *aNode)
     return true;
   }
 
-  PRUint32 numChildren = parentNode->Length();
-  if (offset+1 == (PRInt32)numChildren) // easy case, we are last dom child
+  uint32_t numChildren = parentNode->Length();
+  if (offset+1 == (int32_t)numChildren) // easy case, we are last dom child
     return true;
   // need to check if any nodes after us are really visible.
   // Mike wrote something for me along these lines in nsSelectionController,
   // but I don't think it's ready for use yet - revisit.
   // HACK: for now, simply consider all whitespace text nodes to be 
   // invisible formatting nodes.
-  j = (PRInt32)numChildren-1;
+  j = (int32_t)numChildren-1;
   nsCOMPtr<nsIDOMNodeList>childList;
   nsCOMPtr<nsIDOMNode> child;
   rv = parent->GetChildNodes(getter_AddRefs(childList));
@@ -1975,3 +1997,26 @@ NS_NewHTMLCopyTextEncoder(nsIDocumentEncoder** aResult)
  NS_ADDREF(*aResult);
  return NS_OK;
 }
+
+int32_t
+nsHTMLCopyEncoder::GetImmediateContextCount(const nsTArray<nsINode*>& aAncestorArray)
+{
+  int32_t i = aAncestorArray.Length(), j = 0;
+  while (j < i) {
+    nsINode *node = aAncestorArray.ElementAt(j);
+    if (!node) {
+      break;
+    }
+    nsCOMPtr<nsIContent> content(do_QueryInterface(node));
+    if (!content || !content->IsHTML() || content->Tag() != nsGkAtoms::tr    &&
+                                          content->Tag() != nsGkAtoms::thead &&
+                                          content->Tag() != nsGkAtoms::tbody &&
+                                          content->Tag() != nsGkAtoms::tfoot &&
+                                          content->Tag() != nsGkAtoms::table) {
+      break;
+    }
+    ++j;
+  }
+  return j;
+}
+
