@@ -1,57 +1,26 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* vim:set ts=2 sw=2 sts=2 et cindent: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: ML 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla code.
- *
- * The Initial Developer of the Original Code is the Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2010
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *  Chris Double <chris.double@double.co.nz>
- *  Chris Pearce <chris@pearce.org.nz>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 #if !defined(nsBuiltinDecoderReader_h_)
 #define nsBuiltinDecoderReader_h_
 
 #include <nsDeque.h>
-#include "ImageLayers.h"
 #include "nsSize.h"
 #include "mozilla/ReentrantMonitor.h"
+#include "MediaStreamGraph.h"
+#include "SharedBuffer.h"
+#include "ImageLayers.h"
 
 // Stores info relevant to presenting media frames.
 class nsVideoInfo {
 public:
   nsVideoInfo()
-    : mAudioRate(0),
-      mAudioChannels(0),
+    : mAudioRate(44100),
+      mAudioChannels(2),
       mDisplay(0,0),
-      mStereoMode(mozilla::layers::STEREO_MODE_MONO),
+      mStereoMode(mozilla::STEREO_MODE_MONO),
       mHasAudio(false),
       mHasVideo(false)
   {}
@@ -65,17 +34,17 @@ public:
                                     const nsIntSize& aDisplay);
 
   // Sample rate.
-  PRUint32 mAudioRate;
+  uint32_t mAudioRate;
 
   // Number of audio channels.
-  PRUint32 mAudioChannels;
+  uint32_t mAudioChannels;
 
   // Size in pixels at which the video is rendered. This is after it has
   // been scaled by its aspect ratio.
   nsIntSize mDisplay;
 
   // Indicates the frame layout for single track stereo videos.
-  mozilla::layers::StereoMode mStereoMode;
+  mozilla::StereoMode mStereoMode;
 
   // True if we have an active audio bitstream.
   bool mHasAudio;
@@ -84,41 +53,39 @@ public:
   bool mHasVideo;
 };
 
-#ifdef MOZ_TREMOR
+#ifdef MOZ_SAMPLE_TYPE_S16LE
 #include <ogg/os_types.h>
 typedef ogg_int32_t VorbisPCMValue;
 typedef short AudioDataValue;
 
-#define MOZ_AUDIO_DATA_FORMAT (nsAudioStream::FORMAT_S16_LE)
 #define MOZ_CLIP_TO_15(x) ((x)<-32768?-32768:(x)<=32767?(x):32767)
 // Convert the output of vorbis_synthesis_pcmout to a AudioDataValue
 #define MOZ_CONVERT_VORBIS_SAMPLE(x) \
  (static_cast<AudioDataValue>(MOZ_CLIP_TO_15((x)>>9)))
 // Convert a AudioDataValue to a float for the Audio API
 #define MOZ_CONVERT_AUDIO_SAMPLE(x) ((x)*(1.F/32768))
-#define MOZ_SAMPLE_TYPE_S16LE 1
 
-#else /*MOZ_VORBIS*/
+#else /* MOZ_SAMPLE_TYPE_FLOAT32 */
 
 typedef float VorbisPCMValue;
 typedef float AudioDataValue;
 
-#define MOZ_AUDIO_DATA_FORMAT (nsAudioStream::FORMAT_FLOAT32)
 #define MOZ_CONVERT_VORBIS_SAMPLE(x) (x)
 #define MOZ_CONVERT_AUDIO_SAMPLE(x) (x)
-#define MOZ_SAMPLE_TYPE_FLOAT32 1
 
 #endif
 
 // Holds chunk a decoded audio frames.
 class AudioData {
 public:
-  AudioData(PRInt64 aOffset,
-            PRInt64 aTime,
-            PRInt64 aDuration,
-            PRUint32 aFrames,
+  typedef mozilla::SharedBuffer SharedBuffer;
+
+  AudioData(int64_t aOffset,
+            int64_t aTime,
+            int64_t aDuration,
+            uint32_t aFrames,
             AudioDataValue* aData,
-            PRUint32 aChannels)
+            uint32_t aChannels)
   : mOffset(aOffset),
     mTime(aTime),
     mDuration(aDuration),
@@ -134,14 +101,23 @@ public:
     MOZ_COUNT_DTOR(AudioData);
   }
 
+  // If mAudioBuffer is null, creates it from mAudioData.
+  void EnsureAudioBuffer();
+
+  int64_t GetEnd() { return mTime + mDuration; }
+
   // Approximate byte offset of the end of the page on which this chunk
   // ends.
-  const PRInt64 mOffset;
+  const int64_t mOffset;
 
-  PRInt64 mTime; // Start time of data in usecs.
-  const PRInt64 mDuration; // In usecs.
-  const PRUint32 mFrames;
-  const PRUint32 mChannels;
+  int64_t mTime; // Start time of data in usecs.
+  const int64_t mDuration; // In usecs.
+  const uint32_t mFrames;
+  const uint32_t mChannels;
+  // At least one of mAudioBuffer/mAudioData must be non-null.
+  // mChannels channels, each with mFrames frames
+  nsRefPtr<SharedBuffer> mAudioBuffer;
+  // mFrames frames, each with mChannels values
   nsAutoArrayPtr<AudioDataValue> mAudioData;
 };
 
@@ -157,10 +133,12 @@ public:
   //   2 = Cr
   struct YCbCrBuffer {
     struct Plane {
-      PRUint8* mData;
-      PRUint32 mWidth;
-      PRUint32 mHeight;
-      PRUint32 mStride;
+      uint8_t* mData;
+      uint32_t mWidth;
+      uint32_t mHeight;
+      uint32_t mStride;
+      uint32_t mOffset;
+      uint32_t mSkip;
     };
 
     Plane mPlanes[3];
@@ -168,35 +146,34 @@ public:
 
   // Constructs a VideoData object. Makes a copy of YCbCr data in aBuffer.
   // aTimecode is a codec specific number representing the timestamp of
-  // the frame of video data. Returns nsnull if an error occurs. This may
+  // the frame of video data. Returns nullptr if an error occurs. This may
   // indicate that memory couldn't be allocated to create the VideoData
   // object, or it may indicate some problem with the input data (e.g.
   // negative stride).
   static VideoData* Create(nsVideoInfo& aInfo,
                            ImageContainer* aContainer,
-                           PRInt64 aOffset,
-                           PRInt64 aTime,
-                           PRInt64 aEndTime,
+                           int64_t aOffset,
+                           int64_t aTime,
+                           int64_t aEndTime,
                            const YCbCrBuffer &aBuffer,
                            bool aKeyframe,
-                           PRInt64 aTimecode,
+                           int64_t aTimecode,
                            nsIntRect aPicture);
 
   // Constructs a duplicate VideoData object. This intrinsically tells the
   // player that it does not need to update the displayed frame when this
   // frame is played; this frame is identical to the previous.
-  static VideoData* CreateDuplicate(PRInt64 aOffset,
-                                    PRInt64 aTime,
-                                    PRInt64 aEndTime,
-                                    PRInt64 aTimecode)
+  static VideoData* CreateDuplicate(int64_t aOffset,
+                                    int64_t aTime,
+                                    int64_t aEndTime,
+                                    int64_t aTimecode)
   {
     return new VideoData(aOffset, aTime, aEndTime, aTimecode);
   }
 
-  ~VideoData()
-  {
-    MOZ_COUNT_DTOR(VideoData);
-  }
+  ~VideoData();
+
+  int64_t GetEnd() { return mEndTime; }
 
   // Dimensions at which to display the video frame. The picture region
   // will be scaled to this size. This is should be the picture region's
@@ -204,17 +181,17 @@ public:
   nsIntSize mDisplay;
 
   // Approximate byte offset of the end of the frame in the media.
-  PRInt64 mOffset;
+  int64_t mOffset;
 
   // Start time of frame in microseconds.
-  PRInt64 mTime;
+  int64_t mTime;
 
   // End time of frame in microseconds.
-  PRInt64 mEndTime;
+  int64_t mEndTime;
 
   // Codec specific internal time code. For Ogg based codecs this is the
   // granulepos.
-  PRInt64 mTimecode;
+  int64_t mTimecode;
 
   // This frame's image.
   nsRefPtr<Image> mImage;
@@ -225,35 +202,14 @@ public:
   bool mKeyframe;
 
 public:
-  VideoData(PRInt64 aOffset, PRInt64 aTime, PRInt64 aEndTime, PRInt64 aTimecode)
-    : mOffset(aOffset),
-      mTime(aTime),
-      mEndTime(aEndTime),
-      mTimecode(aTimecode),
-      mDuplicate(true),
-      mKeyframe(false)
-  {
-    MOZ_COUNT_CTOR(VideoData);
-    NS_ASSERTION(aEndTime >= aTime, "Frame must start before it ends.");
-  }
+  VideoData(int64_t aOffset, int64_t aTime, int64_t aEndTime, int64_t aTimecode);
 
-  VideoData(PRInt64 aOffset,
-            PRInt64 aTime,
-            PRInt64 aEndTime,
+  VideoData(int64_t aOffset,
+            int64_t aTime,
+            int64_t aEndTime,
             bool aKeyframe,
-            PRInt64 aTimecode,
-            nsIntSize aDisplay)
-    : mDisplay(aDisplay),
-      mOffset(aOffset),
-      mTime(aTime),
-      mEndTime(aEndTime),
-      mTimecode(aTimecode),
-      mDuplicate(false),
-      mKeyframe(aKeyframe)
-  {
-    MOZ_COUNT_CTOR(VideoData);
-    NS_ASSERTION(aEndTime >= aTime, "Frame must start before it ends.");
-  }
+            int64_t aTimecode,
+            nsIntSize aDisplay);
 
 };
 
@@ -262,7 +218,7 @@ template <class T>
 class MediaQueueDeallocator : public nsDequeFunctor {
   virtual void* operator() (void* anObject) {
     delete static_cast<T*>(anObject);
-    return nsnull;
+    return nullptr;
   }
 };
 
@@ -274,28 +230,28 @@ template <class T> class MediaQueue : private nsDeque {
    MediaQueue()
      : nsDeque(new MediaQueueDeallocator<T>()),
        mReentrantMonitor("mediaqueue"),
-       mEndOfStream(0)
+       mEndOfStream(false)
    {}
   
   ~MediaQueue() {
     Reset();
   }
 
-  inline PRInt32 GetSize() { 
+  inline int32_t GetSize() { 
     ReentrantMonitorAutoEnter mon(mReentrantMonitor);
     return nsDeque::GetSize();
   }
-  
+
   inline void Push(T* aItem) {
     ReentrantMonitorAutoEnter mon(mReentrantMonitor);
     nsDeque::Push(aItem);
   }
-  
+
   inline void PushFront(T* aItem) {
     ReentrantMonitorAutoEnter mon(mReentrantMonitor);
     nsDeque::PushFront(aItem);
   }
-  
+
   inline T* Pop() {
     ReentrantMonitorAutoEnter mon(mReentrantMonitor);
     return static_cast<T*>(nsDeque::Pop());
@@ -340,7 +296,7 @@ template <class T> class MediaQueue : private nsDeque {
     return GetSize() == 0 && mEndOfStream;
   }
 
-  // Returns true if the media queue has had it last item added to it.
+  // Returns true if the media queue has had its last item added to it.
   // This happens when the media stream has been completely decoded. Note this
   // does not mean that the corresponding stream has finished playback.
   bool IsFinished() {
@@ -355,7 +311,7 @@ template <class T> class MediaQueue : private nsDeque {
   }
 
   // Returns the approximate number of microseconds of items in the queue.
-  PRInt64 Duration() {
+  int64_t Duration() {
     ReentrantMonitorAutoEnter mon(mReentrantMonitor);
     if (GetSize() < 2) {
       return 0;
@@ -370,6 +326,25 @@ template <class T> class MediaQueue : private nsDeque {
     ForEach(aFunctor);
   }
 
+  // Extracts elements from the queue into aResult, in order.
+  // Elements whose start time is before aTime are ignored.
+  void GetElementsAfter(int64_t aTime, nsTArray<T*>* aResult) {
+    ReentrantMonitorAutoEnter mon(mReentrantMonitor);
+    if (!GetSize())
+      return;
+    int32_t i;
+    for (i = GetSize() - 1; i > 0; --i) {
+      T* v = static_cast<T*>(ObjectAt(i));
+      if (v->GetEnd() < aTime)
+        break;
+    }
+    // Elements less than i have a end time before aTime. It's also possible
+    // that the element at i has a end time before aTime, but that's OK.
+    for (; i < GetSize(); ++i) {
+      aResult->AppendElement(static_cast<T*>(ObjectAt(i)));
+    }
+  }
+
 private:
   mutable ReentrantMonitor mReentrantMonitor;
 
@@ -408,28 +383,30 @@ public:
   // than aTimeThreshold will be decoded (unless they're not keyframes
   // and aKeyframeSkip is true), but will not be added to the queue.
   virtual bool DecodeVideoFrame(bool &aKeyframeSkip,
-                                  PRInt64 aTimeThreshold) = 0;
+                                int64_t aTimeThreshold) = 0;
 
   virtual bool HasAudio() = 0;
   virtual bool HasVideo() = 0;
 
-  // Read header data for all bitstreams in the file. Fills mInfo with
-  // the data required to present the media. Returns NS_OK on success,
-  // or NS_ERROR_FAILURE on failure.
-  virtual nsresult ReadMetadata(nsVideoInfo* aInfo) = 0;
+  // Read header data for all bitstreams in the file. Fills aInfo with
+  // the data required to present the media, and optionally fills *aTags
+  // with tag metadata from the file.
+  // Returns NS_OK on success, or NS_ERROR_FAILURE on failure.
+  virtual nsresult ReadMetadata(nsVideoInfo* aInfo,
+                                nsHTMLMediaElement::MetadataTags** aTags) = 0;
 
   // Stores the presentation time of the first frame we'd be able to play if
   // we started playback at the current position. Returns the first video
   // frame, if we have video.
-  VideoData* FindStartTime(PRInt64& aOutStartTime);
+  VideoData* FindStartTime(int64_t& aOutStartTime);
 
   // Moves the decode head to aTime microseconds. aStartTime and aEndTime
   // denote the start and end times of the media in usecs, and aCurrentTime
   // is the current playback position in microseconds.
-  virtual nsresult Seek(PRInt64 aTime,
-                        PRInt64 aStartTime,
-                        PRInt64 aEndTime,
-                        PRInt64 aCurrentTime) = 0;
+  virtual nsresult Seek(int64_t aTime,
+                        int64_t aStartTime,
+                        int64_t aEndTime,
+                        int64_t aCurrentTime) = 0;
 
   // Queue of audio frames. This queue is threadsafe, and is accessed from
   // the audio, decoder, state machine, and main threads.
@@ -444,29 +421,21 @@ public:
   // the media time corresponding to playback time/position 0. This function
   // should only be called on the main thread.
   virtual nsresult GetBuffered(nsTimeRanges* aBuffered,
-                               PRInt64 aStartTime) = 0;
+                               int64_t aStartTime) = 0;
+
+  // True if we can seek using only buffered ranges. This is backend dependant.
+  virtual bool IsSeekableInBufferedRanges() = 0;
 
   class VideoQueueMemoryFunctor : public nsDequeFunctor {
   public:
     VideoQueueMemoryFunctor() : mResult(0) {}
 
-    virtual void* operator()(void* anObject) {
-      const VideoData* v = static_cast<const VideoData*>(anObject);
-      if (!v->mImage) {
-        return nsnull;
-      }
-      NS_ASSERTION(v->mImage->GetFormat() == mozilla::layers::Image::PLANAR_YCBCR,
-                   "Wrong format?");
-      mozilla::layers::PlanarYCbCrImage* vi = static_cast<mozilla::layers::PlanarYCbCrImage*>(v->mImage.get());
+    virtual void* operator()(void* anObject);
 
-      mResult += vi->GetDataSize();
-      return nsnull;
-    }
-
-    PRInt64 mResult;
+    int64_t mResult;
   };
 
-  PRInt64 VideoQueueMemoryInUse() {
+  int64_t VideoQueueMemoryInUse() {
     VideoQueueMemoryFunctor functor;
     mVideoQueue.LockedForEach(functor);
     return functor.mResult;
@@ -479,13 +448,13 @@ public:
     virtual void* operator()(void* anObject) {
       const AudioData* audioData = static_cast<const AudioData*>(anObject);
       mResult += audioData->mFrames * audioData->mChannels * sizeof(AudioDataValue);
-      return nsnull;
+      return nullptr;
     }
 
-    PRInt64 mResult;
+    int64_t mResult;
   };
 
-  PRInt64 AudioQueueMemoryInUse() {
+  int64_t AudioQueueMemoryInUse() {
     AudioQueueMemoryFunctor functor;
     mAudioQueue.LockedForEach(functor);
     return functor.mResult;
@@ -493,13 +462,13 @@ public:
 
   // Only used by nsWebMReader for now, so stub here rather than in every
   // reader than inherits from nsBuiltinDecoderReader.
-  virtual void NotifyDataArrived(const char* aBuffer, PRUint32 aLength, PRInt64 aOffset) {}
+  virtual void NotifyDataArrived(const char* aBuffer, uint32_t aLength, int64_t aOffset) {}
 
 protected:
 
   // Pumps the decode until we reach frames required to play at time aTarget
   // (usecs).
-  nsresult DecodeToTarget(PRInt64 aTarget);
+  nsresult DecodeToTarget(int64_t aTarget);
 
   // Reader decode function. Matches DecodeVideoFrame() and
   // DecodeAudioData().
@@ -511,7 +480,7 @@ protected:
   Data* DecodeToFirstData(DecodeFn aDecodeFn,
                           MediaQueue<Data>& aQueue);
 
-  // Wrapper so that DecodeVideoFrame(bool&,PRInt64) can be called from
+  // Wrapper so that DecodeVideoFrame(bool&,int64_t) can be called from
   // DecodeToFirstData().
   bool DecodeVideoFrame() {
     bool f = false;
