@@ -1,40 +1,8 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* vim:set ts=2 sw=2 sts=2 et cindent: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla code.
- *
- * The Initial Developer of the Original Code is the Mozilla Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2009
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *  Robert O'Callahan <robert@ocallahan.org>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifndef nsMediaCache_h_
 #define nsMediaCache_h_
@@ -230,13 +198,12 @@ public:
   // aClient provides the underlying transport that cache will use to read
   // data for this stream.
   nsMediaCacheStream(ChannelMediaResource* aClient)
-    : mClient(aClient), mResourceID(0), mInitialized(false),
+    : mClient(aClient), mInitialized(false),
       mHasHadUpdate(false),
       mClosed(false),
-      mDidNotifyDataEnded(false),
+      mDidNotifyDataEnded(false), mResourceID(0),
       mIsSeekable(false), mCacheSuspended(false),
       mChannelEnded(false),
-      mUsingNullPrincipal(false),
       mChannelOffset(0), mStreamLength(-1),  
       mStreamOffset(0), mPlaybackBytesPerSecond(10000),
       mPinCount(0), mCurrentMode(MODE_PLAYBACK),
@@ -273,7 +240,8 @@ public:
     return !mClosed &&
       (!mDidNotifyDataEnded || NS_SUCCEEDED(mNotifyDataEndedStatus));
   }
-  // Get the principal for this stream.
+  // Get the principal for this stream. Anything accessing the contents of
+  // this stream must have a principal that subsumes this principal.
   nsIPrincipal* GetCurrentPrincipal() { return mPrincipal; }
   // Ensure a global media cache update has run with this stream present.
   // This ensures the cache has had a chance to suspend or unsuspend this stream.
@@ -294,7 +262,7 @@ public:
   // data available based on an incorrect reported length. Seeks relative
   // EOF also depend on the reported length if we haven't managed to
   // read the whole stream yet.
-  void NotifyDataLength(PRInt64 aLength);
+  void NotifyDataLength(int64_t aLength);
   // Notifies the cache that a load has begun. We pass the offset
   // because in some cases the offset might not be what the cache
   // requested. In particular we might unexpectedly start providing
@@ -302,14 +270,14 @@ public:
   // offset that the cache requested in
   // ChannelMediaResource::CacheClientSeek. This can be called at any
   // time by the client, not just after a CacheClientSeek.
-  void NotifyDataStarted(PRInt64 aOffset);
+  void NotifyDataStarted(int64_t aOffset);
   // Notifies the cache that data has been received. The stream already
   // knows the offset because data is received in sequence and
   // the starting offset is known via NotifyDataStarted or because
   // the cache requested the offset in
   // ChannelMediaResource::CacheClientSeek, or because it defaulted to 0.
   // We pass in the principal that was used to load this data.
-  void NotifyDataReceived(PRInt64 aSize, const char* aData,
+  void NotifyDataReceived(int64_t aSize, const char* aData,
                           nsIPrincipal* aPrincipal);
   // Notifies the cache that the channel has closed with the given status.
   void NotifyDataEnded(nsresult aStatus);
@@ -325,15 +293,16 @@ public:
   // the stream ended normally we return the length we actually got.
   // If we've successfully read data beyond the originally reported length,
   // we return the end of the data we've read.
-  PRInt64 GetLength();
-  // Returns the unique resource ID
-  PRInt64 GetResourceID() { return mResourceID; }
+  int64_t GetLength();
+  // Returns the unique resource ID. Call only on the main thread or while
+  // holding the media cache lock.
+  int64_t GetResourceID() { return mResourceID; }
   // Returns the end of the bytes starting at the given offset
   // which are in cache.
-  PRInt64 GetCachedDataEnd(PRInt64 aOffset);
+  int64_t GetCachedDataEnd(int64_t aOffset);
   // Returns the offset of the first byte of cached data at or after aOffset,
   // or -1 if there is no such cached data.
-  PRInt64 GetNextCachedData(PRInt64 aOffset);
+  int64_t GetNextCachedData(int64_t aOffset);
   // Fills aRanges with the ByteRanges representing the data which is currently
   // cached. Locks the media cache while running, to prevent any ranges
   // growing. The stream should be pinned while this runs and while its results
@@ -345,21 +314,21 @@ public:
   // thread. It's the caller's responsibility to wrap the call in a pin/unpin,
   // and also to check that the range they want is cached before calling this.
   nsresult ReadFromCache(char* aBuffer,
-                         PRInt64 aOffset,
-                         PRInt64 aCount);
+                         int64_t aOffset,
+                         int64_t aCount);
 
   // IsDataCachedToEndOfStream returns true if all the data from
   // aOffset to the end of the stream (the server-reported end, if the
   // real end is not known) is in cache. If we know nothing about the
   // end of the stream, this returns false.
-  bool IsDataCachedToEndOfStream(PRInt64 aOffset);
+  bool IsDataCachedToEndOfStream(int64_t aOffset);
   // The mode is initially MODE_PLAYBACK.
   void SetReadMode(ReadMode aMode);
   // This is the client's estimate of the playback rate assuming
   // the media plays continuously. The cache can't guess this itself
   // because it doesn't know when the decoder was paused, buffering, etc.
   // Do not pass zero.
-  void SetPlaybackRate(PRUint32 aBytesPerSecond);
+  void SetPlaybackRate(uint32_t aBytesPerSecond);
   // Returns the last set value of SetSeekable.
   bool IsSeekable();
 
@@ -374,13 +343,13 @@ public:
   // stream.
   // This can fail when aWhence is NS_SEEK_END and no stream length
   // is known.
-  nsresult Seek(PRInt32 aWhence, PRInt64 aOffset);
-  PRInt64 Tell();
+  nsresult Seek(int32_t aWhence, int64_t aOffset);
+  int64_t Tell();
   // *aBytes gets the number of bytes that were actually read. This can
   // be less than aCount. If the first byte of data is not in the cache,
   // this will block until the data is available or the stream is
   // closed, otherwise it won't block.
-  nsresult Read(char* aBuffer, PRUint32 aCount, PRUint32* aBytes);
+  nsresult Read(char* aBuffer, uint32_t aCount, uint32_t* aBytes);
 
 private:
   friend class nsMediaCache;
@@ -401,23 +370,23 @@ private:
       NS_ASSERTION(mFirstBlock == -1 && mCount == 0,
                    "Destroying non-empty block list");
     }
-    void AddFirstBlock(PRInt32 aBlock);
-    void AddAfter(PRInt32 aBlock, PRInt32 aBefore);
-    void RemoveBlock(PRInt32 aBlock);
+    void AddFirstBlock(int32_t aBlock);
+    void AddAfter(int32_t aBlock, int32_t aBefore);
+    void RemoveBlock(int32_t aBlock);
     // Returns the first block in the list, or -1 if empty
-    PRInt32 GetFirstBlock() const { return mFirstBlock; }
+    int32_t GetFirstBlock() const { return mFirstBlock; }
     // Returns the last block in the list, or -1 if empty
-    PRInt32 GetLastBlock() const;
+    int32_t GetLastBlock() const;
     // Returns the next block in the list after aBlock or -1 if
     // aBlock is the last block
-    PRInt32 GetNextBlock(PRInt32 aBlock) const;
+    int32_t GetNextBlock(int32_t aBlock) const;
     // Returns the previous block in the list before aBlock or -1 if
     // aBlock is the first block
-    PRInt32 GetPrevBlock(PRInt32 aBlock) const;
+    int32_t GetPrevBlock(int32_t aBlock) const;
     bool IsEmpty() const { return mFirstBlock < 0; }
-    PRInt32 GetCount() const { return mCount; }
+    int32_t GetCount() const { return mCount; }
     // The contents of aBlockIndex1 and aBlockIndex2 have been swapped
-    void NotifyBlockSwapped(PRInt32 aBlockIndex1, PRInt32 aBlockIndex2);
+    void NotifyBlockSwapped(int32_t aBlockIndex1, int32_t aBlockIndex2);
 #ifdef DEBUG
     // Verify linked-list invariants
     void Verify();
@@ -431,27 +400,27 @@ private:
       Entry(const Entry& toCopy) : nsUint32HashKey(&toCopy.GetKey()),
         mNextBlock(toCopy.mNextBlock), mPrevBlock(toCopy.mPrevBlock) {}
 
-      PRInt32 mNextBlock;
-      PRInt32 mPrevBlock;
+      int32_t mNextBlock;
+      int32_t mPrevBlock;
     };
     nsTHashtable<Entry> mEntries;
 
     // The index of the first block in the list, or -1 if the list is empty.
-    PRInt32 mFirstBlock;
+    int32_t mFirstBlock;
     // The number of blocks in the list.
-    PRInt32 mCount;
+    int32_t mCount;
   };
 
   // Returns the end of the bytes starting at the given offset
   // which are in cache.
   // This method assumes that the cache monitor is held and can be called on
   // any thread.
-  PRInt64 GetCachedDataEndInternal(PRInt64 aOffset);
+  int64_t GetCachedDataEndInternal(int64_t aOffset);
   // Returns the offset of the first byte of cached data at or after aOffset,
   // or -1 if there is no such cached data.
   // This method assumes that the cache monitor is held and can be called on
   // any thread.
-  PRInt64 GetNextCachedDataInternal(PRInt64 aOffset);
+  int64_t GetNextCachedDataInternal(int64_t aOffset);
   // A helper function to do the work of closing the stream. Assumes
   // that the cache monitor is held. Main thread only.
   // aReentrantMonitor is the nsAutoReentrantMonitor wrapper holding the cache monitor.
@@ -459,15 +428,11 @@ private:
   // blocked on reading from this stream.
   void CloseInternal(ReentrantMonitorAutoEnter& aReentrantMonitor);
   // Update mPrincipal given that data has been received from aPrincipal
-  void UpdatePrincipal(nsIPrincipal* aPrincipal);
+  bool UpdatePrincipal(nsIPrincipal* aPrincipal);
 
   // These fields are main-thread-only.
   ChannelMediaResource*  mClient;
   nsCOMPtr<nsIPrincipal> mPrincipal;
-  // This is a unique ID representing the resource we're loading.
-  // All streams with the same mResourceID are loading the same
-  // underlying resource and should share data.
-  PRInt64                mResourceID;
   // Set to true when Init or InitAsClone has been called
   bool                   mInitialized;
   // Set to true when nsMediaCache::Update() has finished while this stream
@@ -479,9 +444,14 @@ private:
   // True if CacheClientNotifyDataEnded has been called for this stream.
   bool                   mDidNotifyDataEnded;
 
-  // The following fields are protected by the cache's monitor but are
-  // only written on the main thread. 
+  // The following fields must be written holding the cache's monitor and
+  // only on the main thread, thus can be read either on the main thread
+  // or while holding the cache's monitor.
 
+  // This is a unique ID representing the resource we're loading.
+  // All streams with the same mResourceID are loading the same
+  // underlying resource and should share data.
+  int64_t mResourceID;
   // The last reported seekability state for the underlying channel
   bool mIsSeekable;
   // True if the cache has suspended our channel because the cache is
@@ -490,23 +460,20 @@ private:
   bool mCacheSuspended;
   // True if the channel ended and we haven't seeked it again.
   bool mChannelEnded;
-  // True if mPrincipal is a null principal because we saw data from
-  // multiple origins
-  bool mUsingNullPrincipal;
   // The offset where the next data from the channel will arrive
-  PRInt64      mChannelOffset;
+  int64_t      mChannelOffset;
   // The reported or discovered length of the data, or -1 if nothing is
   // known
-  PRInt64      mStreamLength;
+  int64_t      mStreamLength;
 
   // The following fields are protected by the cache's monitor can can be written
   // by any thread.
 
   // The offset where the reader is positioned in the stream
-  PRInt64           mStreamOffset;
+  int64_t           mStreamOffset;
   // For each block in the stream data, maps to the cache entry for the
   // block, or -1 if the block is not cached.
-  nsTArray<PRInt32> mBlocks;
+  nsTArray<int32_t> mBlocks;
   // The list of read-ahead blocks, ordered by stream offset; the first
   // block is the earliest in the stream (so the last block will be the
   // least valuable).
@@ -516,10 +483,10 @@ private:
   // The list of played-back blocks; the first block is the most recently used
   BlockList         mPlayedBlocks;
   // The last reported estimate of the decoder's playback rate
-  PRUint32          mPlaybackBytesPerSecond;
+  uint32_t          mPlaybackBytesPerSecond;
   // The number of times this stream has been Pinned without a
   // corresponding Unpin
-  PRUint32          mPinCount;
+  uint32_t          mPinCount;
   // The status used when we did CacheClientNotifyDataEnded. Only valid
   // when mDidNotifyDataEnded is true.
   nsresult          mNotifyDataEndedStatus;
@@ -535,8 +502,8 @@ private:
   // to wait here so we can write back a complete block. The first
   // mChannelOffset%BLOCK_SIZE bytes have been filled in with good data,
   // the rest are garbage.
-  // Use PRInt64 so that the data is well-aligned.
-  PRInt64           mPartialBlockBuffer[BLOCK_SIZE/sizeof(PRInt64)];
+  // Use int64_t so that the data is well-aligned.
+  int64_t           mPartialBlockBuffer[BLOCK_SIZE/sizeof(int64_t)];
 };
 
 #endif
