@@ -1,41 +1,8 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* vim:set ts=2 sw=2 sts=2 et cindent: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla code.
- *
- * The Initial Developer of the Original Code is the Mozilla Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2010
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *  Chris Double <chris.double@double.co.nz>
- *  Chris Pearce <chris@pearce.org.nz>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 #include "nsDebug.h"
 #include "nsOggCodecState.h"
 #include "nsOggDecoder.h"
@@ -45,6 +12,9 @@
 #include "nsBuiltinDecoderReader.h"
 
 #include "mozilla/StandardInteger.h"
+#include "mozilla/Util.h" // DebugOnly
+
+using namespace mozilla;
 
 #ifdef PR_LOGGING
 extern PRLogModuleInfo* gBuiltinDecoderLog;
@@ -53,6 +23,36 @@ extern PRLogModuleInfo* gBuiltinDecoderLog;
 #define LOG(type, msg)
 #endif
 
+// Reads a little-endian encoded unsigned 32bit integer at p.
+static uint32_t LEUint32(const unsigned char* p)
+{
+  return p[0] +
+        (p[1] << 8) +
+        (p[2] << 16) +
+        (p[3] << 24);
+}
+
+// Reads a little-endian encoded 64bit integer at p.
+static int64_t LEInt64(const unsigned char* p)
+{
+  uint32_t lo = LEUint32(p);
+  uint32_t hi = LEUint32(p + 4);
+  return static_cast<int64_t>(lo) | (static_cast<int64_t>(hi) << 32);
+}
+
+// Reads a little-endian encoded unsigned 16bit integer at p.
+static uint16_t LEUint16(const unsigned char* p)
+{
+  return p[0] + (p[1] << 8);
+}
+
+// Reads a little-endian encoded signed 16bit integer at p.
+static int16_t LEInt16(const unsigned char* p)
+{
+  return static_cast<int16_t>(LEUint16(p));
+}
+
+/** Decoder base class for Ogg-encapsulated streams. */
 nsOggCodecState*
 nsOggCodecState::Create(ogg_page* aPage)
 {
@@ -62,12 +62,16 @@ nsOggCodecState::Create(ogg_page* aPage)
     codecState = new nsTheoraState(aPage);
   } else if (aPage->body_len > 6 && memcmp(aPage->body+1, "vorbis", 6) == 0) {
     codecState = new nsVorbisState(aPage);
+#ifdef MOZ_OPUS
+  } else if (aPage->body_len > 8 && memcmp(aPage->body, "OpusHead", 8) == 0) {
+    codecState = new nsOpusState(aPage);
+#endif
   } else if (aPage->body_len > 8 && memcmp(aPage->body, "fishead\0", 8) == 0) {
     codecState = new nsSkeletonState(aPage);
   } else {
     codecState = new nsOggCodecState(aPage, false);
   }
-  return codecState->nsOggCodecState::Init() ? codecState.forget() : nsnull;
+  return codecState->nsOggCodecState::Init() ? codecState.forget() : nullptr;
 }
 
 nsOggCodecState::nsOggCodecState(ogg_page* aBosPage, bool aActive) :
@@ -101,7 +105,7 @@ nsresult nsOggCodecState::Reset() {
 
 void nsOggCodecState::ClearUnstamped()
 {
-  for (PRUint32 i = 0; i < mUnstamped.Length(); ++i) {
+  for (uint32_t i = 0; i < mUnstamped.Length(); ++i) {
     nsOggCodecState::ReleasePacket(mUnstamped[i]);
   }
   mUnstamped.Clear();
@@ -158,7 +162,7 @@ void nsPacketQueue::Append(ogg_packet* aPacket) {
 
 ogg_packet* nsOggCodecState::PacketOut() {
   if (mPackets.IsEmpty()) {
-    return nsnull;
+    return nullptr;
   }
   return mPackets.PopFront();
 }
@@ -166,7 +170,7 @@ ogg_packet* nsOggCodecState::PacketOut() {
 nsresult nsOggCodecState::PageIn(ogg_page* aPage) {
   if (!mActive)
     return NS_OK;
-  NS_ASSERTION(static_cast<PRUint32>(ogg_page_serialno(aPage)) == mSerial,
+  NS_ASSERTION(static_cast<uint32_t>(ogg_page_serialno(aPage)) == mSerial,
                "Page must be for this stream!");
   if (ogg_stream_pagein(&mState, aPage) == -1)
     return NS_ERROR_FAILURE;
@@ -237,8 +241,8 @@ bool nsTheoraState::Init() {
   if (!mActive)
     return false;
 
-  PRInt64 n = mInfo.aspect_numerator;
-  PRInt64 d = mInfo.aspect_denominator;
+  int64_t n = mInfo.aspect_numerator;
+  int64_t d = mInfo.aspect_denominator;
 
   mPixelAspectRatio = (n == 0 || d == 0) ?
     1.0f : static_cast<float>(n) / static_cast<float>(d);
@@ -262,6 +266,7 @@ bool nsTheoraState::Init() {
 bool
 nsTheoraState::DecodeHeader(ogg_packet* aPacket)
 {
+  nsAutoRef<ogg_packet> autoRelease(aPacket);
   mPacketCount++;
   int ret = th_decode_headerin(&mInfo,
                                &mComment,
@@ -285,18 +290,18 @@ nsTheoraState::DecodeHeader(ogg_packet* aPacket)
   bool isSetupHeader = aPacket->bytes > 0 && aPacket->packet[0] == 0x82;
   if (ret < 0 || mPacketCount > 3) {
     // We've received an error, or the first three packets weren't valid
-    // header packets, assume bad input, and don't activate the bitstream.
-    mDoneReadingHeaders = true;
+    // header packets. Assume bad input.
+    // Our caller will deactivate the bitstream.
+    return false;
   } else if (ret > 0 && isSetupHeader && mPacketCount == 3) {
     // Successfully read the three header packets.
     mDoneReadingHeaders = true;
-    mActive = true;
   }
-  return mDoneReadingHeaders;
+  return true;
 }
 
-PRInt64
-nsTheoraState::Time(PRInt64 granulepos) {
+int64_t
+nsTheoraState::Time(int64_t granulepos) {
   if (!mActive) {
     return -1;
   }
@@ -313,7 +318,7 @@ nsTheoraState::IsHeader(ogg_packet* aPacket) {
  (((_info)->version_minor>(_min)||(_info)->version_minor==(_min))&& \
  (_info)->version_subminor>=(_sub)))
 
-PRInt64 nsTheoraState::Time(th_info* aInfo, PRInt64 aGranulepos)
+int64_t nsTheoraState::Time(th_info* aInfo, int64_t aGranulepos)
 {
   if (aGranulepos < 0 || aInfo->fps_numerator == 0) {
     return -1;
@@ -323,25 +328,25 @@ PRInt64 nsTheoraState::Time(th_info* aInfo, PRInt64 aGranulepos)
   int shift = aInfo->keyframe_granule_shift; 
   ogg_int64_t iframe = aGranulepos >> shift;
   ogg_int64_t pframe = aGranulepos - (iframe << shift);
-  PRInt64 frameno = iframe + pframe - TH_VERSION_CHECK(aInfo, 3, 2, 1);
+  int64_t frameno = iframe + pframe - TH_VERSION_CHECK(aInfo, 3, 2, 1);
   CheckedInt64 t = ((CheckedInt64(frameno) + 1) * USECS_PER_S) * aInfo->fps_denominator;
-  if (!t.valid())
+  if (!t.isValid())
     return -1;
   t /= aInfo->fps_numerator;
-  return t.valid() ? t.value() : -1;
+  return t.isValid() ? t.value() : -1;
 }
 
-PRInt64 nsTheoraState::StartTime(PRInt64 granulepos) {
+int64_t nsTheoraState::StartTime(int64_t granulepos) {
   if (granulepos < 0 || !mActive || mInfo.fps_numerator == 0) {
     return -1;
   }
   CheckedInt64 t = (CheckedInt64(th_granule_frame(mCtx, granulepos)) * USECS_PER_S) * mInfo.fps_denominator;
-  if (!t.valid())
+  if (!t.isValid())
     return -1;
   return t.value() / mInfo.fps_numerator;
 }
 
-PRInt64
+int64_t
 nsTheoraState::MaxKeyframeOffset()
 {
   // Determine the maximum time in microseconds by which a key frame could
@@ -349,10 +354,10 @@ nsTheoraState::MaxKeyframeOffset()
   // ((key_frame_number << granule_shift) + frame_offset).
   // Therefore the maximum possible time by which any frame could be offset
   // from a keyframe is the duration of (1 << granule_shift) - 1) frames.
-  PRInt64 frameDuration;
+  int64_t frameDuration;
   
   // Max number of frames keyframe could possibly be offset.
-  PRInt64 keyframeDiff = (1 << mInfo.keyframe_granule_shift) - 1;
+  int64_t keyframeDiff = (1 << mInfo.keyframe_granule_shift) - 1;
 
   // Length of frame in usecs.
   frameDuration = (mInfo.fps_denominator * USECS_PER_S) / mInfo.fps_numerator;
@@ -366,7 +371,7 @@ nsTheoraState::PageIn(ogg_page* aPage)
 {
   if (!mActive)
     return NS_OK;
-  NS_ASSERTION(static_cast<PRUint32>(ogg_page_serialno(aPage)) == mSerial,
+  NS_ASSERTION(static_cast<uint32_t>(ogg_page_serialno(aPage)) == mSerial,
                "Page must be for this stream!");
   if (ogg_stream_pagein(&mState, aPage) == -1)
     return NS_ERROR_FAILURE;
@@ -378,7 +383,7 @@ nsTheoraState::PageIn(ogg_page* aPage)
     // We've found a packet with a granulepos, and we've loaded our metadata
     // and initialized our decoder. Determine granulepos of buffered packets.
     ReconstructTheoraGranulepos();
-    for (PRUint32 i = 0; i < mUnstamped.Length(); ++i) {
+    for (uint32_t i = 0; i < mUnstamped.Length(); ++i) {
       ogg_packet* packet = mUnstamped[i];
 #ifdef DEBUG
       NS_ASSERTION(!IsHeader(packet), "Don't try to recover header packet gp");
@@ -437,7 +442,7 @@ void nsTheoraState::ReconstructTheoraGranulepos()
   // variable in the loop below, store the frame number for Theora
   // version >= 3.2.1 streams, and store the frame index for Theora
   // version < 3.2.1 streams.
-  for (PRUint32 i = 0; i < mUnstamped.Length() - 1; ++i) {
+  for (uint32_t i = 0; i < mUnstamped.Length() - 1; ++i) {
     ogg_int64_t frame = firstFrame + i;
     ogg_int64_t granulepos;
     ogg_packet* packet = mUnstamped[i];
@@ -523,6 +528,7 @@ nsVorbisState::~nsVorbisState() {
 }
 
 bool nsVorbisState::DecodeHeader(ogg_packet* aPacket) {
+  nsAutoRef<ogg_packet> autoRelease(aPacket);
   mPacketCount++;
   int ret = vorbis_synthesis_headerin(&mInfo,
                                       &mComment,
@@ -547,15 +553,15 @@ bool nsVorbisState::DecodeHeader(ogg_packet* aPacket) {
 
   if (ret < 0 || mPacketCount > 3) {
     // We've received an error, or the first three packets weren't valid
-    // header packets, assume bad input, and deactivate the bitstream.
-    mDoneReadingHeaders = true;
-    mActive = false;
+    // header packets. Assume bad input. Our caller will deactivate the
+    // bitstream.
+    return false;
   } else if (ret == 0 && isSetupHeader && mPacketCount == 3) {
     // Successfully read the three header packets.
     // The bitstream remains active.
     mDoneReadingHeaders = true;
   }
-  return mDoneReadingHeaders;
+  return true;
 }
 
 bool nsVorbisState::Init()
@@ -579,7 +585,7 @@ bool nsVorbisState::Init()
   return true;
 }
 
-PRInt64 nsVorbisState::Time(PRInt64 granulepos)
+int64_t nsVorbisState::Time(int64_t granulepos)
 {
   if (!mActive) {
     return -1;
@@ -588,13 +594,13 @@ PRInt64 nsVorbisState::Time(PRInt64 granulepos)
   return nsVorbisState::Time(&mInfo, granulepos);
 }
 
-PRInt64 nsVorbisState::Time(vorbis_info* aInfo, PRInt64 aGranulepos)
+int64_t nsVorbisState::Time(vorbis_info* aInfo, int64_t aGranulepos)
 {
   if (aGranulepos == -1 || aInfo->rate == 0) {
     return -1;
   }
   CheckedInt64 t = CheckedInt64(aGranulepos) * USECS_PER_S;
-  if (!t.valid())
+  if (!t.isValid())
     t = 0;
   return t.value() / aInfo->rate;
 }
@@ -615,7 +621,7 @@ nsVorbisState::PageIn(ogg_page* aPage)
 {
   if (!mActive)
     return NS_OK;
-  NS_ASSERTION(static_cast<PRUint32>(ogg_page_serialno(aPage)) == mSerial,
+  NS_ASSERTION(static_cast<uint32_t>(ogg_page_serialno(aPage)) == mSerial,
                "Page must be for this stream!");
   if (ogg_stream_pagein(&mState, aPage) == -1)
     return NS_ERROR_FAILURE;
@@ -627,7 +633,7 @@ nsVorbisState::PageIn(ogg_page* aPage)
     // We've found a packet with a granulepos, and we've loaded our metadata
     // and initialized our decoder. Determine granulepos of buffered packets.
     ReconstructVorbisGranulepos();
-    for (PRUint32 i = 0; i < mUnstamped.Length(); ++i) {
+    for (uint32_t i = 0; i < mUnstamped.Length(); ++i) {
       ogg_packet* packet = mUnstamped[i];
       AssertHasRecordedPacketSamples(packet);
       NS_ASSERTION(!IsHeader(packet), "Don't try to recover header packet gp");
@@ -687,7 +693,7 @@ nsresult nsVorbisState::ReconstructVorbisGranulepos()
 
   bool unknownGranulepos = last->granulepos == -1;
   int totalSamples = 0;
-  for (PRInt32 i = mUnstamped.Length() - 1; i > 0; i--) {
+  for (int32_t i = mUnstamped.Length() - 1; i > 0; i--) {
     ogg_packet* packet = mUnstamped[i];
     ogg_packet* prev = mUnstamped[i-1];
     ogg_int64_t granulepos = packet->granulepos;
@@ -710,7 +716,7 @@ nsresult nsVorbisState::ReconstructVorbisGranulepos()
   }
 
   if (unknownGranulepos) {
-    for (PRUint32 i = 0; i < mUnstamped.Length(); i++) {
+    for (uint32_t i = 0; i < mUnstamped.Length(); i++) {
       ogg_packet* packet = mUnstamped[i];
       packet->granulepos += mGranulepos + totalSamples + 1;
     }
@@ -725,7 +731,7 @@ nsresult nsVorbisState::ReconstructVorbisGranulepos()
 
   long samples = (mPrevVorbisBlockSize == 0) ? 0 :
                   mPrevVorbisBlockSize / 4 + blockSize / 4;
-  PRInt64 start = first->granulepos - samples;
+  int64_t start = first->granulepos - samples;
   RecordVorbisPacketSamples(first, samples);
 
   if (last->e_o_s && start < mGranulepos) {
@@ -734,8 +740,8 @@ nsresult nsVorbisState::ReconstructVorbisGranulepos()
     // and we will need to prune the trailing samples when we come to decode it.
     // We must correct the timestamps so that they follow the last Vorbis page's
     // samples.
-    PRInt64 pruned = mGranulepos - start;
-    for (PRUint32 i = 0; i < mUnstamped.Length() - 1; i++) {
+    int64_t pruned = mGranulepos - start;
+    for (uint32_t i = 0; i < mUnstamped.Length() - 1; i++) {
       mUnstamped[i]->granulepos += pruned;
     }
 #ifdef VALIDATE_VORBIS_SAMPLE_CALCULATION
@@ -750,12 +756,384 @@ nsresult nsVorbisState::ReconstructVorbisGranulepos()
   return NS_OK;
 }
 
+#ifdef MOZ_OPUS
+nsOpusState::nsOpusState(ogg_page* aBosPage) :
+  nsOggCodecState(aBosPage, true),
+  mRate(0),
+  mNominalRate(0),
+  mChannels(0),
+  mPreSkip(0),
+#ifdef MOZ_SAMPLE_TYPE_FLOAT32
+  mGain(1.0f),
+#else
+  mGain_Q16(65536),
+#endif
+  mChannelMapping(0),
+  mStreams(0),
+  mCoupledStreams(0),
+  mDecoder(NULL),
+  mSkip(0),
+  mPrevPacketGranulepos(0),
+  mPrevPageGranulepos(0)
+{
+  MOZ_COUNT_CTOR(nsOpusState);
+}
 
-nsSkeletonState::nsSkeletonState(ogg_page* aBosPage)
-  : nsOggCodecState(aBosPage, true),
-    mVersion(0),
-    mPresentationTime(0),
-    mLength(0)
+nsOpusState::~nsOpusState() {
+  MOZ_COUNT_DTOR(nsOpusState);
+  Reset();
+
+  if (mDecoder) {
+    opus_multistream_decoder_destroy(mDecoder);
+    mDecoder = NULL;
+  }
+}
+
+nsresult nsOpusState::Reset()
+{
+  return Reset(false);
+}
+
+nsresult nsOpusState::Reset(bool aStart)
+{
+  nsresult res = NS_OK;
+
+  if (mActive && mDecoder) {
+    // Reset the decoder.
+    opus_multistream_decoder_ctl(mDecoder, OPUS_RESET_STATE);
+    // Let the seek logic handle pre-roll if we're not seeking to the start.
+    mSkip = aStart ? mPreSkip : 0;
+    // This lets us distinguish the first page being the last page vs. just
+    // not having processed the previous page when we encounter the last page.
+    mPrevPageGranulepos = aStart ? 0 : -1;
+    mPrevPacketGranulepos = aStart ? 0 : -1;
+  }
+
+  // Clear queued data.
+  if (NS_FAILED(nsOggCodecState::Reset())) {
+    return NS_ERROR_FAILURE;
+  }
+
+  LOG(PR_LOG_DEBUG, ("Opus decoder reset, to skip %d", mSkip));
+
+  return res;
+}
+
+bool nsOpusState::Init(void)
+{
+  if (!mActive)
+    return false;
+
+  int error;
+
+  NS_ASSERTION(mDecoder == NULL, "leaking OpusDecoder");
+
+  mDecoder = opus_multistream_decoder_create(mRate,
+                                             mChannels,
+                                             mStreams,
+                                             mCoupledStreams,
+                                             mMappingTable,
+                                             &error);
+
+  mSkip = mPreSkip;
+
+  LOG(PR_LOG_DEBUG, ("Opus decoder init, to skip %d", mSkip));
+
+  return error == OPUS_OK;
+}
+
+bool nsOpusState::DecodeHeader(ogg_packet* aPacket)
+{
+  nsAutoRef<ogg_packet> autoRelease(aPacket);
+  switch(mPacketCount++) {
+    // Parse the id header.
+    case 0: {
+      if (aPacket->bytes < 19 || memcmp(aPacket->packet, "OpusHead", 8)) {
+        LOG(PR_LOG_DEBUG, ("Invalid Opus file: unrecognized header"));
+        return false;
+      }
+
+      mRate = 48000; // The Opus decoder runs at 48 kHz regardless.
+
+      int version = aPacket->packet[8];
+      // Accept file format versions 0.x.
+      if ((version & 0xf0) != 0) {
+        LOG(PR_LOG_DEBUG, ("Rejecting unknown Opus file version %d", version));
+        return false;
+      }
+
+      mChannels = aPacket->packet[9];
+      if (mChannels<1) {
+        LOG(PR_LOG_DEBUG, ("Invalid Opus file: Number of channels %d", mChannels));
+        return false;
+      }
+#ifndef MOZ_SAMPLE_TYPE_FLOAT32
+      // Downmixing more than 2 channels it is not supported for integer
+      // output samples. It is only supported for float output.
+      if (mChannels>2)
+        return false;
+#endif
+      mPreSkip = LEUint16(aPacket->packet + 10);
+      mNominalRate = LEUint32(aPacket->packet + 12);
+      double gain_dB = LEInt16(aPacket->packet + 16) / 256.0;
+#ifdef MOZ_SAMPLE_TYPE_FLOAT32
+      mGain = static_cast<float>(pow(10,0.05*gain_dB));
+#else
+      mGain_Q16 = static_cast<int32_t>(NS_MIN(65536*pow(10,0.05*gain_dB)+0.5,
+                                              static_cast<double>(PR_INT32_MAX)));
+#endif
+      mChannelMapping = aPacket->packet[18];
+
+      if (mChannelMapping == 0) {
+        mStreams = 1;
+        mCoupledStreams = mChannels - 1;
+        mMappingTable[0] = 0;
+        mMappingTable[1] = 1;
+      } else if (aPacket->bytes>20+mChannels) {
+        mStreams = aPacket->packet[19];
+        mCoupledStreams = aPacket->packet[20];
+        int i;
+        for (i=0; i<mChannels; i++)
+          mMappingTable[i] = aPacket->packet[21+i];
+      } else {
+        LOG(PR_LOG_DEBUG, ("Invalid Opus file: channel mapping %d,"
+                           " but no channel mapping table", mChannelMapping));
+        return false;
+      }
+
+#ifdef DEBUG
+      LOG(PR_LOG_DEBUG, ("Opus stream header:"));
+      LOG(PR_LOG_DEBUG, (" channels: %d", mChannels));
+      LOG(PR_LOG_DEBUG, ("  preskip: %d", mPreSkip));
+      LOG(PR_LOG_DEBUG, (" original: %d Hz", mNominalRate));
+      LOG(PR_LOG_DEBUG, ("     gain: %.2f dB", gain_dB));
+      LOG(PR_LOG_DEBUG, ("Channel Mapping:"));
+      LOG(PR_LOG_DEBUG, ("   family: %d", mChannelMapping));
+      LOG(PR_LOG_DEBUG, ("  streams: %d", mStreams));
+#endif
+    }
+    break;
+
+    // Parse the metadata header.
+    case 1: {
+      if (aPacket->bytes < 16 || memcmp(aPacket->packet, "OpusTags", 8))
+        return false;
+
+      // We don't actually need any of the data here, but validating the
+      // contents helps reduce the propagation of broken files.
+      // This only checks for actual malicious content: too little data, too
+      // many comments, or comments that are too long.
+      // It does not ensure they are valid UTF-8, nor does it validate the
+      // required ASCII_TAG=value format of the user comments.
+      const unsigned char *buf = aPacket->packet + 8;
+      uint32_t bytes = aPacket->bytes - 8;
+      uint32_t len;
+      // Skip the vendor string.
+      len = LEUint32(buf);
+      buf += 4;
+      bytes -= 4;
+      if (len > bytes)
+        return false;
+      buf += len;
+      bytes -= len;
+      // Skip the user comments.
+      if (bytes < 4)
+        return false;
+      uint32_t ncomments = LEUint32(buf);
+      buf += 4;
+      bytes -= 4;
+      // If there are so many comments even their length fields won't fit in
+      // the packet, stop reading now.
+      if (ncomments > (bytes>>2))
+        return false;
+      uint32_t i;
+      for (i = 0; i < ncomments; i++) {
+        if (bytes < 4)
+          return false;
+        len = LEUint32(buf);
+        buf += 4;
+        bytes -= 4;
+        if (len > bytes)
+          return false;
+        buf += len;
+        bytes -= len;
+      }
+    }
+    break;
+
+    // We made it to the first data packet (which includes reconstructing
+    // timestamps for it in PageIn). Success!
+    default: {
+      mDoneReadingHeaders = true;
+      // Put it back on the queue so we can decode it.
+      mPackets.PushFront(autoRelease.disown());
+    }
+    break;
+  }
+  return true;
+}
+
+/* Return the timestamp (in microseconds) equivalent to a granulepos. */
+int64_t nsOpusState::Time(int64_t aGranulepos)
+{
+  if (!mActive)
+    return -1;
+
+  return Time(mPreSkip, aGranulepos);
+}
+
+int64_t nsOpusState::Time(int aPreSkip, int64_t aGranulepos)
+{
+  if (aGranulepos < 0)
+    return -1;
+
+  // Ogg Opus always runs at a granule rate of 48 kHz.
+  CheckedInt64 t = CheckedInt64(aGranulepos - aPreSkip) * USECS_PER_S;
+  return t.isValid() ? t.value() / 48000 : -1;
+}
+
+bool nsOpusState::IsHeader(ogg_packet* aPacket)
+{
+  return aPacket->bytes >= 16 &&
+         (!memcmp(aPacket->packet, "OpusHead", 8) ||
+          !memcmp(aPacket->packet, "OpusTags", 8));
+}
+
+nsresult nsOpusState::PageIn(ogg_page* aPage)
+{
+  if (!mActive)
+    return NS_OK;
+  NS_ASSERTION(static_cast<uint32_t>(ogg_page_serialno(aPage)) == mSerial,
+               "Page must be for this stream!");
+  if (ogg_stream_pagein(&mState, aPage) == -1)
+    return NS_ERROR_FAILURE;
+
+  bool haveGranulepos;
+  nsresult rv = PacketOutUntilGranulepos(haveGranulepos);
+  if (NS_FAILED(rv) || !haveGranulepos || mPacketCount < 2)
+    return rv;
+  if(!ReconstructOpusGranulepos())
+    return NS_ERROR_FAILURE;
+  for (uint32_t i = 0; i < mUnstamped.Length(); i++) {
+    ogg_packet* packet = mUnstamped[i];
+    NS_ASSERTION(!IsHeader(packet), "Don't try to play a header packet");
+    NS_ASSERTION(packet->granulepos != -1, "Packet should have a granulepos");
+    mPackets.Append(packet);
+  }
+  mUnstamped.Clear();
+  return NS_OK;
+}
+
+// Helper method to return the change in granule position due to an Opus packet
+// (as distinct from the number of samples in the packet, which depends on the
+// decoder rate). It should work with a multistream Opus file, and continue to
+// work should we ever allow the decoder to decode at a rate other than 48 kHz.
+// It even works before we've created the actual Opus decoder.
+static int GetOpusDeltaGP(ogg_packet* packet)
+{
+  int nframes;
+  nframes = opus_packet_get_nb_frames(packet->packet, packet->bytes);
+  if (nframes > 0) {
+    return nframes*opus_packet_get_samples_per_frame(packet->packet, 48000);
+  }
+  NS_WARNING("Invalid Opus packet.");
+  return nframes;
+}
+
+bool nsOpusState::ReconstructOpusGranulepos(void)
+{
+  NS_ASSERTION(mUnstamped.Length() > 0, "Must have unstamped packets");
+  ogg_packet* last = mUnstamped[mUnstamped.Length()-1];
+  NS_ASSERTION(last->e_o_s || last->granulepos > 0,
+      "Must know last granulepos!");
+  int64_t gp;
+  // If this is the last page, and we've seen at least one previous page (or
+  // this is the first page)...
+  if (last->e_o_s) {
+    if (mPrevPageGranulepos != -1) {
+      // If this file only has one page and the final granule position is
+      // smaller than the pre-skip amount, we MUST reject the stream.
+      if (!mDoneReadingHeaders && last->granulepos < mPreSkip)
+        return false;
+      int64_t last_gp = last->granulepos;
+      gp = mPrevPageGranulepos;
+      // Loop through the packets forwards, adding the current packet's
+      // duration to the previous granulepos to get the value for the
+      // current packet.
+      for (uint32_t i = 0; i < mUnstamped.Length() - 1; ++i) {
+        ogg_packet* packet = mUnstamped[i];
+        int offset = GetOpusDeltaGP(packet);
+        // Check for error (negative offset) and overflow.
+        if (offset >= 0 && gp <= PR_INT64_MAX - offset) {
+          gp += offset;
+          if (gp >= last_gp) {
+            NS_WARNING("Opus end trimming removed more than a full packet.");
+            // We were asked to remove a full packet's worth of data or more.
+            // Encoders SHOULD NOT produce streams like this, but we'll handle
+            // it for them anyway.
+            gp = last_gp;
+            for (uint32_t j = i+1; j < mUnstamped.Length(); ++j) {
+              nsOggCodecState::ReleasePacket(mUnstamped[j]);
+            }
+            mUnstamped.RemoveElementsAt(i+1, mUnstamped.Length() - (i+1));
+            last = packet;
+            last->e_o_s = 1;
+          }
+        }
+        packet->granulepos = gp;
+      }
+      mPrevPageGranulepos = last_gp;
+      return true;
+    } else {
+      NS_WARNING("No previous granule position to use for Opus end trimming.");
+      // If we don't have a previous granule position, fall through.
+      // We simply won't trim any samples from the end.
+      // TODO: Are we guaranteed to have seen a previous page if there is one?
+    }
+  }
+
+  gp = last->granulepos;
+  // Loop through the packets backwards, subtracting the next
+  // packet's duration from its granulepos to get the value
+  // for the current packet.
+  for (uint32_t i = mUnstamped.Length() - 1; i > 0; i--) {
+    int offset = GetOpusDeltaGP(mUnstamped[i]);
+    // Check for error (negative offset) and overflow.
+    if (offset >= 0) {
+      if (offset <= gp) {
+        gp -= offset;
+      } else {
+        // If the granule position of the first data page is smaller than the
+        // number of decodable audio samples on that page, then we MUST reject
+        // the stream.
+        if (!mDoneReadingHeaders)
+          return false;
+        // It's too late to reject the stream.
+        // If we get here, this almost certainly means the file has screwed-up
+        // timestamps somewhere after the first page.
+        NS_WARNING("Clamping negative Opus granulepos to zero.");
+        gp = 0;
+      }
+    }
+    mUnstamped[i - 1]->granulepos = gp;
+  }
+
+  // Check to make sure the first granule position is at least as large as the
+  // total number of samples decodable from the first page with completed
+  // packets. This requires looking at the duration of the first packet, too.
+  // We MUST reject such streams.
+  if (!mDoneReadingHeaders && GetOpusDeltaGP(mUnstamped[0]) > gp)
+    return false;
+  mPrevPageGranulepos = last->granulepos;
+  return true;
+}
+#endif /* MOZ_OPUS */
+
+nsSkeletonState::nsSkeletonState(ogg_page* aBosPage) :
+  nsOggCodecState(aBosPage, true),
+  mVersion(0),
+  mPresentationTime(0),
+  mLength(0)
 {
   MOZ_COUNT_CTOR(nsSkeletonState);
 }
@@ -810,43 +1188,20 @@ static bool IsSkeletonIndex(ogg_packet* aPacket)
          memcmp(reinterpret_cast<char*>(aPacket->packet), "index", 5) == 0;
 }
 
-// Reads a little-endian encoded unsigned 32bit integer at p.
-static PRUint32 LEUint32(const unsigned char* p)
-{
-  return p[0] +
-        (p[1] << 8) + 
-        (p[2] << 16) +
-        (p[3] << 24);
-}
-
-// Reads a little-endian encoded 64bit integer at p.
-static PRInt64 LEInt64(const unsigned char* p)
-{
-  PRUint32 lo = LEUint32(p);
-  PRUint32 hi = LEUint32(p + 4);
-  return static_cast<PRInt64>(lo) | (static_cast<PRInt64>(hi) << 32);
-}
-
-// Reads a little-endian encoded unsigned 16bit integer at p.
-static PRUint16 LEUint16(const unsigned char* p)
-{
-  return p[0] + (p[1] << 8);  
-}
-
 // Reads a variable length encoded integer at p. Will not read
 // past aLimit. Returns pointer to character after end of integer.
 static const unsigned char* ReadVariableLengthInt(const unsigned char* p,
                                                   const unsigned char* aLimit,
-                                                  PRInt64& n)
+                                                  int64_t& n)
 {
   int shift = 0;
-  PRInt64 byte = 0;
+  int64_t byte = 0;
   n = 0;
   while (p < aLimit &&
          (byte & 0x80) != 0x80 &&
          shift < 57)
   {
-    byte = static_cast<PRInt64>(*p);
+    byte = static_cast<int64_t>(*p);
     n |= ((byte & 0x7f) << shift);
     shift += 7;
     p++;
@@ -862,13 +1217,13 @@ bool nsSkeletonState::DecodeIndex(ogg_packet* aPacket)
     return false;
   }
 
-  PRUint32 serialno = LEUint32(aPacket->packet + INDEX_SERIALNO_OFFSET);
-  PRInt64 numKeyPoints = LEInt64(aPacket->packet + INDEX_NUM_KEYPOINTS_OFFSET);
+  uint32_t serialno = LEUint32(aPacket->packet + INDEX_SERIALNO_OFFSET);
+  int64_t numKeyPoints = LEInt64(aPacket->packet + INDEX_NUM_KEYPOINTS_OFFSET);
 
-  PRInt64 endTime = 0, startTime = 0;
+  int64_t endTime = 0, startTime = 0;
   const unsigned char* p = aPacket->packet;
 
-  PRInt64 timeDenom = LEInt64(aPacket->packet + INDEX_TIME_DENOM_OFFSET);
+  int64_t timeDenom = LEInt64(aPacket->packet + INDEX_TIME_DENOM_OFFSET);
   if (timeDenom == 0) {
     LOG(PR_LOG_DEBUG, ("Ogg Skeleton Index packet for stream %u has 0 "
                        "timestamp denominator.", serialno));
@@ -877,7 +1232,7 @@ bool nsSkeletonState::DecodeIndex(ogg_packet* aPacket)
 
   // Extract the start time.
   CheckedInt64 t = CheckedInt64(LEInt64(p + INDEX_FIRST_NUMER_OFFSET)) * USECS_PER_S;
-  if (!t.valid()) {
+  if (!t.isValid()) {
     return (mActive = false);
   } else {
     startTime = t.value() / timeDenom;
@@ -885,7 +1240,7 @@ bool nsSkeletonState::DecodeIndex(ogg_packet* aPacket)
 
   // Extract the end time.
   t = LEInt64(p + INDEX_LAST_NUMER_OFFSET) * USECS_PER_S;
-  if (!t.valid()) {
+  if (!t.isValid()) {
     return (mActive = false);
   } else {
     endTime = t.value() / timeDenom;
@@ -894,13 +1249,13 @@ bool nsSkeletonState::DecodeIndex(ogg_packet* aPacket)
   // Check the numKeyPoints value read, ensure we're not going to run out of
   // memory while trying to decode the index packet.
   CheckedInt64 minPacketSize = (CheckedInt64(numKeyPoints) * MIN_KEY_POINT_SIZE) + INDEX_KEYPOINT_OFFSET;
-  if (!minPacketSize.valid())
+  if (!minPacketSize.isValid())
   {
     return (mActive = false);
   }
   
-  PRInt64 sizeofIndex = aPacket->bytes - INDEX_KEYPOINT_OFFSET;
-  PRInt64 maxNumKeyPoints = sizeofIndex / MIN_KEY_POINT_SIZE;
+  int64_t sizeofIndex = aPacket->bytes - INDEX_KEYPOINT_OFFSET;
+  int64_t maxNumKeyPoints = sizeofIndex / MIN_KEY_POINT_SIZE;
   if (aPacket->bytes < minPacketSize.value() ||
       numKeyPoints > maxNumKeyPoints || 
       numKeyPoints < 0)
@@ -922,17 +1277,17 @@ bool nsSkeletonState::DecodeIndex(ogg_packet* aPacket)
   
   p = aPacket->packet + INDEX_KEYPOINT_OFFSET;
   const unsigned char* limit = aPacket->packet + aPacket->bytes;
-  PRInt64 numKeyPointsRead = 0;
+  int64_t numKeyPointsRead = 0;
   CheckedInt64 offset = 0;
   CheckedInt64 time = 0;
   while (p < limit &&
          numKeyPointsRead < numKeyPoints)
   {
-    PRInt64 delta = 0;
+    int64_t delta = 0;
     p = ReadVariableLengthInt(p, limit, delta);
     offset += delta;
     if (p == limit ||
-        !offset.valid() ||
+        !offset.isValid() ||
         offset.value() > mLength ||
         offset.value() < 0)
     {
@@ -940,21 +1295,21 @@ bool nsSkeletonState::DecodeIndex(ogg_packet* aPacket)
     }
     p = ReadVariableLengthInt(p, limit, delta);
     time += delta;
-    if (!time.valid() ||
+    if (!time.isValid() ||
         time.value() > endTime ||
         time.value() < startTime)
     {
       return (mActive = false);
     }
     CheckedInt64 timeUsecs = time * USECS_PER_S;
-    if (!timeUsecs.valid())
+    if (!timeUsecs.isValid())
       return mActive = false;
     timeUsecs /= timeDenom;
     keyPoints->Add(offset.value(), timeUsecs.value());
     numKeyPointsRead++;
   }
 
-  PRInt32 keyPointsRead = keyPoints->Length();
+  int32_t keyPointsRead = keyPoints->Length();
   if (keyPointsRead > 0) {
     mIndex.Put(serialno, keyPoints.forget());
   }
@@ -964,11 +1319,11 @@ bool nsSkeletonState::DecodeIndex(ogg_packet* aPacket)
   return true;
 }
 
-nsresult nsSkeletonState::IndexedSeekTargetForTrack(PRUint32 aSerialno,
-                                                    PRInt64 aTarget,
+nsresult nsSkeletonState::IndexedSeekTargetForTrack(uint32_t aSerialno,
+                                                    int64_t aTarget,
                                                     nsKeyPoint& aResult)
 {
-  nsKeyFrameIndex* index = nsnull;
+  nsKeyFrameIndex* index = nullptr;
   mIndex.Get(aSerialno, &index);
 
   if (!index ||
@@ -999,8 +1354,8 @@ nsresult nsSkeletonState::IndexedSeekTargetForTrack(PRUint32 aSerialno,
   return NS_OK;
 }
 
-nsresult nsSkeletonState::IndexedSeekTarget(PRInt64 aTarget,
-                                            nsTArray<PRUint32>& aTracks,
+nsresult nsSkeletonState::IndexedSeekTarget(int64_t aTarget,
+                                            nsTArray<uint32_t>& aTracks,
                                             nsSeekTarget& aResult)
 {
   if (!mActive || mVersion < SKELETON_VERSION(4,0)) {
@@ -1011,7 +1366,7 @@ nsresult nsSkeletonState::IndexedSeekTarget(PRInt64 aTarget,
   // our seek result. User must seek to the one with lowest offset to ensure we
   // pass "keyframes" on all tracks when we decode forwards to the seek target.
   nsSeekTarget r;
-  for (PRUint32 i=0; i<aTracks.Length(); i++) {
+  for (uint32_t i=0; i<aTracks.Length(); i++) {
     nsKeyPoint k;
     if (NS_SUCCEEDED(IndexedSeekTargetForTrack(aTracks[i], aTarget, k)) &&
         k.mOffset < r.mKeyPoint.mOffset)
@@ -1029,8 +1384,8 @@ nsresult nsSkeletonState::IndexedSeekTarget(PRInt64 aTarget,
   return NS_OK;
 }
 
-nsresult nsSkeletonState::GetDuration(const nsTArray<PRUint32>& aTracks,
-                                      PRInt64& aDuration)
+nsresult nsSkeletonState::GetDuration(const nsTArray<uint32_t>& aTracks,
+                                      int64_t& aDuration)
 {
   if (!mActive ||
       mVersion < SKELETON_VERSION(4,0) ||
@@ -1039,10 +1394,10 @@ nsresult nsSkeletonState::GetDuration(const nsTArray<PRUint32>& aTracks,
   {
     return NS_ERROR_FAILURE;
   }
-  PRInt64 endTime = INT64_MIN;
-  PRInt64 startTime = INT64_MAX;
-  for (PRUint32 i=0; i<aTracks.Length(); i++) {
-    nsKeyFrameIndex* index = nsnull;
+  int64_t endTime = INT64_MIN;
+  int64_t startTime = INT64_MAX;
+  for (uint32_t i=0; i<aTracks.Length(); i++) {
+    nsKeyFrameIndex* index = nullptr;
     mIndex.Get(aTracks[i], &index);
     if (!index) {
       // Can't get the timestamps for one of the required tracks, fail.
@@ -1057,54 +1412,43 @@ nsresult nsSkeletonState::GetDuration(const nsTArray<PRUint32>& aTracks,
   }
   NS_ASSERTION(endTime > startTime, "Duration must be positive");
   CheckedInt64 duration = CheckedInt64(endTime) - startTime;
-  aDuration = duration.valid() ? duration.value() : 0;
-  return duration.valid() ? NS_OK : NS_ERROR_FAILURE;
+  aDuration = duration.isValid() ? duration.value() : 0;
+  return duration.isValid() ? NS_OK : NS_ERROR_FAILURE;
 }
 
 bool nsSkeletonState::DecodeHeader(ogg_packet* aPacket)
 {
+  nsAutoRef<ogg_packet> autoRelease(aPacket);
   if (IsSkeletonBOS(aPacket)) {
-    PRUint16 verMajor = LEUint16(aPacket->packet + SKELETON_VERSION_MAJOR_OFFSET);
-    PRUint16 verMinor = LEUint16(aPacket->packet + SKELETON_VERSION_MINOR_OFFSET);
+    uint16_t verMajor = LEUint16(aPacket->packet + SKELETON_VERSION_MAJOR_OFFSET);
+    uint16_t verMinor = LEUint16(aPacket->packet + SKELETON_VERSION_MINOR_OFFSET);
 
     // Read the presentation time. We read this before the version check as the
     // presentation time exists in all versions.
-    PRInt64 n = LEInt64(aPacket->packet + SKELETON_PRESENTATION_TIME_NUMERATOR_OFFSET);
-    PRInt64 d = LEInt64(aPacket->packet + SKELETON_PRESENTATION_TIME_DENOMINATOR_OFFSET);
+    int64_t n = LEInt64(aPacket->packet + SKELETON_PRESENTATION_TIME_NUMERATOR_OFFSET);
+    int64_t d = LEInt64(aPacket->packet + SKELETON_PRESENTATION_TIME_DENOMINATOR_OFFSET);
     mPresentationTime = d == 0 ? 0 : (static_cast<float>(n) / static_cast<float>(d)) * USECS_PER_S;
 
     mVersion = SKELETON_VERSION(verMajor, verMinor);
+    // We can only care to parse Skeleton version 4.0+.
     if (mVersion < SKELETON_VERSION(4,0) ||
         mVersion >= SKELETON_VERSION(5,0) ||
         aPacket->bytes < SKELETON_4_0_MIN_HEADER_LEN)
-    {
-      // We can only care to parse Skeleton version 4.0+.
-      mActive = false;
-      return mDoneReadingHeaders = true;
-    }
+      return false;
 
     // Extract the segment length.
     mLength = LEInt64(aPacket->packet + SKELETON_FILE_LENGTH_OFFSET);
 
     LOG(PR_LOG_DEBUG, ("Skeleton segment length: %lld", mLength));
 
-    // Initialize the serianlno-to-index map.
-    bool init = mIndex.Init();
-    if (!init) {
-      NS_WARNING("Failed to initialize Ogg skeleton serialno-to-index map");
-      mActive = false;
-      return mDoneReadingHeaders = true;
-    }
-    mActive = true;
+    // Initialize the serialno-to-index map.
+    mIndex.Init();
+    return true;
   } else if (IsSkeletonIndex(aPacket) && mVersion >= SKELETON_VERSION(4,0)) {
-    if (!DecodeIndex(aPacket)) {
-      // Failed to parse index, or invalid/hostile index. DecodeIndex() will
-      // have deactivated the track.
-      return mDoneReadingHeaders = true;
-    }
-
+    return DecodeIndex(aPacket);
   } else if (aPacket->e_o_s) {
     mDoneReadingHeaders = true;
+    return true;
   }
-  return mDoneReadingHeaders;
+  return false;
 }
