@@ -1,43 +1,12 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla SVG Project code.
- *
- * The Initial Developer of the Original Code is the Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2010
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsSVGElement.h"
 #include "DOMSVGPathSegList.h"
 #include "DOMSVGPathSeg.h"
-#include "nsDOMError.h"
+#include "nsError.h"
 #include "SVGAnimatedPathSegList.h"
 #include "nsCOMPtr.h"
 #include "nsSVGAttrTearoffTable.h"
@@ -86,14 +55,13 @@ DOMSVGPathSegList::GetDOMWrapper(void *aList,
                                  nsSVGElement *aElement,
                                  bool aIsAnimValList)
 {
-  DOMSVGPathSegList *wrapper =
+  nsRefPtr<DOMSVGPathSegList> wrapper =
     sSVGPathSegListTearoffTable.GetTearoff(aList);
   if (!wrapper) {
     wrapper = new DOMSVGPathSegList(aElement, aIsAnimValList);
     sSVGPathSegListTearoffTable.AddTearoff(aList, wrapper);
   }
-  NS_ADDREF(wrapper);
-  return wrapper;
+  return wrapper.forget();
 }
 
 /* static */ DOMSVGPathSegList*
@@ -115,12 +83,12 @@ DOMSVGPathSegList::~DOMSVGPathSegList()
 JSObject*
 DOMSVGPathSegList::WrapObject(JSContext *cx, JSObject *scope, bool *triedToWrap)
 {
-  return mozilla::dom::binding::SVGPathSegList::create(cx, scope, this,
+  return mozilla::dom::oldproxybindings::SVGPathSegList::create(cx, scope, this,
                                                        triedToWrap);
 }
 
 nsIDOMSVGPathSeg*
-DOMSVGPathSegList::GetItemAt(PRUint32 aIndex)
+DOMSVGPathSegList::GetItemAt(uint32_t aIndex)
 {
   if (IsAnimValList()) {
     Element()->FlushAnimations();
@@ -129,7 +97,7 @@ DOMSVGPathSegList::GetItemAt(PRUint32 aIndex)
     EnsureItemAt(aIndex);
     return ItemAt(aIndex);
   }
-  return nsnull;
+  return nullptr;
 }
 
 void
@@ -168,13 +136,13 @@ DOMSVGPathSegList::InternalListWillChangeTo(const SVGPathData& aNewValue)
   // our items and let them be recreated lazily, since that would break what
   // script sees!
 
-  PRUint32 length = mItems.Length();
-  PRUint32 index = 0;
+  uint32_t length = mItems.Length();
+  uint32_t index = 0;
 
-  PRUint32 dataLength = aNewValue.mData.Length();
-  PRUint32 dataIndex = 0; // index into aNewValue's raw data array
+  uint32_t dataLength = aNewValue.mData.Length();
+  uint32_t dataIndex = 0; // index into aNewValue's raw data array
 
-  PRUint32 newSegType;
+  uint32_t newSegType;
 
   nsRefPtr<DOMSVGPathSegList> kungFuDeathGrip;
   if (length) {
@@ -198,7 +166,7 @@ DOMSVGPathSegList::InternalListWillChangeTo(const SVGPathData& aNewValue)
     newSegType = SVGPathSegUtils::DecodeType(aNewValue.mData[dataIndex]);
     if (ItemAt(index) && ItemAt(index)->Type() != newSegType) {
       ItemAt(index)->RemovingFromList();
-      ItemAt(index) = nsnull;
+      ItemAt(index) = nullptr;
     }
     // Only after the RemovingFromList() can we touch mInternalDataIndex!
     mItems[index].mInternalDataIndex = dataIndex;
@@ -213,13 +181,13 @@ DOMSVGPathSegList::InternalListWillChangeTo(const SVGPathData& aNewValue)
   if (index < length) {
     // aNewValue has fewer items than our previous internal counterpart
 
-    PRUint32 newLength = index;
+    uint32_t newLength = index;
 
     // Remove excess items from the list:
     for (; index < length; ++index) {
       if (ItemAt(index)) {
         ItemAt(index)->RemovingFromList();
-        ItemAt(index) = nsnull;
+        ItemAt(index) = nullptr;
       }
     }
 
@@ -237,7 +205,7 @@ DOMSVGPathSegList::InternalListWillChangeTo(const SVGPathData& aNewValue)
         // have FEWER items than it does.
         return;
       }
-      if (!mItems.AppendElement(ItemProxy(nsnull, dataIndex))) {
+      if (!mItems.AppendElement(ItemProxy(nullptr, dataIndex))) {
         // OOM
         Clear();
         return;
@@ -274,7 +242,7 @@ DOMSVGPathSegList::InternalAList() const
 // nsIDOMSVGPathSegList implementation:
 
 NS_IMETHODIMP
-DOMSVGPathSegList::GetNumberOfItems(PRUint32 *aNumberOfItems)
+DOMSVGPathSegList::GetNumberOfItems(uint32_t *aNumberOfItems)
 {
   if (IsAnimValList()) {
     Element()->FlushAnimations();
@@ -320,7 +288,7 @@ NS_IMETHODIMP
 DOMSVGPathSegList::Initialize(nsIDOMSVGPathSeg *aNewItem,
                               nsIDOMSVGPathSeg **_retval)
 {
-  *_retval = nsnull;
+  *_retval = nullptr;
   if (IsAnimValList()) {
     return NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR;
   }
@@ -346,7 +314,7 @@ DOMSVGPathSegList::Initialize(nsIDOMSVGPathSeg *aNewItem,
 }
 
 NS_IMETHODIMP
-DOMSVGPathSegList::GetItem(PRUint32 aIndex,
+DOMSVGPathSegList::GetItem(uint32_t aIndex,
                            nsIDOMSVGPathSeg **_retval)
 {
   *_retval = GetItemAt(aIndex);
@@ -359,15 +327,15 @@ DOMSVGPathSegList::GetItem(PRUint32 aIndex,
 
 NS_IMETHODIMP
 DOMSVGPathSegList::InsertItemBefore(nsIDOMSVGPathSeg *aNewItem,
-                                    PRUint32 aIndex,
+                                    uint32_t aIndex,
                                     nsIDOMSVGPathSeg **_retval)
 {
-  *_retval = nsnull;
+  *_retval = nullptr;
   if (IsAnimValList()) {
     return NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR;
   }
 
-  PRUint32 internalIndex;
+  uint32_t internalIndex;
   if (aIndex < Length()) {
     internalIndex = mItems[aIndex].mInternalDataIndex;
   } else {
@@ -386,7 +354,7 @@ DOMSVGPathSegList::InsertItemBefore(nsIDOMSVGPathSeg *aNewItem,
     domItem = domItem->Clone(); // must do this before changing anything!
   }
 
-  PRUint32 argCount = SVGPathSegUtils::ArgCountForType(domItem->Type());
+  uint32_t argCount = SVGPathSegUtils::ArgCountForType(domItem->Type());
 
   // Ensure we have enough memory so we can avoid complex error handling below:
   if (!mItems.SetCapacity(mItems.Length() + 1) ||
@@ -415,16 +383,16 @@ DOMSVGPathSegList::InsertItemBefore(nsIDOMSVGPathSeg *aNewItem,
   if (AttrIsAnimating()) {
     Element()->AnimationNeedsResample();
   }
-  *_retval = domItem.forget().get();
+  domItem.forget(_retval);
   return NS_OK;
 }
 
 NS_IMETHODIMP
 DOMSVGPathSegList::ReplaceItem(nsIDOMSVGPathSeg *aNewItem,
-                               PRUint32 aIndex,
+                               uint32_t aIndex,
                                nsIDOMSVGPathSeg **_retval)
 {
-  *_retval = nsnull;
+  *_retval = nullptr;
   if (IsAnimValList()) {
     return NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR;
   }
@@ -447,12 +415,12 @@ DOMSVGPathSegList::ReplaceItem(nsIDOMSVGPathSeg *aNewItem,
     ItemAt(aIndex)->RemovingFromList();
   }
 
-  PRUint32 internalIndex = mItems[aIndex].mInternalDataIndex;
+  uint32_t internalIndex = mItems[aIndex].mInternalDataIndex;
   // We use InternalList() to get oldArgCount since we may not have a DOM
   // wrapper at the index being replaced.
-  PRUint32 oldType = SVGPathSegUtils::DecodeType(InternalList().mData[internalIndex]);
-  PRUint32 oldArgCount = SVGPathSegUtils::ArgCountForType(oldType);
-  PRUint32 newArgCount = SVGPathSegUtils::ArgCountForType(domItem->Type());
+  uint32_t oldType = SVGPathSegUtils::DecodeType(InternalList().mData[internalIndex]);
+  uint32_t oldArgCount = SVGPathSegUtils::ArgCountForType(oldType);
+  uint32_t newArgCount = SVGPathSegUtils::ArgCountForType(domItem->Type());
 
   float segAsRaw[1 + NS_SVG_PATH_SEG_MAX_ARGS];
   domItem->ToSVGPathSegEncodedData(segAsRaw);
@@ -469,9 +437,9 @@ DOMSVGPathSegList::ReplaceItem(nsIDOMSVGPathSeg *aNewItem,
   // would end up reading bad data from InternalList()!
   domItem->InsertingIntoList(this, aIndex, IsAnimValList());
 
-  PRUint32 delta = newArgCount - oldArgCount;
+  uint32_t delta = newArgCount - oldArgCount;
   if (delta != 0) {
-    for (PRUint32 i = aIndex + 1; i < Length(); ++i) {
+    for (uint32_t i = aIndex + 1; i < Length(); ++i) {
       mItems[i].mInternalDataIndex += delta;
     }
   }
@@ -485,10 +453,10 @@ DOMSVGPathSegList::ReplaceItem(nsIDOMSVGPathSeg *aNewItem,
 }
 
 NS_IMETHODIMP
-DOMSVGPathSegList::RemoveItem(PRUint32 aIndex,
+DOMSVGPathSegList::RemoveItem(uint32_t aIndex,
                               nsIDOMSVGPathSeg **_retval)
 {
-  *_retval = nsnull;
+  *_retval = nullptr;
   if (IsAnimValList()) {
     return NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR;
   }
@@ -505,9 +473,9 @@ DOMSVGPathSegList::RemoveItem(PRUint32 aIndex,
   ItemAt(aIndex)->RemovingFromList();
   NS_ADDREF(*_retval = ItemAt(aIndex));
 
-  PRUint32 internalIndex = mItems[aIndex].mInternalDataIndex;
-  PRUint32 segType = SVGPathSegUtils::DecodeType(InternalList().mData[internalIndex]);
-  PRUint32 argCount = SVGPathSegUtils::ArgCountForType(segType);
+  uint32_t internalIndex = mItems[aIndex].mInternalDataIndex;
+  uint32_t segType = SVGPathSegUtils::DecodeType(InternalList().mData[internalIndex]);
+  uint32_t argCount = SVGPathSegUtils::ArgCountForType(segType);
 
   // Now that we know we're removing, keep animVal list in sync as necessary.
   // Do this *before* touching InternalList() so the removed item can get its
@@ -534,13 +502,13 @@ DOMSVGPathSegList::AppendItem(nsIDOMSVGPathSeg *aNewItem,
 }
 
 NS_IMETHODIMP
-DOMSVGPathSegList::GetLength(PRUint32 *aNumberOfItems)
+DOMSVGPathSegList::GetLength(uint32_t *aNumberOfItems)
 {
   return GetNumberOfItems(aNumberOfItems);
 }
 
 void
-DOMSVGPathSegList::EnsureItemAt(PRUint32 aIndex)
+DOMSVGPathSegList::EnsureItemAt(uint32_t aIndex)
 {
   if (!ItemAt(aIndex)) {
     ItemAt(aIndex) = DOMSVGPathSeg::CreateFor(this, aIndex, IsAnimValList());
@@ -549,9 +517,9 @@ DOMSVGPathSegList::EnsureItemAt(PRUint32 aIndex)
 
 void
 DOMSVGPathSegList::
-  MaybeInsertNullInAnimValListAt(PRUint32 aIndex,
-                                 PRUint32 aInternalIndex,
-                                 PRUint32 aArgCountForItem)
+  MaybeInsertNullInAnimValListAt(uint32_t aIndex,
+                                 uint32_t aInternalIndex,
+                                 uint32_t aArgCountForItem)
 {
   NS_ABORT_IF_FALSE(!IsAnimValList(), "call from baseVal to animVal");
 
@@ -571,15 +539,15 @@ DOMSVGPathSegList::
   NS_ABORT_IF_FALSE(animVal->mItems.Length() == mItems.Length(),
                     "animVal list not in sync!");
 
-  animVal->mItems.InsertElementAt(aIndex, ItemProxy(nsnull, aInternalIndex));
+  animVal->mItems.InsertElementAt(aIndex, ItemProxy(nullptr, aInternalIndex));
 
   animVal->UpdateListIndicesFromIndex(aIndex + 1, 1 + aArgCountForItem);
 }
 
 void
 DOMSVGPathSegList::
-  MaybeRemoveItemFromAnimValListAt(PRUint32 aIndex,
-                                   PRUint32 aArgCountForItem)
+  MaybeRemoveItemFromAnimValListAt(uint32_t aIndex,
+                                   uint32_t aArgCountForItem)
 {
   NS_ABORT_IF_FALSE(!IsAnimValList(), "call from baseVal to animVal");
 
@@ -609,12 +577,12 @@ DOMSVGPathSegList::
 }
 
 void
-DOMSVGPathSegList::UpdateListIndicesFromIndex(PRUint32 aStartingIndex,
-                                              PRInt32  aInternalDataIndexDelta)
+DOMSVGPathSegList::UpdateListIndicesFromIndex(uint32_t aStartingIndex,
+                                              int32_t  aInternalDataIndexDelta)
 {
-  PRUint32 length = mItems.Length();
+  uint32_t length = mItems.Length();
 
-  for (PRUint32 i = aStartingIndex; i < length; ++i) {
+  for (uint32_t i = aStartingIndex; i < length; ++i) {
     mItems[i].mInternalDataIndex += aInternalDataIndexDelta;
     if (ItemAt(i)) {
       ItemAt(i)->UpdateListIndex(i);
