@@ -1,40 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   David Hyatt <hyatt@netscape.com> (Original Author)
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsIAtom.h"
 #include "nsString.h"
@@ -91,6 +58,11 @@ nsXBLProtoImplMethod::AddParameter(const nsAString& aText)
   NS_PRECONDITION(!IsCompiled(),
                   "Must not be compiled when accessing uncompiled method");
 
+  if (aText.IsEmpty()) {
+    NS_WARNING("Empty name attribute in xbl:parameter!");
+    return;
+  }
+
   nsXBLUncompiledMethod* uncompiledMethod = GetUncompiledMethod();
   if (!uncompiledMethod) {
     uncompiledMethod = new nsXBLUncompiledMethod();
@@ -103,7 +75,7 @@ nsXBLProtoImplMethod::AddParameter(const nsAString& aText)
 }
 
 void
-nsXBLProtoImplMethod::SetLineNumber(PRUint32 aLineNumber)
+nsXBLProtoImplMethod::SetLineNumber(uint32_t aLineNumber)
 {
   NS_PRECONDITION(!IsCompiled(),
                   "Must not be compiled when accessing uncompiled method");
@@ -122,46 +94,39 @@ nsXBLProtoImplMethod::SetLineNumber(PRUint32 aLineNumber)
 nsresult
 nsXBLProtoImplMethod::InstallMember(nsIScriptContext* aContext,
                                     nsIContent* aBoundElement, 
-                                    void* aScriptObject,
-                                    void* aTargetClassObject,
+                                    JSObject* aScriptObject,
+                                    JSObject* aTargetClassObject,
                                     const nsCString& aClassStr)
 {
   NS_PRECONDITION(IsCompiled(),
                   "Should not be installing an uncompiled method");
   JSContext* cx = aContext->GetNativeContext();
 
-  nsIDocument *ownerDoc = aBoundElement->OwnerDoc();
-  nsIScriptGlobalObject *sgo;
+  nsIScriptGlobalObject* sgo = aBoundElement->OwnerDoc()->GetScopeObject();
 
-  if (!(sgo = ownerDoc->GetScopeObject())) {
+  if (!sgo) {
     return NS_ERROR_UNEXPECTED;
   }
 
-  JSObject * scriptObject = (JSObject *) aScriptObject;
-  NS_ASSERTION(scriptObject, "uh-oh, script Object should NOT be null or bad things will happen");
-  if (!scriptObject)
+  NS_ASSERTION(aScriptObject, "uh-oh, script Object should NOT be null or bad things will happen");
+  if (!aScriptObject)
     return NS_ERROR_FAILURE;
 
-  JSObject * targetClassObject = (JSObject *) aTargetClassObject;
-  JSObject * globalObject = sgo->GetGlobalJSObject();
+  JSObject* globalObject = sgo->GetGlobalJSObject();
 
   // now we want to reevaluate our property using aContext and the script object for this window...
-  if (mJSMethodObject && targetClassObject) {
+  if (mJSMethodObject && aTargetClassObject) {
     nsDependentString name(mName);
     JSAutoRequest ar(cx);
-    JSAutoEnterCompartment ac;
-
-    if (!ac.enter(cx, globalObject)) {
-      return NS_ERROR_UNEXPECTED;
-    }
+    JSAutoCompartment ac(cx, globalObject);
 
     JSObject * method = ::JS_CloneFunctionObject(cx, mJSMethodObject, globalObject);
     if (!method) {
       return NS_ERROR_OUT_OF_MEMORY;
     }
 
-    if (!::JS_DefineUCProperty(cx, targetClassObject,
-                               reinterpret_cast<const jschar*>(mName), 
+    if (!::JS_DefineUCProperty(cx, aTargetClassObject,
+                               static_cast<const jschar*>(mName),
                                name.Length(), OBJECT_TO_JSVAL(method),
                                NULL, NULL, JSPROP_ENUMERATE)) {
       return NS_ERROR_OUT_OF_MEMORY;
@@ -185,7 +150,7 @@ nsXBLProtoImplMethod::CompileMember(nsIScriptContext* aContext, const nsCString&
   // No parameters or body was supplied, so don't install method.
   if (!uncompiledMethod) {
     // Early return after which we consider ourselves compiled.
-    mJSMethodObject = nsnull;
+    mJSMethodObject = nullptr;
 
     return NS_OK;
   }
@@ -195,22 +160,22 @@ nsXBLProtoImplMethod::CompileMember(nsIScriptContext* aContext, const nsCString&
     delete uncompiledMethod;
 
     // Early return after which we consider ourselves compiled.
-    mJSMethodObject = nsnull;
+    mJSMethodObject = nullptr;
 
     return NS_OK;
   }
 
   // We have a method.
   // Allocate an array for our arguments.
-  PRInt32 paramCount = uncompiledMethod->GetParameterCount();
-  char** args = nsnull;
+  int32_t paramCount = uncompiledMethod->GetParameterCount();
+  char** args = nullptr;
   if (paramCount > 0) {
     args = new char*[paramCount];
     if (!args)
       return NS_ERROR_OUT_OF_MEMORY;
 
     // Add our parameters to our args array.
-    PRInt32 argPos = 0; 
+    int32_t argPos = 0; 
     for (nsXBLParameter* curr = uncompiledMethod->mParameters; 
          curr; 
          curr = curr->mNext) {
@@ -229,12 +194,12 @@ nsXBLProtoImplMethod::CompileMember(nsIScriptContext* aContext, const nsCString&
   // and then define it.
   NS_ConvertUTF16toUTF8 cname(mName);
   nsCAutoString functionUri(aClassStr);
-  PRInt32 hash = functionUri.RFindChar('#');
+  int32_t hash = functionUri.RFindChar('#');
   if (hash != kNotFound) {
     functionUri.Truncate(hash);
   }
 
-  JSObject* methodObject = nsnull;
+  JSObject* methodObject = nullptr;
   nsresult rv = aContext->CompileFunction(aClassObject,
                                           cname,
                                           paramCount,
@@ -250,7 +215,7 @@ nsXBLProtoImplMethod::CompileMember(nsIScriptContext* aContext, const nsCString&
   delete uncompiledMethod;
   delete [] args;
   if (NS_FAILED(rv)) {
-    SetUncompiledMethod(nsnull);
+    SetUncompiledMethod(nullptr);
     return rv;
   }
 
@@ -263,7 +228,7 @@ void
 nsXBLProtoImplMethod::Trace(TraceCallback aCallback, void *aClosure) const
 {
   if (IsCompiled() && mJSMethodObject) {
-    aCallback(nsIProgrammingLanguage::JAVASCRIPT, mJSMethodObject, "mJSMethodObject", aClosure);
+    aCallback(mJSMethodObject, "mJSMethodObject", aClosure);
   }
 }
 
@@ -273,7 +238,7 @@ nsXBLProtoImplMethod::Read(nsIScriptContext* aContext,
 {
   nsresult rv = XBL_DeserializeFunction(aContext, aStream, &mJSMethodObject);
   if (NS_FAILED(rv)) {
-    SetUncompiledMethod(nsnull);
+    SetUncompiledMethod(nullptr);
     return rv;
   }
 
@@ -341,10 +306,7 @@ nsXBLProtoImplAnonymousMethod::Execute(nsIContent* aBoundElement)
   JSObject* thisObject = JSVAL_TO_OBJECT(v);
 
   JSAutoRequest ar(cx);
-  JSAutoEnterCompartment ac;
-
-  if (!ac.enter(cx, thisObject))
-    return NS_ERROR_UNEXPECTED;
+  JSAutoCompartment ac(cx, thisObject);
 
   // Clone the function object, using thisObject as the parent so "this" is in
   // the scope chain of the resulting function (for backwards compat to the
@@ -367,7 +329,7 @@ nsXBLProtoImplAnonymousMethod::Execute(nsIContent* aBoundElement)
   if (NS_SUCCEEDED(rv)) {
     jsval retval;
     ok = ::JS_CallFunctionValue(cx, thisObject, OBJECT_TO_JSVAL(method),
-                                0 /* argc */, nsnull /* argv */, &retval);
+                                0 /* argc */, nullptr /* argv */, &retval);
   }
 
   if (!ok) {
