@@ -1,40 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Original Author: David W. Hyatt (hyatt@netscape.com)
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "mozilla/Util.h"
 
@@ -49,7 +16,6 @@
 #include "nsINameSpaceManager.h"
 #include "nsIScriptContext.h"
 #include "nsIDocument.h"
-#include "nsIDOMDocument.h"
 #include "nsIJSEventListener.h"
 #include "nsIController.h"
 #include "nsIControllers.h"
@@ -61,8 +27,6 @@
 #include "nsEventListenerManager.h"
 #include "nsIDOMEventTarget.h"
 #include "nsIDOMEventListener.h"
-#include "nsIPrivateDOMEvent.h"
-#include "nsIDOMNSEvent.h"
 #include "nsPIDOMWindow.h"
 #include "nsPIWindowRoot.h"
 #include "nsIDOMWindow.h"
@@ -87,22 +51,25 @@ using namespace mozilla;
 static NS_DEFINE_CID(kDOMScriptObjectFactoryCID,
                      NS_DOM_SCRIPT_OBJECT_FACTORY_CID);
 
-PRUint32 nsXBLPrototypeHandler::gRefCnt = 0;
+uint32_t nsXBLPrototypeHandler::gRefCnt = 0;
 
-PRInt32 nsXBLPrototypeHandler::kMenuAccessKey = -1;
-PRInt32 nsXBLPrototypeHandler::kAccelKey = -1;
+int32_t nsXBLPrototypeHandler::kMenuAccessKey = -1;
+int32_t nsXBLPrototypeHandler::kAccelKey = -1;
 
-const PRInt32 nsXBLPrototypeHandler::cShift = (1<<0);
-const PRInt32 nsXBLPrototypeHandler::cAlt = (1<<1);
-const PRInt32 nsXBLPrototypeHandler::cControl = (1<<2);
-const PRInt32 nsXBLPrototypeHandler::cMeta = (1<<3);
+const int32_t nsXBLPrototypeHandler::cShift = (1<<0);
+const int32_t nsXBLPrototypeHandler::cAlt = (1<<1);
+const int32_t nsXBLPrototypeHandler::cControl = (1<<2);
+const int32_t nsXBLPrototypeHandler::cMeta = (1<<3);
+const int32_t nsXBLPrototypeHandler::cOS = (1<<4);
 
-const PRInt32 nsXBLPrototypeHandler::cShiftMask = (1<<4);
-const PRInt32 nsXBLPrototypeHandler::cAltMask = (1<<5);
-const PRInt32 nsXBLPrototypeHandler::cControlMask = (1<<6);
-const PRInt32 nsXBLPrototypeHandler::cMetaMask = (1<<7);
+const int32_t nsXBLPrototypeHandler::cShiftMask = (1<<5);
+const int32_t nsXBLPrototypeHandler::cAltMask = (1<<6);
+const int32_t nsXBLPrototypeHandler::cControlMask = (1<<7);
+const int32_t nsXBLPrototypeHandler::cMetaMask = (1<<8);
+const int32_t nsXBLPrototypeHandler::cOSMask = (1<<9);
 
-const PRInt32 nsXBLPrototypeHandler::cAllModifiers = cShiftMask | cAltMask | cControlMask | cMetaMask;
+const int32_t nsXBLPrototypeHandler::cAllModifiers =
+  cShiftMask | cAltMask | cControlMask | cMetaMask | cOSMask;
 
 nsXBLPrototypeHandler::nsXBLPrototypeHandler(const PRUnichar* aEvent,
                                              const PRUnichar* aPhase,
@@ -117,24 +84,24 @@ nsXBLPrototypeHandler::nsXBLPrototypeHandler(const PRUnichar* aEvent,
                                              const PRUnichar* aPreventDefault,
                                              const PRUnichar* aAllowUntrusted,
                                              nsXBLPrototypeBinding* aBinding,
-                                             PRUint32 aLineNumber)
-  : mHandlerText(nsnull),
+                                             uint32_t aLineNumber)
+  : mHandlerText(nullptr),
     mLineNumber(aLineNumber),
-    mNextHandler(nsnull),
+    mNextHandler(nullptr),
     mPrototypeBinding(aBinding)
 {
   Init();
 
-  ConstructPrototype(nsnull, aEvent, aPhase, aAction, aCommand, aKeyCode,
+  ConstructPrototype(nullptr, aEvent, aPhase, aAction, aCommand, aKeyCode,
                      aCharCode, aModifiers, aButton, aClickCount,
                      aGroup, aPreventDefault, aAllowUntrusted);
 }
 
 nsXBLPrototypeHandler::nsXBLPrototypeHandler(nsIContent* aHandlerElement)
-  : mHandlerElement(nsnull),
+  : mHandlerElement(nullptr),
     mLineNumber(0),
-    mNextHandler(nsnull),
-    mPrototypeBinding(nsnull)
+    mNextHandler(nullptr),
+    mPrototypeBinding(nullptr)
 {
   Init();
 
@@ -143,9 +110,9 @@ nsXBLPrototypeHandler::nsXBLPrototypeHandler(nsIContent* aHandlerElement)
 }
 
 nsXBLPrototypeHandler::nsXBLPrototypeHandler(nsXBLPrototypeBinding* aBinding)
-  : mHandlerText(nsnull),
-    mLineNumber(nsnull),
-    mNextHandler(nsnull),
+  : mHandlerText(nullptr),
+    mLineNumber(0),
+    mNextHandler(nullptr),
     mPrototypeBinding(aBinding)
 {
   Init();
@@ -169,12 +136,12 @@ nsXBLPrototypeHandler::GetHandlerElement()
 {
   if (mType & NS_HANDLER_TYPE_XUL) {
     nsCOMPtr<nsIContent> element = do_QueryReferent(mHandlerElement);
-    nsIContent* el = nsnull;
+    nsIContent* el = nullptr;
     element.swap(el);
     return el;
   }
 
-  return nsnull;
+  return nullptr;
 }
 
 void
@@ -242,11 +209,8 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventTarget* aTarget,
   // XUL handlers and commands shouldn't be triggered by non-trusted
   // events.
   if (isXULKey || isXBLCommand) {
-    nsCOMPtr<nsIDOMNSEvent> domNSEvent = do_QueryInterface(aEvent);
     bool trustedEvent = false;
-    if (domNSEvent) {
-      domNSEvent->GetIsTrusted(&trustedEvent);
-    }
+    aEvent->GetIsTrusted(&trustedEvent);
 
     if (!trustedEvent)
       return NS_OK;
@@ -314,6 +278,12 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventTarget* aTarget,
     scriptTarget = aTarget;
   }
 
+  // We're about to create a new nsJSEventListener, which means that we're
+  // responsible for pushing the context of the event target. See the similar
+  // comment in nsEventManagerListener.cpp.
+  nsCxPusher pusher;
+  NS_ENSURE_STATE(pusher.Push(aTarget));
+
   rv = EnsureEventHandler(boundGlobal, boundContext, onEventAtom, handler);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -362,7 +332,7 @@ nsXBLPrototypeHandler::EnsureEventHandler(nsIScriptGlobalObject* aGlobal,
   nsCAutoString bindingURI;
   mPrototypeBinding->DocURI()->GetSpec(bindingURI);
 
-  PRUint32 argCount;
+  uint32_t argCount;
   const char **argNames;
   nsContentUtils::GetEventArgNames(kNameSpaceID_XBL, aName, &argCount,
                                    &argNames);
@@ -386,21 +356,17 @@ nsXBLPrototypeHandler::DispatchXBLCommand(nsIDOMEventTarget* aTarget, nsIDOMEven
   // This is a special-case optimization to make command handling fast.
   // It isn't really a part of XBL, but it helps speed things up.
 
-  // See if preventDefault has been set.  If so, don't execute.
-  bool preventDefault = false;
-  nsCOMPtr<nsIDOMNSEvent> domNSEvent = do_QueryInterface(aEvent);
-  if (domNSEvent) {
-    domNSEvent->GetPreventDefault(&preventDefault);
-  }
-
-  if (preventDefault)
-    return NS_OK;
-
-  nsCOMPtr<nsIPrivateDOMEvent> privateEvent = do_QueryInterface(aEvent);
-  if (privateEvent) {
-    bool dispatchStopped = privateEvent->IsDispatchStopped();
-    if (dispatchStopped)
+  if (aEvent) {
+    // See if preventDefault has been set.  If so, don't execute.
+    bool preventDefault = false;
+    aEvent->GetPreventDefault(&preventDefault);
+    if (preventDefault) {
       return NS_OK;
+    }
+    bool dispatchStopped = aEvent->IsDispatchStopped();
+    if (dispatchStopped) {
+      return NS_OK;
+    }
   }
 
   // Instead of executing JS, let's get the controller for the bound
@@ -526,6 +492,8 @@ nsXBLPrototypeHandler::DispatchXULKeyCommand(nsIDOMEvent* aEvent)
     return NS_ERROR_FAILURE;
   }
 
+  // XXX We should use widget::Modifiers for supporting all modifiers.
+
   bool isAlt = false;
   bool isControl = false;
   bool isShift = false;
@@ -536,7 +504,7 @@ nsXBLPrototypeHandler::DispatchXULKeyCommand(nsIDOMEvent* aEvent)
   keyEvent->GetMetaKey(&isMeta);
 
   nsContentUtils::DispatchXULCommand(handlerElement, true,
-                                     nsnull, nsnull,
+                                     nullptr, nullptr,
                                      isControl, isAlt, isShift, isMeta);
   return NS_OK;
 }
@@ -585,19 +553,19 @@ nsXBLPrototypeHandler::GetController(nsIDOMEventTarget* aTarget)
   if (controllers) {
     controllers->GetControllerAt(0, &controller);  // return reference
   }
-  else controller = nsnull;
+  else controller = nullptr;
 
   return controller;
 }
 
 bool
 nsXBLPrototypeHandler::KeyEventMatched(nsIDOMKeyEvent* aKeyEvent,
-                                       PRUint32 aCharCode,
+                                       uint32_t aCharCode,
                                        bool aIgnoreShiftKey)
 {
   if (mDetail != -1) {
     // Get the keycode or charcode of the key event.
-    PRUint32 code;
+    uint32_t code;
 
     if (mMisc) {
       if (aCharCode)
@@ -610,7 +578,7 @@ nsXBLPrototypeHandler::KeyEventMatched(nsIDOMKeyEvent* aKeyEvent,
     else
       aKeyEvent->GetKeyCode(&code);
 
-    if (code != PRUint32(mDetail))
+    if (code != uint32_t(mDetail))
       return false;
   }
 
@@ -623,12 +591,12 @@ nsXBLPrototypeHandler::MouseEventMatched(nsIDOMMouseEvent* aMouseEvent)
   if (mDetail == -1 && mMisc == 0 && (mKeyMask & cAllModifiers) == 0)
     return true; // No filters set up. It's generic.
 
-  PRUint16 button;
+  uint16_t button;
   aMouseEvent->GetButton(&button);
   if (mDetail != -1 && (button != mDetail))
     return false;
 
-  PRInt32 clickcount;
+  int32_t clickcount;
   aMouseEvent->GetDetail(&clickcount);
   if (mMisc != 0 && (clickcount != mMisc))
     return false;
@@ -639,7 +607,7 @@ nsXBLPrototypeHandler::MouseEventMatched(nsIDOMMouseEvent* aMouseEvent)
 struct keyCodeData {
   const char* str;
   size_t strlength;
-  PRUint32 keycode;
+  uint32_t keycode;
 };
 
 // All of these must be uppercase, since the function below does
@@ -647,138 +615,22 @@ struct keyCodeData {
 // XXX: be sure to check this periodically for new symbol additions!
 static const keyCodeData gKeyCodes[] = {
 
-#define KEYCODE_ENTRY(str) {#str, sizeof(#str) - 1, nsIDOMKeyEvent::DOM_##str}
-#define KEYCODE_ENTRY2(str, code) {str, sizeof(str) - 1, code}
-
-  KEYCODE_ENTRY(VK_CANCEL),
-  KEYCODE_ENTRY2("VK_BACK", nsIDOMKeyEvent::DOM_VK_BACK_SPACE),
-  KEYCODE_ENTRY(VK_TAB),
-  KEYCODE_ENTRY(VK_CLEAR),
-  KEYCODE_ENTRY(VK_RETURN),
-  KEYCODE_ENTRY(VK_ENTER),
-  KEYCODE_ENTRY(VK_SHIFT),
-  KEYCODE_ENTRY(VK_CONTROL),
-  KEYCODE_ENTRY(VK_ALT),
-  KEYCODE_ENTRY(VK_PAUSE),
-  KEYCODE_ENTRY(VK_CAPS_LOCK),
-  KEYCODE_ENTRY(VK_ESCAPE),
-  KEYCODE_ENTRY(VK_SPACE),
-  KEYCODE_ENTRY(VK_PAGE_UP),
-  KEYCODE_ENTRY(VK_PAGE_DOWN),
-  KEYCODE_ENTRY(VK_END),
-  KEYCODE_ENTRY(VK_HOME),
-  KEYCODE_ENTRY(VK_LEFT),
-  KEYCODE_ENTRY(VK_UP),
-  KEYCODE_ENTRY(VK_RIGHT),
-  KEYCODE_ENTRY(VK_DOWN),
-  KEYCODE_ENTRY(VK_PRINTSCREEN),
-  KEYCODE_ENTRY(VK_INSERT),
-  KEYCODE_ENTRY(VK_HELP),
-  KEYCODE_ENTRY(VK_DELETE),
-  KEYCODE_ENTRY(VK_0),
-  KEYCODE_ENTRY(VK_1),
-  KEYCODE_ENTRY(VK_2),
-  KEYCODE_ENTRY(VK_3),
-  KEYCODE_ENTRY(VK_4),
-  KEYCODE_ENTRY(VK_5),
-  KEYCODE_ENTRY(VK_6),
-  KEYCODE_ENTRY(VK_7),
-  KEYCODE_ENTRY(VK_8),
-  KEYCODE_ENTRY(VK_9),
-  KEYCODE_ENTRY(VK_SEMICOLON),
-  KEYCODE_ENTRY(VK_EQUALS),
-  KEYCODE_ENTRY(VK_A),
-  KEYCODE_ENTRY(VK_B),
-  KEYCODE_ENTRY(VK_C),
-  KEYCODE_ENTRY(VK_D),
-  KEYCODE_ENTRY(VK_E),
-  KEYCODE_ENTRY(VK_F),
-  KEYCODE_ENTRY(VK_G),
-  KEYCODE_ENTRY(VK_H),
-  KEYCODE_ENTRY(VK_I),
-  KEYCODE_ENTRY(VK_J),
-  KEYCODE_ENTRY(VK_K),
-  KEYCODE_ENTRY(VK_L),
-  KEYCODE_ENTRY(VK_M),
-  KEYCODE_ENTRY(VK_N),
-  KEYCODE_ENTRY(VK_O),
-  KEYCODE_ENTRY(VK_P),
-  KEYCODE_ENTRY(VK_Q),
-  KEYCODE_ENTRY(VK_R),
-  KEYCODE_ENTRY(VK_S),
-  KEYCODE_ENTRY(VK_T),
-  KEYCODE_ENTRY(VK_U),
-  KEYCODE_ENTRY(VK_V),
-  KEYCODE_ENTRY(VK_W),
-  KEYCODE_ENTRY(VK_X),
-  KEYCODE_ENTRY(VK_Y),
-  KEYCODE_ENTRY(VK_Z),
-  KEYCODE_ENTRY(VK_NUMPAD0),
-  KEYCODE_ENTRY(VK_NUMPAD1),
-  KEYCODE_ENTRY(VK_NUMPAD2),
-  KEYCODE_ENTRY(VK_NUMPAD3),
-  KEYCODE_ENTRY(VK_NUMPAD4),
-  KEYCODE_ENTRY(VK_NUMPAD5),
-  KEYCODE_ENTRY(VK_NUMPAD6),
-  KEYCODE_ENTRY(VK_NUMPAD7),
-  KEYCODE_ENTRY(VK_NUMPAD8),
-  KEYCODE_ENTRY(VK_NUMPAD9),
-  KEYCODE_ENTRY(VK_MULTIPLY),
-  KEYCODE_ENTRY(VK_ADD),
-  KEYCODE_ENTRY(VK_SEPARATOR),
-  KEYCODE_ENTRY(VK_SUBTRACT),
-  KEYCODE_ENTRY(VK_DECIMAL),
-  KEYCODE_ENTRY(VK_DIVIDE),
-  KEYCODE_ENTRY(VK_F1),
-  KEYCODE_ENTRY(VK_F2),
-  KEYCODE_ENTRY(VK_F3),
-  KEYCODE_ENTRY(VK_F4),
-  KEYCODE_ENTRY(VK_F5),
-  KEYCODE_ENTRY(VK_F6),
-  KEYCODE_ENTRY(VK_F7),
-  KEYCODE_ENTRY(VK_F8),
-  KEYCODE_ENTRY(VK_F9),
-  KEYCODE_ENTRY(VK_F10),
-  KEYCODE_ENTRY(VK_F11),
-  KEYCODE_ENTRY(VK_F12),
-  KEYCODE_ENTRY(VK_F13),
-  KEYCODE_ENTRY(VK_F14),
-  KEYCODE_ENTRY(VK_F15),
-  KEYCODE_ENTRY(VK_F16),
-  KEYCODE_ENTRY(VK_F17),
-  KEYCODE_ENTRY(VK_F18),
-  KEYCODE_ENTRY(VK_F19),
-  KEYCODE_ENTRY(VK_F20),
-  KEYCODE_ENTRY(VK_F21),
-  KEYCODE_ENTRY(VK_F22),
-  KEYCODE_ENTRY(VK_F23),
-  KEYCODE_ENTRY(VK_F24),
-  KEYCODE_ENTRY(VK_NUM_LOCK),
-  KEYCODE_ENTRY(VK_SCROLL_LOCK),
-  KEYCODE_ENTRY(VK_COMMA),
-  KEYCODE_ENTRY(VK_PERIOD),
-  KEYCODE_ENTRY(VK_SLASH),
-  KEYCODE_ENTRY(VK_BACK_QUOTE),
-  KEYCODE_ENTRY(VK_OPEN_BRACKET),
-  KEYCODE_ENTRY(VK_BACK_SLASH),
-  KEYCODE_ENTRY(VK_CLOSE_BRACKET),
-  KEYCODE_ENTRY(VK_QUOTE)
-
-#undef KEYCODE_ENTRY
-#undef KEYCODE_ENTRY2
-
+#define NS_DEFINE_VK(aDOMKeyName, aDOMKeyCode) \
+  { #aDOMKeyName, sizeof(#aDOMKeyName) - 1, aDOMKeyCode }
+#include "nsVKList.h"
+#undef NS_DEFINE_VK
 };
 
-PRInt32 nsXBLPrototypeHandler::GetMatchingKeyCode(const nsAString& aKeyName)
+int32_t nsXBLPrototypeHandler::GetMatchingKeyCode(const nsAString& aKeyName)
 {
   nsCAutoString keyName;
   keyName.AssignWithConversion(aKeyName);
   ToUpperCase(keyName); // We want case-insensitive comparison with data
                         // stored as uppercase.
 
-  PRUint32 keyNameLength = keyName.Length();
+  uint32_t keyNameLength = keyName.Length();
   const char* keyNameStr = keyName.get();
-  for (PRUint16 i = 0; i < (sizeof(gKeyCodes) / sizeof(gKeyCodes[0])); ++i)
+  for (uint16_t i = 0; i < (sizeof(gKeyCodes) / sizeof(gKeyCodes[0])); ++i)
     if (keyNameLength == gKeyCodes[i].strlength &&
         !nsCRT::strcmp(gKeyCodes[i].str, keyNameStr))
       return gKeyCodes[i].keycode;
@@ -786,17 +638,18 @@ PRInt32 nsXBLPrototypeHandler::GetMatchingKeyCode(const nsAString& aKeyName)
   return 0;
 }
 
-PRInt32 nsXBLPrototypeHandler::KeyToMask(PRInt32 key)
+int32_t nsXBLPrototypeHandler::KeyToMask(int32_t key)
 {
   switch (key)
   {
     case nsIDOMKeyEvent::DOM_VK_META:
       return cMeta | cMetaMask;
-      break;
+
+    case nsIDOMKeyEvent::DOM_VK_WIN:
+      return cOS | cOSMask;
 
     case nsIDOMKeyEvent::DOM_VK_ALT:
       return cAlt | cAltMask;
-      break;
 
     case nsIDOMKeyEvent::DOM_VK_CONTROL:
     default:
@@ -847,7 +700,7 @@ nsXBLPrototypeHandler::ConstructPrototype(nsIContent* aKeyElement,
   }
   else {
     mType |= aCommand ? NS_HANDLER_TYPE_XBL_COMMAND : NS_HANDLER_TYPE_XBL_JS;
-    mHandlerText = nsnull;
+    mHandlerText = nullptr;
   }
 
   mDetail = -1;
@@ -903,6 +756,8 @@ nsXBLPrototypeHandler::ConstructPrototype(nsIContent* aKeyElement,
         mKeyMask |= cAlt | cAltMask;
       else if (PL_strcmp(token, "meta") == 0)
         mKeyMask |= cMeta | cMetaMask;
+      else if (PL_strcmp(token, "os") == 0)
+        mKeyMask |= cOS | cOSMask;
       else if (PL_strcmp(token, "control") == 0)
         mKeyMask |= cControl | cControlMask;
       else if (PL_strcmp(token, "accel") == 0)
@@ -910,7 +765,7 @@ nsXBLPrototypeHandler::ConstructPrototype(nsIContent* aKeyElement,
       else if (PL_strcmp(token, "access") == 0)
         mKeyMask |= KeyToMask(kMenuAccessKey);
       else if (PL_strcmp(token, "any") == 0)
-        mKeyMask &= ~(mKeyMask << 4);
+        mKeyMask &= ~(mKeyMask << 5);
     
       token = nsCRT::strtok( newStr, ", \t", &newStr );
     }
@@ -930,18 +785,21 @@ nsXBLPrototypeHandler::ConstructPrototype(nsIContent* aKeyElement,
   if (!key.IsEmpty()) {
     if (mKeyMask == 0)
       mKeyMask = cAllModifiers;
-    nsContentUtils::ASCIIToLower(key);
+    ToLowerCase(key);
 
     // We have a charcode.
     mMisc = 1;
     mDetail = key[0];
-    const PRUint8 GTK2Modifiers = cShift | cControl | cShiftMask | cControlMask;
+    const uint8_t GTK2Modifiers = cShift | cControl | cShiftMask | cControlMask;
     if ((mKeyMask & GTK2Modifiers) == GTK2Modifiers &&
-        modifiers.First() != PRUnichar(',') && mDetail == 'u')
+        modifiers.First() != PRUnichar(',') &&
+        (mDetail == 'u' || mDetail == 'U'))
       ReportKeyConflict(key.get(), modifiers.get(), aKeyElement, "GTK2Conflict");
-    const PRUint8 WinModifiers = cControl | cAlt | cControlMask | cAltMask;
+    const uint8_t WinModifiers = cControl | cAlt | cControlMask | cAltMask;
     if ((mKeyMask & WinModifiers) == WinModifiers &&
-        modifiers.First() != PRUnichar(',') && ('a' <= mDetail && mDetail <= 'z'))
+        modifiers.First() != PRUnichar(',') &&
+        (('A' <= mDetail && mDetail <= 'Z') ||
+         ('a' <= mDetail && mDetail <= 'z')))
       ReportKeyConflict(key.get(), modifiers.get(), aKeyElement, "WinConflict");
   }
   else {
@@ -992,39 +850,45 @@ nsXBLPrototypeHandler::ReportKeyConflict(const PRUnichar* aKey, const PRUnichar*
                                   nsContentUtils::eXBL_PROPERTIES,
                                   aMessageName,
                                   params, ArrayLength(params),
-                                  nsnull, EmptyString(), mLineNumber);
+                                  nullptr, EmptyString(), mLineNumber);
 }
 
 bool
 nsXBLPrototypeHandler::ModifiersMatchMask(nsIDOMUIEvent* aEvent,
                                           bool aIgnoreShiftKey)
 {
-  nsCOMPtr<nsIDOMKeyEvent> key(do_QueryInterface(aEvent));
-  nsCOMPtr<nsIDOMMouseEvent> mouse(do_QueryInterface(aEvent));
+  nsEvent* event = aEvent->GetInternalNSEvent();
+  NS_ENSURE_TRUE(event && NS_IS_INPUT_EVENT(event), false);
+  nsInputEvent* inputEvent = static_cast<nsInputEvent*>(event);
 
-  bool keyPresent;
   if (mKeyMask & cMetaMask) {
-    key ? key->GetMetaKey(&keyPresent) : mouse->GetMetaKey(&keyPresent);
-    if (keyPresent != ((mKeyMask & cMeta) != 0))
+    if (inputEvent->IsMeta() != ((mKeyMask & cMeta) != 0)) {
       return false;
+    }
+  }
+
+  if (mKeyMask & cOSMask) {
+    if (inputEvent->IsOS() != ((mKeyMask & cOS) != 0)) {
+      return false;
+    }
   }
 
   if (mKeyMask & cShiftMask && !aIgnoreShiftKey) {
-    key ? key->GetShiftKey(&keyPresent) : mouse->GetShiftKey(&keyPresent);
-    if (keyPresent != ((mKeyMask & cShift) != 0))
+    if (inputEvent->IsShift() != ((mKeyMask & cShift) != 0)) {
       return false;
+    }
   }
 
   if (mKeyMask & cAltMask) {
-    key ? key->GetAltKey(&keyPresent) : mouse->GetAltKey(&keyPresent);
-    if (keyPresent != ((mKeyMask & cAlt) != 0))
+    if (inputEvent->IsAlt() != ((mKeyMask & cAlt) != 0)) {
       return false;
+    }
   }
 
   if (mKeyMask & cControlMask) {
-    key ? key->GetCtrlKey(&keyPresent) : mouse->GetCtrlKey(&keyPresent);
-    if (keyPresent != ((mKeyMask & cControl) != 0))
+    if (inputEvent->IsControl() != ((mKeyMask & cControl) != 0)) {
       return false;
+    }
   }
 
   return true;
@@ -1035,14 +899,14 @@ nsXBLPrototypeHandler::Read(nsIScriptContext* aContext, nsIObjectInputStream* aS
 {
   nsresult rv = aStream->Read8(&mPhase);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = aStream->Read8(&mKeyMask);
-  NS_ENSURE_SUCCESS(rv, rv);
   rv = aStream->Read8(&mType);
   NS_ENSURE_SUCCESS(rv, rv);
   rv = aStream->Read8(&mMisc);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  PRUint32 detail; 
+  rv = aStream->Read32(reinterpret_cast<uint32_t*>(&mKeyMask));
+  NS_ENSURE_SUCCESS(rv, rv);
+  uint32_t detail; 
   rv = aStream->Read32(&detail);
   NS_ENSURE_SUCCESS(rv, rv);
   mDetail = detail;
@@ -1077,12 +941,12 @@ nsXBLPrototypeHandler::Write(nsIScriptContext* aContext, nsIObjectOutputStream*
   nsresult rv = aStream->Write8(type);
   rv = aStream->Write8(mPhase);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = aStream->Write8(mKeyMask);
-  NS_ENSURE_SUCCESS(rv, rv);
   rv = aStream->Write8(mType);
   NS_ENSURE_SUCCESS(rv, rv);
   rv = aStream->Write8(mMisc);
   NS_ENSURE_SUCCESS(rv, rv);
+  rv = aStream->Write32(static_cast<uint32_t>(mKeyMask));
+  NS_ENSURE_SUCCESS(rv, rv);
   rv = aStream->Write32(mDetail);
   NS_ENSURE_SUCCESS(rv, rv);
 
