@@ -1,40 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is the Mozilla XTF project.
- *
- * The Initial Developer of the Original Code is
- * Alex Fritze.
- * Portions created by the Initial Developer are Copyright (C) 2004
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Alex Fritze <alex@croczilla.com> (original author)
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsXTFElementWrapper.h"
 #include "nsIXTFElement.h"
@@ -81,7 +48,11 @@ nsXTFElementWrapper::nsXTFElementWrapper(already_AddRefed<nsINodeInfo> aNodeInfo
 nsXTFElementWrapper::~nsXTFElementWrapper()
 {
   mXTFElement->OnDestroyed();
-  mXTFElement = nsnull;
+  mXTFElement = nullptr;
+  if (mClassInfo) {
+    mClassInfo->Disconnect();
+    mClassInfo = nullptr;
+  }
 }
 
 nsresult
@@ -89,7 +60,7 @@ nsXTFElementWrapper::Init()
 {
   // pass a weak wrapper (non base object ref-counted), so that
   // our mXTFElement can safely addref/release.
-  nsISupports* weakWrapper = nsnull;
+  nsISupports* weakWrapper = nullptr;
   nsresult rv = NS_NewXTFWeakTearoff(NS_GET_IID(nsIXTFElementWrapper),
                                      (nsIXTFElementWrapper*)this,
                                      &weakWrapper);
@@ -126,13 +97,19 @@ nsXTFElementWrapper::QueryInterface(REFNSIID aIID, void** aInstancePtr)
   NS_IMPL_QUERY_CYCLE_COLLECTION(nsXTFElementWrapper)
   if (aIID.Equals(NS_GET_IID(nsIClassInfo)) ||
       aIID.Equals(NS_GET_IID(nsXPCClassInfo))) {
-    *aInstancePtr = static_cast<nsIClassInfo*>(this);
-    NS_ADDREF_THIS();
+    if (!mClassInfo) {
+      mClassInfo = new nsXTFClassInfo(this);
+    }
+    NS_ADDREF(mClassInfo);
+    *aInstancePtr = static_cast<nsIClassInfo*>(mClassInfo);
     return NS_OK;
   }
   if (aIID.Equals(NS_GET_IID(nsIXPCScriptable))) {
-    *aInstancePtr = static_cast<nsIXPCScriptable*>(this);
-    NS_ADDREF_THIS();
+    if (!mClassInfo) {
+      mClassInfo = new nsXTFClassInfo(this);
+    }
+    NS_ADDREF(mClassInfo);
+    *aInstancePtr = static_cast<nsIXPCScriptable*>(mClassInfo);
     return NS_OK;
   }
   if (aIID.Equals(NS_GET_IID(nsIXTFElementWrapper))) {
@@ -161,6 +138,15 @@ nsXTFElementWrapper::QueryInterface(REFNSIID aIID, void** aInstancePtr)
   return NS_ERROR_NO_INTERFACE;
 }
 
+nsXPCClassInfo*
+nsXTFElementWrapper::GetClassInfo()
+{
+  if (!mClassInfo) {
+    mClassInfo = new nsXTFClassInfo(this);
+  }
+  return mClassInfo;
+}
+
 //----------------------------------------------------------------------
 // nsIContent methods:
 
@@ -224,14 +210,14 @@ nsXTFElementWrapper::UnbindFromTree(bool aDeep, bool aNullParent)
   bool inDoc = IsInDoc();
   if (inDoc &&
       (mNotificationMask & nsIXTFElement::NOTIFY_WILL_CHANGE_DOCUMENT)) {
-    GetXTFElement()->WillChangeDocument(nsnull);
+    GetXTFElement()->WillChangeDocument(nullptr);
   }
 
   bool parentChanged = aNullParent && GetParent();
 
   if (parentChanged &&
       (mNotificationMask & nsIXTFElement::NOTIFY_WILL_CHANGE_PARENT)) {
-    GetXTFElement()->WillChangeParent(nsnull);
+    GetXTFElement()->WillChangeParent(nullptr);
   }
 
   if (mNotificationMask & nsIXTFElement::NOTIFY_PERFORM_ACCESSKEY)
@@ -241,17 +227,17 @@ nsXTFElementWrapper::UnbindFromTree(bool aDeep, bool aNullParent)
 
   if (parentChanged &&
       (mNotificationMask & nsIXTFElement::NOTIFY_PARENT_CHANGED)) {
-    GetXTFElement()->ParentChanged(nsnull);
+    GetXTFElement()->ParentChanged(nullptr);
   }
 
   if (inDoc &&
       (mNotificationMask & nsIXTFElement::NOTIFY_DOCUMENT_CHANGED)) {
-    GetXTFElement()->DocumentChanged(nsnull);
+    GetXTFElement()->DocumentChanged(nullptr);
   }
 }
 
 nsresult
-nsXTFElementWrapper::InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
+nsXTFElementWrapper::InsertChildAt(nsIContent* aKid, uint32_t aIndex,
                                    bool aNotify)
 {
   nsresult rv;
@@ -271,7 +257,7 @@ nsXTFElementWrapper::InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
 }
 
 void
-nsXTFElementWrapper::RemoveChildAt(PRUint32 aIndex, bool aNotify)
+nsXTFElementWrapper::RemoveChildAt(uint32_t aIndex, bool aNotify)
 {
   if (mNotificationMask & nsIXTFElement::NOTIFY_WILL_REMOVE_CHILD)
     GetXTFElement()->WillRemoveChild(aIndex);
@@ -288,7 +274,7 @@ nsXTFElementWrapper::GetIDAttributeName() const
 }
 
 nsresult
-nsXTFElementWrapper::SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
+nsXTFElementWrapper::SetAttr(int32_t aNameSpaceID, nsIAtom* aName,
                              nsIAtom* aPrefix, const nsAString& aValue,
                              bool aNotify)
 {
@@ -322,7 +308,7 @@ nsXTFElementWrapper::SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
 }
 
 bool
-nsXTFElementWrapper::GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, 
+nsXTFElementWrapper::GetAttr(int32_t aNameSpaceID, nsIAtom* aName, 
                              nsAString& aResult) const
 {
   if (aNameSpaceID==kNameSpaceID_None && HandledByInner(aName)) {
@@ -336,7 +322,7 @@ nsXTFElementWrapper::GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
 }
 
 bool
-nsXTFElementWrapper::HasAttr(PRInt32 aNameSpaceID, nsIAtom* aName) const
+nsXTFElementWrapper::HasAttr(int32_t aNameSpaceID, nsIAtom* aName) const
 {
   if (aNameSpaceID==kNameSpaceID_None && HandledByInner(aName)) {
     bool rval = false;
@@ -349,7 +335,7 @@ nsXTFElementWrapper::HasAttr(PRInt32 aNameSpaceID, nsIAtom* aName) const
 }
 
 bool
-nsXTFElementWrapper::AttrValueIs(PRInt32 aNameSpaceID,
+nsXTFElementWrapper::AttrValueIs(int32_t aNameSpaceID,
                                  nsIAtom* aName,
                                  const nsAString& aValue,
                                  nsCaseTreatment aCaseSensitive) const
@@ -372,7 +358,7 @@ nsXTFElementWrapper::AttrValueIs(PRInt32 aNameSpaceID,
 }
 
 bool
-nsXTFElementWrapper::AttrValueIs(PRInt32 aNameSpaceID,
+nsXTFElementWrapper::AttrValueIs(int32_t aNameSpaceID,
                                  nsIAtom* aName,
                                  nsIAtom* aValue,
                                  nsCaseTreatment aCaseSensitive) const
@@ -398,8 +384,8 @@ nsXTFElementWrapper::AttrValueIs(PRInt32 aNameSpaceID,
                                               aCaseSensitive);
 }
 
-PRInt32
-nsXTFElementWrapper::FindAttrValueIn(PRInt32 aNameSpaceID,
+int32_t
+nsXTFElementWrapper::FindAttrValueIn(int32_t aNameSpaceID,
                                      nsIAtom* aName,
                                      AttrValuesArray* aValues,
                                      nsCaseTreatment aCaseSensitive) const
@@ -414,7 +400,7 @@ nsXTFElementWrapper::FindAttrValueIn(PRInt32 aNameSpaceID,
       return ATTR_MISSING;
     }
     
-    for (PRInt32 i = 0; aValues[i]; ++i) {
+    for (int32_t i = 0; aValues[i]; ++i) {
       if (aCaseSensitive == eCaseMatters) {
         if ((*aValues[i])->Equals(ourVal)) {
           return i;
@@ -435,7 +421,7 @@ nsXTFElementWrapper::FindAttrValueIn(PRInt32 aNameSpaceID,
 }
 
 nsresult
-nsXTFElementWrapper::UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttr, 
+nsXTFElementWrapper::UnsetAttr(int32_t aNameSpaceID, nsIAtom* aAttr, 
                                bool aNotify)
 {
   nsresult rv;
@@ -478,9 +464,9 @@ nsXTFElementWrapper::UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttr,
 }
 
 const nsAttrName*
-nsXTFElementWrapper::GetAttrNameAt(PRUint32 aIndex) const
+nsXTFElementWrapper::GetAttrNameAt(uint32_t aIndex) const
 {
-  PRUint32 innerCount=0;
+  uint32_t innerCount=0;
   if (mAttributeHandler) {
     mAttributeHandler->GetAttributeCount(&innerCount);
   }
@@ -488,7 +474,7 @@ nsXTFElementWrapper::GetAttrNameAt(PRUint32 aIndex) const
   if (aIndex < innerCount) {
     nsCOMPtr<nsIAtom> localName;
     nsresult rv = mAttributeHandler->GetAttributeNameAt(aIndex, getter_AddRefs(localName));
-    NS_ENSURE_SUCCESS(rv, nsnull);
+    NS_ENSURE_SUCCESS(rv, nullptr);
 
     const_cast<nsXTFElementWrapper*>(this)->mTmpAttrName.SetTo(localName);
     return &mTmpAttrName;
@@ -498,10 +484,10 @@ nsXTFElementWrapper::GetAttrNameAt(PRUint32 aIndex) const
   }
 }
 
-PRUint32
+uint32_t
 nsXTFElementWrapper::GetAttrCount() const
 {
-  PRUint32 innerCount = 0;
+  uint32_t innerCount = 0;
   if (mAttributeHandler) {
     mAttributeHandler->GetAttributeCount(&innerCount);
   }
@@ -533,7 +519,7 @@ nsXTFElementWrapper::GetExistingAttrNameFromQName(const nsAString& aStr) const
     nsCOMPtr<nsIAtom> nameAtom = do_GetAtom(aStr);
     if (HandledByInner(nameAtom)) 
       nodeInfo = mNodeInfo->NodeInfoManager()->
-        GetNodeInfo(nameAtom, nsnull, kNameSpaceID_None,
+        GetNodeInfo(nameAtom, nullptr, kNameSpaceID_None,
                     nsIDOMNode::ATTRIBUTE_NODE).get();
   }
   
@@ -570,7 +556,7 @@ nsXTFElementWrapper::PerformAccesskey(bool aKeyCausesActivation,
 nsresult
 nsXTFElementWrapper::Clone(nsINodeInfo *aNodeInfo, nsINode **aResult) const
 {
-  *aResult = nsnull;
+  *aResult = nullptr;
   nsCOMPtr<nsIContent> it;
   nsContentUtils::GetXTFService()->CreateElement(getter_AddRefs(it),
                                                  aNodeInfo);
@@ -579,13 +565,13 @@ nsXTFElementWrapper::Clone(nsINodeInfo *aNodeInfo, nsINode **aResult) const
 
   nsXTFElementWrapper* wrapper =
     static_cast<nsXTFElementWrapper*>(it.get());
-  nsresult rv = CopyInnerTo(wrapper);
+  nsresult rv = const_cast<nsXTFElementWrapper*>(this)->CopyInnerTo(wrapper);
 
   if (NS_SUCCEEDED(rv)) {
     if (mAttributeHandler) {
-      PRUint32 innerCount = 0;
+      uint32_t innerCount = 0;
       mAttributeHandler->GetAttributeCount(&innerCount);
-      for (PRUint32 i = 0; i < innerCount; ++i) {
+      for (uint32_t i = 0; i < innerCount; ++i) {
         nsCOMPtr<nsIAtom> attrName;
         mAttributeHandler->GetAttributeNameAt(i, getter_AddRefs(attrName));
         if (attrName) {
@@ -673,16 +659,16 @@ nsXTFElementWrapper::HasAttribute(const nsAString& aName, bool* aReturn)
 //----------------------------------------------------------------------
 // nsIClassInfo implementation
 
-/* void getInterfaces (out PRUint32 count, [array, size_is (count), retval] out nsIIDPtr array); */
+/* void getInterfaces (out uint32_t count, [array, size_is (count), retval] out nsIIDPtr array); */
 NS_IMETHODIMP 
-nsXTFElementWrapper::GetInterfaces(PRUint32* aCount, nsIID*** aArray)
+nsXTFElementWrapper::GetInterfaces(uint32_t* aCount, nsIID*** aArray)
 {
-  *aArray = nsnull;
+  *aArray = nullptr;
   *aCount = 0;
-  PRUint32 baseCount = 0;
-  nsIID** baseArray = nsnull;
-  PRUint32 xtfCount = 0;
-  nsIID** xtfArray = nsnull;
+  uint32_t baseCount = 0;
+  nsIID** baseArray = nullptr;
+  uint32_t xtfCount = 0;
+  nsIID** xtfArray = nullptr;
 
   nsCOMPtr<nsIClassInfo> baseCi = GetBaseXPCClassInfo();
   if (baseCi) {
@@ -700,12 +686,12 @@ nsXTFElementWrapper::GetInterfaces(PRUint32* aCount, nsIID*** aArray)
     return NS_OK;
   }
 
-  PRUint32 count = baseCount + xtfCount;
+  uint32_t count = baseCount + xtfCount;
   nsIID** iids = static_cast<nsIID**>
                             (nsMemory::Alloc(count * sizeof(nsIID*)));
   NS_ENSURE_TRUE(iids, NS_ERROR_OUT_OF_MEMORY);
 
-  PRUint32 i = 0;
+  uint32_t i = 0;
   for (; i < baseCount; ++i) {
     iids[i] = static_cast<nsIID*>
                          (nsMemory::Clone(baseArray[i], sizeof(nsIID)));
@@ -736,12 +722,12 @@ nsXTFElementWrapper::GetInterfaces(PRUint32* aCount, nsIID*** aArray)
   return NS_OK;
 }
 
-/* nsISupports getHelperForLanguage (in PRUint32 language); */
+/* nsISupports getHelperForLanguage (in uint32_t language); */
 NS_IMETHODIMP 
-nsXTFElementWrapper::GetHelperForLanguage(PRUint32 language,
+nsXTFElementWrapper::GetHelperForLanguage(uint32_t language,
                                           nsISupports** aHelper)
 {
-  *aHelper = nsnull;
+  *aHelper = nullptr;
   nsCOMPtr<nsIClassInfo> ci = GetBaseXPCClassInfo();
   return
     ci ? ci->GetHelperForLanguage(language, aHelper) : NS_ERROR_NOT_AVAILABLE;
@@ -751,7 +737,7 @@ nsXTFElementWrapper::GetHelperForLanguage(PRUint32 language,
 NS_IMETHODIMP 
 nsXTFElementWrapper::GetContractID(char * *aContractID)
 {
-  *aContractID = nsnull;
+  *aContractID = nullptr;
   return NS_OK;
 }
 
@@ -759,7 +745,7 @@ nsXTFElementWrapper::GetContractID(char * *aContractID)
 NS_IMETHODIMP 
 nsXTFElementWrapper::GetClassDescription(char * *aClassDescription)
 {
-  *aClassDescription = nsnull;
+  *aClassDescription = nullptr;
   return NS_OK;
 }
 
@@ -767,21 +753,21 @@ nsXTFElementWrapper::GetClassDescription(char * *aClassDescription)
 NS_IMETHODIMP 
 nsXTFElementWrapper::GetClassID(nsCID * *aClassID)
 {
-  *aClassID = nsnull;
+  *aClassID = nullptr;
   return NS_OK;
 }
 
-/* readonly attribute PRUint32 implementationLanguage; */
+/* readonly attribute uint32_t implementationLanguage; */
 NS_IMETHODIMP 
-nsXTFElementWrapper::GetImplementationLanguage(PRUint32 *aImplementationLanguage)
+nsXTFElementWrapper::GetImplementationLanguage(uint32_t *aImplementationLanguage)
 {
   *aImplementationLanguage = nsIProgrammingLanguage::UNKNOWN;
   return NS_OK;
 }
 
-/* readonly attribute PRUint32 flags; */
+/* readonly attribute uint32_t flags; */
 NS_IMETHODIMP 
-nsXTFElementWrapper::GetFlags(PRUint32 *aFlags)
+nsXTFElementWrapper::GetFlags(uint32_t *aFlags)
 {
   *aFlags = nsIClassInfo::DOM_OBJECT;
   return NS_OK;
@@ -810,7 +796,7 @@ nsXTFElementWrapper::GetElementNode(nsIDOMElement * *aElementNode)
 NS_IMETHODIMP
 nsXTFElementWrapper::GetDocumentFrameElement(nsIDOMElement * *aDocumentFrameElement)
 {
-  *aDocumentFrameElement = nsnull;
+  *aDocumentFrameElement = nullptr;
   
   nsIDocument *doc = GetCurrentDoc();
   if (!doc) {
@@ -834,13 +820,13 @@ nsXTFElementWrapper::GetDocumentFrameElement(nsIDOMElement * *aDocumentFrameElem
 
 /* attribute unsigned long notificationMask; */
 NS_IMETHODIMP
-nsXTFElementWrapper::GetNotificationMask(PRUint32 *aNotificationMask)
+nsXTFElementWrapper::GetNotificationMask(uint32_t *aNotificationMask)
 {
   *aNotificationMask = mNotificationMask;
   return NS_OK;
 }
 NS_IMETHODIMP
-nsXTFElementWrapper::SetNotificationMask(PRUint32 aNotificationMask)
+nsXTFElementWrapper::SetNotificationMask(uint32_t aNotificationMask)
 {
   mNotificationMask = aNotificationMask;
   return NS_OK;
@@ -856,7 +842,7 @@ nsXTFElementWrapper::QueryInterfaceInner(REFNSIID aIID, void** result)
   if (aIID.Equals(NS_GET_IID(nsIXPConnectWrappedJS))) return false;
 
   GetXTFElement()->QueryInterface(aIID, result);
-  return (*result!=nsnull);
+  return (*result!=nullptr);
 }
 
 bool
@@ -926,7 +912,7 @@ nsXTFElementWrapper::GetClassAttributeName() const
 const nsAttrValue*
 nsXTFElementWrapper::DoGetClasses() const
 {
-  const nsAttrValue* val = nsnull;
+  const nsAttrValue* val = nullptr;
   nsIAtom* clazzAttr = GetClassAttributeName();
   if (clazzAttr) {
     val = mAttrsAndChildren.GetAttr(clazzAttr);
@@ -984,9 +970,9 @@ nsXTFElementWrapper::RegUnregAccessKey(bool aDoReg)
   accesskeyNode->GetValue(accessKey);
 
   if (aDoReg && !accessKey.IsEmpty())
-    esm->RegisterAccessKey(this, (PRUint32)accessKey.First());
+    esm->RegisterAccessKey(this, (uint32_t)accessKey.First());
   else
-    esm->UnregisterAccessKey(this, (PRUint32)accessKey.First());
+    esm->UnregisterAccessKey(this, (uint32_t)accessKey.First());
 }
 
 nsresult
@@ -994,7 +980,7 @@ NS_NewXTFElementWrapper(nsIXTFElement* aXTFElement,
                         already_AddRefed<nsINodeInfo> aNodeInfo,
                         nsIContent** aResult)
 {
-  *aResult = nsnull;
+  *aResult = nullptr;
    NS_ENSURE_ARG(aXTFElement);
 
   nsXTFElementWrapper* result = new nsXTFElementWrapper(aNodeInfo, aXTFElement);
@@ -1013,3 +999,8 @@ NS_NewXTFElementWrapper(nsIXTFElement* aXTFElement,
   *aResult = result;
   return NS_OK;
 }
+
+NS_IMPL_ISUPPORTS3(nsXTFClassInfo,
+                   nsIClassInfo,
+                   nsXPCClassInfo,
+                   nsIXPCScriptable)
