@@ -1,44 +1,7 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Chris Waterson <waterson@netscape.com>
- *   Peter Annema <disttsc@bart.nl>
- *   Mike Shaver <shaver@mozilla.org>
- *   Ben Goodger <ben@netscape.com>
- *   Mark Hammond <mhammond@skippinet.com.au>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /*
 
@@ -62,12 +25,10 @@
 #include "nsEventListenerManager.h"
 #include "nsIRDFCompositeDataSource.h"
 #include "nsIRDFResource.h"
-#include "nsIScriptObjectOwner.h"
 #include "nsBindingManager.h"
 #include "nsIURI.h"
 #include "nsIXULTemplateBuilder.h"
 #include "nsIBoxObject.h"
-#include "nsIXBLService.h"
 #include "nsLayoutCID.h"
 #include "nsAttrAndChildArray.h"
 #include "nsGkAtoms.h"
@@ -112,8 +73,7 @@ class nsXULPrototypeAttribute
 {
 public:
     nsXULPrototypeAttribute()
-        : mName(nsGkAtoms::id),  // XXX this is a hack, but names have to have a value
-          mEventHandler(nsnull)
+        : mName(nsGkAtoms::id)  // XXX this is a hack, but names have to have a value
     {
         XUL_PROTOTYPE_ATTRIBUTE_METER(gNumAttributes);
         MOZ_COUNT_CTOR(nsXULPrototypeAttribute);
@@ -123,57 +83,14 @@ public:
 
     nsAttrName mName;
     nsAttrValue mValue;
-    // mEventHandler is only valid for the language ID specified in the
-    // containing nsXULPrototypeElement.  We would ideally use
-    // nsScriptObjectHolder, but want to avoid the extra lang ID.
-    JSObject* mEventHandler;
 
 #ifdef XUL_PROTOTYPE_ATTRIBUTE_METERING
-    /**
-      If enough attributes, on average, are event handlers, it pays to keep
-      mEventHandler here, instead of maintaining a separate mapping in each
-      nsXULElement associating those mName values with their mEventHandlers.
-      Assume we don't need to keep mNameSpaceID along with mName in such an
-      event-handler-only name-to-function-pointer mapping.
-
-      Let
-        minAttrSize  = sizeof(mNodeInof) + sizeof(mValue)
-        mappingSize  = sizeof(mNodeInfo) + sizeof(mEventHandler)
-        elemOverhead = nElems * sizeof(MappingPtr)
-
-      Then
-        nAttrs * minAttrSize + nEventHandlers * mappingSize + elemOverhead
-        > nAttrs * (minAttrSize + mappingSize - sizeof(mNodeInfo))
-      which simplifies to
-        nEventHandlers * mappingSize + elemOverhead
-        > nAttrs * (mappingSize - sizeof(mNodeInfo))
-      or
-        nEventHandlers + (nElems * sizeof(MappingPtr)) / mappingSize
-        > nAttrs * (1 - sizeof(mNodeInfo) / mappingSize)
-
-      If nsCOMPtr and all other pointers are the same size, this reduces to
-        nEventHandlers + nElems / 2 > nAttrs / 2
-
-      To measure how many attributes are event handlers, compile XUL source
-      with XUL_PROTOTYPE_ATTRIBUTE_METERING and watch the counters below.
-      Plug into the above relation -- if true, it pays to put mEventHandler
-      in nsXULPrototypeAttribute rather than to keep a separate mapping.
-
-      Recent numbers after opening four browser windows:
-        nElems 3537, nAttrs 2528, nEventHandlers 1042
-      giving 1042 + 3537/2 > 2528/2 or 2810 > 1264.
-
-      As it happens, mEventHandler also makes this struct power-of-2 sized,
-      8 words on most architectures, which makes for strength-reduced array
-      index-to-pointer calculations.
-     */
-    static PRUint32   gNumElements;
-    static PRUint32   gNumAttributes;
-    static PRUint32   gNumEventHandlers;
-    static PRUint32   gNumCacheTests;
-    static PRUint32   gNumCacheHits;
-    static PRUint32   gNumCacheSets;
-    static PRUint32   gNumCacheFills;
+    static uint32_t   gNumElements;
+    static uint32_t   gNumAttributes;
+    static uint32_t   gNumCacheTests;
+    static uint32_t   gNumCacheHits;
+    static uint32_t   gNumCacheSets;
+    static uint32_t   gNumCacheFills;
 #endif /* !XUL_PROTOTYPE_ATTRIBUTE_METERING */
 };
 
@@ -181,21 +98,17 @@ public:
 /**
 
   A prototype content model element that holds the "primordial" values
-  that have been parsed from the original XUL document. A
-  'lightweight' nsXULElement may delegate its representation to this
-  structure, which is shared.
+  that have been parsed from the original XUL document.
 
  */
 
-class nsXULPrototypeNode : public nsISupports
+class nsXULPrototypeNode
 {
 public:
     enum Type { eType_Element, eType_Script, eType_Text, eType_PI };
 
     Type                     mType;
 
-    NS_DECL_CYCLE_COLLECTING_ISUPPORTS
-
     virtual ~nsXULPrototypeNode() {}
     virtual nsresult Serialize(nsIObjectOutputStream* aStream,
                                nsIScriptGlobalObject* aGlobal,
@@ -207,7 +120,7 @@ public:
 
 #ifdef NS_BUILD_REFCNT_LOGGING
     virtual const char* ClassName() = 0;
-    virtual PRUint32 ClassSize() = 0;
+    virtual uint32_t ClassSize() = 0;
 #endif
 
     /**
@@ -220,7 +133,8 @@ public:
      */
     virtual void ReleaseSubtree() { }
 
-    NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS(nsXULPrototypeNode)
+    NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_NATIVE_CLASS(nsXULPrototypeNode)
+    NS_INLINE_DECL_CYCLE_COLLECTING_NATIVE_REFCOUNTING(nsXULPrototypeNode)
 
 protected:
     nsXULPrototypeNode(Type aType)
@@ -233,11 +147,10 @@ public:
     nsXULPrototypeElement()
         : nsXULPrototypeNode(eType_Element),
           mNumAttributes(0),
-          mAttributes(nsnull),
           mHasIdAttribute(false),
           mHasClassAttribute(false),
           mHasStyleAttribute(false),
-          mHoldsScriptObject(false)
+          mAttributes(nullptr)
     {
     }
 
@@ -248,12 +161,12 @@ public:
 
 #ifdef NS_BUILD_REFCNT_LOGGING
     virtual const char* ClassName() { return "nsXULPrototypeElement"; }
-    virtual PRUint32 ClassSize() { return sizeof(*this); }
+    virtual uint32_t ClassSize() { return sizeof(*this); }
 #endif
 
     virtual void ReleaseSubtree()
     {
-        for (PRInt32 i = mChildren.Length() - 1; i >= 0; i--) {
+        for (int32_t i = mChildren.Length() - 1; i >= 0; i--) {
             if (mChildren[i].get())
                 mChildren[i]->ReleaseSubtree();
         }
@@ -269,7 +182,7 @@ public:
                                  nsIURI* aDocumentURI,
                                  const nsCOMArray<nsINodeInfo> *aNodeInfos);
 
-    nsresult SetAttrAt(PRUint32 aPos, const nsAString& aValue, nsIURI* aDocumentURI);
+    nsresult SetAttrAt(uint32_t aPos, const nsAString& aValue, nsIURI* aDocumentURI);
 
     void Unlink();
 
@@ -277,14 +190,11 @@ public:
 
     nsCOMPtr<nsINodeInfo>    mNodeInfo;           // [OWNER]
 
-    PRUint32                 mNumAttributes;
+    uint32_t                 mNumAttributes:29;
+    uint32_t                 mHasIdAttribute:1;
+    uint32_t                 mHasClassAttribute:1;
+    uint32_t                 mHasStyleAttribute:1;
     nsXULPrototypeAttribute* mAttributes;         // [OWNER]
-    
-    bool                     mHasIdAttribute:1;
-    bool                     mHasClassAttribute:1;
-    bool                     mHasStyleAttribute:1;
-    bool                     mHoldsScriptObject:1;
-
 };
 
 class nsXULDocument;
@@ -292,12 +202,12 @@ class nsXULDocument;
 class nsXULPrototypeScript : public nsXULPrototypeNode
 {
 public:
-    nsXULPrototypeScript(PRUint32 aLangID, PRUint32 aLineNo, PRUint32 version);
+    nsXULPrototypeScript(uint32_t aLineNo, uint32_t version);
     virtual ~nsXULPrototypeScript();
 
 #ifdef NS_BUILD_REFCNT_LOGGING
     virtual const char* ClassName() { return "nsXULPrototypeScript"; }
-    virtual PRUint32 ClassSize() { return sizeof(*this); }
+    virtual uint32_t ClassSize() { return sizeof(*this); }
 #endif
 
     virtual nsresult Serialize(nsIObjectOutputStream* aStream,
@@ -312,8 +222,8 @@ public:
     nsresult DeserializeOutOfLine(nsIObjectInputStream* aInput,
                                   nsIScriptGlobalObject* aGlobal);
 
-    nsresult Compile(const PRUnichar* aText, PRInt32 aTextLength,
-                     nsIURI* aURI, PRUint32 aLineNo,
+    nsresult Compile(const PRUnichar* aText, int32_t aTextLength,
+                     nsIURI* aURI, uint32_t aLineNo,
                      nsIDocument* aDocument,
                      nsIScriptGlobalObjectOwner* aGlobalOwner);
 
@@ -321,29 +231,24 @@ public:
 
     void Set(nsScriptObjectHolder<JSScript>& aHolder)
     {
-        NS_ASSERTION(mScriptObject.mLangID == aHolder.getScriptTypeID(),
-                     "Wrong language, this will leak the previous object.");
-
-        mScriptObject.mLangID = aHolder.getScriptTypeID();
         Set(aHolder.get());
     }
     void Set(JSScript* aObject);
 
     struct ScriptObjectHolder
     {
-        ScriptObjectHolder(PRUint32 aLangID) : mLangID(aLangID),
-                                               mObject(nsnull)
+        ScriptObjectHolder() : mObject(nullptr)
         {
         }
-        PRUint32 mLangID;
         JSScript* mObject;
     };
+
     nsCOMPtr<nsIURI>         mSrcURI;
-    PRUint32                 mLineNo;
+    uint32_t                 mLineNo;
     bool                     mSrcLoading;
     bool                     mOutOfLine;
     nsXULDocument*           mSrcLoadWaiters;   // [OWNER] but not COMPtr
-    PRUint32                 mLangVersion;
+    uint32_t                 mLangVersion;
     ScriptObjectHolder       mScriptObject;
 };
 
@@ -361,7 +266,7 @@ public:
 
 #ifdef NS_BUILD_REFCNT_LOGGING
     virtual const char* ClassName() { return "nsXULPrototypeText"; }
-    virtual PRUint32 ClassSize() { return sizeof(*this); }
+    virtual uint32_t ClassSize() { return sizeof(*this); }
 #endif
 
     virtual nsresult Serialize(nsIObjectOutputStream* aStream,
@@ -389,7 +294,7 @@ public:
 
 #ifdef NS_BUILD_REFCNT_LOGGING
     virtual const char* ClassName() { return "nsXULPrototypePI"; }
-    virtual PRUint32 ClassSize() { return sizeof(*this); }
+    virtual uint32_t ClassSize() { return sizeof(*this); }
 #endif
 
     virtual nsresult Serialize(nsIObjectOutputStream* aStream,
@@ -428,23 +333,9 @@ public:
     {
         if (aContent->IsXUL())
             return static_cast<nsXULElement*>(aContent);
-        return nsnull;
+        return nullptr;
     }
 
-public:
-    static nsIXBLService* GetXBLService() {
-        if (!gXBLService)
-            CallGetService("@mozilla.org/xbl;1", &gXBLService);
-        return gXBLService;
-    }
-    static void ReleaseGlobals() {
-        NS_IF_RELEASE(gXBLService);
-    }
-
-protected:
-    // pseudo-constants
-    static nsIXBLService*       gXBLService;
-
 public:
     nsXULElement(already_AddRefed<nsINodeInfo> aNodeInfo);
 
@@ -465,47 +356,27 @@ public:
                                 nsIContent* aBindingParent,
                                 bool aCompileEventHandlers);
     virtual void UnbindFromTree(bool aDeep, bool aNullParent);
-    virtual void RemoveChildAt(PRUint32 aIndex, bool aNotify);
-    virtual bool GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                           nsAString& aResult) const;
-    virtual bool HasAttr(PRInt32 aNameSpaceID, nsIAtom* aName) const;
-    virtual bool AttrValueIs(PRInt32 aNameSpaceID, nsIAtom* aName,
-                               const nsAString& aValue,
-                               nsCaseTreatment aCaseSensitive) const;
-    virtual bool AttrValueIs(PRInt32 aNameSpaceID, nsIAtom* aName,
-                               nsIAtom* aValue,
-                               nsCaseTreatment aCaseSensitive) const;
-    virtual PRInt32 FindAttrValueIn(PRInt32 aNameSpaceID,
-                                    nsIAtom* aName,
-                                    AttrValuesArray* aValues,
-                                    nsCaseTreatment aCaseSensitive) const;
-    virtual nsresult UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                               bool aNotify);
-    virtual const nsAttrName* GetAttrNameAt(PRUint32 aIndex) const;
-    virtual PRUint32 GetAttrCount() const;
+    virtual void RemoveChildAt(uint32_t aIndex, bool aNotify);
     virtual void DestroyContent();
 
 #ifdef DEBUG
-    virtual void List(FILE* out, PRInt32 aIndent) const;
-    virtual void DumpContent(FILE* out, PRInt32 aIndent,bool aDumpAll) const
+    virtual void List(FILE* out, int32_t aIndent) const;
+    virtual void DumpContent(FILE* out, int32_t aIndent,bool aDumpAll) const
     {
     }
 #endif
 
     virtual void PerformAccesskey(bool aKeyCausesActivation,
                                   bool aIsTrustedEvent);
-    nsresult ClickWithInputSource(PRUint16 aInputSource);
+    nsresult ClickWithInputSource(uint16_t aInputSource);
 
     virtual nsIContent *GetBindingParent() const;
-    virtual bool IsNodeOfType(PRUint32 aFlags) const;
-    virtual bool IsFocusable(PRInt32 *aTabIndex = nsnull, bool aWithMouse = false);
-    virtual nsIAtom* DoGetID() const;
-    virtual const nsAttrValue* DoGetClasses() const;
+    virtual bool IsNodeOfType(uint32_t aFlags) const;
+    virtual bool IsFocusable(int32_t *aTabIndex = nullptr, bool aWithMouse = false);
 
     NS_IMETHOD WalkContentStyleRules(nsRuleWalker* aRuleWalker);
-    virtual mozilla::css::StyleRule* GetInlineStyleRule();
     virtual nsChangeHint GetAttributeChangeHint(const nsIAtom* aAttribute,
-                                                PRInt32 aModType) const;
+                                                int32_t aModType) const;
     NS_IMETHOD_(bool) IsAttributeMapped(const nsIAtom* aAttribute) const;
 
     // XUL element methods
@@ -529,8 +400,6 @@ public:
     virtual nsresult Clone(nsINodeInfo *aNodeInfo, nsINode **aResult) const;
     virtual nsEventStates IntrinsicState() const;
 
-    nsresult EnsureLocalStyle();
-
     nsresult GetFrameLoader(nsIFrameLoader** aFrameLoader);
     already_AddRefed<nsFrameLoader> GetFrameLoader();
     nsresult SwapFrameLoaders(nsIFrameLoaderOwner* aOtherOwner);
@@ -545,18 +414,10 @@ public:
       mBindingParent = aBindingParent;
     }
 
-    /**
-     * Get the attr info for the given namespace ID and attribute name.
-     * The namespace ID must not be kNameSpaceID_Unknown and the name
-     * must not be null.
-     */
-    virtual nsAttrInfo GetAttrInfo(PRInt32 aNamespaceID, nsIAtom* aName) const;
-
     virtual nsXPCClassInfo* GetClassInfo();
+
+    virtual nsIDOMNode* AsDOMNode() { return this; }
 protected:
-    // XXX This can be removed when nsNodeUtils::CloneAndAdopt doesn't need
-    //     access to mPrototype anymore.
-    friend class nsNodeUtils;
 
     // This can be removed if EnsureContentsGenerated dies.
     friend class nsNSElementTearoff;
@@ -589,9 +450,6 @@ protected:
 
     nsresult LoadSrc();
 
-    // Required fields
-    nsRefPtr<nsXULPrototypeElement>     mPrototype;
-
     /**
      * The nearest enclosing content node with a binding
      * that created us. [Weak]
@@ -601,22 +459,17 @@ protected:
     /**
      * Abandon our prototype linkage, and copy all attributes locally
      */
-    nsresult MakeHeavyweight();
+    nsresult MakeHeavyweight(nsXULPrototypeElement* aPrototype);
 
-    const nsAttrValue* FindLocalOrProtoAttr(PRInt32 aNameSpaceID,
-                                            nsIAtom *aName) const {
-        return nsXULElement::GetAttrInfo(aNameSpaceID, aName).mValue;
-    }
-
-    virtual nsresult BeforeSetAttr(PRInt32 aNamespaceID, nsIAtom* aName,
+    virtual nsresult BeforeSetAttr(int32_t aNamespaceID, nsIAtom* aName,
                                    const nsAttrValueOrString* aValue,
                                    bool aNotify);
-    virtual nsresult AfterSetAttr(PRInt32 aNamespaceID, nsIAtom* aName,
+    virtual nsresult AfterSetAttr(int32_t aNamespaceID, nsIAtom* aName,
                                   const nsAttrValue* aValue, bool aNotify);
 
     virtual void UpdateEditableState(bool aNotify);
 
-    virtual bool ParseAttribute(PRInt32 aNamespaceID,
+    virtual bool ParseAttribute(int32_t aNamespaceID,
                                   nsIAtom* aAttribute,
                                   const nsAString& aValue,
                                   nsAttrValue& aResult);
@@ -624,11 +477,6 @@ protected:
     virtual nsEventListenerManager*
       GetEventListenerManagerForAttr(nsIAtom* aAttrName, bool* aDefer);
   
-    /**
-     * Return our prototype's attribute, if one exists.
-     */
-    nsXULPrototypeAttribute *FindPrototypeAttribute(PRInt32 aNameSpaceID,
-                                                    nsIAtom *aName) const;
     /**
      * Add a listener for the specified attribute, if appropriate.
      */
@@ -646,8 +494,6 @@ protected:
 
     void SetDrawsInTitlebar(bool aState);
 
-    const nsAttrName* InternalGetExistingAttrNameFromQName(const nsAString& aStr) const;
-
     void RemoveBroadcaster(const nsAString & broadcasterId);
 
 protected:
@@ -655,7 +501,7 @@ protected:
     // appropriate value.
     nsIControllers *Controllers() {
       nsDOMSlots* slots = GetExistingDOMSlots();
-      return slots ? slots->mControllers : nsnull; 
+      return slots ? slots->mControllers : nullptr; 
     }
 
     void UnregisterAccessKey(const nsAString& aOldValue);
@@ -670,8 +516,6 @@ protected:
     Create(nsXULPrototypeElement* aPrototype, nsINodeInfo *aNodeInfo,
            bool aIsScriptable);
 
-    friend class nsScriptEventHandlerOwnerTearoff;
-
     bool IsReadWriteTextElement() const
     {
         const nsIAtom* tag = Tag();
