@@ -1,45 +1,10 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Chris Waterson <waterson@netscape.com>
- *   Ben Goodger <ben@netscape.com>
- *   Jan Varga <varga@ku.sk>
- *   Neil Deakin <enndeakin@sympatico.ca>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nscore.h"
+#include "nsError.h"
 #include "nsIContent.h"
 #include "nsINodeInfo.h"
 #include "nsIDOMElement.h"
@@ -119,27 +84,27 @@ protected:
      * <treerow> in the rule's <action>.
      */
     nsresult
-    GetTemplateActionRowFor(PRInt32 aRow, nsIContent** aResult);
+    GetTemplateActionRowFor(int32_t aRow, nsIContent** aResult);
 
     /**
      * Given a row and a column ID, use the row's match to figure out
      * the appropriate <treecell> in the rule's <action>.
      */
     nsresult
-    GetTemplateActionCellFor(PRInt32 aRow, nsITreeColumn* aCol, nsIContent** aResult);
+    GetTemplateActionCellFor(int32_t aRow, nsITreeColumn* aCol, nsIContent** aResult);
 
     /**
      * Return the resource corresponding to a row in the tree.
      */
     nsresult
-    GetResourceFor(PRInt32 aRow, nsIRDFResource** aResource);
+    GetResourceFor(int32_t aRow, nsIRDFResource** aResource);
 
     /**
      * Open a container row, inserting the container's children into
      * the view.
      */
     nsresult
-    OpenContainer(PRInt32 aIndex, nsIXULTemplateResult* aResult);
+    OpenContainer(int32_t aIndex, nsIXULTemplateResult* aResult);
 
     /**
      * Helper for OpenContainer, recursively open subtrees, remembering
@@ -147,24 +112,24 @@ protected:
      */
     nsresult
     OpenSubtreeOf(nsTreeRows::Subtree* aSubtree,
-                  PRInt32 aIndex,
+                  int32_t aIndex,
                   nsIXULTemplateResult *aResult,
-                  PRInt32* aDelta);
+                  int32_t* aDelta);
 
     nsresult
     OpenSubtreeForQuerySet(nsTreeRows::Subtree* aSubtree,
-                           PRInt32 aIndex,
+                           int32_t aIndex,
                            nsIXULTemplateResult *aResult,
                            nsTemplateQuerySet* aQuerySet,
-                           PRInt32* aDelta,
-                           nsTArray<PRInt32>& open);
+                           int32_t* aDelta,
+                           nsTArray<int32_t>& open);
 
     /**
      * Close a container row, removing the container's childrem from
      * the view.
      */
     nsresult
-    CloseContainer(PRInt32 aIndex);
+    CloseContainer(int32_t aIndex);
 
     /**
      * Remove the matches for the rows in a subtree
@@ -190,7 +155,7 @@ protected:
     /**
      * The real sort routine
      */
-    PRInt32
+    int32_t
     CompareResults(nsIXULTemplateResult* aLeft, nsIXULTemplateResult* aRight);
 
     /**
@@ -270,7 +235,7 @@ protected:
     /*
      * Sort hints (compare case, etc)
      */
-    PRUint32 mSortHints;
+    uint32_t mSortHints;
 
     /** 
      * The builder observers.
@@ -283,9 +248,9 @@ protected:
 nsresult
 NS_NewXULTreeBuilder(nsISupports* aOuter, REFNSIID aIID, void** aResult)
 {
-    *aResult = nsnull;
+    *aResult = nullptr;
 
-    NS_PRECONDITION(aOuter == nsnull, "no aggregation");
+    NS_PRECONDITION(aOuter == nullptr, "no aggregation");
     if (aOuter)
         return NS_ERROR_NO_AGGREGATION;
 
@@ -352,7 +317,7 @@ nsXULTreeBuilder::nsXULTreeBuilder()
 void
 nsXULTreeBuilder::Uninit(bool aIsFinal)
 {
-    PRInt32 count = mRows.Count();
+    int32_t count = mRows.Count();
     mRows.Clear();
 
     if (mBoxObject) {
@@ -373,7 +338,7 @@ nsXULTreeBuilder::Uninit(bool aIsFinal)
 //
 
 NS_IMETHODIMP
-nsXULTreeBuilder::GetResourceAtIndex(PRInt32 aRowIndex, nsIRDFResource** aResult)
+nsXULTreeBuilder::GetResourceAtIndex(int32_t aRowIndex, nsIRDFResource** aResult)
 {
     if (aRowIndex < 0 || aRowIndex >= mRows.Count())
         return NS_ERROR_INVALID_ARG;
@@ -382,7 +347,7 @@ nsXULTreeBuilder::GetResourceAtIndex(PRInt32 aRowIndex, nsIRDFResource** aResult
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::GetIndexOfResource(nsIRDFResource* aResource, PRInt32* aResult)
+nsXULTreeBuilder::GetIndexOfResource(nsIRDFResource* aResource, int32_t* aResult)
 {
     NS_ENSURE_ARG_POINTER(aResource);
     nsTreeRows::iterator iter = mRows.FindByResource(aResource);
@@ -481,7 +446,7 @@ nsXULTreeBuilder::Sort(nsIDOMElement* aElement)
 //
 
 NS_IMETHODIMP
-nsXULTreeBuilder::GetRowCount(PRInt32* aRowCount)
+nsXULTreeBuilder::GetRowCount(int32_t* aRowCount)
 {
     *aRowCount = mRows.Count();
     return NS_OK;
@@ -505,7 +470,7 @@ nsXULTreeBuilder::SetSelection(nsITreeSelection* aSelection)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::GetRowProperties(PRInt32 aIndex, nsISupportsArray* aProperties)
+nsXULTreeBuilder::GetRowProperties(int32_t aIndex, nsISupportsArray* aProperties)
 {
     NS_ENSURE_ARG_POINTER(aProperties);
     NS_PRECONDITION(aIndex >= 0 && aIndex < mRows.Count(), "bad row");
@@ -530,7 +495,7 @@ nsXULTreeBuilder::GetRowProperties(PRInt32 aIndex, nsISupportsArray* aProperties
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::GetCellProperties(PRInt32 aRow, nsITreeColumn* aCol, nsISupportsArray* aProperties)
+nsXULTreeBuilder::GetCellProperties(int32_t aRow, nsITreeColumn* aCol, nsISupportsArray* aProperties)
 {
     NS_ENSURE_ARG_POINTER(aCol);
     NS_ENSURE_ARG_POINTER(aProperties);
@@ -566,7 +531,7 @@ nsXULTreeBuilder::GetColumnProperties(nsITreeColumn* aCol,
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::IsContainer(PRInt32 aIndex, bool* aResult)
+nsXULTreeBuilder::IsContainer(int32_t aIndex, bool* aResult)
 {
     NS_PRECONDITION(aIndex >= 0 && aIndex < mRows.Count(), "bad row");
     if (aIndex < 0 || aIndex >= mRows.Count())
@@ -586,7 +551,7 @@ nsXULTreeBuilder::IsContainer(PRInt32 aIndex, bool* aResult)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::IsContainerOpen(PRInt32 aIndex, bool* aOpen)
+nsXULTreeBuilder::IsContainerOpen(int32_t aIndex, bool* aOpen)
 {
     NS_PRECONDITION(aIndex >= 0 && aIndex < mRows.Count(), "bad row");
     if (aIndex < 0 || aIndex >= mRows.Count())
@@ -608,7 +573,7 @@ nsXULTreeBuilder::IsContainerOpen(PRInt32 aIndex, bool* aOpen)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::IsContainerEmpty(PRInt32 aIndex, bool* aResult)
+nsXULTreeBuilder::IsContainerEmpty(int32_t aIndex, bool* aResult)
 {
     NS_PRECONDITION(aIndex >= 0 && aIndex < mRows.Count(), "bad row");
     if (aIndex < 0 || aIndex >= mRows.Count())
@@ -640,7 +605,7 @@ nsXULTreeBuilder::IsContainerEmpty(PRInt32 aIndex, bool* aResult)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::IsSeparator(PRInt32 aIndex, bool* aResult)
+nsXULTreeBuilder::IsSeparator(int32_t aIndex, bool* aResult)
 {
     NS_PRECONDITION(aIndex >= 0 && aIndex < mRows.Count(), "bad row");
     if (aIndex < 0 || aIndex >= mRows.Count())
@@ -656,7 +621,7 @@ nsXULTreeBuilder::IsSeparator(PRInt32 aIndex, bool* aResult)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::GetParentIndex(PRInt32 aRowIndex, PRInt32* aResult)
+nsXULTreeBuilder::GetParentIndex(int32_t aRowIndex, int32_t* aResult)
 {
     NS_PRECONDITION(aRowIndex >= 0 && aRowIndex < mRows.Count(), "bad row");
     if (aRowIndex < 0 || aRowIndex >= mRows.Count())
@@ -670,7 +635,7 @@ nsXULTreeBuilder::GetParentIndex(PRInt32 aRowIndex, PRInt32* aResult)
 
     // Now walk through our previous siblings, subtracting off each
     // one's subtree size
-    PRInt32 index = iter.GetChildIndex();
+    int32_t index = iter.GetChildIndex();
     while (--index >= 0)
         aRowIndex -= mRows.GetSubtreeSizeFor(parent, index) + 1;
 
@@ -680,7 +645,7 @@ nsXULTreeBuilder::GetParentIndex(PRInt32 aRowIndex, PRInt32* aResult)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::HasNextSibling(PRInt32 aRowIndex, PRInt32 aAfterIndex, bool* aResult)
+nsXULTreeBuilder::HasNextSibling(int32_t aRowIndex, int32_t aAfterIndex, bool* aResult)
 {
     NS_PRECONDITION(aRowIndex >= 0 && aRowIndex < mRows.Count(), "bad row");
     if (aRowIndex < 0 || aRowIndex >= mRows.Count())
@@ -699,7 +664,7 @@ nsXULTreeBuilder::HasNextSibling(PRInt32 aRowIndex, PRInt32 aAfterIndex, bool* a
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::GetLevel(PRInt32 aRowIndex, PRInt32* aResult)
+nsXULTreeBuilder::GetLevel(int32_t aRowIndex, int32_t* aResult)
 {
     NS_PRECONDITION(aRowIndex >= 0 && aRowIndex < mRows.Count(), "bad row");
     if (aRowIndex < 0 || aRowIndex >= mRows.Count())
@@ -713,7 +678,7 @@ nsXULTreeBuilder::GetLevel(PRInt32 aRowIndex, PRInt32* aResult)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::GetImageSrc(PRInt32 aRow, nsITreeColumn* aCol, nsAString& aResult)
+nsXULTreeBuilder::GetImageSrc(int32_t aRow, nsITreeColumn* aCol, nsAString& aResult)
 {
     NS_ENSURE_ARG_POINTER(aCol);
     NS_PRECONDITION(aRow >= 0 && aRow < mRows.Count(), "bad index");
@@ -737,7 +702,7 @@ nsXULTreeBuilder::GetImageSrc(PRInt32 aRow, nsITreeColumn* aCol, nsAString& aRes
 
 
 NS_IMETHODIMP
-nsXULTreeBuilder::GetProgressMode(PRInt32 aRow, nsITreeColumn* aCol, PRInt32* aResult)
+nsXULTreeBuilder::GetProgressMode(int32_t aRow, nsITreeColumn* aCol, int32_t* aResult)
 {
     NS_ENSURE_ARG_POINTER(aCol);
     NS_PRECONDITION(aRow >= 0 && aRow < mRows.Count(), "bad index");
@@ -766,7 +731,7 @@ nsXULTreeBuilder::GetProgressMode(PRInt32 aRow, nsITreeColumn* aCol, PRInt32* aR
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::GetCellValue(PRInt32 aRow, nsITreeColumn* aCol, nsAString& aResult)
+nsXULTreeBuilder::GetCellValue(int32_t aRow, nsITreeColumn* aCol, nsAString& aResult)
 {
     NS_ENSURE_ARG_POINTER(aCol);
     NS_PRECONDITION(aRow >= 0 && aRow < mRows.Count(), "bad index");
@@ -789,7 +754,7 @@ nsXULTreeBuilder::GetCellValue(PRInt32 aRow, nsITreeColumn* aCol, nsAString& aRe
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::GetCellText(PRInt32 aRow, nsITreeColumn* aCol, nsAString& aResult)
+nsXULTreeBuilder::GetCellText(int32_t aRow, nsITreeColumn* aCol, nsAString& aResult)
 {
     NS_ENSURE_ARG_POINTER(aCol);
     NS_PRECONDITION(aRow >= 0 && aRow < mRows.Count(), "bad index");
@@ -870,7 +835,7 @@ nsXULTreeBuilder::SetTree(nsITreeBoxObject* aTree)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::ToggleOpenState(PRInt32 aIndex)
+nsXULTreeBuilder::ToggleOpenState(int32_t aIndex)
 {
     if (aIndex < 0 || aIndex >= mRows.Count())
         return NS_ERROR_INVALID_ARG;
@@ -891,9 +856,9 @@ nsXULTreeBuilder::ToggleOpenState(PRInt32 aIndex)
     }
 
     if (mObservers) {
-        PRUint32 count;
+        uint32_t count;
         mObservers->Count(&count);
-        for (PRUint32 i = 0; i < count; ++i) {
+        for (uint32_t i = 0; i < count; ++i) {
             nsCOMPtr<nsIXULTreeBuilderObserver> observer = do_QueryElementAt(mObservers, i);
             if (observer)
                 observer->OnToggleOpenState(aIndex);
@@ -947,9 +912,9 @@ nsXULTreeBuilder::CycleHeader(nsITreeColumn* aCol)
         nsAutoString id;
         aCol->GetId(id);
 
-        PRUint32 count;
+        uint32_t count;
         mObservers->Count(&count);
-        for (PRUint32 i = 0; i < count; ++i) {
+        for (uint32_t i = 0; i < count; ++i) {
             nsCOMPtr<nsIXULTreeBuilderObserver> observer = do_QueryElementAt(mObservers, i);
             if (observer)
                 observer->OnCycleHeader(id.get(), element);
@@ -963,9 +928,9 @@ NS_IMETHODIMP
 nsXULTreeBuilder::SelectionChanged()
 {
     if (mObservers) {
-        PRUint32 count;
+        uint32_t count;
         mObservers->Count(&count);
-        for (PRUint32 i = 0; i < count; ++i) {
+        for (uint32_t i = 0; i < count; ++i) {
             nsCOMPtr<nsIXULTreeBuilderObserver> observer = do_QueryElementAt(mObservers, i);
             if (observer)
                 observer->OnSelectionChanged();
@@ -976,16 +941,16 @@ nsXULTreeBuilder::SelectionChanged()
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::CycleCell(PRInt32 aRow, nsITreeColumn* aCol)
+nsXULTreeBuilder::CycleCell(int32_t aRow, nsITreeColumn* aCol)
 {
     NS_ENSURE_ARG_POINTER(aCol);
     if (mObservers) {
         nsAutoString id;
         aCol->GetId(id);
 
-        PRUint32 count;
+        uint32_t count;
         mObservers->Count(&count);
-        for (PRUint32 i = 0; i < count; ++i) {
+        for (uint32_t i = 0; i < count; ++i) {
             nsCOMPtr<nsIXULTreeBuilderObserver> observer = do_QueryElementAt(mObservers, i);
             if (observer)
                 observer->OnCycleCell(aRow, id.get());
@@ -996,7 +961,7 @@ nsXULTreeBuilder::CycleCell(PRInt32 aRow, nsITreeColumn* aCol)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::IsEditable(PRInt32 aRow, nsITreeColumn* aCol, bool* _retval)
+nsXULTreeBuilder::IsEditable(int32_t aRow, nsITreeColumn* aCol, bool* _retval)
 {
     *_retval = true;
     NS_ENSURE_ARG_POINTER(aCol);
@@ -1022,7 +987,7 @@ nsXULTreeBuilder::IsEditable(PRInt32 aRow, nsITreeColumn* aCol, bool* _retval)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::IsSelectable(PRInt32 aRow, nsITreeColumn* aCol, bool* _retval)
+nsXULTreeBuilder::IsSelectable(int32_t aRow, nsITreeColumn* aCol, bool* _retval)
 {
     NS_PRECONDITION(aRow >= 0 && aRow < mRows.Count(), "bad index");
     if (aRow < 0 || aRow >= mRows.Count())
@@ -1048,14 +1013,14 @@ nsXULTreeBuilder::IsSelectable(PRInt32 aRow, nsITreeColumn* aCol, bool* _retval)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::SetCellValue(PRInt32 aRow, nsITreeColumn* aCol, const nsAString& aValue)
+nsXULTreeBuilder::SetCellValue(int32_t aRow, nsITreeColumn* aCol, const nsAString& aValue)
 {
     NS_ENSURE_ARG_POINTER(aCol);
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::SetCellText(PRInt32 aRow, nsITreeColumn* aCol, const nsAString& aValue)
+nsXULTreeBuilder::SetCellText(int32_t aRow, nsITreeColumn* aCol, const nsAString& aValue)
 {
     NS_ENSURE_ARG_POINTER(aCol);
     return NS_OK;
@@ -1065,9 +1030,9 @@ NS_IMETHODIMP
 nsXULTreeBuilder::PerformAction(const PRUnichar* aAction)
 {
     if (mObservers) {  
-        PRUint32 count;
+        uint32_t count;
         mObservers->Count(&count);
-        for (PRUint32 i = 0; i < count; ++i) {
+        for (uint32_t i = 0; i < count; ++i) {
             nsCOMPtr<nsIXULTreeBuilderObserver> observer = do_QueryElementAt(mObservers, i);
             if (observer)
                 observer->OnPerformAction(aAction);
@@ -1078,12 +1043,12 @@ nsXULTreeBuilder::PerformAction(const PRUnichar* aAction)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::PerformActionOnRow(const PRUnichar* aAction, PRInt32 aRow)
+nsXULTreeBuilder::PerformActionOnRow(const PRUnichar* aAction, int32_t aRow)
 {
     if (mObservers) {  
-        PRUint32 count;
+        uint32_t count;
         mObservers->Count(&count);
-        for (PRUint32 i = 0; i < count; ++i) {
+        for (uint32_t i = 0; i < count; ++i) {
             nsCOMPtr<nsIXULTreeBuilderObserver> observer = do_QueryElementAt(mObservers, i);
             if (observer)
                 observer->OnPerformActionOnRow(aAction, aRow);
@@ -1094,16 +1059,16 @@ nsXULTreeBuilder::PerformActionOnRow(const PRUnichar* aAction, PRInt32 aRow)
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::PerformActionOnCell(const PRUnichar* aAction, PRInt32 aRow, nsITreeColumn* aCol)
+nsXULTreeBuilder::PerformActionOnCell(const PRUnichar* aAction, int32_t aRow, nsITreeColumn* aCol)
 {
     NS_ENSURE_ARG_POINTER(aCol);
     if (mObservers) {  
         nsAutoString id;
         aCol->GetId(id);
 
-        PRUint32 count;
+        uint32_t count;
         mObservers->Count(&count);
-        for (PRUint32 i = 0; i < count; ++i) {
+        for (uint32_t i = 0; i < count; ++i) {
             nsCOMPtr<nsIXULTreeBuilderObserver> observer = do_QueryElementAt(mObservers, i);
             if (observer)
                 observer->OnPerformActionOnCell(aAction, aRow, id.get());
@@ -1151,7 +1116,7 @@ bool
 nsXULTreeBuilder::GetInsertionLocations(nsIXULTemplateResult* aResult,
                                         nsCOMArray<nsIContent>** aLocations)
 {
-    *aLocations = nsnull;
+    *aLocations = nullptr;
 
     // Get the reference point and check if it is an open container. Rows
     // should not be generated otherwise.
@@ -1195,12 +1160,12 @@ nsXULTreeBuilder::ReplaceMatch(nsIXULTemplateResult* aOldResult,
             return NS_ERROR_FAILURE;
 
         // Remove the rows from the view
-        PRInt32 row = iter.GetRowIndex();
+        int32_t row = iter.GetRowIndex();
 
         // If the row contains children, remove the matches from the
         // children so that they can be regenerated again if the element
         // gets added back.
-        PRInt32 delta = mRows.GetSubtreeSizeFor(iter);
+        int32_t delta = mRows.GetSubtreeSizeFor(iter);
         if (delta)
             RemoveMatchesFor(*(iter->mSubtree));
 
@@ -1226,8 +1191,8 @@ nsXULTreeBuilder::ReplaceMatch(nsIXULTemplateResult* aOldResult,
 
     if (aNewMatch && aNewMatch->mResult) {
         // Insertion.
-        PRInt32 row = -1;
-        nsTreeRows::Subtree* parent = nsnull;
+        int32_t row = -1;
+        nsTreeRows::Subtree* parent = nullptr;
         nsIXULTemplateResult* result = aNewMatch->mResult;
 
         nsAutoString ref;
@@ -1275,17 +1240,17 @@ nsXULTreeBuilder::ReplaceMatch(nsIXULTemplateResult* aOldResult,
             // If we get here, then we're inserting into an open
             // container. By default, place the new element at the
             // end of the container
-            PRInt32 index = parent->Count();
+            int32_t index = parent->Count();
 
             if (mSortVariable) {
                 // Figure out where to put the new element by doing an
                 // insertion sort.
-                PRInt32 left = 0;
-                PRInt32 right = index;
+                int32_t left = 0;
+                int32_t right = index;
 
                 while (left < right) {
                     index = (left + right) / 2;
-                    PRInt32 cmp = CompareResults((*parent)[index].mMatch->mResult, result);
+                    int32_t cmp = CompareResults((*parent)[index].mMatch->mResult, result);
                     if (cmp < 0)
                         left = ++index;
                     else if (cmp > 0)
@@ -1336,7 +1301,7 @@ nsXULTreeBuilder::SynchronizeResult(nsIXULTemplateResult* aResult)
         if (iter == mRows.Last())
             return NS_ERROR_FAILURE;
 
-        PRInt32 row = iter.GetRowIndex();
+        int32_t row = iter.GetRowIndex();
         if (row >= 0)
             mBoxObject->InvalidateRow(row);
 
@@ -1377,7 +1342,7 @@ nsXULTreeBuilder::EnsureSortVariables()
                     mSortVariable = do_GetAtom(sort);
 
                     static nsIContent::AttrValuesArray strings[] =
-                      {&nsGkAtoms::ascending, &nsGkAtoms::descending, nsnull};
+                      {&nsGkAtoms::ascending, &nsGkAtoms::descending, nullptr};
                     switch (child->FindAttrValueIn(kNameSpaceID_None,
                                                    nsGkAtoms::sortDirection,
                                                    strings, eCaseMatters)) {
@@ -1416,7 +1381,7 @@ nsXULTreeBuilder::RebuildAll()
         Uninit(false);
     }
     else if (mBoxObject) {
-        PRInt32 count = mRows.Count();
+        int32_t count = mRows.Count();
         mRows.Clear();
         mBoxObject->RowCountChanged(0, -count);
     }
@@ -1448,7 +1413,7 @@ nsXULTreeBuilder::RebuildAll()
 }
 
 nsresult
-nsXULTreeBuilder::GetTemplateActionRowFor(PRInt32 aRow, nsIContent** aResult)
+nsXULTreeBuilder::GetTemplateActionRowFor(int32_t aRow, nsIContent** aResult)
 {
     // Get the template in the DOM from which we're supposed to
     // generate text
@@ -1457,7 +1422,7 @@ nsXULTreeBuilder::GetTemplateActionRowFor(PRInt32 aRow, nsIContent** aResult)
     // The match stores the indices of the rule and query to use. Use these
     // to look up the right nsTemplateRule and use that rule's action to get
     // the treerow in the template.
-    PRInt16 ruleindex = row.mMatch->RuleIndex();
+    int16_t ruleindex = row.mMatch->RuleIndex();
     if (ruleindex >= 0) {
         nsTemplateQuerySet* qs = mQuerySets[row.mMatch->QuerySetPriority()];
         nsTemplateRule* rule = qs->GetRuleAt(ruleindex);
@@ -1480,16 +1445,16 @@ nsXULTreeBuilder::GetTemplateActionRowFor(PRInt32 aRow, nsIContent** aResult)
         }
     }
 
-    *aResult = nsnull;
+    *aResult = nullptr;
     return NS_OK;
 }
 
 nsresult
-nsXULTreeBuilder::GetTemplateActionCellFor(PRInt32 aRow,
+nsXULTreeBuilder::GetTemplateActionCellFor(int32_t aRow,
                                            nsITreeColumn* aCol,
                                            nsIContent** aResult)
 {
-    *aResult = nsnull;
+    *aResult = nullptr;
 
     if (!aCol) return NS_ERROR_INVALID_ARG;
 
@@ -1497,11 +1462,11 @@ nsXULTreeBuilder::GetTemplateActionCellFor(PRInt32 aRow,
     GetTemplateActionRowFor(aRow, getter_AddRefs(row));
     if (row) {
         nsCOMPtr<nsIAtom> colAtom;
-        PRInt32 colIndex;
+        int32_t colIndex;
         aCol->GetAtom(getter_AddRefs(colAtom));
         aCol->GetIndex(&colIndex);
 
-        PRUint32 j = 0;
+        uint32_t j = 0;
         for (nsIContent* child = row->GetFirstChild();
              child;
              child = child->GetNextSibling()) {
@@ -1514,7 +1479,7 @@ nsXULTreeBuilder::GetTemplateActionCellFor(PRInt32 aRow,
                     *aResult = child;
                     break;
                 }
-                else if (j == (PRUint32)colIndex)
+                else if (j == (uint32_t)colIndex)
                     *aResult = child;
                 j++;
             }
@@ -1526,14 +1491,14 @@ nsXULTreeBuilder::GetTemplateActionCellFor(PRInt32 aRow,
 }
 
 nsresult
-nsXULTreeBuilder::GetResourceFor(PRInt32 aRow, nsIRDFResource** aResource)
+nsXULTreeBuilder::GetResourceFor(int32_t aRow, nsIRDFResource** aResource)
 {
     nsTreeRows::Row& row = *(mRows[aRow]);
     return GetResultResource(row.mMatch->mResult, aResource);
 }
 
 nsresult
-nsXULTreeBuilder::OpenContainer(PRInt32 aIndex, nsIXULTemplateResult* aResult)
+nsXULTreeBuilder::OpenContainer(int32_t aIndex, nsIXULTemplateResult* aResult)
 {
     // A row index of -1 in this case means ``open tree body''
     NS_ASSERTION(aIndex >= -1 && aIndex < mRows.Count(), "bad row");
@@ -1555,7 +1520,7 @@ nsXULTreeBuilder::OpenContainer(PRInt32 aIndex, nsIXULTemplateResult* aResult)
     if (! container)
         return NS_ERROR_OUT_OF_MEMORY;
 
-    PRInt32 count;
+    int32_t count;
     OpenSubtreeOf(container, aIndex, aResult, &count);
 
     // Notify the box object
@@ -1572,31 +1537,31 @@ nsXULTreeBuilder::OpenContainer(PRInt32 aIndex, nsIXULTemplateResult* aResult)
 
 nsresult
 nsXULTreeBuilder::OpenSubtreeOf(nsTreeRows::Subtree* aSubtree,
-                                PRInt32 aIndex,
+                                int32_t aIndex,
                                 nsIXULTemplateResult *aResult,
-                                PRInt32* aDelta)
+                                int32_t* aDelta)
 {
-    nsAutoTArray<PRInt32, 8> open;
-    PRInt32 count = 0;
+    nsAutoTArray<int32_t, 8> open;
+    int32_t count = 0;
 
-    PRInt32 rulecount = mQuerySets.Length();
+    int32_t rulecount = mQuerySets.Length();
 
-    for (PRInt32 r = 0; r < rulecount; r++) {
+    for (int32_t r = 0; r < rulecount; r++) {
         nsTemplateQuerySet* queryset = mQuerySets[r];
         OpenSubtreeForQuerySet(aSubtree, aIndex, aResult, queryset, &count, open);
     }
 
     // Now recursively deal with any open sub-containers that just got
     // inserted. We need to do this back-to-front to avoid skewing offsets.
-    for (PRInt32 i = open.Length() - 1; i >= 0; --i) {
-        PRInt32 index = open[i];
+    for (int32_t i = open.Length() - 1; i >= 0; --i) {
+        int32_t index = open[i];
 
         nsTreeRows::Subtree* child =
             mRows.EnsureSubtreeFor(aSubtree, index);
 
         nsIXULTemplateResult* result = (*aSubtree)[index].mMatch->mResult;
 
-        PRInt32 delta;
+        int32_t delta;
         OpenSubtreeOf(child, aIndex + index, result, &delta);
         count += delta;
     }
@@ -1616,13 +1581,13 @@ nsXULTreeBuilder::OpenSubtreeOf(nsTreeRows::Subtree* aSubtree,
 
 nsresult
 nsXULTreeBuilder::OpenSubtreeForQuerySet(nsTreeRows::Subtree* aSubtree,
-                                         PRInt32 aIndex,
+                                         int32_t aIndex,
                                          nsIXULTemplateResult* aResult,
                                          nsTemplateQuerySet* aQuerySet,
-                                         PRInt32* aDelta,
-                                         nsTArray<PRInt32>& open)
+                                         int32_t* aDelta,
+                                         nsTArray<int32_t>& open)
 {
-    PRInt32 count = *aDelta;
+    int32_t count = *aDelta;
     
     nsCOMPtr<nsISimpleEnumerator> results;
     nsresult rv = mQueryProcessor->GenerateResults(mDataSource, aResult,
@@ -1659,8 +1624,8 @@ nsXULTreeBuilder::OpenSubtreeForQuerySet(nsTreeRows::Subtree* aSubtree,
 
         bool generateContent = true;
 
-        nsTemplateMatch* prevmatch = nsnull;
-        nsTemplateMatch* existingmatch = nsnull;
+        nsTemplateMatch* prevmatch = nullptr;
+        nsTemplateMatch* existingmatch = nullptr;
         if (mMatchMap.Get(resultid, &existingmatch)){
             // check if there is an existing match that matched a rule
             while (existingmatch) {
@@ -1673,7 +1638,7 @@ nsXULTreeBuilder::OpenSubtreeForQuerySet(nsTreeRows::Subtree* aSubtree,
 
         nsTemplateMatch *newmatch =
             nsTemplateMatch::Create(mPool, aQuerySet->Priority(),
-                                    nextresult, nsnull);
+                                    nextresult, nullptr);
         if (!newmatch)
             return NS_ERROR_OUT_OF_MEMORY;
 
@@ -1703,9 +1668,9 @@ nsXULTreeBuilder::OpenSubtreeForQuerySet(nsTreeRows::Subtree* aSubtree,
                 continue;
             }
 
-            PRInt16 ruleindex;
-            nsTemplateRule* matchedrule = nsnull;
-            rv = DetermineMatchedRule(nsnull, nextresult, aQuerySet,
+            int16_t ruleindex;
+            nsTemplateRule* matchedrule = nullptr;
+            rv = DetermineMatchedRule(nullptr, nextresult, aQuerySet,
                                       &matchedrule, &ruleindex);
             if (NS_FAILED(rv)) {
                 nsTemplateMatch::Destroy(mPool, newmatch, false);
@@ -1728,7 +1693,7 @@ nsXULTreeBuilder::OpenSubtreeForQuerySet(nsTreeRows::Subtree* aSubtree,
                 bool isOpen = false;
                 IsContainerOpen(nextresult, &isOpen);
                 if (isOpen) {
-                    if (open.AppendElement(count) == nsnull)
+                    if (open.AppendElement(count) == nullptr)
                         return NS_ERROR_OUT_OF_MEMORY;
                 }
 
@@ -1743,9 +1708,8 @@ nsXULTreeBuilder::OpenSubtreeForQuerySet(nsTreeRows::Subtree* aSubtree,
         if (prevmatch) {
             prevmatch->mNext = newmatch;
         }
-        else if (!mMatchMap.Put(resultid, newmatch)) {
-            nsTemplateMatch::Destroy(mPool, newmatch, true);
-            return NS_ERROR_OUT_OF_MEMORY;
+        else {
+            mMatchMap.Put(resultid, newmatch);
         }
     }
 
@@ -1754,7 +1718,7 @@ nsXULTreeBuilder::OpenSubtreeForQuerySet(nsTreeRows::Subtree* aSubtree,
 }
 
 nsresult
-nsXULTreeBuilder::CloseContainer(PRInt32 aIndex)
+nsXULTreeBuilder::CloseContainer(int32_t aIndex)
 {
     NS_ASSERTION(aIndex >= 0 && aIndex < mRows.Count(), "bad row");
     if (aIndex < 0 || aIndex >= mRows.Count())
@@ -1766,7 +1730,7 @@ nsXULTreeBuilder::CloseContainer(PRInt32 aIndex)
         RemoveMatchesFor(*iter->mSubtree);
 
 
-    PRInt32 count = mRows.GetSubtreeSizeFor(iter);
+    int32_t count = mRows.GetSubtreeSizeFor(iter);
     mRows.RemoveSubtreeFor(iter);
 
     iter->mContainerState = nsTreeRows::eContainerState_Closed;
@@ -1784,7 +1748,7 @@ nsXULTreeBuilder::CloseContainer(PRInt32 aIndex)
 nsresult
 nsXULTreeBuilder::RemoveMatchesFor(nsTreeRows::Subtree& subtree)
 {
-    for (PRInt32 i = subtree.Count() - 1; i >= 0; --i) {
+    for (int32_t i = subtree.Count() - 1; i >= 0; --i) {
         nsTreeRows::Row& row = subtree[i];
 
         nsTemplateMatch* match = row.mMatch;
@@ -1858,7 +1822,7 @@ nsXULTreeBuilder::Compare(const void* aLeft, const void* aRight, void* aClosure)
     return self->CompareResults(left->mMatch->mResult, right->mMatch->mResult);
 }
 
-PRInt32
+int32_t
 nsXULTreeBuilder::CompareResults(nsIXULTemplateResult* aLeft, nsIXULTemplateResult* aRight)
 {
     // this is an extra check done for RDF queries such that results appear in
@@ -1895,7 +1859,7 @@ nsXULTreeBuilder::CompareResults(nsIXULTemplateResult* aLeft, nsIXULTemplateResu
             if (isSequence) {
                 // Determine the indices of the left and right elements
                 // in the container.
-                PRInt32 lindex = 0, rindex = 0;
+                int32_t lindex = 0, rindex = 0;
 
                 nsCOMPtr<nsIRDFResource> leftitem;
                 aLeft->GetResource(getter_AddRefs(leftitem));
@@ -1918,7 +1882,7 @@ nsXULTreeBuilder::CompareResults(nsIXULTemplateResult* aLeft, nsIXULTemplateResu
         }
     }
 
-    PRInt32 sortorder;
+    int32_t sortorder;
     if (!mQueryProcessor)
         return 0;
 
@@ -1938,7 +1902,7 @@ nsXULTreeBuilder::SortSubtree(nsTreeRows::Subtree* aSubtree)
                  Compare,
                  this);
 
-    for (PRInt32 i = aSubtree->Count() - 1; i >= 0; --i) {
+    for (int32_t i = aSubtree->Count() - 1; i >= 0; --i) {
         nsTreeRows::Subtree* child = (*aSubtree)[i].mSubtree;
         if (child)
             SortSubtree(child);
@@ -1950,14 +1914,14 @@ nsXULTreeBuilder::SortSubtree(nsTreeRows::Subtree* aSubtree)
 
 /* boolean canDrop (in long index, in long orientation); */
 NS_IMETHODIMP
-nsXULTreeBuilder::CanDrop(PRInt32 index, PRInt32 orientation,
+nsXULTreeBuilder::CanDrop(int32_t index, int32_t orientation,
                           nsIDOMDataTransfer* dataTransfer, bool *_retval)
 {
     *_retval = false;
     if (mObservers) {
-        PRUint32 count;
+        uint32_t count;
         mObservers->Count(&count);
-        for (PRUint32 i = 0; i < count; ++i) {
+        for (uint32_t i = 0; i < count; ++i) {
             nsCOMPtr<nsIXULTreeBuilderObserver> observer = do_QueryElementAt(mObservers, i);
             if (observer) {
                 observer->CanDrop(index, orientation, dataTransfer, _retval);
@@ -1971,12 +1935,12 @@ nsXULTreeBuilder::CanDrop(PRInt32 index, PRInt32 orientation,
 }
 
 NS_IMETHODIMP
-nsXULTreeBuilder::Drop(PRInt32 row, PRInt32 orient, nsIDOMDataTransfer* dataTransfer)
+nsXULTreeBuilder::Drop(int32_t row, int32_t orient, nsIDOMDataTransfer* dataTransfer)
 {
     if (mObservers) {
-        PRUint32 count;
+        uint32_t count;
         mObservers->Count(&count);
-        for (PRUint32 i = 0; i < count; ++i) {
+        for (uint32_t i = 0; i < count; ++i) {
             nsCOMPtr<nsIXULTreeBuilderObserver> observer = do_QueryElementAt(mObservers, i);
             if (observer) {
                 bool canDrop = false;
@@ -1993,7 +1957,7 @@ nsXULTreeBuilder::Drop(PRInt32 row, PRInt32 orient, nsIDOMDataTransfer* dataTran
 NS_IMETHODIMP
 nsXULTreeBuilder::IsSorted(bool *_retval)
 {
-  *_retval = (mSortVariable != nsnull);
+  *_retval = (mSortVariable != nullptr);
   return NS_OK;
 }
 
