@@ -1,42 +1,9 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
  * vim: set ts=4 sw=4 et tw=80:
  *
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is the Mozilla browser.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications, Inc.
- * Portions created by the Initial Developer are Copyright (C) 1999
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Travis Bogard <travis@netscape.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifndef nsDocShell_h__
 #define nsDocShell_h__
@@ -140,11 +107,11 @@ public:
     NS_DECL_ISUPPORTS
     NS_DECL_NSITIMERCALLBACK
 
-    PRInt32 GetDelay() { return mDelay ;}
+    int32_t GetDelay() { return mDelay ;}
 
     nsRefPtr<nsDocShell>  mDocShell;
     nsCOMPtr<nsIURI>      mURI;
-    PRInt32               mDelay;
+    int32_t               mDelay;
     bool                  mRepeat;
     bool                  mMetaRefresh;
     
@@ -152,8 +119,6 @@ protected:
     virtual ~nsRefreshTimer();
 };
 
-#define NS_ERROR_DOCSHELL_REQUEST_REJECTED  NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_GENERAL,1001)
-
 typedef enum {
     eCharsetReloadInit,
     eCharsetReloadRequested,
@@ -215,7 +180,6 @@ public:
     NS_DECL_NSIWEBPAGEDESCRIPTOR
     NS_DECL_NSIAUTHPROMPTPROVIDER
     NS_DECL_NSIOBSERVER
-    NS_DECL_NSILOADCONTEXT
     NS_DECL_NSICLIPBOARDCOMMANDS
     NS_DECL_NSIWEBSHELLSERVICES
 
@@ -248,12 +212,21 @@ public:
         const PRUnichar* aTargetSpec);
     NS_IMETHOD OnLeaveLink();
 
-    nsDocShellInfoLoadType ConvertLoadTypeToDocShellLoadInfo(PRUint32 aLoadType);
-    PRUint32 ConvertDocShellLoadInfoToLoadType(nsDocShellInfoLoadType aDocShellLoadType);
+    nsDocShellInfoLoadType ConvertLoadTypeToDocShellLoadInfo(uint32_t aLoadType);
+    uint32_t ConvertDocShellLoadInfoToLoadType(nsDocShellInfoLoadType aDocShellLoadType);
 
     // nsIScriptGlobalObjectOwner methods
     virtual nsIScriptGlobalObject* GetScriptGlobalObject();
 
+    // Don't use NS_DECL_NSILOADCONTEXT because some of nsILoadContext's methods
+    // are shared with nsIDocShell (appID, etc.) and can't be declared twice.
+    NS_IMETHOD GetAssociatedWindow(nsIDOMWindow**);
+    NS_IMETHOD GetTopWindow(nsIDOMWindow**);
+    NS_IMETHOD IsAppOfType(uint32_t, bool*);
+    NS_IMETHOD GetIsContent(bool*);
+    NS_IMETHOD GetUsePrivateBrowsing(bool*);
+    NS_IMETHOD SetUsePrivateBrowsing(bool);
+
     // Restores a cached presentation from history (mLSHE).
     // This method swaps out the content viewer and simulates loads for
     // subframes.  It then simulates the completion of the toplevel load.
@@ -263,7 +236,7 @@ public:
     // ForceRefreshURI method on nsIRefreshURI, but makes sure to take
     // the timer involved out of mRefreshURIList if it's there.
     // aTimer must not be null.
-    nsresult ForceRefreshURIFromTimer(nsIURI * aURI, PRInt32 aDelay,
+    nsresult ForceRefreshURIFromTimer(nsIURI * aURI, int32_t aDelay,
                                       bool aMetaRefresh, nsITimer* aTimer);
 
     friend class OnLinkClickEvent;
@@ -272,11 +245,11 @@ public:
     // updating security info.
     void FireDummyOnLocationChange()
     {
-        FireOnLocationChange(this, nsnull, mCurrentURI,
+        FireOnLocationChange(this, nullptr, mCurrentURI,
                              LOCATION_CHANGE_SAME_DOCUMENT);
     }
 
-    nsresult HistoryTransactionRemoved(PRInt32 aIndex);
+    nsresult HistoryTransactionRemoved(int32_t aIndex);
 protected:
     // Object Management
     virtual ~nsDocShell();
@@ -335,7 +308,7 @@ protected:
                                    bool aBypassClassifier);
 
     nsresult ScrollToAnchor(nsACString & curHash, nsACString & newHash,
-                            PRUint32 aLoadType);
+                            uint32_t aLoadType);
 
     // Tries to serialize a given variant using structured clone.  This only
     // works if the variant is backed by a JSVal.
@@ -361,7 +334,7 @@ protected:
     // If OnNewURI calls AddToSessionHistory, it will pass its
     // aCloneSHChildren argument as aCloneChildren.
     bool OnNewURI(nsIURI * aURI, nsIChannel * aChannel, nsISupports* aOwner,
-                    PRUint32 aLoadType,
+                    uint32_t aLoadType,
                     bool aFireOnLocationChange,
                     bool aAddToGlobalHistory,
                     bool aCloneSHChildren);
@@ -380,10 +353,10 @@ protected:
                                          nsISupports* aOwner,
                                          bool aCloneChildren,
                                          nsISHEntry ** aNewEntry);
-    nsresult DoAddChildSHEntry(nsISHEntry* aNewEntry, PRInt32 aChildOffset,
+    nsresult DoAddChildSHEntry(nsISHEntry* aNewEntry, int32_t aChildOffset,
                                bool aCloneChildren);
 
-    NS_IMETHOD LoadHistoryEntry(nsISHEntry * aEntry, PRUint32 aLoadType);
+    NS_IMETHOD LoadHistoryEntry(nsISHEntry * aEntry, uint32_t aLoadType);
     NS_IMETHOD PersistLayoutHistoryState();
 
     // Clone a session history tree for subframe navigation.
@@ -396,7 +369,7 @@ protected:
     // |aCloneID| will be cloned into |aReplaceEntry|.
     static nsresult CloneAndReplace(nsISHEntry *aSrcEntry,
                                     nsDocShell *aSrcShell,
-                                    PRUint32 aCloneID,
+                                    uint32_t aCloneID,
                                     nsISHEntry *aReplaceEntry,
                                     bool aCloneChildren,
                                     nsISHEntry **aDestEntry);
@@ -404,7 +377,7 @@ protected:
     // Child-walking callback for CloneAndReplace
     static nsresult CloneAndReplaceChild(nsISHEntry *aEntry,
                                          nsDocShell *aShell,
-                                         PRInt32 aChildIndex, void *aData);
+                                         int32_t aChildIndex, void *aData);
 
     nsresult GetRootSessionHistory(nsISHistory ** aReturn);
     nsresult GetHttpChannel(nsIChannel * aChannel, nsIHttpChannel ** aReturn);
@@ -428,7 +401,7 @@ protected:
     // Child-walking callback for SetHistoryEntry
     static nsresult SetChildHistoryEntry(nsISHEntry *aEntry,
                                          nsDocShell *aShell,
-                                         PRInt32 aEntryIndex, void *aData);
+                                         int32_t aEntryIndex, void *aData);
 
     // Callback prototype for WalkHistoryEntries.
     // aEntry is the child history entry, aShell is its corresponding docshell,
@@ -436,7 +409,7 @@ protected:
     // the opaque pointer passed to WalkHistoryEntries.
     typedef nsresult (*WalkHistoryEntriesFunc)(nsISHEntry *aEntry,
                                                nsDocShell *aShell,
-                                               PRInt32 aChildIndex,
+                                               int32_t aChildIndex,
                                                void *aData);
 
     // For each child of aRootEntry, find the corresponding docshell which is
@@ -451,8 +424,8 @@ protected:
     // normal OnStateChange with flags STATE_REDIRECTING
     virtual void OnRedirectStateChange(nsIChannel* aOldChannel,
                                        nsIChannel* aNewChannel,
-                                       PRUint32 aRedirectFlags,
-                                       PRUint32 aStateFlags);
+                                       uint32_t aRedirectFlags,
+                                       uint32_t aStateFlags);
 
     /**
      * Helper function that determines if channel is an HTTP POST.
@@ -481,7 +454,7 @@ protected:
      */
     void ExtractLastVisit(nsIChannel* aChannel,
                           nsIURI** aURI,
-                          PRUint32* aChannelRedirectFlags);
+                          uint32_t* aChannelRedirectFlags);
 
     /**
      * Helper function that caches a URI and a transition for saving later.
@@ -495,7 +468,7 @@ protected:
      */
     void SaveLastVisit(nsIChannel* aChannel,
                        nsIURI* aURI,
-                       PRUint32 aChannelRedirectFlags);
+                       uint32_t aChannelRedirectFlags);
 
     /**
      * Helper function for adding a URI visit using IHistory.  If IHistory is
@@ -526,15 +499,15 @@ protected:
     void AddURIVisit(nsIURI* aURI,
                      nsIURI* aReferrerURI,
                      nsIURI* aPreviousURI,
-                     PRUint32 aChannelRedirectFlags,
-                     PRUint32 aResponseStatus=0);
+                     uint32_t aChannelRedirectFlags,
+                     uint32_t aResponseStatus=0);
 
     // Helper Routines
     nsresult   ConfirmRepost(bool * aRepost);
     NS_IMETHOD GetPromptAndStringBundle(nsIPrompt ** aPrompt,
         nsIStringBundle ** aStringBundle);
     NS_IMETHOD GetChildOffset(nsIDOMNode * aChild, nsIDOMNode * aParent,
-        PRInt32 * aOffset);
+        int32_t * aOffset);
     nsIScrollableFrame* GetRootScrollFrame();
     NS_IMETHOD EnsureScriptEnvironment();
     NS_IMETHOD EnsureEditorData();
@@ -543,7 +516,7 @@ protected:
     nsresult   RefreshURIFromQueue();
     NS_IMETHOD DisplayLoadError(nsresult aError, nsIURI *aURI,
                                 const PRUnichar *aURL,
-                                nsIChannel* aFailedChannel = nsnull);
+                                nsIChannel* aFailedChannel = nullptr);
     NS_IMETHOD LoadErrorPage(nsIURI *aURI, const PRUnichar *aURL,
                              const char *aErrorPage,
                              const PRUnichar *aErrorType,
@@ -555,11 +528,11 @@ protected:
 
     nsresult SetBaseUrlForWyciwyg(nsIContentViewer * aContentViewer);
 
-    static  inline  PRUint32
+    static  inline  uint32_t
     PRTimeToSeconds(PRTime t_usec)
     {
       PRTime usec_per_sec;
-      PRUint32 t_sec;
+      uint32_t t_sec;
       LL_I2L(usec_per_sec, PR_USEC_PER_SEC);
       LL_DIV(t_usec, t_usec, usec_per_sec);
       LL_L2I(t_sec, t_usec);
@@ -598,7 +571,7 @@ protected:
     // In all other cases false is returned.
     bool SetCurrentURI(nsIURI *aURI, nsIRequest *aRequest,
                        bool aFireOnLocationChange,
-                       PRUint32 aLocationFlags);
+                       uint32_t aLocationFlags);
 
     // The following methods deal with saving and restoring content viewers
     // in session history.
@@ -634,7 +607,7 @@ protected:
     // be loaded in place of the current document, or null if such a request
     // has not been created yet. |aNewDocument| should be the document that will
     // replace the current document.
-    bool CanSavePresentation(PRUint32 aLoadType,
+    bool CanSavePresentation(uint32_t aLoadType,
                                nsIRequest *aNewRequest,
                                nsIDocument *aNewDocument);
 
@@ -648,12 +621,12 @@ protected:
     // RestoreFromHistory is called from a PLEvent.
     nsresult RestorePresentation(nsISHEntry *aSHEntry, bool *aRestoring);
 
-    // Call BeginRestore(nsnull, false) for each child of this shell.
+    // Call BeginRestore(nullptr, false) for each child of this shell.
     nsresult BeginRestoreChildren();
 
     // Method to get our current position and size without flushing
-    void DoGetPositionAndSize(PRInt32 * x, PRInt32 * y, PRInt32 * cx,
-                              PRInt32 * cy);
+    void DoGetPositionAndSize(int32_t * x, int32_t * y, int32_t * cx,
+                              int32_t * cy);
     
     // Call this when a URI load is handed to us (via OnLinkClick or
     // InternalLoad).  This makes sure that we're not inside unload, or that if
@@ -675,6 +648,8 @@ protected:
     nsresult EnsureCommandHandler();
 
     nsIChannel* GetCurrentDocChannel();
+
+    bool ShouldBlockLoadingForBackButton();
 protected:
     // Override the parent setter from nsDocLoader
     virtual nsresult SetDocLoaderParent(nsDocLoader * aLoader);
@@ -688,13 +663,22 @@ protected:
     public:
         NS_DECL_NSIRUNNABLE
         RestorePresentationEvent(nsDocShell *ds) : mDocShell(ds) {}
-        void Revoke() { mDocShell = nsnull; }
+        void Revoke() { mDocShell = nullptr; }
     private:
         nsRefPtr<nsDocShell> mDocShell;
     };
 
     bool JustStartedNetworkLoad();
 
+    enum FrameType {
+        eFrameTypeRegular  = 0x0, // 0000
+        eFrameTypeBrowser  = 0x1, // 0001
+        eFrameTypeApp      = 0x2  // 0010
+    };
+
+    FrameType GetInheritedFrameType();
+    FrameType GetFrameType();
+
     // hash of session storages, keyed by domain
     nsInterfaceHashtable<nsCStringHashKey, nsIDOMStorage> mStorages;
 
@@ -761,7 +745,7 @@ protected:
     // for which these objects are needed.
     nsCOMPtr<nsIURI>           mFailedURI;
     nsCOMPtr<nsIChannel>       mFailedChannel;
-    PRUint32                   mFailedLoadType;
+    uint32_t                   mFailedLoadType;
 
     // WEAK REFERENCES BELOW HERE.
     // Note these are intentionally not addrefd.  Doing so will create a cycle.
@@ -774,22 +758,24 @@ protected:
 
     // Offset in the parent's child list.
     // -1 if the docshell is added dynamically to the parent shell.
-    PRUint32                   mChildOffset;
-    PRUint32                   mBusyFlags;
-    PRUint32                   mAppType;
-    PRUint32                   mLoadType;
+    uint32_t                   mChildOffset;
+    uint32_t                   mBusyFlags;
+    uint32_t                   mAppType;
+    uint32_t                   mLoadType;
 
-    PRInt32                    mMarginWidth;
-    PRInt32                    mMarginHeight;
+    int32_t                    mMarginWidth;
+    int32_t                    mMarginHeight;
 
     // This can either be a content docshell or a chrome docshell.  After
     // Create() is called, the type is not expected to change.
-    PRInt32                    mItemType;
+    int32_t                    mItemType;
 
     // Index into the SHTransaction list, indicating the previous and current
     // transaction at the time that this DocShell begins to load
-    PRInt32                    mPreviousTransIndex;
-    PRInt32                    mLoadedTransIndex;
+    int32_t                    mPreviousTransIndex;
+    int32_t                    mLoadedTransIndex;
+
+    uint32_t                   mSandboxFlags;
 
     bool                       mCreated;
     bool                       mAllowSubframes;
@@ -834,22 +820,26 @@ protected:
     // presentation of the page, and to SetupNewViewer() that the old viewer
     // should be passed a SHEntry to save itself into.
     bool                       mSavingOldViewer;
-
+    
     // @see nsIDocShellHistory::createdDynamically
     bool                       mDynamicallyCreated;
 #ifdef DEBUG
     bool                       mInEnsureScriptEnv;
 #endif
-    PRUint64                   mHistoryID;
+    uint64_t                   mHistoryID;
 
     static nsIURIFixup *sURIFixup;
 
     nsRefPtr<nsDOMNavigationTiming> mTiming;
 
+    uint32_t mAppId;
+
 private:
     nsCOMPtr<nsIAtom> mForcedCharset;
     nsCOMPtr<nsIAtom> mParentCharset;
-    PRInt32          mParentCharsetSource;
+    nsTObserverArray<nsWeakPtr> mPrivacyObservers;
+    int32_t           mParentCharsetSource;
+    nsCString         mOriginalUriString;
 
 #ifdef DEBUG
     // We're counting the number of |nsDocShells| to help find leaks
