@@ -1,41 +1,8 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* vim: set ts=2 et sw=2 tw=80: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Indexed Database.
- *
- * The Initial Developer of the Original Code is
- * The Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2010
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Ben Turner <bent.mozilla@gmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "TransactionThreadPool.h"
 
@@ -53,11 +20,11 @@ USING_INDEXEDDB_NAMESPACE
 
 namespace {
 
-const PRUint32 kThreadLimit = 20;
-const PRUint32 kIdleThreadLimit = 5;
-const PRUint32 kIdleThreadTimeoutMs = 30000;
+const uint32_t kThreadLimit = 20;
+const uint32_t kIdleThreadLimit = 5;
+const uint32_t kIdleThreadTimeoutMs = 30000;
 
-TransactionThreadPool* gInstance = nsnull;
+TransactionThreadPool* gInstance = nullptr;
 bool gShutdown = false;
 
 inline
@@ -67,11 +34,11 @@ CheckOverlapAndMergeObjectStores(nsTArray<nsString>& aLockedStores,
                                  bool aShouldMerge,
                                  bool* aStoresOverlap)
 {
-  PRUint32 length = aObjectStores.Length();
+  uint32_t length = aObjectStores.Length();
 
   bool overlap = false;
 
-  for (PRUint32 index = 0; index < length; index++) {
+  for (uint32_t index = 0; index < length; index++) {
     const nsString& storeName = aObjectStores[index];
     if (aLockedStores.Contains(storeName)) {
       overlap = true;
@@ -90,7 +57,7 @@ CheckOverlapAndMergeObjectStores(nsTArray<nsString>& aLockedStores,
 
 BEGIN_INDEXEDDB_NAMESPACE
 
-class FinishTransactionRunnable : public nsIRunnable
+class FinishTransactionRunnable MOZ_FINAL : public nsIRunnable
 {
 public:
   NS_DECL_ISUPPORTS
@@ -116,7 +83,7 @@ TransactionThreadPool::~TransactionThreadPool()
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
   NS_ASSERTION(gInstance == this, "Different instances!");
-  gInstance = nsnull;
+  gInstance = nullptr;
 }
 
 // static
@@ -128,7 +95,7 @@ TransactionThreadPool::GetOrCreate()
     nsAutoPtr<TransactionThreadPool> pool(new TransactionThreadPool());
 
     nsresult rv = pool->Init();
-    NS_ENSURE_SUCCESS(rv, nsnull);
+    NS_ENSURE_SUCCESS(rv, nullptr);
 
     gInstance = pool.forget();
   }
@@ -155,7 +122,7 @@ TransactionThreadPool::Shutdown()
       NS_WARNING("Failed to shutdown thread pool!");
     }
     delete gInstance;
-    gInstance = nsnull;
+    gInstance = nullptr;
   }
 }
 
@@ -164,15 +131,15 @@ TransactionThreadPool::Init()
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
 
-  if (!mTransactionsInProgress.Init()) {
-    NS_WARNING("Failed to init hash!");
-    return NS_ERROR_FAILURE;
-  }
+  mTransactionsInProgress.Init();
 
   nsresult rv;
   mThreadPool = do_CreateInstance(NS_THREADPOOL_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
+  rv = mThreadPool->SetName(NS_LITERAL_CSTRING("IndexedDB Trans"));
+  NS_ENSURE_SUCCESS(rv, rv);
+
   rv = mThreadPool->SetThreadLimit(kThreadLimit);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -195,18 +162,18 @@ TransactionThreadPool::Cleanup()
 
   // Make sure the pool is still accessible while any callbacks generated from
   // the other threads are processed.
-  rv = NS_ProcessPendingEvents(nsnull);
+  rv = NS_ProcessPendingEvents(nullptr);
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (!mCompleteCallbacks.IsEmpty()) {
     // Run all callbacks manually now.
-    for (PRUint32 index = 0; index < mCompleteCallbacks.Length(); index++) {
+    for (uint32_t index = 0; index < mCompleteCallbacks.Length(); index++) {
       mCompleteCallbacks[index].mCallback->Run();
     }
     mCompleteCallbacks.Clear();
 
     // And make sure they get processed.
-    rv = NS_ProcessPendingEvents(nsnull);
+    rv = NS_ProcessPendingEvents(nullptr);
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
@@ -233,7 +200,7 @@ TransactionThreadPool::FinishTransaction(IDBTransaction* aTransaction)
   nsTArray<TransactionInfo>& transactionsInProgress =
     dbTransactionInfo->transactions;
 
-  PRUint32 transactionCount = transactionsInProgress.Length();
+  uint32_t transactionCount = transactionsInProgress.Length();
 
 #ifdef DEBUG
   if (aTransaction->mMode == IDBTransaction::VERSION_CHANGE) {
@@ -252,15 +219,21 @@ TransactionThreadPool::FinishTransaction(IDBTransaction* aTransaction)
     mTransactionsInProgress.Remove(databaseId);
 
     // See if we need to fire any complete callbacks.
-    for (PRUint32 index = 0; index < mCompleteCallbacks.Length(); index++) {
-      MaybeFireCallback(index);
+    uint32_t index = 0;
+    while (index < mCompleteCallbacks.Length()) {
+      if (MaybeFireCallback(mCompleteCallbacks[index])) {
+        mCompleteCallbacks.RemoveElementAt(index);
+      }
+      else {
+        index++;
+      }
     }
   }
   else {
     // We need to rebuild the locked object store list.
     nsTArray<nsString> storesWriting, storesReading;
 
-    for (PRUint32 index = 0, count = transactionCount; index < count; index++) {
+    for (uint32_t index = 0, count = transactionCount; index < count; index++) {
       IDBTransaction* transaction = transactionsInProgress[index].transaction;
       if (transaction == aTransaction) {
         NS_ASSERTION(count == transactionCount, "More than one match?!");
@@ -306,7 +279,7 @@ TransactionThreadPool::FinishTransaction(IDBTransaction* aTransaction)
   queuedDispatch.SwapElements(mDelayedDispatchQueue);
 
   transactionCount = queuedDispatch.Length();
-  for (PRUint32 index = 0; index < transactionCount; index++) {
+  for (uint32_t index = 0; index < transactionCount; index++) {
     if (NS_FAILED(Dispatch(queuedDispatch[index]))) {
       NS_WARNING("Dispatch failed!");
     }
@@ -325,24 +298,24 @@ TransactionThreadPool::TransactionCanRun(IDBTransaction* aTransaction,
 
   nsIAtom* databaseId = aTransaction->mDatabase->Id();
   const nsTArray<nsString>& objectStoreNames = aTransaction->mObjectStoreNames;
-  const PRUint16 mode = aTransaction->mMode;
+  const uint16_t mode = aTransaction->mMode;
 
   // See if we can run this transaction now.
   DatabaseTransactionInfo* dbTransactionInfo;
   if (!mTransactionsInProgress.Get(databaseId, &dbTransactionInfo)) {
     // First transaction for this database, fine to run.
     *aCanRun = true;
-    *aExistingQueue = nsnull;
+    *aExistingQueue = nullptr;
     return NS_OK;
   }
 
   nsTArray<TransactionInfo>& transactionsInProgress =
     dbTransactionInfo->transactions;
 
-  PRUint32 transactionCount = transactionsInProgress.Length();
+  uint32_t transactionCount = transactionsInProgress.Length();
   NS_ASSERTION(transactionCount, "Should never be 0!");
 
-  for (PRUint32 index = 0; index < transactionCount; index++) {
+  for (uint32_t index = 0; index < transactionCount; index++) {
     // See if this transaction is in out list of current transactions.
     const TransactionInfo& info = transactionsInProgress[index];
     if (info.transaction == aTransaction) {
@@ -372,12 +345,12 @@ TransactionThreadPool::TransactionCanRun(IDBTransaction* aTransaction,
   if (writeOverlap ||
       (readOverlap && mode == IDBTransaction::READ_WRITE)) {
     *aCanRun = false;
-    *aExistingQueue = nsnull;
+    *aExistingQueue = nullptr;
     return NS_OK;
   }
 
   *aCanRun = true;
-  *aExistingQueue = nsnull;
+  *aExistingQueue = nullptr;
   return NS_OK;
 }
 
@@ -424,7 +397,7 @@ TransactionThreadPool::Dispatch(IDBTransaction* aTransaction,
 
 #ifdef DEBUG
   if (aTransaction->mMode == IDBTransaction::VERSION_CHANGE) {
-    NS_ASSERTION(!mTransactionsInProgress.Get(databaseId, nsnull),
+    NS_ASSERTION(!mTransactionsInProgress.Get(databaseId, nullptr),
                  "Shouldn't have anything in progress!");
   }
 #endif
@@ -468,10 +441,7 @@ TransactionThreadPool::Dispatch(IDBTransaction* aTransaction,
   }
 
   if (autoDBTransactionInfo) {
-    if (!mTransactionsInProgress.Put(databaseId, autoDBTransactionInfo)) {
-      NS_WARNING("Failed to put!");
-      return NS_ERROR_OUT_OF_MEMORY;
-    }
+    mTransactionsInProgress.Put(databaseId, autoDBTransactionInfo);
     autoDBTransactionInfo.forget();
   }
 
@@ -480,8 +450,8 @@ TransactionThreadPool::Dispatch(IDBTransaction* aTransaction,
 
 bool
 TransactionThreadPool::WaitForAllDatabasesToComplete(
-                                   nsTArray<nsRefPtr<IDBDatabase> >& aDatabases,
-                                   nsIRunnable* aCallback)
+                                            nsTArray<IDBDatabase*>& aDatabases,
+                                            nsIRunnable* aCallback)
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
   NS_ASSERTION(!aDatabases.IsEmpty(), "No databases to wait on!");
@@ -498,7 +468,10 @@ TransactionThreadPool::WaitForAllDatabasesToComplete(
     NS_ERROR("This should never fail!");
   }
 
-  MaybeFireCallback(mCompleteCallbacks.Length() - 1);
+  if (MaybeFireCallback(*callback)) {
+    mCompleteCallbacks.RemoveElementAt(mCompleteCallbacks.Length() - 1);
+  }
+
   return true;
 }
 
@@ -521,10 +494,10 @@ TransactionThreadPool::AbortTransactionsForDatabase(IDBDatabase* aDatabase)
   nsTArray<TransactionInfo>& transactionsInProgress =
     dbTransactionInfo->transactions;
 
-  PRUint32 transactionCount = transactionsInProgress.Length();
+  uint32_t transactionCount = transactionsInProgress.Length();
   NS_ASSERTION(transactionCount, "Should never be 0!");
 
-  for (PRUint32 index = 0; index < transactionCount; index++) {
+  for (uint32_t index = 0; index < transactionCount; index++) {
     // See if any transaction belongs to this IDBDatabase instance
     IDBTransaction* transaction = transactionsInProgress[index].transaction;
     if (transaction->Database() == aDatabase) {
@@ -533,7 +506,7 @@ TransactionThreadPool::AbortTransactionsForDatabase(IDBDatabase* aDatabase)
   }
 
   // Collect any pending transactions.
-  for (PRUint32 index = 0; index < mDelayedDispatchQueue.Length(); index++) {
+  for (uint32_t index = 0; index < mDelayedDispatchQueue.Length(); index++) {
     // See if any transaction belongs to this IDBDatabase instance
     IDBTransaction* transaction = mDelayedDispatchQueue[index].transaction;
     if (transaction->Database() == aDatabase) {
@@ -543,7 +516,7 @@ TransactionThreadPool::AbortTransactionsForDatabase(IDBDatabase* aDatabase)
 
   // Abort transactions. Do this after collecting the transactions in case
   // calling Abort() modifies the data structures we're iterating above.
-  for (PRUint32 index = 0; index < transactions.Length(); index++) {
+  for (uint32_t index = 0; index < transactions.Length(); index++) {
     // This can fail, for example if the transaction is in the process of
     // being comitted. That is expected and fine, so we ignore any returned
     // errors.
@@ -566,10 +539,10 @@ TransactionThreadPool::HasTransactionsForDatabase(IDBDatabase* aDatabase)
   nsTArray<TransactionInfo>& transactionsInProgress =
     dbTransactionInfo->transactions;
 
-  PRUint32 transactionCount = transactionsInProgress.Length();
+  uint32_t transactionCount = transactionsInProgress.Length();
   NS_ASSERTION(transactionCount, "Should never be 0!");
 
-  for (PRUint32 index = 0; index < transactionCount; index++) {
+  for (uint32_t index = 0; index < transactionCount; index++) {
     // See if any transaction belongs to this IDBDatabase instance
     if (transactionsInProgress[index].transaction->Database() == aDatabase) {
       return true;
@@ -579,25 +552,20 @@ TransactionThreadPool::HasTransactionsForDatabase(IDBDatabase* aDatabase)
   return false;
 }
 
-void
-TransactionThreadPool::MaybeFireCallback(PRUint32 aCallbackIndex)
+bool
+TransactionThreadPool::MaybeFireCallback(DatabasesCompleteCallback& aCallback)
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
 
-  DatabasesCompleteCallback& callback = mCompleteCallbacks[aCallbackIndex];
-
-  bool freeToRun = true;
-  for (PRUint32 index = 0; index < callback.mDatabases.Length(); index++) {
-    if (mTransactionsInProgress.Get(callback.mDatabases[index]->Id(), nsnull)) {
-      freeToRun = false;
-      break;
+  for (uint32_t index = 0; index < aCallback.mDatabases.Length(); index++) {
+    if (mTransactionsInProgress.Get(aCallback.mDatabases[index]->Id(),
+                                    nullptr)) {
+      return false;
     }
   }
 
-  if (freeToRun) {
-    callback.mCallback->Run();
-    mCompleteCallbacks.RemoveElementAt(aCallbackIndex);
-  }
+  aCallback.mCallback->Run();
+  return true;
 }
 
 TransactionThreadPool::
@@ -665,11 +633,11 @@ TransactionThreadPool::TransactionQueue::Run()
       }
     }
 
-    PRUint32 count = queue.Length();
-    for (PRUint32 index = 0; index < count; index++) {
+    uint32_t count = queue.Length();
+    for (uint32_t index = 0; index < count; index++) {
       nsCOMPtr<nsIRunnable>& runnable = queue[index];
       runnable->Run();
-      runnable = nsnull;
+      runnable = nullptr;
     }
 
     if (count) {
@@ -712,7 +680,7 @@ FinishTransactionRunnable::Run()
 
   if (mFinishRunnable) {
     mFinishRunnable->Run();
-    mFinishRunnable = nsnull;
+    mFinishRunnable = nullptr;
   }
 
   return NS_OK;
