@@ -1,46 +1,12 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2001
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *  Conrad Carlen <ccarlen@netscape.com>
- *  Ere Maijala <emaijala@kolumbus.fi>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsPluginDirServiceProvider.h"
 
 #include "nsCRT.h"
-#include "nsILocalFile.h"
+#include "nsIFile.h"
 #include "nsDependentString.h"
 #include "prmem.h"
 #include "nsArrayEnumerator.h"
@@ -134,7 +100,7 @@ TranslateVersionStr(const WCHAR* szVersion, verBlock *vbVersion)
   WCHAR* szNum4 = NULL;
   WCHAR* szJavaBuild = NULL;
 
-  WCHAR *strVer = nsnull;
+  WCHAR *strVer = nullptr;
   if (szVersion) {
     strVer = wcsdup(szVersion);
   }
@@ -223,12 +189,12 @@ NS_IMETHODIMP
 nsPluginDirServiceProvider::GetFile(const char *charProp, bool *persistant,
                                     nsIFile **_retval)
 {
-  nsCOMPtr<nsILocalFile>  localFile;
+  nsCOMPtr<nsIFile>  localFile;
   nsresult rv = NS_ERROR_FAILURE;
 
   NS_ENSURE_ARG(charProp);
 
-  *_retval = nsnull;
+  *_retval = nullptr;
   *persistant = false;
 
   nsCOMPtr<nsIWindowsRegKey> regKey =
@@ -256,18 +222,18 @@ nsPluginDirServiceProvider::GetFile(const char *charProp, bool *persistant,
     regKey->ReadStringValue(NS_LITERAL_STRING("BrowserJavaVersion"),
                             browserJavaVersion);
 
-    PRUint32 childCount = 0;
+    uint32_t childCount = 0;
     regKey->GetChildCount(&childCount);
 
     // We must enumerate through the keys because what if there is
     // more than one version?
-    for (PRUint32 index = 0; index < childCount; ++index) {
+    for (uint32_t index = 0; index < childCount; ++index) {
       nsAutoString childName;
       rv = regKey->GetChildName(index, childName);
       if (NS_SUCCEEDED(rv)) {
         // Skip major.minor as it always points to latest in its family
-        PRUint32 numChars = 0;
-        PRInt32 offset = 0;
+        uint32_t numChars = 0;
+        int32_t offset = 0;
         while ((offset = childName.FindChar(L'.', offset + 1)) >= 0) {
           ++numChars;
         }
@@ -436,10 +402,10 @@ nsPluginDirServiceProvider::GetFile(const char *charProp, bool *persistant,
 
     // We must enumerate through the keys because what if there is
     // more than one version?
-    PRUint32 childCount = 0;
+    uint32_t childCount = 0;
     regKey->GetChildCount(&childCount);
 
-    for (PRUint32 index = 0; index < childCount; ++index) {
+    for (uint32_t index = 0; index < childCount; ++index) {
       nsAutoString childName;
       rv = regKey->GetChildName(index, childName);
       if (NS_SUCCEEDED(rv)) {
@@ -483,9 +449,9 @@ nsresult
 nsPluginDirServiceProvider::GetPLIDDirectories(nsISimpleEnumerator **aEnumerator)
 {
   NS_ENSURE_ARG_POINTER(aEnumerator);
-  *aEnumerator = nsnull;
+  *aEnumerator = nullptr;
 
-  nsCOMArray<nsILocalFile> dirs;
+  nsCOMArray<nsIFile> dirs;
 
   GetPLIDDirectoriesWithRootKey(nsIWindowsRegKey::ROOT_KEY_CURRENT_USER, dirs);
   GetPLIDDirectoriesWithRootKey(nsIWindowsRegKey::ROOT_KEY_LOCAL_MACHINE, dirs);
@@ -494,7 +460,7 @@ nsPluginDirServiceProvider::GetPLIDDirectories(nsISimpleEnumerator **aEnumerator
 }
 
 nsresult
-nsPluginDirServiceProvider::GetPLIDDirectoriesWithRootKey(PRUint32 aKey, nsCOMArray<nsILocalFile> &aDirs)
+nsPluginDirServiceProvider::GetPLIDDirectoriesWithRootKey(uint32_t aKey, nsCOMArray<nsIFile> &aDirs)
 {
   nsCOMPtr<nsIWindowsRegKey> regKey =
     do_CreateInstance("@mozilla.org/windows-registry-key;1");
@@ -505,10 +471,10 @@ nsPluginDirServiceProvider::GetPLIDDirectoriesWithRootKey(PRUint32 aKey, nsCOMAr
                              nsIWindowsRegKey::ACCESS_READ);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  PRUint32 childCount = 0;
+  uint32_t childCount = 0;
   regKey->GetChildCount(&childCount);
 
-  for (PRUint32 index = 0; index < childCount; ++index) {
+  for (uint32_t index = 0; index < childCount; ++index) {
     nsAutoString childName;
     rv = regKey->GetChildName(index, childName);
     if (NS_SUCCEEDED(rv)) {
@@ -519,7 +485,7 @@ nsPluginDirServiceProvider::GetPLIDDirectoriesWithRootKey(PRUint32 aKey, nsCOMAr
         nsAutoString path;
         rv = childKey->ReadStringValue(NS_LITERAL_STRING("Path"), path);
         if (NS_SUCCEEDED(rv)) {
-          nsCOMPtr<nsILocalFile> localFile;
+          nsCOMPtr<nsIFile> localFile;
           if (NS_SUCCEEDED(NS_NewLocalFile(path, true,
                                            getter_AddRefs(localFile))) &&
               localFile) {
@@ -530,7 +496,7 @@ nsPluginDirServiceProvider::GetPLIDDirectoriesWithRootKey(PRUint32 aKey, nsCOMAr
               nsCOMPtr<nsIFile> temp;
               localFile->GetParent(getter_AddRefs(temp));
               if (temp)
-                localFile = do_QueryInterface(temp);
+                localFile = temp;
             }
 
             // Now we check to make sure it's actually on disk and
@@ -538,8 +504,8 @@ nsPluginDirServiceProvider::GetPLIDDirectoriesWithRootKey(PRUint32 aKey, nsCOMAr
             bool isFileThere = false;
             bool isDupEntry = false;
             if (NS_SUCCEEDED(localFile->Exists(&isFileThere)) && isFileThere) {
-              PRInt32 c = aDirs.Count();
-              for (PRInt32 i = 0; i < c; i++) {
+              int32_t c = aDirs.Count();
+              for (int32_t i = 0; i < c; i++) {
                 nsIFile *dup = static_cast<nsIFile*>(aDirs[i]);
                 if (dup &&
                     NS_SUCCEEDED(dup->Equals(localFile, &isDupEntry)) &&
