@@ -1,43 +1,10 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Mozilla Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2006
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Neil Deakin <enndeakin@sympatico.ca>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsCOMPtr.h"
-#include "nsDOMError.h"
+#include "nsError.h"
 #include "nsDOMStorage.h"
 #include "nsDOMStorageDBWrapper.h"
 #include "nsDOMStoragePersistentDB.h"
@@ -53,6 +20,9 @@
 #include "mozIStorageValueArray.h"
 #include "mozIStorageFunction.h"
 #include "nsNetUtil.h"
+#include "mozilla/Attributes.h"
+
+#include "sampler.h"
 
 using namespace mozilla;
 
@@ -60,7 +30,7 @@ using namespace mozilla;
 // then this time in seconds since the load
 #define TEMP_TABLE_MAX_AGE (10) // seconds
 
-class nsReverseStringSQLFunction : public mozIStorageFunction
+class nsReverseStringSQLFunction MOZ_FINAL : public mozIStorageFunction
 {
   NS_DECL_ISUPPORTS
   NS_DECL_MOZISTORAGEFUNCTION
@@ -93,7 +63,7 @@ nsReverseStringSQLFunction::OnFunctionCall(
   return NS_OK;
 }
 
-class nsIsOfflineSQLFunction : public mozIStorageFunction
+class nsIsOfflineSQLFunction MOZ_FINAL : public mozIStorageFunction
 {
   NS_DECL_ISUPPORTS
   NS_DECL_MOZISTORAGEFUNCTION
@@ -434,7 +404,7 @@ nsDOMStoragePersistentDB::GetAllKeys(DOMStorageImpl* aStorage,
     rv = stmt->GetString(1, value);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    PRInt32 secureInt = 0;
+    int32_t secureInt = 0;
     rv = stmt->GetInt32(2, &secureInt);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -457,6 +427,7 @@ nsDOMStoragePersistentDB::GetKeyValue(DOMStorageImpl* aStorage,
                                       nsAString& aValue,
                                       bool* aSecure)
 {
+  SAMPLE_LABEL("nsDOMStoragePersistentDB", "GetKeyValue");
   nsresult rv;
 
   rv = MaybeCommitInsertTransaction();
@@ -484,7 +455,7 @@ nsDOMStoragePersistentDB::GetKeyValue(DOMStorageImpl* aStorage,
   rv = stmt->ExecuteStep(&exists);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  PRInt32 secureInt = 0;
+  int32_t secureInt = 0;
   if (exists) {
     rv = stmt->GetString(0, aValue);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -506,16 +477,16 @@ nsDOMStoragePersistentDB::SetKey(DOMStorageImpl* aStorage,
                                  const nsAString& aKey,
                                  const nsAString& aValue,
                                  bool aSecure,
-                                 PRInt32 aQuota,
+                                 int32_t aQuota,
                                  bool aExcludeOfflineFromUsage,
-                                 PRInt32 *aNewUsage)
+                                 int32_t *aNewUsage)
 {
   nsresult rv;
 
   rv = EnsureLoadTemporaryTableForStorage(aStorage);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  PRInt32 usage = 0;
+  int32_t usage = 0;
   if (!aStorage->GetQuotaDomainDBKey(!aExcludeOfflineFromUsage).IsEmpty()) {
     rv = GetUsage(aStorage, aExcludeOfflineFromUsage, &usage);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -618,7 +589,7 @@ nsresult
 nsDOMStoragePersistentDB::RemoveKey(DOMStorageImpl* aStorage,
                                     const nsAString& aKey,
                                     bool aExcludeOfflineFromUsage,
-                                    PRInt32 aKeyUsage)
+                                    int32_t aKeyUsage)
 {
   nsresult rv;
 
@@ -747,7 +718,7 @@ nsDOMStoragePersistentDB::RemoveOwners(const nsTArray<nsString> &aOwners,
     expression.AppendLiteral("DELETE FROM webappsstore2_view WHERE scope NOT IN (");
   }
 
-  for (PRUint32 i = 0; i < aOwners.Length(); i++) {
+  for (uint32_t i = 0; i < aOwners.Length(); i++) {
     if (i)
       expression.AppendLiteral(" UNION ");
 
@@ -772,7 +743,7 @@ nsDOMStoragePersistentDB::RemoveOwners(const nsTArray<nsString> &aOwners,
                                     getter_AddRefs(statement));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  for (PRUint32 i = 0; i < aOwners.Length(); i++) {
+  for (uint32_t i = 0; i < aOwners.Length(); i++) {
     nsCAutoString quotaKey;
     rv = nsDOMStorageDBWrapper::CreateDomainScopeDBKey(
       NS_ConvertUTF16toUTF8(aOwners[i]), quotaKey);
@@ -827,7 +798,7 @@ nsDOMStoragePersistentDB::RemoveAll()
 nsresult
 nsDOMStoragePersistentDB::GetUsage(DOMStorageImpl* aStorage,
                                    bool aExcludeOfflineFromUsage,
-                                   PRInt32 *aUsage)
+                                   int32_t *aUsage)
 {
   return GetUsageInternal(aStorage->GetQuotaDomainDBKey(!aExcludeOfflineFromUsage),
                                                         aExcludeOfflineFromUsage,
@@ -837,7 +808,7 @@ nsDOMStoragePersistentDB::GetUsage(DOMStorageImpl* aStorage,
 nsresult
 nsDOMStoragePersistentDB::GetUsage(const nsACString& aDomain,
                                    bool aIncludeSubDomains,
-                                   PRInt32 *aUsage)
+                                   int32_t *aUsage)
 {
   nsresult rv;
 
@@ -854,7 +825,7 @@ nsDOMStoragePersistentDB::GetUsage(const nsACString& aDomain,
 nsresult
 nsDOMStoragePersistentDB::GetUsageInternal(const nsACString& aQuotaDomainDBKey,
                                            bool aExcludeOfflineFromUsage,
-                                           PRInt32 *aUsage)
+                                           int32_t *aUsage)
 {
   if (aQuotaDomainDBKey == mCachedOwner) {
     *aUsage = mCachedUsage;
