@@ -1,51 +1,17 @@
 /* -*- Mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; tab-width: 40 -*- */
 /* vim: set ts=2 et sw=2 tw=40: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Telephony.
- *
- * The Initial Developer of the Original Code is
- *   The Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2011
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Ben Turner <bent.mozilla@gmail.com> (Original Author)
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "Telephony.h"
 
-#include "nsIDocument.h"
 #include "nsIURI.h"
 #include "nsIURL.h"
 #include "nsPIDOMWindow.h"
 
 #include "jsapi.h"
-#include "mozilla/Preferences.h"
+#include "nsIPermissionManager.h"
 #include "nsCharSeparatedTokenizer.h"
 #include "nsContentUtils.h"
 #include "nsDOMClassInfo.h"
@@ -53,15 +19,13 @@
 #include "nsNetUtil.h"
 #include "nsServiceManagerUtils.h"
 #include "SystemWorkerManager.h"
+#include "nsRadioInterfaceLayer.h"
 
 #include "CallEvent.h"
 #include "TelephonyCall.h"
 
 USING_TELEPHONY_NAMESPACE
 using namespace mozilla::dom::gonk;
-using mozilla::Preferences;
-
-#define DOM_TELEPHONY_APP_PHONE_URL_PREF "dom.telephony.app.phone.url"
 
 namespace {
 
@@ -79,28 +43,24 @@ nsTArrayToJSArray(JSContext* aCx, JSObject* aGlobal,
   NS_ASSERTION(aGlobal, "Null global!");
 
   JSAutoRequest ar(aCx);
-  JSAutoEnterCompartment ac;
-  if (!ac.enter(aCx, aGlobal)) {
-    NS_WARNING("Failed to enter compartment!");
-    return NS_ERROR_FAILURE;
-  }
+  JSAutoCompartment ac(aCx, aGlobal);
 
   JSObject* arrayObj;
 
   if (aSourceArray.IsEmpty()) {
-    arrayObj = JS_NewArrayObject(aCx, 0, nsnull);
+    arrayObj = JS_NewArrayObject(aCx, 0, nullptr);
   } else {
-    nsTArray<jsval> valArray;
-    valArray.SetLength(aSourceArray.Length());
-
-    for (PRUint32 index = 0; index < valArray.Length(); index++) {
+    uint32_t valLength = aSourceArray.Length();
+    mozilla::ScopedDeleteArray<jsval> valArray(new jsval[valLength]);
+    JS::AutoArrayRooter tvr(aCx, 0, valArray);
+    for (uint32_t index = 0; index < valLength; index++) {
       nsISupports* obj = aSourceArray[index]->ToISupports();
       nsresult rv =
         nsContentUtils::WrapNative(aCx, aGlobal, obj, &valArray[index]);
       NS_ENSURE_SUCCESS(rv, rv);
+      tvr.changeLength(index + 1);
     }
-
-    arrayObj = JS_NewArrayObject(aCx, valArray.Length(), valArray.Elements());
+    arrayObj = JS_NewArrayObject(aCx, valLength, valArray);
   }
 
   if (!arrayObj) {
@@ -119,7 +79,7 @@ nsTArrayToJSArray(JSContext* aCx, JSObject* aGlobal,
 } // anonymous namespace
 
 Telephony::Telephony()
-: mActiveCall(nsnull), mCallsArray(nsnull), mRooted(false)
+: mActiveCall(nullptr), mCallsArray(nullptr), mRooted(false)
 {
   if (!gTelephonyList) {
     gTelephonyList = new TelephonyList();
@@ -131,7 +91,7 @@ Telephony::Telephony()
 Telephony::~Telephony()
 {
   if (mRIL && mRILTelephonyCallback) {
-    mRIL->UnregisterCallback(mRILTelephonyCallback);
+    mRIL->UnregisterTelephonyCallback(mRILTelephonyCallback);
   }
 
   if (mRooted) {
@@ -143,7 +103,7 @@ Telephony::~Telephony()
 
   if (gTelephonyList->Length() == 1) {
     delete gTelephonyList;
-    gTelephonyList = nsnull;
+    gTelephonyList = nullptr;
   }
   else {
     gTelephonyList->RemoveElement(this);
@@ -152,16 +112,16 @@ Telephony::~Telephony()
 
 // static
 already_AddRefed<Telephony>
-Telephony::Create(nsPIDOMWindow* aOwner, nsIRadioInterfaceLayer* aRIL)
+Telephony::Create(nsPIDOMWindow* aOwner, nsIRILContentHelper* aRIL)
 {
   NS_ASSERTION(aOwner, "Null owner!");
   NS_ASSERTION(aRIL, "Null RIL!");
 
   nsCOMPtr<nsIScriptGlobalObject> sgo = do_QueryInterface(aOwner);
-  NS_ENSURE_TRUE(sgo, nsnull);
+  NS_ENSURE_TRUE(sgo, nullptr);
 
   nsCOMPtr<nsIScriptContext> scriptContext = sgo->GetContext();
-  NS_ENSURE_TRUE(scriptContext, nsnull);
+  NS_ENSURE_TRUE(scriptContext, nullptr);
 
   nsRefPtr<Telephony> telephony = new Telephony();
 
@@ -171,10 +131,10 @@ Telephony::Create(nsPIDOMWindow* aOwner, nsIRadioInterfaceLayer* aRIL)
   telephony->mRILTelephonyCallback = new RILTelephonyCallback(telephony);
 
   nsresult rv = aRIL->EnumerateCalls(telephony->mRILTelephonyCallback);
-  NS_ENSURE_SUCCESS(rv, nsnull);
+  NS_ENSURE_SUCCESS(rv, nullptr);
 
-  rv = aRIL->RegisterCallback(telephony->mRILTelephonyCallback);
-  NS_ENSURE_SUCCESS(rv, nsnull);
+  rv = aRIL->RegisterTelephonyCallback(telephony->mRILTelephonyCallback);
+  NS_ENSURE_SUCCESS(rv, nullptr);
 
   return telephony.forget();
 }
@@ -205,6 +165,10 @@ Telephony::NotifyCallsChanged(TelephonyCall* aCall)
   nsRefPtr<CallEvent> event = CallEvent::Create(aCall);
   NS_ASSERTION(event, "This should never fail!");
 
+  if (aCall->CallState() == nsIRadioInterfaceLayer::CALL_STATE_DIALING) {
+    mActiveCall = aCall;
+  }
+
   nsresult rv =
     event->Dispatch(ToIDOMEventTarget(), NS_LITERAL_STRING("callschanged"));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -212,6 +176,47 @@ Telephony::NotifyCallsChanged(TelephonyCall* aCall)
   return NS_OK;
 }
 
+nsresult
+Telephony::DialInternal(bool isEmergency,
+                        const nsAString& aNumber,
+                        nsIDOMTelephonyCall** aResult)
+{
+  NS_ENSURE_ARG(!aNumber.IsEmpty());
+
+  for (uint32_t index = 0; index < mCalls.Length(); index++) {
+    const nsRefPtr<TelephonyCall>& tempCall = mCalls[index];
+    if (tempCall->IsOutgoing() &&
+        tempCall->CallState() < nsIRadioInterfaceLayer::CALL_STATE_CONNECTED) {
+      // One call has been dialed already and we only support one outgoing call
+      // at a time.
+      NS_WARNING("Only permitted to dial one call at a time!");
+      return NS_ERROR_NOT_AVAILABLE;
+    }
+  }
+
+  nsresult rv;
+  if (isEmergency) {
+    rv = mRIL->DialEmergency(aNumber);
+  } else {
+    rv = mRIL->Dial(aNumber);
+  }
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsRefPtr<TelephonyCall> call = CreateNewDialingCall(aNumber);
+
+  // Notify other telephony objects that we just dialed.
+  for (uint32_t index = 0; index < gTelephonyList->Length(); index++) {
+    Telephony*& telephony = gTelephonyList->ElementAt(index);
+    if (telephony != this) {
+      nsRefPtr<Telephony> kungFuDeathGrip = telephony;
+      telephony->NoteDialedCallFromOtherInstance(aNumber);
+    }
+  }
+
+  call.forget(aResult);
+  return NS_OK;
+}
+
 NS_IMPL_CYCLE_COLLECTION_CLASS(Telephony)
 
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN_INHERITED(Telephony,
@@ -219,7 +224,7 @@ NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN_INHERITED(Telephony,
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE_SCRIPT_OBJECTS
   NS_CYCLE_COLLECTION_TRAVERSE_EVENT_HANDLER(incoming)
   NS_CYCLE_COLLECTION_TRAVERSE_EVENT_HANDLER(callschanged)
-  for (PRUint32 index = 0; index < tmp->mCalls.Length(); index++) {
+  for (uint32_t index = 0; index < tmp->mCalls.Length(); index++) {
     NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(cb, "mCalls[i]");
     cb.NoteXPCOMChild(tmp->mCalls[index]->ToISupports());
   }
@@ -227,7 +232,7 @@ NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 NS_IMPL_CYCLE_COLLECTION_TRACE_BEGIN_INHERITED(Telephony,
                                                nsDOMEventTargetHelper)
-  NS_IMPL_CYCLE_COLLECTION_TRACE_JS_CALLBACK(tmp->mCallsArray, "mCallsArray")
+  NS_IMPL_CYCLE_COLLECTION_TRACE_JS_MEMBER_CALLBACK(mCallsArray)
 NS_IMPL_CYCLE_COLLECTION_TRACE_END
 
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN_INHERITED(Telephony,
@@ -235,8 +240,8 @@ NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN_INHERITED(Telephony,
   NS_CYCLE_COLLECTION_UNLINK_EVENT_HANDLER(incoming)
   NS_CYCLE_COLLECTION_UNLINK_EVENT_HANDLER(callschanged)
   tmp->mCalls.Clear();
-  tmp->mActiveCall = nsnull;
-  tmp->mCallsArray = nsnull;
+  tmp->mActiveCall = nullptr;
+  tmp->mCallsArray = nullptr;
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION_INHERITED(Telephony)
@@ -254,34 +259,16 @@ NS_IMPL_ISUPPORTS1(Telephony::RILTelephonyCallback, nsIRILTelephonyCallback)
 NS_IMETHODIMP
 Telephony::Dial(const nsAString& aNumber, nsIDOMTelephonyCall** aResult)
 {
-  NS_ENSURE_ARG(!aNumber.IsEmpty());
+  DialInternal(false, aNumber, aResult);
 
-  for (PRUint32 index = 0; index < mCalls.Length(); index++) {
-    const nsRefPtr<TelephonyCall>& tempCall = mCalls[index];
-    if (tempCall->IsOutgoing() &&
-        tempCall->CallState() < nsIRadioInterfaceLayer::CALL_STATE_CONNECTED) {
-      // One call has been dialed already and we only support one outgoing call
-      // at a time.
-      NS_WARNING("Only permitted to dial one call at a time!");
-      return NS_ERROR_NOT_AVAILABLE;
-    }
-  }
+  return NS_OK;
+}
 
-  nsresult rv = mRIL->Dial(aNumber);
-  NS_ENSURE_SUCCESS(rv, rv);
+NS_IMETHODIMP
+Telephony::DialEmergency(const nsAString& aNumber, nsIDOMTelephonyCall** aResult)
+{
+  DialInternal(true, aNumber, aResult);
 
-  nsRefPtr<TelephonyCall> call = CreateNewDialingCall(aNumber);
-
-  // Notify other telephony objects that we just dialed.
-  for (PRUint32 index = 0; index < gTelephonyList->Length(); index++) {
-    Telephony*& telephony = gTelephonyList->ElementAt(index);
-    if (telephony != this) {
-      nsRefPtr<Telephony> kungFuDeathGrip = telephony;
-      telephony->NoteDialedCallFromOtherInstance(aNumber);
-    }
-  }
-
-  call.forget(aResult);
   return NS_OK;
 }
 
@@ -402,8 +389,8 @@ NS_IMPL_EVENT_HANDLER(Telephony, incoming)
 NS_IMPL_EVENT_HANDLER(Telephony, callschanged)
 
 NS_IMETHODIMP
-Telephony::CallStateChanged(PRUint32 aCallIndex, PRUint16 aCallState,
-                            const nsAString& aNumber)
+Telephony::CallStateChanged(uint32_t aCallIndex, uint16_t aCallState,
+                            const nsAString& aNumber, bool aIsActive)
 {
   NS_ASSERTION(aCallIndex != kOutgoingPlaceholderCallIndex,
                "This should never happen!");
@@ -411,7 +398,7 @@ Telephony::CallStateChanged(PRUint32 aCallIndex, PRUint16 aCallState,
   nsRefPtr<TelephonyCall> modifiedCall;
   nsRefPtr<TelephonyCall> outgoingCall;
 
-  for (PRUint32 index = 0; index < mCalls.Length(); index++) {
+  for (uint32_t index = 0; index < mCalls.Length(); index++) {
     nsRefPtr<TelephonyCall>& tempCall = mCalls[index];
     if (tempCall->CallIndex() == kOutgoingPlaceholderCallIndex) {
       NS_ASSERTION(!outgoingCall, "More than one outgoing call not supported!");
@@ -424,7 +411,7 @@ Telephony::CallStateChanged(PRUint32 aCallIndex, PRUint16 aCallState,
     } else if (tempCall->CallIndex() == aCallIndex) {
       // We already know about this call so just update its state.
       modifiedCall = tempCall;
-      outgoingCall = nsnull;
+      outgoingCall = nullptr;
       break;
     }
   }
@@ -440,14 +427,23 @@ Telephony::CallStateChanged(PRUint32 aCallIndex, PRUint16 aCallState,
   }
 
   if (modifiedCall) {
-    // Change state.
-    modifiedCall->ChangeState(aCallState);
 
     // See if this should replace our current active call.
-    if (aCallState == nsIRadioInterfaceLayer::CALL_STATE_CONNECTED) {
-      mActiveCall = modifiedCall;
+    if (aIsActive) {
+      if (aCallState == nsIRadioInterfaceLayer::CALL_STATE_DISCONNECTED) {
+        mActiveCall = nullptr;
+      } else {
+        mActiveCall = modifiedCall;
+      }
+    } else {
+      if (mActiveCall && mActiveCall->CallIndex() == aCallIndex) {
+        mActiveCall = nullptr;
+      }
     }
 
+    // Change state.
+    modifiedCall->ChangeState(aCallState);
+
     return NS_OK;
   }
 
@@ -473,13 +469,13 @@ Telephony::CallStateChanged(PRUint32 aCallIndex, PRUint16 aCallState,
 }
 
 NS_IMETHODIMP
-Telephony::EnumerateCallState(PRUint32 aCallIndex, PRUint16 aCallState,
+Telephony::EnumerateCallState(uint32_t aCallIndex, uint16_t aCallState,
                               const nsAString& aNumber, bool aIsActive,
                               bool* aContinue)
 {
 #ifdef DEBUG
   // Make sure we don't somehow add duplicates.
-  for (PRUint32 index = 0; index < mCalls.Length(); index++) {
+  for (uint32_t index = 0; index < mCalls.Length(); index++) {
     NS_ASSERTION(mCalls[index]->CallIndex() != aCallIndex,
                  "Something is really wrong here!");
   }
@@ -499,83 +495,75 @@ Telephony::EnumerateCallState(PRUint32 aCallIndex, PRUint16 aCallState,
   return NS_OK;
 }
 
+NS_IMETHODIMP
+Telephony::NotifyError(int32_t aCallIndex,
+                       const nsAString& aError)
+{
+  nsRefPtr<TelephonyCall> callToNotify;
+  if (!mCalls.IsEmpty()) {
+    // The connection is not established yet. Get the latest call object.
+    if (aCallIndex == -1) {
+      callToNotify = mCalls[mCalls.Length() - 1];
+    } else {
+      // The connection has been established. Get the failed call.
+      for (uint32_t index = 0; index < mCalls.Length(); index++) {
+        nsRefPtr<TelephonyCall>& call = mCalls[index];
+        if (call->CallIndex() == aCallIndex) {
+          callToNotify = call;
+          break;
+        }
+      }
+    }
+  }
+
+  if (!callToNotify) {
+    NS_ERROR("Don't call me with a bad call index!");
+    return NS_ERROR_UNEXPECTED;
+  }
+
+  if (mActiveCall && mActiveCall->CallIndex() == callToNotify->CallIndex()) {
+    mActiveCall = nullptr;
+  }
+
+  // Set the call state to 'disconnected' and remove it from the calls list.
+  callToNotify->NotifyError(aError);
+
+  return NS_OK;
+}
+
 nsresult
 NS_NewTelephony(nsPIDOMWindow* aWindow, nsIDOMTelephony** aTelephony)
 {
   NS_ASSERTION(aWindow, "Null pointer!");
 
-  // Make sure we're dealing with an inner window.
   nsPIDOMWindow* innerWindow = aWindow->IsInnerWindow() ?
-                               aWindow :
-                               aWindow->GetCurrentInnerWindow();
-  NS_ENSURE_TRUE(innerWindow, NS_ERROR_FAILURE);
+    aWindow :
+    aWindow->GetCurrentInnerWindow();
 
-  // Make sure we're being called from a window that we have permission to
-  // access.
-  if (!nsContentUtils::CanCallerAccess(innerWindow)) {
-    return NS_ERROR_DOM_SECURITY_ERR;
-  }
-
-  // Need the document in order to make security decisions.
+  // Need the document for security check.
   nsCOMPtr<nsIDocument> document =
     do_QueryInterface(innerWindow->GetExtantDocument());
   NS_ENSURE_TRUE(document, NS_NOINTERFACE);
 
-  // Do security checks. We assume that chrome is always allowed and we also
-  // allow a single page specified by preferences.
-  if (!nsContentUtils::IsSystemPrincipal(document->NodePrincipal())) {
-    nsCOMPtr<nsIURI> originalURI;
-    nsresult rv =
-      document->NodePrincipal()->GetURI(getter_AddRefs(originalURI));
-    NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsIPrincipal> principal = document->NodePrincipal();
+  NS_ENSURE_TRUE(principal, NS_ERROR_UNEXPECTED);
 
-    nsCOMPtr<nsIURI> documentURI;
-    rv = originalURI->Clone(getter_AddRefs(documentURI));
-    NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsIPermissionManager> permMgr =
+    do_GetService(NS_PERMISSIONMANAGER_CONTRACTID);
+  NS_ENSURE_TRUE(permMgr, NS_ERROR_UNEXPECTED);
 
-    // Strip the query string (if there is one) before comparing.
-    nsCOMPtr<nsIURL> documentURL = do_QueryInterface(documentURI);
-    if (documentURL) {
-      rv = documentURL->SetQuery(EmptyCString());
-      NS_ENSURE_SUCCESS(rv, rv);
-    }
+  uint32_t permission;
+  nsresult rv =
+    permMgr->TestPermissionFromPrincipal(principal, "telephony", &permission);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-    bool allowed = false;
-
-    // The pref may not exist but in that case we deny access just as we do if
-    // the url doesn't match.
-    nsCString whitelist;
-    if (NS_SUCCEEDED(Preferences::GetCString(DOM_TELEPHONY_APP_PHONE_URL_PREF,
-                                             &whitelist))) {
-      nsCOMPtr<nsIIOService> ios = do_GetIOService();
-      NS_ENSURE_TRUE(ios, NS_ERROR_FAILURE);
-
-      nsCCharSeparatedTokenizer tokenizer(whitelist, ',');
-      while (tokenizer.hasMoreTokens()) {
-        nsCOMPtr<nsIURI> uri;
-        if (NS_SUCCEEDED(NS_NewURI(getter_AddRefs(uri), tokenizer.nextToken(),
-                                   nsnull, nsnull, ios))) {
-          rv = documentURI->EqualsExceptRef(uri, &allowed);
-          NS_ENSURE_SUCCESS(rv, rv);
-
-          if (allowed) {
-            break;
-          }
-        }
-      }
-    }
-
-    if (!allowed) {
-      *aTelephony = nsnull;
-      return NS_OK;
-    }
+  if (permission != nsIPermissionManager::ALLOW_ACTION) {
+    *aTelephony = nullptr;
+    return NS_OK;
   }
 
-  // Security checks passed, make a telephony object.
-  nsIInterfaceRequestor* ireq = SystemWorkerManager::GetInterfaceRequestor();
-  NS_ENSURE_TRUE(ireq, NS_ERROR_UNEXPECTED);
-
-  nsCOMPtr<nsIRadioInterfaceLayer> ril = do_GetInterface(ireq);
+  nsCOMPtr<nsIRILContentHelper> ril =
+    do_GetService(NS_RILCONTENTHELPER_CONTRACTID);
   NS_ENSURE_TRUE(ril, NS_ERROR_UNEXPECTED);
 
   nsRefPtr<Telephony> telephony = Telephony::Create(innerWindow, ril);
