@@ -1,41 +1,8 @@
 /* -*- Mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; tab-width: 40 -*- */
 /* vim: set ts=2 et sw=2 tw=80: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Web Workers.
- *
- * The Initial Developer of the Original Code is
- *   The Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2011
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Ben Turner <bent.mozilla@gmail.com> (Original Author)
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "mozilla/Util.h"
 
@@ -253,7 +220,7 @@ private:
   }
 
   static JSBool
-  GetProperty(JSContext* aCx, JSObject* aObj, jsid aIdval, jsval* aVp)
+  GetProperty(JSContext* aCx, JSHandleObject aObj, JSHandleId aIdval, JSMutableHandleValue aVp)
   {
     JS_ASSERT(JSID_IS_INT(aIdval));
 
@@ -264,18 +231,18 @@ private:
       return false;
     }
 
-    *aVp = JS_GetReservedSlot(aObj, slot);
+    aVp.set(JS_GetReservedSlot(aObj, slot));
     return true;
   }
 
   static JSBool
-  GetConstant(JSContext* aCx, JSObject* aObj, jsid idval, jsval* aVp)
+  GetConstant(JSContext* aCx, JSHandleObject aObj, JSHandleId idval, JSMutableHandleValue aVp)
   {
     JS_ASSERT(JSID_IS_INT(idval));
     JS_ASSERT(JSID_TO_INT(idval) >= CAPTURING_PHASE &&
               JSID_TO_INT(idval) <= BUBBLING_PHASE);
 
-    *aVp = INT_TO_JSVAL(JSID_TO_INT(idval));
+    aVp.set(INT_TO_JSVAL(JSID_TO_INT(idval)));
     return true;
   }
 
@@ -374,24 +341,25 @@ DECL_EVENT_CLASS(Event::sMainRuntimeClass, "WorkerEvent")
 #undef DECL_EVENT_CLASS
 
 JSPropertySpec Event::sProperties[] = {
-  { "type", SLOT_type, PROPERTY_FLAGS, GetProperty, js_GetterOnlyPropertyStub },
-  { "target", SLOT_target, PROPERTY_FLAGS, GetProperty,
-    js_GetterOnlyPropertyStub },
-  { "currentTarget", SLOT_currentTarget, PROPERTY_FLAGS, GetProperty,
-    js_GetterOnlyPropertyStub },
-  { "eventPhase", SLOT_eventPhase, PROPERTY_FLAGS, GetProperty,
-    js_GetterOnlyPropertyStub },
-  { "bubbles", SLOT_bubbles, PROPERTY_FLAGS, GetProperty,
-    js_GetterOnlyPropertyStub },
-  { "cancelable", SLOT_cancelable, PROPERTY_FLAGS, GetProperty,
-    js_GetterOnlyPropertyStub },
-  { "timeStamp", SLOT_timeStamp, PROPERTY_FLAGS, GetProperty,
-    js_GetterOnlyPropertyStub },
-  { "defaultPrevented", SLOT_defaultPrevented, PROPERTY_FLAGS, GetProperty,
-    js_GetterOnlyPropertyStub },
-  { "isTrusted", SLOT_isTrusted, PROPERTY_FLAGS, GetProperty,
-    js_GetterOnlyPropertyStub },
-  { 0, 0, 0, NULL, NULL }
+  { "type", SLOT_type, PROPERTY_FLAGS, JSOP_WRAPPER(GetProperty),
+    JSOP_WRAPPER(js_GetterOnlyPropertyStub) },
+  { "target", SLOT_target, PROPERTY_FLAGS, JSOP_WRAPPER(GetProperty),
+    JSOP_WRAPPER(js_GetterOnlyPropertyStub) },
+  { "currentTarget", SLOT_currentTarget, PROPERTY_FLAGS, JSOP_WRAPPER(GetProperty),
+    JSOP_WRAPPER(js_GetterOnlyPropertyStub) },
+  { "eventPhase", SLOT_eventPhase, PROPERTY_FLAGS, JSOP_WRAPPER(GetProperty),
+    JSOP_WRAPPER(js_GetterOnlyPropertyStub) },
+  { "bubbles", SLOT_bubbles, PROPERTY_FLAGS, JSOP_WRAPPER(GetProperty),
+    JSOP_WRAPPER(js_GetterOnlyPropertyStub) },
+  { "cancelable", SLOT_cancelable, PROPERTY_FLAGS, JSOP_WRAPPER(GetProperty),
+    JSOP_WRAPPER(js_GetterOnlyPropertyStub) },
+  { "timeStamp", SLOT_timeStamp, PROPERTY_FLAGS, JSOP_WRAPPER(GetProperty),
+    JSOP_WRAPPER(js_GetterOnlyPropertyStub) },
+  { "defaultPrevented", SLOT_defaultPrevented, PROPERTY_FLAGS, JSOP_WRAPPER(GetProperty),
+    JSOP_WRAPPER(js_GetterOnlyPropertyStub) },
+  { "isTrusted", SLOT_isTrusted, PROPERTY_FLAGS, JSOP_WRAPPER(GetProperty),
+    JSOP_WRAPPER(js_GetterOnlyPropertyStub) },
+  { 0, 0, 0, JSOP_NULLWRAPPER, JSOP_NULLWRAPPER }
 };
 
 JSFunctionSpec Event::sFunctions[] = {
@@ -403,10 +371,12 @@ JSFunctionSpec Event::sFunctions[] = {
 };
 
 JSPropertySpec Event::sStaticProperties[] = {
-  { "CAPTURING_PHASE", CAPTURING_PHASE, CONSTANT_FLAGS, GetConstant, NULL },
-  { "AT_TARGET", AT_TARGET, CONSTANT_FLAGS, GetConstant, NULL },
-  { "BUBBLING_PHASE", BUBBLING_PHASE, CONSTANT_FLAGS, GetConstant, NULL },
-  { 0, 0, 0, NULL, NULL }
+  { "CAPTURING_PHASE", CAPTURING_PHASE, CONSTANT_FLAGS,
+    JSOP_WRAPPER(GetConstant), JSOP_NULLWRAPPER },
+  { "AT_TARGET", AT_TARGET, CONSTANT_FLAGS, JSOP_WRAPPER(GetConstant), JSOP_NULLWRAPPER },
+  { "BUBBLING_PHASE", BUBBLING_PHASE, CONSTANT_FLAGS,
+    JSOP_WRAPPER(GetConstant), JSOP_NULLWRAPPER },
+  { 0, 0, 0, JSOP_NULLWRAPPER, JSOP_NULLWRAPPER }
 };
 
 class MessageEvent : public Event
@@ -543,7 +513,7 @@ private:
   }
 
   static JSBool
-  GetProperty(JSContext* aCx, JSObject* aObj, jsid aIdval, jsval* aVp)
+  GetProperty(JSContext* aCx, JSHandleObject aObj, JSHandleId aIdval, JSMutableHandleValue aVp)
   {
     JS_ASSERT(JSID_IS_INT(aIdval));
 
@@ -577,11 +547,11 @@ private:
       }
       JS_SetReservedSlot(aObj, slot, data);
 
-      *aVp = data;
+      aVp.set(data);
       return true;
     }
 
-    *aVp = JS_GetReservedSlot(aObj, slot);
+    aVp.set(JS_GetReservedSlot(aObj, slot));
     return true;
   }
 
@@ -626,12 +596,13 @@ DECL_MESSAGEEVENT_CLASS(MessageEvent::sMainRuntimeClass, "WorkerMessageEvent")
 #undef DECL_MESSAGEEVENT_CLASS
 
 JSPropertySpec MessageEvent::sProperties[] = {
-  { "data", SLOT_data, PROPERTY_FLAGS, GetProperty, js_GetterOnlyPropertyStub },
-  { "origin", SLOT_origin, PROPERTY_FLAGS, GetProperty,
-    js_GetterOnlyPropertyStub },
-  { "source", SLOT_source, PROPERTY_FLAGS, GetProperty,
-    js_GetterOnlyPropertyStub },
-  { 0, 0, 0, NULL, NULL }
+  { "data", SLOT_data, PROPERTY_FLAGS, JSOP_WRAPPER(GetProperty),
+    JSOP_WRAPPER(js_GetterOnlyPropertyStub) },
+  { "origin", SLOT_origin, PROPERTY_FLAGS, JSOP_WRAPPER(GetProperty),
+    JSOP_WRAPPER(js_GetterOnlyPropertyStub) },
+  { "source", SLOT_source, PROPERTY_FLAGS, JSOP_WRAPPER(GetProperty),
+    JSOP_WRAPPER(js_GetterOnlyPropertyStub) },
+  { 0, 0, 0, JSOP_NULLWRAPPER, JSOP_NULLWRAPPER }
 };
 
 JSFunctionSpec MessageEvent::sFunctions[] = {
@@ -751,7 +722,7 @@ private:
   }
 
   static JSBool
-  GetProperty(JSContext* aCx, JSObject* aObj, jsid aIdval, jsval* aVp)
+  GetProperty(JSContext* aCx, JSHandleObject aObj, JSHandleId aIdval, JSMutableHandleValue aVp)
   {
     JS_ASSERT(JSID_IS_INT(aIdval));
 
@@ -765,7 +736,7 @@ private:
       return false;
     }
 
-    *aVp = JS_GetReservedSlot(aObj, slot);
+    aVp.set(JS_GetReservedSlot(aObj, slot));
     return true;
   }
 
@@ -811,13 +782,13 @@ DECL_ERROREVENT_CLASS(ErrorEvent::sMainRuntimeClass, "WorkerErrorEvent")
 #undef DECL_ERROREVENT_CLASS
 
 JSPropertySpec ErrorEvent::sProperties[] = {
-  { "message", SLOT_message, PROPERTY_FLAGS, GetProperty,
-    js_GetterOnlyPropertyStub },
-  { "filename", SLOT_filename, PROPERTY_FLAGS, GetProperty,
-    js_GetterOnlyPropertyStub },
-  { "lineno", SLOT_lineno, PROPERTY_FLAGS, GetProperty,
-    js_GetterOnlyPropertyStub },
-  { 0, 0, 0, NULL, NULL }
+  { "message", SLOT_message, PROPERTY_FLAGS, JSOP_WRAPPER(GetProperty),
+    JSOP_WRAPPER(js_GetterOnlyPropertyStub) },
+  { "filename", SLOT_filename, PROPERTY_FLAGS, JSOP_WRAPPER(GetProperty),
+    JSOP_WRAPPER(js_GetterOnlyPropertyStub) },
+  { "lineno", SLOT_lineno, PROPERTY_FLAGS, JSOP_WRAPPER(GetProperty),
+    JSOP_WRAPPER(js_GetterOnlyPropertyStub) },
+  { 0, 0, 0, JSOP_NULLWRAPPER, JSOP_NULLWRAPPER }
 };
 
 JSFunctionSpec ErrorEvent::sFunctions[] = {
@@ -931,7 +902,7 @@ private:
   }
 
   static JSBool
-  GetProperty(JSContext* aCx, JSObject* aObj, jsid aIdval, jsval* aVp)
+  GetProperty(JSContext* aCx, JSHandleObject aObj, JSHandleId aIdval, JSMutableHandleValue aVp)
   {
     JS_ASSERT(JSID_IS_INT(aIdval));
 
@@ -945,7 +916,7 @@ private:
       return false;
     }
 
-    *aVp = JS_GetReservedSlot(aObj, slot);
+    aVp.set(JS_GetReservedSlot(aObj, slot));
     return true;
   }
 
@@ -985,13 +956,13 @@ JSClass ProgressEvent::sClass = {
 };
 
 JSPropertySpec ProgressEvent::sProperties[] = {
-  { "lengthComputable", SLOT_lengthComputable, PROPERTY_FLAGS, GetProperty,
-    js_GetterOnlyPropertyStub },
-  { "loaded", SLOT_loaded, PROPERTY_FLAGS, GetProperty, 
-    js_GetterOnlyPropertyStub },
-  { "total", SLOT_total, PROPERTY_FLAGS, GetProperty, 
-    js_GetterOnlyPropertyStub },
-  { 0, 0, 0, NULL, NULL }
+  { "lengthComputable", SLOT_lengthComputable, PROPERTY_FLAGS,
+    JSOP_WRAPPER(GetProperty), JSOP_WRAPPER(js_GetterOnlyPropertyStub) },
+  { "loaded", SLOT_loaded, PROPERTY_FLAGS, JSOP_WRAPPER(GetProperty),
+    JSOP_WRAPPER(js_GetterOnlyPropertyStub) },
+  { "total", SLOT_total, PROPERTY_FLAGS, JSOP_WRAPPER(GetProperty),
+    JSOP_WRAPPER(js_GetterOnlyPropertyStub) },
+  { 0, 0, 0, JSOP_NULLWRAPPER, JSOP_NULLWRAPPER }
 };
 
 JSFunctionSpec ProgressEvent::sFunctions[] = {
