@@ -1,40 +1,7 @@
 /* -*- Mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; tab-width: 40 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Web Workers.
- *
- * The Initial Developer of the Original Code is
- *   The Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2011
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Ben Turner <bent.mozilla@gmail.com> (Original Author)
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifndef mozilla_dom_workers_workerprivate_h__
 #define mozilla_dom_workers_workerprivate_h__
@@ -64,7 +31,6 @@
 class JSAutoStructuredCloneBuffer;
 class nsIDocument;
 class nsIPrincipal;
-class nsIMemoryMultiReporter;
 class nsIScriptContext;
 class nsIURI;
 class nsPIDOMWindow;
@@ -73,6 +39,7 @@ class nsIXPCScriptNotify;
 
 BEGIN_WORKERS_NAMESPACE
 
+class WorkerMemoryReporter;
 class WorkerPrivate;
 
 class WorkerRunnable : public nsIRunnable
@@ -143,13 +110,13 @@ private:
 class WorkerSyncRunnable : public WorkerRunnable
 {
 protected:
-  PRUint32 mSyncQueueKey;
+  uint32_t mSyncQueueKey;
   bool mBypassSyncQueue;
 
 protected:
   friend class WorkerPrivate;
 
-  WorkerSyncRunnable(WorkerPrivate* aWorkerPrivate, PRUint32 aSyncQueueKey,
+  WorkerSyncRunnable(WorkerPrivate* aWorkerPrivate, uint32_t aSyncQueueKey,
                      bool aBypassSyncQueue = false,
                      ClearingBehavior aClearingBehavior = SkipWhenClearing)
   : WorkerRunnable(aWorkerPrivate, WorkerThread, UnchangedBusyCount,
@@ -219,11 +186,11 @@ private:
   // Only used for top level workers.
   nsTArray<nsRefPtr<WorkerRunnable> > mQueuedRunnables;
 
-  PRUint64 mBusyCount;
+  uint64_t mBusyCount;
   Status mParentStatus;
-  PRUint32 mJSContextOptions;
-  PRUint32 mJSRuntimeHeapSize;
-  PRUint8 mGCZeal;
+  uint32_t mJSContextOptions;
+  uint32_t mJSRuntimeHeapSize;
+  uint8_t mGCZeal;
   bool mJSObjectRooted;
   bool mParentSuspended;
   bool mIsChromeWorker;
@@ -291,10 +258,10 @@ public:
   Resume(JSContext* aCx);
 
   virtual void
-  _Trace(JSTracer* aTrc) MOZ_OVERRIDE;
+  _trace(JSTracer* aTrc) MOZ_OVERRIDE;
 
   virtual void
-  _Finalize(JSFreeOp* aFop) MOZ_OVERRIDE;
+  _finalize(JSFreeOp* aFop) MOZ_OVERRIDE;
 
   void
   Finish(JSContext* aCx)
@@ -325,18 +292,18 @@ public:
   bool
   PostMessage(JSContext* aCx, jsval aMessage);
 
-  PRUint64
+  uint64_t
   GetInnerWindowId();
 
   void
-  UpdateJSContextOptions(JSContext* aCx, PRUint32 aOptions);
+  UpdateJSContextOptions(JSContext* aCx, uint32_t aOptions);
 
   void
-  UpdateJSRuntimeHeapSize(JSContext* aCx, PRUint32 aJSRuntimeHeapSize);
+  UpdateJSRuntimeHeapSize(JSContext* aCx, uint32_t aJSRuntimeHeapSize);
 
 #ifdef JS_GC_ZEAL
   void
-  UpdateGCZeal(JSContext* aCx, PRUint8 aGCZeal);
+  UpdateGCZeal(JSContext* aCx, uint8_t aGCZeal);
 #endif
 
   void
@@ -482,20 +449,20 @@ public:
     return mLocationInfo;
   }
 
-  PRUint32
+  uint32_t
   GetJSContextOptions() const
   {
     return mJSContextOptions;
   }
 
-  PRUint32
+  uint32_t
   GetJSRuntimeHeapSize() const
   {
     return mJSRuntimeHeapSize;
   }
 
 #ifdef JS_GC_ZEAL
-  PRUint8
+  uint8_t
   GetGCZeal() const
   {
     return mGCZeal;
@@ -528,6 +495,7 @@ public:
 class WorkerPrivate : public WorkerPrivateParent<WorkerPrivate>
 {
   friend class WorkerPrivateParent<WorkerPrivate>;
+  friend class WorkerMemoryReporter;
   typedef WorkerPrivateParent<WorkerPrivate> ParentType;
 
   struct TimeoutInfo;
@@ -567,11 +535,11 @@ class WorkerPrivate : public WorkerPrivateParent<WorkerPrivate>
   nsTArray<nsAutoPtr<TimeoutInfo> > mTimeouts;
 
   nsCOMPtr<nsITimer> mTimer;
-  nsCOMPtr<nsIMemoryMultiReporter> mMemoryReporter;
+  nsRefPtr<WorkerMemoryReporter> mMemoryReporter;
 
   mozilla::TimeStamp mKillTime;
-  PRUint32 mErrorHandlerRecursionCount;
-  PRUint32 mNextTimeoutId;
+  uint32_t mErrorHandlerRecursionCount;
+  uint32_t mNextTimeoutId;
   Status mStatus;
   bool mSuspended;
   bool mTimerRunning;
@@ -579,7 +547,6 @@ class WorkerPrivate : public WorkerPrivateParent<WorkerPrivate>
   bool mCloseHandlerStarted;
   bool mCloseHandlerFinished;
   bool mMemoryReporterRunning;
-  bool mMemoryReporterDisabled;
 
 #ifdef DEBUG
   nsCOMPtr<nsIThread> mThread;
@@ -661,14 +628,14 @@ public:
              mFeatures.IsEmpty());
   }
 
-  PRUint32
+  uint32_t
   CreateNewSyncLoop();
 
   bool
-  RunSyncLoop(JSContext* aCx, PRUint32 aSyncLoopKey);
+  RunSyncLoop(JSContext* aCx, uint32_t aSyncLoopKey);
 
   void
-  StopSyncLoop(PRUint32 aSyncLoopKey, bool aSyncResult);
+  StopSyncLoop(uint32_t aSyncLoopKey, bool aSyncResult);
 
   bool
   PostMessageToParent(JSContext* aCx, jsval aMessage);
@@ -706,23 +673,23 @@ public:
   }
 
   void
-  UpdateJSContextOptionsInternal(JSContext* aCx, PRUint32 aOptions);
+  UpdateJSContextOptionsInternal(JSContext* aCx, uint32_t aOptions);
 
   void
-  UpdateJSRuntimeHeapSizeInternal(JSContext* aCx, PRUint32 aJSRuntimeHeapSize);
+  UpdateJSRuntimeHeapSizeInternal(JSContext* aCx, uint32_t aJSRuntimeHeapSize);
 
   void
   ScheduleDeletion(bool aWasPending);
 
   bool
-  BlockAndCollectRuntimeStats(bool isQuick, void* aData, bool* aDisabled);
+  BlockAndCollectRuntimeStats(bool isQuick, void* aData);
 
   bool
   DisableMemoryReporter();
 
 #ifdef JS_GC_ZEAL
   void
-  UpdateGCZealInternal(JSContext* aCx, PRUint8 aGCZeal);
+  UpdateGCZealInternal(JSContext* aCx, uint8_t aGCZeal);
 #endif
 
   void
@@ -792,7 +759,7 @@ private:
     return true;
   }
 
-  PRUint32
+  uint32_t
   RemainingRunTimeMS() const;
 
   void
@@ -809,7 +776,7 @@ private:
     mozilla::MutexAutoLock lock(mMutex);
 
     mStatus = Dead;
-    mJSContext = nsnull;
+    mJSContext = nullptr;
 
     ClearQueue(&mControlQueue);
     ClearQueue(&mQueue);
