@@ -1,43 +1,7 @@
 /* vim:set ts=4 sw=4 sts=4 et cindent: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is the Negotiateauth
- *
- * The Initial Developer of the Original Code is Daniel Kouril.
- * Portions created by the Initial Developer are Copyright (C) 2003
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Daniel Kouril <kouril@ics.muni.cz> (original author)
- *   Wyllys Ingersoll <wyllys.ingersoll@sun.com>
- *   Christopher Nebergall <cneberg@sandia.gov>
- *   Darin Fisher <darin@meer.net>
- *   Mark Mentovai <mark@moxienet.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //
 // GSSAPI Authentication Support Module
@@ -111,7 +75,7 @@ static struct GSSFunction {
 };
 
 static bool      gssNativeImp = true;
-static PRLibrary* gssLibrary = nsnull;
+static PRLibrary* gssLibrary = nullptr;
 
 #define gss_display_status_ptr      ((gss_display_status_type)*gssFuncs[0].func)
 #define gss_init_sec_context_ptr    ((gss_init_sec_context_type)*gssFuncs[1].func)
@@ -362,7 +326,7 @@ nsAuthGSSAPI::Shutdown()
 {
     if (gssLibrary) {
         PR_UnloadLibrary(gssLibrary);
-        gssLibrary = nsnull;
+        gssLibrary = nullptr;
     }
 }
 
@@ -371,7 +335,7 @@ NS_IMPL_THREADSAFE_ISUPPORTS1(nsAuthGSSAPI, nsIAuthModule)
 
 NS_IMETHODIMP
 nsAuthGSSAPI::Init(const char *serviceName,
-                   PRUint32    serviceFlags,
+                   uint32_t    serviceFlags,
                    const PRUnichar *domain,
                    const PRUnichar *username,
                    const PRUnichar *password)
@@ -394,9 +358,9 @@ nsAuthGSSAPI::Init(const char *serviceName,
 
 NS_IMETHODIMP
 nsAuthGSSAPI::GetNextToken(const void *inToken,
-                           PRUint32    inTokenLen,
+                           uint32_t    inTokenLen,
                            void      **outToken,
-                           PRUint32   *outTokenLen)
+                           uint32_t   *outTokenLen)
 {
     OM_uint32 major_status, minor_status;
     OM_uint32 req_flags = 0;
@@ -481,10 +445,10 @@ nsAuthGSSAPI::GetNextToken(const void *inToken,
                                             GSS_C_INDEFINITE,
                                             GSS_C_NO_CHANNEL_BINDINGS,
                                             in_token_ptr,
-                                            nsnull,
+                                            nullptr,
                                             &output_token,
-                                            nsnull,
-                                            nsnull);
+                                            nullptr,
+                                            nullptr);
 
     if (GSS_ERROR(major_status)) {
         LogGssError(major_status, minor_status, "gss_init_sec_context() failed");
@@ -527,9 +491,9 @@ end:
 
 NS_IMETHODIMP
 nsAuthGSSAPI::Unwrap(const void *inToken,
-                     PRUint32    inTokenLen,
+                     uint32_t    inTokenLen,
                      void      **outToken,
-                     PRUint32   *outTokenLen)
+                     uint32_t   *outTokenLen)
 {
     OM_uint32 major_status, minor_status;
 
@@ -566,10 +530,10 @@ nsAuthGSSAPI::Unwrap(const void *inToken,
  
 NS_IMETHODIMP
 nsAuthGSSAPI::Wrap(const void *inToken,
-                   PRUint32    inTokenLen,
+                   uint32_t    inTokenLen,
                    bool        confidential,
                    void      **outToken,
-                   PRUint32   *outTokenLen)
+                   uint32_t   *outTokenLen)
 {
     OM_uint32 major_status, minor_status;
 
