@@ -59,7 +59,6 @@ NVIDIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #endif
 
 #include "compiler/preprocessor/slglobals.h"
-#include "compiler/preprocessor/lexer_glue.h"
 #include "compiler/util.h"
 
 typedef struct StringInputSrc {
@@ -76,7 +75,7 @@ static int eof_scan(InputSrc *is, yystypepp * yylvalpp)
 
 static void noop(InputSrc *in, int ch, yystypepp * yylvalpp) {}
 
-static InputSrc eof_inputsrc = { 0, &eof_scan, &eof_scan, &noop };
+static InputSrc eof_inputsrc = { 0, &eof_scan, &eof_scan, &noop, 0, 0 };
 
 static int byte_scan(InputSrc *, yystypepp * yylvalpp);
 
@@ -129,22 +128,13 @@ int FreeScanner(void)
     return (FreeCPP());
 }
 
-// Define this to 1 to use the new lexer.
-#define CPP_USE_NEW_LEXER 0
-
 int InitScannerInput(CPPStruct *cpp, int count, const char* const string[], const int length[])
 {
-#if CPP_USE_NEW_LEXER
-    InputSrc* in = LexerInputSrc(count, string, length);
-    if (!in) return 1;
-    cpp->currentInput = in;
-#else
     cpp->PaWhichStr = 0;
     cpp->PaArgv     = string;
     cpp->PaArgc     = count;
     cpp->PaStrLen   = length;
     ScanFromString(string[0]);
-#endif
     return 0;
 }
 
