@@ -1361,53 +1361,53 @@ scaled_bilinear_scanline_565_565_SRC (uint16_t *       dst,
 #endif
 
 FAST_BILINEAR_MAINLOOP_COMMON (565_565_cover_SRC,
-			       scaled_bilinear_scanline_565_565_SRC,
+			       scaled_bilinear_scanline_565_565_SRC, NULL,
 			       uint16_t, uint32_t, uint16_t,
 			       COVER, FLAG_NONE)
 FAST_BILINEAR_MAINLOOP_COMMON (565_565_pad_SRC,
-			       scaled_bilinear_scanline_565_565_SRC,
+			       scaled_bilinear_scanline_565_565_SRC, NULL,
 			       uint16_t, uint32_t, uint16_t,
 			       PAD, FLAG_NONE)
 FAST_BILINEAR_MAINLOOP_COMMON (565_565_none_SRC,
-			       scaled_bilinear_scanline_565_565_SRC,
+			       scaled_bilinear_scanline_565_565_SRC, NULL,
 			       uint16_t, uint32_t, uint16_t,
 			       NONE, FLAG_NONE)
 FAST_BILINEAR_MAINLOOP_COMMON (565_565_normal_SRC,
-			       scaled_bilinear_scanline_565_565_SRC,
+			       scaled_bilinear_scanline_565_565_SRC, NULL,
 			       uint16_t, uint32_t, uint16_t,
 			       NORMAL, FLAG_NONE)
 
 FAST_BILINEAR_MAINLOOP_COMMON (8888_565_cover_OVER,
-			       scaled_bilinear_scanline_8888_565_OVER,
+			       scaled_bilinear_scanline_8888_565_OVER, NULL,
 			       uint32_t, uint32_t, uint16_t,
 			       COVER, FLAG_NONE)
 FAST_BILINEAR_MAINLOOP_COMMON (8888_565_pad_OVER,
-			       scaled_bilinear_scanline_8888_565_OVER,
+			       scaled_bilinear_scanline_8888_565_OVER, NULL,
 			       uint32_t, uint32_t, uint16_t,
 			       PAD, FLAG_NONE)
 FAST_BILINEAR_MAINLOOP_COMMON (8888_565_none_OVER,
-			       scaled_bilinear_scanline_8888_565_OVER,
+			       scaled_bilinear_scanline_8888_565_OVER, NULL,
 			       uint32_t, uint32_t, uint16_t,
 			       NONE, FLAG_NONE)
 FAST_BILINEAR_MAINLOOP_COMMON (8888_565_normal_OVER,
-			       scaled_bilinear_scanline_8888_565_OVER,
+			       scaled_bilinear_scanline_8888_565_OVER, NULL,
 			       uint32_t, uint32_t, uint16_t,
 			       NORMAL, FLAG_NONE)
 
 FAST_BILINEAR_MAINLOOP_COMMON (8888_8888_cover_OVER,
-			       scaled_bilinear_scanline_8888_8888_OVER,
+			       scaled_bilinear_scanline_8888_8888_OVER, NULL,
 			       uint32_t, uint32_t, uint32_t,
 			       COVER, FLAG_NONE)
 FAST_BILINEAR_MAINLOOP_COMMON (8888_8888_pad_OVER,
-			       scaled_bilinear_scanline_8888_8888_OVER,
+			       scaled_bilinear_scanline_8888_8888_OVER, NULL,
 			       uint32_t, uint32_t, uint32_t,
 			       PAD, FLAG_NONE)
 FAST_BILINEAR_MAINLOOP_COMMON (8888_8888_none_OVER,
-			       scaled_bilinear_scanline_8888_8888_OVER,
+			       scaled_bilinear_scanline_8888_8888_OVER, NULL,
 			       uint32_t, uint32_t, uint32_t,
 			       NONE, FLAG_NONE)
 FAST_BILINEAR_MAINLOOP_COMMON (8888_8888_normal_OVER,
-			       scaled_bilinear_scanline_8888_8888_OVER,
+			       scaled_bilinear_scanline_8888_8888_OVER, NULL,
 			       uint32_t, uint32_t, uint32_t,
 			       NORMAL, FLAG_NONE)
 
