@@ -1,43 +1,7 @@
 /* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Corporation code.
- *
- * The Initial Developer of the Original Code is Mozilla Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2009
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Stuart Parmenter <stuart@mozilla.com>
- *   Masayuki Nakano <masayuki@d-toybox.com>
- *   Mats Palmgren <mats.palmgren@bredband.net>
- *   John Daggett <jdaggett@mozilla.com>
- *   Jonathan Kew <jfkthame@gmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "mozilla/Util.h"
 
@@ -172,12 +136,12 @@ FT2FontEntry::CreateScaledFont(const gfxFontStyle *aStyle)
 FT2FontEntry::~FT2FontEntry()
 {
     // Do nothing for mFTFace here since FTFontDestroyFunc is called by cairo.
-    mFTFace = nsnull;
+    mFTFace = nullptr;
 
 #ifndef ANDROID
     if (mFontFace) {
         cairo_font_face_destroy(mFontFace);
-        mFontFace = nsnull;
+        mFontFace = nullptr;
     }
 #endif
 }
@@ -194,8 +158,8 @@ FT2FontEntry::CreateFontInstance(const gfxFontStyle *aFontStyle, bool aNeedsBold
 /* static */
 FT2FontEntry*
 FT2FontEntry::CreateFontEntry(const gfxProxyFontEntry &aProxyEntry,
-                              const PRUint8 *aFontData,
-                              PRUint32 aLength)
+                              const uint8_t *aFontData,
+                              uint32_t aLength)
 {
     // Ownership of aFontData is passed in here; the fontEntry must
     // retain it as long as the FT_Face needs it, and ensure it is
@@ -206,9 +170,18 @@ FT2FontEntry::CreateFontEntry(const gfxProxyFontEntry &aProxyEntry,
                            aFontData, aLength, 0, &face);
     if (error != FT_Err_Ok) {
         NS_Free((void*)aFontData);
-        return nsnull;
+        return nullptr;
     }
-    FT2FontEntry* fe = FT2FontEntry::CreateFontEntry(face, nsnull, 0, aFontData);
+    if (FT_Err_Ok != FT_Select_Charmap(face, FT_ENCODING_UNICODE)) {
+        FT_Done_Face(face);
+        NS_Free((void*)aFontData);
+        return nullptr;
+    }
+    // Create our FT2FontEntry, which inherits the name of the proxy
+    // as it's not guaranteed that the face has valid names (bug 737315)
+    FT2FontEntry* fe =
+        FT2FontEntry::CreateFontEntry(face, nullptr, 0, aProxyEntry.Name(),
+                                      aFontData);
     if (fe) {
         fe->mItalic = aProxyEntry.mItalic;
         fe->mWeight = aProxyEntry.mWeight;
@@ -219,7 +192,7 @@ FT2FontEntry::CreateFontEntry(const gfxProxyFontEntry &aProxyEntry,
 
 class FTUserFontData {
 public:
-    FTUserFontData(FT_Face aFace, const PRUint8* aData)
+    FTUserFontData(FT_Face aFace, const uint8_t* aData)
         : mFace(aFace), mFontData(aData)
     {
     }
@@ -234,7 +207,7 @@ public:
 
 private:
     FT_Face        mFace;
-    const PRUint8 *mFontData;
+    const uint8_t *mFontData;
 };
 
 static void
@@ -260,23 +233,13 @@ FT2FontEntry::CreateFontEntry(const FontListEntry& aFLE)
 /* static */
 FT2FontEntry*
 FT2FontEntry::CreateFontEntry(FT_Face aFace,
-                              const char* aFilename, PRUint8 aIndex,
-                              const PRUint8 *aFontData)
+                              const char* aFilename, uint8_t aIndex,
+                              const nsAString& aName,
+                              const uint8_t *aFontData)
 {
     static cairo_user_data_key_t key;
 
-    if (!aFace->family_name) {
-        FT_Done_Face(aFace);
-        return nsnull;
-    }
-    // Construct font name from family name and style name, regular fonts
-    // do not have the modifier by convention.
-    NS_ConvertUTF8toUTF16 fontName(aFace->family_name);
-    if (aFace->style_name && strcmp("Regular", aFace->style_name)) {
-        fontName.AppendLiteral(" ");
-        AppendUTF8toUTF16(aFace->style_name, fontName);
-    }
-    FT2FontEntry *fe = new FT2FontEntry(fontName);
+    FT2FontEntry *fe = new FT2FontEntry(aName);
     fe->mItalic = aFace->style_flags & FT_STYLE_FLAG_ITALIC;
     fe->mFTFace = aFace;
     int flags = gfxPlatform::GetPlatform()->FontHintingEnabled() ?
@@ -290,7 +253,7 @@ FT2FontEntry::CreateFontEntry(FT_Face aFace,
                                   userFontData, FTFontDestroyFunc);
 
     TT_OS2 *os2 = static_cast<TT_OS2*>(FT_Get_Sfnt_Table(aFace, ft_sfnt_os2));
-    PRUint16 os2weight = 0;
+    uint16_t os2weight = 0;
     if (os2 && os2->version != 0xffff) {
         // Technically, only 100 to 900 are valid, but some fonts
         // have this set wrong -- e.g. "Microsoft Logo Bold Italic" has
@@ -314,6 +277,23 @@ FT2FontEntry::CreateFontEntry(FT_Face aFace,
     return fe;
 }
 
+// construct font entry name for an installed font from names in the FT_Face,
+// and then create our FT2FontEntry
+static FT2FontEntry*
+CreateNamedFontEntry(FT_Face aFace, const char* aFilename, uint8_t aIndex)
+{
+    if (!aFace->family_name) {
+        return nullptr;
+    }
+    nsAutoString fontName;
+    AppendUTF8toUTF16(aFace->family_name, fontName);
+    if (aFace->style_name && strcmp("Regular", aFace->style_name)) {
+        fontName.AppendLiteral(" ");
+        AppendUTF8toUTF16(aFace->style_name, fontName);
+    }
+    return FT2FontEntry::CreateFontEntry(aFace, aFilename, aIndex, fontName);
+}
+
 FT2FontEntry*
 gfxFT2Font::GetFontEntry()
 {
@@ -327,13 +307,20 @@ FT2FontEntry::CairoFontFace()
 
     if (!mFontFace) {
         FT_Face face;
-        FT_New_Face(gfxToolkitPlatform::GetPlatform()->GetFTLibrary(), mFilename.get(), mFTFontIndex, &face);
+        if (FT_Err_Ok != FT_New_Face(gfxToolkitPlatform::GetPlatform()->GetFTLibrary(),
+                                     mFilename.get(), mFTFontIndex, &face)) {
+            NS_WARNING("failed to create freetype face");
+            return nullptr;
+        }
+        if (FT_Err_Ok != FT_Select_Charmap(face, FT_ENCODING_UNICODE)) {
+            NS_WARNING("failed to select Unicode charmap, text may be garbled");
+        }
         mFTFace = face;
         int flags = gfxPlatform::GetPlatform()->FontHintingEnabled() ?
                     FT_LOAD_DEFAULT :
                     (FT_LOAD_NO_AUTOHINT | FT_LOAD_NO_HINTING);
         mFontFace = cairo_ft_font_face_create_for_ft_face(face, flags);
-        FTUserFontData *userFontData = new FTUserFontData(face, nsnull);
+        FTUserFontData *userFontData = new FTUserFontData(face, nullptr);
         cairo_font_face_set_user_data(mFontFace, &key,
                                       userFontData, FTFontDestroyFunc);
     }
@@ -349,7 +336,7 @@ FT2FontEntry::ReadCMAP()
 
     nsRefPtr<gfxCharacterMap> charmap = new gfxCharacterMap();
 
-    AutoFallibleTArray<PRUint8,16384> buffer;
+    AutoFallibleTArray<uint8_t,16384> buffer;
     nsresult rv = GetFontTable(TTAG_cmap, buffer);
     
     if (NS_SUCCEEDED(rv)) {
@@ -372,8 +359,8 @@ FT2FontEntry::ReadCMAP()
 }
 
 nsresult
-FT2FontEntry::GetFontTable(PRUint32 aTableTag,
-                           FallibleTArray<PRUint8>& aBuffer)
+FT2FontEntry::GetFontTable(uint32_t aTableTag,
+                           FallibleTArray<uint8_t>& aBuffer)
 {
     // Ensure existence of mFTFace
     CairoFontFace();
@@ -381,16 +368,17 @@ FT2FontEntry::GetFontTable(PRUint32 aTableTag,
 
     FT_Error status;
     FT_ULong len = 0;
-    status = FT_Load_Sfnt_Table(mFTFace, aTableTag, 0, nsnull, &len);
-    NS_ENSURE_TRUE(status == 0, NS_ERROR_FAILURE);
-    NS_ENSURE_TRUE(len != 0, NS_ERROR_FAILURE);
+    status = FT_Load_Sfnt_Table(mFTFace, aTableTag, 0, nullptr, &len);
+    if (status != FT_Err_Ok || len == 0) {
+        return NS_ERROR_FAILURE;
+    }
 
     if (!aBuffer.SetLength(len)) {
         return NS_ERROR_OUT_OF_MEMORY;
     }
-    PRUint8 *buf = aBuffer.Elements();
+    uint8_t *buf = aBuffer.Elements();
     status = FT_Load_Sfnt_Table(mFTFace, aTableTag, 0, buf, &len);
-    NS_ENSURE_TRUE(status == 0, NS_ERROR_FAILURE);
+    NS_ENSURE_TRUE(status == FT_Err_Ok, NS_ERROR_FAILURE);
 
     return NS_OK;
 }
@@ -463,10 +451,10 @@ public:
             NULL
         };
 
-        if (!PL_DHashTableInit(&mMap, &mOps, nsnull,
+        if (!PL_DHashTableInit(&mMap, &mOps, nullptr,
                                sizeof(FNCMapEntry), 0))
         {
-            mMap.ops = nsnull;
+            mMap.ops = nullptr;
             LOG(("initializing the map failed"));
         }
 
@@ -498,7 +486,7 @@ public:
         if (!mMap.ops || !mCache) {
             return;
         }
-        PRUint32 size;
+        uint32_t size;
         char* buf;
         if (NS_FAILED(mCache->GetBuffer(CACHE_KEY, &buf, &size))) {
             return;
@@ -519,12 +507,12 @@ public:
             if (!(end = strchr(beginning, ';'))) {
                 break;
             }
-            PRUint32 timestamp = strtoul(beginning, NULL, 10);
+            uint32_t timestamp = strtoul(beginning, NULL, 10);
             beginning = end + 1;
             if (!(end = strchr(beginning, ';'))) {
                 break;
             }
-            PRUint32 filesize = strtoul(beginning, NULL, 10);
+            uint32_t filesize = strtoul(beginning, NULL, 10);
 
             FNCMapEntry* mapEntry =
                 static_cast<FNCMapEntry*>
@@ -549,7 +537,7 @@ public:
 
     virtual void
     GetInfoForFile(nsCString& aFileName, nsCString& aFaceList,
-                   PRUint32 *aTimestamp, PRUint32 *aFilesize)
+                   uint32_t *aTimestamp, uint32_t *aFilesize)
     {
         if (!mMap.ops) {
             return;
@@ -573,7 +561,7 @@ public:
 
     virtual void
     CacheFileInfo(nsCString& aFileName, nsCString& aFaceList,
-                  PRUint32 aTimestamp, PRUint32 aFilesize)
+                  uint32_t aTimestamp, uint32_t aFilesize)
     {
         if (!mMap.ops) {
             return;
@@ -600,7 +588,7 @@ private:
 
     static PLDHashOperator WriteOutMap(PLDHashTable *aTable,
                                        PLDHashEntryHdr *aHdr,
-                                       PRUint32 aNumber, void *aData)
+                                       uint32_t aNumber, void *aData)
     {
         FNCMapEntry* entry = static_cast<FNCMapEntry*>(aHdr);
         if (!entry->mFileExists) {
@@ -623,8 +611,8 @@ private:
     typedef struct : public PLDHashEntryHdr {
     public:
         nsCString mFilename;
-        PRUint32  mTimestamp;
-        PRUint32  mFilesize;
+        uint32_t  mTimestamp;
+        uint32_t  mFilesize;
         nsCString mFaces;
         bool      mFileExists;
     } FNCMapEntry;
@@ -689,7 +677,7 @@ gfxFT2FontList::AppendFacesFromCachedFaceList(nsCString& aFileName,
         if (!(end = strchr(beginning, ','))) {
             break;
         }
-        PRUint32 index = strtoul(beginning, NULL, 10);
+        uint32_t index = strtoul(beginning, NULL, 10);
         beginning = end + 1;
         if (!(end = strchr(beginning, ','))) {
             break;
@@ -699,12 +687,12 @@ gfxFT2FontList::AppendFacesFromCachedFaceList(nsCString& aFileName,
         if (!(end = strchr(beginning, ','))) {
             break;
         }
-        PRUint32 weight = strtoul(beginning, NULL, 10);
+        uint32_t weight = strtoul(beginning, NULL, 10);
         beginning = end + 1;
         if (!(end = strchr(beginning, ','))) {
             break;
         }
-        PRInt32 stretch = strtol(beginning, NULL, 10);
+        int32_t stretch = strtol(beginning, NULL, 10);
 
         FontListEntry fle(familyName, faceName, aFileName,
                           weight, stretch, italic, index);
@@ -736,7 +724,7 @@ AppendToFaceList(nsCString& aFaceList,
 void
 FT2FontEntry::CheckForBrokenFont()
 {
-    NS_ASSERTION(mFamily != nsnull, "font entry must belong to a family");
+    NS_ASSERTION(mFamily != nullptr, "font entry must belong to a family");
 
     // note if the family is in the "bad underline" blacklist
     if (mFamily->IsBadUnderlineFamily()) {
@@ -768,7 +756,7 @@ gfxFT2FontList::AppendFacesFromFontFile(nsCString& aFileName,
                                         FontNameCache *aCache)
 {
     nsCString faceList;
-    PRUint32 filesize = 0, timestamp = 0;
+    uint32_t filesize = 0, timestamp = 0;
     if (aCache) {
         aCache->GetInfoForFile(aFileName, faceList, &timestamp, &filesize);
     }
@@ -799,9 +787,12 @@ gfxFT2FontList::AppendFacesFromFontFile(nsCString& aFileName,
             if (FT_Err_Ok != FT_New_Face(ftLibrary, aFileName.get(), i, &face)) {
                 continue;
             }
-
+            if (FT_Err_Ok != FT_Select_Charmap(face, FT_ENCODING_UNICODE)) {
+                FT_Done_Face(face);
+                continue;
+            }
             FT2FontEntry* fe =
-                FT2FontEntry::CreateFontEntry(face, aFileName.get(), i);
+                CreateNamedFontEntry(face, aFileName.get(), i);
             if (fe) {
                 NS_ConvertUTF8toUTF16 name(face->family_name);
                 BuildKeyNameFromFontName(name);       
@@ -848,7 +839,7 @@ FinalizeFamilyMemberList(nsStringHashKey::KeyType aKey,
                          void* aUserArg)
 {
     gfxFontFamily *family = aFamily.get();
-    bool sortFaces = (aUserArg != nsnull);
+    bool sortFaces = (aUserArg != nullptr);
 
     family->SetHasStyles(true);
 
@@ -883,9 +874,9 @@ gfxFT2FontList::FindFonts()
         searchPaths.AppendElement(resPath);
     }
     WIN32_FIND_DATAW results;
-    for (PRUint32 i = 0;  i < searchPaths.Length(); i++) {
+    for (uint32_t i = 0;  i < searchPaths.Length(); i++) {
         const nsString& path(searchPaths[i]);
-        for (PRUint32 j = 0; j < fontPatterns.Length(); j++) { 
+        for (uint32_t j = 0; j < fontPatterns.Length(); j++) { 
             nsAutoString pattern(path);
             pattern.Append(fontPatterns[j]);
             HANDLE handle = FindFirstFileExW(pattern.get(),
@@ -920,13 +911,13 @@ gfxFT2FontList::FindFonts()
         // Content process: ask the Chrome process to give us the list
         InfallibleTArray<FontListEntry> fonts;
         mozilla::dom::ContentChild::GetSingleton()->SendReadFontList(&fonts);
-        for (PRUint32 i = 0, n = fonts.Length(); i < n; ++i) {
+        for (uint32_t i = 0, n = fonts.Length(); i < n; ++i) {
             AppendFaceFromFontListEntry(fonts[i], false);
         }
         // Passing null for userdata tells Finalize that it does not need
         // to sort faces (because they were already sorted by chrome,
         // so we just maintain the existing order)
-        mFontFamilies.Enumerate(FinalizeFamilyMemberList, nsnull);
+        mFontFamilies.Enumerate(FinalizeFamilyMemberList, nullptr);
         LOG(("got font list from chrome process: %d faces in %d families",
             fonts.Length(), mFontFamilies.Count()));
         return;
@@ -1064,7 +1055,7 @@ gfxFT2FontList::InitFontList()
 
 struct FullFontNameSearch {
     FullFontNameSearch(const nsAString& aFullName)
-        : mFullName(aFullName), mFontEntry(nsnull)
+        : mFullName(aFullName), mFontEntry(nullptr)
     { }
 
     nsString     mFullName;
@@ -1133,13 +1124,13 @@ gfxFT2FontList::GetDefaultFont(const gfxFontStyle* aStyle, bool& aNeedsBold)
     }
 #endif
     /* TODO: what about Qt or other platforms that may use this? */
-    return nsnull;
+    return nullptr;
 }
 
 gfxFontEntry*
 gfxFT2FontList::MakePlatformFont(const gfxProxyFontEntry *aProxyEntry,
-                                 const PRUint8 *aFontData,
-                                 PRUint32 aLength)
+                                 const uint8_t *aFontData,
+                                 uint32_t aLength)
 {
     // The FT2 font needs the font data to persist, so we do NOT free it here
     // but instead pass ownership to the font entry.
