@@ -1,40 +1,7 @@
 /* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Foundation code.
- *
- * The Initial Developer of the Original Code is Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2005
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Vladimir Vukicevic <vladimir@mozilla.com>
- *   Masayuki Nakano <masayuki@d-toybox.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifndef GFX_FT2FONTS_H
 #define GFX_FT2FONTS_H
@@ -73,20 +40,20 @@ public: // new functions
         CachedGlyphData()
             : glyphIndex(0xffffffffU) { }
 
-        CachedGlyphData(PRUint32 gid)
+        CachedGlyphData(uint32_t gid)
             : glyphIndex(gid) { }
 
-        PRUint32 glyphIndex;
-        PRInt32 lsbDelta;
-        PRInt32 rsbDelta;
-        PRInt32 xAdvance;
+        uint32_t glyphIndex;
+        int32_t lsbDelta;
+        int32_t rsbDelta;
+        int32_t xAdvance;
     };
 
-    const CachedGlyphData* GetGlyphDataForChar(PRUint32 ch) {
+    const CachedGlyphData* GetGlyphDataForChar(uint32_t ch) {
         CharGlyphMapEntryType *entry = mCharGlyphCache.PutEntry(ch);
 
         if (!entry)
-            return nsnull;
+            return nullptr;
 
         if (entry->mData.glyphIndex == 0xffffffffU) {
             // this is a new entry, fill it
@@ -107,7 +74,7 @@ protected:
                            const PRUnichar *aString,
                            bool aPreferPlatformShaping = false);
 
-    void FillGlyphDataForChar(PRUint32 ch, CachedGlyphData *gd);
+    void FillGlyphDataForChar(uint32_t ch, CachedGlyphData *gd);
 
     void AddRange(gfxShapedWord *aShapedWord, const PRUnichar *str);
 
@@ -144,10 +111,10 @@ protected: // new functions
                                nsIAtom *aLangGroup,
                                nsTArray<nsRefPtr<gfxFontEntry> > *aFontEntryList);
     already_AddRefed<gfxFT2Font> WhichFontSupportsChar(const nsTArray<nsRefPtr<gfxFontEntry> >& aFontEntryList,
-                                                       PRUint32 aCh);
-    already_AddRefed<gfxFont> WhichPrefFontSupportsChar(PRUint32 aCh);
+                                                       uint32_t aCh);
+    already_AddRefed<gfxFont> WhichPrefFontSupportsChar(uint32_t aCh);
     already_AddRefed<gfxFont>
-        WhichSystemFontSupportsChar(PRUint32 aCh, PRInt32 aRunScript);
+        WhichSystemFontSupportsChar(uint32_t aCh, int32_t aRunScript);
 
     nsTArray<gfxTextRange> mRanges;
     nsString mString;
