@@ -1,43 +1,7 @@
 /* vim: set sw=4 sts=4 et cin: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is OS/2 code in Thebes.
- *
- * The Initial Developer of the Original Code is
- * Peter Weilbacher <mozilla@Weilbacher.org>.
- * Portions created by the Initial Developer are Copyright (C) 2006-2007
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   authors of code taken from gfxPangoFonts:
- *     Mozilla Foundation
- *     Vladimir Vukicevic <vladimir@mozilla.com>
- *     Masayuki Nakano <masayuki@d-toybox.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "gfxContext.h"
 
@@ -59,8 +23,8 @@ using namespace mozilla;
 
 gfxOS2Font::gfxOS2Font(gfxOS2FontEntry *aFontEntry, const gfxFontStyle *aFontStyle)
     : gfxFont(aFontEntry, aFontStyle),
-      mFontFace(nsnull),
-      mMetrics(nsnull), mAdjustedSize(0),
+      mFontFace(nullptr),
+      mMetrics(nullptr), mAdjustedSize(0),
       mHinting(FC_HINT_MEDIUM), mAntialias(FcTrue)
 {
 #ifdef DEBUG_thebes_2
@@ -69,7 +33,7 @@ gfxOS2Font::gfxOS2Font(gfxOS2FontEntry *aFontEntry, const gfxFontStyle *aFontSty
            NS_LossyConvertUTF16toASCII(aFontEntry->Name()).get());
 #endif
     // try to get the preferences for hinting, antialias, and embolden options
-    PRInt32 value;
+    int32_t value;
     nsresult rv = Preferences::GetInt("gfx.os2.font.hinting", &value);
     if (NS_SUCCEEDED(rv) && value >= FC_HINT_NONE && value <= FC_HINT_FULL) {
         mHinting = value;
@@ -95,9 +59,9 @@ gfxOS2Font::~gfxOS2Font()
         cairo_scaled_font_destroy(mScaledFont);
     }
     delete mMetrics;
-    mFontFace = nsnull;
-    mScaledFont = nsnull;
-    mMetrics = nsnull;
+    mFontFace = nullptr;
+    mScaledFont = nullptr;
+    mMetrics = nullptr;
 }
 
 // fill font metrics structure with default values in case of error
@@ -312,7 +276,7 @@ const gfxFont::Metrics& gfxOS2Font::GetMetrics()
 
 // weight list copied from fontconfig.h
 // unfortunately, the OS/2 version so far only supports regular and bold
-static const PRInt8 nFcWeight = 2; // 10; // length of weight list
+static const int8_t nFcWeight = 2; // 10; // length of weight list
 static const int fcWeight[] = {
     //FC_WEIGHT_THIN,
     //FC_WEIGHT_EXTRALIGHT, // == FC_WEIGHT_ULTRALIGHT
@@ -350,13 +314,13 @@ cairo_font_face_t *gfxOS2Font::CairoFontFace()
         // The requirements outlined in gfxFont.h are difficult to meet without
         // having a table of available font weights, so we map the gfxFont
         // weight to possible FontConfig weights.
-        PRInt8 weight = GetStyle()->ComputeWeight();
+        int8_t weight = GetStyle()->ComputeWeight();
         // gfxFont weight   FC weight
         //    400              80
         //    700             200
-        PRInt16 fcW = 40 * weight - 80; // match gfxFont weight to base FC weight
+        int16_t fcW = 40 * weight - 80; // match gfxFont weight to base FC weight
         // find the correct weight in the list
-        PRInt8 i = 0;
+        int8_t i = 0;
         while (i < nFcWeight && fcWeight[i] < fcW) {
             i++;
         }
@@ -370,7 +334,7 @@ cairo_font_face_t *gfxOS2Font::CairoFontFace()
         // add weight to pattern
         FcPatternAddInteger(fcPattern, FC_WEIGHT, fcW);
 
-        PRUint8 fcProperty;
+        uint8_t fcProperty;
         // add style to pattern
         switch (GetStyle()->style) {
         case NS_FONT_STYLE_ITALIC:
@@ -464,7 +428,7 @@ cairo_scaled_font_t *gfxOS2Font::CairoScaledFont()
 
     cairo_font_face_t * face = CairoFontFace();
     if (!face)
-        return nsnull;
+        return nullptr;
 
     cairo_font_options_t *fontOptions = cairo_font_options_create();
     mScaledFont = cairo_scaled_font_create(face, &fontMatrix,
@@ -509,10 +473,10 @@ already_AddRefed<gfxOS2Font> gfxOS2Font::GetOrMakeFont(const nsAString& aName,
     if (!font) {
         font = new gfxOS2Font(fe, aStyle);
         if (!font)
-            return nsnull;
+            return nullptr;
         gfxFontCache::GetCache()->AddNew(font);
     }
-    gfxFont *f = nsnull;
+    gfxFont *f = nullptr;
     font.swap(f);
     return static_cast<gfxOS2Font *>(f);
 }
@@ -559,7 +523,7 @@ gfxOS2FontGroup::gfxOS2FontGroup(const nsAString& aFamilies,
         familyArray.AppendElement(NS_LITERAL_STRING("Helv"));
     }
 
-    for (PRUint32 i = 0; i < familyArray.Length(); i++) {
+    for (uint32_t i = 0; i < familyArray.Length(); i++) {
         nsRefPtr<gfxOS2Font> font = gfxOS2Font::GetOrMakeFont(familyArray[i], &mStyle);
         if (font) {
             mFonts.AppendElement(font);
@@ -584,25 +548,25 @@ gfxFontGroup *gfxOS2FontGroup::Copy(const gfxFontStyle *aStyle)
  * string. This forces Pango to honour our direction even if there are neutral
  * characters in the string.
  */
-static PRInt32 AppendDirectionalIndicatorUTF8(bool aIsRTL, nsACString& aString)
+static int32_t AppendDirectionalIndicatorUTF8(bool aIsRTL, nsACString& aString)
 {
     static const PRUnichar overrides[2][2] = { { 0x202d, 0 }, { 0x202e, 0 }}; // LRO, RLO
     AppendUTF16toUTF8(overrides[aIsRTL], aString);
     return 3; // both overrides map to 3 bytes in UTF8
 }
 
-gfxTextRun *gfxOS2FontGroup::MakeTextRun(const PRUnichar* aString, PRUint32 aLength,
-                                         const Parameters* aParams, PRUint32 aFlags)
+gfxTextRun *gfxOS2FontGroup::MakeTextRun(const PRUnichar* aString, uint32_t aLength,
+                                         const Parameters* aParams, uint32_t aFlags)
 {
     NS_ASSERTION(aLength > 0, "should use MakeEmptyTextRun for zero-length text");
     gfxTextRun *textRun = gfxTextRun::Create(aParams, aLength, this, aFlags);
     if (!textRun)
-        return nsnull;
+        return nullptr;
 
     mEnableKerning = !(aFlags & gfxTextRunFactory::TEXT_OPTIMIZE_SPEED);
 
     nsCAutoString utf8;
-    PRInt32 headerLen = AppendDirectionalIndicatorUTF8(textRun->IsRightToLeft(), utf8);
+    int32_t headerLen = AppendDirectionalIndicatorUTF8(textRun->IsRightToLeft(), utf8);
     AppendUTF16toUTF8(Substring(aString, aString + aLength), utf8);
 
 #ifdef DEBUG_thebes_2
@@ -611,27 +575,27 @@ gfxTextRun *gfxOS2FontGroup::MakeTextRun(const PRUnichar* aString, PRUint32 aLen
            (unsigned)this, NS_LossyConvertUTF16toASCII(u16).get(), aLength, (unsigned)aParams, aFlags);
 #endif
 
-    InitTextRun(textRun, (PRUint8 *)utf8.get(), utf8.Length(), headerLen);
+    InitTextRun(textRun, (uint8_t *)utf8.get(), utf8.Length(), headerLen);
 
     textRun->FetchGlyphExtents(aParams->mContext);
 
     return textRun;
 }
 
-gfxTextRun *gfxOS2FontGroup::MakeTextRun(const PRUint8* aString, PRUint32 aLength,
-                                         const Parameters* aParams, PRUint32 aFlags)
+gfxTextRun *gfxOS2FontGroup::MakeTextRun(const uint8_t* aString, uint32_t aLength,
+                                         const Parameters* aParams, uint32_t aFlags)
 {
 #ifdef DEBUG_thebes_2
     const char *cStr = reinterpret_cast<const char *>(aString);
     NS_ConvertASCIItoUTF16 us(cStr, aLength);
-    printf("gfxOS2FontGroup[%#x]::MakeTextRun(PRUint8 %s, %d, %#x, %d)\n",
+    printf("gfxOS2FontGroup[%#x]::MakeTextRun(uint8_t %s, %d, %#x, %d)\n",
            (unsigned)this, NS_LossyConvertUTF16toASCII(us).get(), aLength, (unsigned)aParams, aFlags);
 #endif
     NS_ASSERTION(aLength > 0, "should use MakeEmptyTextRun for zero-length text");
     NS_ASSERTION(aFlags & TEXT_IS_8BIT, "8bit should have been set");
     gfxTextRun *textRun = gfxTextRun::Create(aParams, aLength, this, aFlags);
     if (!textRun)
-        return nsnull;
+        return nullptr;
 
     mEnableKerning = !(aFlags & gfxTextRunFactory::TEXT_OPTIMIZE_SPEED);
 
@@ -640,16 +604,16 @@ gfxTextRun *gfxOS2FontGroup::MakeTextRun(const PRUint8* aString, PRUint32 aLengt
     if ((aFlags & TEXT_IS_ASCII) && !isRTL) {
         // We don't need to send an override character here, the characters must be all
         // LTR
-        InitTextRun(textRun, (PRUint8 *)chars, aLength, 0);
+        InitTextRun(textRun, (uint8_t *)chars, aLength, 0);
     } else {
         // Although chars in not necessarily ASCII (as it may point to the low
         // bytes of any UCS-2 characters < 256), NS_ConvertASCIItoUTF16 seems
         // to DTRT.
         NS_ConvertASCIItoUTF16 unicodeString(chars, aLength);
         nsCAutoString utf8;
-        PRInt32 headerLen = AppendDirectionalIndicatorUTF8(isRTL, utf8);
+        int32_t headerLen = AppendDirectionalIndicatorUTF8(isRTL, utf8);
         AppendUTF16toUTF8(unicodeString, utf8);
-        InitTextRun(textRun, (PRUint8 *)utf8.get(), utf8.Length(), headerLen);
+        InitTextRun(textRun, (uint8_t *)utf8.get(), utf8.Length(), headerLen);
     }
 
     textRun->FetchGlyphExtents(aParams->mContext);
@@ -657,9 +621,9 @@ gfxTextRun *gfxOS2FontGroup::MakeTextRun(const PRUint8* aString, PRUint32 aLengt
     return textRun;
 }
 
-void gfxOS2FontGroup::InitTextRun(gfxTextRun *aTextRun, const PRUint8 *aUTF8Text,
-                                  PRUint32 aUTF8Length,
-                                  PRUint32 aUTF8HeaderLength)
+void gfxOS2FontGroup::InitTextRun(gfxTextRun *aTextRun, const uint8_t *aUTF8Text,
+                                  uint32_t aUTF8Length,
+                                  uint32_t aUTF8HeaderLength)
 {
     CreateGlyphRunsFT(aTextRun, aUTF8Text + aUTF8HeaderLength,
                       aUTF8Length - aUTF8HeaderLength);
@@ -668,7 +632,7 @@ void gfxOS2FontGroup::InitTextRun(gfxTextRun *aTextRun, const PRUint8 *aUTF8Text
 // Helper function to return the leading UTF-8 character in a char pointer
 // as 32bit number. Also sets the length of the current character (i.e. the
 // offset to the next one) in the second argument
-PRUint32 getUTF8CharAndNext(const PRUint8 *aString, PRUint8 *aLength)
+uint32_t getUTF8CharAndNext(const uint8_t *aString, uint8_t *aLength)
 {
     *aLength = 1;
     if (aString[0] < 0x80) { // normal 7bit ASCII char
@@ -691,24 +655,24 @@ PRUint32 getUTF8CharAndNext(const PRUint8 *aString, PRUint8 *aLength)
     return aString[0];
 }
 
-void gfxOS2FontGroup::CreateGlyphRunsFT(gfxTextRun *aTextRun, const PRUint8 *aUTF8,
-                                        PRUint32 aUTF8Length)
+void gfxOS2FontGroup::CreateGlyphRunsFT(gfxTextRun *aTextRun, const uint8_t *aUTF8,
+                                        uint32_t aUTF8Length)
 {
 #ifdef DEBUG_thebes_2
     printf("gfxOS2FontGroup::CreateGlyphRunsFT(%#x, _aUTF8_, %d)\n",
            (unsigned)aTextRun, /*aUTF8,*/ aUTF8Length);
-    for (PRUint32 i = 0; i < FontListLength(); i++) {
+    for (uint32_t i = 0; i < FontListLength(); i++) {
         gfxOS2Font *font = GetFontAt(i);
         printf("  i=%d, name=%s, size=%f\n", i, NS_LossyConvertUTF16toASCII(font->GetName()).get(),
                font->GetStyle()->size);
     }
 #endif
-    PRUint32 lastFont = FontListLength()-1;
+    uint32_t lastFont = FontListLength()-1;
     gfxOS2Font *font0 = GetFontAt(0);
-    const PRUint8 *p = aUTF8;
-    PRUint32 utf16Offset = 0;
+    const uint8_t *p = aUTF8;
+    uint32_t utf16Offset = 0;
     gfxTextRun::CompressedGlyph g;
-    const PRUint32 appUnitsPerDevUnit = aTextRun->GetAppUnitsPerDevUnit();
+    const uint32_t appUnitsPerDevUnit = aTextRun->GetAppUnitsPerDevUnit();
     gfxOS2Platform *platform = gfxOS2Platform::GetPlatform();
 
     aTextRun->AddGlyphRun(font0, gfxTextRange::kFontGroup, 0, false);
@@ -718,8 +682,8 @@ void gfxOS2FontGroup::CreateGlyphRunsFT(gfxTextRun *aTextRun, const PRUint8 *aUT
     while (p < aUTF8 + aUTF8Length) {
         bool glyphFound = false;
         // convert UTF-8 character and step to the next one in line
-        PRUint8 chLen;
-        PRUint32 ch = getUTF8CharAndNext(p, &chLen);
+        uint8_t chLen;
+        uint32_t ch = getUTF8CharAndNext(p, &chLen);
         p += chLen; // move to next char
 #ifdef DEBUG_thebes_2
         printf("\'%c\' (%d, %#x, %s) [%#x %#x]:", (char)ch, ch, ch, ch >=0x10000 ? "non-BMP!" : "BMP", ch >=0x10000 ? H_SURROGATE(ch) : 0, ch >=0x10000 ? L_SURROGATE(ch) : 0);
@@ -734,7 +698,7 @@ void gfxOS2FontGroup::CreateGlyphRunsFT(gfxTextRun *aTextRun, const PRUint8 *aUT
             // If we don't find the glyph even in the last font, we will fall
             // back to searching all fonts on the system and finally set the
             // missing glyph symbol after trying the last font.
-            for (PRUint32 i = 0; i <= lastFont; i++) {
+            for (uint32_t i = 0; i <= lastFont; i++) {
                 gfxOS2Font *font = font0;
                 FT_Face face = face0;
                 if (i > 0) {
@@ -774,7 +738,7 @@ void gfxOS2FontGroup::CreateGlyphRunsFT(gfxTextRun *aTextRun, const PRUint8 *aUT
                 // select the current font into the text run
                 aTextRun->AddGlyphRun(font, gfxTextRange::kFontGroup, utf16Offset, false);
 
-                PRInt32 advance = 0;
+                int32_t advance = 0;
                 if (gid == font->GetSpaceGlyph()) {
                     advance = (int)(font->GetMetrics().spaceWidth * appUnitsPerDevUnit);
                 } else if (gid == 0) {
@@ -782,7 +746,7 @@ void gfxOS2FontGroup::CreateGlyphRunsFT(gfxTextRun *aTextRun, const PRUint8 *aUT
                 } else {
                     // find next character and its glyph -- in case they exist
                     // and exist in the current font face -- to compute kerning
-                    PRUint32 chNext = 0;
+                    uint32_t chNext = 0;
                     FT_UInt gidNext = 0;
                     FT_Pos lsbDeltaNext = 0;
 #ifdef DEBUG_thebes_2
