@@ -1,39 +1,7 @@
 /* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Corporation code.
- *
- * The Initial Developer of the Original Code is Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2010
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Vladimir Vukicevic <vladimir@pobox.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "gfxUtils.h"
 #include "gfxContext.h"
@@ -52,15 +20,15 @@ using namespace mozilla;
 using namespace mozilla::layers;
 using namespace mozilla::gfx;
 
-static PRUint8 sUnpremultiplyTable[256*256];
-static PRUint8 sPremultiplyTable[256*256];
+static uint8_t sUnpremultiplyTable[256*256];
+static uint8_t sPremultiplyTable[256*256];
 static bool sTablesInitialized = false;
 
-static const PRUint8 PremultiplyValue(PRUint8 a, PRUint8 v) {
+static const uint8_t PremultiplyValue(uint8_t a, uint8_t v) {
     return sPremultiplyTable[a*256+v];
 }
 
-static const PRUint8 UnpremultiplyValue(PRUint8 a, PRUint8 v) {
+static const uint8_t UnpremultiplyValue(uint8_t a, uint8_t v) {
     return sUnpremultiplyTable[a*256+v];
 }
 
@@ -76,13 +44,13 @@ CalculateTables()
     // Unpremultiply table
 
     // a == 0 case
-    for (PRUint32 c = 0; c <= 255; c++) {
+    for (uint32_t c = 0; c <= 255; c++) {
         sUnpremultiplyTable[c] = c;
     }
 
     for (int a = 1; a <= 255; a++) {
         for (int c = 0; c <= 255; c++) {
-            sUnpremultiplyTable[a*256+c] = (PRUint8)((c * 255) / a);
+            sUnpremultiplyTable[a*256+c] = (uint8_t)((c * 255) / a);
         }
     }
 
@@ -125,26 +93,26 @@ gfxUtils::PremultiplyImageSurface(gfxImageSurface *aSourceSurface,
     if (!sTablesInitialized)
         CalculateTables();
 
-    PRUint8 *src = aSourceSurface->Data();
-    PRUint8 *dst = aDestSurface->Data();
+    uint8_t *src = aSourceSurface->Data();
+    uint8_t *dst = aDestSurface->Data();
 
-    PRUint32 dim = aSourceSurface->Width() * aSourceSurface->Height();
-    for (PRUint32 i = 0; i < dim; ++i) {
+    uint32_t dim = aSourceSurface->Width() * aSourceSurface->Height();
+    for (uint32_t i = 0; i < dim; ++i) {
 #ifdef IS_LITTLE_ENDIAN
-        PRUint8 b = *src++;
-        PRUint8 g = *src++;
-        PRUint8 r = *src++;
-        PRUint8 a = *src++;
+        uint8_t b = *src++;
+        uint8_t g = *src++;
+        uint8_t r = *src++;
+        uint8_t a = *src++;
 
         *dst++ = PremultiplyValue(a, b);
         *dst++ = PremultiplyValue(a, g);
         *dst++ = PremultiplyValue(a, r);
         *dst++ = a;
 #else
-        PRUint8 a = *src++;
-        PRUint8 r = *src++;
-        PRUint8 g = *src++;
-        PRUint8 b = *src++;
+        uint8_t a = *src++;
+        uint8_t r = *src++;
+        uint8_t g = *src++;
+        uint8_t b = *src++;
 
         *dst++ = a;
         *dst++ = PremultiplyValue(a, r);
@@ -182,26 +150,26 @@ gfxUtils::UnpremultiplyImageSurface(gfxImageSurface *aSourceSurface,
     if (!sTablesInitialized)
         CalculateTables();
 
-    PRUint8 *src = aSourceSurface->Data();
-    PRUint8 *dst = aDestSurface->Data();
+    uint8_t *src = aSourceSurface->Data();
+    uint8_t *dst = aDestSurface->Data();
 
-    PRUint32 dim = aSourceSurface->Width() * aSourceSurface->Height();
-    for (PRUint32 i = 0; i < dim; ++i) {
+    uint32_t dim = aSourceSurface->Width() * aSourceSurface->Height();
+    for (uint32_t i = 0; i < dim; ++i) {
 #ifdef IS_LITTLE_ENDIAN
-        PRUint8 b = *src++;
-        PRUint8 g = *src++;
-        PRUint8 r = *src++;
-        PRUint8 a = *src++;
+        uint8_t b = *src++;
+        uint8_t g = *src++;
+        uint8_t r = *src++;
+        uint8_t a = *src++;
 
         *dst++ = UnpremultiplyValue(a, b);
         *dst++ = UnpremultiplyValue(a, g);
         *dst++ = UnpremultiplyValue(a, r);
         *dst++ = a;
 #else
-        PRUint8 a = *src++;
-        PRUint8 r = *src++;
-        PRUint8 g = *src++;
-        PRUint8 b = *src++;
+        uint8_t a = *src++;
+        uint8_t r = *src++;
+        uint8_t g = *src++;
+        uint8_t b = *src++;
 
         *dst++ = a;
         *dst++ = UnpremultiplyValue(a, r);
@@ -229,14 +197,14 @@ gfxUtils::ConvertBGRAtoRGBA(gfxImageSurface *aSourceSurface,
     NS_ABORT_IF_FALSE(aSourceSurface->Format() == gfxASurface::ImageFormatARGB32,
                       "Surfaces must be ARGB32");
 
-    PRUint8 *src = aSourceSurface->Data();
-    PRUint8 *dst = aDestSurface->Data();
+    uint8_t *src = aSourceSurface->Data();
+    uint8_t *dst = aDestSurface->Data();
 
-    PRUint32 dim = aSourceSurface->Width() * aSourceSurface->Height();
-    PRUint8 *srcEnd = src + 4*dim;
+    uint32_t dim = aSourceSurface->Width() * aSourceSurface->Height();
+    uint8_t *srcEnd = src + 4*dim;
 
     if (src == dst) {
-        PRUint8 buffer[4];
+        uint8_t buffer[4];
         for (; src != srcEnd; src += 4) {
             buffer[0] = src[2];
             buffer[1] = src[1];
@@ -315,13 +283,13 @@ CreateSamplingRestrictedDrawable(gfxDrawable* aDrawable,
     // matter what we do here, but we should avoid trying to
     // create a zero-size surface.
     if (needed.IsEmpty())
-        return nsnull;
+        return nullptr;
 
-    gfxIntSize size(PRInt32(needed.Width()), PRInt32(needed.Height()));
+    gfxIntSize size(int32_t(needed.Width()), int32_t(needed.Height()));
     nsRefPtr<gfxASurface> temp =
         gfxPlatform::GetPlatform()->CreateOffscreenSurface(size, gfxASurface::ContentFromFormat(aFormat));
     if (!temp || temp->CairoStatus())
-        return nsnull;
+        return nullptr;
 
     nsRefPtr<gfxContext> tmpCtx = new gfxContext(temp);
     tmpCtx->SetOperator(OptimalFillOperator());
@@ -330,7 +298,7 @@ CreateSamplingRestrictedDrawable(gfxDrawable* aDrawable,
 
     nsRefPtr<gfxPattern> resultPattern = new gfxPattern(temp);
     if (!resultPattern)
-        return nsnull;
+        return nullptr;
 
     nsRefPtr<gfxDrawable> drawable = 
         new gfxSurfaceDrawable(temp, size, gfxMatrix().Translate(-needed.TopLeft()));
@@ -491,7 +459,7 @@ gfxUtils::DrawPixelSnapped(gfxContext*      aContext,
                            const gfxRect&   aFill,
                            const gfxImageSurface::gfxImageFormat aFormat,
                            gfxPattern::GraphicsFilter aFilter,
-                           PRUint32         aImageFlags)
+                           uint32_t         aImageFlags)
 {
     SAMPLE_LABEL("gfxUtils", "DrawPixelSnapped");
     bool doTile = !aImageRect.Contains(aSourceRect) &&
@@ -583,7 +551,7 @@ PathFromRegionInternal(gfxContext* aContext, const nsIntRegion& aRegion,
   aContext->NewPath();
   nsIntRegionRectIterator iter(aRegion);
   const nsIntRect* r;
-  while ((r = iter.Next()) != nsnull) {
+  while ((r = iter.Next()) != nullptr) {
     aContext->Rectangle(gfxRect(r->x, r->y, r->width, r->height), aSnap);
   }
 }
@@ -633,7 +601,9 @@ gfxUtils::ClampToScaleFactor(gfxFloat aVal)
     power = ceil(power);
   }
 
-  return pow(kScaleResolution, power);
+  gfxFloat scale = pow(kScaleResolution, power);
+
+  return NS_MAX(scale, 1.0);
 }
 
 
@@ -653,8 +623,8 @@ gfxUtils::PathFromRegionSnapped(gfxContext* aContext, const nsIntRegion& aRegion
 bool
 gfxUtils::GfxRectToIntRect(const gfxRect& aIn, nsIntRect* aOut)
 {
-  *aOut = nsIntRect(PRInt32(aIn.X()), PRInt32(aIn.Y()),
-  PRInt32(aIn.Width()), PRInt32(aIn.Height()));
+  *aOut = nsIntRect(int32_t(aIn.X()), int32_t(aIn.Y()),
+  int32_t(aIn.Width()), int32_t(aIn.Height()));
   return gfxRect(aOut->x, aOut->y, aOut->width, aOut->height).IsEqualEdges(aIn);
 }
 
@@ -717,8 +687,14 @@ gfxUtils::ConvertYCbCrToRGB(const PlanarYCbCrImage::Data& aData,
                             const gfxASurface::gfxImageFormat& aDestFormat,
                             const gfxIntSize& aDestSize,
                             unsigned char* aDestBuffer,
-                            PRInt32 aStride)
+                            int32_t aStride)
 {
+  // ConvertYCbCrToRGB et al. assume the chroma planes are rounded up if the
+  // luma plane is odd sized.
+  MOZ_ASSERT((aData.mCbCrSize.width == aData.mYSize.width ||
+              aData.mCbCrSize.width == (aData.mYSize.width + 1) >> 1) &&
+             (aData.mCbCrSize.height == aData.mYSize.height ||
+              aData.mCbCrSize.height == (aData.mYSize.height + 1) >> 1));
   gfx::YUVType yuvtype =
     gfx::TypeFromSize(aData.mYSize.width,
                       aData.mYSize.height,
