@@ -1,41 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2001
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Stuart Parmenter <pavlov@netscape.com>
- *   Ehsan Akhgari <ehsan.akhgari@gmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "mozilla/Attributes.h"
 #include "mozilla/FunctionTimer.h"
@@ -98,44 +64,6 @@
 using namespace mozilla;
 using namespace mozilla::image;
 
-#if defined(DEBUG_pavlov) || defined(DEBUG_timeless)
-#include "nsISimpleEnumerator.h"
-#include "nsXPCOM.h"
-#include "nsISupportsPrimitives.h"
-#include "nsXPIDLString.h"
-#include "nsComponentManagerUtils.h"
-
-
-static void PrintImageDecoders()
-{
-  nsCOMPtr<nsIComponentRegistrar> compMgr;
-  if (NS_FAILED(NS_GetComponentRegistrar(getter_AddRefs(compMgr))) || !compMgr)
-    return;
-  nsCOMPtr<nsISimpleEnumerator> enumer;
-  if (NS_FAILED(compMgr->EnumerateContractIDs(getter_AddRefs(enumer))) || !enumer)
-    return;
-  
-  nsCString str;
-  nsCOMPtr<nsISupports> s;
-  bool more = false;
-  while (NS_SUCCEEDED(enumer->HasMoreElements(&more)) && more) {
-    enumer->GetNext(getter_AddRefs(s));
-    if (s) {
-      nsCOMPtr<nsISupportsCString> ss(do_QueryInterface(s));
-
-      nsCAutoString xcs;
-      ss->GetData(xcs);
-
-      NS_NAMED_LITERAL_CSTRING(decoderContract, "@mozilla.org/image/decoder;3?type=");
-
-      if (StringBeginsWith(xcs, decoderContract)) {
-        printf("Have decoder for mime type: %s\n", xcs.get()+decoderContract.Length());
-      }
-    }
-  }
-}
-#endif
-
 NS_MEMORY_REPORTER_MALLOC_SIZEOF_FUN(ImagesMallocSizeOf, "images")
 
 class imgMemoryReporter MOZ_FINAL :
@@ -224,7 +152,7 @@ public:
     return NS_OK;
   }
 
-  NS_IMETHOD GetExplicitNonHeap(PRInt64 *n)
+  NS_IMETHOD GetExplicitNonHeap(int64_t *n)
   {
     size_t n2 = 0;
     imgLoader::sChromeCache.EnumerateRead(EntryExplicitNonHeapSize, &n2);
@@ -233,7 +161,7 @@ public:
     return NS_OK;
   }
 
-  static PRInt64 GetImagesContentUsedUncompressed()
+  static int64_t GetImagesContentUsedUncompressed()
   {
     size_t n = 0;
     imgLoader::sCache.EnumerateRead(EntryUsedUncompressedSize, &n);
@@ -334,8 +262,8 @@ NS_IMPL_ISUPPORTS3(nsProgressNotificationProxy,
 NS_IMETHODIMP
 nsProgressNotificationProxy::OnProgress(nsIRequest* request,
                                         nsISupports* ctxt,
-                                        PRUint64 progress,
-                                        PRUint64 progressMax)
+                                        uint64_t progress,
+                                        uint64_t progressMax)
 {
   nsCOMPtr<nsILoadGroup> loadGroup;
   request->GetLoadGroup(getter_AddRefs(loadGroup));
@@ -372,7 +300,7 @@ nsProgressNotificationProxy::OnStatus(nsIRequest* request,
 NS_IMETHODIMP
 nsProgressNotificationProxy::AsyncOnChannelRedirect(nsIChannel *oldChannel,
                                                     nsIChannel *newChannel,
-                                                    PRUint32 flags,
+                                                    uint32_t flags,
                                                     nsIAsyncVerifyRedirectCallback *cb)
 {
   // Tell the original original callbacks about it too
@@ -467,7 +395,7 @@ static bool ShouldRevalidateEntry(imgCacheEntry *aEntry,
 // to CORS.
 static bool
 ValidateCORSAndPrincipal(imgRequest* request, bool forcePrincipalCheck,
-                         PRInt32 corsmode, nsIPrincipal* loadingPrincipal)
+                         int32_t corsmode, nsIPrincipal* loadingPrincipal)
 {
   // If the entry's CORS mode doesn't match, or the CORS mode matches but the
   // document principal isn't the same, we can't use this request.
@@ -537,8 +465,8 @@ static nsresult NewImageChannel(nsIChannel **aResult,
   //
   rv = NS_NewChannel(aResult,
                      aURI,        // URI 
-                     nsnull,      // Cached IOService
-                     nsnull,      // LoadGroup
+                     nullptr,      // Cached IOService
+                     nullptr,      // LoadGroup
                      callbacks,   // Notification Callbacks
                      aLoadFlags,
                      aPolicy);
@@ -563,7 +491,7 @@ static nsresult NewImageChannel(nsIChannel **aResult,
   // Image channels are loaded by default with reduced priority.
   nsCOMPtr<nsISupportsPriority> p = do_QueryInterface(*aResult);
   if (p) {
-    PRUint32 priority = nsISupportsPriority::PRIORITY_LOW;
+    uint32_t priority = nsISupportsPriority::PRIORITY_LOW;
 
     if (aLoadFlags & nsIRequest::LOAD_BACKGROUND)
       ++priority; // further reduce priority for background loads
@@ -578,9 +506,9 @@ static nsresult NewImageChannel(nsIChannel **aResult,
   return NS_OK;
 }
 
-static PRUint32 SecondsFromPRTime(PRTime prTime)
+static uint32_t SecondsFromPRTime(PRTime prTime)
 {
-  return PRUint32(PRInt64(prTime) / PRInt64(PR_USEC_PER_SEC));
+  return uint32_t(int64_t(prTime) / int64_t(PR_USEC_PER_SEC));
 }
 
 imgCacheEntry::imgCacheEntry(imgRequest *request, bool forcePrincipalCheck)
@@ -611,7 +539,7 @@ void imgCacheEntry::Touch(bool updateTime /* = true */)
   UpdateCache();
 }
 
-void imgCacheEntry::UpdateCache(PRInt32 diff /* = 0 */)
+void imgCacheEntry::UpdateCache(int32_t diff /* = 0 */)
 {
   // Don't update the cache if we've been removed from it or it doesn't care
   // about our size or usage.
@@ -644,12 +572,12 @@ imgCacheQueue::imgCacheQueue()
    mSize(0)
 {}
 
-void imgCacheQueue::UpdateSize(PRInt32 diff)
+void imgCacheQueue::UpdateSize(int32_t diff)
 {
   mSize += diff;
 }
 
-PRUint32 imgCacheQueue::GetSize() const
+uint32_t imgCacheQueue::GetSize() const
 {
   return mSize;
 }
@@ -679,7 +607,7 @@ void imgCacheQueue::Push(imgCacheEntry *entry)
 already_AddRefed<imgCacheEntry> imgCacheQueue::Pop()
 {
   if (mQueue.empty())
-    return nsnull;
+    return nullptr;
   if (IsDirty())
     Refresh();
 
@@ -709,7 +637,7 @@ bool imgCacheQueue::IsDirty()
   return mDirty;
 }
 
-PRUint32 imgCacheQueue::GetNumElements() const
+uint32_t imgCacheQueue::GetNumElements() const
 {
   return mQueue.size();
 }
@@ -846,8 +774,8 @@ imgCacheQueue imgLoader::sCacheQueue;
 imgLoader::imgCacheTable imgLoader::sChromeCache;
 imgCacheQueue imgLoader::sChromeCacheQueue;
 
-PRFloat64 imgLoader::sCacheTimeWeight;
-PRUint32 imgLoader::sCacheMaxSize;
+double imgLoader::sCacheTimeWeight;
+uint32_t imgLoader::sCacheMaxSize;
 
 NS_IMPL_ISUPPORTS5(imgLoader, imgILoader, nsIContentSniffer, imgICache, nsISupportsWeakReference, nsIObserver)
 
@@ -867,9 +795,9 @@ void imgLoader::VerifyCacheSizes()
   if (!gCacheTracker)
     return;
 
-  PRUint32 cachesize = sCache.Count() + sChromeCache.Count();
-  PRUint32 queuesize = sCacheQueue.GetNumElements() + sChromeCacheQueue.GetNumElements();
-  PRUint32 trackersize = 0;
+  uint32_t cachesize = sCache.Count() + sChromeCache.Count();
+  uint32_t queuesize = sCacheQueue.GetNumElements() + sChromeCacheQueue.GetNumElements();
+  uint32_t trackersize = 0;
   for (nsExpirationTracker<imgCacheEntry, 3>::Iterator it(gCacheTracker); it.Next(); )
     trackersize++;
   NS_ABORT_IF_FALSE(queuesize == trackersize, "Queue and tracker sizes out of sync!");
@@ -915,19 +843,17 @@ nsresult imgLoader::InitCache()
 
   gCacheTracker = new imgCacheExpirationTracker();
 
-  if (!sCache.Init())
-      return NS_ERROR_OUT_OF_MEMORY;
-  if (!sChromeCache.Init())
-      return NS_ERROR_OUT_OF_MEMORY;
+  sCache.Init();
+  sChromeCache.Init();
 
-  PRInt32 timeweight;
+  int32_t timeweight;
   rv = Preferences::GetInt("image.cache.timeweight", &timeweight);
   if (NS_SUCCEEDED(rv))
     sCacheTimeWeight = timeweight / 1000.0;
   else
     sCacheTimeWeight = 0.5;
 
-  PRInt32 cachesize;
+  int32_t cachesize;
   rv = Preferences::GetInt("image.cache.size", &cachesize);
   if (NS_SUCCEEDED(rv))
     sCacheMaxSize = cachesize;
@@ -1013,7 +939,7 @@ NS_IMETHODIMP imgLoader::FindEntryProperties(nsIURI *uri, nsIProperties **_retva
   imgCacheTable &cache = GetCache(uri);
 
   uri->GetSpec(spec);
-  *_retval = nsnull;
+  *_retval = nullptr;
 
   if (cache.Get(spec, getter_AddRefs(entry)) && entry) {
     if (gCacheTracker && entry->HasNoProxies())
@@ -1035,7 +961,7 @@ void imgLoader::Shutdown()
   ClearImageCache();
   NS_IF_RELEASE(gCacheObserver);
   delete gCacheTracker;
-  gCacheTracker = nsnull;
+  gCacheTracker = nullptr;
 }
 
 nsresult imgLoader::ClearChromeImageCache()
@@ -1069,22 +995,21 @@ bool imgLoader::PutIntoCache(nsIURI *key, imgCacheEntry *entry)
   nsRefPtr<imgCacheEntry> tmpCacheEntry;
   if (cache.Get(spec, getter_AddRefs(tmpCacheEntry)) && tmpCacheEntry) {
     PR_LOG(gImgLog, PR_LOG_DEBUG,
-           ("[this=%p] imgLoader::PutIntoCache -- Element already in the cache", nsnull));
+           ("[this=%p] imgLoader::PutIntoCache -- Element already in the cache", nullptr));
     nsRefPtr<imgRequest> tmpRequest = getter_AddRefs(tmpCacheEntry->GetRequest());
 
     // If it already exists, and we're putting the same key into the cache, we
     // should remove the old version.
     PR_LOG(gImgLog, PR_LOG_DEBUG,
-           ("[this=%p] imgLoader::PutIntoCache -- Replacing cached element", nsnull));
+           ("[this=%p] imgLoader::PutIntoCache -- Replacing cached element", nullptr));
 
     RemoveFromCache(key);
   } else {
     PR_LOG(gImgLog, PR_LOG_DEBUG,
-           ("[this=%p] imgLoader::PutIntoCache -- Element NOT already in the cache", nsnull));
+           ("[this=%p] imgLoader::PutIntoCache -- Element NOT already in the cache", nullptr));
   }
 
-  if (!cache.Put(spec, entry))
-    return false;
+  cache.Put(spec, entry);
 
   // We can be called to resurrect an evicted entry.
   if (entry->Evicted())
@@ -1167,7 +1092,7 @@ bool imgLoader::SetHasProxies(nsIURI *key)
   return false;
 }
 
-void imgLoader::CacheEntriesChanged(nsIURI *uri, PRInt32 sizediff /* = 0 */)
+void imgLoader::CacheEntriesChanged(nsIURI *uri, int32_t sizediff /* = 0 */)
 {
   imgCacheQueue &queue = GetCacheQueue(uri);
   queue.MarkDirty();
@@ -1215,7 +1140,7 @@ bool imgLoader::ValidateRequestWithNewChannel(imgRequest *request,
                                                 imgIRequest **aProxyRequest,
                                                 nsIChannelPolicy *aPolicy,
                                                 nsIPrincipal* aLoadingPrincipal,
-                                                PRInt32 aCORSMode)
+                                                int32_t aCORSMode)
 {
   // now we need to insert a new channel request object inbetween the real
   // request and the proxy that basically delays loading the image until it
@@ -1316,7 +1241,7 @@ bool imgLoader::ValidateRequestWithNewChannel(imgRequest *request,
     // Add the proxy without notifying
     hvc->AddProxy(proxy);
 
-    rv = newChannel->AsyncOpen(listener, nsnull);
+    rv = newChannel->AsyncOpen(listener, nullptr);
     if (NS_SUCCEEDED(rv))
       NS_ADDREF(*aProxyRequest = req.get());
 
@@ -1337,12 +1262,12 @@ bool imgLoader::ValidateEntry(imgCacheEntry *aEntry,
                                 imgIRequest **aProxyRequest,
                                 nsIChannelPolicy *aPolicy,
                                 nsIPrincipal* aLoadingPrincipal,
-                                PRInt32 aCORSMode)
+                                int32_t aCORSMode)
 {
   LOG_SCOPE(gImgLog, "imgLoader::ValidateEntry");
 
   bool hasExpired;
-  PRUint32 expirationTime = aEntry->GetExpiryTime();
+  uint32_t expirationTime = aEntry->GetExpiryTime();
   if (expirationTime <= SecondsFromPRTime(PR_Now())) {
     hasExpired = true;
   } else {
@@ -1354,12 +1279,12 @@ bool imgLoader::ValidateEntry(imgCacheEntry *aEntry,
   // Special treatment for file URLs - aEntry has expired if file has changed
   nsCOMPtr<nsIFileURL> fileUrl(do_QueryInterface(aURI));
   if (fileUrl) {
-    PRUint32 lastModTime = aEntry->GetTouchedTime();
+    uint32_t lastModTime = aEntry->GetTouchedTime();
 
     nsCOMPtr<nsIFile> theFile;
     rv = fileUrl->GetFile(getter_AddRefs(theFile));
     if (NS_SUCCEEDED(rv)) {
-      PRInt64 fileLastMod;
+      PRTime fileLastMod;
       rv = theFile->GetLastModifiedTime(&fileLastMod);
       if (NS_SUCCEEDED(rv)) {
         // nsIFile uses millisec, NSPR usec
@@ -1383,7 +1308,7 @@ bool imgLoader::ValidateEntry(imgCacheEntry *aEntry,
   // If the request's loadId is the same as the aCX, then it is ok to use
   // this one because it has already been validated for this context.
   //
-  // XXX: nsnull seems to be a 'special' key value that indicates that NO
+  // XXX: nullptr seems to be a 'special' key value that indicates that NO
   //      validation is required.
   //
   void *key = (void *)aCX;
@@ -1534,7 +1459,7 @@ nsresult imgLoader::EvictEntries(imgCacheTable &aCacheToClear)
   nsTArray<nsRefPtr<imgCacheEntry> > entries;
   aCacheToClear.Enumerate(EnumEvictEntries, &entries);
 
-  for (PRUint32 i = 0; i < entries.Length(); ++i)
+  for (uint32_t i = 0; i < entries.Length(); ++i)
     if (!RemoveFromCache(entries[i]))
       return NS_ERROR_FAILURE;
 
@@ -1551,7 +1476,7 @@ nsresult imgLoader::EvictEntries(imgCacheQueue &aQueueToClear)
   for (imgCacheQueue::const_iterator i = aQueueToClear.begin(); i != aQueueToClear.end(); ++i)
     entries.AppendElement(*i);
 
-  for (PRUint32 i = 0; i < entries.Length(); ++i)
+  for (uint32_t i = 0; i < entries.Length(); ++i)
     if (!RemoveFromCache(entries[i]))
       return NS_ERROR_FAILURE;
 
@@ -1592,7 +1517,7 @@ NS_IMETHODIMP imgLoader::LoadImage(nsIURI *aURI,
   aURI->GetSpec(spec);
   LOG_SCOPE_WITH_PARAM(gImgLog, "imgLoader::LoadImage", "aURI", spec.get());
 
-  *_retval = nsnull;
+  *_retval = nullptr;
 
   nsRefPtr<imgRequest> request;
 
@@ -1625,7 +1550,7 @@ NS_IMETHODIMP imgLoader::LoadImage(nsIURI *aURI,
     requestFlags |= nsIRequest::LOAD_BACKGROUND;
   }
 
-  PRInt32 corsmode = imgIRequest::CORS_NONE;
+  int32_t corsmode = imgIRequest::CORS_NONE;
   if (aLoadFlags & imgILoader::LOAD_CORS_ANONYMOUS) {
     corsmode = imgIRequest::CORS_ANONYMOUS;
   } else if (aLoadFlags & imgILoader::LOAD_CORS_USE_CREDENTIALS) {
@@ -1665,7 +1590,7 @@ NS_IMETHODIMP imgLoader::LoadImage(nsIURI *aURI,
     else {
       // We can't use this entry. We'll try to load it off the network, and if
       // successful, overwrite the old entry in the cache with a new one.
-      entry = nsnull;
+      entry = nullptr;
     }
   }
 
@@ -1740,7 +1665,7 @@ NS_IMETHODIMP imgLoader::LoadImage(nsIURI *aURI,
     PR_LOG(gImgLog, PR_LOG_DEBUG,
            ("[this=%p] imgLoader::LoadImage -- Calling channel->AsyncOpen()\n", this));
 
-    nsresult openRes = newChannel->AsyncOpen(listener, nsnull);
+    nsresult openRes = newChannel->AsyncOpen(listener, nullptr);
 
     if (NS_FAILED(openRes)) {
       PR_LOG(gImgLog, PR_LOG_DEBUG,
@@ -1846,9 +1771,9 @@ NS_IMETHODIMP imgLoader::LoadImageWithChannel(nsIChannel *channel, imgIDecoderOb
       //
       // XXX -- should this be changed? it's pretty much verbatim from the old
       // code, but seems nonsensical.
-      if (ValidateEntry(entry, uri, nsnull, nsnull, nsnull, aObserver, aCX,
-                        requestFlags, false, nsnull, nsnull, nsnull,
-                        nsnull, imgIRequest::CORS_NONE)) {
+      if (ValidateEntry(entry, uri, nullptr, nullptr, nullptr, aObserver, aCX,
+                        requestFlags, false, nullptr, nullptr, nullptr,
+                        nullptr, imgIRequest::CORS_NONE)) {
         request = getter_AddRefs(entry->GetRequest());
       } else {
         nsCOMPtr<nsICachingChannel> cacheChan(do_QueryInterface(channel));
@@ -1860,7 +1785,7 @@ NS_IMETHODIMP imgLoader::LoadImageWithChannel(nsIChannel *channel, imgIDecoderOb
           bUseCacheCopy = false;
 
         if (!bUseCacheCopy)
-          entry = nsnull;
+          entry = nullptr;
         else {
           request = getter_AddRefs(entry->GetRequest());
         }
@@ -1892,10 +1817,10 @@ NS_IMETHODIMP imgLoader::LoadImageWithChannel(nsIChannel *channel, imgIDecoderOb
 
     channel->Cancel(NS_ERROR_PARSED_DATA_CACHED); // this should fire an OnStopRequest
 
-    *listener = nsnull; // give them back a null nsIStreamListener
+    *listener = nullptr; // give them back a null nsIStreamListener
 
     rv = CreateNewProxyForRequest(request, loadGroup, aObserver,
-                                  requestFlags, nsnull, _retval);
+                                  requestFlags, nullptr, _retval);
     static_cast<imgRequestProxy*>(*_retval)->NotifyListener();
   } else {
     // Default to doing a principal check because we don't know who
@@ -1909,7 +1834,7 @@ NS_IMETHODIMP imgLoader::LoadImageWithChannel(nsIChannel *channel, imgIDecoderOb
 
     // No principal specified here, because we're not passed one.
     request->Init(originalURI, uri, channel, channel, entry,
-                  aCX, nsnull, imgIRequest::CORS_NONE);
+                  aCX, nullptr, imgIRequest::CORS_NONE);
 
     ProxyListener *pl = new ProxyListener(static_cast<nsIStreamListener *>(request.get()));
     NS_ADDREF(pl);
@@ -1923,7 +1848,7 @@ NS_IMETHODIMP imgLoader::LoadImageWithChannel(nsIChannel *channel, imgIDecoderOb
     PutIntoCache(originalURI, entry);
 
     rv = CreateNewProxyForRequest(request, loadGroup, aObserver,
-                                  requestFlags, nsnull, _retval);
+                                  requestFlags, nullptr, _retval);
 
     // Explicitly don't notify our proxy, because we're loading off the
     // network, and necko (or things called from necko, such as
@@ -1947,15 +1872,15 @@ NS_IMETHODIMP imgLoader::SupportImageWithMimeType(const char* aMimeType, bool *_
 }
 
 NS_IMETHODIMP imgLoader::GetMIMETypeFromContent(nsIRequest* aRequest,
-                                                const PRUint8* aContents,
-                                                PRUint32 aLength,
+                                                const uint8_t* aContents,
+                                                uint32_t aLength,
                                                 nsACString& aContentType)
 {
   return GetMimeTypeFromContent((const char*)aContents, aLength, aContentType);
 }
 
 /* static */
-nsresult imgLoader::GetMimeTypeFromContent(const char* aContents, PRUint32 aLength, nsACString& aContentType)
+nsresult imgLoader::GetMimeTypeFromContent(const char* aContents, uint32_t aLength, nsACString& aContentType)
 {
   /* Is it a GIF? */
   if (aLength >= 6 && (!nsCRT::strncmp(aContents, "GIF87a", 6) ||
@@ -2073,7 +1998,7 @@ NS_IMETHODIMP ProxyListener::OnStartRequest(nsIRequest *aRequest, nsISupports *c
           rv = convServ->AsyncConvertData("multipart/x-mixed-replace",
                                           "*/*",
                                           toListener,
-                                          nsnull,
+                                          nullptr,
                                           getter_AddRefs(fromListener));
           if (NS_SUCCEEDED(rv))
             mDestListener = fromListener;
@@ -2097,7 +2022,7 @@ NS_IMETHODIMP ProxyListener::OnStopRequest(nsIRequest *aRequest, nsISupports *ct
 /** nsIStreamListener methods **/
 
 /* void onDataAvailable (in nsIRequest request, in nsISupports ctxt, in nsIInputStream inStr, in unsigned long sourceOffset, in unsigned long count); */
-NS_IMETHODIMP ProxyListener::OnDataAvailable(nsIRequest *aRequest, nsISupports *ctxt, nsIInputStream *inStr, PRUint32 sourceOffset, PRUint32 count)
+NS_IMETHODIMP ProxyListener::OnDataAvailable(nsIRequest *aRequest, nsISupports *ctxt, nsIInputStream *inStr, uint32_t sourceOffset, uint32_t count)
 {
   if (!mDestListener)
     return NS_ERROR_FAILURE;
@@ -2129,7 +2054,7 @@ imgCacheValidator::imgCacheValidator(nsProgressNotificationProxy* progress,
 imgCacheValidator::~imgCacheValidator()
 {
   if (mRequest) {
-    mRequest->mValidator = nsnull;
+    mRequest->mValidator = nullptr;
   }
 }
 
@@ -2163,8 +2088,8 @@ NS_IMETHODIMP imgCacheValidator::OnStartRequest(nsIRequest *aRequest, nsISupport
       channelURI->Equals(mRequest->mCurrentURI, &sameURI);
 
     if (isFromCache && sameURI) {
-      PRUint32 count = mProxies.Count();
-      for (PRInt32 i = count-1; i>=0; i--) {
+      uint32_t count = mProxies.Count();
+      for (int32_t i = count-1; i>=0; i--) {
         imgRequestProxy *proxy = static_cast<imgRequestProxy *>(mProxies[i]);
 
         // Proxies waiting on cache validation should be deferring notifications.
@@ -2183,12 +2108,12 @@ NS_IMETHODIMP imgCacheValidator::OnStartRequest(nsIRequest *aRequest, nsISupport
       aRequest->Cancel(NS_BINDING_ABORTED);
 
       mRequest->SetLoadId(mContext);
-      mRequest->mValidator = nsnull;
+      mRequest->mValidator = nullptr;
 
-      mRequest = nsnull;
+      mRequest = nullptr;
 
-      mNewRequest = nsnull;
-      mNewEntry = nsnull;
+      mNewRequest = nullptr;
+      mNewEntry = nullptr;
 
       return NS_OK;
     }
@@ -2205,14 +2130,14 @@ NS_IMETHODIMP imgCacheValidator::OnStartRequest(nsIRequest *aRequest, nsISupport
   LOG_MSG_WITH_PARAM(gImgLog, "imgCacheValidator::OnStartRequest creating new request", "uri", spec.get());
 #endif
 
-  PRInt32 corsmode = mRequest->GetCORSMode();
+  int32_t corsmode = mRequest->GetCORSMode();
   nsCOMPtr<nsIPrincipal> loadingPrincipal = mRequest->GetLoadingPrincipal();
 
   // Doom the old request's cache entry
   mRequest->RemoveFromCache();
 
-  mRequest->mValidator = nsnull;
-  mRequest = nsnull;
+  mRequest->mValidator = nullptr;
+  mRequest = nullptr;
 
   // We use originalURI here to fulfil the imgIRequest contract on GetURI.
   nsCOMPtr<nsIURI> originalURI;
@@ -2228,8 +2153,8 @@ NS_IMETHODIMP imgCacheValidator::OnStartRequest(nsIRequest *aRequest, nsISupport
   // changes the caching behaviour for imgRequests.
   sImgLoader.PutIntoCache(originalURI, mNewEntry);
 
-  PRUint32 count = mProxies.Count();
-  for (PRInt32 i = count-1; i>=0; i--) {
+  uint32_t count = mProxies.Count();
+  for (int32_t i = count-1; i>=0; i--) {
     imgRequestProxy *proxy = static_cast<imgRequestProxy *>(mProxies[i]);
     proxy->ChangeOwner(mNewRequest);
 
@@ -2245,8 +2170,8 @@ NS_IMETHODIMP imgCacheValidator::OnStartRequest(nsIRequest *aRequest, nsISupport
     proxy->SyncNotifyListener();
   }
 
-  mNewRequest = nsnull;
-  mNewEntry = nsnull;
+  mNewRequest = nullptr;
+  mNewEntry = nullptr;
 
   return mDestListener->OnStartRequest(aRequest, ctxt);
 }
@@ -2264,12 +2189,12 @@ NS_IMETHODIMP imgCacheValidator::OnStopRequest(nsIRequest *aRequest, nsISupports
 
 
 /* void onDataAvailable (in nsIRequest request, in nsISupports ctxt, in nsIInputStream inStr, in unsigned long sourceOffset, in unsigned long count); */
-NS_IMETHODIMP imgCacheValidator::OnDataAvailable(nsIRequest *aRequest, nsISupports *ctxt, nsIInputStream *inStr, PRUint32 sourceOffset, PRUint32 count)
+NS_IMETHODIMP imgCacheValidator::OnDataAvailable(nsIRequest *aRequest, nsISupports *ctxt, nsIInputStream *inStr, uint32_t sourceOffset, uint32_t count)
 {
   if (!mDestListener) {
     // XXX see bug 113959
-    PRUint32 _retval;
-    inStr->ReadSegments(NS_DiscardSegment, nsnull, count, &_retval);
+    uint32_t _retval;
+    inStr->ReadSegments(NS_DiscardSegment, nullptr, count, &_retval);
     return NS_OK;
   }
 
@@ -2292,7 +2217,7 @@ NS_IMETHODIMP imgCacheValidator::GetInterface(const nsIID & aIID, void **aResult
 
 /** nsIChannelEventSink methods **/
 NS_IMETHODIMP imgCacheValidator::AsyncOnChannelRedirect(nsIChannel *oldChannel,
-                                                        nsIChannel *newChannel, PRUint32 flags,
+                                                        nsIChannel *newChannel, uint32_t flags,
                                                         nsIAsyncVerifyRedirectCallback *callback)
 {
   // Note all cache information we get from the old channel.
@@ -2310,8 +2235,8 @@ NS_IMETHODIMP imgCacheValidator::OnRedirectVerifyCallback(nsresult aResult)
   // If we've already been told to abort, just do so.
   if (NS_FAILED(aResult)) {
       mRedirectCallback->OnRedirectVerifyCallback(aResult);
-      mRedirectCallback = nsnull;
-      mRedirectChannel = nsnull;
+      mRedirectCallback = nullptr;
+      mRedirectChannel = nullptr;
       return NS_OK;
   }
 
@@ -2330,7 +2255,7 @@ NS_IMETHODIMP imgCacheValidator::OnRedirectVerifyCallback(nsresult aResult)
   }
 
   mRedirectCallback->OnRedirectVerifyCallback(result);
-  mRedirectCallback = nsnull;
-  mRedirectChannel = nsnull;
+  mRedirectCallback = nullptr;
+  mRedirectChannel = nullptr;
   return NS_OK;
 }
