@@ -14,18 +14,26 @@
 namespace js {
 namespace gc {
 
-/*
- * Sanity check that our compiled configuration matches the currently running
- * instance and initialize any runtime data needed for allocation.
- */
+// Sanity check that our compiled configuration matches the currently running
+// instance and initialize any runtime data needed for allocation.
 void InitMemorySubsystem();
 
+// Allocate or deallocate pages from the system with the given alignment.
 void *MapAlignedPages(size_t size, size_t alignment);
 void UnmapPages(void *p, size_t size);
 
+// Tell the OS that the given pages are not in use, so they should not
+// be written to a paging file. This may be a no-op on some platforms.
 bool MarkPagesUnused(void *p, size_t size);
+
+// Undo |MarkPagesUnused|: tell the OS that the given pages are of interest
+// and should be paged in and out normally. This may be a no-op on some
+// platforms.
 bool MarkPagesInUse(void *p, size_t size);
 
+// Returns #(hard faults) + #(soft faults)
+size_t GetPageFaultCount();
+
 } /* namespace gc */
 } /* namespace js */
 
