@@ -1,41 +1,8 @@
 /* -*- Mode: c++; c-basic-offset: 4; tab-width: 40; indent-tabs-mode: nil -*- */
 /* vim: set ts=40 sw=4 et tw=99: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is the Mozilla SpiderMonkey bytecode analysis
- *
- * The Initial Developer of the Original Code is
- *   Mozilla Foundation
- * Portions created by the Initial Developer are Copyright (C) 2010
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Brian Hackett <bhackett@mozilla.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "jsanalyze.h"
 #include "jsautooplen.h"
@@ -54,8 +21,10 @@ namespace analyze {
 
 #ifdef DEBUG
 void
-PrintBytecode(JSContext *cx, JSScript *script, jsbytecode *pc)
+PrintBytecode(JSContext *cx, JSScript *scriptArg, jsbytecode *pc)
 {
+    RootedScript script(cx, scriptArg);
+
     printf("#%u:", script->id());
     Sprinter sprinter(cx);
     if (!sprinter.init())
@@ -78,7 +47,7 @@ ScriptAnalysis::addJump(JSContext *cx, unsigned offset,
 
     Bytecode *&code = codeArray[offset];
     if (!code) {
-        code = cx->typeLifoAlloc().new_<Bytecode>();
+        code = cx->analysisLifoAlloc().new_<Bytecode>();
         if (!code) {
             setOOM(cx);
             return false;
@@ -127,15 +96,13 @@ ScriptAnalysis::analyzeBytecode(JSContext *cx)
 {
     JS_ASSERT(cx->compartment->activeAnalysis);
     JS_ASSERT(!ranBytecode());
-    LifoAlloc &tla = cx->typeLifoAlloc();
-
-    unsigned length = script->length;
-    unsigned nargs = script->function() ? script->function()->nargs : 0;
+    LifoAlloc &alloc = cx->analysisLifoAlloc();
 
     numSlots = TotalSlots(script);
 
-    codeArray = tla.newArray<Bytecode*>(length);
-    escapedSlots = tla.newArray<bool>(numSlots);
+    unsigned length = script->length;
+    codeArray = alloc.newArray<Bytecode*>(length);
+    escapedSlots = alloc.newArray<bool>(numSlots);
 
     if (!codeArray || !escapedSlots) {
         setOOM(cx);
@@ -149,34 +116,23 @@ ScriptAnalysis::analyzeBytecode(JSContext *cx)
      * other than through ARG* and LOCAL* opcodes (though arguments can still
      * be indirectly read but not written through 'arguments' properties).
      * All escaping locals are treated as having possible use-before-defs.
-     * Conservatively use 'hasArgsBinding' instead of 'needsArgsObj'
-     * (needsArgsObj requires SSA which requires escapedSlots).
+     * Conservatively use 'argumentsHasVarBinding' instead of 'needsArgsObj'
+     * (needsArgsObj requires SSA which requires escapedSlots). Lastly, the
+     * debugger can access any local at any time. Even though debugger
+     * reads/writes are monitored by the DebugScopeProxy, this monitoring
+     * updates the flow-insensitive type sets, so we cannot use SSA.
      */
 
     PodZero(escapedSlots, numSlots);
 
-    if (script->bindingsAccessedDynamically || script->compartment()->debugMode() ||
-        script->argumentsHasLocalBinding())
-    {
-        for (unsigned i = 0; i < nargs; i++)
-            escapedSlots[ArgSlot(i)] = true;
-    } else {
-        for (uint32_t i = 0; i < script->numClosedArgs(); i++) {
-            unsigned arg = script->getClosedArg(i);
-            JS_ASSERT(arg < nargs);
-            escapedSlots[ArgSlot(arg)] = true;
-        }
-    }
+    bool allVarsAliased = script->compartment()->debugMode();
+    bool allArgsAliased = allVarsAliased || script->argumentsHasVarBinding();
 
-    if (script->bindingsAccessedDynamically || script->compartment()->debugMode()) {
-        for (unsigned i = 0; i < script->nfixed; i++)
-            escapedSlots[LocalSlot(script, i)] = true;
-    } else {
-        for (uint32_t i = 0; i < script->numClosedVars(); i++) {
-            unsigned local = script->getClosedVar(i);
-            JS_ASSERT(local < script->nfixed);
-            escapedSlots[LocalSlot(script, local)] = true;
-        }
+    for (BindingIter bi(script->bindings); bi; bi++) {
+        if (bi->kind() == ARGUMENT)
+            escapedSlots[ArgSlot(bi.frameIndex())] = allArgsAliased || bi->aliased();
+        else
+            escapedSlots[LocalSlot(script, bi.frameIndex())] = allVarsAliased || bi->aliased();
     }
 
     /*
@@ -188,18 +144,14 @@ ScriptAnalysis::analyzeBytecode(JSContext *cx)
 
     bool heavyweight = script->function() && script->function()->isHeavyweight();
 
-    isCompileable = true;
+    isJaegerCompileable = true;
 
     isInlineable = true;
-    if (script->numClosedArgs() || script->numClosedVars() || heavyweight ||
-        script->bindingsAccessedDynamically || script->argumentsHasLocalBinding() ||
-        cx->compartment->debugMode())
-    {
+    if (heavyweight || script->argumentsHasVarBinding() || cx->compartment->debugMode())
         isInlineable = false;
-    }
 
     modifiesArguments_ = false;
-    if (script->numClosedArgs() || heavyweight)
+    if (heavyweight)
         modifiesArguments_ = true;
 
     canTrackVars = true;
@@ -221,7 +173,7 @@ ScriptAnalysis::analyzeBytecode(JSContext *cx)
     unsigned forwardCatch = 0;
 
     /* Fill in stack depth and definitions at initial bytecode. */
-    Bytecode *startcode = tla.new_<Bytecode>();
+    Bytecode *startcode = alloc.new_<Bytecode>();
     if (!startcode) {
         setOOM(cx);
         return;
@@ -318,10 +270,10 @@ ScriptAnalysis::analyzeBytecode(JSContext *cx)
          */
         if ((js_CodeSpec[op].format & JOF_TYPESET) && cx->typeInferenceEnabled()) {
             if (nTypeSets < script->nTypeSets) {
-                code->observedTypes = &typeArray[nTypeSets++];
+                code->observedTypes = typeArray[nTypeSets++].toStackTypeSet();
             } else {
                 JS_ASSERT(nTypeSets == UINT16_MAX);
-                code->observedTypes = &typeArray[nTypeSets - 1];
+                code->observedTypes = typeArray[nTypeSets - 1].toStackTypeSet();
             }
         }
 
@@ -340,49 +292,39 @@ ScriptAnalysis::analyzeBytecode(JSContext *cx)
 
           case JSOP_QNAMEPART:
           case JSOP_QNAMECONST:
-            isCompileable = false;
+            isJaegerCompileable = false;
+            /* FALL THROUGH */
           case JSOP_NAME:
           case JSOP_CALLNAME:
           case JSOP_BINDNAME:
           case JSOP_SETNAME:
           case JSOP_DELNAME:
-            usesScopeChain_ = true;
-            isInlineable = false;
-            break;
-
           case JSOP_GETALIASEDVAR:
           case JSOP_CALLALIASEDVAR:
           case JSOP_SETALIASEDVAR:
-            JS_ASSERT(!isInlineable);
             usesScopeChain_ = true;
-            /* XXX: this can be removed after bug 659577. */
-            if (ScopeCoordinate(pc).binding >= script->nfixed)
-                localsAliasStack_ = true;
+            isInlineable = false;
             break;
 
           case JSOP_DEFFUN:
           case JSOP_DEFVAR:
           case JSOP_DEFCONST:
           case JSOP_SETCONST:
-            extendsScope_ = true;
             isInlineable = canTrackVars = false;
             break;
 
           case JSOP_EVAL:
-            extendsScope_ = true;
             isInlineable = canTrackVars = false;
             break;
 
           case JSOP_ENTERWITH:
-            addsScopeObjects_ = true;
-            isCompileable = isInlineable = canTrackVars = false;
+            isJaegerCompileable = isInlineable = canTrackVars = false;
             break;
 
           case JSOP_ENTERLET0:
           case JSOP_ENTERLET1:
           case JSOP_ENTERBLOCK:
           case JSOP_LEAVEBLOCK:
-            addsScopeObjects_ = true;
             isInlineable = false;
             break;
 
@@ -518,6 +460,14 @@ ScriptAnalysis::analyzeBytecode(JSContext *cx)
             isInlineable = false;
             break;
 
+          case JSOP_GETPROP:
+          case JSOP_CALLPROP:
+          case JSOP_LENGTH:
+          case JSOP_GETELEM:
+          case JSOP_CALLELEM:
+            numPropertyReads_++;
+            break;
+
           /* Additional opcodes which can be compiled but which can't be inlined. */
           case JSOP_ARGUMENTS:
           case JSOP_THROW:
@@ -567,11 +517,6 @@ ScriptAnalysis::analyzeBytecode(JSContext *cx)
           case JSOP_TYPEOF:
           case JSOP_TYPEOFEXPR:
           case JSOP_VOID:
-          case JSOP_GETPROP:
-          case JSOP_CALLPROP:
-          case JSOP_LENGTH:
-          case JSOP_GETELEM:
-          case JSOP_CALLELEM:
           case JSOP_TOID:
           case JSOP_SETELEM:
           case JSOP_IMPLICITTHIS:
@@ -611,6 +556,8 @@ ScriptAnalysis::analyzeBytecode(JSContext *cx)
           case JSOP_RETRVAL:
           case JSOP_GETGNAME:
           case JSOP_CALLGNAME:
+          case JSOP_INTRINSICNAME:
+          case JSOP_CALLINTRINSIC:
           case JSOP_SETGNAME:
           case JSOP_REGEXP:
           case JSOP_OBJECT:
@@ -621,11 +568,12 @@ ScriptAnalysis::analyzeBytecode(JSContext *cx)
           case JSOP_HOLE:
           case JSOP_LOOPHEAD:
           case JSOP_LOOPENTRY:
+          case JSOP_ACTUALSFILLED:
             break;
 
           default:
             if (!(js_CodeSpec[op].format & JOF_DECOMPOSE))
-                isCompileable = isInlineable = false;
+                isJaegerCompileable = isInlineable = false;
             break;
         }
 
@@ -657,7 +605,7 @@ ScriptAnalysis::analyzeBytecode(JSContext *cx)
             Bytecode *&nextcode = codeArray[successorOffset];
 
             if (!nextcode) {
-                nextcode = tla.new_<Bytecode>();
+                nextcode = alloc.new_<Bytecode>();
                 if (!nextcode) {
                     setOOM(cx);
                     return;
@@ -689,6 +637,25 @@ ScriptAnalysis::analyzeBytecode(JSContext *cx)
      */
     if (!script->analyzedArgsUsage())
         analyzeSSA(cx);
+
+    /*
+     * If the script has JIT information (we are reanalyzing the script after
+     * a purge), add safepoints for the targets of any cross chunk edges in
+     * the script. These safepoints are normally added when the JITScript is
+     * constructed, but will have been lost during the purge.
+     */
+#ifdef JS_METHODJIT
+    mjit::JITScript *jit = NULL;
+    for (int constructing = 0; constructing <= 1 && !jit; constructing++) {
+        for (int barriers = 0; barriers <= 1 && !jit; barriers++)
+            jit = script->getJIT((bool) constructing, (bool) barriers);
+    }
+    if (jit) {
+        mjit::CrossChunkEdge *edges = jit->edges();
+        for (size_t i = 0; i < jit->nedges; i++)
+            getCode(edges[i].target).safePoint = true;
+    }
+#endif
 }
 
 /////////////////////////////////////////////////////////////////////
@@ -706,9 +673,9 @@ ScriptAnalysis::analyzeLifetimes(JSContext *cx)
             return;
     }
 
-    LifoAlloc &tla = cx->typeLifoAlloc();
+    LifoAlloc &alloc = cx->analysisLifoAlloc();
 
-    lifetimes = tla.newArray<LifetimeVariable>(numSlots);
+    lifetimes = alloc.newArray<LifetimeVariable>(numSlots);
     if (!lifetimes) {
         setOOM(cx);
         return;
@@ -830,7 +797,7 @@ ScriptAnalysis::analyzeLifetimes(JSContext *cx)
             /* Restore all saved variables. :FIXME: maybe do this precisely. */
             for (unsigned i = 0; i < savedCount; i++) {
                 LifetimeVariable &var = *saved[i];
-                var.lifetime = tla.new_<Lifetime>(offset, var.savedEnd, var.saved);
+                var.lifetime = alloc.new_<Lifetime>(offset, var.savedEnd, var.saved);
                 if (!var.lifetime) {
                     cx->free_(saved);
                     setOOM(cx);
@@ -897,7 +864,7 @@ ScriptAnalysis::analyzeLifetimes(JSContext *cx)
                 if (loop && loop->entry > loop->lastBlock)
                     loop->lastBlock = loop->entry;
 
-                LoopAnalysis *nloop = tla.new_<LoopAnalysis>();
+                LoopAnalysis *nloop = alloc.new_<LoopAnalysis>();
                 if (!nloop) {
                     cx->free_(saved);
                     setOOM(cx);
@@ -947,7 +914,7 @@ ScriptAnalysis::analyzeLifetimes(JSContext *cx)
                          * Jumping to a place where this variable is live. Make a new
                          * lifetime segment for the variable.
                          */
-                        var.lifetime = tla.new_<Lifetime>(offset, var.savedEnd, var.saved);
+                        var.lifetime = alloc.new_<Lifetime>(offset, var.savedEnd, var.saved);
                         if (!var.lifetime) {
                             cx->free_(saved);
                             setOOM(cx);
@@ -977,7 +944,7 @@ ScriptAnalysis::analyzeLifetimes(JSContext *cx)
     ranLifetimes_ = true;
 }
 
-#ifdef DEBUG
+#ifdef JS_METHODJIT_SPEW
 void
 LifetimeVariable::print() const
 {
@@ -1011,7 +978,7 @@ ScriptAnalysis::addVariable(JSContext *cx, LifetimeVariable &var, unsigned offse
                 }
             }
         }
-        var.lifetime = cx->typeLifoAlloc().new_<Lifetime>(offset, var.savedEnd, var.saved);
+        var.lifetime = cx->analysisLifoAlloc().new_<Lifetime>(offset, var.savedEnd, var.saved);
         if (!var.lifetime) {
             setOOM(cx);
             return;
@@ -1026,7 +993,7 @@ ScriptAnalysis::killVariable(JSContext *cx, LifetimeVariable &var, unsigned offs
 {
     if (!var.lifetime) {
         /* Make a point lifetime indicating the write. */
-        Lifetime *lifetime = cx->typeLifoAlloc().new_<Lifetime>(offset, var.savedEnd, var.saved);
+        Lifetime *lifetime = cx->analysisLifoAlloc().new_<Lifetime>(offset, var.savedEnd, var.saved);
         if (!lifetime) {
             setOOM(cx);
             return;
@@ -1057,7 +1024,7 @@ ScriptAnalysis::killVariable(JSContext *cx, LifetimeVariable &var, unsigned offs
          * We set the new interval's savedEnd to 0, since it will always be
          * adjacent to the old interval, so it never needs to be extended.
          */
-        var.lifetime = cx->typeLifoAlloc().new_<Lifetime>(start, 0, var.lifetime);
+        var.lifetime = cx->analysisLifoAlloc().new_<Lifetime>(start, 0, var.lifetime);
         if (!var.lifetime) {
             setOOM(cx);
             return;
@@ -1146,7 +1113,7 @@ ScriptAnalysis::extendVariable(JSContext *cx, LifetimeVariable &var,
         }
         JS_ASSERT(savedEnd <= end);
         if (savedEnd > segment->end) {
-            Lifetime *tail = cx->typeLifoAlloc().new_<Lifetime>(savedEnd, 0, segment->next);
+            Lifetime *tail = cx->analysisLifoAlloc().new_<Lifetime>(savedEnd, 0, segment->next);
             if (!tail) {
                 setOOM(cx);
                 return;
@@ -1223,7 +1190,7 @@ ScriptAnalysis::analyzeSSA(JSContext *cx)
             return;
     }
 
-    LifoAlloc &tla = cx->typeLifoAlloc();
+    LifoAlloc &alloc = cx->analysisLifoAlloc();
     unsigned maxDepth = script->nslots - script->nfixed;
 
     /*
@@ -1407,7 +1374,7 @@ ScriptAnalysis::analyzeSSA(JSContext *cx)
         unsigned xuses = ExtendedUse(pc) ? nuses + 1 : nuses;
 
         if (xuses) {
-            code->poppedValues = tla.newArray<SSAValue>(xuses);
+            code->poppedValues = alloc.newArray<SSAValue>(xuses);
             if (!code->poppedValues) {
                 setOOM(cx);
                 return;
@@ -1430,7 +1397,7 @@ ScriptAnalysis::analyzeSSA(JSContext *cx)
             }
 
             if (xuses) {
-                SSAUseChain *useChains = tla.newArray<SSAUseChain>(xuses);
+                SSAUseChain *useChains = alloc.newArray<SSAUseChain>(xuses);
                 if (!useChains) {
                     setOOM(cx);
                     return;
@@ -1457,7 +1424,7 @@ ScriptAnalysis::analyzeSSA(JSContext *cx)
 
         unsigned xdefs = ExtendedDef(pc) ? ndefs + 1 : ndefs;
         if (xdefs) {
-            code->pushedUses = tla.newArray<SSAUseChain *>(xdefs);
+            code->pushedUses = alloc.newArray<SSAUseChain *>(xdefs);
             if (!code->pushedUses) {
                 setOOM(cx);
                 return;
@@ -1500,6 +1467,11 @@ ScriptAnalysis::analyzeSSA(JSContext *cx)
             stack[stackDepth - 1].v = code->poppedValues[1];
             break;
 
+          case JSOP_SPREAD:
+          case JSOP_INITELEM_INC:
+            stack[stackDepth - 2].v = code->poppedValues[2];
+            break;
+
           case JSOP_INITELEM:
             stack[stackDepth - 1].v = code->poppedValues[2];
             break;
@@ -1565,7 +1537,7 @@ ScriptAnalysis::analyzeSSA(JSContext *cx)
             break;
           }
 
-          case JSOP_TRY: { 
+          case JSOP_TRY: {
             JSTryNote *tn = script->trynotes()->vector;
             JSTryNote *tnlimit = tn + script->trynotes()->length;
             for (; tn < tnlimit; tn++) {
@@ -1633,8 +1605,8 @@ PhiNodeCapacity(unsigned length)
 bool
 ScriptAnalysis::makePhi(JSContext *cx, uint32_t slot, uint32_t offset, SSAValue *pv)
 {
-    SSAPhiNode *node = cx->typeLifoAlloc().new_<SSAPhiNode>();
-    SSAValue *options = cx->typeLifoAlloc().newArray<SSAValue>(PhiNodeCapacity(0));
+    SSAPhiNode *node = cx->analysisLifoAlloc().new_<SSAPhiNode>();
+    SSAValue *options = cx->analysisLifoAlloc().newArray<SSAValue>(PhiNodeCapacity(0));
     if (!node || !options) {
         setOOM(cx);
         return false;
@@ -1666,7 +1638,7 @@ ScriptAnalysis::insertPhi(JSContext *cx, SSAValue &phi, const SSAValue &v)
     if (trackUseChain(v)) {
         SSAUseChain *&uses = useChain(v);
 
-        SSAUseChain *use = cx->typeLifoAlloc().new_<SSAUseChain>();
+        SSAUseChain *use = cx->analysisLifoAlloc().new_<SSAUseChain>();
         if (!use) {
             setOOM(cx);
             return;
@@ -1685,7 +1657,7 @@ ScriptAnalysis::insertPhi(JSContext *cx, SSAValue &phi, const SSAValue &v)
     }
 
     SSAValue *newOptions =
-        cx->typeLifoAlloc().newArray<SSAValue>(PhiNodeCapacity(node->length + 1));
+        cx->analysisLifoAlloc().newArray<SSAValue>(PhiNodeCapacity(node->length + 1));
     if (!newOptions) {
         setOOM(cx);
         return;
@@ -1885,7 +1857,7 @@ ScriptAnalysis::freezeNewValues(JSContext *cx, uint32_t offset)
         return;
     }
 
-    code.newValues = cx->typeLifoAlloc().newArray<SlotValue>(count + 1);
+    code.newValues = cx->analysisLifoAlloc().newArray<SlotValue>(count + 1);
     if (!code.newValues) {
         setOOM(cx);
         return;
@@ -1899,18 +1871,8 @@ ScriptAnalysis::freezeNewValues(JSContext *cx, uint32_t offset)
     cx->delete_(pending);
 }
 
-struct NeedsArgsObjState
-{
-    JSContext *cx;
-    Vector<SSAValue, 16> seen;
-    bool canOptimizeApply;
-    bool haveOptimizedApply;
-    NeedsArgsObjState(JSContext *cx)
-      : cx(cx), seen(cx), canOptimizeApply(true), haveOptimizedApply(false) {}
-};
-
 bool
-ScriptAnalysis::needsArgsObj(NeedsArgsObjState &state, const SSAValue &v)
+ScriptAnalysis::needsArgsObj(JSContext *cx, SeenVector &seen, const SSAValue &v)
 {
     /*
      * trackUseChain is false for initial values of variables, which
@@ -1919,18 +1881,18 @@ ScriptAnalysis::needsArgsObj(NeedsArgsObjState &state, const SSAValue &v)
     if (!trackUseChain(v))
         return false;
 
-    for (unsigned i = 0; i < state.seen.length(); i++) {
-        if (v == state.seen[i])
+    for (unsigned i = 0; i < seen.length(); i++) {
+        if (v == seen[i])
             return false;
     }
-    if (!state.seen.append(v)) {
-        state.cx->compartment->types.setPendingNukeTypes(state.cx);
+    if (!seen.append(v)) {
+        cx->compartment->types.setPendingNukeTypes(cx);
         return true;
     }
 
     SSAUseChain *use = useChain(v);
     while (use) {
-        if (needsArgsObj(state, use))
+        if (needsArgsObj(cx, seen, use))
             return true;
         use = use->next;
     }
@@ -1939,10 +1901,10 @@ ScriptAnalysis::needsArgsObj(NeedsArgsObjState &state, const SSAValue &v)
 }
 
 bool
-ScriptAnalysis::needsArgsObj(NeedsArgsObjState &state, SSAUseChain *use)
+ScriptAnalysis::needsArgsObj(JSContext *cx, SeenVector &seen, SSAUseChain *use)
 {
     if (!use->popped)
-        return needsArgsObj(state, SSAValue::PhiValue(use->offset, use->u.phi));
+        return needsArgsObj(cx, seen, SSAValue::PhiValue(use->offset, use->u.phi));
 
     jsbytecode *pc = script->code + use->offset;
     JSOp op = JSOp(*pc);
@@ -1950,27 +1912,21 @@ ScriptAnalysis::needsArgsObj(NeedsArgsObjState &state, SSAUseChain *use)
     if (op == JSOP_POP || op == JSOP_POPN)
         return false;
 
-#ifdef JS_METHODJIT
     /* SplatApplyArgs can read fp->canonicalActualArg(i) directly. */
-    if (state.canOptimizeApply && op == JSOP_FUNAPPLY && GET_ARGC(pc) == 2 && use->u.which == 0) {
+    if (op == JSOP_FUNAPPLY && GET_ARGC(pc) == 2 && use->u.which == 0) {
+#ifdef JS_METHODJIT
         JS_ASSERT(mjit::IsLowerableFunCallOrApply(pc));
-        state.haveOptimizedApply = true;
-        state.canOptimizeApply = false;
+#endif
         return false;
     }
-#endif
 
     /* arguments[i] can read fp->canonicalActualArg(i) directly. */
-    if (!state.haveOptimizedApply && op == JSOP_GETELEM && use->u.which == 1) {
-        state.canOptimizeApply = false;
+    if (op == JSOP_GETELEM && use->u.which == 1)
         return false;
-    }
 
     /* arguments.length length can read fp->numActualArgs() directly. */
-    if (!state.haveOptimizedApply && op == JSOP_LENGTH) {
-        state.canOptimizeApply = false;
+    if (op == JSOP_LENGTH)
         return false;
-    }
 
     /* Allow assignments to non-closed locals (but not arguments). */
 
@@ -1978,12 +1934,12 @@ ScriptAnalysis::needsArgsObj(NeedsArgsObjState &state, SSAUseChain *use)
         uint32_t slot = GetBytecodeSlot(script, pc);
         if (!trackSlot(slot))
             return true;
-        return needsArgsObj(state, SSAValue::PushedValue(use->offset, 0)) ||
-               needsArgsObj(state, SSAValue::WrittenVar(slot, use->offset));
+        return needsArgsObj(cx, seen, SSAValue::PushedValue(use->offset, 0)) ||
+               needsArgsObj(cx, seen, SSAValue::WrittenVar(slot, use->offset));
     }
 
     if (op == JSOP_GETLOCAL)
-        return needsArgsObj(state, SSAValue::PushedValue(use->offset, 0));
+        return needsArgsObj(cx, seen, SSAValue::PushedValue(use->offset, 0));
 
     return true;
 }
@@ -1991,20 +1947,23 @@ ScriptAnalysis::needsArgsObj(NeedsArgsObjState &state, SSAUseChain *use)
 bool
 ScriptAnalysis::needsArgsObj(JSContext *cx)
 {
-    JS_ASSERT(script->argumentsHasLocalBinding());
+    JS_ASSERT(script->argumentsHasVarBinding());
 
     /*
      * Since let variables and dynamic name access are not tracked, we cannot
-     * soundly perform this analysis in their presence. Also, debuggers may
-     * want to see 'arguments', so assume every arguments object escapes.
+     * soundly perform this analysis in their presence. Generators can be
+     * suspended when the speculation fails, so disallow it also.
      */
-    if (script->bindingsAccessedDynamically || localsAliasStack() || cx->compartment->debugMode())
+    if (script->bindingsAccessedDynamically || script->funHasAnyAliasedFormal ||
+        localsAliasStack() || cx->compartment->debugMode() || script->isGenerator)
+    {
         return true;
+    }
 
     unsigned pcOff = script->argumentsBytecode() - script->code;
 
-    NeedsArgsObjState state(cx);
-    return needsArgsObj(state, SSAValue::PushedValue(pcOff, 0));
+    SeenVector seen(cx);
+    return needsArgsObj(cx, seen, SSAValue::PushedValue(pcOff, 0));
 }
 
 CrossSSAValue
@@ -2077,7 +2036,7 @@ CrossScriptSSA::foldValue(const CrossSSAValue &cv)
              */
             ScriptAnalysis *analysis = frame.script->analysis();
             SSAValue toidv = analysis->poppedValue(pc, 0);
-            if (analysis->getValueTypes(toidv)->getKnownTypeTag(cx) == JSVAL_TYPE_INT32)
+            if (analysis->getValueTypes(toidv)->getKnownTypeTag() == JSVAL_TYPE_INT32)
                 return foldValue(CrossSSAValue(cv.frame, toidv));
             break;
           }
@@ -2137,7 +2096,7 @@ ScriptAnalysis::printSSA(JSContext *cx)
         }
     }
 
-    printf("\n"); 
+    printf("\n");
 }
 
 void
