@@ -1,41 +1,8 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
  *
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code, released
- * March 31, 1998.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifndef jsstrinlines_h___
 #define jsstrinlines_h___
@@ -53,7 +20,7 @@ namespace js {
 
 class RopeBuilder {
     JSContext *cx;
-    RootedVarString res;
+    RootedString res;
 
     RopeBuilder(const RopeBuilder &other) MOZ_DELETE;
     void operator=(const RopeBuilder &other) MOZ_DELETE;
@@ -79,8 +46,8 @@ class StringSegmentRange
      * If malloc() shows up in any profiles from this vector, we can add a new
      * StackAllocPolicy which stashes a reusable freed-at-gc buffer in the cx.
      */
-    Vector<JSString *, 32> stack;
-    JSLinearString *cur;
+    AutoStringVector stack;
+    Rooted<JSLinearString*> cur;
 
     bool settle(JSString *str) {
         while (str->isRope()) {
@@ -95,7 +62,7 @@ class StringSegmentRange
 
   public:
     StringSegmentRange(JSContext *cx)
-      : stack(cx), cur(NULL)
+      : stack(cx), cur(cx)
     {}
 
     JS_WARN_UNUSED_RESULT bool init(JSString *str) {
@@ -143,7 +110,7 @@ SkipSpace(const jschar *s, const jschar *end)
 inline bool
 CompareChars(const jschar *s1, size_t l1, const jschar *s2, size_t l2, int32_t *result)
 {
-    size_t n = JS_MIN(l1, l2);
+    size_t n = Min(l1, l2);
     for (size_t i = 0; i < n; i++) {
         if (int32_t cmp = s1[i] - s2[i]) {
             *result = cmp;
