@@ -1,42 +1,9 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
  * vim: set ts=8 sw=4 et tw=99:
  *
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is SpiderMonkey JavaScript shell.
- *
- * The Initial Developer of the Original Code is
- * Mozilla Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2010
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Christopher D. Leary <cdleary@mozilla.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "jsoptparse.h"
 #include <ctype.h>
@@ -120,7 +87,7 @@ PrintParagraph(const char *text, unsigned startColno, const unsigned limitColno,
         while (!isspace(*limit) && *limit != '\0')
             ++limit;
 
-        /* 
+        /*
          * If the current token is longer than the available number of columns,
          * then make a line break before printing the token.
          */
@@ -211,7 +178,7 @@ OptionParser::printHelp(const char *progname)
         size_t fmtChars = sizeof(fmt) - 2;
         size_t lhsLen = 0;
         for (Option **it = arguments.begin(), **end = arguments.end(); it != end; ++it)
-            lhsLen = JS_MAX(lhsLen, strlen((*it)->longflag) + fmtChars);
+            lhsLen = Max(lhsLen, strlen((*it)->longflag) + fmtChars);
 
         for (Option **it = arguments.begin(), **end = arguments.end(); it != end; ++it) {
             Option *arg = *it;
@@ -226,7 +193,7 @@ OptionParser::printHelp(const char *progname)
 
     if (!options.empty()) {
         printf("Options:\n");
-                                
+
         /* Calculate sizes for column alignment. */
         size_t lhsLen = 0;
         for (Option **it = options.begin(), **end = options.end(); it != end; ++it) {
@@ -239,7 +206,7 @@ OptionParser::printHelp(const char *progname)
             size_t len = fmtLen + longflagLen;
             if (opt->isValued())
                 len += strlen(opt->asValued()->metavar);
-            lhsLen = JS_MAX(lhsLen, len);
+            lhsLen = Max(lhsLen, len);
         }
 
         /* Print option help text. */
@@ -303,7 +270,7 @@ OptionParser::handleOption(Option *opt, size_t argc, char **argv, size_t *i, boo
         opt->asBoolOption()->value = true;
         return Okay;
       }
-      /* 
+      /*
        * Valued options are allowed to specify their values either via
        * successive arguments or a single --longflag=value argument.
        */
