@@ -1,43 +1,8 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 // vim:cindent:ts=8:et:sw=4:
-/*
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code, released
- * March 31, 1998.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   John Bandhauer <jband@netscape.com> (original author)
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /* Manage the shared info about interfaces for use by wrappedNatives. */
 
@@ -95,7 +60,7 @@ XPCNativeMember::Resolve(XPCCallContext& ccx, XPCNativeInterface* iface,
         jsval resultVal;
 
         if (!XPCConvert::NativeData2JS(ccx, &resultVal, &v.val, v.type,
-                                       nsnull, nsnull))
+                                       nullptr, nullptr))
             return false;
 
         *vp = resultVal;
@@ -153,7 +118,7 @@ XPCNativeInterface::GetNewOrUsed(XPCCallContext& ccx, const nsIID* iid)
 
     IID2NativeInterfaceMap* map = rt->GetIID2NativeInterfaceMap();
     if (!map)
-        return nsnull;
+        return nullptr;
 
     {   // scoped lock
         XPCAutoLock lock(rt->GetMapLock());
@@ -166,11 +131,11 @@ XPCNativeInterface::GetNewOrUsed(XPCCallContext& ccx, const nsIID* iid)
     nsCOMPtr<nsIInterfaceInfo> info;
     ccx.GetXPConnect()->GetInfoForIID(iid, getter_AddRefs(info));
     if (!info)
-        return nsnull;
+        return nullptr;
 
     iface = NewInstance(ccx, info);
     if (!iface)
-        return nsnull;
+        return nullptr;
 
     {   // scoped lock
         XPCAutoLock lock(rt->GetMapLock());
@@ -178,7 +143,7 @@ XPCNativeInterface::GetNewOrUsed(XPCCallContext& ccx, const nsIID* iid)
         if (!iface2) {
             NS_ERROR("failed to add our interface!");
             DestroyInstance(iface);
-            iface = nsnull;
+            iface = nullptr;
         } else if (iface2 != iface) {
             DestroyInstance(iface);
             iface = iface2;
@@ -196,13 +161,13 @@ XPCNativeInterface::GetNewOrUsed(XPCCallContext& ccx, nsIInterfaceInfo* info)
 
     const nsIID* iid;
     if (NS_FAILED(info->GetIIDShared(&iid)) || !iid)
-        return nsnull;
+        return nullptr;
 
     XPCJSRuntime* rt = ccx.GetRuntime();
 
     IID2NativeInterfaceMap* map = rt->GetIID2NativeInterfaceMap();
     if (!map)
-        return nsnull;
+        return nullptr;
 
     {   // scoped lock
         XPCAutoLock lock(rt->GetMapLock());
@@ -214,7 +179,7 @@ XPCNativeInterface::GetNewOrUsed(XPCCallContext& ccx, nsIInterfaceInfo* info)
 
     iface = NewInstance(ccx, info);
     if (!iface)
-        return nsnull;
+        return nullptr;
 
     {   // scoped lock
         XPCAutoLock lock(rt->GetMapLock());
@@ -222,7 +187,7 @@ XPCNativeInterface::GetNewOrUsed(XPCCallContext& ccx, nsIInterfaceInfo* info)
         if (!iface2) {
             NS_ERROR("failed to add our interface!");
             DestroyInstance(iface);
-            iface = nsnull;
+            iface = nullptr;
         } else if (iface2 != iface) {
             DestroyInstance(iface);
             iface = iface2;
@@ -238,7 +203,7 @@ XPCNativeInterface::GetNewOrUsed(XPCCallContext& ccx, const char* name)
 {
     nsCOMPtr<nsIInterfaceInfo> info;
     ccx.GetXPConnect()->GetInfoForName(name, getter_AddRefs(info));
-    return info ? GetNewOrUsed(ccx, info) : nsnull;
+    return info ? GetNewOrUsed(ccx, info) : nullptr;
 }
 
 // static
@@ -254,17 +219,17 @@ XPCNativeInterface*
 XPCNativeInterface::NewInstance(XPCCallContext& ccx,
                                 nsIInterfaceInfo* aInfo)
 {
-    static const PRUint16 MAX_LOCAL_MEMBER_COUNT = 16;
+    static const uint16_t MAX_LOCAL_MEMBER_COUNT = 16;
     XPCNativeMember local_members[MAX_LOCAL_MEMBER_COUNT];
-    XPCNativeInterface* obj = nsnull;
-    XPCNativeMember* members = nsnull;
+    XPCNativeInterface* obj = nullptr;
+    XPCNativeMember* members = nullptr;
 
     int i;
     JSBool failed = false;
-    PRUint16 constCount;
-    PRUint16 methodCount;
-    PRUint16 totalCount;
-    PRUint16 realTotalCount = 0;
+    uint16_t constCount;
+    uint16_t methodCount;
+    uint16_t totalCount;
+    uint16_t realTotalCount = 0;
     XPCNativeMember* cur;
     JSString* str = NULL;
     jsid name;
@@ -279,11 +244,11 @@ XPCNativeInterface::NewInstance(XPCCallContext& ccx,
 
     bool canScript;
     if (NS_FAILED(aInfo->IsScriptable(&canScript)) || !canScript)
-        return nsnull;
+        return nullptr;
 
     if (NS_FAILED(aInfo->GetMethodCount(&methodCount)) ||
         NS_FAILED(aInfo->GetConstantCount(&constCount)))
-        return nsnull;
+        return nullptr;
 
     // If the interface does not have nsISupports in its inheritance chain
     // then we know we can't reflect its methods. However, some interfaces that
@@ -298,7 +263,7 @@ XPCNativeInterface::NewInstance(XPCCallContext& ccx,
     if (totalCount > MAX_LOCAL_MEMBER_COUNT) {
         members = new XPCNativeMember[totalCount];
         if (!members)
-            return nsnull;
+            return nullptr;
     } else {
         members = local_members;
     }
@@ -377,7 +342,7 @@ XPCNativeInterface::NewInstance(XPCCallContext& ccx,
     if (!failed) {
         const char* bytes;
         if (NS_FAILED(aInfo->GetNameShared(&bytes)) || !bytes ||
-            nsnull == (str = JS_InternString(ccx, bytes))) {
+            nullptr == (str = JS_InternString(ccx, bytes))) {
             failed = true;
         }
         interfaceName = INTERNED_STRING_TO_JSID(ccx, str);
@@ -423,7 +388,7 @@ XPCNativeInterface::SizeOfIncludingThis(nsMallocSizeOfFun mallocSizeOf)
 }
 
 void
-XPCNativeInterface::DebugDump(PRInt16 depth)
+XPCNativeInterface::DebugDump(int16_t depth)
 {
 #ifdef DEBUG
     depth--;
@@ -448,14 +413,14 @@ XPCNativeSet::GetNewOrUsed(XPCCallContext& ccx, const nsIID* iid)
     AutoMarkingNativeInterfacePtr iface(ccx);
     iface = XPCNativeInterface::GetNewOrUsed(ccx, iid);
     if (!iface)
-        return nsnull;
+        return nullptr;
 
-    XPCNativeSetKey key(nsnull, iface, 0);
+    XPCNativeSetKey key(nullptr, iface, 0);
 
     XPCJSRuntime* rt = ccx.GetRuntime();
     NativeSetMap* map = rt->GetNativeSetMap();
     if (!map)
-        return nsnull;
+        return nullptr;
 
     {   // scoped lock
         XPCAutoLock lock(rt->GetMapLock());
@@ -469,7 +434,7 @@ XPCNativeSet::GetNewOrUsed(XPCCallContext& ccx, const nsIID* iid)
     XPCNativeInterface* temp[] = {iface};
     set = NewInstance(ccx, temp, 1);
     if (!set)
-        return nsnull;
+        return nullptr;
 
     {   // scoped lock
         XPCAutoLock lock(rt->GetMapLock());
@@ -477,7 +442,7 @@ XPCNativeSet::GetNewOrUsed(XPCCallContext& ccx, const nsIID* iid)
         if (!set2) {
             NS_ERROR("failed to add our set!");
             DestroyInstance(set);
-            set = nsnull;
+            set = nullptr;
         } else if (set2 != set) {
             DestroyInstance(set);
             set = set2;
@@ -496,7 +461,7 @@ XPCNativeSet::GetNewOrUsed(XPCCallContext& ccx, nsIClassInfo* classInfo)
 
     ClassInfo2NativeSetMap* map = rt->GetClassInfo2NativeSetMap();
     if (!map)
-        return nsnull;
+        return nullptr;
 
     {   // scoped lock
         XPCAutoLock lock(rt->GetMapLock());
@@ -506,9 +471,9 @@ XPCNativeSet::GetNewOrUsed(XPCCallContext& ccx, nsIClassInfo* classInfo)
     if (set)
         return set;
 
-    nsIID** iidArray = nsnull;
+    nsIID** iidArray = nullptr;
     AutoMarkingNativeInterfacePtrArrayPtr interfaceArray(ccx);
-    PRUint32 iidCount = 0;
+    uint32_t iidCount = 0;
 
     if (NS_FAILED(classInfo->GetInterfaces(&iidCount, &iidArray))) {
         // Note: I'm making it OK for this call to fail so that one can add
@@ -516,7 +481,7 @@ XPCNativeSet::GetNewOrUsed(XPCCallContext& ccx, nsIClassInfo* classInfo)
         // method to be implemented.
 
         // Make sure these are set correctly...
-        iidArray = nsnull;
+        iidArray = nullptr;
         iidCount = 0;
     }
 
@@ -534,9 +499,9 @@ XPCNativeSet::GetNewOrUsed(XPCCallContext& ccx, nsIClassInfo* classInfo)
 
         XPCNativeInterface** currentInterface = interfaceArray;
         nsIID**              currentIID = iidArray;
-        PRUint16             interfaceCount = 0;
+        uint16_t             interfaceCount = 0;
 
-        for (PRUint32 i = 0; i < iidCount; i++) {
+        for (uint32_t i = 0; i < iidCount; i++) {
             nsIID* iid = *(currentIID++);
             if (!iid) {
                 NS_ERROR("Null found in classinfo interface list");
@@ -562,7 +527,7 @@ XPCNativeSet::GetNewOrUsed(XPCCallContext& ccx, nsIClassInfo* classInfo)
                 if (!map2)
                     goto out;
 
-                XPCNativeSetKey key(set, nsnull, 0);
+                XPCNativeSetKey key(set, nullptr, 0);
 
                 {   // scoped lock
                     XPCAutoLock lock(rt->GetMapLock());
@@ -570,7 +535,7 @@ XPCNativeSet::GetNewOrUsed(XPCCallContext& ccx, nsIClassInfo* classInfo)
                     if (!set2) {
                         NS_ERROR("failed to add our set!");
                         DestroyInstance(set);
-                        set = nsnull;
+                        set = nullptr;
                         goto out;
                     }
                     if (set2 != set) {
@@ -623,13 +588,13 @@ XPCNativeSet*
 XPCNativeSet::GetNewOrUsed(XPCCallContext& ccx,
                            XPCNativeSet* otherSet,
                            XPCNativeInterface* newInterface,
-                           PRUint16 position)
+                           uint16_t position)
 {
     AutoMarkingNativeSetPtr set(ccx);
     XPCJSRuntime* rt = ccx.GetRuntime();
     NativeSetMap* map = rt->GetNativeSetMap();
     if (!map)
-        return nsnull;
+        return nullptr;
 
     XPCNativeSetKey key(otherSet, newInterface, position);
 
@@ -647,7 +612,7 @@ XPCNativeSet::GetNewOrUsed(XPCCallContext& ccx,
         set = NewInstance(ccx, &newInterface, 1);
 
     if (!set)
-        return nsnull;
+        return nullptr;
 
     {   // scoped lock
         XPCAutoLock lock(rt->GetMapLock());
@@ -655,7 +620,7 @@ XPCNativeSet::GetNewOrUsed(XPCCallContext& ccx,
         if (!set2) {
             NS_ERROR("failed to add our set!");
             DestroyInstance(set);
-            set = nsnull;
+            set = nullptr;
         } else if (set2 != set) {
             DestroyInstance(set);
             set = set2;
@@ -673,8 +638,8 @@ XPCNativeSet::GetNewOrUsed(XPCCallContext& ccx,
                            bool preserveFirstSetOrder)
 {
     // Figure out how many interfaces we'll need in the new set.
-    PRUint32 uniqueCount = firstSet->mInterfaceCount;
-    for (PRUint32 i = 0; i < secondSet->mInterfaceCount; ++i) {
+    uint32_t uniqueCount = firstSet->mInterfaceCount;
+    for (uint32_t i = 0; i < secondSet->mInterfaceCount; ++i) {
         if (!firstSet->HasInterface(secondSet->mInterfaces[i]))
             uniqueCount++;
     }
@@ -697,14 +662,14 @@ XPCNativeSet::GetNewOrUsed(XPCCallContext& ccx,
     // existing set. So let's just do the slow and easy thing and hope that the
     // above optimizations handle the common cases.
     XPCNativeSet* currentSet = firstSet;
-    for (PRUint32 i = 0; i < secondSet->mInterfaceCount; ++i) {
+    for (uint32_t i = 0; i < secondSet->mInterfaceCount; ++i) {
         XPCNativeInterface* iface = secondSet->mInterfaces[i];
         if (!currentSet->HasInterface(iface)) {
             // Create a new augmented set, inserting this interface at the end.
-            PRUint32 pos = currentSet->mInterfaceCount;
+            uint32_t pos = currentSet->mInterfaceCount;
             currentSet = XPCNativeSet::GetNewOrUsed(ccx, currentSet, iface, pos);
             if (!currentSet)
-                return nsnull;
+                return nullptr;
         }
     }
 
@@ -717,12 +682,12 @@ XPCNativeSet::GetNewOrUsed(XPCCallContext& ccx,
 XPCNativeSet*
 XPCNativeSet::NewInstance(XPCCallContext& ccx,
                           XPCNativeInterface** array,
-                          PRUint16 count)
+                          uint16_t count)
 {
-    XPCNativeSet* obj = nsnull;
+    XPCNativeSet* obj = nullptr;
 
     if (!array || !count)
-        return nsnull;
+        return nullptr;
 
     // We impose the invariant:
     // "All sets have exactly one nsISupports interface and it comes first."
@@ -730,9 +695,9 @@ XPCNativeSet::NewInstance(XPCCallContext& ccx,
     // that don't exactly follow the rule.
 
     XPCNativeInterface* isup = XPCNativeInterface::GetISupports(ccx);
-    PRUint16 slots = count+1;
+    uint16_t slots = count+1;
 
-    PRUint16 i;
+    uint16_t i;
     XPCNativeInterface** pcur;
 
     for (i = 0, pcur = array; i < count; i++, pcur++) {
@@ -753,7 +718,7 @@ XPCNativeSet::NewInstance(XPCCallContext& ccx,
         // Stick the nsISupports in front and skip additional nsISupport(s)
         XPCNativeInterface** inp = array;
         XPCNativeInterface** outp = (XPCNativeInterface**) &obj->mInterfaces;
-        PRUint16 memberCount = 1;   // for the one member in nsISupports
+        uint16_t memberCount = 1;   // for the one member in nsISupports
 
         *(outp++) = isup;
 
@@ -776,14 +741,14 @@ XPCNativeSet::NewInstance(XPCCallContext& ccx,
 XPCNativeSet*
 XPCNativeSet::NewInstanceMutate(XPCNativeSet*       otherSet,
                                 XPCNativeInterface* newInterface,
-                                PRUint16            position)
+                                uint16_t            position)
 {
-    XPCNativeSet* obj = nsnull;
+    XPCNativeSet* obj = nullptr;
 
     if (!newInterface)
-        return nsnull;
+        return nullptr;
     if (otherSet && position > otherSet->mInterfaceCount)
-        return nsnull;
+        return nullptr;
 
     // Use placement new to create an object with the right amount of space
     // to hold the members array
@@ -802,7 +767,7 @@ XPCNativeSet::NewInstanceMutate(XPCNativeSet*       otherSet,
 
             XPCNativeInterface** src = otherSet->mInterfaces;
             XPCNativeInterface** dest = obj->mInterfaces;
-            for (PRUint16 i = 0; i < obj->mInterfaceCount; i++) {
+            for (uint16_t i = 0; i < obj->mInterfaceCount; i++) {
                 if (i == position)
                     *dest++ = newInterface;
                 else
@@ -833,7 +798,7 @@ XPCNativeSet::SizeOfIncludingThis(nsMallocSizeOfFun mallocSizeOf)
 }
 
 void
-XPCNativeSet::DebugDump(PRInt16 depth)
+XPCNativeSet::DebugDump(int16_t depth)
 {
 #ifdef DEBUG
     depth--;
@@ -842,7 +807,7 @@ XPCNativeSet::DebugDump(PRInt16 depth)
 
         XPC_LOG_ALWAYS(("mInterfaceCount of %d", mInterfaceCount));
         if (depth) {
-            for (PRUint16 i = 0; i < mInterfaceCount; i++)
+            for (uint16_t i = 0; i < mInterfaceCount; i++)
                 mInterfaces[i]->DebugDump(depth);
         }
         XPC_LOG_ALWAYS(("mMemberCount of %d", mMemberCount));
