@@ -1,41 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   L. David Baron <dbaron@dbaron.org>, Mozilla Corporation
- *   Ehsan Akhgari <ehsan.akhgari@gmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /* a presentation of a document, part 1 */
 
@@ -48,11 +14,12 @@
 #include "nsIContentViewer.h"
 #include "nsPIDOMWindow.h"
 #include "nsStyleSet.h"
-#include "nsImageLoader.h"
 #include "nsIContent.h"
 #include "nsIFrame.h"
 #include "nsIURL.h"
 #include "nsIDocument.h"
+#include "nsIPrintSettings.h"
+#include "nsILanguageAtomService.h"
 #include "nsStyleContext.h"
 #include "mozilla/LookAndFeel.h"
 #include "nsIComponentManager.h"
@@ -67,7 +34,6 @@
 #include "nsIWeakReferenceUtils.h"
 #include "nsCSSRendering.h"
 #include "prprf.h"
-#include "nsIDOMDocument.h"
 #include "nsAutoPtr.h"
 #include "nsEventStateManager.h"
 #include "nsThreadUtils.h"
@@ -87,16 +53,16 @@
 #include "nsStyleStructInlines.h"
 #include "nsIAppShell.h"
 #include "prenv.h"
-#include "nsIPrivateDOMEvent.h"
 #include "nsIDOMEventTarget.h"
 #include "nsObjectFrame.h"
 #include "nsTransitionManager.h"
 #include "nsAnimationManager.h"
 #include "mozilla/dom/Element.h"
-#include "nsIFrameMessageManager.h"
+#include "nsIMessageManager.h"
 #include "FrameLayerBuilder.h"
 #include "nsDOMMediaQueryList.h"
 #include "nsSMILAnimationController.h"
+#include "mozilla/css/ImageLoader.h"
 
 #ifdef IBMBIDI
 #include "nsBidiPresUtils.h"
@@ -143,18 +109,20 @@ private:
 
 } // anonymous namespace
 
-static nscolor
-MakeColorPref(const nsString& aColor)
+nscolor
+nsPresContext::MakeColorPref(const nsString& aColor)
 {
-  nscolor color;
   nsCSSParser parser;
-  nsresult rv =
-    parser.ParseColorString(aColor, nsnull, 0, &color);
-  if (NS_FAILED(rv)) {
+  nsCSSValue value;
+  if (!parser.ParseColorString(aColor, nullptr, 0, value)) {
     // Any better choices?
-    color = NS_RGB(0, 0, 0);
+    return NS_RGB(0, 0, 0);
   }
-  return color;
+
+  nscolor color;
+  return nsRuleNode::ComputeColor(value, this, nullptr, color)
+    ? color
+    : NS_RGB(0, 0, 0);
 }
 
 int
@@ -162,8 +130,8 @@ nsPresContext::PrefChangedCallback(const char* aPrefName, void* instance_data)
 {
   nsPresContext*  presContext = (nsPresContext*)instance_data;
 
-  NS_ASSERTION(nsnull != presContext, "bad instance data");
-  if (nsnull != presContext) {
+  NS_ASSERTION(nullptr != presContext, "bad instance data");
+  if (nullptr != presContext) {
     presContext->PreferenceChanged(aPrefName);
   }
   return 0;  // PREF_OK
@@ -174,7 +142,7 @@ void
 nsPresContext::PrefChangedUpdateTimerCallback(nsITimer *aTimer, void *aClosure)
 {
   nsPresContext*  presContext = (nsPresContext*)aClosure;
-  NS_ASSERTION(presContext != nsnull, "bad instance data");
+  NS_ASSERTION(presContext != nullptr, "bad instance data");
   if (presContext)
     presContext->UpdateAfterPreferencesChanged();
 }
@@ -194,14 +162,6 @@ IsVisualCharset(const nsCString& aCharset)
 }
 #endif // IBMBIDI
 
-
-static PLDHashOperator
-destroy_loads(nsIFrame* aKey, nsRefPtr<nsImageLoader>& aData, void* closure)
-{
-  aData->Destroy();
-  return PL_DHASH_NEXT;
-}
-
 #include "nsContentCID.h"
 
   // NOTE! nsPresContext::operator new() zeroes out all members, so don't
@@ -256,7 +216,7 @@ nsPresContext::nsPresContext(nsIDocument* aDocument, nsPresContextType aType)
     mNeverAnimate = false;
   }
   NS_ASSERTION(mDocument, "Null document");
-  mUserFontSet = nsnull;
+  mUserFontSet = nullptr;
   mUserFontSetDirty = true;
 
   PR_INIT_CLIST(&mDOMMediaQueryLists);
@@ -265,7 +225,7 @@ nsPresContext::nsPresContext(nsIDocument* aDocument, nsPresContextType aType)
 nsPresContext::~nsPresContext()
 {
   NS_PRECONDITION(!mShell, "Presshell forgot to clear our mShell pointer");
-  SetShell(nsnull);
+  SetShell(nullptr);
 
   NS_ABORT_IF_FALSE(PR_CLIST_IS_EMPTY(&mDOMMediaQueryLists),
                     "must not have media query lists left");
@@ -279,15 +239,13 @@ nsPresContext::~nsPresContext()
   if (mEventManager) {
     // unclear if these are needed, but can't hurt
     mEventManager->NotifyDestroyPresContext(this);
-    mEventManager->SetPresContext(nsnull);
-
-    NS_RELEASE(mEventManager);
+    mEventManager->SetPresContext(nullptr);
   }
 
   if (mPrefChangedTimer)
   {
     mPrefChangedTimer->Cancel();
-    mPrefChangedTimer = nsnull;
+    mPrefChangedTimer = nullptr;
   }
 
   // Unregister preference callbacks
@@ -329,9 +287,6 @@ nsPresContext::~nsPresContext()
   Preferences::UnregisterCallback(nsPresContext::PrefChangedCallback,
                                   "layout.css.devPixelsPerPx",
                                   this);
-
-  NS_IF_RELEASE(mDeviceContext);
-  NS_IF_RELEASE(mLanguage);
 }
 
 NS_IMPL_CYCLE_COLLECTION_CLASS(nsPresContext)
@@ -344,28 +299,12 @@ NS_INTERFACE_MAP_END
 NS_IMPL_CYCLE_COLLECTING_ADDREF(nsPresContext)
 NS_IMPL_CYCLE_COLLECTING_RELEASE(nsPresContext)
 
-static PLDHashOperator
-TraverseImageLoader(nsIFrame* aKey, nsRefPtr<nsImageLoader>& aData,
-                    void* aClosure)
-{
-  nsCycleCollectionTraversalCallback *cb =
-    static_cast<nsCycleCollectionTraversalCallback*>(aClosure);
-
-  NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(*cb, "mImageLoaders[i] item");
-  cb->NoteXPCOMChild(aData);
-
-  return PL_DHASH_NEXT;
-}
-
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsPresContext)
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mDocument);
   // NS_IMPL_CYCLE_COLLECTION_TRAVERSE_RAWPTR(mDeviceContext); // not xpcom
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR_AMBIGUOUS(mEventManager, nsIObserver);
   // NS_IMPL_CYCLE_COLLECTION_TRAVERSE_RAWPTR(mLanguage); // an atom
 
-  for (PRUint32 i = 0; i < IMAGE_LOAD_TYPE_COUNT; ++i)
-    tmp->mImageLoaders[i].Enumerate(TraverseImageLoader, &cb);
-
   // We own only the items in mDOMMediaQueryLists that have listeners;
   // this reference is managed by their AddListener and RemoveListener
   // methods.
@@ -386,13 +325,12 @@ NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(nsPresContext)
   NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mDocument);
-  NS_RELEASE(tmp->mDeviceContext); // worth bothering?
+  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mDeviceContext); // worth bothering?
   if (tmp->mEventManager) {
     // unclear if these are needed, but can't hurt
     tmp->mEventManager->NotifyDestroyPresContext(tmp);
-    tmp->mEventManager->SetPresContext(nsnull);
-
-    NS_RELEASE(tmp->mEventManager);
+    tmp->mEventManager->SetPresContext(nullptr);
+    tmp->mEventManager = nullptr;
   }
 
   // We own only the items in mDOMMediaQueryLists that have listeners;
@@ -414,7 +352,7 @@ NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(nsPresContext)
   if (tmp->mPrefChangedTimer)
   {
     tmp->mPrefChangedTimer->Cancel();
-    tmp->mPrefChangedTimer = nsnull;
+    tmp->mPrefChangedTimer = nullptr;
   }
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
@@ -453,7 +391,7 @@ nsPresContext::GetFontPrefsForLang(nsIAtom *aLanguage) const
   // Get language group for aLanguage:
 
   nsresult rv;
-  nsIAtom *langGroupAtom = nsnull;
+  nsIAtom *langGroupAtom = nullptr;
   if (!aLanguage) {
     aLanguage = mLanguage;
   }
@@ -474,7 +412,7 @@ nsPresContext::GetFontPrefsForLang(nsIAtom *aLanguage) const
   LangGroupFontPrefs *prefs =
     const_cast<LangGroupFontPrefs*>(&mLangGroupFontPrefs);
   if (prefs->mLangGroup) { // if initialized
-    DebugOnly<PRUint32> count = 0;
+    DebugOnly<uint32_t> count = 0;
     for (;;) {
       NS_ASSERTION(++count < 35, "Lang group count exceeded!!!");
       if (prefs->mLangGroup == langGroupAtom) {
@@ -518,7 +456,7 @@ nsPresContext::GetFontPrefsForLang(nsIAtom *aLanguage) const
 
   // get the current applicable font-size unit
   enum {eUnit_unknown = -1, eUnit_px, eUnit_pt};
-  PRInt32 unit = eUnit_px;
+  int32_t unit = eUnit_px;
 
   nsAdoptingCString cvalue =
     Preferences::GetCString("font.size.unit");
@@ -542,7 +480,7 @@ nsPresContext::GetFontPrefsForLang(nsIAtom *aLanguage) const
 
   MAKE_FONT_PREF_KEY(pref, "font.minimum-size.", langGroup);
 
-  PRInt32 size = Preferences::GetInt(pref.get());
+  int32_t size = Preferences::GetInt(pref.get());
   if (unit == eUnit_px) {
     prefs->mMinimumFontSize = CSSPixelsToAppUnits(size);
   }
@@ -568,7 +506,7 @@ nsPresContext::GetFontPrefsForLang(nsIAtom *aLanguage) const
   // code to look up the font prefs to convert generic names to specific
   // family names as necessary.
   nsCAutoString generic_dot_langGroup;
-  for (PRUint32 eType = 0; eType < ArrayLength(fontTypes); ++eType) {
+  for (uint32_t eType = 0; eType < ArrayLength(fontTypes); ++eType) {
     generic_dot_langGroup.Assign(kGenericFont[eType]);
     generic_dot_langGroup.Append(langGroup);
 
@@ -646,11 +584,11 @@ nsPresContext::GetFontPrefsForLang(nsIAtom *aLanguage) const
 void
 nsPresContext::GetDocumentColorPreferences()
 {
-  PRInt32 useAccessibilityTheme = 0;
+  int32_t useAccessibilityTheme = 0;
   bool usePrefColors = true;
   nsCOMPtr<nsIDocShellTreeItem> docShell(do_QueryReferent(mContainer));
   if (docShell) {
-    PRInt32 docShellType;
+    int32_t docShellType;
     docShell->GetItemType(&docShellType);
     if (nsIDocShellTreeItem::typeChrome == docShellType) {
       usePrefColors = false;
@@ -795,9 +733,9 @@ nsPresContext::GetUserPreferences()
   else // dynamic change to invalid value should act like it does initially
     mImageAnimationModePref = imgIContainer::kNormalAnimMode;
 
-  PRUint32 bidiOptions = GetBidi();
+  uint32_t bidiOptions = GetBidi();
 
-  PRInt32 prefInt =
+  int32_t prefInt =
     Preferences::GetInt(IBMBIDI_TEXTDIRECTION_STR,
                         GET_BIDI_OPTION_DIRECTION(bidiOptions));
   SET_BIDI_OPTION_DIRECTION(bidiOptions, prefInt);
@@ -834,7 +772,7 @@ nsPresContext::InvalidateThebesLayers()
     // FrameLayerBuilder caches invalidation-related values that depend on the
     // appunits-per-dev-pixel ratio, so ensure that all ThebesLayer drawing
     // is completely flushed.
-    FrameLayerBuilder::InvalidateThebesLayersInSubtree(rootFrame);
+    FrameLayerBuilder::InvalidateThebesLayersInSubtreeWithUntrustedFrameGeometry(rootFrame);
   }
 }
 
@@ -858,7 +796,7 @@ nsPresContext::PreferenceChanged(const char* aPrefName)
   nsDependentCString prefName(aPrefName);
   if (prefName.EqualsLiteral("layout.css.dpi") ||
       prefName.EqualsLiteral("layout.css.devPixelsPerPx")) {
-    PRInt32 oldAppUnitsPerDevPixel = AppUnitsPerDevPixel();
+    int32_t oldAppUnitsPerDevPixel = AppUnitsPerDevPixel();
     if (mDeviceContext->CheckDPIChange() && mShell) {
       // Re-fetch the view manager's window dimensions in case there's a deferred
       // resize which hasn't affected our mVisibleArea yet
@@ -911,11 +849,11 @@ nsPresContext::PreferenceChanged(const char* aPrefName)
 void
 nsPresContext::UpdateAfterPreferencesChanged()
 {
-  mPrefChangedTimer = nsnull;
+  mPrefChangedTimer = nullptr;
 
   nsCOMPtr<nsIDocShellTreeItem> docShell(do_QueryReferent(mContainer));
   if (docShell) {
-    PRInt32 docShellType;
+    int32_t docShellType;
     docShell->GetItemType(&docShellType);
     if (nsIDocShellTreeItem::typeChrome == docShellType)
       return;
@@ -948,18 +886,12 @@ nsPresContext::Init(nsDeviceContext* aDeviceContext)
   NS_ENSURE_ARG(aDeviceContext);
 
   mDeviceContext = aDeviceContext;
-  NS_ADDREF(mDeviceContext);
 
   if (mDeviceContext->SetPixelScale(mFullZoom))
     mDeviceContext->FlushFontCache();
   mCurAppUnitsPerDevPixel = AppUnitsPerDevPixel();
 
-  for (PRUint32 i = 0; i < IMAGE_LOAD_TYPE_COUNT; ++i)
-    if (!mImageLoaders[i].Init())
-      return NS_ERROR_OUT_OF_MEMORY;
-
   mEventManager = new nsEventStateManager();
-  NS_ADDREF(mEventManager);
 
   mTransitionManager = new nsTransitionManager(this);
 
@@ -1116,11 +1048,11 @@ nsPresContext::SetShell(nsIPresShell* aShell)
   } else {
     if (mTransitionManager) {
       mTransitionManager->Disconnect();
-      mTransitionManager = nsnull;
+      mTransitionManager = nullptr;
     }
     if (mAnimationManager) {
       mAnimationManager->Disconnect();
-      mAnimationManager = nsnull;
+      mAnimationManager = nullptr;
     }
 
     if (IsRoot()) {
@@ -1131,18 +1063,6 @@ nsPresContext::SetShell(nsIPresShell* aShell)
   }
 }
 
-void
-nsPresContext::DestroyImageLoaders()
-{
-  // Destroy image loaders. This is important to do when frames are being
-  // destroyed because imageloaders can have pointers to frames and we don't
-  // want those pointers to outlive the destruction of the frame arena.
-  for (PRUint32 i = 0; i < IMAGE_LOAD_TYPE_COUNT; ++i) {
-    mImageLoaders[i].Enumerate(destroy_loads, nsnull);
-    mImageLoaders[i].Clear();
-  }
-}
-
 void
 nsPresContext::DoChangeCharSet(const nsCString& aCharSet)
 {
@@ -1155,14 +1075,12 @@ void
 nsPresContext::UpdateCharSet(const nsCString& aCharSet)
 {
   if (mLangService) {
-    NS_IF_RELEASE(mLanguage);
-    mLanguage = mLangService->LookupCharSet(aCharSet.get()).get();  // addrefs
+    mLanguage = mLangService->LookupCharSet(aCharSet.get());
     // this will be a language group (or script) code rather than a true language code
 
     // bug 39570: moved from nsLanguageAtomService::LookupCharSet()
     if (mLanguage == nsGkAtoms::Unicode) {
-      NS_RELEASE(mLanguage);
-      NS_IF_ADDREF(mLanguage = mLangService->GetLocaleLanguage()); 
+      mLanguage = mLangService->GetLocaleLanguage();
     }
     ResetCachedFontPrefs();
   }
@@ -1201,10 +1119,10 @@ nsPresContext::Observe(nsISupports* aSubject,
   return NS_ERROR_FAILURE;
 }
 
-static nsPresContext*
-GetParentPresContext(nsPresContext* aPresContext)
+nsPresContext*
+nsPresContext::GetParentPresContext()
 {
-  nsIPresShell* shell = aPresContext->GetPresShell();
+  nsIPresShell* shell = GetPresShell();
   if (shell) {
     nsIFrame* rootFrame = shell->FrameManager()->GetRootFrame();
     if (rootFrame) {
@@ -1213,7 +1131,21 @@ GetParentPresContext(nsPresContext* aPresContext)
         return f->PresContext();
     }
   }
-  return nsnull;
+  return nullptr;
+}
+
+nsPresContext*
+nsPresContext::GetToplevelContentDocumentPresContext()
+{
+  if (IsChrome())
+    return nullptr;
+  nsPresContext* pc = this;
+  for (;;) {
+    nsPresContext* parent = pc->GetParentPresContext();
+    if (!parent || parent->IsChrome())
+      return pc;
+    pc = parent;
+  }
 }
 
 // We may want to replace this with something faster, maybe caching the root prescontext
@@ -1222,12 +1154,39 @@ nsPresContext::GetRootPresContext()
 {
   nsPresContext* pc = this;
   for (;;) {
-    nsPresContext* parent = GetParentPresContext(pc);
+    nsPresContext* parent = pc->GetParentPresContext();
     if (!parent)
       break;
     pc = parent;
   }
-  return pc->IsRoot() ? static_cast<nsRootPresContext*>(pc) : nsnull;
+  return pc->IsRoot() ? static_cast<nsRootPresContext*>(pc) : nullptr;
+}
+
+nsRootPresContext*
+nsPresContext::GetDisplayRootPresContext()
+{
+  nsPresContext* pc = this;
+  for (;;) {
+    nsPresContext* parent = pc->GetParentPresContext();
+    if (!parent) {
+      // Not sure if this is always strictly the parent, but it works for GetRootPresContext
+      // where the current pres context has no frames.
+      nsIDocument *doc = pc->Document();
+      if (doc) {
+        doc = doc->GetParentDocument();
+        if (doc) {
+          nsIPresShell* shell = doc->GetShell();
+          if (shell) {
+            parent = shell->GetPresContext();
+          }
+        }
+      }
+    }
+    if (!parent || parent == pc)
+      break;
+    pc = parent;
+  }
+  return pc->IsRoot() ? static_cast<nsRootPresContext*>(pc) : nullptr;
 }
 
 void
@@ -1242,7 +1201,7 @@ nsPresContext::CompatibilityModeChanged()
 }
 
 // Helper function for setting Anim Mode on image
-static void SetImgAnimModeOnImgReq(imgIRequest* aImgReq, PRUint16 aMode)
+static void SetImgAnimModeOnImgReq(imgIRequest* aImgReq, uint16_t aMode)
 {
   if (aImgReq) {
     nsCOMPtr<imgIContainer> imgCon;
@@ -1253,24 +1212,12 @@ static void SetImgAnimModeOnImgReq(imgIRequest* aImgReq, PRUint16 aMode)
   }
 }
 
- // Enumeration call back for HashTable
-static PLDHashOperator
-set_animation_mode(nsIFrame* aKey, nsRefPtr<nsImageLoader>& aData, void* closure)
-{
-  for (nsImageLoader *loader = aData; loader;
-       loader = loader->GetNextLoader()) {
-    imgIRequest* imgReq = loader->GetRequest();
-    SetImgAnimModeOnImgReq(imgReq, (PRUint16)NS_PTR_TO_INT32(closure));
-  }
-  return PL_DHASH_NEXT;
-}
-
 // IMPORTANT: Assumption is that all images for a Presentation 
 // have the same Animation Mode (pavlov said this was OK)
 //
 // Walks content and set the animation mode
 // this is a way to turn on/off image animations
-void nsPresContext::SetImgAnimations(nsIContent *aParent, PRUint16 aMode)
+void nsPresContext::SetImgAnimations(nsIContent *aParent, uint16_t aMode)
 {
   nsCOMPtr<nsIImageLoadingContent> imgContent(do_QueryInterface(aParent));
   if (imgContent) {
@@ -1280,15 +1227,15 @@ void nsPresContext::SetImgAnimations(nsIContent *aParent, PRUint16 aMode)
     SetImgAnimModeOnImgReq(imgReq, aMode);
   }
   
-  PRUint32 count = aParent->GetChildCount();
-  for (PRUint32 i = 0; i < count; ++i) {
+  uint32_t count = aParent->GetChildCount();
+  for (uint32_t i = 0; i < count; ++i) {
     SetImgAnimations(aParent->GetChildAt(i), aMode);
   }
 }
 
 void
-nsPresContext::SetSMILAnimations(nsIDocument *aDoc, PRUint16 aNewMode,
-                                 PRUint16 aOldMode)
+nsPresContext::SetSMILAnimations(nsIDocument *aDoc, uint16_t aNewMode,
+                                 uint16_t aOldMode)
 {
   if (aDoc->HasAnimationController()) {
     nsSMILAnimationController* controller = aDoc->GetAnimationController();
@@ -1309,7 +1256,7 @@ nsPresContext::SetSMILAnimations(nsIDocument *aDoc, PRUint16 aNewMode,
 }
 
 void
-nsPresContext::SetImageAnimationModeInternal(PRUint16 aMode)
+nsPresContext::SetImageAnimationModeInternal(uint16_t aMode)
 {
   NS_ASSERTION(aMode == imgIContainer::kNormalAnimMode ||
                aMode == imgIContainer::kDontAnimMode ||
@@ -1319,15 +1266,13 @@ nsPresContext::SetImageAnimationModeInternal(PRUint16 aMode)
   if (!IsDynamic())
     return;
 
-  // Set the mode on the image loaders.
-  for (PRUint32 i = 0; i < IMAGE_LOAD_TYPE_COUNT; ++i)
-    mImageLoaders[i].Enumerate(set_animation_mode, NS_INT32_TO_PTR(aMode));
-
   // Now walk the content tree and set the animation mode 
   // on all the images.
-  if (mShell != nsnull) {
+  if (mShell != nullptr) {
     nsIDocument *doc = mShell->GetDocument();
     if (doc) {
+      doc->StyleImageLoader()->SetAnimationMode(aMode);
+
       Element *rootElement = doc->GetRootElement();
       if (rootElement) {
         SetImgAnimations(rootElement, aMode);
@@ -1340,13 +1285,13 @@ nsPresContext::SetImageAnimationModeInternal(PRUint16 aMode)
 }
 
 void
-nsPresContext::SetImageAnimationModeExternal(PRUint16 aMode)
+nsPresContext::SetImageAnimationModeExternal(uint16_t aMode)
 {
   SetImageAnimationModeInternal(aMode);
 }
 
 const nsFont*
-nsPresContext::GetDefaultFont(PRUint8 aFontID, nsIAtom *aLanguage) const
+nsPresContext::GetDefaultFont(uint8_t aFontID, nsIAtom *aLanguage) const
 {
   const LangGroupFontPrefs *prefs = GetFontPrefsForLang(aLanguage);
 
@@ -1376,7 +1321,7 @@ nsPresContext::GetDefaultFont(PRUint8 aFontID, nsIAtom *aLanguage) const
       font = &prefs->mDefaultFantasyFont;
       break;
     default:
-      font = nsnull;
+      font = nullptr;
       NS_ERROR("invalid arg");
       break;
   }
@@ -1438,70 +1383,6 @@ nsPresContext::ScreenWidthInchesForFontInflation(bool* aChanged)
   return deviceWidthInches;
 }
 
-void
-nsPresContext::SetImageLoaders(nsIFrame* aTargetFrame,
-                               ImageLoadType aType,
-                               nsImageLoader* aImageLoaders)
-{
-  NS_ASSERTION(mShell || !aImageLoaders,
-               "Shouldn't add new image loader after the shell is gone");
-
-  nsRefPtr<nsImageLoader> oldLoaders;
-  mImageLoaders[aType].Get(aTargetFrame, getter_AddRefs(oldLoaders));
-
-  if (aImageLoaders) {
-    mImageLoaders[aType].Put(aTargetFrame, aImageLoaders);
-  } else if (oldLoaders) {
-    mImageLoaders[aType].Remove(aTargetFrame);
-  }
-
-  if (oldLoaders)
-    oldLoaders->Destroy();
-}
-
-void
-nsPresContext::SetupBackgroundImageLoaders(nsIFrame* aFrame,
-                                     const nsStyleBackground* aStyleBackground)
-{
-  nsRefPtr<nsImageLoader> loaders;
-  NS_FOR_VISIBLE_BACKGROUND_LAYERS_BACK_TO_FRONT(i, aStyleBackground) {
-    if (aStyleBackground->mLayers[i].mImage.GetType() == eStyleImageType_Image) {
-      PRUint32 actions = nsImageLoader::ACTION_REDRAW_ON_DECODE;
-      imgIRequest *image = aStyleBackground->mLayers[i].mImage.GetImageData();
-      loaders = nsImageLoader::Create(aFrame, image, actions, loaders);
-    }
-  }
-  SetImageLoaders(aFrame, BACKGROUND_IMAGE, loaders);
-}
-
-void
-nsPresContext::SetupBorderImageLoaders(nsIFrame* aFrame,
-                                       const nsStyleBorder* aStyleBorder)
-{
-  // We get called the first time we try to draw a border-image, and
-  // also when the border image changes (including when it changes from
-  // non-null to null).
-  imgIRequest *borderImage = aStyleBorder->GetBorderImage();
-  if (!borderImage) {
-    SetImageLoaders(aFrame, BORDER_IMAGE, nsnull);
-    return;
-  }
-
-  PRUint32 actions = nsImageLoader::ACTION_REDRAW_ON_LOAD;
-  if (aStyleBorder->ImageBorderDiffers())
-    actions |= nsImageLoader::ACTION_REFLOW_ON_LOAD;
-  nsRefPtr<nsImageLoader> loader =
-    nsImageLoader::Create(aFrame, borderImage, actions, nsnull);
-  SetImageLoaders(aFrame, BORDER_IMAGE, loader);
-}
-
-void
-nsPresContext::StopImagesFor(nsIFrame* aTargetFrame)
-{
-  for (PRUint32 i = 0; i < IMAGE_LOAD_TYPE_COUNT; ++i)
-    SetImageLoaders(aTargetFrame, ImageLoadType(i), nsnull);
-}
-
 void
 nsPresContext::SetContainer(nsISupports* aHandler)
 {
@@ -1515,7 +1396,7 @@ nsPresContext::SetContainer(nsISupports* aHandler)
 already_AddRefed<nsISupports>
 nsPresContext::GetContainerInternal() const
 {
-  nsISupports *result = nsnull;
+  nsISupports *result = nullptr;
   if (mContainer)
     CallQueryReferent(mContainer.get(), &result);
 
@@ -1529,6 +1410,18 @@ nsPresContext::GetContainerExternal() const
 }
 
 #ifdef IBMBIDI
+bool
+nsPresContext::BidiEnabledExternal() const
+{
+  return BidiEnabledInternal();
+}
+
+bool
+nsPresContext::BidiEnabledInternal() const
+{
+  return Document()->GetBidiEnabled();
+}
+
 void
 nsPresContext::SetBidiEnabled() const
 {
@@ -1541,7 +1434,7 @@ nsPresContext::SetBidiEnabled() const
 }
 
 void
-nsPresContext::SetBidi(PRUint32 aSource, bool aForceRestyle)
+nsPresContext::SetBidi(uint32_t aSource, bool aForceRestyle)
 {
   // Don't do all this stuff unless the options have changed.
   if (aSource == GetBidi()) {
@@ -1576,7 +1469,7 @@ nsPresContext::SetBidi(PRUint32 aSource, bool aForceRestyle)
   }
 }
 
-PRUint32
+uint32_t
 nsPresContext::GetBidi() const
 {
   return Document()->GetBidiOptions();
@@ -1762,7 +1655,7 @@ nsPresContext::MediaFeatureValuesChanged(bool aCallerWillRebuildStyleData)
       nsCOMPtr<nsIDOMEventTarget> et = do_QueryInterface(win);
       nsCxPusher pusher;
 
-      for (PRUint32 i = 0, i_end = notifyList.Length(); i != i_end; ++i) {
+      for (uint32_t i = 0, i_end = notifyList.Length(); i != i_end; ++i) {
         if (pusher.RePush(et)) {
           nsAutoMicroTask mt;
           nsDOMMediaQueryList::HandleChangeData &d = notifyList[i];
@@ -1814,6 +1707,12 @@ nsPresContext::MatchMedia(const nsAString& aMediaQueryList,
   result.forget(aResult);
 }
 
+nsCompatibility
+nsPresContext::CompatibilityMode() const
+{
+  return Document()->GetCompatibilityMode();
+}
+
 void
 nsPresContext::SetPaginatedScrolling(bool aPaginated)
 {
@@ -1870,7 +1769,7 @@ nsPresContext::IsChromeSlow() const
     nsresult result;
     nsCOMPtr<nsIDocShellTreeItem> docShell(do_QueryInterface(container, &result));
     if (NS_SUCCEEDED(result) && docShell) {
-      PRInt32 docShellType;
+      int32_t docShellType;
       result = docShell->GetItemType(&docShellType);
       if (NS_SUCCEEDED(result)) {
         isChrome = nsIDocShellTreeItem::typeChrome == docShellType;
@@ -1889,7 +1788,7 @@ nsPresContext::InvalidateIsChromeCacheExternal()
 }
 
 /* virtual */ bool
-nsPresContext::HasAuthorSpecifiedRules(nsIFrame *aFrame, PRUint32 ruleTypeMask) const
+nsPresContext::HasAuthorSpecifiedRules(nsIFrame *aFrame, uint32_t ruleTypeMask) const
 {
   return
     nsRuleNode::HasAuthorSpecifiedRules(aFrame->GetStyleContext(),
@@ -2072,7 +1971,7 @@ nsPresContext::FireDOMPaintEvent()
       // If the pref is set, we still don't post events when they're
       // entirely cross-doc.
       notifyContent = false;
-      for (PRUint32 i = 0; i < mInvalidateRequests.mRequests.Length(); ++i) {
+      for (uint32_t i = 0; i < mInvalidateRequests.mRequests.Length(); ++i) {
         if (!(mInvalidateRequests.mRequests[i].mFlags &
               nsIFrame::INVALIDATE_CROSS_DOC)) {
           notifyContent = true;
@@ -2096,18 +1995,19 @@ nsPresContext::FireDOMPaintEvent()
   // This will empty our list in case dispatching the event causes more damage
   // (hopefully it won't, or we're likely to get an infinite loop! At least
   // it won't be blocking app execution though).
-  NS_NewDOMNotifyPaintEvent(getter_AddRefs(event), this, nsnull,
+  NS_NewDOMNotifyPaintEvent(getter_AddRefs(event), this, nullptr,
                             NS_AFTERPAINT,
                             &mInvalidateRequests);
-  nsCOMPtr<nsIPrivateDOMEvent> pEvent = do_QueryInterface(event);
-  if (!pEvent) return;
+  if (!event) {
+    return;
+  }
 
   // Even if we're not telling the window about the event (so eventTarget is
   // the chrome event handler, not the window), the window is still
   // logically the event target.
-  pEvent->SetTarget(eventTarget);
-  pEvent->SetTrusted(true);
-  nsEventDispatcher::DispatchDOMEvent(dispatchTarget, nsnull, event, this, nsnull);
+  event->SetTarget(eventTarget);
+  event->SetTrusted(true);
+  nsEventDispatcher::DispatchDOMEvent(dispatchTarget, nullptr, event, this, nullptr);
 }
 
 static bool
@@ -2122,7 +2022,7 @@ MayHavePaintEventListener(nsPIDOMWindow* aInnerWindow)
   if (!parentTarget)
     return false;
 
-  nsEventListenerManager* manager = nsnull;
+  nsEventListenerManager* manager = nullptr;
   if ((manager = parentTarget->GetListenerManager(false)) &&
       manager->MayHavePaintEventListener()) {
     return true;
@@ -2162,7 +2062,7 @@ nsPresContext::MayHavePaintEventListener()
 }
 
 void
-nsPresContext::NotifyInvalidation(const nsRect& aRect, PRUint32 aFlags)
+nsPresContext::NotifyInvalidation(const nsRect& aRect, uint32_t aFlags)
 {
   // If there is no paint event listener, then we don't need to fire
   // the asynchronous event. We don't even need to record invalidation.
@@ -2173,7 +2073,7 @@ nsPresContext::NotifyInvalidation(const nsRect& aRect, PRUint32 aFlags)
     return;
 
   nsPresContext* pc;
-  for (pc = this; pc; pc = GetParentPresContext(pc)) {
+  for (pc = this; pc; pc = pc->GetParentPresContext()) {
     if (pc->mFireAfterPaintEvents)
       break;
     pc->mFireAfterPaintEvents = true;
@@ -2224,7 +2124,7 @@ nsPresContext::NotifyDidPaintForSubtree()
     nsContentUtils::AddScriptRunner(ev);
   }
 
-  mDocument->EnumerateSubDocuments(NotifyDidPaintSubdocumentCallback, nsnull);
+  mDocument->EnumerateSubDocuments(NotifyDidPaintSubdocumentCallback, nullptr);
 }
 
 bool
@@ -2245,17 +2145,17 @@ enum InterruptMode {
 static InterruptMode sInterruptMode = ModeEvent;
 // Used for the "random" mode.  Controlled by the GECKO_REFLOW_INTERRUPT_SEED
 // env var.
-static PRUint32 sInterruptSeed = 1;
+static uint32_t sInterruptSeed = 1;
 // Used for the "counter" mode.  This is the number of unskipped interrupt
 // checks that have to happen before we interrupt.  Controlled by the
 // GECKO_REFLOW_INTERRUPT_FREQUENCY env var.
-static PRUint32 sInterruptMaxCounter = 10;
+static uint32_t sInterruptMaxCounter = 10;
 // Used for the "counter" mode.  This counts up to sInterruptMaxCounter and is
 // then reset to 0.
-static PRUint32 sInterruptCounter;
+static uint32_t sInterruptCounter;
 // Number of interrupt checks to skip before really trying to interrupt.
 // Controlled by the GECKO_REFLOW_INTERRUPT_CHECKS_TO_SKIP env var.
-static PRUint32 sInterruptChecksToSkip = 200;
+static uint32_t sInterruptChecksToSkip = 200;
 // Number of milliseconds that a reflow should be allowed to run for before we
 // actually allow interruption.  Controlled by the
 // GECKO_REFLOW_MIN_NOINTERRUPT_DURATION env var.  Can't be initialized here,
@@ -2389,6 +2289,18 @@ nsPresContext::CheckForInterrupt(nsIFrame* aFrame)
   return mHasPendingInterrupt;
 }
 
+nsIFrame*
+nsPresContext::GetPrimaryFrameFor(nsIContent* aContent)
+{
+  NS_PRECONDITION(aContent, "Don't do that");
+  if (GetPresShell() &&
+      GetPresShell()->GetDocument() == aContent->GetCurrentDoc()) {
+    return aContent->GetPrimaryFrame();
+  }
+  return nullptr;
+}
+
+
 size_t
 nsPresContext::SizeOfExcludingThis(nsMallocSizeOfFun aMallocSizeOf) const
 {
@@ -2431,7 +2343,6 @@ nsPresContext::IsRootContentDocument()
 nsRootPresContext::nsRootPresContext(nsIDocument* aDocument,
                                      nsPresContextType aType)
   : nsPresContext(aDocument, aType),
-    mUpdatePluginGeometryForFrame(nsnull),
     mDOMGeneration(0),
     mNeedsToUpdatePluginGeometry(false)
 {
@@ -2447,20 +2358,20 @@ nsRootPresContext::~nsRootPresContext()
 }
 
 void
-nsRootPresContext::RegisterPluginForGeometryUpdates(nsObjectFrame* aPlugin)
+nsRootPresContext::RegisterPluginForGeometryUpdates(nsIContent* aPlugin)
 {
   mRegisteredPlugins.PutEntry(aPlugin);
 }
 
 void
-nsRootPresContext::UnregisterPluginForGeometryUpdates(nsObjectFrame* aPlugin)
+nsRootPresContext::UnregisterPluginForGeometryUpdates(nsIContent* aPlugin)
 {
   mRegisteredPlugins.RemoveEntry(aPlugin);
 }
 
 struct PluginGeometryClosure {
   nsIFrame* mRootFrame;
-  PRInt32   mRootAPD;
+  int32_t   mRootAPD;
   nsIFrame* mChangedSubtree;
   nsRect    mChangedRect;
   nsTHashtable<nsPtrHashKey<nsObjectFrame> > mAffectedPlugins;
@@ -2468,13 +2379,17 @@ struct PluginGeometryClosure {
   nsTArray<nsIWidget::Configuration>* mOutputConfigurations;
 };
 static PLDHashOperator
-PluginBoundsEnumerator(nsPtrHashKey<nsObjectFrame>* aEntry, void* userArg)
+PluginBoundsEnumerator(nsRefPtrHashKey<nsIContent>* aEntry, void* userArg)
 {
   PluginGeometryClosure* closure = static_cast<PluginGeometryClosure*>(userArg);
-  nsObjectFrame* f = aEntry->GetKey();
+  nsObjectFrame* f = static_cast<nsObjectFrame*>(aEntry->GetKey()->GetPrimaryFrame());
+  if (!f) {
+    NS_WARNING("Null frame in PluginBoundsEnumerator");
+    return PL_DHASH_NEXT;
+  }
   nsRect fBounds = f->GetContentRect() +
       f->GetParent()->GetOffsetToCrossDoc(closure->mRootFrame);
-  PRInt32 APD = f->PresContext()->AppUnitsPerDevPixel();
+  int32_t APD = f->PresContext()->AppUnitsPerDevPixel();
   fBounds = fBounds.ConvertAppUnitsRoundOut(APD, closure->mRootAPD);
   // We're identifying the plugins that may have been affected by changes
   // to the frame subtree rooted at aChangedRoot. Any plugin that overlaps
@@ -2563,7 +2478,7 @@ nsRootPresContext::GetPluginGeometryUpdates(nsIFrame* aChangedSubtree,
   closure.mChangedSubtree = aChangedSubtree;
   closure.mChangedRect = aChangedSubtree->GetVisualOverflowRect() +
       aChangedSubtree->GetOffsetToCrossDoc(closure.mRootFrame);
-  PRInt32 subtreeAPD = aChangedSubtree->PresContext()->AppUnitsPerDevPixel();
+  int32_t subtreeAPD = aChangedSubtree->PresContext()->AppUnitsPerDevPixel();
   closure.mChangedRect =
     closure.mChangedRect.ConvertAppUnitsRoundOut(subtreeAPD, closure.mRootAPD);
   closure.mAffectedPlugins.Init();
@@ -2615,8 +2530,8 @@ HasOverlap(const nsIntPoint& aOffset1, const nsTArray<nsIntRect>& aClipRects1,
            const nsIntPoint& aOffset2, const nsTArray<nsIntRect>& aClipRects2)
 {
   nsIntPoint offsetDelta = aOffset1 - aOffset2;
-  for (PRUint32 i = 0; i < aClipRects1.Length(); ++i) {
-    for (PRUint32 j = 0; j < aClipRects2.Length(); ++j) {
+  for (uint32_t i = 0; i < aClipRects1.Length(); ++i) {
+    for (uint32_t j = 0; j < aClipRects2.Length(); ++j) {
       if ((aClipRects1[i] + offsetDelta).Intersects(aClipRects2[j]))
         return true;
     }
@@ -2650,11 +2565,11 @@ SortConfigurations(nsTArray<nsIWidget::Configuration>* aConfigurations)
   // window, we just move the last window in the list anyway.
   while (!pluginsToMove.IsEmpty()) {
     // Find a window whose destination does not overlap any other window
-    PRUint32 i;
+    uint32_t i;
     for (i = 0; i + 1 < pluginsToMove.Length(); ++i) {
       nsIWidget::Configuration* config = &pluginsToMove[i];
       bool foundOverlap = false;
-      for (PRUint32 j = 0; j < pluginsToMove.Length(); ++j) {
+      for (uint32_t j = 0; j < pluginsToMove.Length(); ++j) {
         if (i == j)
           continue;
         nsIntRect bounds;
@@ -2688,21 +2603,13 @@ nsRootPresContext::UpdatePluginGeometry()
   // update when we don't actually want one.
   CancelUpdatePluginGeometryTimer();
 
-  nsIFrame* f = mUpdatePluginGeometryForFrame;
-  if (f) {
-    mUpdatePluginGeometryForFrame->PresContext()->
-      SetContainsUpdatePluginGeometryFrame(false);
-    mUpdatePluginGeometryForFrame = nsnull;
-  } else {
-    f = FrameManager()->GetRootFrame();
-  }
-
+  nsIFrame* f = FrameManager()->GetRootFrame();
   nsTArray<nsIWidget::Configuration> configurations;
   GetPluginGeometryUpdates(f, &configurations);
   if (configurations.IsEmpty())
     return;
   SortConfigurations(&configurations);
-  nsIWidget* widget = FrameManager()->GetRootFrame()->GetNearestWidget();
+  nsIWidget* widget = f->GetNearestWidget();
   NS_ASSERTION(widget, "Plugins must have a parent window");
   widget->ConfigureChildren(configurations);
   DidApplyPluginGeometryUpdates();
@@ -2715,7 +2622,7 @@ UpdatePluginGeometryCallback(nsITimer *aTimer, void *aClosure)
 }
 
 void
-nsRootPresContext::RequestUpdatePluginGeometry(nsIFrame* aFrame)
+nsRootPresContext::RequestUpdatePluginGeometry()
 {
   if (mRegisteredPlugins.Count() == 0)
     return;
@@ -2738,23 +2645,17 @@ nsRootPresContext::RequestUpdatePluginGeometry(nsIFrame* aFrame)
                              nsITimer::TYPE_ONE_SHOT);
     }
     mNeedsToUpdatePluginGeometry = true;
-    mUpdatePluginGeometryForFrame = aFrame;
-    mUpdatePluginGeometryForFrame->PresContext()->
-      SetContainsUpdatePluginGeometryFrame(true);
-  } else {
-    if (!mUpdatePluginGeometryForFrame ||
-        aFrame == mUpdatePluginGeometryForFrame)
-      return;
-    mUpdatePluginGeometryForFrame->PresContext()->
-      SetContainsUpdatePluginGeometryFrame(false);
-    mUpdatePluginGeometryForFrame = nsnull;
   }
 }
 
 static PLDHashOperator
-PluginDidSetGeometryEnumerator(nsPtrHashKey<nsObjectFrame>* aEntry, void* userArg)
+PluginDidSetGeometryEnumerator(nsRefPtrHashKey<nsIContent>* aEntry, void* userArg)
 {
-  nsObjectFrame* f = aEntry->GetKey();
+  nsObjectFrame* f = static_cast<nsObjectFrame*>(aEntry->GetKey()->GetPrimaryFrame());
+  if (!f) {
+    NS_WARNING("Null frame in PluginDidSetGeometryEnumerator");
+    return PL_DHASH_NEXT;
+  }
   f->DidSetWidgetGeometry();
   return PL_DHASH_NEXT;
 }
@@ -2762,27 +2663,7 @@ PluginDidSetGeometryEnumerator(nsPtrHashKey<nsObjectFrame>* aEntry, void* userAr
 void
 nsRootPresContext::DidApplyPluginGeometryUpdates()
 {
-  mRegisteredPlugins.EnumerateEntries(PluginDidSetGeometryEnumerator, nsnull);
-}
-
-void
-nsRootPresContext::RootForgetUpdatePluginGeometryFrame(nsIFrame* aFrame)
-{
-  if (aFrame == mUpdatePluginGeometryForFrame) {
-    mUpdatePluginGeometryForFrame->PresContext()->
-      SetContainsUpdatePluginGeometryFrame(false);
-    mUpdatePluginGeometryForFrame = nsnull;
-  }
-}
-
-void
-nsRootPresContext::RootForgetUpdatePluginGeometryFrameForPresContext(
-  nsPresContext* aPresContext)
-{
-  if (aPresContext->GetContainsUpdatePluginGeometryFrame()) {
-    aPresContext->SetContainsUpdatePluginGeometryFrame(false);
-    mUpdatePluginGeometryForFrame = nsnull;
-  }
+  mRegisteredPlugins.EnumerateEntries(PluginDidSetGeometryEnumerator, nullptr);
 }
 
 static void
@@ -2821,10 +2702,10 @@ nsRootPresContext::AddWillPaintObserver(nsIRunnable* aRunnable)
 void
 nsRootPresContext::FlushWillPaintObservers()
 {
-  mWillPaintFallbackEvent = nsnull;
+  mWillPaintFallbackEvent = nullptr;
   nsTArray<nsCOMPtr<nsIRunnable> > observers;
   observers.SwapElements(mWillPaintObservers);
-  for (PRUint32 i = 0; i < observers.Length(); ++i) {
+  for (uint32_t i = 0; i < observers.Length(); ++i) {
     observers[i]->Run();
   }
 }
@@ -2840,8 +2721,4 @@ nsRootPresContext::SizeOfExcludingThis(nsMallocSizeOfFun aMallocSizeOf) const
   // - mRegisteredPlugins
   // - mWillPaintObservers
   // - mWillPaintFallbackEvent
-  //
-  // The following member are not measured:
-  // - mUpdatePluginGeometryForFrame, because it is non-owning
 }
-
