@@ -1,40 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   L. David Baron <dbaron@dbaron.org>, Mozilla Corporation
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /* a presentation of a document, part 1 */
 
@@ -44,37 +11,27 @@
 #include "nsISupports.h"
 #include "nsColor.h"
 #include "nsCoord.h"
-#include "nsAString.h"
 #include "nsCOMPtr.h"
 #include "nsIPresShell.h"
 #include "nsRect.h"
 #include "nsDeviceContext.h"
 #include "nsFont.h"
-#include "nsIWeakReference.h"
-#include "nsITheme.h"
-#include "nsILanguageAtomService.h"
 #include "nsIObserver.h"
 #include "nsITimer.h"
 #include "nsCRT.h"
-#include "nsIPrintSettings.h"
 #include "FramePropertyTable.h"
 #include "nsGkAtoms.h"
-#include "nsIDocument.h"
 #include "nsRefPtrHashtable.h"
 #include "nsCycleCollectionParticipant.h"
 #include "nsChangeHint.h"
 // This also pulls in gfxTypes.h, which we cannot include directly.
 #include "gfxRect.h"
-#include "nsRegion.h"
 #include "nsTArray.h"
 #include "nsAutoPtr.h"
-#include "nsThreadUtils.h"
 #include "nsIWidget.h"
 #include "mozilla/TimeStamp.h"
-#include "nsIContent.h"
 #include "prclist.h"
 
-class nsImageLoader;
 #ifdef IBMBIDI
 class nsBidiPresUtils;
 #endif // IBMBIDI
@@ -83,6 +40,12 @@ struct nsRect;
 
 class imgIRequest;
 
+class nsAString;
+class nsIPrintSettings;
+class nsIDocument;
+class nsILanguageAtomService;
+class nsITheme;
+class nsIContent;
 class nsFontMetrics;
 class nsIFrame;
 class nsFrameManager;
@@ -92,7 +55,6 @@ class nsIAtom;
 class nsEventStateManager;
 class nsIURI;
 class nsICSSPseudoComparator;
-class nsIAtom;
 struct nsStyleBackground;
 struct nsStyleBorder;
 class nsIRunnable;
@@ -125,8 +87,8 @@ enum nsPresContext_CachedIntPrefType {
 
 // IDs for the default variable and fixed fonts (not to be changed, see nsFont.h)
 // To be used for Get/SetDefaultFont(). The other IDs in nsFont.h are also supported.
-const PRUint8 kPresContext_DefaultVariableFont_ID = 0x00; // kGenericFont_moz_variable
-const PRUint8 kPresContext_DefaultFixedFont_ID    = 0x01; // kGenericFont_moz_fixed
+const uint8_t kPresContext_DefaultVariableFont_ID = 0x00; // kGenericFont_moz_variable
+const uint8_t kPresContext_DefaultFixedFont_ID    = 0x01; // kGenericFont_moz_fixed
 
 #ifdef DEBUG
 struct nsAutoLayoutPhase;
@@ -143,7 +105,7 @@ class nsInvalidateRequestList {
 public:
   struct Request {
     nsRect   mRect;
-    PRUint32 mFlags;
+    uint32_t mFlags;
   };
 
   nsTArray<Request> mRequests;
@@ -153,6 +115,7 @@ public:
 #define NS_AUTHOR_SPECIFIED_BACKGROUND      (1 << 0)
 #define NS_AUTHOR_SPECIFIED_BORDER          (1 << 1)
 #define NS_AUTHOR_SPECIFIED_PADDING         (1 << 2)
+#define NS_AUTHOR_SPECIFIED_TEXT_SHADOW     (1 << 3)
 
 class nsRootPresContext;
 
@@ -202,12 +165,25 @@ public:
 
   nsIPresShell* GetPresShell() const { return mShell; }
 
+  /**
+   * Returns the parent prescontext for this one. Returns null if this is a
+   * root.
+   */
+  nsPresContext* GetParentPresContext();
+
+  /**
+   * Returns the prescontext of the toplevel content document that contains
+   * this presentation, or null if there isn't one.
+   */
+  nsPresContext* GetToplevelContentDocumentPresContext();
+
   /**
    * Return the presentation context for the root of the view manager
-   * hierarchy that contains this presentation context, or nsnull if it can't
+   * hierarchy that contains this presentation context, or nullptr if it can't
    * be found (e.g. it's detached).
    */
   nsRootPresContext* GetRootPresContext();
+  nsRootPresContext* GetDisplayRootPresContext();
   virtual bool IsRoot() { return false; }
 
   nsIDocument* Document() const
@@ -222,7 +198,7 @@ public:
   nsStyleSet* StyleSet() { return GetPresShell()->StyleSet(); }
 
   nsFrameManager* FrameManager()
-    { return GetPresShell()->FrameManager(); } 
+    { return GetPresShell()->FrameManager(); }
 
   nsTransitionManager* TransitionManager() { return mTransitionManager; }
   nsAnimationManager* AnimationManager() { return mAnimationManager; }
@@ -261,9 +237,8 @@ public:
    * Access compatibility mode for this context.  This is the same as
    * our document's compatibility mode.
    */
-  nsCompatibility CompatibilityMode() const {
-    return Document()->GetCompatibilityMode();
-  }
+  nsCompatibility CompatibilityMode() const;
+
   /**
    * Notify the context that the document's compatibility mode has changed
    */
@@ -272,14 +247,14 @@ public:
   /**
    * Access the image animation mode for this context
    */
-  PRUint16     ImageAnimationMode() const { return mImageAnimationMode; }
-  virtual NS_HIDDEN_(void) SetImageAnimationModeExternal(PRUint16 aMode);
-  NS_HIDDEN_(void) SetImageAnimationModeInternal(PRUint16 aMode);
+  uint16_t     ImageAnimationMode() const { return mImageAnimationMode; }
+  virtual NS_HIDDEN_(void) SetImageAnimationModeExternal(uint16_t aMode);
+  NS_HIDDEN_(void) SetImageAnimationModeInternal(uint16_t aMode);
 #ifdef _IMPL_NS_LAYOUT
-  void SetImageAnimationMode(PRUint16 aMode)
+  void SetImageAnimationMode(uint16_t aMode)
   { SetImageAnimationModeInternal(aMode); }
 #else
-  void SetImageAnimationMode(PRUint16 aMode)
+  void SetImageAnimationMode(uint16_t aMode)
   { SetImageAnimationModeExternal(aMode); }
 #endif
 
@@ -292,7 +267,7 @@ public:
   {
     if (mShell)
       return mShell->AllocateMisc(aSize);
-    return nsnull;
+    return nullptr;
   }
 
   void FreeToShell(size_t aSize, void* aFreeChunk)
@@ -304,7 +279,7 @@ public:
 
   /**
    * Get the default font for the given language and generic font ID.
-   * If aLanguage is nsnull, the document's language is used.
+   * If aLanguage is nullptr, the document's language is used.
    *
    * This object is read-only, you must copy the font to modify it.
    * 
@@ -321,7 +296,7 @@ public:
    * the user's preference for font size for that generic and the
    * given language.
    */
-  NS_HIDDEN_(const nsFont*) GetDefaultFont(PRUint8 aFontID,
+  NS_HIDDEN_(const nsFont*) GetDefaultFont(uint8_t aFontID,
                                            nsIAtom *aLanguage) const;
 
   /** Get a cached boolean pref, by its type */
@@ -346,7 +321,7 @@ public:
 
   /** Get a cached integer pref, by its type */
   // *  - initially created for bugs 30910, 61883, 74186, 84398
-  PRInt32 GetCachedIntPref(nsPresContext_CachedIntPrefType aPrefType) const
+  int32_t GetCachedIntPref(nsPresContext_CachedIntPrefType aPrefType) const
   {
     // If called with a constant parameter, the compiler should optimize
     // this switch statement away.
@@ -380,52 +355,9 @@ public:
   void SetBodyTextColor(nscolor aColor) { mBodyTextColor = aColor; }
 
   bool GetUseFocusColors() const { return mUseFocusColors; }
-  PRUint8 FocusRingWidth() const { return mFocusRingWidth; }
+  uint8_t FocusRingWidth() const { return mFocusRingWidth; }
   bool GetFocusRingOnAnything() const { return mFocusRingOnAnything; }
-  PRUint8 GetFocusRingStyle() const { return mFocusRingStyle; }
-
-  /**
-   * The types of image load types that the pres context needs image
-   * loaders to track invalidation for.
-   */
-  enum ImageLoadType {
-    BACKGROUND_IMAGE,
-    BORDER_IMAGE,
-    IMAGE_LOAD_TYPE_COUNT
-  };
-
-  /**
-   * Set the list of image loaders that track invalidation for a
-   * specific frame and type of image.  This list will replace any
-   * previous list for that frame and image type (and null will remove
-   * any previous list).
-   */
-  NS_HIDDEN_(void) SetImageLoaders(nsIFrame* aTargetFrame,
-                                   ImageLoadType aType,
-                                   nsImageLoader* aImageLoaders);
-
-  /**
-   * Make an appropriate SetImageLoaders call (including potentially
-   * with null aImageLoaders) given that aFrame draws its background
-   * based on aStyleBackground.
-   */
-  NS_HIDDEN_(void) SetupBackgroundImageLoaders(nsIFrame* aFrame,
-                                               const nsStyleBackground*
-                                                 aStyleBackground);
-
-  /**
-   * Make an appropriate SetImageLoaders call (including potentially
-   * with null aImageLoaders) given that aFrame draws its border
-   * based on aStyleBorder.
-   */
-  NS_HIDDEN_(void) SetupBorderImageLoaders(nsIFrame* aFrame,
-                                           const nsStyleBorder* aStyleBorder);
-
-  /**
-   * This method is called when a frame is being destroyed to
-   * ensure that the image loads get disassociated from the prescontext
-   */
-  NS_HIDDEN_(void) StopImagesFor(nsIFrame* aTargetFrame);
+  uint8_t GetFocusRingStyle() const { return mFocusRingStyle; }
 
   NS_HIDDEN_(void) SetContainer(nsISupports* aContainer);
 
@@ -537,14 +469,14 @@ public:
 
   /**
    * Get the minimum font size for the specified language. If aLanguage
-   * is nsnull, then the document's language is used.
+   * is nullptr, then the document's language is used.
    */
-  PRInt32 MinFontSize(nsIAtom *aLanguage) const {
+  int32_t MinFontSize(nsIAtom *aLanguage) const {
     const LangGroupFontPrefs *prefs = GetFontPrefsForLang(aLanguage);
     return NS_MAX(mMinFontSize, prefs->mMinimumFontSize);
   }
 
-  void SetMinFontSize(PRInt32 aMinFontSize) {
+  void SetMinFontSize(int32_t aMinFontSize) {
     if (aMinFontSize == mMinFontSize)
       return;
 
@@ -573,13 +505,13 @@ public:
    *  a. the last time the function was called with non-null aChanged, or
    *  b. the first time the function was called.
    */
-  float ScreenWidthInchesForFontInflation(bool* aChanged = nsnull);
+  float ScreenWidthInchesForFontInflation(bool* aChanged = nullptr);
 
-  static PRInt32 AppUnitsPerCSSPixel() { return nsDeviceContext::AppUnitsPerCSSPixel(); }
-  PRUint32 AppUnitsPerDevPixel() const  { return mDeviceContext->AppUnitsPerDevPixel(); }
-  static PRInt32 AppUnitsPerCSSInch() { return nsDeviceContext::AppUnitsPerCSSInch(); }
+  static int32_t AppUnitsPerCSSPixel() { return nsDeviceContext::AppUnitsPerCSSPixel(); }
+  uint32_t AppUnitsPerDevPixel() const  { return mDeviceContext->AppUnitsPerDevPixel(); }
+  static int32_t AppUnitsPerCSSInch() { return nsDeviceContext::AppUnitsPerCSSInch(); }
 
-  static nscoord CSSPixelsToAppUnits(PRInt32 aPixels)
+  static nscoord CSSPixelsToAppUnits(int32_t aPixels)
   { return NSIntPixelsToAppUnits(aPixels,
                                  nsDeviceContext::AppUnitsPerCSSPixel()); }
 
@@ -587,7 +519,7 @@ public:
   { return NSFloatPixelsToAppUnits(aPixels,
              float(nsDeviceContext::AppUnitsPerCSSPixel())); }
 
-  static PRInt32 AppUnitsToIntCSSPixels(nscoord aAppUnits)
+  static int32_t AppUnitsToIntCSSPixels(nscoord aAppUnits)
   { return NSAppUnitsToIntPixels(aAppUnits,
              float(nsDeviceContext::AppUnitsPerCSSPixel())); }
 
@@ -595,15 +527,15 @@ public:
   { return NSAppUnitsToFloatPixels(aAppUnits,
              float(nsDeviceContext::AppUnitsPerCSSPixel())); }
 
-  nscoord DevPixelsToAppUnits(PRInt32 aPixels) const
+  nscoord DevPixelsToAppUnits(int32_t aPixels) const
   { return NSIntPixelsToAppUnits(aPixels,
                                  mDeviceContext->AppUnitsPerDevPixel()); }
 
-  PRInt32 AppUnitsToDevPixels(nscoord aAppUnits) const
+  int32_t AppUnitsToDevPixels(nscoord aAppUnits) const
   { return NSAppUnitsToIntPixels(aAppUnits,
              float(mDeviceContext->AppUnitsPerDevPixel())); }
 
-  PRInt32 CSSPixelsToDevPixels(PRInt32 aPixels)
+  int32_t CSSPixelsToDevPixels(int32_t aPixels)
   { return AppUnitsToDevPixels(CSSPixelsToAppUnits(aPixels)); }
 
   float CSSPixelsToDevPixels(float aPixels)
@@ -612,10 +544,10 @@ public:
                                    float(mDeviceContext->AppUnitsPerDevPixel()));
   }
 
-  PRInt32 DevPixelsToIntCSSPixels(PRInt32 aPixels)
+  int32_t DevPixelsToIntCSSPixels(int32_t aPixels)
   { return AppUnitsToIntCSSPixels(DevPixelsToAppUnits(aPixels)); }
 
-  float DevPixelsToFloatCSSPixels(PRInt32 aPixels)
+  float DevPixelsToFloatCSSPixels(int32_t aPixels)
   { return AppUnitsToFloatCSSPixels(DevPixelsToAppUnits(aPixels)); }
 
   // If there is a remainder, it is rounded to nearest app units.
@@ -652,8 +584,8 @@ public:
   struct ScrollbarStyles {
     // Always one of NS_STYLE_OVERFLOW_SCROLL, NS_STYLE_OVERFLOW_HIDDEN,
     // or NS_STYLE_OVERFLOW_AUTO.
-    PRUint8 mHorizontal, mVertical;
-    ScrollbarStyles(PRUint8 h, PRUint8 v) : mHorizontal(h), mVertical(v) {}
+    uint8_t mHorizontal, mVertical;
+    ScrollbarStyles(uint8_t h, uint8_t v) : mHorizontal(h), mVertical(v) {}
     ScrollbarStyles() {}
     bool operator==(const ScrollbarStyles& aStyles) const {
       return aStyles.mHorizontal == mHorizontal && aStyles.mVertical == mVertical;
@@ -662,7 +594,7 @@ public:
       return aStyles.mHorizontal != mHorizontal || aStyles.mVertical != mVertical;
     }
   };
-  void SetViewportOverflowOverride(PRUint8 aX, PRUint8 aY)
+  void SetViewportOverflowOverride(uint8_t aX, uint8_t aY)
   {
     mViewportStyleOverflow.mHorizontal = aX;
     mViewportStyleOverflow.mVertical = aY;
@@ -695,13 +627,13 @@ public:
    *
    *  @lina 07/12/2000
    */
-  virtual bool BidiEnabledExternal() const { return BidiEnabledInternal(); }
-  bool BidiEnabledInternal() const { return Document()->GetBidiEnabled(); }
 #ifdef _IMPL_NS_LAYOUT
   bool BidiEnabled() const { return BidiEnabledInternal(); }
 #else
   bool BidiEnabled() const { return BidiEnabledExternal(); }
 #endif
+  virtual bool BidiEnabledExternal() const;
+  bool BidiEnabledInternal() const;
 
   /**
    *  Set bidi enabled. This means we should apply the Unicode Bidi Algorithm
@@ -741,15 +673,15 @@ public:
   /**
    * Set the Bidi options for the presentation context
    */  
-  NS_HIDDEN_(void) SetBidi(PRUint32 aBidiOptions,
+  NS_HIDDEN_(void) SetBidi(uint32_t aBidiOptions,
                            bool aForceRestyle = false);
 
   /**
    * Get the Bidi options for the presentation context
    * Not inline so consumers of nsPresContext are not forced to
    * include nsIDocument.
-   */  
-  NS_HIDDEN_(PRUint32) GetBidi() const;
+   */
+  NS_HIDDEN_(uint32_t) GetBidi() const;
 #endif // IBMBIDI
 
   /**
@@ -829,7 +761,7 @@ public:
 #endif
 
   // Public API for native theme code to get style internals.
-  virtual bool HasAuthorSpecifiedRules(nsIFrame *aFrame, PRUint32 ruleTypeMask) const;
+  virtual bool HasAuthorSpecifiedRules(nsIFrame *aFrame, uint32_t ruleTypeMask) const;
 
   // Is it OK to let the page specify colors and backgrounds?
   bool UseDocumentColors() const {
@@ -860,7 +792,7 @@ public:
   // Returns true on success and false on failure (not safe).
   bool EnsureSafeToHandOutCSSRules();
 
-  void NotifyInvalidation(const nsRect& aRect, PRUint32 aFlags);
+  void NotifyInvalidation(const nsRect& aRect, uint32_t aFlags);
   void NotifyDidPaintForSubtree();
   void FireDOMPaintEvent();
 
@@ -949,32 +881,12 @@ public:
    * content's primary frame.  Otherwise, return null.  Only use this
    * if you care about which presshell the primary frame is in.
    */
-  nsIFrame* GetPrimaryFrameFor(nsIContent* aContent) {
-    NS_PRECONDITION(aContent, "Don't do that");
-    if (GetPresShell() &&
-        GetPresShell()->GetDocument() == aContent->GetCurrentDoc()) {
-      return aContent->GetPrimaryFrame();
-    }
-    return nsnull;
-  }
+  nsIFrame* GetPrimaryFrameFor(nsIContent* aContent);
 
   void NotifyDestroyingFrame(nsIFrame* aFrame)
   {
     PropertyTable()->DeleteAllFor(aFrame);
   }
-  inline void ForgetUpdatePluginGeometryFrame(nsIFrame* aFrame);
-
-  void DestroyImageLoaders();
-
-  bool GetContainsUpdatePluginGeometryFrame()
-  {
-    return mContainsUpdatePluginGeometryFrame;
-  }
-
-  void SetContainsUpdatePluginGeometryFrame(bool aValue)
-  {
-    mContainsUpdatePluginGeometryFrame = aValue;
-  }
 
   bool MayHaveFixedBackgroundFrames() { return mMayHaveFixedBackgroundFrames; }
   void SetHasFixedBackgroundFrame() { mMayHaveFixedBackgroundFrames = true; }
@@ -991,9 +903,9 @@ protected:
   NS_HIDDEN_(void) ThemeChangedInternal();
   NS_HIDDEN_(void) SysColorChangedInternal();
 
-  NS_HIDDEN_(void) SetImgAnimations(nsIContent *aParent, PRUint16 aMode);
-  NS_HIDDEN_(void) SetSMILAnimations(nsIDocument *aDoc, PRUint16 aNewMode,
-                                     PRUint16 aOldMode);
+  NS_HIDDEN_(void) SetImgAnimations(nsIContent *aParent, uint16_t aMode);
+  NS_HIDDEN_(void) SetSMILAnimations(nsIDocument *aDoc, uint16_t aNewMode,
+                                     uint16_t aOldMode);
   NS_HIDDEN_(void) GetDocumentColorPreferences();
 
   NS_HIDDEN_(void) PreferenceChanged(const char* aPrefName);
@@ -1011,7 +923,7 @@ protected:
   struct LangGroupFontPrefs {
     // Font sizes default to zero; they will be set in GetFontPreferences
     LangGroupFontPrefs()
-      : mLangGroup(nsnull)
+      : mLangGroup(nullptr)
       , mMinimumFontSize(0)
       , mDefaultVariableFont("serif", NS_FONT_STYLE_NORMAL, NS_FONT_VARIANT_NORMAL,
                              NS_FONT_WEIGHT_NORMAL, NS_FONT_STRETCH_NORMAL, 0, 0)
@@ -1070,10 +982,10 @@ protected:
 
   void ResetCachedFontPrefs() {
     // Throw away any other LangGroupFontPrefs objects:
-    mLangGroupFontPrefs.mNext = nsnull;
+    mLangGroupFontPrefs.mNext = nullptr;
 
     // Make GetFontPreferences reinitialize mLangGroupFontPrefs:
-    mLangGroupFontPrefs.mLangGroup = nsnull;
+    mLangGroupFontPrefs.mLangGroup = nullptr;
   }
 
   NS_HIDDEN_(void) UpdateCharSet(const nsCString& aCharSet);
@@ -1102,16 +1014,16 @@ protected:
   // IMPORTANT: The ownership implicit in the following member variables
   // has been explicitly checked.  If you add any members to this class,
   // please make the ownership explicit (pinkerton, scc).
-  
+
   nsPresContextType     mType;
   nsIPresShell*         mShell;         // [WEAK]
   nsCOMPtr<nsIDocument> mDocument;
-  nsDeviceContext*     mDeviceContext; // [STRONG] could be weak, but
-                                        // better safe than sorry.
-                                        // Cannot reintroduce cycles
-                                        // since there is no dependency
-                                        // from gfx back to layout.
-  nsEventStateManager* mEventManager;   // [STRONG]
+  nsRefPtr<nsDeviceContext> mDeviceContext; // [STRONG] could be weak, but
+                                            // better safe than sorry.
+                                            // Cannot reintroduce cycles
+                                            // since there is no dependency
+                                            // from gfx back to layout.
+  nsRefPtr<nsEventStateManager> mEventManager;
   nsRefPtr<nsRefreshDriver> mRefreshDriver;
   nsRefPtr<nsTransitionManager> mTransitionManager;
   nsRefPtr<nsAnimationManager> mAnimationManager;
@@ -1125,7 +1037,7 @@ protected:
   // This may in fact hold a langGroup such as x-western rather than
   // a specific language, however (e.g, if it is inferred from the
   // charset rather than explicitly specified as a lang attribute).
-  nsIAtom*              mLanguage;      // [STRONG]
+  nsCOMPtr<nsIAtom>     mLanguage;
 
 public:
   // The following are public member variables so that we can use them
@@ -1137,21 +1049,18 @@ public:
 
 protected:
 
-  nsRefPtrHashtable<nsPtrHashKey<nsIFrame>, nsImageLoader>
-                        mImageLoaders[IMAGE_LOAD_TYPE_COUNT];
-
   nsWeakPtr             mContainer;
 
   PRCList               mDOMMediaQueryLists;
 
-  PRInt32               mMinFontSize;   // Min font size, defaults to 0
+  int32_t               mMinFontSize;   // Min font size, defaults to 0
   float                 mTextZoom;      // Text zoom, defaults to 1.0
   float                 mFullZoom;      // Page zoom, defaults to 1.0
 
   float                 mLastFontInflationScreenWidth;
 
-  PRInt32               mCurAppUnitsPerDevPixel;
-  PRInt32               mAutoQualityMinFontSizePixelsPref;
+  int32_t               mCurAppUnitsPerDevPixel;
+  int32_t               mAutoQualityMinFontSizePixelsPref;
 
   nsCOMPtr<nsITheme> mTheme;
   nsCOMPtr<nsILanguageAtomService> mLangService;
@@ -1164,7 +1073,7 @@ protected:
 
   // container for per-context fonts (downloadable, SVG, etc.)
   nsUserFontSet*        mUserFontSet;
-  
+
   nsRect                mVisibleArea;
   nsSize                mPageSize;
   float                 mPageScale;
@@ -1183,16 +1092,16 @@ protected:
   nscolor               mBodyTextColor;
 
   ScrollbarStyles       mViewportStyleOverflow;
-  PRUint8               mFocusRingWidth;
+  uint8_t               mFocusRingWidth;
 
-  PRUint16              mImageAnimationMode;
-  PRUint16              mImageAnimationModePref;
+  uint16_t              mImageAnimationMode;
+  uint16_t              mImageAnimationModePref;
 
   LangGroupFontPrefs    mLangGroupFontPrefs;
 
   nscoord               mBorderWidthTable[3];
 
-  PRUint32              mInterruptChecksToSkip;
+  uint32_t              mInterruptChecksToSkip;
 
   mozilla::TimeStamp    mReflowStartTime;
 
@@ -1238,7 +1147,6 @@ protected:
   unsigned              mProcessingRestyles : 1;
   unsigned              mProcessingAnimationStyleChange : 1;
 
-  unsigned              mContainsUpdatePluginGeometryFrame : 1;
   unsigned              mFireAfterPaintEvents : 1;
 
   // Cache whether we are chrome or not because it is expensive.  
@@ -1268,12 +1176,14 @@ protected:
     eDefaultFont_COUNT
   };
 
+  nscolor MakeColorPref(const nsString& aColor);
+
 #ifdef DEBUG
 private:
   friend struct nsAutoLayoutPhase;
-  PRUint32 mLayoutPhaseCount[eLayoutPhase_COUNT];
+  uint32_t mLayoutPhaseCount[eLayoutPhase_COUNT];
 public:
-  PRUint32 LayoutPhaseCount(nsLayoutPhase aPhase) {
+  uint32_t LayoutPhaseCount(nsLayoutPhase aPhase) {
     return mLayoutPhaseCount[aPhase];
   }
 #endif
@@ -1295,7 +1205,7 @@ public:
   {
     if (mNotifyDidPaintTimer) {
       mNotifyDidPaintTimer->Cancel();
-      mNotifyDidPaintTimer = nsnull;
+      mNotifyDidPaintTimer = nullptr;
     }
   }
 
@@ -1305,13 +1215,13 @@ public:
    * Callers must call UnregisterPluginForGeometryUpdates before
    * the aPlugin frame is destroyed.
    */
-  void RegisterPluginForGeometryUpdates(nsObjectFrame* aPlugin);
+  void RegisterPluginForGeometryUpdates(nsIContent* aPlugin);
   /**
    * Stops a plugin receiving geometry updates (position and clip
    * region). If the plugin was not already registered, this does
    * nothing.
    */
-  void UnregisterPluginForGeometryUpdates(nsObjectFrame* aPlugin);
+  void UnregisterPluginForGeometryUpdates(nsIContent* aPlugin);
 
   /**
    * Iterate through all plugins that are registered for geometry updates
@@ -1344,21 +1254,7 @@ public:
    * of any windowed plugins is updated. aFrame is the root of the
    * frame subtree whose geometry has changed.
    */
-  void RequestUpdatePluginGeometry(nsIFrame* aFrame);
-
-  /**
-   * Call this when a frame is being destroyed and
-   * mContainsUpdatePluginGeometryFrame is set in the frame's prescontext.
-   */
-  void RootForgetUpdatePluginGeometryFrame(nsIFrame* aFrame);
-
-  /**
-   * Call this when a document is going to no longer be valid for plugin updates
-   * (say by going into the bfcache). If mContainsUpdatePluginGeometryFrame is
-   * set in the prescontext then it will be cleared along with
-   * mUpdatePluginGeometryForFrame.
-   */
-  void RootForgetUpdatePluginGeometryFrameForPresContext(nsPresContext* aPresContext);
+  void RequestUpdatePluginGeometry();
 
   /**
    * Increment DOM-modification generation counter to indicate that
@@ -1369,8 +1265,11 @@ public:
 
   /**
    * Get the current DOM generation counter.
+   *
+   * See nsFrameManagerBase::GetGlobalGenerationNumber() for a
+   * global generation number.
    */
-  PRUint32 GetDOMGeneration() { return mDOMGeneration; }
+  uint32_t GetDOMGeneration() { return mDOMGeneration; }
 
   /**
    * Add a runnable that will get called before the next paint. They will get
@@ -1390,7 +1289,7 @@ protected:
   class RunWillPaintObservers : public nsRunnable {
   public:
     RunWillPaintObservers(nsRootPresContext* aPresContext) : mPresContext(aPresContext) {}
-    void Revoke() { mPresContext = nsnull; }
+    void Revoke() { mPresContext = nullptr; }
     NS_IMETHOD Run()
     {
       if (mPresContext) {
@@ -1406,34 +1305,22 @@ protected:
   {
     if (mUpdatePluginGeometryTimer) {
       mUpdatePluginGeometryTimer->Cancel();
-      mUpdatePluginGeometryTimer = nsnull;
+      mUpdatePluginGeometryTimer = nullptr;
     }
   }
 
   nsCOMPtr<nsITimer> mNotifyDidPaintTimer;
   nsCOMPtr<nsITimer> mUpdatePluginGeometryTimer;
-  nsTHashtable<nsPtrHashKey<nsObjectFrame> > mRegisteredPlugins;
+  nsTHashtable<nsRefPtrHashKey<nsIContent> > mRegisteredPlugins;
   // if mNeedsToUpdatePluginGeometry is set, then this is the frame to
   // use as the root of the subtree to search for plugin updates, or
   // null to use the root frame of this prescontext
   nsTArray<nsCOMPtr<nsIRunnable> > mWillPaintObservers;
   nsRevocableEventPtr<RunWillPaintObservers> mWillPaintFallbackEvent;
-  nsIFrame* mUpdatePluginGeometryForFrame;
-  PRUint32 mDOMGeneration;
+  uint32_t mDOMGeneration;
   bool mNeedsToUpdatePluginGeometry;
 };
 
-inline void
-nsPresContext::ForgetUpdatePluginGeometryFrame(nsIFrame* aFrame)
-{
-  if (mContainsUpdatePluginGeometryFrame) {
-    nsRootPresContext* rootPC = GetRootPresContext();
-    if (rootPC) {
-      rootPC->RootForgetUpdatePluginGeometryFrame(aFrame);
-    }
-  }
-}
-
 #ifdef MOZ_REFLOW_PERF
 
 #define DO_GLOBAL_REFLOW_COUNT(_name) \
