@@ -1,39 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /* rendering object for list-item bullets */
 
@@ -58,7 +26,6 @@
 
 #include "nsIServiceManager.h"
 #include "nsIComponentManager.h"
-#include "nsContentUtils.h"
 
 #ifdef ACCESSIBILITY
 #include "nsAccessibilityService.h"
@@ -66,7 +33,7 @@
 
 using namespace mozilla;
 
-NS_DECLARE_FRAME_PROPERTY(FontSizeInflationProperty, nsnull)
+NS_DECLARE_FRAME_PROPERTY(FontSizeInflationProperty, nullptr)
 
 NS_IMPL_FRAMEARENA_HELPERS(nsBulletFrame)
 
@@ -84,18 +51,18 @@ nsBulletFrame::DestroyFrom(nsIFrame* aDestructRoot)
                                           mImageRequest,
                                           &mRequestRegistered);
     mImageRequest->CancelAndForgetObserver(NS_ERROR_FAILURE);
-    mImageRequest = nsnull;
+    mImageRequest = nullptr;
   }
 
   if (mListener) {
-    mListener->SetFrame(nsnull);
+    mListener->SetFrame(nullptr);
   }
 
   // Let base class do the rest
   nsFrame::DestroyFrom(aDestructRoot);
 }
 
-#ifdef NS_DEBUG
+#ifdef DEBUG
 NS_IMETHODIMP
 nsBulletFrame::GetFrameName(nsAString& aResult) const
 {
@@ -152,7 +119,7 @@ nsBulletFrame::DidSetStyleContext(nsStyleContext* aOldStyleContext)
           nsLayoutUtils::DeregisterImageRequest(PresContext(), mImageRequest,
                                                 &mRequestRegistered);
           mImageRequest->Cancel(NS_ERROR_FAILURE);
-          mImageRequest = nsnull;
+          mImageRequest = nullptr;
         }
       }
     }
@@ -172,7 +139,7 @@ nsBulletFrame::DidSetStyleContext(nsStyleContext* aOldStyleContext)
                                             &mRequestRegistered);
 
       mImageRequest->Cancel(NS_ERROR_FAILURE);
-      mImageRequest = nsnull;
+      mImageRequest = nullptr;
     }
   }
 
@@ -259,10 +226,10 @@ nsBulletFrame::PaintBullet(nsRenderingContext& aRenderingContext, nsPoint aPt,
                            const nsRect& aDirtyRect)
 {
   const nsStyleList* myList = GetStyleList();
-  PRUint8 listStyleType = myList->mListStyleType;
+  uint8_t listStyleType = myList->mListStyleType;
 
   if (myList->GetListStyleImage() && mImageRequest) {
-    PRUint32 status;
+    uint32_t status;
     mImageRequest->GetImageStatus(&status);
     if (status & imgIRequest::STATUS_LOAD_COMPLETE &&
         !(status & imgIRequest::STATUS_ERROR)) {
@@ -383,12 +350,12 @@ nsBulletFrame::PaintBullet(nsRenderingContext& aRenderingContext, nsPoint aPt,
   }
 }
 
-PRInt32
-nsBulletFrame::SetListItemOrdinal(PRInt32 aNextOrdinal,
+int32_t
+nsBulletFrame::SetListItemOrdinal(int32_t aNextOrdinal,
                                   bool* aChanged)
 {
   // Assume that the ordinal comes from the caller
-  PRInt32 oldOrdinal = mOrdinal;
+  int32_t oldOrdinal = mOrdinal;
   mOrdinal = aNextOrdinal;
 
   // Try to get value directly from the list-item, if it specifies a
@@ -421,21 +388,21 @@ nsBulletFrame::SetListItemOrdinal(PRInt32 aNextOrdinal,
  * could represent mOrder in the desired numbering system.  false
  * means we had to fall back to decimal
  */
-static bool DecimalToText(PRInt32 ordinal, nsString& result)
+static bool DecimalToText(int32_t ordinal, nsString& result)
 {
    char cbuf[40];
    PR_snprintf(cbuf, sizeof(cbuf), "%ld", ordinal);
    result.AppendASCII(cbuf);
    return true;
 }
-static bool DecimalLeadingZeroToText(PRInt32 ordinal, nsString& result)
+static bool DecimalLeadingZeroToText(int32_t ordinal, nsString& result)
 {
    char cbuf[40];
    PR_snprintf(cbuf, sizeof(cbuf), "%02ld", ordinal);
    result.AppendASCII(cbuf);
    return true;
 }
-static bool OtherDecimalToText(PRInt32 ordinal, PRUnichar zeroChar, nsString& result)
+static bool OtherDecimalToText(int32_t ordinal, PRUnichar zeroChar, nsString& result)
 {
    PRUnichar diff = zeroChar - PRUnichar('0');
    DecimalToText(ordinal, result);
@@ -444,11 +411,11 @@ static bool OtherDecimalToText(PRInt32 ordinal, PRUnichar zeroChar, nsString& re
      // skip the leading '-'
      ++p;
    }     
-   for(; nsnull != *p ; p++) 
+   for(; '\0' != *p ; p++) 
       *p += diff;
    return true;
 }
-static bool TamilToText(PRInt32 ordinal,  nsString& result)
+static bool TamilToText(int32_t ordinal,  nsString& result)
 {
    PRUnichar diff = 0x0BE6 - PRUnichar('0');
    DecimalToText(ordinal, result); 
@@ -457,7 +424,7 @@ static bool TamilToText(PRInt32 ordinal,  nsString& result)
      return false;
    }
    PRUnichar* p = result.BeginWriting();
-   for(; nsnull != *p ; p++) 
+   for(; '\0' != *p ; p++) 
       if(*p != PRUnichar('0'))
          *p += diff;
    return true;
@@ -469,7 +436,7 @@ static const char gUpperRomanCharsA[] = "IXCM";
 static const char gLowerRomanCharsB[] = "vld";
 static const char gUpperRomanCharsB[] = "VLD";
 
-static bool RomanToText(PRInt32 ordinal, nsString& result, const char* achars, const char* bchars)
+static bool RomanToText(int32_t ordinal, nsString& result, const char* achars, const char* bchars)
 {
   if (ordinal < 1 || ordinal > 3999) {
     DecimalToText(ordinal, result);
@@ -477,11 +444,11 @@ static bool RomanToText(PRInt32 ordinal, nsString& result, const char* achars, c
   }
   nsAutoString addOn, decStr;
   decStr.AppendInt(ordinal, 10);
-  PRIntn len = decStr.Length();
+  int len = decStr.Length();
   const PRUnichar* dp = decStr.get();
   const PRUnichar* end = dp + len;
-  PRIntn romanPos = len;
-  PRIntn n;
+  int romanPos = len;
+  int n;
 
   for (; dp < end; dp++) {
     romanPos--;
@@ -696,17 +663,17 @@ static const PRUnichar gEthiopicHalehameTiEtChars[ETHIOPIC_HALEHAME_TI_ET_CHARS_
 
 #define NUM_BUF_SIZE 34 
 
-static bool CharListToText(PRInt32 ordinal, nsString& result, const PRUnichar* chars, PRInt32 aBase)
+static bool CharListToText(int32_t ordinal, nsString& result, const PRUnichar* chars, int32_t aBase)
 {
   PRUnichar buf[NUM_BUF_SIZE];
-  PRInt32 idx = NUM_BUF_SIZE;
+  int32_t idx = NUM_BUF_SIZE;
   if (ordinal < 1) {
     DecimalToText(ordinal, result);
     return false;
   }
   do {
     ordinal--; // a == 0
-    PRInt32 cur = ordinal % aBase;
+    int32_t cur = ordinal % aBase;
     buf[--idx] = chars[cur];
     ordinal /= aBase ;
   } while ( ordinal > 0);
@@ -751,13 +718,13 @@ static const PRUnichar gCJKIdeographic10KUnit3[4] =
   0x000, 0x4E07, 0x5104, 0x5146
 };
 
-static const bool CJKIdeographicToText(PRInt32 ordinal, nsString& result, 
+static const bool CJKIdeographicToText(int32_t ordinal, nsString& result, 
                                    const PRUnichar* digits,
                                    const PRUnichar *unit, 
                                    const PRUnichar* unit10k)
 {
 // In theory, we need the following if condiction,
-// However, the limit, 10 ^ 16, is greater than the max of PRUint32
+// However, the limit, 10 ^ 16, is greater than the max of uint32_t
 // so we don't really need to test it here.
 // if( ordinal > 9999999999999999)
 // {
@@ -773,15 +740,15 @@ static const bool CJKIdeographicToText(PRInt32 ordinal, nsString& result,
   PRUnichar c10kUnit = 0;
   PRUnichar cUnit = 0;
   PRUnichar cDigit = 0;
-  PRUint32 ud = 0;
+  uint32_t ud = 0;
   PRUnichar buf[NUM_BUF_SIZE];
-  PRInt32 idx = NUM_BUF_SIZE;
+  int32_t idx = NUM_BUF_SIZE;
   bool bOutputZero = ( 0 == ordinal );
   do {
     if(0 == (ud % 4)) {
       c10kUnit = unit10k[ud/4];
     }
-    PRInt32 cur = ordinal % 10;
+    int32_t cur = ordinal % 10;
     cDigit = digits[cur];
     if( 0 == cur)
     {
@@ -827,7 +794,7 @@ static const PRUnichar gHebrewDigit[22] =
 0x05E7, 0x05E8, 0x05E9, 0x05EA
 };
 
-static bool HebrewToText(PRInt32 ordinal, nsString& result)
+static bool HebrewToText(int32_t ordinal, nsString& result)
 {
   if (ordinal < 1 || ordinal > 999999) {
     DecimalToText(ordinal, result);
@@ -837,9 +804,9 @@ static bool HebrewToText(PRInt32 ordinal, nsString& result)
   nsAutoString allText, thousandsGroup;
   do {
     thousandsGroup.Truncate();
-    PRInt32 n3 = ordinal % 1000;
+    int32_t n3 = ordinal % 1000;
     // Process digit for 100 - 900
-    for(PRInt32 n1 = 400; n1 > 0; )
+    for(int32_t n1 = 400; n1 > 0; )
     {
       if( n3 >= n1)
       {
@@ -851,7 +818,7 @@ static bool HebrewToText(PRInt32 ordinal, nsString& result)
     } // for
 
     // Process digit for 10 - 90
-    PRInt32 n2;
+    int32_t n2;
     if( n3 >= 10 )
     {
       // Special process for 15 and 16
@@ -886,7 +853,7 @@ static bool HebrewToText(PRInt32 ordinal, nsString& result)
 }
 
 
-static bool ArmenianToText(PRInt32 ordinal, nsString& result)
+static bool ArmenianToText(int32_t ordinal, nsString& result)
 {
   if (ordinal < 1 || ordinal > 9999) { // zero or reach the limit of Armenian numbering system
     DecimalToText(ordinal, result);
@@ -894,10 +861,10 @@ static bool ArmenianToText(PRInt32 ordinal, nsString& result)
   }
 
   PRUnichar buf[NUM_BUF_SIZE];
-  PRInt32 idx = NUM_BUF_SIZE;
-  PRInt32 d = 0;
+  int32_t idx = NUM_BUF_SIZE;
+  int32_t d = 0;
   do {
-    PRInt32 cur = ordinal % 10;
+    int32_t cur = ordinal % 10;
     if (cur > 0)
     {
       PRUnichar u = 0x0530 + (d * 9) + cur;
@@ -923,7 +890,7 @@ static const PRUnichar gGeorgianValue [ 37 ] = { // 4 * 9 + 1 = 37
 //  10000
    0x10F5
 };
-static bool GeorgianToText(PRInt32 ordinal, nsString& result)
+static bool GeorgianToText(int32_t ordinal, nsString& result)
 {
   if (ordinal < 1 || ordinal > 19999) { // zero or reach the limit of Georgian numbering system
     DecimalToText(ordinal, result);
@@ -931,10 +898,10 @@ static bool GeorgianToText(PRInt32 ordinal, nsString& result)
   }
 
   PRUnichar buf[NUM_BUF_SIZE];
-  PRInt32 idx = NUM_BUF_SIZE;
-  PRInt32 d = 0;
+  int32_t idx = NUM_BUF_SIZE;
+  int32_t d = 0;
   do {
-    PRInt32 cur = ordinal % 10;
+    int32_t cur = ordinal % 10;
     if (cur > 0)
     {
       PRUnichar u = gGeorgianValue[(d * 9 ) + ( cur - 1)];
@@ -957,7 +924,7 @@ static bool GeorgianToText(PRInt32 ordinal, nsString& result)
 #define ETHIOPIC_HUNDRED         0x137B
 #define ETHIOPIC_TEN_THOUSAND    0x137C
 
-static bool EthiopicToText(PRInt32 ordinal, nsString& result)
+static bool EthiopicToText(int32_t ordinal, nsString& result)
 {
   nsAutoString asciiNumberString;      // decimal string representation of ordinal
   DecimalToText(ordinal, asciiNumberString);
@@ -965,7 +932,7 @@ static bool EthiopicToText(PRInt32 ordinal, nsString& result)
     result.Append(asciiNumberString);
     return false;
   }
-  PRUint8 asciiStringLength = asciiNumberString.Length();
+  uint8_t asciiStringLength = asciiNumberString.Length();
 
   // If number length is odd, add a leading "0"
   // the leading "0" preconditions the string to always have the
@@ -981,12 +948,12 @@ static bool EthiopicToText(PRInt32 ordinal, nsString& result)
   // Iterate from the highest digits to lowest
   // indexFromLeft       indexes digits (0 = most significant)
   // groupIndexFromRight indexes pairs of digits (0 = least significant)
-  for (PRUint8 indexFromLeft = 0, groupIndexFromRight = asciiStringLength >> 1;
+  for (uint8_t indexFromLeft = 0, groupIndexFromRight = asciiStringLength >> 1;
        indexFromLeft <= asciiStringLength;
        indexFromLeft += 2, groupIndexFromRight--) {
-    PRUint8 tensValue  = asciiNumberString.CharAt(indexFromLeft) & 0x0F;
-    PRUint8 unitsValue = asciiNumberString.CharAt(indexFromLeft + 1) & 0x0F;
-    PRUint8 groupValue = tensValue * 10 + unitsValue;
+    uint8_t tensValue  = asciiNumberString.CharAt(indexFromLeft) & 0x0F;
+    uint8_t unitsValue = asciiNumberString.CharAt(indexFromLeft + 1) & 0x0F;
+    uint8_t groupValue = tensValue * 10 + unitsValue;
 
     bool oddGroup = (groupIndexFromRight & 1);
 
@@ -1023,8 +990,8 @@ static bool EthiopicToText(PRInt32 ordinal, nsString& result)
 
 
 /* static */ bool
-nsBulletFrame::AppendCounterText(PRInt32 aListStyleType,
-                                 PRInt32 aOrdinal,
+nsBulletFrame::AppendCounterText(int32_t aListStyleType,
+                                 int32_t aOrdinal,
                                  nsString& result)
 {
   bool success = true;
@@ -1304,16 +1271,13 @@ nsBulletFrame::GetDesiredSize(nsPresContext*  aCX,
   RemoveStateBits(BULLET_FRAME_IMAGE_LOADING);
 
   if (myList->GetListStyleImage() && mImageRequest) {
-    PRUint32 status;
+    uint32_t status;
     mImageRequest->GetImageStatus(&status);
     if (status & imgIRequest::STATUS_SIZE_AVAILABLE &&
         !(status & imgIRequest::STATUS_ERROR)) {
       // auto size the image
-      mComputedSize.width = mIntrinsicSize.width;
-      mComputedSize.height = mIntrinsicSize.height;
-
-      aMetrics.width = mComputedSize.width;
-      aMetrics.ascent = aMetrics.height = mComputedSize.height;
+      aMetrics.width = mIntrinsicSize.width;
+      aMetrics.ascent = aMetrics.height = mIntrinsicSize.height;
 
       AddStateBits(BULLET_FRAME_IMAGE_LOADING);
 
@@ -1469,7 +1433,7 @@ NS_IMETHODIMP nsBulletFrame::OnStartContainer(imgIRequest *aRequest,
   if (!aImage) return NS_ERROR_INVALID_ARG;
   if (!aRequest) return NS_ERROR_INVALID_ARG;
 
-  PRUint32 status;
+  uint32_t status;
   aRequest->GetImageStatus(&status);
   if (status & imgIRequest::STATUS_ERROR) {
     return NS_OK;
@@ -1566,7 +1530,7 @@ nsBulletFrame::GetLoadGroup(nsPresContext *aPresContext, nsILoadGroup **aLoadGro
   if (!aPresContext)
     return;
 
-  NS_PRECONDITION(nsnull != aLoadGroup, "null OUT parameter pointer");
+  NS_PRECONDITION(nullptr != aLoadGroup, "null OUT parameter pointer");
 
   nsIPresShell *shell = aPresContext->GetPresShell();
 
@@ -1581,7 +1545,7 @@ nsBulletFrame::GetLoadGroup(nsPresContext *aPresContext, nsILoadGroup **aLoadGro
 }
 
 union VoidPtrOrFloat {
-  VoidPtrOrFloat() : p(nsnull) {}
+  VoidPtrOrFloat() : p(nullptr) {}
 
   void *p;
   float f;
@@ -1658,7 +1622,7 @@ nsBulletFrame::GetBaseline() const
 NS_IMPL_ISUPPORTS2(nsBulletListener, imgIDecoderObserver, imgIContainerObserver)
 
 nsBulletListener::nsBulletListener() :
-  mFrame(nsnull)
+  mFrame(nullptr)
 {
 }
 
