@@ -1,38 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is the Mozilla SVG project.
- *
- * The Initial Developer of the Original Code is IBM Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2005
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 // Main header first:
 #include "nsSVGFilterFrame.h"
@@ -43,9 +12,11 @@
 #include "nsGkAtoms.h"
 #include "nsRenderingContext.h"
 #include "nsSVGEffects.h"
+#include "nsSVGElement.h"
 #include "nsSVGFilterElement.h"
 #include "nsSVGFilterInstance.h"
 #include "nsSVGFilterPaintCallback.h"
+#include "nsSVGIntegrationUtils.h"
 #include "nsSVGUtils.h"
 
 nsIFrame*
@@ -56,24 +27,71 @@ NS_NewSVGFilterFrame(nsIPresShell* aPresShell, nsStyleContext* aContext)
 
 NS_IMPL_FRAMEARENA_HELPERS(nsSVGFilterFrame)
 
+/**
+ * Converts an nsRect that is relative to a filtered frame's origin (i.e. the
+ * top-left corner of its border box) into filter space.
+ * Returns the entire filter region (a rect the width/height of aFilterRes) if
+ * aFrameRect is null, or if the result is too large to be stored in an
+ * nsIntRect.
+ */
 static nsIntRect
-MapDeviceRectToFilterSpace(const gfxMatrix& aMatrix,
-                           const gfxIntSize& aFilterSize,
-                           const nsIntRect* aDeviceRect)
+MapFrameRectToFilterSpace(const nsRect* aRect,
+                          int32_t aAppUnitsPerCSSPx,
+                          const gfxMatrix& aFrameSpaceInCSSPxToFilterSpace,
+                          const gfxIntSize& aFilterRes)
 {
-  nsIntRect rect(0, 0, aFilterSize.width, aFilterSize.height);
-  if (aDeviceRect) {
-    gfxRect r = aMatrix.TransformBounds(gfxRect(aDeviceRect->x, aDeviceRect->y,
-                                                aDeviceRect->width, aDeviceRect->height));
-    r.RoundOut();
+  nsIntRect rect(0, 0, aFilterRes.width, aFilterRes.height);
+  if (aRect) {
+    gfxRect rectInCSSPx =
+      nsLayoutUtils::RectToGfxRect(*aRect, aAppUnitsPerCSSPx);
+    gfxRect rectInFilterSpace =
+      aFrameSpaceInCSSPxToFilterSpace.TransformBounds(rectInCSSPx);
+    rectInFilterSpace.RoundOut();
     nsIntRect intRect;
-    if (gfxUtils::GfxRectToIntRect(r, &intRect)) {
+    if (gfxUtils::GfxRectToIntRect(rectInFilterSpace, &intRect)) {
       rect = intRect;
     }
   }
   return rect;
 }
 
+/**
+ * Returns the transform from frame space to the coordinate space that
+ * GetCanvasTM transforms to. "Frame space" is the origin of a frame, aka the
+ * top-left corner of its border box, aka the top left corner of its mRect.
+ */
+static gfxMatrix
+GetUserToFrameSpaceInCSSPxTransform(nsIFrame *aFrame)
+{
+  gfxMatrix userToFrameSpaceInCSSPx;
+
+  if ((aFrame->GetStateBits() & NS_FRAME_SVG_LAYOUT)) {
+    int32_t appUnitsPerCSSPx = aFrame->PresContext()->AppUnitsPerCSSPixel();
+    // As currently implemented by Mozilla for the purposes of filters, user
+    // space is the coordinate system established by GetCanvasTM(), since
+    // that's what we use to set filterToDeviceSpace above. In other words,
+    // for SVG, user space is actually the coordinate system aTarget
+    // establishes for _its_ children (i.e. after taking account of any x/y
+    // and viewBox attributes), not the coordinate system that is established
+    // for it by its 'transform' attribute (or by its _parent_) as it's
+    // normally defined. (XXX We should think about fixing this.) The only
+    // frame type for which these extra transforms are not simply an x/y
+    // translation is nsSVGInnerSVGFrame, hence we treat it specially here.
+    if (aFrame->GetType() == nsGkAtoms::svgInnerSVGFrame) {
+      userToFrameSpaceInCSSPx =
+        static_cast<nsSVGElement*>(aFrame->GetContent())->
+          PrependLocalTransformsTo(gfxMatrix());
+    } else {
+      gfxPoint targetsUserSpaceOffset =
+        nsLayoutUtils::RectToGfxRect(aFrame->GetRect(), appUnitsPerCSSPx).
+                         TopLeft();
+      userToFrameSpaceInCSSPx.Translate(-targetsUserSpaceOffset);
+    }
+  }
+  // else, for all other frames, leave as the identity matrix
+  return userToFrameSpaceInCSSPx;
+}
+
 class nsSVGFilterFrame::AutoFilterReferencer
 {
 public:
@@ -97,9 +115,10 @@ public:
   nsAutoFilterInstance(nsIFrame *aTarget,
                        nsSVGFilterFrame *aFilterFrame,
                        nsSVGFilterPaintCallback *aPaint,
-                       const nsIntRect *aDirtyOutputRect,
-                       const nsIntRect *aDirtyInputRect,
-                       const nsIntRect *aOverrideSourceBBox);
+                       const nsRect *aPostFilterDirtyRect,
+                       const nsRect *aPreFilterDirtyRect,
+                       const nsRect *aOverridePreFilterVisualOverflowRect,
+                       const gfxRect *aOverrideBBox = nullptr);
   ~nsAutoFilterInstance() {}
 
   // If this returns null, then draw nothing. Either the filter draws
@@ -113,24 +132,19 @@ private:
 nsAutoFilterInstance::nsAutoFilterInstance(nsIFrame *aTarget,
                                            nsSVGFilterFrame *aFilterFrame,
                                            nsSVGFilterPaintCallback *aPaint,
-                                           const nsIntRect *aDirtyOutputRect,
-                                           const nsIntRect *aDirtyInputRect,
-                                           const nsIntRect *aOverrideSourceBBox)
+                                           const nsRect *aPostFilterDirtyRect,
+                                           const nsRect *aPreFilterDirtyRect,
+                                           const nsRect *aPreFilterVisualOverflowRectOverride,
+                                           const gfxRect *aOverrideBBox)
 {
   const nsSVGFilterElement *filter = aFilterFrame->GetFilterContent();
 
-  PRUint16 filterUnits =
+  uint16_t filterUnits =
     aFilterFrame->GetEnumValue(nsSVGFilterElement::FILTERUNITS);
-  PRUint16 primitiveUnits =
+  uint16_t primitiveUnits =
     aFilterFrame->GetEnumValue(nsSVGFilterElement::PRIMITIVEUNITS);
 
-  gfxRect bbox;
-  if (aOverrideSourceBBox) {
-    bbox = gfxRect(aOverrideSourceBBox->x, aOverrideSourceBBox->y,
-                   aOverrideSourceBBox->width, aOverrideSourceBBox->height);
-  } else {
-    bbox = nsSVGUtils::GetBBox(aTarget);
-  }
+  gfxRect bbox = aOverrideBBox ? *aOverrideBBox : nsSVGUtils::GetBBox(aTarget);
 
   // Get the filter region (in the filtered element's user space):
 
@@ -152,6 +166,7 @@ nsAutoFilterInstance::nsAutoFilterInstance(nsIFrame *aTarget,
   XYWH[1] = *aFilterFrame->GetLengthValue(nsSVGFilterElement::Y);
   XYWH[2] = *aFilterFrame->GetLengthValue(nsSVGFilterElement::WIDTH);
   XYWH[3] = *aFilterFrame->GetLengthValue(nsSVGFilterElement::HEIGHT);
+  // The filter region in user space, in user units:
   gfxRect filterRegion = nsSVGUtils::GetRelativeRect(filterUnits,
     XYWH, bbox, aTarget);
 
@@ -161,21 +176,18 @@ nsAutoFilterInstance::nsAutoFilterInstance(nsIFrame *aTarget,
     return;
   }
 
-  gfxMatrix userToDeviceSpace = nsSVGUtils::GetCanvasTM(aTarget);
-  if (userToDeviceSpace.IsSingular()) {
-    // nothing to draw
-    return;
-  }
-  
   // Calculate filterRes (the width and height of the pixel buffer of the
-  // temporary offscreen surface that we'll paint into):
+  // temporary offscreen surface that we would/will create to paint into when
+  // painting the entire filtered element) and, if necessary, adjust
+  // filterRegion out slightly so that it aligns with pixel boundaries of this
+  // buffer:
 
   gfxIntSize filterRes;
   const nsSVGIntegerPair* filterResAttrs =
     aFilterFrame->GetIntegerPairValue(nsSVGFilterElement::FILTERRES);
   if (filterResAttrs->IsExplicitlySet()) {
-    PRInt32 filterResX = filterResAttrs->GetAnimValue(nsSVGIntegerPair::eFirst);
-    PRInt32 filterResY = filterResAttrs->GetAnimValue(nsSVGIntegerPair::eSecond);
+    int32_t filterResX = filterResAttrs->GetAnimValue(nsSVGIntegerPair::eFirst);
+    int32_t filterResY = filterResAttrs->GetAnimValue(nsSVGIntegerPair::eSecond);
     if (filterResX <= 0 || filterResY <= 0) {
       // 0 disables rendering, < 0 is error. dispatch error console warning?
       return;
@@ -195,7 +207,13 @@ nsAutoFilterInstance::nsAutoFilterInstance(nsIFrame *aTarget,
   } else {
     // Match filterRes as closely as possible to the pixel density of the nearest
     // outer 'svg' device space:
-    float scale = nsSVGUtils::MaxExpansion(userToDeviceSpace);
+    gfxMatrix canvasTM =
+      nsSVGUtils::GetCanvasTM(aTarget, nsISVGChildFrame::FOR_OUTERSVG_TM);
+    if (canvasTM.IsSingular()) {
+      // nothing to draw
+      return;
+    }
+    float scale = nsSVGUtils::MaxExpansion(canvasTM);
 
     filterRegion.Scale(scale);
     filterRegion.RoundOut();
@@ -207,43 +225,59 @@ nsAutoFilterInstance::nsAutoFilterInstance(nsIFrame *aTarget,
     filterRegion.Scale(1.0 / scale);
   }
 
-  // XXX we haven't taken account of the fact that filterRegion may be
-  // partially or entirely outside the current clip region. :-/
-
-  // Convert the dirty rects to filter space, and create our nsSVGFilterInstance:
+  // Get various transforms:
 
   gfxMatrix filterToUserSpace(filterRegion.Width() / filterRes.width, 0.0f,
                               0.0f, filterRegion.Height() / filterRes.height,
                               filterRegion.X(), filterRegion.Y());
-  gfxMatrix filterToDeviceSpace = filterToUserSpace * userToDeviceSpace;
-  
-  // filterToDeviceSpace is always invertible
-  gfxMatrix deviceToFilterSpace = filterToDeviceSpace;
-  deviceToFilterSpace.Invert();
 
-  nsIntRect dirtyOutputRect =
-    MapDeviceRectToFilterSpace(deviceToFilterSpace, filterRes, aDirtyOutputRect);
-  nsIntRect dirtyInputRect =
-    MapDeviceRectToFilterSpace(deviceToFilterSpace, filterRes, aDirtyInputRect);
-  nsIntRect targetBoundsDeviceSpace;
-  nsISVGChildFrame* svgTarget = do_QueryFrame(aTarget);
-  if (svgTarget) {
-    targetBoundsDeviceSpace.UnionRect(targetBoundsDeviceSpace,
-      svgTarget->GetCoveredRegion().ToOutsidePixels(aTarget->PresContext()->AppUnitsPerDevPixel()));
+  // Only used (so only set) when we paint:
+  gfxMatrix filterToDeviceSpace;
+  if (aPaint) {
+    filterToDeviceSpace = filterToUserSpace *
+              nsSVGUtils::GetCanvasTM(aTarget, nsISVGChildFrame::FOR_PAINTING);
+  }
+
+  // Convert the passed in rects from frame to filter space:
+
+  int32_t appUnitsPerCSSPx = aTarget->PresContext()->AppUnitsPerCSSPixel();
+
+  gfxMatrix filterToFrameSpaceInCSSPx =
+    filterToUserSpace * GetUserToFrameSpaceInCSSPxTransform(aTarget);
+  // filterToFrameSpaceInCSSPx is always invertible
+  gfxMatrix frameSpaceInCSSPxTofilterSpace = filterToFrameSpaceInCSSPx;
+  frameSpaceInCSSPxTofilterSpace.Invert();
+
+  nsIntRect postFilterDirtyRect =
+    MapFrameRectToFilterSpace(aPostFilterDirtyRect, appUnitsPerCSSPx,
+                              frameSpaceInCSSPxTofilterSpace, filterRes);
+  nsIntRect preFilterDirtyRect =
+    MapFrameRectToFilterSpace(aPreFilterDirtyRect, appUnitsPerCSSPx,
+                              frameSpaceInCSSPxTofilterSpace, filterRes);
+  nsIntRect preFilterVisualOverflowRect;
+  if (aPreFilterVisualOverflowRectOverride) {
+    preFilterVisualOverflowRect =
+      MapFrameRectToFilterSpace(aPreFilterVisualOverflowRectOverride,
+                                appUnitsPerCSSPx,
+                                frameSpaceInCSSPxTofilterSpace, filterRes);
+  } else {
+    nsRect preFilterVOR = aTarget->GetPreEffectsVisualOverflowRect();
+    preFilterVisualOverflowRect =
+      MapFrameRectToFilterSpace(&preFilterVOR, appUnitsPerCSSPx,
+                                frameSpaceInCSSPxTofilterSpace, filterRes);
   }
-  nsIntRect targetBoundsFilterSpace =
-    MapDeviceRectToFilterSpace(deviceToFilterSpace, filterRes, &targetBoundsDeviceSpace);
 
   // Setup instance data
-  mInstance = new nsSVGFilterInstance(aTarget, aPaint, filter, bbox, filterRegion,
-                                      nsIntSize(filterRes.width, filterRes.height),
-                                      filterToDeviceSpace, targetBoundsFilterSpace,
-                                      dirtyOutputRect, dirtyInputRect,
-                                      primitiveUnits);
+  mInstance =
+    new nsSVGFilterInstance(aTarget, aPaint, filter, bbox, filterRegion,
+                            nsIntSize(filterRes.width, filterRes.height),
+                            filterToDeviceSpace, filterToFrameSpaceInCSSPx,
+                            preFilterVisualOverflowRect, postFilterDirtyRect,
+                            preFilterDirtyRect, primitiveUnits);
 }
 
-PRUint16
-nsSVGFilterFrame::GetEnumValue(PRUint32 aIndex, nsIContent *aDefault)
+uint16_t
+nsSVGFilterFrame::GetEnumValue(uint32_t aIndex, nsIContent *aDefault)
 {
   nsSVGEnum& thisEnum =
     static_cast<nsSVGFilterElement *>(mContent)->mEnumAttributes[aIndex];
@@ -260,7 +294,7 @@ nsSVGFilterFrame::GetEnumValue(PRUint32 aIndex, nsIContent *aDefault)
 }
 
 const nsSVGIntegerPair *
-nsSVGFilterFrame::GetIntegerPairValue(PRUint32 aIndex, nsIContent *aDefault)
+nsSVGFilterFrame::GetIntegerPairValue(uint32_t aIndex, nsIContent *aDefault)
 {
   const nsSVGIntegerPair *thisIntegerPair =
     &static_cast<nsSVGFilterElement *>(mContent)->mIntegerPairAttributes[aIndex];
@@ -276,7 +310,7 @@ nsSVGFilterFrame::GetIntegerPairValue(PRUint32 aIndex, nsIContent *aDefault)
 }
 
 const nsSVGLength2 *
-nsSVGFilterFrame::GetLengthValue(PRUint32 aIndex, nsIContent *aDefault)
+nsSVGFilterFrame::GetLengthValue(uint32_t aIndex, nsIContent *aDefault)
 {
   const nsSVGLength2 *thisLength =
     &static_cast<nsSVGFilterElement *>(mContent)->mLengthAttributes[aIndex];
@@ -315,7 +349,7 @@ nsSVGFilterFrame *
 nsSVGFilterFrame::GetReferencedFilter()
 {
   if (mNoHRefURI)
-    return nsnull;
+    return nullptr;
 
   nsSVGPaintingProperty *property = static_cast<nsSVGPaintingProperty*>
     (Properties().Get(nsSVGEffects::HrefProperty()));
@@ -327,7 +361,7 @@ nsSVGFilterFrame::GetReferencedFilter()
     filter->mStringAttributes[nsSVGFilterElement::HREF].GetAnimValue(href, filter);
     if (href.IsEmpty()) {
       mNoHRefURI = true;
-      return nsnull; // no URL
+      return nullptr; // no URL
     }
 
     // Convert href to an nsIURI
@@ -339,16 +373,16 @@ nsSVGFilterFrame::GetReferencedFilter()
     property =
       nsSVGEffects::GetPaintingProperty(targetURI, this, nsSVGEffects::HrefProperty());
     if (!property)
-      return nsnull;
+      return nullptr;
   }
 
   nsIFrame *result = property->GetReferencedFrame();
   if (!result)
-    return nsnull;
+    return nullptr;
 
   nsIAtom* frameType = result->GetType();
   if (frameType != nsGkAtoms::svgFilterFrame)
-    return nsnull;
+    return nullptr;
 
   return static_cast<nsSVGFilterFrame*>(result);
 }
@@ -358,21 +392,21 @@ nsSVGFilterFrame::GetReferencedFilterIfNotInUse()
 {
   nsSVGFilterFrame *referenced = GetReferencedFilter();
   if (!referenced)
-    return nsnull;
+    return nullptr;
 
   if (referenced->mLoopFlag) {
     // XXXjwatt: we should really send an error to the JavaScript Console here:
     NS_WARNING("Filter reference loop detected while inheriting attribute!");
-    return nsnull;
+    return nullptr;
   }
 
   return referenced;
 }
 
 NS_IMETHODIMP
-nsSVGFilterFrame::AttributeChanged(PRInt32  aNameSpaceID,
+nsSVGFilterFrame::AttributeChanged(int32_t  aNameSpaceID,
                                    nsIAtom* aAttribute,
-                                   PRInt32  aModType)
+                                   int32_t  aModType)
 {
   if (aNameSpaceID == kNameSpaceID_None &&
       (aAttribute == nsGkAtoms::x ||
@@ -382,30 +416,30 @@ nsSVGFilterFrame::AttributeChanged(PRInt32  aNameSpaceID,
        aAttribute == nsGkAtoms::filterRes ||
        aAttribute == nsGkAtoms::filterUnits ||
        aAttribute == nsGkAtoms::primitiveUnits)) {
-    nsSVGEffects::InvalidateRenderingObservers(this);
+    nsSVGEffects::InvalidateDirectRenderingObservers(this);
   } else if (aNameSpaceID == kNameSpaceID_XLink &&
              aAttribute == nsGkAtoms::href) {
     // Blow away our reference, if any
     Properties().Delete(nsSVGEffects::HrefProperty());
     mNoHRefURI = false;
     // And update whoever references us
-    nsSVGEffects::InvalidateRenderingObservers(this);
+    nsSVGEffects::InvalidateDirectRenderingObservers(this);
   }
   return nsSVGFilterFrameBase::AttributeChanged(aNameSpaceID,
                                                 aAttribute, aModType);
 }
 
 nsresult
-nsSVGFilterFrame::FilterPaint(nsRenderingContext *aContext,
-                              nsIFrame *aTarget,
-                              nsSVGFilterPaintCallback *aPaintCallback,
-                              const nsIntRect *aDirtyRect)
+nsSVGFilterFrame::PaintFilteredFrame(nsRenderingContext *aContext,
+                                     nsIFrame *aFilteredFrame,
+                                     nsSVGFilterPaintCallback *aPaintCallback,
+                                     const nsRect *aDirtyArea)
 {
-  nsAutoFilterInstance instance(aTarget, this, aPaintCallback,
-    aDirtyRect, nsnull, nsnull);
-  if (!instance.get())
+  nsAutoFilterInstance instance(aFilteredFrame, this, aPaintCallback,
+                                aDirtyArea, nullptr, nullptr);
+  if (!instance.get()) {
     return NS_OK;
-
+  }
   nsRefPtr<gfxASurface> result;
   nsresult rv = instance.get()->Render(getter_AddRefs(result));
   if (NS_SUCCEEDED(rv) && result) {
@@ -415,80 +449,79 @@ nsSVGFilterFrame::FilterPaint(nsRenderingContext *aContext,
   return rv;
 }
 
-static nsresult
-TransformFilterSpaceToDeviceSpace(nsSVGFilterInstance *aInstance, nsIntRect *aRect)
+static nsRect
+TransformFilterSpaceToFrameSpace(nsSVGFilterInstance *aInstance,
+                                 nsIntRect *aRect)
 {
-  gfxMatrix m = aInstance->GetFilterSpaceToDeviceSpaceTransform();
+  gfxMatrix m = aInstance->GetFilterSpaceToFrameSpaceInCSSPxTransform();
   gfxRect r(aRect->x, aRect->y, aRect->width, aRect->height);
   r = m.TransformBounds(r);
-  r.RoundOut();
-  nsIntRect deviceRect;
-  if (!gfxUtils::GfxRectToIntRect(r, &deviceRect))
-    return NS_ERROR_FAILURE;
-  *aRect = deviceRect;
-  return NS_OK;
+  return nsLayoutUtils::RoundGfxRectToAppRect(r, aInstance->AppUnitsPerCSSPixel());
 }
 
-nsIntRect
-nsSVGFilterFrame::GetInvalidationBBox(nsIFrame *aTarget, const nsIntRect& aRect)
+nsRect
+nsSVGFilterFrame::GetPostFilterDirtyArea(nsIFrame *aFilteredFrame,
+                                         const nsRect& aPreFilterDirtyRect)
 {
-  nsAutoFilterInstance instance(aTarget, this, nsnull, nsnull, &aRect, nsnull);
-  if (!instance.get())
-    return nsIntRect();
-
+  nsAutoFilterInstance instance(aFilteredFrame, this, nullptr, nullptr,
+                                &aPreFilterDirtyRect, nullptr);
+  if (!instance.get()) {
+    return nsRect();
+  }
   // We've passed in the source's dirty area so the instance knows about it.
   // Now we can ask the instance to compute the area of the filter output
   // that's dirty.
   nsIntRect dirtyRect;
-  nsresult rv = instance.get()->ComputeOutputDirtyRect(&dirtyRect);
+  nsresult rv = instance.get()->ComputePostFilterDirtyRect(&dirtyRect);
   if (NS_SUCCEEDED(rv)) {
-    rv = TransformFilterSpaceToDeviceSpace(instance.get(), &dirtyRect);
-    if (NS_SUCCEEDED(rv))
-      return dirtyRect;
+    return TransformFilterSpaceToFrameSpace(instance.get(), &dirtyRect);
   }
-
-  return nsIntRect();
+  return nsRect();
 }
 
-nsIntRect
-nsSVGFilterFrame::GetSourceForInvalidArea(nsIFrame *aTarget, const nsIntRect& aRect)
+nsRect
+nsSVGFilterFrame::GetPreFilterNeededArea(nsIFrame *aFilteredFrame,
+                                         const nsRect& aPostFilterDirtyRect)
 {
-  nsAutoFilterInstance instance(aTarget, this, nsnull, &aRect, nsnull, nsnull);
-  if (!instance.get())
-    return nsIntRect();
-
+  nsAutoFilterInstance instance(aFilteredFrame, this, nullptr,
+                                &aPostFilterDirtyRect, nullptr, nullptr);
+  if (!instance.get()) {
+    return nsRect();
+  }
   // Now we can ask the instance to compute the area of the source
   // that's needed.
   nsIntRect neededRect;
   nsresult rv = instance.get()->ComputeSourceNeededRect(&neededRect);
   if (NS_SUCCEEDED(rv)) {
-    rv = TransformFilterSpaceToDeviceSpace(instance.get(), &neededRect);
-    if (NS_SUCCEEDED(rv))
-      return neededRect;
+    return TransformFilterSpaceToFrameSpace(instance.get(), &neededRect);
   }
-
-  return nsIntRect();
+  return nsRect();
 }
 
-nsIntRect
-nsSVGFilterFrame::GetFilterBBox(nsIFrame *aTarget, const nsIntRect *aSourceBBox)
+nsRect
+nsSVGFilterFrame::GetPostFilterBounds(nsIFrame *aFilteredFrame,
+                                      const gfxRect *aOverrideBBox,
+                                      const nsRect *aPreFilterBounds)
 {
-  nsAutoFilterInstance instance(aTarget, this, nsnull, nsnull, nsnull, aSourceBBox);
-  if (!instance.get())
-    return nsIntRect();
+  MOZ_ASSERT(!(aFilteredFrame->GetStateBits() & NS_FRAME_SVG_LAYOUT) ||
+             !(aFilteredFrame->GetStateBits() & NS_STATE_SVG_NONDISPLAY_CHILD),
+             "Non-display SVG do not maintain visual overflow rects");
 
+  nsAutoFilterInstance instance(aFilteredFrame, this, nullptr, nullptr,
+                                aPreFilterBounds, aPreFilterBounds,
+                                aOverrideBBox);
+  if (!instance.get()) {
+    return nsRect();
+  }
   // We've passed in the source's bounding box so the instance knows about
   // it. Now we can ask the instance to compute the bounding box of
   // the filter output.
   nsIntRect bbox;
   nsresult rv = instance.get()->ComputeOutputBBox(&bbox);
   if (NS_SUCCEEDED(rv)) {
-    rv = TransformFilterSpaceToDeviceSpace(instance.get(), &bbox);
-    if (NS_SUCCEEDED(rv))
-      return bbox;
+    return TransformFilterSpaceToFrameSpace(instance.get(), &bbox);
   }
-  
-  return nsIntRect();
+  return nsRect();
 }
   
 #ifdef DEBUG
