@@ -1,38 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is the Mozilla SVG project.
- *
- * The Initial Developer of the Original Code is IBM Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2005
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 // Main header first:
 #include "nsSVGFilterInstance.h"
@@ -41,13 +10,14 @@
 #include "gfxPlatform.h"
 #include "gfxUtils.h"
 #include "nsIDOMSVGUnitTypes.h"
+#include "nsISVGChildFrame.h"
 #include "nsRenderingContext.h"
 #include "nsSVGFilterElement.h"
 #include "nsSVGFilterPaintCallback.h"
 #include "nsSVGUtils.h"
 
 float
-nsSVGFilterInstance::GetPrimitiveNumber(PRUint8 aCtxType, float aValue) const
+nsSVGFilterInstance::GetPrimitiveNumber(uint8_t aCtxType, float aValue) const
 {
   nsSVGLength2 val;
   val.Init(aCtxType, 0xff, aValue,
@@ -62,14 +32,14 @@ nsSVGFilterInstance::GetPrimitiveNumber(PRUint8 aCtxType, float aValue) const
 
   switch (aCtxType) {
   case nsSVGUtils::X:
-    return value * mFilterSpaceSize.width / mFilterRect.Width();
+    return value * mFilterSpaceSize.width / mFilterRegion.Width();
   case nsSVGUtils::Y:
-    return value * mFilterSpaceSize.height / mFilterRect.Height();
+    return value * mFilterSpaceSize.height / mFilterRegion.Height();
   case nsSVGUtils::XY:
   default:
     return value * nsSVGUtils::ComputeNormalizedHypotenuse(
-                     mFilterSpaceSize.width / mFilterRect.Width(),
-                     mFilterSpaceSize.height / mFilterRect.Height());
+                     mFilterSpaceSize.width / mFilterRegion.Width(),
+                     mFilterSpaceSize.height / mFilterRegion.Height());
   }
 }
 
@@ -102,11 +72,11 @@ nsSVGFilterInstance::CreateImage()
                         gfxASurface::ImageFormatARGB32);
 
   if (!surface || surface->CairoStatus())
-    return nsnull;
+    return nullptr;
 
   surface->SetDeviceOffset(gfxPoint(-mSurfaceRect.x, -mSurfaceRect.y));
 
-  gfxImageSurface *retval = nsnull;
+  gfxImageSurface *retval = nullptr;
   surface.swap(retval);
   return retval;
 }
@@ -114,27 +84,27 @@ nsSVGFilterInstance::CreateImage()
 gfxRect
 nsSVGFilterInstance::UserSpaceToFilterSpace(const gfxRect& aRect) const
 {
-  gfxRect r = aRect - mFilterRect.TopLeft();
-  r.Scale(mFilterSpaceSize.width / mFilterRect.Width(),
-          mFilterSpaceSize.height / mFilterRect.Height());
+  gfxRect r = aRect - mFilterRegion.TopLeft();
+  r.Scale(mFilterSpaceSize.width / mFilterRegion.Width(),
+          mFilterSpaceSize.height / mFilterRegion.Height());
   return r;
 }
 
 gfxPoint
 nsSVGFilterInstance::FilterSpaceToUserSpace(const gfxPoint& aPt) const
 {
-  return gfxPoint(aPt.x * mFilterRect.Width() / mFilterSpaceSize.width + mFilterRect.X(),
-                  aPt.y * mFilterRect.Height() / mFilterSpaceSize.height + mFilterRect.Y());
+  return gfxPoint(aPt.x * mFilterRegion.Width() / mFilterSpaceSize.width + mFilterRegion.X(),
+                  aPt.y * mFilterRegion.Height() / mFilterSpaceSize.height + mFilterRegion.Y());
 }
 
 gfxMatrix
 nsSVGFilterInstance::GetUserSpaceToFilterSpaceTransform() const
 {
-  gfxFloat widthScale = mFilterSpaceSize.width / mFilterRect.Width();
-  gfxFloat heightScale = mFilterSpaceSize.height / mFilterRect.Height();
+  gfxFloat widthScale = mFilterSpaceSize.width / mFilterRegion.Width();
+  gfxFloat heightScale = mFilterSpaceSize.height / mFilterRegion.Height();
   return gfxMatrix(widthScale, 0.0f,
                    0.0f, heightScale,
-                   -mFilterRect.X() * widthScale, -mFilterRect.Y() * heightScale);
+                   -mFilterRegion.X() * widthScale, -mFilterRegion.Y() * heightScale);
 }
 
 void
@@ -144,7 +114,7 @@ nsSVGFilterInstance::ComputeFilterPrimitiveSubregion(PrimitiveInfo* aPrimitive)
 
   gfxRect defaultFilterSubregion(0,0,0,0);
   if (fE->SubregionIsUnionOfRegions()) {
-    for (PRUint32 i = 0; i < aPrimitive->mInputs.Length(); ++i) {
+    for (uint32_t i = 0; i < aPrimitive->mInputs.Length(); ++i) {
       defaultFilterSubregion =
           defaultFilterSubregion.Union(
               aPrimitive->mInputs[i]->mImage.mFilterPrimitiveSubregion);
@@ -180,6 +150,8 @@ nsSVGFilterInstance::BuildSources()
   gfxRect filterRegion = gfxRect(0, 0, mFilterSpaceSize.width, mFilterSpaceSize.height);
   mSourceColorAlpha.mImage.mFilterPrimitiveSubregion = filterRegion;
   mSourceAlpha.mImage.mFilterPrimitiveSubregion = filterRegion;
+  mFillPaint.mImage.mFilterPrimitiveSubregion = filterRegion;
+  mStrokePaint.mImage.mFilterPrimitiveSubregion = filterRegion;
 
   nsIntRect sourceBoundsInt;
   gfxRect sourceBounds = UserSpaceToFilterSpace(mTargetBBox);
@@ -191,6 +163,8 @@ nsSVGFilterInstance::BuildSources()
 
   mSourceColorAlpha.mResultBoundingBox = sourceBoundsInt;
   mSourceAlpha.mResultBoundingBox = sourceBoundsInt;
+  mFillPaint.mResultBoundingBox = sourceBoundsInt;
+  mStrokePaint.mResultBoundingBox = sourceBoundsInt;
   return NS_OK;
 }
 
@@ -215,13 +189,13 @@ nsSVGFilterInstance::BuildPrimitives()
   nsTHashtable<ImageAnalysisEntry> imageTable;
   imageTable.Init(10);
 
-  for (PRUint32 i = 0; i < mPrimitives.Length(); ++i) {
+  for (uint32_t i = 0; i < mPrimitives.Length(); ++i) {
     PrimitiveInfo* info = &mPrimitives[i];
     nsSVGFE* filter = info->mFE;
     nsAutoTArray<nsSVGStringInfo,2> sources;
     filter->GetSourceImageNames(sources);
  
-    for (PRUint32 j=0; j<sources.Length(); ++j) {
+    for (uint32_t j=0; j<sources.Length(); ++j) {
       nsAutoString str;
       sources[j].mString->GetAnimValue(str, sources[j].mElement);
       PrimitiveInfo* sourceInfo;
@@ -230,10 +204,12 @@ nsSVGFilterInstance::BuildPrimitives()
         sourceInfo = &mSourceColorAlpha;
       } else if (str.EqualsLiteral("SourceAlpha")) {
         sourceInfo = &mSourceAlpha;
+      } else if (str.EqualsLiteral("FillPaint")) {
+        sourceInfo = &mFillPaint;
+      } else if (str.EqualsLiteral("StrokePaint")) {
+        sourceInfo = &mStrokePaint;
       } else if (str.EqualsLiteral("BackgroundImage") ||
-                 str.EqualsLiteral("BackgroundAlpha") ||
-                 str.EqualsLiteral("FillPaint") ||
-                 str.EqualsLiteral("StrokePaint")) {
+                 str.EqualsLiteral("BackgroundAlpha")) {
         return NS_ERROR_NOT_IMPLEMENTED;
       } else if (str.EqualsLiteral("")) {
         sourceInfo = i == 0 ? &mSourceColorAlpha : &mPrimitives[i - 1];
@@ -270,10 +246,10 @@ nsSVGFilterInstance::BuildPrimitives()
 void
 nsSVGFilterInstance::ComputeResultBoundingBoxes()
 {
-  for (PRUint32 i = 0; i < mPrimitives.Length(); ++i) {
+  for (uint32_t i = 0; i < mPrimitives.Length(); ++i) {
     PrimitiveInfo* info = &mPrimitives[i];
     nsAutoTArray<nsIntRect,2> sourceBBoxes;
-    for (PRUint32 j = 0; j < info->mInputs.Length(); ++j) {
+    for (uint32_t j = 0; j < info->mInputs.Length(); ++j) {
       sourceBBoxes.AppendElement(info->mInputs[j]->mResultBoundingBox);
     }
     
@@ -287,10 +263,10 @@ nsSVGFilterInstance::ComputeResultBoundingBoxes()
 void
 nsSVGFilterInstance::ComputeResultChangeBoxes()
 {
-  for (PRUint32 i = 0; i < mPrimitives.Length(); ++i) {
+  for (uint32_t i = 0; i < mPrimitives.Length(); ++i) {
     PrimitiveInfo* info = &mPrimitives[i];
     nsAutoTArray<nsIntRect,2> sourceChangeBoxes;
-    for (PRUint32 j = 0; j < info->mInputs.Length(); ++j) {
+    for (uint32_t j = 0; j < info->mInputs.Length(); ++j) {
       sourceChangeBoxes.AppendElement(info->mInputs[j]->mResultChangeBox);
     }
 
@@ -308,19 +284,19 @@ nsSVGFilterInstance::ComputeNeededBoxes()
   // In the end, we need whatever the final filter primitive will draw that
   // intersects the destination dirty area.
   mPrimitives[mPrimitives.Length() - 1].mResultNeededBox.IntersectRect(
-    mPrimitives[mPrimitives.Length() - 1].mResultBoundingBox, mDirtyOutputRect);
+    mPrimitives[mPrimitives.Length() - 1].mResultBoundingBox, mPostFilterDirtyRect);
 
-  for (PRInt32 i = mPrimitives.Length() - 1; i >= 0; --i) {
+  for (int32_t i = mPrimitives.Length() - 1; i >= 0; --i) {
     PrimitiveInfo* info = &mPrimitives[i];
     nsAutoTArray<nsIntRect,2> sourceBBoxes;
-    for (PRUint32 j = 0; j < info->mInputs.Length(); ++j) {
+    for (uint32_t j = 0; j < info->mInputs.Length(); ++j) {
       sourceBBoxes.AppendElement(info->mInputs[j]->mResultBoundingBox);
     }
     
     info->mFE->ComputeNeededSourceBBoxes(
       info->mResultNeededBox, sourceBBoxes, *this);
     // Update each source with the rectangle we need
-    for (PRUint32 j = 0; j < info->mInputs.Length(); ++j) {
+    for (uint32_t j = 0; j < info->mInputs.Length(); ++j) {
       nsIntRect* r = &info->mInputs[j]->mResultNeededBox;
       r->UnionRect(*r, sourceBBoxes[j]);
       // Keep everything within the filter effects region
@@ -336,12 +312,86 @@ nsSVGFilterInstance::ComputeUnionOfAllNeededBoxes()
   nsIntRect r;
   r.UnionRect(mSourceColorAlpha.mResultNeededBox,
               mSourceAlpha.mResultNeededBox);
-  for (PRUint32 i = 0; i < mPrimitives.Length(); ++i) {
+  r.UnionRect(r, mFillPaint.mResultNeededBox);
+  r.UnionRect(r, mStrokePaint.mResultNeededBox);
+  for (uint32_t i = 0; i < mPrimitives.Length(); ++i) {
     r.UnionRect(r, mPrimitives[i].mResultNeededBox);
   }
   return r;
 }
 
+nsresult
+nsSVGFilterInstance::BuildSourcePaint(PrimitiveInfo *aPrimitive)
+{
+  NS_ASSERTION(aPrimitive->mImageUsers > 0, "Some user must have needed this");
+
+  nsRefPtr<gfxImageSurface> image = CreateImage();
+  if (!image)
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  nsRefPtr<gfxASurface> offscreen =
+    gfxPlatform::GetPlatform()->CreateOffscreenSurface(
+            gfxIntSize(mSurfaceRect.width, mSurfaceRect.height),
+            gfxASurface::CONTENT_COLOR_ALPHA);
+  if (!offscreen || offscreen->CairoStatus())
+    return NS_ERROR_OUT_OF_MEMORY;
+  offscreen->SetDeviceOffset(gfxPoint(-mSurfaceRect.x, -mSurfaceRect.y));
+
+  nsRenderingContext tmpCtx;
+  tmpCtx.Init(mTargetFrame->PresContext()->DeviceContext(), offscreen);
+
+  gfxRect r = aPrimitive->mImage.mFilterPrimitiveSubregion;
+  gfxMatrix m = GetUserSpaceToFilterSpaceTransform();
+  m.Invert();
+  r = m.TransformBounds(r);
+
+  gfxMatrix deviceToFilterSpace = GetFilterSpaceToDeviceSpaceTransform().Invert();
+  gfxContext *gfx = tmpCtx.ThebesContext();
+  gfx->Multiply(deviceToFilterSpace);
+
+  gfx->Save();
+
+  gfxMatrix matrix =
+    nsSVGUtils::GetCanvasTM(mTargetFrame, nsISVGChildFrame::FOR_PAINTING);
+  if (!matrix.IsSingular()) {
+    gfx->Multiply(matrix);
+    gfx->Rectangle(r);
+    if ((aPrimitive == &mFillPaint && 
+         nsSVGUtils::SetupCairoFillPaint(mTargetFrame, gfx)) ||
+        (aPrimitive == &mStrokePaint &&
+         nsSVGUtils::SetupCairoStrokePaint(mTargetFrame, gfx))) {
+      gfx->Fill();
+    }
+  }
+  gfx->Restore();
+
+  gfxContext copyContext(image);
+  copyContext.SetSource(offscreen);
+  copyContext.Paint();
+
+  aPrimitive->mImage.mImage = image;
+  // color model is PREMULTIPLIED SRGB by default.
+
+  return NS_OK;
+}
+
+nsresult
+nsSVGFilterInstance::BuildSourcePaints()
+{
+  nsresult rv = NS_OK;
+
+  if (!mFillPaint.mResultNeededBox.IsEmpty()) {
+    rv = BuildSourcePaint(&mFillPaint);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+
+  if (!mStrokePaint.mResultNeededBox.IsEmpty()) {
+    rv = BuildSourcePaint(&mStrokePaint);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+  return  rv;
+}
+
 nsresult
 nsSVGFilterInstance::BuildSourceImages()
 {
@@ -369,10 +419,9 @@ nsSVGFilterInstance::BuildSourceImages()
 
     nsRenderingContext tmpCtx;
     tmpCtx.Init(mTargetFrame->PresContext()->DeviceContext(), offscreen);
-    gfxMatrix userSpaceToFilterSpace = GetUserSpaceToFilterSpaceTransform();
 
     gfxRect r(neededRect.x, neededRect.y, neededRect.width, neededRect.height);
-    gfxMatrix m = userSpaceToFilterSpace;
+    gfxMatrix m = GetUserSpaceToFilterSpaceTransform();
     m.Invert();
     r = m.TransformBounds(r);
     r.RoundOut();
@@ -415,11 +464,11 @@ nsSVGFilterInstance::BuildSourceImages()
     // color model is PREMULTIPLIED SRGB by default.
 
     // Clear the color channel
-    const PRUint32* src = reinterpret_cast<PRUint32*>(sourceColorAlpha->Data());
-    PRUint32* dest = reinterpret_cast<PRUint32*>(mSourceAlpha.mImage.mImage->Data());
-    for (PRInt32 y = 0; y < mSurfaceRect.height; y++) {
-      PRUint32 rowOffset = (mSourceAlpha.mImage.mImage->Stride()*y) >> 2;
-      for (PRInt32 x = 0; x < mSurfaceRect.width; x++) {
+    const uint32_t* src = reinterpret_cast<uint32_t*>(sourceColorAlpha->Data());
+    uint32_t* dest = reinterpret_cast<uint32_t*>(mSourceAlpha.mImage.mImage->Data());
+    for (int32_t y = 0; y < mSurfaceRect.height; y++) {
+      uint32_t rowOffset = (mSourceAlpha.mImage.mImage->Stride()*y) >> 2;
+      for (int32_t x = 0; x < mSurfaceRect.width; x++) {
         dest[rowOffset + x] = src[rowOffset + x] & 0xFF000000U;
       }
     }
@@ -437,8 +486,8 @@ nsSVGFilterInstance::EnsureColorModel(PrimitiveInfo* aPrimitive,
   if (aColorModel == currentModel)
     return;
 
-  PRUint8* data = aPrimitive->mImage.mImage->Data();
-  PRInt32 stride = aPrimitive->mImage.mImage->Stride();
+  uint8_t* data = aPrimitive->mImage.mImage->Data();
+  int32_t stride = aPrimitive->mImage.mImage->Stride();
 
   nsIntRect r = aPrimitive->mResultNeededBox - mSurfaceRect.TopLeft();
 
@@ -461,7 +510,7 @@ nsSVGFilterInstance::EnsureColorModel(PrimitiveInfo* aPrimitive,
 nsresult
 nsSVGFilterInstance::Render(gfxASurface** aOutput)
 {
-  *aOutput = nsnull;
+  *aOutput = nullptr;
 
   nsresult rv = BuildSources();
   if (NS_FAILED(rv))
@@ -483,10 +532,13 @@ nsSVGFilterInstance::Render(gfxASurface** aOutput)
   mSurfaceRect = ComputeUnionOfAllNeededBoxes();
 
   rv = BuildSourceImages();
+  if (NS_FAILED(rv))
+    return rv;
+  rv = BuildSourcePaints();
   if (NS_FAILED(rv))
     return rv;
 
-  for (PRUint32 i = 0; i < mPrimitives.Length(); ++i) {
+  for (uint32_t i = 0; i < mPrimitives.Length(); ++i) {
     PrimitiveInfo* primitive = &mPrimitives[i];
 
     nsIntRect dataRect;
@@ -501,7 +553,7 @@ nsSVGFilterInstance::Render(gfxASurface** aOutput)
       return NS_ERROR_OUT_OF_MEMORY;
 
     nsAutoTArray<const Image*,2> inputs;
-    for (PRUint32 j = 0; j < primitive->mInputs.Length(); ++j) {
+    for (uint32_t j = 0; j < primitive->mInputs.Length(); ++j) {
       PrimitiveInfo* input = primitive->mInputs[j];
       
       if (!input->mImage.mImage) {
@@ -532,13 +584,13 @@ nsSVGFilterInstance::Render(gfxASurface** aOutput)
     if (NS_FAILED(rv))
       return rv;
 
-    for (PRUint32 j = 0; j < primitive->mInputs.Length(); ++j) {
+    for (uint32_t j = 0; j < primitive->mInputs.Length(); ++j) {
       PrimitiveInfo* input = primitive->mInputs[j];
       --input->mImageUsers;
       NS_ASSERTION(input->mImageUsers >= 0, "Bad mImageUsers tracking");
       if (input->mImageUsers == 0) {
         // Release the image, it's no longer needed
-        input->mImage.mImage = nsnull;
+        input->mImage.mImage = nullptr;
       }
     }
   }
@@ -546,16 +598,16 @@ nsSVGFilterInstance::Render(gfxASurface** aOutput)
   PrimitiveInfo* result = &mPrimitives[mPrimitives.Length() - 1];
   ColorModel premulSRGB; // default
   EnsureColorModel(result, premulSRGB);
-  gfxImageSurface* surf = nsnull;
+  gfxImageSurface* surf = nullptr;
   result->mImage.mImage.swap(surf);
   *aOutput = surf;
   return NS_OK;
 }
 
 nsresult
-nsSVGFilterInstance::ComputeOutputDirtyRect(nsIntRect* aDirty)
+nsSVGFilterInstance::ComputePostFilterDirtyRect(nsIntRect* aPostFilterDirtyRect)
 {
-  *aDirty = nsIntRect();
+  *aPostFilterDirtyRect = nsIntRect();
 
   nsresult rv = BuildSources();
   if (NS_FAILED(rv))
@@ -572,12 +624,12 @@ nsSVGFilterInstance::ComputeOutputDirtyRect(nsIntRect* aDirty)
 
   ComputeResultBoundingBoxes();
 
-  mSourceColorAlpha.mResultChangeBox = mDirtyInputRect;
-  mSourceAlpha.mResultChangeBox = mDirtyInputRect;
+  mSourceColorAlpha.mResultChangeBox = mPreFilterDirtyRect;
+  mSourceAlpha.mResultChangeBox = mPreFilterDirtyRect;
   ComputeResultChangeBoxes();
 
   PrimitiveInfo* result = &mPrimitives[mPrimitives.Length() - 1];
-  *aDirty = result->mResultChangeBox;
+  *aPostFilterDirtyRect = result->mResultChangeBox;
   return NS_OK;
 }
 
