@@ -1,41 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is the Mozilla SVG project.
- *
- * The Initial Developer of the Original Code is
- * Scooter Morris.
- * Portions created by the Initial Developer are Copyright (C) 2004
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Scooter Morris <scootermorris@comcast.net>
- *   Jonathan Watt <jwatt@jwatt.org>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 // Main header first:
 #include "nsSVGGradientFrame.h"
@@ -90,27 +56,27 @@ NS_IMPL_FRAMEARENA_HELPERS(nsSVGGradientFrame)
 /* virtual */ void
 nsSVGGradientFrame::DidSetStyleContext(nsStyleContext* aOldStyleContext)
 {
-  nsSVGEffects::InvalidateRenderingObservers(this);
+  nsSVGEffects::InvalidateDirectRenderingObservers(this);
   nsSVGGradientFrameBase::DidSetStyleContext(aOldStyleContext);
 }
 
 NS_IMETHODIMP
-nsSVGGradientFrame::AttributeChanged(PRInt32         aNameSpaceID,
+nsSVGGradientFrame::AttributeChanged(int32_t         aNameSpaceID,
                                      nsIAtom*        aAttribute,
-                                     PRInt32         aModType)
+                                     int32_t         aModType)
 {
   if (aNameSpaceID == kNameSpaceID_None &&
       (aAttribute == nsGkAtoms::gradientUnits ||
        aAttribute == nsGkAtoms::gradientTransform ||
        aAttribute == nsGkAtoms::spreadMethod)) {
-    nsSVGEffects::InvalidateRenderingObservers(this);
+    nsSVGEffects::InvalidateDirectRenderingObservers(this);
   } else if (aNameSpaceID == kNameSpaceID_XLink &&
              aAttribute == nsGkAtoms::href) {
     // Blow away our reference, if any
     Properties().Delete(nsSVGEffects::HrefProperty());
     mNoHRefURI = false;
     // And update whoever references us
-    nsSVGEffects::InvalidateRenderingObservers(this);
+    nsSVGEffects::InvalidateDirectRenderingObservers(this);
   }
 
   return nsSVGGradientFrameBase::AttributeChanged(aNameSpaceID,
@@ -119,14 +85,14 @@ nsSVGGradientFrame::AttributeChanged(PRInt32         aNameSpaceID,
 
 //----------------------------------------------------------------------
 
-PRUint32
+uint32_t
 nsSVGGradientFrame::GetStopCount()
 {
-  return GetStopFrame(-1, nsnull);
+  return GetStopFrame(-1, nullptr);
 }
 
 void
-nsSVGGradientFrame::GetStopInformation(PRInt32 aIndex,
+nsSVGGradientFrame::GetStopInformation(int32_t aIndex,
                                        float *aOffset,
                                        nscolor *aStopColor,
                                        float *aStopOpacity)
@@ -135,7 +101,7 @@ nsSVGGradientFrame::GetStopInformation(PRInt32 aIndex,
   *aStopColor = NS_RGBA(0, 0, 0, 0);
   *aStopOpacity = 1.0f;
 
-  nsIFrame *stopFrame = nsnull;
+  nsIFrame *stopFrame = nullptr;
   GetStopFrame(aIndex, &stopFrame);
   nsCOMPtr<nsIDOMSVGStopElement> stopElement =
     do_QueryInterface(stopFrame->GetContent());
@@ -155,8 +121,8 @@ nsSVGGradientFrame::GetStopInformation(PRInt32 aIndex,
   *aStopOpacity = stopFrame->GetStyleSVGReset()->mStopOpacity;
 }
 
-PRUint16
-nsSVGGradientFrame::GetEnumValue(PRUint32 aIndex, nsIContent *aDefault)
+uint16_t
+nsSVGGradientFrame::GetEnumValue(uint32_t aIndex, nsIContent *aDefault)
 {
   const nsSVGEnum& thisEnum =
     static_cast<nsSVGGradientElement *>(mContent)->mEnumAttributes[aIndex];
@@ -172,14 +138,14 @@ nsSVGGradientFrame::GetEnumValue(PRUint32 aIndex, nsIContent *aDefault)
       mEnumAttributes[aIndex].GetAnimValue();
 }
 
-PRUint16
+uint16_t
 nsSVGGradientFrame::GetGradientUnits()
 {
   // This getter is called every time the others are called - maybe cache it?
   return GetEnumValue(nsSVGGradientElement::GRADIENTUNITS);
 }
 
-PRUint16
+uint16_t
 nsSVGGradientFrame::GetSpreadMethod()
 {
   return GetEnumValue(nsSVGGradientElement::SPREADMETHOD);
@@ -191,7 +157,7 @@ nsSVGGradientFrame::GetGradientTransformList(nsIContent* aDefault)
   SVGAnimatedTransformList *thisTransformList =
     static_cast<nsSVGGradientElement *>(mContent)->GetAnimatedTransformList();
 
-  if (thisTransformList->IsExplicitlySet())
+  if (thisTransformList && thisTransformList->IsExplicitlySet())
     return thisTransformList;
 
   AutoGradientReferencer gradientRef(this);
@@ -208,7 +174,7 @@ nsSVGGradientFrame::GetGradientTransform(nsIFrame *aSource,
 {
   gfxMatrix bboxMatrix;
 
-  PRUint16 gradientUnits = GetGradientUnits();
+  uint16_t gradientUnits = GetGradientUnits();
   if (gradientUnits == nsIDOMSVGUnitTypes::SVG_UNIT_TYPE_USERSPACEONUSE) {
     // If this gradient is applied to text, our caller
     // will be the glyph, which is not a container, so we
@@ -240,7 +206,7 @@ nsSVGGradientFrame::GetGradientTransform(nsIFrame *aSource,
 }
 
 nsSVGLinearGradientElement *
-nsSVGGradientFrame::GetLinearGradientWithLength(PRUint32 aIndex,
+nsSVGGradientFrame::GetLinearGradientWithLength(uint32_t aIndex,
   nsSVGLinearGradientElement* aDefault)
 {
   // If this was a linear gradient with the required length, we would have
@@ -254,7 +220,7 @@ nsSVGGradientFrame::GetLinearGradientWithLength(PRUint32 aIndex,
 }
 
 nsSVGRadialGradientElement *
-nsSVGGradientFrame::GetRadialGradientWithLength(PRUint32 aIndex,
+nsSVGGradientFrame::GetRadialGradientWithLength(uint32_t aIndex,
   nsSVGRadialGradientElement* aDefault)
 {
   // If this was a radial gradient with the required length, we would have
@@ -272,16 +238,18 @@ nsSVGGradientFrame::GetRadialGradientWithLength(PRUint32 aIndex,
 
 already_AddRefed<gfxPattern>
 nsSVGGradientFrame::GetPaintServerPattern(nsIFrame *aSource,
-                                           float aGraphicOpacity,
-                                           const gfxRect *aOverrideBounds)
+                                          const gfxMatrix& aContextMatrix,
+                                          nsStyleSVGPaint nsStyleSVG::*aFillOrStroke,
+                                          float aGraphicOpacity,
+                                          const gfxRect *aOverrideBounds)
 {
   // Get the transform list (if there is one)
   gfxMatrix patternMatrix = GetGradientTransform(aSource, aOverrideBounds);
 
   if (patternMatrix.IsSingular())
-    return nsnull;
+    return nullptr;
 
-  PRUint32 nStops = GetStopCount();
+  uint32_t nStops = GetStopCount();
 
   // SVG specification says that no stops should be treated like
   // the corresponding fill or stroke had "none" specified.
@@ -290,13 +258,18 @@ nsSVGGradientFrame::GetPaintServerPattern(nsIFrame *aSource,
     return pattern.forget();
   }
 
+  // revert the vector effect transform so that the gradient appears unchanged
+  if (aFillOrStroke == &nsStyleSVG::mStroke) {
+    patternMatrix.Multiply(nsSVGUtils::GetStrokeTransform(aSource).Invert());
+  }
+
   patternMatrix.Invert();
 
   nsRefPtr<gfxPattern> gradient = CreateGradient();
   if (!gradient || gradient->CairoStatus())
-    return nsnull;
+    return nullptr;
 
-  PRUint16 aSpread = GetSpreadMethod();
+  uint16_t aSpread = GetSpreadMethod();
   if (aSpread == nsIDOMSVGGradientElement::SVG_SPREADMETHOD_PAD)
     gradient->SetExtend(gfxPattern::EXTEND_PAD);
   else if (aSpread == nsIDOMSVGGradientElement::SVG_SPREADMETHOD_REFLECT)
@@ -309,7 +282,7 @@ nsSVGGradientFrame::GetPaintServerPattern(nsIFrame *aSource,
   // setup stops
   float lastOffset = 0.0f;
 
-  for (PRUint32 i = 0; i < nStops; i++) {
+  for (uint32_t i = 0; i < nStops; i++) {
     float offset, stopOpacity;
     nscolor stopColor;
 
@@ -337,7 +310,7 @@ nsSVGGradientFrame *
 nsSVGGradientFrame::GetReferencedGradient()
 {
   if (mNoHRefURI)
-    return nsnull;
+    return nullptr;
 
   nsSVGPaintingProperty *property = static_cast<nsSVGPaintingProperty*>
     (Properties().Get(nsSVGEffects::HrefProperty()));
@@ -349,7 +322,7 @@ nsSVGGradientFrame::GetReferencedGradient()
     grad->mStringAttributes[nsSVGGradientElement::HREF].GetAnimValue(href, grad);
     if (href.IsEmpty()) {
       mNoHRefURI = true;
-      return nsnull; // no URL
+      return nullptr; // no URL
     }
 
     // Convert href to an nsIURI
@@ -361,17 +334,17 @@ nsSVGGradientFrame::GetReferencedGradient()
     property =
       nsSVGEffects::GetPaintingProperty(targetURI, this, nsSVGEffects::HrefProperty());
     if (!property)
-      return nsnull;
+      return nullptr;
   }
 
   nsIFrame *result = property->GetReferencedFrame();
   if (!result)
-    return nsnull;
+    return nullptr;
 
   nsIAtom* frameType = result->GetType();
   if (frameType != nsGkAtoms::svgLinearGradientFrame &&
       frameType != nsGkAtoms::svgRadialGradientFrame)
-    return nsnull;
+    return nullptr;
 
   return static_cast<nsSVGGradientFrame*>(result);
 }
@@ -381,22 +354,22 @@ nsSVGGradientFrame::GetReferencedGradientIfNotInUse()
 {
   nsSVGGradientFrame *referenced = GetReferencedGradient();
   if (!referenced)
-    return nsnull;
+    return nullptr;
 
   if (referenced->mLoopFlag) {
     // XXXjwatt: we should really send an error to the JavaScript Console here:
     NS_WARNING("gradient reference loop detected while inheriting attribute!");
-    return nsnull;
+    return nullptr;
   }
 
   return referenced;
 }
 
-PRInt32
-nsSVGGradientFrame::GetStopFrame(PRInt32 aIndex, nsIFrame * *aStopFrame)
+int32_t
+nsSVGGradientFrame::GetStopFrame(int32_t aIndex, nsIFrame * *aStopFrame)
 {
-  PRInt32 stopCount = 0;
-  nsIFrame *stopFrame = nsnull;
+  int32_t stopCount = 0;
+  nsIFrame *stopFrame = nullptr;
   for (stopFrame = mFrames.FirstChild(); stopFrame;
        stopFrame = stopFrame->GetNextSibling()) {
     if (stopFrame->GetType() == nsGkAtoms::svgStopFrame) {
@@ -416,7 +389,7 @@ nsSVGGradientFrame::GetStopFrame(PRInt32 aIndex, nsIFrame * *aStopFrame)
   AutoGradientReferencer gradientRef(this);
   nsSVGGradientFrame* next = GetReferencedGradientIfNotInUse();
   if (!next)
-    return nsnull;
+    return 0;
 
   return next->GetStopFrame(aIndex, aStopFrame);
 }
@@ -445,16 +418,16 @@ nsSVGLinearGradientFrame::GetType() const
 }
 
 NS_IMETHODIMP
-nsSVGLinearGradientFrame::AttributeChanged(PRInt32         aNameSpaceID,
+nsSVGLinearGradientFrame::AttributeChanged(int32_t         aNameSpaceID,
                                            nsIAtom*        aAttribute,
-                                           PRInt32         aModType)
+                                           int32_t         aModType)
 {
   if (aNameSpaceID == kNameSpaceID_None &&
       (aAttribute == nsGkAtoms::x1 ||
        aAttribute == nsGkAtoms::y1 ||
        aAttribute == nsGkAtoms::x2 ||
        aAttribute == nsGkAtoms::y2)) {
-    nsSVGEffects::InvalidateRenderingObservers(this);
+    nsSVGEffects::InvalidateDirectRenderingObservers(this);
   }
 
   return nsSVGGradientFrame::AttributeChanged(aNameSpaceID,
@@ -464,7 +437,7 @@ nsSVGLinearGradientFrame::AttributeChanged(PRInt32         aNameSpaceID,
 //----------------------------------------------------------------------
 
 float
-nsSVGLinearGradientFrame::GetLengthValue(PRUint32 aIndex)
+nsSVGLinearGradientFrame::GetLengthValue(uint32_t aIndex)
 {
   nsSVGLinearGradientElement* lengthElement =
     GetLinearGradientWithLength(aIndex,
@@ -479,7 +452,7 @@ nsSVGLinearGradientFrame::GetLengthValue(PRUint32 aIndex)
   // transform in GetGradientTransform, but we need to handle user
   // space units as part of the individual Get* routines.  Fixes 323669.
 
-  PRUint16 gradientUnits = GetGradientUnits();
+  uint16_t gradientUnits = GetGradientUnits();
   if (gradientUnits == nsIDOMSVGUnitTypes::SVG_UNIT_TYPE_USERSPACEONUSE) {
     return nsSVGUtils::UserSpace(mSource, &length);
   }
@@ -488,11 +461,11 @@ nsSVGLinearGradientFrame::GetLengthValue(PRUint32 aIndex)
     gradientUnits == nsIDOMSVGUnitTypes::SVG_UNIT_TYPE_OBJECTBOUNDINGBOX,
     "Unknown gradientUnits type");
 
-  return length.GetAnimValue(static_cast<nsSVGSVGElement*>(nsnull));
+  return length.GetAnimValue(static_cast<nsSVGSVGElement*>(nullptr));
 }
 
 nsSVGLinearGradientElement *
-nsSVGLinearGradientFrame::GetLinearGradientWithLength(PRUint32 aIndex,
+nsSVGLinearGradientFrame::GetLinearGradientWithLength(uint32_t aIndex,
   nsSVGLinearGradientElement* aDefault)
 {
   nsSVGLinearGradientElement* thisElement =
@@ -546,9 +519,9 @@ nsSVGRadialGradientFrame::GetType() const
 }
 
 NS_IMETHODIMP
-nsSVGRadialGradientFrame::AttributeChanged(PRInt32         aNameSpaceID,
+nsSVGRadialGradientFrame::AttributeChanged(int32_t         aNameSpaceID,
                                            nsIAtom*        aAttribute,
-                                           PRInt32         aModType)
+                                           int32_t         aModType)
 {
   if (aNameSpaceID == kNameSpaceID_None &&
       (aAttribute == nsGkAtoms::r ||
@@ -556,7 +529,7 @@ nsSVGRadialGradientFrame::AttributeChanged(PRInt32         aNameSpaceID,
        aAttribute == nsGkAtoms::cy ||
        aAttribute == nsGkAtoms::fx ||
        aAttribute == nsGkAtoms::fy)) {
-    nsSVGEffects::InvalidateRenderingObservers(this);
+    nsSVGEffects::InvalidateDirectRenderingObservers(this);
   }
 
   return nsSVGGradientFrame::AttributeChanged(aNameSpaceID,
@@ -566,7 +539,7 @@ nsSVGRadialGradientFrame::AttributeChanged(PRInt32         aNameSpaceID,
 //----------------------------------------------------------------------
 
 float
-nsSVGRadialGradientFrame::GetLengthValue(PRUint32 aIndex)
+nsSVGRadialGradientFrame::GetLengthValue(uint32_t aIndex)
 {
   nsSVGRadialGradientElement* lengthElement =
     GetRadialGradientWithLength(aIndex,
@@ -579,17 +552,17 @@ nsSVGRadialGradientFrame::GetLengthValue(PRUint32 aIndex)
 }
 
 float
-nsSVGRadialGradientFrame::GetLengthValue(PRUint32 aIndex, float aDefaultValue)
+nsSVGRadialGradientFrame::GetLengthValue(uint32_t aIndex, float aDefaultValue)
 {
   nsSVGRadialGradientElement* lengthElement =
-    GetRadialGradientWithLength(aIndex, nsnull);
+    GetRadialGradientWithLength(aIndex, nullptr);
 
   return lengthElement ? GetLengthValueFromElement(aIndex, *lengthElement)
                        : aDefaultValue;
 }
 
 float
-nsSVGRadialGradientFrame::GetLengthValueFromElement(PRUint32 aIndex,
+nsSVGRadialGradientFrame::GetLengthValueFromElement(uint32_t aIndex,
   nsSVGRadialGradientElement& aElement)
 {
   const nsSVGLength2 &length = aElement.mLengthAttributes[aIndex];
@@ -598,7 +571,7 @@ nsSVGRadialGradientFrame::GetLengthValueFromElement(PRUint32 aIndex,
   // transform in GetGradientTransform, but we need to handle user
   // space units as part of the individual Get* routines.  Fixes 323669.
 
-  PRUint16 gradientUnits = GetGradientUnits();
+  uint16_t gradientUnits = GetGradientUnits();
   if (gradientUnits == nsIDOMSVGUnitTypes::SVG_UNIT_TYPE_USERSPACEONUSE) {
     return nsSVGUtils::UserSpace(mSource, &length);
   }
@@ -607,11 +580,11 @@ nsSVGRadialGradientFrame::GetLengthValueFromElement(PRUint32 aIndex,
     gradientUnits == nsIDOMSVGUnitTypes::SVG_UNIT_TYPE_OBJECTBOUNDINGBOX,
     "Unknown gradientUnits type");
 
-  return length.GetAnimValue(static_cast<nsSVGSVGElement*>(nsnull));
+  return length.GetAnimValue(static_cast<nsSVGSVGElement*>(nullptr));
 }
 
 nsSVGRadialGradientElement *
-nsSVGRadialGradientFrame::GetRadialGradientWithLength(PRUint32 aIndex,
+nsSVGRadialGradientFrame::GetRadialGradientWithLength(uint32_t aIndex,
   nsSVGRadialGradientElement* aDefault)
 {
   nsSVGRadialGradientElement* thisElement =
