@@ -1,40 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is the Mozilla SVG project.
- *
- * The Initial Developer of the Original Code is
- * Scooter Morris.
- * Portions created by the Initial Developer are Copyright (C) 2004
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Scooter Morris <scootermorris@comcast.net>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifndef __NS_SVGGRADIENTFRAME_H__
 #define __NS_SVGGRADIENTFRAME_H__
@@ -77,15 +44,17 @@ public:
   // nsSVGPaintServerFrame methods:
   virtual already_AddRefed<gfxPattern>
     GetPaintServerPattern(nsIFrame *aSource,
+                          const gfxMatrix& aContextMatrix,
+                          nsStyleSVGPaint nsStyleSVG::*aFillOrStroke,
                           float aGraphicOpacity,
                           const gfxRect *aOverrideBounds);
 
   // nsIFrame interface:
   virtual void DidSetStyleContext(nsStyleContext* aOldStyleContext);
 
-  NS_IMETHOD AttributeChanged(PRInt32         aNameSpaceID,
+  NS_IMETHOD AttributeChanged(int32_t         aNameSpaceID,
                               nsIAtom*        aAttribute,
-                              PRInt32         aModType);
+                              int32_t         aModType);
 
 #ifdef DEBUG
   NS_IMETHOD GetFrameName(nsAString& aResult) const
@@ -102,10 +71,10 @@ private:
   nsSVGGradientFrame* GetReferencedGradient();
 
   // Optionally get a stop frame (returns stop index/count)
-  PRInt32 GetStopFrame(PRInt32 aIndex, nsIFrame * *aStopFrame);
+  int32_t GetStopFrame(int32_t aIndex, nsIFrame * *aStopFrame);
 
-  PRUint32 GetStopCount();
-  void GetStopInformation(PRInt32 aIndex,
+  uint32_t GetStopCount();
+  void GetStopInformation(int32_t aIndex,
                           float *aOffset, nscolor *aColor, float *aStopOpacity);
 
   const mozilla::SVGAnimatedTransformList* GetGradientTransformList(
@@ -122,20 +91,20 @@ protected:
   nsSVGGradientFrame* GetReferencedGradientIfNotInUse();
 
   // Accessors to lookup gradient attributes
-  PRUint16 GetEnumValue(PRUint32 aIndex, nsIContent *aDefault);
-  PRUint16 GetEnumValue(PRUint32 aIndex)
+  uint16_t GetEnumValue(uint32_t aIndex, nsIContent *aDefault);
+  uint16_t GetEnumValue(uint32_t aIndex)
   {
     return GetEnumValue(aIndex, mContent);
   }
-  PRUint16 GetGradientUnits();
-  PRUint16 GetSpreadMethod();
+  uint16_t GetGradientUnits();
+  uint16_t GetSpreadMethod();
 
   // Gradient-type-specific lookups since the length values differ between
   // linear and radial gradients
   virtual nsSVGLinearGradientElement * GetLinearGradientWithLength(
-    PRUint32 aIndex, nsSVGLinearGradientElement* aDefault);
+    uint32_t aIndex, nsSVGLinearGradientElement* aDefault);
   virtual nsSVGRadialGradientElement * GetRadialGradientWithLength(
-    PRUint32 aIndex, nsSVGRadialGradientElement* aDefault);
+    uint32_t aIndex, nsSVGRadialGradientElement* aDefault);
 
   // The frame our gradient is (currently) being applied to
   nsIFrame*                              mSource;
@@ -177,9 +146,9 @@ public:
 
   virtual nsIAtom* GetType() const;  // frame type: nsGkAtoms::svgLinearGradientFrame
 
-  NS_IMETHOD AttributeChanged(PRInt32         aNameSpaceID,
+  NS_IMETHOD AttributeChanged(int32_t         aNameSpaceID,
                               nsIAtom*        aAttribute,
-                              PRInt32         aModType);
+                              int32_t         aModType);
 
 #ifdef DEBUG
   NS_IMETHOD GetFrameName(nsAString& aResult) const
@@ -189,9 +158,9 @@ public:
 #endif // DEBUG
 
 protected:
-  float GetLengthValue(PRUint32 aIndex);
+  float GetLengthValue(uint32_t aIndex);
   virtual nsSVGLinearGradientElement * GetLinearGradientWithLength(
-    PRUint32 aIndex, nsSVGLinearGradientElement* aDefault);
+    uint32_t aIndex, nsSVGLinearGradientElement* aDefault);
   virtual already_AddRefed<gfxPattern> CreateGradient();
 };
 
@@ -221,9 +190,9 @@ public:
 
   virtual nsIAtom* GetType() const;  // frame type: nsGkAtoms::svgRadialGradientFrame
 
-  NS_IMETHOD AttributeChanged(PRInt32         aNameSpaceID,
+  NS_IMETHOD AttributeChanged(int32_t         aNameSpaceID,
                               nsIAtom*        aAttribute,
-                              PRInt32         aModType);
+                              int32_t         aModType);
 
 #ifdef DEBUG
   NS_IMETHOD GetFrameName(nsAString& aResult) const
@@ -233,12 +202,12 @@ public:
 #endif // DEBUG
 
 protected:
-  float GetLengthValue(PRUint32 aIndex);
-  float GetLengthValue(PRUint32 aIndex, float aDefaultValue);
-  float GetLengthValueFromElement(PRUint32 aIndex,
+  float GetLengthValue(uint32_t aIndex);
+  float GetLengthValue(uint32_t aIndex, float aDefaultValue);
+  float GetLengthValueFromElement(uint32_t aIndex,
                                   nsSVGRadialGradientElement& aElement);
   virtual nsSVGRadialGradientElement * GetRadialGradientWithLength(
-    PRUint32 aIndex, nsSVGRadialGradientElement* aDefault);
+    uint32_t aIndex, nsSVGRadialGradientElement* aDefault);
   virtual already_AddRefed<gfxPattern> CreateGradient();
 };
 
