@@ -1,43 +1,7 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Original Author: David W. Hyatt (hyatt@netscape.com)
- *   Michael Lowe <michael.lowe@bigfoot.com>
- *   Pierre Phaneuf <pp@ludusdesign.com>
- *   Dean Tessman <dean_tessman@hotmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsGkAtoms.h"
 #include "nsHTMLParts.h"
@@ -63,7 +27,6 @@
 #include "nsCSSFrameConstructor.h"
 #include "nsIDOMKeyEvent.h"
 #include "nsEventDispatcher.h"
-#include "nsIPrivateDOMEvent.h"
 #include "nsXPIDLString.h"
 #include "nsReadableUtils.h"
 #include "nsUnicharUtils.h"
@@ -78,6 +41,7 @@
 #include "mozilla/Services.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/LookAndFeel.h"
+#include "mozilla/Attributes.h"
 
 using namespace mozilla;
 
@@ -94,9 +58,9 @@ AssertNotCalled(void* aPropertyValue)
 }
 NS_DECLARE_FRAME_PROPERTY(PopupListProperty, AssertNotCalled)
 
-static PRInt32 gEatMouseMove = false;
+static int32_t gEatMouseMove = false;
 
-const PRInt32 kBlinkDelay = 67; // milliseconds
+const int32_t kBlinkDelay = 67; // milliseconds
 
 // this class is used for dispatching menu activation events asynchronously.
 class nsMenuActivateEvent : public nsRunnable
@@ -128,16 +92,15 @@ public:
     }
 
     nsCOMPtr<nsIDOMEvent> event;
-    if (NS_SUCCEEDED(nsEventDispatcher::CreateEvent(mPresContext, nsnull,
+    if (NS_SUCCEEDED(nsEventDispatcher::CreateEvent(mPresContext, nullptr,
                                                     NS_LITERAL_STRING("Events"),
                                                     getter_AddRefs(event)))) {
       event->InitEvent(domEventToFire, true, true);
 
-      nsCOMPtr<nsIPrivateDOMEvent> privateEvent(do_QueryInterface(event));
-      privateEvent->SetTrusted(true);
+      event->SetTrusted(true);
 
-      nsEventDispatcher::DispatchDOMEvent(mMenu, nsnull, event,
-                                          mPresContext, nsnull);
+      nsEventDispatcher::DispatchDOMEvent(mMenu, nullptr, event,
+                                          mPresContext, nullptr);
     }
 
     return NS_OK;
@@ -220,7 +183,7 @@ nsMenuFrame::nsMenuFrame(nsIPresShell* aShell, nsStyleContext* aContext):
     mChecked(false),
     mIgnoreAccelTextChange(false),
     mType(eMenuType_Normal),
-    mMenuParent(nsnull),
+    mMenuParent(nullptr),
     mBlinkState(0)
 {
 }
@@ -236,20 +199,23 @@ void
 nsMenuFrame::InitMenuParent(nsIFrame* aParent)
 {
   while (aParent) {
-    nsIAtom* type = aParent->GetType();
-    if (type == nsGkAtoms::menuPopupFrame) {
-      mMenuParent = static_cast<nsMenuPopupFrame *>(aParent);
+    nsMenuPopupFrame* popup = do_QueryFrame(aParent);
+    if (popup) {
+      mMenuParent = popup;
       break;
     }
-    else if (type == nsGkAtoms::menuBarFrame) {
-      mMenuParent = static_cast<nsMenuBarFrame *>(aParent);
+
+    nsMenuBarFrame* menubar = do_QueryFrame(aParent);
+    if (menubar) {
+      mMenuParent = menubar;
       break;
     }
+
     aParent = aParent->GetParent();
   }
 }
 
-class nsASyncMenuInitialization : public nsIReflowCallback
+class nsASyncMenuInitialization MOZ_FINAL : public nsIReflowCallback
 {
 public:
   nsASyncMenuInitialization(nsIFrame* aFrame)
@@ -260,12 +226,10 @@ public:
   virtual bool ReflowFinished()
   {
     bool shouldFlush = false;
-    if (mWeakFrame.IsAlive()) {
-      if (mWeakFrame.GetFrame()->GetType() == nsGkAtoms::menuFrame) {
-        nsMenuFrame* menu = static_cast<nsMenuFrame*>(mWeakFrame.GetFrame());
-        menu->UpdateMenuType(menu->PresContext());
-        shouldFlush = true;
-      }
+    nsMenuFrame* menu = do_QueryFrame(mWeakFrame.GetFrame());
+    if (menu) {
+      menu->UpdateMenuType(menu->PresContext());
+      shouldFlush = true;
     }
     delete this;
     return shouldFlush;
@@ -325,14 +289,14 @@ nsMenuFrame::GetPopup()
 {
   nsFrameList* popupList = GetPopupList();
   return popupList ? static_cast<nsMenuPopupFrame*>(popupList->FirstChild()) :
-                     nsnull;
+                     nullptr;
 }
 
 nsFrameList*
 nsMenuFrame::GetPopupList() const
 {
   if (!HasPopup()) {
-    return nsnull;
+    return nullptr;
   }
   nsFrameList* prop =
     static_cast<nsFrameList*>(Properties().Get(PopupListProperty()));
@@ -358,9 +322,9 @@ void
 nsMenuFrame::SetPopupFrame(nsFrameList& aFrameList)
 {
   for (nsFrameList::Enumerator e(aFrameList); !e.AtEnd(); e.Next()) {
-    if (e.get()->GetType() == nsGkAtoms::menuPopupFrame) {
+    nsMenuPopupFrame* popupFrame = do_QueryFrame(e.get());
+    if (popupFrame) {
       // Remove the frame from the list and store it in a nsFrameList* property.
-      nsIFrame* popupFrame = e.get();
       aFrameList.RemoveFrame(popupFrame);
       nsFrameList* popupList = new nsFrameList(popupFrame, popupFrame);
       Properties().Set(PopupListProperty(), popupList);
@@ -450,18 +414,18 @@ nsMenuFrame::HandleEvent(nsPresContext* aPresContext,
 
   if (aEvent->message == NS_KEY_PRESS && !IsDisabled()) {
     nsKeyEvent* keyEvent = (nsKeyEvent*)aEvent;
-    PRUint32 keyCode = keyEvent->keyCode;
+    uint32_t keyCode = keyEvent->keyCode;
 #ifdef XP_MACOSX
     // On mac, open menulist on either up/down arrow or space (w/o Cmd pressed)
-    if (!IsOpen() && ((keyEvent->charCode == NS_VK_SPACE && !keyEvent->isMeta) ||
+    if (!IsOpen() && ((keyEvent->charCode == NS_VK_SPACE && !keyEvent->IsMeta()) ||
         (keyCode == NS_VK_UP || keyCode == NS_VK_DOWN))) {
       *aEventStatus = nsEventStatus_eConsumeNoDefault;
       OpenMenu(false);
     }
 #else
     // On other platforms, toggle menulist on unmodified F4 or Alt arrow
-    if ((keyCode == NS_VK_F4 && !keyEvent->isAlt) ||
-        ((keyCode == NS_VK_UP || keyCode == NS_VK_DOWN) && keyEvent->isAlt)) {
+    if ((keyCode == NS_VK_F4 && !keyEvent->IsAlt()) ||
+        ((keyCode == NS_VK_UP || keyCode == NS_VK_DOWN) && keyEvent->IsAlt())) {
       *aEventStatus = nsEventStatus_eConsumeNoDefault;
       ToggleMenuState();
     }
@@ -521,7 +485,7 @@ nsMenuFrame::HandleEvent(nsPresContext* aPresContext,
     // Kill our timer if one is active.
     if (mOpenTimer) {
       mOpenTimer->Cancel();
-      mOpenTimer = nsnull;
+      mOpenTimer = nullptr;
     }
 
     // Deactivate the menu.
@@ -532,7 +496,7 @@ nsMenuFrame::HandleEvent(nsPresContext* aPresContext,
           // Submenus don't get closed up immediately.
         }
         else if (this == mMenuParent->GetCurrentMenuItem()) {
-          mMenuParent->ChangeMenuItem(nsnull, false);
+          mMenuParent->ChangeMenuItem(nullptr, false);
         }
       }
     }
@@ -562,7 +526,7 @@ nsMenuFrame::HandleEvent(nsPresContext* aPresContext,
     // past the menu. This conditional check ensures that only menus have this
     // behaviour
     if (!IsDisabled() && IsMenu() && !IsOpen() && !mOpenTimer && !mMenuParent->IsMenuBar()) {
-      PRInt32 menuDelay =
+      int32_t menuDelay =
         LookAndFeel::GetInt(LookAndFeel::eIntID_SubmenuDelay, 300); // ms
 
       // We're a menu, we're built, we're closed, and no timer has been kicked off.
@@ -640,16 +604,17 @@ nsMenuFrame::SelectMenu(bool aActivateFlag)
     if (aActivateFlag) {
       nsIFrame* parent = GetParent();
       while (parent) {
-        if (parent->GetType() == nsGkAtoms::menuPopupFrame) {
+        nsMenuPopupFrame* menupopup = do_QueryFrame(parent);
+        if (menupopup) {
           // a menu is always the direct parent of a menupopup
-          parent = parent->GetParent();
-          if (parent && parent->GetType() == nsGkAtoms::menuFrame) {
+          nsMenuFrame* menu = do_QueryFrame(menupopup->GetParent());
+          if (menu) {
             // a popup however is not necessarily the direct parent of a menu
-            nsIFrame* popupParent = parent->GetParent();
+            nsIFrame* popupParent = menu->GetParent();
             while (popupParent) {
-              if (popupParent->GetType() == nsGkAtoms::menuPopupFrame) {
-                nsMenuPopupFrame* popup = static_cast<nsMenuPopupFrame *>(popupParent);
-                popup->SetCurrentMenuItem(static_cast<nsMenuFrame *>(parent));
+              menupopup = do_QueryFrame(popupParent);
+              if (menupopup) {
+                menupopup->SetCurrentMenuItem(menu);
                 break;
               }
               popupParent = popupParent->GetParent();
@@ -675,9 +640,9 @@ nsMenuFrame::SelectMenu(bool aActivateFlag)
 }
 
 NS_IMETHODIMP
-nsMenuFrame::AttributeChanged(PRInt32 aNameSpaceID,
+nsMenuFrame::AttributeChanged(int32_t aNameSpaceID,
                               nsIAtom* aAttribute,
-                              PRInt32 aModType)
+                              int32_t aModType)
 {
   if (aAttribute == nsGkAtoms::acceltext && mIgnoreAccelTextChange) {
     // Reset the flag so that only one change is ignored.
@@ -827,7 +792,7 @@ nsMenuFrame::Enter(nsGUIEvent *aEvent)
     }
 #endif   // #ifdef XP_WIN
     // this menu item was disabled - exit
-    return nsnull;
+    return nullptr;
   }
 
   if (!IsOpen()) {
@@ -838,7 +803,7 @@ nsMenuFrame::Enter(nsGUIEvent *aEvent)
       return this;
   }
 
-  return nsnull;
+  return nullptr;
 }
 
 bool
@@ -877,14 +842,14 @@ nsMenuFrame::Notify(nsITimer* aTimer)
 {
   // Our timer has fired.
   if (aTimer == mOpenTimer.get()) {
-    mOpenTimer = nsnull;
+    mOpenTimer = nullptr;
 
     if (!IsOpen() && mMenuParent) {
       // make sure we didn't open a context menu in the meantime
       // (i.e. the user right-clicked while hovering over a submenu).
       nsXULPopupManager* pm = nsXULPopupManager::GetInstance();
       if (pm) {
-        if ((!pm->HasContextMenu(nsnull) || mMenuParent->IsContextMenu()) &&
+        if ((!pm->HasContextMenu(nullptr) || mMenuParent->IsContextMenu()) &&
             mContent->AttrValueIs(kNameSpaceID_None, nsGkAtoms::menuactive,
                                   nsGkAtoms::_true, eCaseMatters)) {
           OpenMenu(false);
@@ -932,7 +897,7 @@ void
 nsMenuFrame::UpdateMenuType(nsPresContext* aPresContext)
 {
   static nsIContent::AttrValuesArray strings[] =
-    {&nsGkAtoms::checkbox, &nsGkAtoms::radio, nsnull};
+    {&nsGkAtoms::checkbox, &nsGkAtoms::radio, nullptr};
   switch (mContent->FindAttrValueIn(kNameSpaceID_None, nsGkAtoms::type,
                                     strings, eCaseMatters)) {
     case 0: mType = eMenuType_Checkbox; break;
@@ -1003,11 +968,10 @@ nsMenuFrame::UpdateMenuSpecialState(nsPresContext* aPresContext)
   nsIFrame* sib = GetParent()->GetFirstPrincipalChild();
 
   while (sib) {
-    if (sib != this && sib->GetType() == nsGkAtoms::menuFrame) {
-      nsMenuFrame* menu = static_cast<nsMenuFrame*>(sib);
-      if (menu->GetMenuType() == eMenuType_Radio &&
-          menu->IsChecked() &&
-          (menu->GetRadioGroupName() == mGroupName)) {      
+    if (sib != this) {
+      nsMenuFrame* menu = do_QueryFrame(sib);
+      if (menu && menu->GetMenuType() == eMenuType_Radio &&
+          menu->IsChecked() && menu->GetRadioGroupName() == mGroupName) {
         /* uncheck the old item */
         sib->GetContent()->UnsetAttr(kNameSpaceID_None, nsGkAtoms::checked,
                                      true);
@@ -1104,7 +1068,7 @@ nsMenuFrame::BuildAcceleratorText(bool aNotify)
     }
   }
 
-  static PRInt32 accelKey = 0;
+  static int32_t accelKey = 0;
 
   if (!accelKey)
   {
@@ -1131,12 +1095,14 @@ nsMenuFrame::BuildAcceleratorText(bool aNotify)
   nsAutoString altText;
   nsAutoString metaText;
   nsAutoString controlText;
+  nsAutoString osText;
   nsAutoString modifierSeparator;
 
   nsContentUtils::GetShiftText(shiftText);
   nsContentUtils::GetAltText(altText);
   nsContentUtils::GetMetaText(metaText);
   nsContentUtils::GetControlText(controlText);
+  nsContentUtils::GetOSText(osText);
   nsContentUtils::GetModifierSeparatorText(modifierSeparator);
 
   while (token) {
@@ -1147,6 +1113,8 @@ nsMenuFrame::BuildAcceleratorText(bool aNotify)
       accelText += altText; 
     else if (PL_strcmp(token, "meta") == 0) 
       accelText += metaText; 
+    else if (PL_strcmp(token, "os") == 0)
+      accelText += osText; 
     else if (PL_strcmp(token, "control") == 0) 
       accelText += controlText; 
     else if (PL_strcmp(token, "accel") == 0) {
@@ -1156,6 +1124,10 @@ nsMenuFrame::BuildAcceleratorText(bool aNotify)
           accelText += metaText;
           break;
 
+        case nsIDOMKeyEvent::DOM_VK_WIN:
+          accelText += osText;
+          break;
+
         case nsIDOMKeyEvent::DOM_VK_ALT:
           accelText += altText;
           break;
@@ -1203,7 +1175,7 @@ nsMenuFrame::Execute(nsGUIEvent *aEvent)
 bool
 nsMenuFrame::ShouldBlink()
 {
-  PRInt32 shouldBlink =
+  int32_t shouldBlink =
     LookAndFeel::GetInt(LookAndFeel::eIntID_ChosenMenuItemsShouldBlink, 0);
   if (!shouldBlink)
     return false;
@@ -1249,9 +1221,9 @@ nsMenuFrame::StopBlinking()
   mBlinkState = 0;
   if (mBlinkTimer) {
     mBlinkTimer->Cancel();
-    mBlinkTimer = nsnull;
+    mBlinkTimer = nullptr;
   }
-  mDelayedMenuCommandEvent = nsnull;
+  mDelayedMenuCommandEvent = nullptr;
 }
 
 void
@@ -1266,10 +1238,10 @@ nsMenuFrame::CreateMenuCommandEvent(nsGUIEvent *aEvent, bool aFlipChecked)
   bool shift = false, control = false, alt = false, meta = false;
   if (aEvent && (aEvent->eventStructType == NS_MOUSE_EVENT ||
                  aEvent->eventStructType == NS_KEY_EVENT)) {
-    shift = static_cast<nsInputEvent *>(aEvent)->isShift;
-    control = static_cast<nsInputEvent *>(aEvent)->isControl;
-    alt = static_cast<nsInputEvent *>(aEvent)->isAlt;
-    meta = static_cast<nsInputEvent *>(aEvent)->isMeta;
+    shift = static_cast<nsInputEvent *>(aEvent)->IsShift();
+    control = static_cast<nsInputEvent *>(aEvent)->IsControl();
+    alt = static_cast<nsInputEvent *>(aEvent)->IsAlt();
+    meta = static_cast<nsInputEvent *>(aEvent)->IsMeta();
   }
 
   // Because the command event is firing asynchronously, a flag is needed to
@@ -1289,7 +1261,7 @@ nsMenuFrame::PassMenuCommandEventToPopupManager()
   if (pm && mMenuParent && mDelayedMenuCommandEvent) {
     pm->ExecuteMenu(mContent, mDelayedMenuCommandEvent);
   }
-  mDelayedMenuCommandEvent = nsnull;
+  mDelayedMenuCommandEvent = nullptr;
 }
 
 NS_IMETHODIMP
@@ -1330,7 +1302,7 @@ nsMenuFrame::InsertFrames(ChildListID     aListID,
     return NS_OK;
 
   if (NS_UNLIKELY(aPrevFrame && aPrevFrame == GetPopup())) {
-    aPrevFrame = nsnull;
+    aPrevFrame = nullptr;
   }
 
   return nsBoxFrame::InsertFrames(aListID, aPrevFrame, aFrameList);
@@ -1366,7 +1338,7 @@ nsMenuFrame::SizeToPopup(nsBoxLayoutState& aState, nsSize& aSize)
   if (!IsCollapsed()) {
     bool widthSet, heightSet;
     nsSize tmpSize(-1, 0);
-    nsIBox::AddCSSPrefSize(this, tmpSize, widthSet, heightSet);
+    nsIFrame::AddCSSPrefSize(this, tmpSize, widthSet, heightSet);
     if (!widthSet && GetFlex(aState) == 0) {
       nsMenuPopupFrame* popupFrame = GetPopup();
       if (!popupFrame)
@@ -1429,7 +1401,7 @@ nsMenuFrame::GetActiveChild(nsIDOMElement** aResult)
 
   nsMenuFrame* menuFrame = popupFrame->GetCurrentMenuItem();
   if (!menuFrame) {
-    *aResult = nsnull;
+    *aResult = nullptr;
   }
   else {
     nsCOMPtr<nsIDOMElement> elt(do_QueryInterface(menuFrame->GetContent()));
@@ -1449,15 +1421,15 @@ nsMenuFrame::SetActiveChild(nsIDOMElement* aChild)
 
   if (!aChild) {
     // Remove the current selection
-    popupFrame->ChangeMenuItem(nsnull, false);
+    popupFrame->ChangeMenuItem(nullptr, false);
     return NS_OK;
   }
 
   nsCOMPtr<nsIContent> child(do_QueryInterface(aChild));
 
-  nsIFrame* kid = child->GetPrimaryFrame();
-  if (kid && kid->GetType() == nsGkAtoms::menuFrame)
-    popupFrame->ChangeMenuItem(static_cast<nsMenuFrame *>(kid), false);
+  nsMenuFrame* menu = do_QueryFrame(child->GetPrimaryFrame());
+  if (menu)
+    popupFrame->ChangeMenuItem(menu, false);
   return NS_OK;
 }
 
@@ -1465,11 +1437,11 @@ nsIScrollableFrame* nsMenuFrame::GetScrollTargetFrame()
 {
   nsMenuPopupFrame* popupFrame = GetPopup();
   if (!popupFrame)
-    return nsnull;
+    return nullptr;
   nsIFrame* childFrame = popupFrame->GetFirstPrincipalChild();
   if (childFrame)
     return popupFrame->GetScrollFrame(childFrame);
-  return nsnull;
+  return nullptr;
 }
 
 // nsMenuTimerMediator implementation.
@@ -1508,5 +1480,5 @@ NS_IMETHODIMP nsMenuTimerMediator::Notify(nsITimer* aTimer)
  */
 void nsMenuTimerMediator::ClearFrame()
 {
-  mFrame = nsnull;
+  mFrame = nullptr;
 }
