@@ -1,49 +1,21 @@
 /* -*- Mode: Java; c-basic-offset: 4; tab-width: 20; indent-tabs-mode: nil; -*-
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Android code.
- *
- * The Initial Developer of the Original Code is Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2011
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Margaret Leibovic <margaret.leibovic@gmail.com>
- *   Sriram Ramasubramanian <sriram@mozilla.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 package org.mozilla.gecko;
 
 import org.mozilla.gecko.gfx.FloatSize;
+import org.mozilla.gecko.util.GeckoEventListener;
+
+import org.json.JSONArray;
+import org.json.JSONException;
+import org.json.JSONObject;
 
 import android.content.Context;
-import android.util.Log;
+import android.content.res.Resources;
 import android.util.AttributeSet;
-import android.util.DisplayMetrics;
+import android.util.Log;
 import android.util.Pair;
 import android.view.LayoutInflater;
 import android.view.View;
@@ -51,21 +23,20 @@ import android.view.ViewGroup;
 import android.view.animation.Animation;
 import android.view.animation.AnimationUtils;
 import android.view.inputmethod.InputMethodManager;
+import android.widget.AdapterView;
 import android.widget.AdapterView.OnItemClickListener;
 import android.widget.ArrayAdapter;
-import android.widget.AdapterView;
 import android.widget.ImageView;
 import android.widget.ListView;
 import android.widget.RelativeLayout;
 import android.widget.TextView;
 
-import org.json.JSONArray;
-import org.json.JSONException;
-import org.json.JSONObject;
+import java.util.Arrays;
+import java.util.Collection;
 
 public class FormAssistPopup extends RelativeLayout implements GeckoEventListener {
     private Context mContext;
-    private Animation mAnimation; 
+    private Animation mAnimation;
 
     private ListView mAutoCompleteList;
     private RelativeLayout mValidationMessage;
@@ -80,20 +51,17 @@ public class FormAssistPopup extends RelativeLayout implements GeckoEventListene
     private static RelativeLayout.LayoutParams sValidationTextLayoutNormal;
     private static RelativeLayout.LayoutParams sValidationTextLayoutInverted;
 
-    // Minimum popup width for autocomplete messages
-    private static final int AUTOCOMPLETE_MIN_WIDTH_IN_DPI = 200;
-
-    // Height of the autocomplete_list_item TextView
-    private static final int AUTOCOMPLETE_ROW_HEIGHT_IN_DPI = 32;
-
-    // Height of the validation_message_text TextView
-    private static final int VALIDATION_MESSAGE_HEIGHT_IN_DPI = 50;
-
-    // Top margin for the validation_message_text TextView
-    private static final int VALIDATION_MESSAGE_MARGIN_TOP_IN_DPI = 6;
-
     private static final String LOGTAG = "FormAssistPopup";
 
+    // The blocklist is so short that ArrayList is probably cheaper than HashSet.
+    private static final Collection<String> sInputMethodBlocklist = Arrays.asList(new String[] {
+                                            InputMethods.METHOD_GOOGLE_JAPANESE_INPUT, // bug 775850
+                                            InputMethods.METHOD_OPENWNN_PLUS,          // bug 768108
+                                            InputMethods.METHOD_SIMEJI,                // bug 768108
+                                            InputMethods.METHOD_SWYPE,                 // bug 755909
+                                            InputMethods.METHOD_SWYPE_BETA,            // bug 755909
+                                            });
+
     public FormAssistPopup(Context context, AttributeSet attrs) {
         super(context, attrs);
         mContext = context;
@@ -103,9 +71,15 @@ public class FormAssistPopup extends RelativeLayout implements GeckoEventListene
 
         setFocusable(false);
 
-        GeckoAppShell.registerGeckoEventListener("FormAssist:AutoComplete", this);
-        GeckoAppShell.registerGeckoEventListener("FormAssist:ValidationMessage", this);
-        GeckoAppShell.registerGeckoEventListener("FormAssist:Hide", this);
+        registerEventListener("FormAssist:AutoComplete");
+        registerEventListener("FormAssist:ValidationMessage");
+        registerEventListener("FormAssist:Hide");
+    }
+
+    void destroy() {
+        unregisterEventListener("FormAssist:AutoComplete");
+        unregisterEventListener("FormAssist:ValidationMessage");
+        unregisterEventListener("FormAssist:Hide");
     }
 
     public void handleMessage(String event, JSONObject message) {
@@ -163,7 +137,7 @@ public class FormAssistPopup extends RelativeLayout implements GeckoEventListene
                     // since they can be different.
                     TextView textView = (TextView) view;
                     String value = (String) textView.getTag();
-                    GeckoAppShell.sendEventToGecko(GeckoEvent.createBroadcastEvent("FormAssist:AutoComplete", value));
+                    broadcastGeckoEvent("FormAssist:AutoComplete", value);
                     hide();
                 }
             });
@@ -186,8 +160,7 @@ public class FormAssistPopup extends RelativeLayout implements GeckoEventListene
             addView(mValidationMessage);
             mValidationMessageText = (TextView) mValidationMessage.findViewById(R.id.validation_message_text);
 
-            DisplayMetrics metrics = GeckoApp.mAppContext.getDisplayMetrics();
-            sValidationTextMarginTop = (int) (VALIDATION_MESSAGE_MARGIN_TOP_IN_DPI * metrics.density);
+            sValidationTextMarginTop = (int) (mContext.getResources().getDimension(R.dimen.validation_message_margin_top));
 
             sValidationTextLayoutNormal = new RelativeLayout.LayoutParams(mValidationMessageText.getLayoutParams());
             sValidationTextLayoutNormal.setMargins(0, sValidationTextMarginTop, 0, 0);
@@ -226,13 +199,11 @@ public class FormAssistPopup extends RelativeLayout implements GeckoEventListene
         if (mValidationMessage != null)
             mValidationMessage.setVisibility(isAutoComplete ? GONE : VISIBLE);
 
-        // Initialize static variables based on DisplayMetrics. We delay this to
-        // make sure DisplayMetrics isn't null to avoid an NPE.
         if (sAutoCompleteMinWidth == 0) {
-            DisplayMetrics metrics = GeckoApp.mAppContext.getDisplayMetrics();
-            sAutoCompleteMinWidth = (int) (AUTOCOMPLETE_MIN_WIDTH_IN_DPI * metrics.density);
-            sAutoCompleteRowHeight = (int) (AUTOCOMPLETE_ROW_HEIGHT_IN_DPI * metrics.density);
-            sValidationMessageHeight = (int) (VALIDATION_MESSAGE_HEIGHT_IN_DPI * metrics.density);
+            Resources res = mContext.getResources();
+            sAutoCompleteMinWidth = (int) (res.getDimension(R.dimen.autocomplete_min_width));
+            sAutoCompleteRowHeight = (int) (res.getDimension(R.dimen.autocomplete_row_height));
+            sValidationMessageHeight = (int) (res.getDimension(R.dimen.validation_message_height));
         }
 
         // These values correspond to the input box for which we want to
@@ -252,7 +223,7 @@ public class FormAssistPopup extends RelativeLayout implements GeckoEventListene
         int popupWidth = RelativeLayout.LayoutParams.FILL_PARENT;
         int popupLeft = left < 0 ? 0 : left;
 
-        FloatSize viewport = GeckoApp.mAppContext.getLayerController().getViewportSize();
+        FloatSize viewport = GeckoApp.mAppContext.getLayerView().getViewportMetrics().getSize();
 
         // For autocomplete suggestions, if the input is smaller than the screen-width,
         // shrink the popup's width. Otherwise, keep it as FILL_PARENT.
@@ -320,10 +291,19 @@ public class FormAssistPopup extends RelativeLayout implements GeckoEventListene
     public void hide() {
         if (isShown()) {
             setVisibility(GONE);
-            GeckoAppShell.sendEventToGecko(GeckoEvent.createBroadcastEvent("FormAssist:Hidden", null));
+            broadcastGeckoEvent("FormAssist:Hidden", null);
         }
     }
 
+    void onInputMethodChanged(String newInputMethod) {
+        boolean blocklisted = sInputMethodBlocklist.contains(newInputMethod);
+        broadcastGeckoEvent("FormAssist:Blocklisted", String.valueOf(blocklisted));
+    }
+
+    private static void broadcastGeckoEvent(String eventName, String eventData) {
+        GeckoAppShell.sendEventToGecko(GeckoEvent.createBroadcastEvent(eventName, eventData));
+    }
+
     private class AutoCompleteListAdapter extends ArrayAdapter<Pair<String, String>> {
         private LayoutInflater mInflater;
         private int mTextViewResourceId;
@@ -346,7 +326,7 @@ public class FormAssistPopup extends RelativeLayout implements GeckoEventListene
                     add(new Pair<String, String>(label, value));
                 }
             } catch (JSONException e) {
-                Log.e(LOGTAG, "JSONException: " + e);
+                Log.e(LOGTAG, "JSONException", e);
             }
         }
 
@@ -367,4 +347,12 @@ public class FormAssistPopup extends RelativeLayout implements GeckoEventListene
             return convertView;
         }
     }
+
+    private void registerEventListener(String event) {
+        GeckoAppShell.getEventDispatcher().registerEventListener(event, this);
+    }
+
+    private void unregisterEventListener(String event) {
+        GeckoAppShell.getEventDispatcher().unregisterEventListener(event, this);
+    }
 }
