@@ -9,18 +9,21 @@
 
 package org.mozilla.gecko;
 
-import java.io.File;
-import java.io.FileReader;
-import java.io.FileWriter;
-import java.io.IOException;
-import java.nio.channels.FileChannel;
-import java.util.HashMap;
+import org.mozilla.gecko.util.INIParser;
+import org.mozilla.gecko.util.INISection;
+
 import android.content.Context;
-import android.os.Build;
 import android.os.SystemClock;
 import android.text.TextUtils;
 import android.util.Log;
 
+import java.io.File;
+import java.io.FileReader;
+import java.io.IOException;
+import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.Hashtable;
+
 public final class GeckoProfile {
     private static final String LOGTAG = "GeckoProfile";
 
@@ -34,24 +37,58 @@ public final class GeckoProfile {
     // this short timeout is a temporary fix until bug 735399 is implemented
     private static final long SESSION_TIMEOUT = 30 * 1000; // 30 seconds
 
+    static private INIParser getProfilesINI(Context context) {
+      File filesDir = context.getFilesDir();
+      File mozillaDir = new File(filesDir, "mozilla");
+      File profilesIni = new File(mozillaDir, "profiles.ini");
+      return new INIParser(profilesIni);
+    }
+
     public static GeckoProfile get(Context context) {
-        return get(context, null);
+        if (context instanceof GeckoApp)
+            return get(context, ((GeckoApp)context).getDefaultProfileName());
+
+        return get(context, "");
     }
 
     public static GeckoProfile get(Context context, String profileName) {
+        return get(context, profileName, null);
+    }
+
+    public static GeckoProfile get(Context context, String profileName, String profilePath) {
         if (context == null) {
             throw new IllegalArgumentException("context must be non-null");
         }
-        if (TextUtils.isEmpty(profileName)) {
-            // XXX: TO-DO read profiles.ini to get the default profile. bug 715307
+
+        // if no profile was passed in, look for the default profile listed in profiles.ini
+        // if that doesn't exist, look for a profile called 'default'
+        if (TextUtils.isEmpty(profileName) && TextUtils.isEmpty(profilePath)) {
             profileName = "default";
+
+            INIParser parser = getProfilesINI(context);
+
+            String profile = "";
+            boolean foundDefault = false;
+            for (Enumeration<INISection> e = parser.getSections().elements(); e.hasMoreElements();) {
+                INISection section = e.nextElement();
+                if (section.getIntProperty("Default") == 1) {
+                    profile = section.getStringProperty("Name");
+                    foundDefault = true;
+                }
+            }
+
+            if (foundDefault)
+                profileName = profile;
         }
 
+        // actually try to look up the profile
         synchronized (sProfileCache) {
             GeckoProfile profile = sProfileCache.get(profileName);
             if (profile == null) {
-                profile = new GeckoProfile(context, profileName);
+                profile = new GeckoProfile(context, profileName, profilePath);
                 sProfileCache.put(profileName, profile);
+            } else {
+                profile.setDir(profilePath);
             }
             return profile;
         }
@@ -70,11 +107,39 @@ public final class GeckoProfile {
         }
     }
 
+    public static boolean removeProfile(Context context, String profileName) {
+        return new GeckoProfile(context, profileName).remove();
+    }
+
     private GeckoProfile(Context context, String profileName) {
         mContext = context;
         mName = profileName;
     }
 
+    private GeckoProfile(Context context, String profileName, String profilePath) {
+        mContext = context;
+        mName = profileName;
+        setDir(profilePath);
+    }
+
+    private void setDir(String profilePath) {
+        if (!TextUtils.isEmpty(profilePath)) {
+            File dir = new File(profilePath);
+            if (dir.exists() && dir.isDirectory()) {
+                if (mDir != null) {
+                    Log.i(LOGTAG, "profile dir changed from "+mDir+" to "+dir);
+                }
+                mDir = dir;
+            } else {
+                Log.w(LOGTAG, "requested profile directory missing: "+profilePath);
+            }
+        }
+    }
+
+    public String getName() {
+        return mName;
+    }
+
     public synchronized File getDir() {
         if (mDir != null) {
             return mDir;
@@ -83,14 +148,17 @@ public final class GeckoProfile {
         try {
             // Check for old profiles that may need migration.
             ProfileMigrator profileMigrator = new ProfileMigrator(mContext);
-            if (!profileMigrator.isProfileMoved()) {
+            if (!GeckoApp.sIsUsingCustomProfile &&
+                !profileMigrator.isProfileMoved()) {
                 Log.i(LOGTAG, "New installation or update, checking for old profiles.");
                 profileMigrator.launchMoveProfile();
             }
 
+            // now check if a profile with this name that already exists
             File mozillaDir = ensureMozillaDirectory(mContext);
             mDir = findProfileDir(mozillaDir);
             if (mDir == null) {
+                // otherwise create it
                 mDir = createProfileDir(mozillaDir);
             } else {
                 Log.d(LOGTAG, "Found profile dir: " + mDir.getAbsolutePath());
@@ -101,6 +169,14 @@ public final class GeckoProfile {
         return mDir;
     }
 
+    public File getFile(String aFile) {
+        File f = getDir();
+        if (f == null)
+            return null;
+
+        return new File(f, aFile);
+    }
+
     public File getFilesDir() {
         return mContext.getFilesDir();
     }
@@ -176,17 +252,74 @@ public final class GeckoProfile {
         }
     }
 
-    private File findProfileDir(File mozillaDir) {
-        String suffix = '.' + mName;
-        File[] candidates = mozillaDir.listFiles();
-        if (candidates == null) {
-            return null;
+    private boolean remove() {
+        try {
+            File mozillaDir = ensureMozillaDirectory(mContext);
+            mDir = findProfileDir(mozillaDir);
+            if (mDir == null)
+                return false;
+
+            INIParser parser = getProfilesINI(mContext);
+
+            Hashtable<String, INISection> sections = parser.getSections();
+            for (Enumeration<INISection> e = sections.elements(); e.hasMoreElements();) {
+                INISection section = e.nextElement();
+                String name = section.getStringProperty("Name");
+
+                if (name == null || !name.equals(mName))
+                    continue;
+
+                if (section.getName().startsWith("Profile")) {
+                    // ok, we have stupid Profile#-named things.  Rename backwards.
+                    try {
+                        int sectionNumber = Integer.parseInt(section.getName().substring("Profile".length()));
+                        String curSection = "Profile" + sectionNumber;
+                        String nextSection = "Profile" + (sectionNumber+1);
+
+                        sections.remove(curSection);
+
+                        while (sections.containsKey(nextSection)) {
+                            parser.renameSection(nextSection, curSection);
+                            sectionNumber++;
+                            
+                            curSection = nextSection;
+                            nextSection = "Profile" + (sectionNumber+1);
+                        }
+                    } catch (NumberFormatException nex) {
+                        // uhm, malformed Profile thing; we can't do much.
+                        Log.e(LOGTAG, "Malformed section name in profiles.ini: " + section.getName());
+                        return false;
+                    }
+                } else {
+                    // this really shouldn't be the case, but handle it anyway
+                    parser.removeSection(mName);
+                    return true;
+                }
+            }
+
+            parser.write();
+            return true;
+        } catch (IOException ex) {
+            Log.w(LOGTAG, "Failed to remove profile " + mName + ":\n" + ex);
+            return false;
         }
-        for (File f : candidates) {
-            if (f.isDirectory() && f.getName().endsWith(suffix)) {
-                return f;
+    }
+
+    private File findProfileDir(File mozillaDir) {
+        // Open profiles.ini to find the correct path
+        INIParser parser = getProfilesINI(mContext);
+
+        for (Enumeration<INISection> e = parser.getSections().elements(); e.hasMoreElements();) {
+            INISection section = e.nextElement();
+            String name = section.getStringProperty("Name");
+            if (name != null && name.equals(mName)) {
+                if (section.getIntProperty("IsRelative") == 1) {
+                    return new File(mozillaDir, section.getStringProperty("Path"));
+                }
+                return new File(section.getStringProperty("Path"));
             }
         }
+
         return null;
     }
 
@@ -202,14 +335,9 @@ public final class GeckoProfile {
     }
 
     private File createProfileDir(File mozillaDir) throws IOException {
-        // XXX: TO-DO If we already have an ini file, we should append the
-        //      new profile information to it. For now we just throw an exception.
-        //      see bug 715391
-        File profileIniFile = new File(mozillaDir, "profiles.ini");
-        if (profileIniFile.exists()) {
-            throw new IOException("Can't create new profiles");
-        }
+        INIParser parser = getProfilesINI(mContext);
 
+        // Salt the name of our requested profile
         String saltedName = saltProfileName(mName);
         File profileDir = new File(mozillaDir, saltedName);
         while (profileDir.exists()) {
@@ -217,25 +345,38 @@ public final class GeckoProfile {
             profileDir = new File(mozillaDir, saltedName);
         }
 
+        // Attempt to create the salted profile dir
         if (! profileDir.mkdirs()) {
             throw new IOException("Unable to create profile at " + profileDir.getAbsolutePath());
         }
         Log.d(LOGTAG, "Created new profile dir at " + profileDir.getAbsolutePath());
 
-        FileWriter out = new FileWriter(profileIniFile, true);
-        try {
-            out.write("[General]\n" +
-                      "StartWithLastProfile=1\n" +
-                      "\n" +
-                      "[Profile0]\n" +
-                      "Name=" + mName + "\n" +
-                      "IsRelative=1\n" +
-                      "Path=" + saltedName + "\n" +
-                      "Default=1\n");
-        } finally {
-            out.close();
+        // Now update profiles.ini
+        // If this is the first time its created, we also add a General section
+        // look for the first profile number that isn't taken yet
+        int profileNum = 0;
+        while (parser.getSection("Profile" + profileNum) != null) {
+            profileNum++;
         }
 
+        INISection profileSection = new INISection("Profile" + profileNum);
+        profileSection.setProperty("Name", mName);
+        profileSection.setProperty("IsRelative", 1);
+        profileSection.setProperty("Path", saltedName);
+
+        if (parser.getSection("General") == null) {
+            INISection generalSection = new INISection("General");
+            generalSection.setProperty("StartWithLastProfile", 1);
+            parser.addSection(generalSection);
+
+            // only set as default if this is the first profile we're creating
+            Log.i(LOGTAG, "WESJ - SET DEFAULT");
+            profileSection.setProperty("Default", 1);
+        }
+
+        parser.addSection(profileSection);
+        parser.write();
+
         return profileDir;
     }
 }
