@@ -1,85 +1,62 @@
 /* -*- Mode: Java; c-basic-offset: 4; tab-width: 20; indent-tabs-mode: nil; -*-
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Android code.
- *
- * The Initial Developer of the Original Code is Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2009-2010
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Patrick Walton <pcwalton@mozilla.com>
- *   Chris Lord <chrislord.net@gmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 package org.mozilla.gecko.gfx;
 
-import org.mozilla.gecko.FloatUtils;
-import org.mozilla.gecko.GeckoApp;
 import org.mozilla.gecko.GeckoAppShell;
 import org.mozilla.gecko.GeckoEvent;
-import org.mozilla.gecko.GeckoEventResponder;
 import org.mozilla.gecko.Tab;
 import org.mozilla.gecko.Tabs;
+import org.mozilla.gecko.ZoomConstraints;
+import org.mozilla.gecko.ui.PanZoomController;
+import org.mozilla.gecko.ui.PanZoomTarget;
+import org.mozilla.gecko.util.EventDispatcher;
+import org.mozilla.gecko.util.GeckoEventResponder;
+
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
+
 import android.content.Context;
-import android.graphics.Bitmap;
-import android.graphics.Point;
 import android.graphics.PointF;
-import android.graphics.Rect;
 import android.graphics.RectF;
 import android.os.SystemClock;
 import android.util.DisplayMetrics;
 import android.util.Log;
-import android.view.View;
-import java.util.Map;
-import java.util.HashMap;
 
-public class GeckoLayerClient implements GeckoEventResponder,
-                                         LayerView.Listener {
+import java.util.HashMap;
+import java.util.Map;
+
+public class GeckoLayerClient
+        implements GeckoEventResponder, LayerView.Listener, PanZoomTarget
+{
     private static final String LOGTAG = "GeckoLayerClient";
 
-    private LayerController mLayerController;
     private LayerRenderer mLayerRenderer;
     private boolean mLayerRendererInitialized;
 
+    private final EventDispatcher mEventDispatcher;
+    private Context mContext;
     private IntSize mScreenSize;
     private IntSize mWindowSize;
     private DisplayPortMetrics mDisplayPort;
     private DisplayPortMetrics mReturnDisplayPort;
 
     private boolean mRecordDrawTimes;
-    private DrawTimingQueue mDrawTimingQueue;
+    private final DrawTimingQueue mDrawTimingQueue;
 
     private VirtualLayer mRootLayer;
 
-    /* The Gecko viewport as per the UI thread. Must be touched only on the UI thread. */
+    /* The Gecko viewport as per the UI thread. Must be touched only on the UI thread.
+     * If any events being sent to Gecko that are relative to the Gecko viewport position,
+     * they must (a) be relative to this viewport, and (b) be sent on the UI thread to
+     * avoid races. As long as these two conditions are satisfied, and the events being
+     * sent to Gecko are processed in FIFO order, the events will properly be relative
+     * to the Gecko viewport position. Note that if Gecko updates its viewport independently,
+     * we get notified synchronously and also update this on the UI thread.
+     */
     private ViewportMetrics mGeckoViewport;
 
     /*
@@ -88,43 +65,75 @@ public class GeckoLayerClient implements GeckoEventResponder,
      */
     private ImmutableViewportMetrics mFrameMetrics;
 
-    private String mLastCheckerboardColor;
-
     /* Used by robocop for testing purposes */
     private DrawListener mDrawListener;
 
     /* Used as a temporary ViewTransform by syncViewportInfo */
-    private ViewTransform mCurrentViewTransform;
+    private final ViewTransform mCurrentViewTransform;
 
-    public GeckoLayerClient(Context context) {
+    /* This is written by the compositor thread and read by the UI thread. */
+    private volatile boolean mCompositorCreated;
+
+    private boolean mForceRedraw;
+
+    /* The current viewport metrics.
+     * This is volatile so that we can read and write to it from different threads.
+     * We avoid synchronization to make getting the viewport metrics from
+     * the compositor as cheap as possible. The viewport is immutable so
+     * we don't need to worry about anyone mutating it while we're reading from it.
+     * Specifically:
+     * 1) reading mViewportMetrics from any thread is fine without synchronization
+     * 2) writing to mViewportMetrics requires synchronizing on the layer controller object
+     * 3) whenver reading multiple fields from mViewportMetrics without synchronization (i.e. in
+     *    case 1 above) you should always frist grab a local copy of the reference, and then use
+     *    that because mViewportMetrics might get reassigned in between reading the different
+     *    fields. */
+    private volatile ImmutableViewportMetrics mViewportMetrics;
+
+    private ZoomConstraints mZoomConstraints;
+
+    private boolean mGeckoIsReady;
+
+    private final PanZoomController mPanZoomController;
+    private LayerView mView;
+
+    public GeckoLayerClient(Context context, LayerView view, EventDispatcher eventDispatcher) {
         // we can fill these in with dummy values because they are always written
         // to before being read
+        mEventDispatcher = eventDispatcher;
+        mContext = context;
         mScreenSize = new IntSize(0, 0);
         mWindowSize = new IntSize(0, 0);
         mDisplayPort = new DisplayPortMetrics();
         mRecordDrawTimes = true;
         mDrawTimingQueue = new DrawTimingQueue();
         mCurrentViewTransform = new ViewTransform(0, 0, 1);
+        mCompositorCreated = false;
+
+        mForceRedraw = true;
+        DisplayMetrics displayMetrics = context.getResources().getDisplayMetrics();
+        mViewportMetrics = new ImmutableViewportMetrics(new ViewportMetrics(displayMetrics));
+        mZoomConstraints = new ZoomConstraints(false);
+
+        mPanZoomController = new PanZoomController(this, mEventDispatcher);
+        mView = view;
     }
 
-    /** Attaches the root layer to the layer controller so that Gecko appears. */
-    void setLayerController(LayerController layerController) {
-        LayerView view = layerController.getView();
+    /** Attaches to root layer so that Gecko appears. */
+    public void notifyGeckoReady() {
+        mGeckoIsReady = true;
 
-        mLayerController = layerController;
+        mRootLayer = new VirtualLayer(new IntSize(mView.getWidth(), mView.getHeight()));
+        mLayerRenderer = new LayerRenderer(mView);
 
-        mRootLayer = new VirtualLayer(new IntSize(view.getWidth(), view.getHeight()));
-        mLayerRenderer = new LayerRenderer(view);
+        registerEventListener("Viewport:Update");
+        registerEventListener("Viewport:PageSize");
+        registerEventListener("Viewport:CalculateDisplayPort");
+        registerEventListener("Checkerboard:Toggle");
+        registerEventListener("Preferences:Data");
 
-        GeckoAppShell.registerGeckoEventListener("Viewport:Update", this);
-        GeckoAppShell.registerGeckoEventListener("Viewport:PageSize", this);
-        GeckoAppShell.registerGeckoEventListener("Viewport:CalculateDisplayPort", this);
-        GeckoAppShell.registerGeckoEventListener("Checkerboard:Toggle", this);
-        GeckoAppShell.registerGeckoEventListener("Preferences:Data", this);
-
-        view.setListener(this);
-        view.setLayerRenderer(mLayerRenderer);
-        layerController.setRoot(mRootLayer);
+        mView.setListener(this);
+        mView.setLayerRenderer(mLayerRenderer);
 
         sendResizeEventIfNecessary(true);
 
@@ -134,17 +143,88 @@ public class GeckoLayerClient implements GeckoEventResponder,
         GeckoAppShell.sendEventToGecko(GeckoEvent.createBroadcastEvent("Preferences:Get", prefs.toString()));
     }
 
-    DisplayPortMetrics getDisplayPort() {
-        return mDisplayPort;
+    public void destroy() {
+        mPanZoomController.destroy();
+        unregisterEventListener("Viewport:Update");
+        unregisterEventListener("Viewport:PageSize");
+        unregisterEventListener("Viewport:CalculateDisplayPort");
+        unregisterEventListener("Checkerboard:Toggle");
+        unregisterEventListener("Preferences:Data");
+    }
+
+    private void registerEventListener(String event) {
+        mEventDispatcher.registerEventListener(event, this);
+    }
+
+    private void unregisterEventListener(String event) {
+        mEventDispatcher.unregisterEventListener(event, this);
+    }
+
+    /**
+     * Returns true if this client is fine with performing a redraw operation or false if it
+     * would prefer that the action didn't take place.
+     */
+    private boolean getRedrawHint() {
+        if (mForceRedraw) {
+            mForceRedraw = false;
+            return true;
+        }
+
+        if (!mPanZoomController.getRedrawHint()) {
+            return false;
+        }
+
+        return DisplayPortCalculator.aboutToCheckerboard(mViewportMetrics,
+                mPanZoomController.getVelocityVector(), mDisplayPort);
+    }
+
+    Layer getRoot() {
+        return mGeckoIsReady ? mRootLayer : null;
+    }
+
+    public LayerView getView() {
+        return mView;
+    }
+
+    public FloatSize getViewportSize() {
+        return mViewportMetrics.getSize();
+    }
+
+    /**
+     * The view calls this function to indicate that the viewport changed size. It must hold the
+     * monitor while calling it.
+     *
+     * TODO: Refactor this to use an interface. Expose that interface only to the view and not
+     * to the layer client. That way, the layer client won't be tempted to call this, which might
+     * result in an infinite loop.
+     */
+    void setViewportSize(FloatSize size) {
+        ViewportMetrics viewportMetrics = new ViewportMetrics(mViewportMetrics);
+        viewportMetrics.setSize(size);
+        mViewportMetrics = new ImmutableViewportMetrics(viewportMetrics);
+
+        if (mGeckoIsReady) {
+            // here we send gecko a resize message. The code in browser.js is responsible for
+            // picking up on that resize event, modifying the viewport as necessary, and informing
+            // us of the new viewport.
+            sendResizeEventIfNecessary(true);
+            // the following call also sends gecko a message, which will be processed after the resize
+            // message above has updated the viewport. this message ensures that if we have just put
+            // focus in a text field, we scroll the content so that the text field is in view.
+            GeckoAppShell.viewSizeChanged();
+        }
+    }
+
+    PanZoomController getPanZoomController() {
+        return mPanZoomController;
     }
 
     /* Informs Gecko that the screen size has changed. */
     private void sendResizeEventIfNecessary(boolean force) {
-        DisplayMetrics metrics = GeckoApp.mAppContext.getDisplayMetrics();
-        View view = mLayerController.getView();
+        DisplayMetrics metrics = mContext.getResources().getDisplayMetrics();
 
         IntSize newScreenSize = new IntSize(metrics.widthPixels, metrics.heightPixels);
-        IntSize newWindowSize = new IntSize(view.getWidth(), view.getHeight());
+        IntSize newWindowSize = new IntSize(mView.getWidth(), mView.getHeight());
 
         boolean screenSizeChanged = !mScreenSize.equals(newScreenSize);
         boolean windowSizeChanged = !mWindowSize.equals(newWindowSize);
@@ -167,32 +247,40 @@ public class GeckoLayerClient implements GeckoEventResponder,
         GeckoEvent event = GeckoEvent.createSizeChangedEvent(mWindowSize.width, mWindowSize.height,
                                                              mScreenSize.width, mScreenSize.height);
         GeckoAppShell.sendEventToGecko(event);
+        GeckoAppShell.sendEventToGecko(GeckoEvent.createBroadcastEvent("Window:Resize", ""));
     }
 
-    public Bitmap getBitmap() {
-        return null;
+    /** Sets the current page rect. You must hold the monitor while calling this. */
+    private void setPageRect(RectF rect, RectF cssRect) {
+        // Since the "rect" is always just a multiple of "cssRect" we don't need to
+        // check both; this function assumes that both "rect" and "cssRect" are relative
+        // the zoom factor in mViewportMetrics.
+        if (mViewportMetrics.getCssPageRect().equals(cssRect))
+            return;
+
+        ViewportMetrics viewportMetrics = new ViewportMetrics(mViewportMetrics);
+        viewportMetrics.setPageRect(rect, cssRect);
+        mViewportMetrics = new ImmutableViewportMetrics(viewportMetrics);
+
+        // Page size is owned by the layer client, so no need to notify it of
+        // this change.
+
+        post(new Runnable() {
+            public void run() {
+                mPanZoomController.pageRectUpdated();
+                mView.requestRender();
+            }
+        });
     }
 
-    void viewportSizeChanged() {
-        // here we send gecko a resize message. The code in browser.js is responsible for
-        // picking up on that resize event, modifying the viewport as necessary, and informing
-        // us of the new viewport.
-        sendResizeEventIfNecessary(true);
-        // the following call also sends gecko a message, which will be processed after the resize
-        // message above has updated the viewport. this message ensures that if we have just put
-        // focus in a text field, we scroll the content so that the text field is in view.
-        GeckoAppShell.viewSizeChanged();
-    }
-
-    void adjustViewport(DisplayPortMetrics displayPort) {
-        ImmutableViewportMetrics metrics = mLayerController.getViewportMetrics();
+    private void adjustViewport(DisplayPortMetrics displayPort) {
+        ImmutableViewportMetrics metrics = getViewportMetrics();
 
         ViewportMetrics clampedMetrics = new ViewportMetrics(metrics);
         clampedMetrics.setViewport(clampedMetrics.getClampedViewport());
 
         if (displayPort == null) {
-            displayPort = DisplayPortCalculator.calculate(metrics,
-                    mLayerController.getPanZoomController().getVelocityVector());
+            displayPort = DisplayPortCalculator.calculate(metrics, mPanZoomController.getVelocityVector());
         }
 
         mDisplayPort = displayPort;
@@ -205,6 +293,17 @@ public class GeckoLayerClient implements GeckoEventResponder,
         GeckoAppShell.sendEventToGecko(GeckoEvent.createViewportEvent(clampedMetrics, displayPort));
     }
 
+    /** Aborts any pan/zoom animation that is currently in progress. */
+    private void abortPanZoomAnimation() {
+        if (mPanZoomController != null) {
+            post(new Runnable() {
+                public void run() {
+                    mPanZoomController.abortAnimation();
+                }
+            });
+        }
+    }
+
     /**
      * The different types of Viewport messages handled. All viewport events
      * expect a display-port to be returned, but can handle one not being
@@ -218,9 +317,9 @@ public class GeckoLayerClient implements GeckoEventResponder,
     /** Viewport message handler. */
     private void handleViewportMessage(JSONObject message, ViewportMessageType type) throws JSONException {
         ViewportMetrics messageMetrics = new ViewportMetrics(message);
-        synchronized (mLayerController) {
+        synchronized (this) {
             final ViewportMetrics newMetrics;
-            ImmutableViewportMetrics oldMetrics = mLayerController.getViewportMetrics();
+            ImmutableViewportMetrics oldMetrics = getViewportMetrics();
 
             switch (type) {
             default:
@@ -228,7 +327,7 @@ public class GeckoLayerClient implements GeckoEventResponder,
                 newMetrics = messageMetrics;
                 // Keep the old viewport size
                 newMetrics.setSize(oldMetrics.getSize());
-                mLayerController.abortPanZoomAnimation();
+                abortPanZoomAnimation();
                 break;
             case PAGE_SIZE:
                 // adjust the page dimensions to account for differences in zoom
@@ -240,13 +339,13 @@ public class GeckoLayerClient implements GeckoEventResponder,
                 break;
             }
 
-            mLayerController.post(new Runnable() {
+            post(new Runnable() {
                 public void run() {
                     mGeckoViewport = newMetrics;
                 }
             });
-            mLayerController.setViewportMetrics(newMetrics);
-            mDisplayPort = DisplayPortCalculator.calculate(mLayerController.getViewportMetrics(), null);
+            setViewportMetrics(newMetrics);
+            mDisplayPort = DisplayPortCalculator.calculate(getViewportMetrics(), null);
         }
         mReturnDisplayPort = mDisplayPort;
     }
@@ -262,9 +361,7 @@ public class GeckoLayerClient implements GeckoEventResponder,
                 ImmutableViewportMetrics newMetrics = new ImmutableViewportMetrics(new ViewportMetrics(message));
                 mReturnDisplayPort = DisplayPortCalculator.calculate(newMetrics, null);
             } else if ("Checkerboard:Toggle".equals(event)) {
-                boolean showChecks = message.getBoolean("value");
-                mLayerController.setCheckerboardShowChecks(showChecks);
-                Log.i(LOGTAG, "Showing checks: " + showChecks);
+                mView.setCheckerboardShouldShowChecks(message.getBoolean("value"));
             } else if ("Preferences:Data".equals(event)) {
                 JSONArray jsonPrefs = message.getJSONArray("preferences");
                 Map<String, Integer> prefValues = new HashMap<String, Integer>();
@@ -282,7 +379,7 @@ public class GeckoLayerClient implements GeckoEventResponder,
                 // right batch of prefs, since other java code may also have sent requests
                 // for prefs.
                 if (DisplayPortCalculator.setStrategy(prefValues) && PluginLayer.setUsePlaceholder(prefValues)) {
-                    GeckoAppShell.unregisterGeckoEventListener("Preferences:Data", this);
+                    unregisterEventListener("Preferences:Data");
                 }
             }
         } catch (JSONException e) {
@@ -307,23 +404,8 @@ public class GeckoLayerClient implements GeckoEventResponder,
         }
     }
 
-    void geometryChanged() {
-        /* Let Gecko know if the screensize has changed */
-        sendResizeEventIfNecessary(false);
-        if (mLayerController.getRedrawHint())
-            adjustViewport(null);
-    }
-
-    /*
-     * This function returns the last viewport that we sent to Gecko. If any additional events are
-     * being sent to Gecko that are relative on the Gecko viewport position, they must (a) be relative
-     * to this viewport, and (b) be sent on the UI thread to avoid races. As long as these two
-     * conditions are satisfied, and the events being sent to Gecko are processed in FIFO order, the
-     * events will properly be relative to the Gecko viewport position. Note that if Gecko updates
-     * its viewport independently, we get notified synchronously and also update this on the UI thread.
-     */
-    public ViewportMetrics getGeckoViewportMetrics() {
-        return mGeckoViewport;
+    void setZoomConstraints(ZoomConstraints constraints) {
+        mZoomConstraints = constraints;
     }
 
     /** This function is invoked by Gecko via JNI; be careful when modifying signature.
@@ -336,29 +418,26 @@ public class GeckoLayerClient implements GeckoEventResponder,
     public void setFirstPaintViewport(float offsetX, float offsetY, float zoom,
             float pageLeft, float pageTop, float pageRight, float pageBottom,
             float cssPageLeft, float cssPageTop, float cssPageRight, float cssPageBottom) {
-        synchronized (mLayerController) {
-            final ViewportMetrics currentMetrics = new ViewportMetrics(mLayerController.getViewportMetrics());
+        synchronized (this) {
+            final ViewportMetrics currentMetrics = new ViewportMetrics(getViewportMetrics());
             currentMetrics.setOrigin(new PointF(offsetX, offsetY));
             currentMetrics.setZoomFactor(zoom);
             currentMetrics.setPageRect(new RectF(pageLeft, pageTop, pageRight, pageBottom),
                                        new RectF(cssPageLeft, cssPageTop, cssPageRight, cssPageBottom));
             // Since we have switched to displaying a different document, we need to update any
-            // viewport-related state we have lying around. This includes mGeckoViewport and the
-            // viewport in mLayerController. Usually this information is updated via handleViewportMessage
+            // viewport-related state we have lying around. This includes mGeckoViewport and
+            // mViewportMetrics. Usually this information is updated via handleViewportMessage
             // while we remain on the same document.
-            mLayerController.post(new Runnable() {
+            post(new Runnable() {
                 public void run() {
                     mGeckoViewport = currentMetrics;
                 }
             });
-            mLayerController.setViewportMetrics(currentMetrics);
+            setViewportMetrics(currentMetrics);
 
             Tab tab = Tabs.getInstance().getSelectedTab();
-            mLayerController.setCheckerboardColor(tab.getCheckerboardColor());
-            mLayerController.setAllowZoom(tab.getAllowZoom());
-            mLayerController.setDefaultZoom(tab.getDefaultZoom());
-            mLayerController.setMinZoom(tab.getMinZoom());
-            mLayerController.setMaxZoom(tab.getMaxZoom());
+            mView.setCheckerboardColor(tab.getCheckerboardColor());
+            setZoomConstraints(tab.getZoomConstraints());
 
             // At this point, we have just switched to displaying a different document than we
             // we previously displaying. This means we need to abort any panning/zooming animations
@@ -367,12 +446,11 @@ public class GeckoLayerClient implements GeckoEventResponder,
             // sends the request after aborting the animation. The display port request is actually
             // a full viewport update, which is fine because if browser.js has somehow moved to
             // be out of sync with this first-paint viewport, then we force them back in sync.
-            mLayerController.abortPanZoomAnimation();
-            mLayerController.getView().setPaintState(LayerView.PAINT_BEFORE_FIRST);
+            abortPanZoomAnimation();
         }
         DisplayPortCalculator.resetPageState();
         mDrawTimingQueue.reset();
-        mLayerController.getView().getRenderer().resetCheckerboard();
+        mView.getRenderer().resetCheckerboard();
         GeckoAppShell.screenshotWholePage(Tabs.getInstance().getSelectedTab());
     }
 
@@ -382,16 +460,11 @@ public class GeckoLayerClient implements GeckoEventResponder,
       * is invoked on a frame, then this function will not be. For any given frame, this
       * function will be invoked before syncViewportInfo.
       */
-    public void setPageRect(float zoom, float pageLeft, float pageTop, float pageRight, float pageBottom,
-            float cssPageLeft, float cssPageTop, float cssPageRight, float cssPageBottom) {
-        synchronized (mLayerController) {
-            // adjust the page dimensions to account for differences in zoom
-            // between the rendered content (which is what the compositor tells us)
-            // and our zoom level (which may have diverged).
-            RectF pageRect = new RectF(pageLeft, pageTop, pageRight, pageBottom);
+    public void setPageRect(float cssPageLeft, float cssPageTop, float cssPageRight, float cssPageBottom) {
+        synchronized (this) {
             RectF cssPageRect = new RectF(cssPageLeft, cssPageTop, cssPageRight, cssPageBottom);
-            float ourZoom = mLayerController.getZoomFactor();
-            mLayerController.setPageRect(RectUtils.scale(pageRect, ourZoom / zoom), cssPageRect);
+            float ourZoom = getViewportMetrics().zoomFactor;
+            setPageRect(RectUtils.scale(cssPageRect, ourZoom), cssPageRect);
             // Here the page size of the document has changed, but the document being displayed
             // is still the same. Therefore, we don't need to send anything to browser.js; any
             // changes we need to make to the display port will get sent the next time we call
@@ -409,14 +482,13 @@ public class GeckoLayerClient implements GeckoEventResponder,
       * which would avoid the copy into mCurrentViewTransform.
       */
     public ViewTransform syncViewportInfo(int x, int y, int width, int height, float resolution, boolean layersUpdated) {
-        // getViewportMetrics is thread safe so we don't need to synchronize
-        // on mLayerController.
+        // getViewportMetrics is thread safe so we don't need to synchronize.
         // We save the viewport metrics here, so we later use it later in
         // createFrame (which will be called by nsWindow::DrawWindowUnderlay on
-        // the native side, by the compositor). The LayerController's viewport
+        // the native side, by the compositor). The viewport
         // metrics can change between here and there, as it's accessed outside
         // of the compositor thread.
-        mFrameMetrics = mLayerController.getViewportMetrics();
+        mFrameMetrics = getViewportMetrics();
 
         mCurrentViewTransform.x = mFrameMetrics.viewportRectLeft;
         mCurrentViewTransform.y = mFrameMetrics.viewportRectTop;
@@ -467,6 +539,14 @@ public class GeckoLayerClient implements GeckoEventResponder,
         mLayerRenderer.deactivateDefaultProgram();
     }
 
+    private void geometryChanged() {
+        /* Let Gecko know if the screensize has changed */
+        sendResizeEventIfNecessary(false);
+        if (getRedrawHint()) {
+            adjustViewport(null);
+        }
+    }
+
     /** Implementation of LayerView.Listener */
     public void renderRequested() {
         GeckoAppShell.scheduleComposite();
@@ -482,7 +562,9 @@ public class GeckoLayerClient implements GeckoEventResponder,
         // Gecko draw events have been processed.  When this returns, composition is
         // definitely paused -- it'll synchronize with the Gecko event loop, which
         // in turn will synchronize with the compositor thread.
-        GeckoAppShell.sendEventToGeckoSync(GeckoEvent.createCompositorPauseEvent());
+        if (mCompositorCreated) {
+            GeckoAppShell.sendEventToGeckoSync(GeckoEvent.createCompositorPauseEvent());
+        }
     }
 
     /** Implementation of LayerView.Listener */
@@ -491,13 +573,15 @@ public class GeckoLayerClient implements GeckoEventResponder,
         // https://bugzilla.mozilla.org/show_bug.cgi?id=735230#c23), so we
         // resume the compositor directly. We still need to inform Gecko about
         // the compositor resuming, so that Gecko knows that it can now draw.
-        GeckoAppShell.scheduleResumeComposition(width, height);
-        GeckoAppShell.sendEventToGecko(GeckoEvent.createCompositorResumeEvent());
+        if (mCompositorCreated) {
+            GeckoAppShell.scheduleResumeComposition(width, height);
+            GeckoAppShell.sendEventToGecko(GeckoEvent.createCompositorResumeEvent());
+        }
     }
 
     /** Implementation of LayerView.Listener */
     public void surfaceChanged(int width, int height) {
-        mLayerController.setViewportSize(new FloatSize(width, height));
+        setViewportSize(new FloatSize(width, height));
 
         // We need to make this call even when the compositor isn't currently
         // paused (e.g. during an orientation change), to make the compositor
@@ -506,14 +590,101 @@ public class GeckoLayerClient implements GeckoEventResponder,
         renderRequested();
     }
 
-    /** Used by robocop for testing purposes. Not for production use! This is called via reflection by robocop. */
+    /** Implementation of LayerView.Listener */
+    public void compositorCreated() {
+        mCompositorCreated = true;
+    }
+
+    /** Implementation of PanZoomTarget */
+    public ImmutableViewportMetrics getViewportMetrics() {
+        return mViewportMetrics;
+    }
+
+    /** Implementation of PanZoomTarget */
+    public ZoomConstraints getZoomConstraints() {
+        return mZoomConstraints;
+    }
+
+    /** Implementation of PanZoomTarget */
+    public void setAnimationTarget(ViewportMetrics viewport) {
+        if (mGeckoIsReady) {
+            // We know what the final viewport of the animation is going to be, so
+            // immediately request a draw of that area by setting the display port
+            // accordingly. This way we should have the content pre-rendered by the
+            // time the animation is done.
+            ImmutableViewportMetrics metrics = new ImmutableViewportMetrics(viewport);
+            DisplayPortMetrics displayPort = DisplayPortCalculator.calculate(metrics, null);
+            adjustViewport(displayPort);
+        }
+    }
+
+    /** Implementation of PanZoomTarget
+     * You must hold the monitor while calling this.
+     */
+    public void setViewportMetrics(ViewportMetrics viewport) {
+        mViewportMetrics = new ImmutableViewportMetrics(viewport);
+        mView.requestRender();
+        if (mGeckoIsReady) {
+            geometryChanged();
+        }
+    }
+
+    /** Implementation of PanZoomTarget */
+    public void setForceRedraw() {
+        mForceRedraw = true;
+        if (mGeckoIsReady) {
+            geometryChanged();
+        }
+    }
+
+    /** Implementation of PanZoomTarget */
+    public boolean post(Runnable action) {
+        return mView.post(action);
+    }
+
+    /** Implementation of PanZoomTarget */
+    public Object getLock() {
+        return this;
+    }
+
+    /** Implementation of PanZoomTarget
+     * Converts a point from layer view coordinates to layer coordinates. In other words, given a
+     * point measured in pixels from the top left corner of the layer view, returns the point in
+     * pixels measured from the last scroll position we sent to Gecko, in CSS pixels. Assuming the
+     * events being sent to Gecko are processed in FIFO order, this calculation should always be
+     * correct.
+     */
+    public PointF convertViewPointToLayerPoint(PointF viewPoint) {
+        if (!mGeckoIsReady) {
+            return null;
+        }
+
+        ImmutableViewportMetrics viewportMetrics = mViewportMetrics;
+        PointF origin = viewportMetrics.getOrigin();
+        float zoom = viewportMetrics.zoomFactor;
+        ViewportMetrics geckoViewport = mGeckoViewport;
+        PointF geckoOrigin = geckoViewport.getOrigin();
+        float geckoZoom = geckoViewport.getZoomFactor();
+
+        // viewPoint + origin gives the coordinate in device pixels from the top-left corner of the page.
+        // Divided by zoom, this gives us the coordinate in CSS pixels from the top-left corner of the page.
+        // geckoOrigin / geckoZoom is where Gecko thinks it is (scrollTo position) in CSS pixels from
+        // the top-left corner of the page. Subtracting the two gives us the offset of the viewPoint from
+        // the current Gecko coordinate in CSS pixels.
+        PointF layerPoint = new PointF(
+                ((viewPoint.x + origin.x) / zoom) - (geckoOrigin.x / geckoZoom),
+                ((viewPoint.y + origin.y) / zoom) - (geckoOrigin.y / geckoZoom));
+
+        return layerPoint;
+    }
+
+    /** Used by robocop for testing purposes. Not for production use! */
     public void setDrawListener(DrawListener listener) {
         mDrawListener = listener;
     }
 
-    /** Used by robocop for testing purposes. Not for production use! This is used via reflection by robocop. */
-    public interface DrawListener {
+    /** Used by robocop for testing purposes. Not for production use! */
+    public static interface DrawListener {
         public void drawFinished();
     }
 }
-
