@@ -1,11 +1,12 @@
 /* This Source Code Form is subject to the terms of the Mozilla Public
- * License, v. 2.0. If a copy of the MPL was not distributed with this file,
- * You can obtain one at http://mozilla.org/MPL/2.0/. */
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 package org.mozilla.gecko.sync;
 
 import java.net.URI;
 import java.net.URISyntaxException;
+import java.util.Collection;
 import java.util.HashSet;
 import java.util.Map;
 import java.util.Set;
@@ -195,7 +196,19 @@ public class SyncConfiguration implements CredentialsSource {
    * Copied from latest downloaded meta/global record and used to generate a
    * fresh meta/global record for upload.
    */
-  public Set<String>     enabledEngineNames;
+  public Set<String> enabledEngineNames;
+
+  /**
+   * Names of stages to sync <it>this sync</it>, or <code>null</code> to sync
+   * all known stages.
+   * <p>
+   * Generated <it>each sync</it> from extras bundle passed to
+   * <code>SyncAdapter.onPerformSync</code> and not persisted.
+   * <p>
+   * Not synchronized! Set this exactly once per global session and don't modify
+   * it -- especially not from multiple threads.
+   */
+  public Collection<String> stagesToSync;
 
   // Fields that maintain a reference to a SharedPreferences instance, used for
   // persistence.
@@ -203,6 +216,9 @@ public class SyncConfiguration implements CredentialsSource {
   public String          prefsPath;
   public PrefsSource     prefsSource;
 
+  public static final String PREF_PREFS_VERSION = "prefs.version";
+  public static final long CURRENT_PREFS_VERSION = 1;
+
   public static final String CLIENTS_COLLECTION_TIMESTAMP = "serverClientsTimestamp";  // When the collection was touched.
   public static final String CLIENT_RECORD_TIMESTAMP = "serverClientRecordTimestamp";  // When our record was touched.
 
@@ -210,6 +226,13 @@ public class SyncConfiguration implements CredentialsSource {
   public static final String PREF_SYNC_ID = "syncID";
   public static final String PREF_ENABLED_ENGINE_NAMES = "enabledEngineNames";
 
+  public static final String PREF_EARLIEST_NEXT_SYNC = "earliestnextsync";
+  public static final String PREF_CLUSTER_URL_IS_STALE = "clusterurlisstale";
+
+  public static final String PREF_ACCOUNT_GUID = "account.guid";
+  public static final String PREF_CLIENT_NAME = "account.clientName";
+  public static final String PREF_NUM_CLIENTS = "account.numClients";
+
   /**
    * Create a new SyncConfiguration instance. Pass in a PrefsSource to
    * provide access to preferences.
@@ -221,7 +244,7 @@ public class SyncConfiguration implements CredentialsSource {
   }
 
   public SharedPreferences getPrefs() {
-    Logger.debug(LOG_TAG, "Returning prefs for " + prefsPath);
+    Logger.trace(LOG_TAG, "Returning prefs for " + prefsPath);
     return prefsSource.getPrefs(prefsPath, Utils.SHARED_PREFERENCES_MODE);
   }
 
@@ -241,14 +264,14 @@ public class SyncConfiguration implements CredentialsSource {
       String u = prefs.getString(PREF_CLUSTER_URL, null);
       try {
         clusterURL = new URI(u);
-        Logger.info(LOG_TAG, "Set clusterURL from bundle: " + u);
+        Logger.trace(LOG_TAG, "Set clusterURL from bundle: " + u);
       } catch (URISyntaxException e) {
         Logger.warn(LOG_TAG, "Ignoring bundle clusterURL (" + u + "): invalid URI.", e);
       }
     }
     if (prefs.contains(PREF_SYNC_ID)) {
       syncID = prefs.getString(PREF_SYNC_ID, null);
-      Logger.info(LOG_TAG, "Set syncID from bundle: " + syncID);
+      Logger.trace(LOG_TAG, "Set syncID from bundle: " + syncID);
     }
     if (prefs.contains(PREF_ENABLED_ENGINE_NAMES)) {
       String json = prefs.getString(PREF_ENABLED_ENGINE_NAMES, null);
@@ -373,7 +396,7 @@ public class SyncConfiguration implements CredentialsSource {
   protected void setAndPersistClusterURL(URI u, SharedPreferences prefs) {
     boolean shouldPersist = (prefs != null) && (clusterURL == null);
 
-    Logger.debug(LOG_TAG, "Setting cluster URL to " + u.toASCIIString() +
+    Logger.trace(LOG_TAG, "Setting cluster URL to " + u.toASCIIString() +
                           (shouldPersist ? ". Persisting." : ". Not persisting."));
     clusterURL = u;
     if (shouldPersist) {
@@ -394,7 +417,7 @@ public class SyncConfiguration implements CredentialsSource {
       return;
     }
     setAndPersistClusterURL(uri.resolve("/"), prefs);
-    Logger.info(LOG_TAG, "Set cluster URL to " + clusterURL.toASCIIString() + ", given input " + u.toASCIIString());
+    Logger.trace(LOG_TAG, "Set cluster URL to " + clusterURL.toASCIIString() + ", given input " + u.toASCIIString());
   }
 
   public void setClusterURL(URI u) {
