@@ -1,17 +1,20 @@
 /* This Source Code Form is subject to the terms of the Mozilla Public
- * License, v. 2.0. If a copy of the MPL was not distributed with this file,
- * You can obtain one at http://mozilla.org/MPL/2.0/. */
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 package org.mozilla.gecko.sync.stage;
 
 import java.io.UnsupportedEncodingException;
 import java.net.URI;
 import java.net.URISyntaxException;
+import java.util.ArrayList;
+import java.util.List;
 import java.util.concurrent.atomic.AtomicInteger;
 
 import org.json.simple.JSONArray;
 import org.json.simple.JSONObject;
 import org.mozilla.gecko.sync.CommandProcessor;
+import org.mozilla.gecko.sync.CommandProcessor.Command;
 import org.mozilla.gecko.sync.CryptoRecord;
 import org.mozilla.gecko.sync.ExtendedJSONObject;
 import org.mozilla.gecko.sync.GlobalSession;
@@ -28,6 +31,7 @@ import org.mozilla.gecko.sync.net.SyncStorageRecordRequest;
 import org.mozilla.gecko.sync.net.SyncStorageResponse;
 import org.mozilla.gecko.sync.net.WBOCollectionRequestDelegate;
 import org.mozilla.gecko.sync.net.WBORequestDelegate;
+import org.mozilla.gecko.sync.repositories.NullCursorException;
 import org.mozilla.gecko.sync.repositories.android.ClientsDatabaseAccessor;
 import org.mozilla.gecko.sync.repositories.android.RepoUtils;
 import org.mozilla.gecko.sync.repositories.domain.ClientRecord;
@@ -37,9 +41,11 @@ import org.mozilla.gecko.sync.repositories.domain.VersionConstants;
 import ch.boye.httpclientandroidlib.HttpStatus;
 
 public class SyncClientsEngineStage implements GlobalSyncStage {
-  public static final String LOG_TAG = "SyncClientsEngineStage";
-  public static final String COLLECTION_NAME = "clients";
-  public static final int CLIENTS_TTL_REFRESH = 604800000; // 7 days
+  private static final String LOG_TAG = "SyncClientsEngineStage";
+
+  public static final String COLLECTION_NAME       = "clients";
+  public static final String STAGE_NAME            = COLLECTION_NAME;
+  public static final int CLIENTS_TTL_REFRESH      = 604800000;   // 7 days in milliseconds.
   public static final int MAX_UPLOAD_FAILURE_COUNT = 5;
 
   protected final GlobalSession session;
@@ -53,6 +59,7 @@ public class SyncClientsEngineStage implements GlobalSyncStage {
   protected volatile boolean shouldWipe;
   protected volatile boolean commandsProcessedShouldUpload;
   protected final AtomicInteger uploadAttemptsCount = new AtomicInteger();
+  protected final List<ClientRecord> toUpload = new ArrayList<ClientRecord>();
 
   public SyncClientsEngineStage(GlobalSession session) {
     if (session == null) {
@@ -116,6 +123,9 @@ public class SyncClientsEngineStage implements GlobalSyncStage {
       session.config.persistServerClientsTimestamp(response.normalizedWeaveTimestamp());
       BaseResource.consumeEntity(response);
 
+      // Wipe the clients table if it still hasn't been wiped but needs to be.
+      wipeAndStore(null);
+
       // If we successfully downloaded all records but ours was not one of them
       // then reset the timestamp.
       if (!localAccountGUIDDownloaded) {
@@ -139,6 +149,13 @@ public class SyncClientsEngineStage implements GlobalSyncStage {
       // TODO: persist the response timestamp to know whether to download next time (Bug 726055).
       clientUploadDelegate = new ClientUploadDelegate();
       clientsDelegate.setClientsCount(clientsCount);
+
+      // If we upload remote records, checkAndUpload() will be called upon
+      // upload success in the delegate. Otherwise call checkAndUpload() now.
+      if (toUpload.size() > 0) {
+        uploadRemoteRecords();
+        return;
+      }
       checkAndUpload();
     }
 
@@ -179,13 +196,16 @@ public class SyncClientsEngineStage implements GlobalSyncStage {
           localAccountGUIDDownloaded = true;
           session.config.persistServerClientRecordTimestamp(r.lastModified);
           processCommands(r.commands);
+        } else {
+          // Only need to store record if it isn't our local one.
+          wipeAndStore(r);
+          addCommands(r);
         }
         RepoUtils.logClient(r);
       } catch (Exception e) {
         session.abort(e, "Exception handling client WBO.");
         return;
       }
-      wipeAndStore(r);
     }
 
     @Override
@@ -193,7 +213,6 @@ public class SyncClientsEngineStage implements GlobalSyncStage {
       try {
         return session.keyBundleForCollection(COLLECTION_NAME);
       } catch (NoCollectionKeysSetException e) {
-        session.abort(e, "No collection keys set.");
         return null;
       }
     }
@@ -201,19 +220,26 @@ public class SyncClientsEngineStage implements GlobalSyncStage {
 
   public class ClientUploadDelegate extends WBORequestDelegate {
     protected static final String LOG_TAG = "ClientUploadDelegate";
+    public Long currentlyUploadingRecordTimestamp;
+    public boolean currentlyUploadingLocalRecord;
 
     @Override
     public String credentials() {
       return session.credentials();
     }
 
+    private void setUploadDetails(boolean isLocalRecord) {
+      // Use the timestamp for the whole collection per Sync storage 1.1 spec.
+      currentlyUploadingRecordTimestamp = session.config.getPersistedServerClientsTimestamp();
+      currentlyUploadingLocalRecord = isLocalRecord;
+    }
+
     @Override
     public String ifUnmodifiedSince() {
-      // Use the timestamp for the whole collection per Sync storage 1.1 spec.
-      Long timestampInMilliseconds = session.config.getPersistedServerClientsTimestamp();
+      Long timestampInMilliseconds = currentlyUploadingRecordTimestamp;
 
       // It's the first upload so we don't care about X-If-Unmodified-Since.
-      if (timestampInMilliseconds == 0) {
+      if (timestampInMilliseconds <= 0) {
         return null;
       }
 
@@ -223,22 +249,35 @@ public class SyncClientsEngineStage implements GlobalSyncStage {
     @Override
     public void handleRequestSuccess(SyncStorageResponse response) {
       Logger.debug(LOG_TAG, "Upload succeeded.");
-      try {
-        commandsProcessedShouldUpload = false;
-        uploadAttemptsCount.set(0);
+      uploadAttemptsCount.set(0);
 
-        // Persist the timestamp for the record we just uploaded,
-        // and bump the collection timestamp, too.
-        long timestamp = response.normalizedWeaveTimestamp();
-        session.config.persistServerClientRecordTimestamp(timestamp);
-        session.config.persistServerClientsTimestamp(timestamp);
-        BaseResource.consumeEntity(response);
+      // X-Weave-Timestamp is the modified time of uploaded records.
+      // Always persist this.
+      final long responseTimestamp = response.normalizedWeaveTimestamp();
+      Logger.trace(LOG_TAG, "Timestamp from header is: " + responseTimestamp);
 
-        Logger.debug(LOG_TAG, "Timestamp is " + timestamp);
-      } catch (Exception e) {
-        session.abort(e, "Unable to fetch timestamp.");
+      if (responseTimestamp == -1) {
+        final String message = "Response did not contain a valid timestamp.";
+        session.abort(new RuntimeException(message), message);
         return;
       }
+
+      BaseResource.consumeEntity(response);
+      session.config.persistServerClientsTimestamp(responseTimestamp);
+
+      // If we're not uploading our record, we're done here; just
+      // clean up and finish.
+      if (!currentlyUploadingLocalRecord) {
+        // TODO: check failed uploads in body.
+        clearRecordsToUpload();
+        checkAndUpload();
+        return;
+      }
+
+      // If we're processing our record, we have a little more cleanup
+      // to do.
+      commandsProcessedShouldUpload = false;
+      session.config.persistServerClientRecordTimestamp(responseTimestamp);
       session.advance();
     }
 
@@ -251,7 +290,11 @@ public class SyncClientsEngineStage implements GlobalSyncStage {
       if (!commandsProcessedShouldUpload ||
           statusCode == HttpStatus.SC_PRECONDITION_FAILED ||
           uploadAttemptsCount.incrementAndGet() > MAX_UPLOAD_FAILURE_COUNT) {
+
         Logger.debug(LOG_TAG, "Client upload failed. Aborting sync.");
+        if (!currentlyUploadingLocalRecord) {
+          toUpload.clear(); // These will be redownloaded.
+        }
         BaseResource.consumeEntity(response); // The exception thrown should need the response body.
         session.abort(new HTTPFailureException(response), "Client upload failed.");
         return;
@@ -275,7 +318,6 @@ public class SyncClientsEngineStage implements GlobalSyncStage {
       try {
         return session.keyBundleForCollection(COLLECTION_NAME);
       } catch (NoCollectionKeysSetException e) {
-        session.abort(e, "No collection keys set.");
         return null;
       }
     }
@@ -283,6 +325,15 @@ public class SyncClientsEngineStage implements GlobalSyncStage {
 
   @Override
   public void execute() throws NoSuchStageException {
+    // We can be disabled just for this sync.
+    boolean disabledThisSync = session.config.stagesToSync != null &&
+                               !session.config.stagesToSync.contains(STAGE_NAME);
+    if (disabledThisSync) {
+      Logger.debug(LOG_TAG, "Stage " + STAGE_NAME + " disabled just for this sync.");
+      session.advance();
+      return;
+    }
+
     if (shouldDownload()) {
       downloadClientRecords();   // Will kick off upload, too…
     } else {
@@ -298,7 +349,7 @@ public class SyncClientsEngineStage implements GlobalSyncStage {
 
     session.getClientsDelegate().setClientsCount(0);
     try {
-      getClientsDatabaseAccessor().wipe();
+      getClientsDatabaseAccessor().wipeDB();
     } finally {
       closeDataAccessor();
     }
@@ -356,12 +407,63 @@ public class SyncClientsEngineStage implements GlobalSyncStage {
     commandsProcessedShouldUpload = true;
     CommandProcessor processor = CommandProcessor.getProcessor();
 
-    // TODO: Bug 715792 - Process commands here.
-    for (int i = 0; i < commands.size(); i++) {
-      processor.processCommand(new ExtendedJSONObject((JSONObject) commands.get(i)));
+    for (Object o : commands) {
+      processor.processCommand(session, new ExtendedJSONObject((JSONObject) o));
     }
   }
 
+  @SuppressWarnings("unchecked")
+  protected void addCommands(ClientRecord record) throws NullCursorException {
+    Logger.trace(LOG_TAG, "Adding commands to " + record.guid);
+    List<Command> commands = db.fetchCommandsForClient(record.guid);
+
+    if (commands == null || commands.size() == 0) {
+      Logger.trace(LOG_TAG, "No commands to add.");
+      return;
+    }
+
+    for (Command command : commands) {
+      JSONObject jsonCommand = command.asJSONObject();
+      if (record.commands == null) {
+        record.commands = new JSONArray();
+      }
+      record.commands.add(jsonCommand);
+    }
+    toUpload.add(record);
+  }
+
+  @SuppressWarnings("unchecked")
+  protected void uploadRemoteRecords() {
+    Logger.trace(LOG_TAG, "In uploadRemoteRecords. Uploading " + toUpload.size() + " records" );
+
+    for (ClientRecord r : toUpload) {
+      Logger.trace(LOG_TAG, ">> Uploading record " + r.guid + ": " + r.name);
+    }
+
+    if (toUpload.size() == 1) {
+      ClientRecord record = toUpload.get(0);
+      Logger.debug(LOG_TAG, "Only 1 remote record to upload.");
+      Logger.debug(LOG_TAG, "Record last modified: " + record.lastModified);
+      CryptoRecord cryptoRecord = encryptClientRecord(record);
+      if (cryptoRecord != null) {
+        clientUploadDelegate.setUploadDetails(false);
+        this.uploadClientRecord(cryptoRecord);
+      }
+      return;
+    }
+
+    JSONArray cryptoRecords = new JSONArray();
+    for (ClientRecord record : toUpload) {
+      Logger.trace(LOG_TAG, "Record " + record.guid + " is being uploaded" );
+
+      CryptoRecord cryptoRecord = encryptClientRecord(record);
+      cryptoRecords.add(cryptoRecord.toJSONObject());
+    }
+    Logger.debug(LOG_TAG, "Uploading records: " + cryptoRecords.size());
+    clientUploadDelegate.setUploadDetails(false);
+    this.uploadClientRecords(cryptoRecords);
+  }
+
   protected void checkAndUpload() {
     if (!shouldUpload()) {
       Logger.debug(LOG_TAG, "Not uploading client record.");
@@ -369,19 +471,41 @@ public class SyncClientsEngineStage implements GlobalSyncStage {
       return;
     }
 
+    final ClientRecord localClient = newLocalClientRecord(session.getClientsDelegate());
+    clientUploadDelegate.setUploadDetails(true);
+    CryptoRecord cryptoRecord = encryptClientRecord(localClient);
+    if (cryptoRecord != null) {
+      this.uploadClientRecord(cryptoRecord);
+    }
+  }
+
+  protected CryptoRecord encryptClientRecord(ClientRecord recordToUpload) {
     // Generate CryptoRecord from ClientRecord to upload.
     final String encryptionFailure = "Couldn't encrypt new client record.";
-    final ClientRecord localClient = newLocalClientRecord(session.getClientsDelegate());
+
     try {
-      CryptoRecord cryptoRecord = localClient.getEnvelope();
+      CryptoRecord cryptoRecord = recordToUpload.getEnvelope();
       cryptoRecord.keyBundle = clientUploadDelegate.keyBundle();
-      cryptoRecord.encrypt();
-      this.uploadClientRecord(cryptoRecord);
+      if (cryptoRecord.keyBundle == null) {
+        session.abort(new NoCollectionKeysSetException(), "No collection keys set.");
+        return null;
+      }
+      return cryptoRecord.encrypt();
     } catch (UnsupportedEncodingException e) {
       session.abort(e, encryptionFailure + " Unsupported encoding.");
     } catch (CryptoException e) {
       session.abort(e, encryptionFailure);
     }
+    return null;
+  }
+
+  public void clearRecordsToUpload() {
+    try {
+      getClientsDatabaseAccessor().wipeCommandsTable();
+      toUpload.clear();
+    } finally {
+      closeDataAccessor();
+    }
   }
 
   protected void downloadClientRecords() {
@@ -389,9 +513,8 @@ public class SyncClientsEngineStage implements GlobalSyncStage {
     clientDownloadDelegate = makeClientDownloadDelegate();
 
     try {
-      URI getURI = session.config.collectionURI(COLLECTION_NAME, true);
-
-      SyncStorageCollectionRequest request = new SyncStorageCollectionRequest(getURI);
+      final URI getURI = session.config.collectionURI(COLLECTION_NAME, true);
+      final SyncStorageCollectionRequest request = new SyncStorageCollectionRequest(getURI);
       request.delegate = clientDownloadDelegate;
 
       Logger.trace(LOG_TAG, "Downloading client records.");
@@ -401,14 +524,28 @@ public class SyncClientsEngineStage implements GlobalSyncStage {
     }
   }
 
+  protected void uploadClientRecords(JSONArray records) {
+    Logger.trace(LOG_TAG, "Uploading " + records.size() + " client records.");
+    try {
+      final URI postURI = session.config.collectionURI(COLLECTION_NAME, false);
+      final SyncStorageRecordRequest request = new SyncStorageRecordRequest(postURI);
+      request.delegate = clientUploadDelegate;
+      request.post(records);
+    } catch (URISyntaxException e) {
+      session.abort(e, "Invalid URI.");
+    } catch (Exception e) {
+      session.abort(e, "Unable to parse body.");
+    }
+  }
+
   /**
    * Upload a client record via HTTP POST to the parent collection.
    */
   protected void uploadClientRecord(CryptoRecord record) {
     Logger.debug(LOG_TAG, "Uploading client record " + record.guid);
     try {
-      URI postURI = session.config.collectionURI(COLLECTION_NAME);
-      SyncStorageRecordRequest request = new SyncStorageRecordRequest(postURI);
+      final URI postURI = session.config.collectionURI(COLLECTION_NAME);
+      final SyncStorageRecordRequest request = new SyncStorageRecordRequest(postURI);
       request.delegate = clientUploadDelegate;
       request.post(record);
     } catch (URISyntaxException e) {
@@ -421,11 +558,13 @@ public class SyncClientsEngineStage implements GlobalSyncStage {
   }
 
   protected void wipeAndStore(ClientRecord record) {
-    ClientsDatabaseAccessor db = getClientsDatabaseAccessor();
+    final ClientsDatabaseAccessor db = getClientsDatabaseAccessor();
     if (shouldWipe) {
-      db.wipe();
+      db.wipeClientsTable();
       shouldWipe = false;
     }
-    db.store(record);
+    if (record != null) {
+      db.store(record);
+    }
   }
 }
