@@ -1,39 +1,6 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Session Store.
- *
- * The Initial Developer of the Original Code is Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2009
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Mark 'evil' Finkle <mfinkle@mozilla.com>
- *   Brian Nicholson <bnicholson@mozilla.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 const Cc = Components.classes;
 const Ci = Components.interfaces;
@@ -438,7 +405,7 @@ SessionStore.prototype = {
       let data = { entries: entries, index: index };
 
       delete aBrowser.__SS_data;
-      this._collectTabData(aBrowser, data);
+      this._collectTabData(aWindow, aBrowser, data);
       this.saveStateNow();
     }
 
@@ -511,7 +478,7 @@ SessionStore.prototype = {
     return data;
   },
 
-  _collectTabData: function ss__collectTabData(aBrowser, aHistory) {
+  _collectTabData: function ss__collectTabData(aWindow, aBrowser, aHistory) {
     // If this browser is being restored, skip any session save activity
     if (aBrowser.__SS_restore)
       return;
@@ -522,6 +489,7 @@ SessionStore.prototype = {
     tabData.entries = aHistory.entries;
     tabData.index = aHistory.index;
     tabData.attributes = { image: aBrowser.mIconURL };
+    tabData.desktopMode = aWindow.BrowserApp.getTabForBrowser(aBrowser).desktopMode;
 
     aBrowser.__SS_data = tabData;
   },
@@ -963,7 +931,8 @@ SessionStore.prototype = {
           Cu.reportError("SessionStore: Could not parse JSON: " + ex);
         }
 
-        if (!data || data.windows.length == 0) {
+        // To do a restore, we must have at least one window with one tab
+        if (!data || data.windows.length == 0 || !data.windows[0].tabs || data.windows[0].tabs.length == 0) {
           notifyObservers("fail");
           return;
         }
@@ -981,7 +950,12 @@ SessionStore.prototype = {
           let entry = tabData.entries[tabData.index - 1];
 
           // Add a tab, but don't load the URL until we need to
-          let params = { selected: isSelected, delayLoad: true, title: entry.title };
+          let params = {
+            selected: isSelected,
+            delayLoad: true,
+            title: entry.title,
+            desktopMode: tabData.desktopMode == true
+          };
           let tab = window.BrowserApp.addTab(entry.url, params);
 
           if (isSelected) {
