@@ -1,41 +1,8 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
  *
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org Code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Jeff Walden <jwalden+code@mit.edu>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsJAR.h"
 #include "nsJARChannel.h"
@@ -69,7 +36,7 @@ static NS_DEFINE_CID(kZipReaderCID, NS_ZIPREADER_CID);
 //
 // set NSPR_LOG_MODULES=nsJarProtocol:5
 //
-static PRLogModuleInfo *gJarProtocolLog = nsnull;
+static PRLogModuleInfo *gJarProtocolLog = nullptr;
 #endif
 
 #define LOG(args)     PR_LOG(gJarProtocolLog, PR_LOG_DEBUG, args)
@@ -116,7 +83,7 @@ public:
         NS_IF_ADDREF(*result = mJarReader);
     }
 
-    PRInt32 GetContentLength()
+    int32_t GetContentLength()
     {
         return mContentLength;
     }
@@ -130,7 +97,7 @@ private:
     nsCString                   mJarDirSpec;
     nsCOMPtr<nsIInputStream>    mJarStream;
     nsCString                   mJarEntry;
-    PRInt32                     mContentLength;
+    int32_t                     mContentLength;
 };
 
 NS_IMPL_THREADSAFE_ISUPPORTS1(nsJARInputThunk, nsIInputStream)
@@ -165,9 +132,12 @@ nsJARInputThunk::EnsureJarStream()
     }
 
     // ask the JarStream for the content length
-    rv = mJarStream->Available((PRUint32 *) &mContentLength);
+    uint64_t avail;
+    rv = mJarStream->Available((uint64_t *) &avail);
     if (NS_FAILED(rv)) return rv;
 
+    mContentLength = avail < PR_INT32_MAX ? (int32_t) avail : -1;
+
     return NS_OK;
 }
 
@@ -181,7 +151,7 @@ nsJARInputThunk::Close()
 }
 
 NS_IMETHODIMP
-nsJARInputThunk::Available(PRUint32 *avail)
+nsJARInputThunk::Available(uint64_t *avail)
 {
     nsresult rv = EnsureJarStream();
     if (NS_FAILED(rv)) return rv;
@@ -190,7 +160,7 @@ nsJARInputThunk::Available(PRUint32 *avail)
 }
 
 NS_IMETHODIMP
-nsJARInputThunk::Read(char *buf, PRUint32 count, PRUint32 *countRead)
+nsJARInputThunk::Read(char *buf, uint32_t count, uint32_t *countRead)
 {
     nsresult rv = EnsureJarStream();
     if (NS_FAILED(rv)) return rv;
@@ -200,7 +170,7 @@ nsJARInputThunk::Read(char *buf, PRUint32 count, PRUint32 *countRead)
 
 NS_IMETHODIMP
 nsJARInputThunk::ReadSegments(nsWriteSegmentFun writer, void *closure,
-                              PRUint32 count, PRUint32 *countRead)
+                              uint32_t count, uint32_t *countRead)
 {
     // stream transport does only calls Read()
     return NS_ERROR_NOT_IMPLEMENTED;
@@ -219,12 +189,13 @@ nsJARInputThunk::IsNonBlocking(bool *nonBlocking)
 
 
 nsJARChannel::nsJARChannel()
-    : mContentLength(-1)
+    : mAppURI(nullptr)
+    , mContentLength(-1)
     , mLoadFlags(LOAD_NORMAL)
     , mStatus(NS_OK)
     , mIsPending(false)
     , mIsUnsafe(true)
-    , mJarInput(nsnull)
+    , mJarInput(nullptr)
 {
 #if defined(PR_LOGGING)
     if (!gJarProtocolLog)
@@ -392,7 +363,7 @@ nsJARChannel::EnsureJarInput(bool blocking)
         // kick off an async download of the base URI...
         rv = NS_NewDownloader(getter_AddRefs(mDownloader), this);
         if (NS_SUCCEEDED(rv))
-            rv = NS_OpenURI(mDownloader, nsnull, mJarBaseURI, nsnull,
+            rv = NS_OpenURI(mDownloader, nullptr, mJarBaseURI, nullptr,
                             mLoadGroup, mCallbacks,
                             mLoadFlags & ~(LOAD_DOCUMENT_URI | LOAD_CALL_CONTENT_SNIFFERS));
     }
@@ -509,67 +480,21 @@ nsJARChannel::SetOriginalURI(nsIURI *aURI)
 NS_IMETHODIMP
 nsJARChannel::GetURI(nsIURI **aURI)
 {
-    NS_IF_ADDREF(*aURI = mJarURI);
+    if (mAppURI) {
+        NS_IF_ADDREF(*aURI = mAppURI);
+    } else {
+        NS_IF_ADDREF(*aURI = mJarURI);
+    }
+
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsJARChannel::GetOwner(nsISupports **result)
+nsJARChannel::GetOwner(nsISupports **aOwner)
 {
-    nsresult rv;
-
-    if (mOwner) {
-        NS_ADDREF(*result = mOwner);
-        return NS_OK;
-    }
-
-    if (!mJarInput) {
-        *result = nsnull;
-        return NS_OK;
-    }
-
-    //-- Verify signature, if one is present, and set owner accordingly
-    nsCOMPtr<nsIZipReader> jarReader;
-    mJarInput->GetJarReader(getter_AddRefs(jarReader));
-    if (!jarReader)
-        return NS_ERROR_NOT_INITIALIZED;
-
-    nsCOMPtr<nsIPrincipal> cert;
-    rv = jarReader->GetCertificatePrincipal(mJarEntry, getter_AddRefs(cert));
-    if (NS_FAILED(rv)) return rv;
-
-    if (cert) {
-        nsCAutoString certFingerprint;
-        rv = cert->GetFingerprint(certFingerprint);
-        if (NS_FAILED(rv)) return rv;
-
-        nsCAutoString subjectName;
-        rv = cert->GetSubjectName(subjectName);
-        if (NS_FAILED(rv)) return rv;
-
-        nsCAutoString prettyName;
-        rv = cert->GetPrettyName(prettyName);
-        if (NS_FAILED(rv)) return rv;
-
-        nsCOMPtr<nsISupports> certificate;
-        rv = cert->GetCertificate(getter_AddRefs(certificate));
-        if (NS_FAILED(rv)) return rv;
-        
-        nsCOMPtr<nsIScriptSecurityManager> secMan = 
-                 do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
-        if (NS_FAILED(rv)) return rv;
-
-        rv = secMan->GetCertificatePrincipal(certFingerprint, subjectName,
-                                             prettyName, certificate,
-                                             mJarBaseURI,
-                                             getter_AddRefs(cert));
-        if (NS_FAILED(rv)) return rv;
-
-        mOwner = do_QueryInterface(cert, &rv);
-        if (NS_FAILED(rv)) return rv;
-
-        NS_ADDREF(*result = mOwner);
-    }
+    // JAR signatures are not processed to avoid main-thread network I/O (bug 726125)
+    *aOwner = mOwner;
+    NS_IF_ADDREF(*aOwner);
     return NS_OK;
 }
 
@@ -609,8 +534,8 @@ nsJARChannel::GetContentType(nsACString &result)
         //
         // generate content type and set it
         //
-        const char *ext = nsnull, *fileName = mJarEntry.get();
-        PRInt32 len = mJarEntry.Length();
+        const char *ext = nullptr, *fileName = mJarEntry.get();
+        int32_t len = mJarEntry.Length();
 
         // check if we're displaying a directory
         // mJarEntry will be empty if we're trying to display
@@ -620,7 +545,7 @@ nsJARChannel::GetContentType(nsACString &result)
         }
         else {
             // not a directory, take a guess by its extension
-            for (PRInt32 i = len-1; i >= 0; i--) {
+            for (int32_t i = len-1; i >= 0; i--) {
                 if (fileName[i] == '.') {
                     ext = &fileName[i + 1];
                     break;
@@ -667,7 +592,7 @@ nsJARChannel::SetContentCharset(const nsACString &aContentCharset)
 }
 
 NS_IMETHODIMP
-nsJARChannel::GetContentDisposition(PRUint32 *aContentDisposition)
+nsJARChannel::GetContentDisposition(uint32_t *aContentDisposition)
 {
     if (mContentDispositionHeader.IsEmpty())
         return NS_ERROR_NOT_AVAILABLE;
@@ -693,7 +618,7 @@ nsJARChannel::GetContentDispositionHeader(nsACString &aContentDispositionHeader)
 }
 
 NS_IMETHODIMP
-nsJARChannel::GetContentLength(PRInt32 *result)
+nsJARChannel::GetContentLength(int32_t *result)
 {
     // if content length is unknown, query mJarInput...
     if (mContentLength < 0 && mJarInput)
@@ -704,7 +629,7 @@ nsJARChannel::GetContentLength(PRInt32 *result)
 }
 
 NS_IMETHODIMP
-nsJARChannel::SetContentLength(PRInt32 aContentLength)
+nsJARChannel::SetContentLength(int32_t aContentLength)
 {
     // XXX does this really make any sense at all?
     mContentLength = aContentLength;
@@ -719,7 +644,7 @@ nsJARChannel::Open(nsIInputStream **stream)
     NS_ENSURE_TRUE(!mJarInput, NS_ERROR_IN_PROGRESS);
     NS_ENSURE_TRUE(!mIsPending, NS_ERROR_IN_PROGRESS);
 
-    mJarFile = nsnull;
+    mJarFile = nullptr;
     mIsUnsafe = true;
 
     nsresult rv = EnsureJarInput(true);
@@ -745,7 +670,7 @@ nsJARChannel::AsyncOpen(nsIStreamListener *listener, nsISupports *ctx)
     NS_ENSURE_ARG_POINTER(listener);
     NS_ENSURE_TRUE(!mIsPending, NS_ERROR_IN_PROGRESS);
 
-    mJarFile = nsnull;
+    mJarFile = nullptr;
     mIsUnsafe = true;
 
     // Initialize mProgressSink
@@ -763,20 +688,20 @@ nsJARChannel::AsyncOpen(nsIStreamListener *listener, nsISupports *ctx)
         // create input stream pump and call AsyncRead as a block
         rv = NS_NewInputStreamPump(getter_AddRefs(mPump), mJarInput);
         if (NS_SUCCEEDED(rv))
-            rv = mPump->AsyncRead(this, nsnull);
+            rv = mPump->AsyncRead(this, nullptr);
 
         // If we failed to create the pump or initiate the AsyncRead,
         // then we need to clear these variables.
         if (NS_FAILED(rv)) {
             mIsPending = false;
-            mListenerContext = nsnull;
-            mListener = nsnull;
+            mListenerContext = nullptr;
+            mListener = nullptr;
             return rv;
         }
     }
 
     if (mLoadGroup)
-        mLoadGroup->AddRequest(this, nsnull);
+        mLoadGroup->AddRequest(this, nullptr);
 
     return NS_OK;
 }
@@ -791,6 +716,20 @@ nsJARChannel::GetIsUnsafe(bool *isUnsafe)
     return NS_OK;
 }
 
+NS_IMETHODIMP
+nsJARChannel::SetAppURI(nsIURI *aURI) {
+    NS_ENSURE_ARG_POINTER(aURI);
+
+    nsCAutoString scheme;
+    aURI->GetScheme(scheme);
+    if (!scheme.EqualsLiteral("app")) {
+        return NS_ERROR_INVALID_ARG;
+    }
+
+    mAppURI = aURI;
+    return NS_OK;
+}
+
 //-----------------------------------------------------------------------------
 // nsIDownloadObserver
 //-----------------------------------------------------------------------------
@@ -806,7 +745,7 @@ nsJARChannel::OnDownloadComplete(nsIDownloader *downloader,
 
     nsCOMPtr<nsIChannel> channel(do_QueryInterface(request));
     if (channel) {
-        PRUint32 loadFlags;
+        uint32_t loadFlags;
         channel->GetLoadFlags(&loadFlags);
         if (loadFlags & LOAD_REPLACE) {
             mLoadFlags |= LOAD_REPLACE;
@@ -880,20 +819,20 @@ nsJARChannel::OnDownloadComplete(nsIDownloader *downloader,
     if (NS_SUCCEEDED(status)) {
         mJarFile = file;
     
-        rv = CreateJarInput(nsnull);
+        rv = CreateJarInput(nullptr);
         if (NS_SUCCEEDED(rv)) {
             // create input stream pump
             rv = NS_NewInputStreamPump(getter_AddRefs(mPump), mJarInput);
             if (NS_SUCCEEDED(rv))
-                rv = mPump->AsyncRead(this, nsnull);
+                rv = mPump->AsyncRead(this, nullptr);
         }
         status = rv;
     }
 
     if (NS_FAILED(status)) {
         mStatus = status;
-        OnStartRequest(nsnull, nsnull);
-        OnStopRequest(nsnull, nsnull, status);
+        OnStartRequest(nullptr, nullptr);
+        OnStopRequest(nullptr, nullptr, status);
     }
 
     return NS_OK;
@@ -927,7 +866,7 @@ nsJARChannel::OnStopRequest(nsIRequest *req, nsISupports *ctx, nsresult status)
     }
 
     if (mLoadGroup)
-        mLoadGroup->RemoveRequest(this, nsnull, status);
+        mLoadGroup->RemoveRequest(this, nullptr, status);
 
     mPump = 0;
     NS_IF_RELEASE(mJarInput);
@@ -944,7 +883,7 @@ nsJARChannel::OnStopRequest(nsIRequest *req, nsISupports *ctx, nsresult status)
 NS_IMETHODIMP
 nsJARChannel::OnDataAvailable(nsIRequest *req, nsISupports *ctx,
                                nsIInputStream *stream,
-                               PRUint32 offset, PRUint32 count)
+                               uint32_t offset, uint32_t count)
 {
 #if defined(PR_LOGGING)
     LOG(("nsJARChannel::OnDataAvailable [this=%x %s]\n", this, mSpec.get()));
@@ -958,8 +897,8 @@ nsJARChannel::OnDataAvailable(nsIRequest *req, nsISupports *ctx,
     // nsITransportEventSink implementation.
     // XXX do the 64-bit stuff for real
     if (mProgressSink && NS_SUCCEEDED(rv) && !(mLoadFlags & LOAD_BACKGROUND))
-        mProgressSink->OnProgress(this, nsnull, PRUint64(offset + count),
-                                  PRUint64(mContentLength));
+        mProgressSink->OnProgress(this, nullptr, uint64_t(offset + count),
+                                  uint64_t(mContentLength));
 
     return rv; // let the pump cancel on failure
 }
