@@ -1,39 +1,7 @@
 /* vim:set ts=4 sw=4 sts=4 et cin: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla.
- *
- * The Initial Developer of the Original Code is IBM Corporation.
- * Portions created by IBM Corporation are Copyright (C) 2003
- * IBM Corporation. All Rights Reserved.
- *
- * Contributor(s):
- *   IBM Corp.
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #if defined(MOZ_LOGGING)
 #define FORCE_PR_LOG
@@ -50,7 +18,7 @@
 
 #include <stdlib.h>
 #include "nsHostResolver.h"
-#include "nsNetError.h"
+#include "nsError.h"
 #include "nsISupportsBase.h"
 #include "nsISupportsUtils.h"
 #include "nsAutoPtr.h"
@@ -63,6 +31,7 @@
 #include "pldhash.h"
 #include "plstr.h"
 #include "nsURLHelper.h"
+#include "nsThreadUtils.h"
 
 #include "mozilla/HashFunctions.h"
 #include "mozilla/FunctionTimer.h"
@@ -97,7 +66,7 @@ PR_STATIC_ASSERT (HighThreadThreshold <= MAX_RESOLVER_THREADS);
 //----------------------------------------------------------------------------
 
 #if defined(PR_LOGGING)
-static PRLogModuleInfo *gHostResolverLog = nsnull;
+static PRLogModuleInfo *gHostResolverLog = nullptr;
 #define LOG(args) PR_LOG(gHostResolverLog, PR_LOG_DEBUG, args)
 #else
 #define LOG(args)
@@ -117,13 +86,13 @@ MoveCList(PRCList &from, PRCList &to)
     }             
 }
 
-static PRUint32
+static uint32_t
 NowInMinutes()
 {
     PRTime now = PR_Now(), minutes, factor;
     LL_I2L(factor, 60 * PR_USEC_PER_SEC);
     LL_DIV(minutes, now, factor);
-    PRUint32 result;
+    uint32_t result;
     LL_L2UI(result, minutes);
     return result;
 }
@@ -160,7 +129,7 @@ public:
         if (PR_IntervalToSeconds(PR_IntervalNow() - mLastReset) < 1)
             return false;
 
-        LOG(("calling res_ninit\n"));
+        LOG(("Calling 'res_ninit'.\n"));
 
         mLastReset = PR_IntervalNow();
         return (res_ninit(&_res) == 0);
@@ -180,11 +149,10 @@ private:
 #define RES_KEY_FLAGS(_f) ((_f) & nsHostResolver::RES_CANON_NAME)
 
 nsHostRecord::nsHostRecord(const nsHostKey *key)
-    : _refc(1)
-    , addr_info_lock("nsHostRecord.addr_info_lock")
+    : addr_info_lock("nsHostRecord.addr_info_lock")
     , addr_info_gencnt(0)
-    , addr_info(nsnull)
-    , addr(nsnull)
+    , addr_info(nullptr)
+    , addr(nullptr)
     , negative(false)
     , resolving(false)
     , onQueue(false)
@@ -195,7 +163,6 @@ nsHostRecord::nsHostRecord(const nsHostKey *key)
     flags = key->flags;
     af = key->af;
 
-    NS_LOG_ADDREF(this, 1, "nsHostRecord", sizeof(nsHostRecord));
     expiration = NowInMinutes();
 
     PR_INIT_CLIST(this);
@@ -212,6 +179,7 @@ nsHostRecord::Create(const nsHostKey *key, nsHostRecord **result)
     // allocated after it.
     void *place = ::operator new(size);
     *result = new(place) nsHostRecord(key);
+    NS_ADDREF(*result);
     return NS_OK;
 }
 
@@ -225,24 +193,25 @@ bool
 nsHostRecord::Blacklisted(PRNetAddr *aQuery)
 {
     // must call locked
-    LOG(("nsHostRecord::Blacklisted() %p %s\n", this, host));
+    LOG(("Checking blacklist for host [%s], host record [%p].\n", host, this));
 
     // skip the string conversion for the common case of no blacklist
-    if (!mBlacklistedItems.Length())
-        return false;
-    
-    char buf[64];
-    if (PR_NetAddrToString(aQuery, buf, sizeof(buf)) != PR_SUCCESS)
+    if (!mBlacklistedItems.Length()) {
         return false;
+    }
 
+    char buf[64];
+    if (PR_NetAddrToString(aQuery, buf, sizeof(buf)) != PR_SUCCESS) {
+        return false;
+    }
     nsDependentCString strQuery(buf);
-    LOG(("nsHostRecord::Blacklisted() query %s\n", buf));
-    
-    for (PRUint32 i = 0; i < mBlacklistedItems.Length(); i++)
+
+    for (uint32_t i = 0; i < mBlacklistedItems.Length(); i++) {
         if (mBlacklistedItems.ElementAt(i).Equals(strQuery)) {
-            LOG(("nsHostRecord::Blacklisted() %s blacklist confirmed\n", buf));
+            LOG(("Address [%s] is blacklisted for host [%s].\n", buf, host));
             return true;
         }
+    }
 
     return false;
 }
@@ -251,11 +220,11 @@ void
 nsHostRecord::ReportUnusable(PRNetAddr *aAddress)
 {
     // must call locked
-    LOG(("nsHostRecord::ReportUnusable() %p %s\n", this, host));
+    LOG(("Adding address to blacklist for host [%s], host record [%p].\n", host, this));
 
     char buf[64];
     if (PR_NetAddrToString(aAddress, buf, sizeof(buf)) == PR_SUCCESS) {
-        LOG(("nsHostrecord::ReportUnusable addr %s\n",buf));
+        LOG(("Successfully adding address [%s] to blacklist for host [%s].\n", buf, host));
         mBlacklistedItems.AppendElement(nsCString(buf));
     }
 }
@@ -264,7 +233,7 @@ void
 nsHostRecord::ResetBlacklist()
 {
     // must call locked
-    LOG(("nsHostRecord::ResetBlacklist() %p %s\n", this, host));
+    LOG(("Resetting blacklist for host [%s], host record [%p].\n", host, this));
     mBlacklistedItems.Clear();
 }
 
@@ -308,18 +277,16 @@ static void
 HostDB_ClearEntry(PLDHashTable *table,
                   PLDHashEntryHdr *entry)
 {
-    LOG(("evicting record\n"));
     nsHostDBEnt *he = static_cast<nsHostDBEnt *>(entry);
+    LOG(("Clearing cache db entry for host [%s].\n", he->rec->host));
 #if defined(DEBUG) && defined(PR_LOGGING)
-    if (!he->rec->addr_info)
-        LOG(("%s: => no addr_info\n", he->rec->host));
-    else {
-        PRInt32 now = (PRInt32) NowInMinutes();
-        PRInt32 diff = (PRInt32) he->rec->expiration - now;
-        LOG(("%s: exp=%d => %s\n",
-            he->rec->host, diff,
-            PR_GetCanonNameFromAddrInfo(he->rec->addr_info)));
-        void *iter = nsnull;
+    if (!he->rec->addr_info) {
+        LOG(("No address info for host [%s].\n", he->rec->host));
+    } else {
+        int32_t now = (int32_t) NowInMinutes();
+        int32_t diff = (int32_t) he->rec->expiration - now;
+        LOG(("Record for [%s] expires in %d minute(s).\n", he->rec->host, diff));
+        void *iter = nullptr;
         PRNetAddr addr;
         char buf[64];
         for (;;) {
@@ -327,7 +294,7 @@ HostDB_ClearEntry(PLDHashTable *table,
             if (!iter)
                 break;
             PR_NetAddrToString(&addr, buf, sizeof(buf));
-            LOG(("  %s\n", buf));
+            LOG(("  [%s]\n", buf));
         }
     }
 #endif
@@ -359,7 +326,7 @@ static PLDHashTableOps gHostDB_ops =
 static PLDHashOperator
 HostDB_RemoveEntry(PLDHashTable *table,
                    PLDHashEntryHdr *hdr,
-                   PRUint32 number,
+                   uint32_t number,
                    void *arg)
 {
     return PL_DHASH_REMOVE;
@@ -367,9 +334,9 @@ HostDB_RemoveEntry(PLDHashTable *table,
 
 //----------------------------------------------------------------------------
 
-nsHostResolver::nsHostResolver(PRUint32 maxCacheEntries,
-                               PRUint32 maxCacheLifetime,
-                               PRUint32 lifetimeGracePeriod)
+nsHostResolver::nsHostResolver(uint32_t maxCacheEntries,
+                               uint32_t maxCacheLifetime,
+                               uint32_t lifetimeGracePeriod)
     : mMaxCacheEntries(maxCacheEntries)
     , mMaxCacheLifetime(maxCacheLifetime)
     , mGracePeriod(lifetimeGracePeriod)
@@ -402,7 +369,7 @@ nsHostResolver::Init()
 {
     NS_TIME_FUNCTION;
 
-    PL_DHashTableInit(&mDB, &gHostDB_ops, nsnull, sizeof(nsHostDBEnt), 0);
+    PL_DHashTableInit(&mDB, &gHostDB_ops, nullptr, sizeof(nsHostDBEnt), 0);
 
     mShutdown = false;
 
@@ -414,7 +381,7 @@ nsHostResolver::Init()
     // during application startup.
     static int initCount = 0;
     if (initCount++ > 0) {
-        LOG(("calling res_ninit\n"));
+        LOG(("Calling 'res_ninit'.\n"));
         res_ninit(&_res);
     }
 #endif
@@ -430,7 +397,7 @@ nsHostResolver::ClearPendingQueue(PRCList *aPendingQ)
         while (node != aPendingQ) {
             nsHostRecord *rec = static_cast<nsHostRecord *>(node);
             node = node->next;
-            OnLookupComplete(rec, NS_ERROR_ABORT, nsnull);
+            OnLookupComplete(rec, NS_ERROR_ABORT, nullptr);
         }
     }
 }
@@ -438,7 +405,7 @@ nsHostResolver::ClearPendingQueue(PRCList *aPendingQ)
 void
 nsHostResolver::Shutdown()
 {
-    LOG(("nsHostResolver::Shutdown\n"));
+    LOG(("Shutting down host resolver.\n"));
 
     PRCList pendingQHigh, pendingQMed, pendingQLow, evictionQ;
     PR_INIT_CLIST(&pendingQHigh);
@@ -462,7 +429,7 @@ nsHostResolver::Shutdown()
             mIdleThreadCV.NotifyAll();
         
         // empty host database
-        PL_DHashTableEnumerate(&mDB, HostDB_RemoveEntry, nsnull);
+        PL_DHashTableEnumerate(&mDB, HostDB_RemoveEntry, nullptr);
     }
     
     ClearPendingQueue(&pendingQHigh);
@@ -495,19 +462,19 @@ nsHostResolver::Shutdown()
 }
 
 static inline bool
-IsHighPriority(PRUint16 flags)
+IsHighPriority(uint16_t flags)
 {
     return !(flags & (nsHostResolver::RES_PRIORITY_LOW | nsHostResolver::RES_PRIORITY_MEDIUM));
 }
 
 static inline bool
-IsMediumPriority(PRUint16 flags)
+IsMediumPriority(uint16_t flags)
 {
     return flags & nsHostResolver::RES_PRIORITY_MEDIUM;
 }
 
 static inline bool
-IsLowPriority(PRUint16 flags)
+IsLowPriority(uint16_t flags)
 {
     return flags & nsHostResolver::RES_PRIORITY_LOW;
 }
@@ -523,13 +490,13 @@ nsHostResolver::MoveQueue(nsHostRecord *aRec, PRCList &aDestQ)
 
 nsresult
 nsHostResolver::ResolveHost(const char            *host,
-                            PRUint16               flags,
-                            PRUint16               af,
+                            uint16_t               flags,
+                            uint16_t               af,
                             nsResolveHostCallback *callback)
 {
     NS_ENSURE_TRUE(host && *host, NS_ERROR_UNEXPECTED);
 
-    LOG(("nsHostResolver::ResolveHost [host=%s]\n", host));
+    LOG(("Resolving host [%s].\n", host));
 
     // ensure that we are working with a valid hostname before proceeding.  see
     // bug 304904 for details.
@@ -571,30 +538,30 @@ nsHostResolver::ResolveHost(const char            *host,
                      he->rec->HasResult() &&
                      NowInMinutes() <= he->rec->expiration + mGracePeriod) {
                         
-                LOG(("using cached record\n"));
+                LOG(("Using cached record for host [%s].\n", host));
                 // put reference to host record on stack...
                 result = he->rec;
-                Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD, METHOD_HIT);
+                Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD2, METHOD_HIT);
 
                 // For entries that are in the grace period, or all cached
                 // negative entries, use the cache but start a new lookup in
                 // the background
                 if (((NowInMinutes() > he->rec->expiration) ||
                      he->rec->negative) && !he->rec->resolving) {
-                    LOG(("Using %s cache entry but starting async renewal",
-                         he->rec->negative ? "negative" :"positive"));
+                    LOG(("Using %s cache entry for host [%s] but starting async renewal.",
+                         he->rec->negative ? "negative" :"positive", host));
                     IssueLookup(he->rec);
 
                     if (!he->rec->negative) {
                         // negative entries are constantly being refreshed, only
                         // track positive grace period induced renewals
-                        Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD,
+                        Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD2,
                                               METHOD_RENEWAL);
                     }
                 }
                 
                 if (he->rec->negative) {
-                    Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD,
+                    Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD2,
                                           METHOD_NEGATIVE_HIT);
                     status = NS_ERROR_UNKNOWN_HOST;
                 }
@@ -602,7 +569,7 @@ nsHostResolver::ResolveHost(const char            *host,
             // if the host name is an IP address literal and has been parsed,
             // go ahead and use it.
             else if (he->rec->addr) {
-                Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD,
+                Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD2,
                                       METHOD_LITERAL);
                 result = he->rec;
             }
@@ -618,14 +585,14 @@ nsHostResolver::ResolveHost(const char            *host,
                 else
                     memcpy(he->rec->addr, &tempAddr, sizeof(PRNetAddr));
                 // put reference to host record on stack...
-                Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD,
+                Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD2,
                                       METHOD_LITERAL);
                 result = he->rec;
             }
             else if (mPendingCount >= MAX_NON_PRIORITY_REQUESTS &&
                      !IsHighPriority(flags) &&
                      !he->rec->resolving) {
-                Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD,
+                Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD2,
                                       METHOD_OVERFLOW);
                 // This is a lower priority request and we are swamped, so refuse it.
                 rv = NS_ERROR_DNS_LOOKUP_QUEUE_FULL;
@@ -638,15 +605,15 @@ nsHostResolver::ResolveHost(const char            *host,
                 if (!he->rec->resolving) {
                     he->rec->flags = flags;
                     rv = IssueLookup(he->rec);
-                    Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD,
+                    Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD2,
                                           METHOD_NETWORK_FIRST);
                     if (NS_FAILED(rv))
                         PR_REMOVE_AND_INIT_LINK(callback);
                     else
-                        LOG(("dns lookup blocking pending getaddrinfo query"));
+                        LOG(("DNS lookup for host [%s] blocking pending 'getaddrinfo' query.", host));
                 }
                 else if (he->rec->onQueue) {
-                    Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD,
+                    Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD2,
                                           METHOD_NETWORK_SHARED);
 
                     // Consider the case where we are on a pending queue of 
@@ -675,8 +642,8 @@ nsHostResolver::ResolveHost(const char            *host,
 
 void
 nsHostResolver::DetachCallback(const char            *host,
-                               PRUint16               flags,
-                               PRUint16               af,
+                               uint16_t               flags,
+                               uint16_t               af,
                                nsResolveHostCallback *callback,
                                nsresult               status)
 {
@@ -736,7 +703,7 @@ nsHostResolver::ConditionallyCreateThread(nsHostRecord *rec)
     }
 #if defined(PR_LOGGING)
     else
-      LOG(("lookup waiting for thread - %s ...\n", rec->host));
+      LOG(("Unable to find a thread for looking up host [%s].\n", rec->host));
 #endif
     return NS_OK;
 }
@@ -770,7 +737,7 @@ nsHostResolver::IssueLookup(nsHostRecord *rec)
 
     rv = ConditionallyCreateThread(rec);
     
-    LOG (("DNS Thread Counters: total=%d any-live=%d idle=%d pending=%d\n",
+    LOG (("DNS thread counters: total=%d any-live=%d idle=%d pending=%d\n",
           mThreadCount,
           mActiveAnyThreadCount,
           mNumIdleThreads,
@@ -910,7 +877,7 @@ nsHostResolver::OnLookupComplete(nsHostRecord *rec, nsresult status, PRAddrInfo
 
                 if (!head->negative) {
                     // record the age of the entry upon eviction.
-                    PRUint32 age =
+                    uint32_t age =
                         NowInMinutes() - (head->expiration - mMaxCacheLifetime);
                     Telemetry::Accumulate(Telemetry::DNS_CLEANUP_AGE, age);
                 }
@@ -937,8 +904,8 @@ nsHostResolver::OnLookupComplete(nsHostRecord *rec, nsresult status, PRAddrInfo
 
 void
 nsHostResolver::CancelAsyncRequest(const char            *host,
-                                   PRUint16               flags,
-                                   PRUint16               af,
+                                   uint16_t               flags,
+                                   uint16_t               af,
                                    nsIDNSListener        *aListener,
                                    nsresult               status)
 
@@ -984,7 +951,11 @@ nsHostResolver::CancelAsyncRequest(const char            *host,
 void
 nsHostResolver::ThreadFunc(void *arg)
 {
-    LOG(("nsHostResolver::ThreadFunc entering\n"));
+    LOG(("DNS lookup thread starting execution.\n"));
+
+    static nsThreadPoolNaming naming;
+    naming.SetThreadPoolName(NS_LITERAL_CSTRING("DNS Resolver"));
+
 #if defined(RES_RETRY_ON_FAILURE)
     nsResState rs;
 #endif
@@ -992,9 +963,9 @@ nsHostResolver::ThreadFunc(void *arg)
     nsHostRecord *rec;
     PRAddrInfo *ai;
     while (resolver->GetHostToLookup(&rec)) {
-        LOG(("resolving %s ...\n", rec->host));
+        LOG(("Calling getaddrinfo for host [%s].\n", rec->host));
 
-        PRIntn flags = PR_AI_ADDRCONFIG;
+        int flags = PR_AI_ADDRCONFIG;
         if (!(rec->flags & RES_CANON_NAME))
             flags |= PR_AI_NOCANONNAME;
 
@@ -1007,7 +978,7 @@ nsHostResolver::ThreadFunc(void *arg)
 #endif
 
         TimeDuration elapsed = TimeStamp::Now() - startTime;
-        PRUint32 millis = static_cast<PRUint32>(elapsed.ToMilliseconds());
+        uint32_t millis = static_cast<uint32_t>(elapsed.ToMilliseconds());
 
         // convert error code to nsresult.
         nsresult status;
@@ -1025,18 +996,18 @@ nsHostResolver::ThreadFunc(void *arg)
         }
         
         resolver->OnLookupComplete(rec, status, ai);
-        LOG(("lookup complete for %s ...\n", rec->host));
+        LOG(("Lookup completed for host [%s].\n", rec->host));
     }
     NS_RELEASE(resolver);
-    LOG(("nsHostResolver::ThreadFunc exiting\n"));
+    LOG(("DNS lookup thread ending execution.\n"));
 }
 
 //----------------------------------------------------------------------------
 
 nsresult
-nsHostResolver::Create(PRUint32         maxCacheEntries,
-                       PRUint32         maxCacheLifetime,
-                       PRUint32         lifetimeGracePeriod,
+nsHostResolver::Create(uint32_t         maxCacheEntries,
+                       uint32_t         maxCacheLifetime,
+                       uint32_t         lifetimeGracePeriod,
                        nsHostResolver **result)
 {
 #if defined(PR_LOGGING)
