@@ -1,40 +1,8 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* vim:set ts=2 sw=2 sts=2 et cindent: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla.
- *
- * The Initial Developer of the Original Code is IBM Corporation.
- * Portions created by IBM Corporation are Copyright (C) 2003
- * IBM Corporation. All Rights Reserved.
- *
- * Contributor(s):
- *   Darin Fisher <darin@meer.net>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include <stdlib.h>
 #include "nsScannerString.h"
@@ -50,10 +18,10 @@
 nsScannerBufferList::Buffer*
 nsScannerBufferList::AllocBufferFromString( const nsAString& aString )
   {
-    PRUint32 len = aString.Length();
+    uint32_t len = aString.Length();
 
     if (len > MAX_CAPACITY)
-      return nsnull;
+      return nullptr;
 
     Buffer* buf = (Buffer*) malloc(sizeof(Buffer) + (len + 1) * sizeof(PRUnichar));
     if (buf)
@@ -75,10 +43,10 @@ nsScannerBufferList::AllocBufferFromString( const nsAString& aString )
   }
 
 nsScannerBufferList::Buffer*
-nsScannerBufferList::AllocBuffer( PRUint32 capacity )
+nsScannerBufferList::AllocBuffer( uint32_t capacity )
   {
     if (capacity > MAX_CAPACITY)
-      return nsnull;
+      return nullptr;
 
     Buffer* buf = (Buffer*) malloc(sizeof(Buffer) + (capacity + 1) * sizeof(PRUnichar));
     if (buf)
@@ -116,12 +84,12 @@ nsScannerBufferList::SplitBuffer( const Position& pos )
     Buffer* bufferToSplit = pos.mBuffer;
     NS_ASSERTION(bufferToSplit, "null pointer");
 
-    PRUint32 splitOffset = pos.mPosition - bufferToSplit->DataStart();
+    uint32_t splitOffset = pos.mPosition - bufferToSplit->DataStart();
     NS_ASSERTION(pos.mPosition >= bufferToSplit->DataStart() &&
                  splitOffset <= bufferToSplit->DataLength(),
                  "split offset is outside buffer");
     
-    PRUint32 len = bufferToSplit->DataLength() - splitOffset;
+    uint32_t len = bufferToSplit->DataLength() - splitOffset;
     Buffer* new_buffer = AllocBuffer(len);
     if (new_buffer)
       {
@@ -171,16 +139,16 @@ nsScannerBufferList::Position::Distance( const Position& aStart, const Position&
  */
 
 nsScannerSubstring::nsScannerSubstring()
-  : mStart(nsnull, nsnull)
-  , mEnd(nsnull, nsnull)
-  , mBufferList(nsnull)
+  : mStart(nullptr, nullptr)
+  , mEnd(nullptr, nullptr)
+  , mBufferList(nullptr)
   , mLength(0)
   , mIsDirty(true)
   {
   }
 
 nsScannerSubstring::nsScannerSubstring( const nsAString& s )
-  : mBufferList(nsnull)
+  : mBufferList(nullptr)
   , mIsDirty(true)
   {
     Rebind(s);
@@ -191,7 +159,7 @@ nsScannerSubstring::~nsScannerSubstring()
     release_ownership_of_buffer_list();
   }
 
-PRInt32
+int32_t
 nsScannerSubstring::CountChar( PRUnichar c ) const
   {
       /*
@@ -204,7 +172,7 @@ nsScannerSubstring::CountChar( PRUnichar c ) const
     nsScannerIterator iter;
     for ( BeginReading(iter); ; )
       {
-        PRInt32 lengthToExamineInThisFragment = iter.size_forward();
+        int32_t lengthToExamineInThisFragment = iter.size_forward();
         const PRUnichar* fromBegin = iter.get();
         result += size_type(NS_COUNT(fromBegin, fromBegin+lengthToExamineInThisFragment, c));
         if ( !(lengthToExamine -= lengthToExamineInThisFragment) )
@@ -461,8 +429,8 @@ nsScannerSharedSubstring::Rebind(const nsScannerIterator &aStart,
     mBufferList = bufferList;
     mString.Rebind(aStart.mPosition, aEnd.mPosition);
   } else {
-    mBuffer = nsnull;
-    mBufferList = nsnull;
+    mBuffer = nullptr;
+    mBufferList = nullptr;
     CopyUnicodeTo(aStart, aEnd, mString);
   }
 }
@@ -484,8 +452,8 @@ nsScannerSharedSubstring::MakeMutable()
 
   ReleaseBuffer();
 
-  mBuffer = nsnull;
-  mBufferList = nsnull;
+  mBuffer = nullptr;
+  mBufferList = nullptr;
 }
 
   /**
@@ -502,7 +470,7 @@ copy_multifragment_string( nsScannerIterator& first, const nsScannerIterator& la
 
     while ( first != last )
       {
-        PRUint32 distance = source_traits::readable_distance(first, last);
+        uint32_t distance = source_traits::readable_distance(first, last);
         sink_traits::write(result, source_traits::read(first), distance);
         NS_ASSERTION(distance > 0, "|copy_multifragment_string| will never terminate");
         source_traits::advance(first, distance);
@@ -549,7 +517,7 @@ AppendUnicodeTo( const nsScannerIterator& aSrcStart,
                  nsAString& aDest )
   {
     nsAString::iterator writer;
-    PRUint32 oldLength = aDest.Length();
+    uint32_t oldLength = aDest.Length();
     if (!EnsureStringLength(aDest, oldLength + Distance(aSrcStart, aSrcEnd)))
       return; // out of memory
     aDest.BeginWriting(writer).advance(oldLength);
@@ -565,7 +533,7 @@ FindCharInReadable( PRUnichar aChar,
   {
     while ( aSearchStart != aSearchEnd )
       {
-        PRInt32 fragmentLength;
+        int32_t fragmentLength;
         if ( SameFragment(aSearchStart, aSearchEnd) ) 
           fragmentLength = aSearchEnd.get() - aSearchStart.get();
         else
