@@ -1,44 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2002
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Conrad Carlen <ccarlen@netscape.com>
- *   Brendan Eich <brendan@mozilla.org>
- *   Colin Blake <colin@theblakes.com>
- *   Javier Pedemonte <pedemont@us.ibm.com>
- *   Mats Palmgren <mats.palmgren@bredband.net>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsProfileStringTypes.h"
 #include "nsProfileLock.h"
@@ -83,7 +46,7 @@ nsProfileLock::nsProfileLock() :
 #elif defined (XP_OS2)
     ,mLockFileHandle(-1)
 #elif defined (XP_UNIX)
-    ,mPidLockFileName(nsnull)
+    ,mPidLockFileName(nullptr)
     ,mLockFileDesc(-1)
 #endif
 {
@@ -117,7 +80,7 @@ nsProfileLock& nsProfileLock::operator=(nsProfileLock& rhs)
     mLockFileDesc = rhs.mLockFileDesc;
     rhs.mLockFileDesc = -1;
     mPidLockFileName = rhs.mPidLockFileName;
-    rhs.mPidLockFileName = nsnull;
+    rhs.mPidLockFileName = nullptr;
     if (mPidLockFileName)
     {
         // rhs had a symlink lock, therefore it was on the list.
@@ -170,7 +133,7 @@ void nsProfileLock::FatalSignalHandler(int signo
     RemovePidLockFiles(true);
 
     // Chain to the old handler, which may exit.
-    struct sigaction *oldact = nsnull;
+    struct sigaction *oldact = nullptr;
 
     switch (signo) {
       case SIGHUP:
@@ -232,7 +195,7 @@ void nsProfileLock::FatalSignalHandler(int signo
     _exit(signo);
 }
 
-nsresult nsProfileLock::LockWithFcntl(nsILocalFile *aLockFile)
+nsresult nsProfileLock::LockWithFcntl(nsIFile *aLockFile)
 {
     nsresult rv = NS_OK;
 
@@ -315,7 +278,7 @@ static bool IsSymlinkStaleLock(struct in_addr* aAddr, const char* aFileName,
                     return true;
                 }
                     
-                char *after = nsnull;
+                char *after = nullptr;
                 pid_t pid = strtol(colon, &after, 0);
                 if (pid != 0 && *after == '\0')
                 {
@@ -342,7 +305,7 @@ static bool IsSymlinkStaleLock(struct in_addr* aAddr, const char* aFileName,
     return true;
 }
 
-nsresult nsProfileLock::LockWithSymlink(nsILocalFile *aLockFile, bool aHaveFcntlLock)
+nsresult nsProfileLock::LockWithSymlink(nsIFile *aLockFile, bool aHaveFcntlLock)
 {
     nsresult rv;
     nsCAutoString lockFilePath;
@@ -395,7 +358,7 @@ nsresult nsProfileLock::LockWithSymlink(nsILocalFile *aLockFile, bool aHaveFcntl
     }
 
     PR_smprintf_free(signature);
-    signature = nsnull;
+    signature = nullptr;
 
     if (symlink_rv == 0)
     {
@@ -463,12 +426,12 @@ PR_BEGIN_MACRO                                                          \
 }
 #endif /* XP_UNIX */
 
-nsresult nsProfileLock::GetReplacedLockTime(PRInt64 *aResult) {
+nsresult nsProfileLock::GetReplacedLockTime(PRTime *aResult) {
     *aResult = mReplacedLockTime;
     return NS_OK;
 }
 
-nsresult nsProfileLock::Lock(nsILocalFile* aProfileDir,
+nsresult nsProfileLock::Lock(nsIFile* aProfileDir,
                              nsIProfileUnlocker* *aUnlocker)
 {
 #if defined (XP_MACOSX)
@@ -483,7 +446,7 @@ nsresult nsProfileLock::Lock(nsILocalFile* aProfileDir,
 
     nsresult rv;
     if (aUnlocker)
-        *aUnlocker = nsnull;
+        *aUnlocker = nullptr;
 
     NS_ENSURE_STATE(!mHaveLock);
 
@@ -494,8 +457,8 @@ nsresult nsProfileLock::Lock(nsILocalFile* aProfileDir,
     if (!isDir)
         return NS_ERROR_FILE_NOT_DIRECTORY;
 
-    nsCOMPtr<nsILocalFile> lockFile;
-    rv = aProfileDir->Clone((nsIFile **)((void **)getter_AddRefs(lockFile)));
+    nsCOMPtr<nsIFile> lockFile;
+    rv = aProfileDir->Clone(getter_AddRefs(lockFile));
     if (NS_FAILED(rv))
         return rv;
 
@@ -527,8 +490,8 @@ nsresult nsProfileLock::Lock(nsILocalFile* aProfileDir,
             unsigned long launchDate;
         };
 
-        PRFileDesc *fd = nsnull;
-        PRInt32 ioBytes;
+        PRFileDesc *fd = nullptr;
+        int32_t ioBytes;
         ProcessInfoRec processInfo;
         LockProcessInfo lockProcessInfo;
 
@@ -565,8 +528,8 @@ nsresult nsProfileLock::Lock(nsILocalFile* aProfileDir,
     }
 #elif defined(XP_UNIX)
     // Get the old lockfile name
-    nsCOMPtr<nsILocalFile> oldLockFile;
-    rv = aProfileDir->Clone((nsIFile **)((void **)getter_AddRefs(oldLockFile)));
+    nsCOMPtr<nsIFile> oldLockFile;
+    rv = aProfileDir->Clone(getter_AddRefs(oldLockFile));
     if (NS_FAILED(rv))
         return rv;
     rv = oldLockFile->Append(OLD_LOCKFILE_NAME);
@@ -614,10 +577,10 @@ nsresult nsProfileLock::Lock(nsILocalFile* aProfileDir,
     mLockFileHandle = CreateFileW(filePath.get(),
                                   GENERIC_READ | GENERIC_WRITE,
                                   0, // no sharing - of course
-                                  nsnull,
+                                  nullptr,
                                   CREATE_ALWAYS,
-                                  nsnull,
-                                  nsnull);
+                                  0,
+                                  nullptr);
     if (mLockFileHandle == INVALID_HANDLE_VALUE) {
         // XXXbsmedberg: provide a profile-unlocker here!
         return NS_ERROR_FILE_ACCESS_DENIED;
@@ -705,7 +668,7 @@ nsresult nsProfileLock::Unlock(bool aFatalSignal)
             // holding this lock, so we'll deadlock. See bug 522332.
             if (!aFatalSignal)
                 free(mPidLockFileName);
-            mPidLockFileName = nsnull;
+            mPidLockFileName = nullptr;
         }
         else if (mLockFileDesc != -1)
         {
