@@ -1,40 +1,7 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Pierre Phaneuf <pp@ludusdesign.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /*
 
@@ -172,7 +139,7 @@ public:
     NS_IMETHOD SetParser(nsParserBase* aParser);
     virtual void FlushPendingNotifications(mozFlushType aType) { }
     NS_IMETHOD SetDocumentCharset(nsACString& aCharset) { return NS_OK; }
-    virtual nsISupports *GetTarget() { return nsnull; }
+    virtual nsISupports *GetTarget() { return nullptr; }
 
     // nsIRDFContentSink
     NS_IMETHOD Init(nsIURI* aURL);
@@ -180,7 +147,7 @@ public:
     NS_IMETHOD GetDataSource(nsIRDFDataSource*& aDataSource);
 
     // pseudo constants
-    static PRInt32 gRefCnt;
+    static int32_t gRefCnt;
     static nsIRDFService* gRDFService;
     static nsIRDFContainerUtils* gRDFContainerUtils;
     static nsIRDFResource* kRDF_type;
@@ -205,7 +172,7 @@ protected:
     void ParseText(nsIRDFNode **aResult);
 
     nsresult FlushText();
-    nsresult AddText(const PRUnichar* aText, PRInt32 aLength);
+    nsresult AddText(const PRUnichar* aText, int32_t aLength);
 
     // RDF-specific parsing
     nsresult OpenRDF(const PRUnichar* aName);
@@ -214,14 +181,14 @@ protected:
     nsresult OpenMember(const PRUnichar* aName, const PRUnichar** aAttributes);
     nsresult OpenValue(const PRUnichar* aName, const PRUnichar** aAttributes);
     
-    nsresult GetIdAboutAttribute(const PRUnichar** aAttributes, nsIRDFResource** aResource, bool* aIsAnonymous = nsnull);
+    nsresult GetIdAboutAttribute(const PRUnichar** aAttributes, nsIRDFResource** aResource, bool* aIsAnonymous = nullptr);
     nsresult GetResourceAttribute(const PRUnichar** aAttributes, nsIRDFResource** aResource);
-    nsresult AddProperties(const PRUnichar** aAttributes, nsIRDFResource* aSubject, PRInt32* aCount = nsnull);
+    nsresult AddProperties(const PRUnichar** aAttributes, nsIRDFResource* aSubject, int32_t* aCount = nullptr);
     void SetParseMode(const PRUnichar **aAttributes);
 
     PRUnichar* mText;
-    PRInt32 mTextLength;
-    PRInt32 mTextSize;
+    int32_t mTextLength;
+    int32_t mTextSize;
 
     /**
      * From the set of given attributes, this method extracts the 
@@ -256,7 +223,7 @@ protected:
     RDFContentSinkParseMode mParseMode;
 
     // content stack management
-    PRInt32         
+    int32_t         
     PushContext(nsIRDFResource *aContext,
                 RDFContentSinkState aState,
                 RDFContentSinkParseMode aParseMode);
@@ -266,7 +233,7 @@ protected:
                RDFContentSinkState     &aState,
                RDFContentSinkParseMode &aParseMode);
 
-    nsIRDFResource* GetContextElement(PRInt32 ancestor = 0);
+    nsIRDFResource* GetContextElement(int32_t ancestor = 0);
 
 
     struct RDFContextStackElement {
@@ -280,7 +247,7 @@ protected:
     nsIURI*      mDocumentURL;
 };
 
-PRInt32         RDFContentSinkImpl::gRefCnt = 0;
+int32_t         RDFContentSinkImpl::gRefCnt = 0;
 nsIRDFService*  RDFContentSinkImpl::gRDFService;
 nsIRDFContainerUtils* RDFContentSinkImpl::gRDFContainerUtils;
 nsIRDFResource* RDFContentSinkImpl::kRDF_type;
@@ -307,13 +274,13 @@ static const nsStaticAtom rdf_atoms[] = {
 };
 
 RDFContentSinkImpl::RDFContentSinkImpl()
-    : mText(nsnull),
+    : mText(nullptr),
       mTextLength(0),
       mTextSize(0),
       mState(eRDFContentSinkState_InProlog),
       mParseMode(eRDFContentSinkParseMode_Literal),
-      mContextStack(nsnull),
-      mDocumentURL(nsnull)
+      mContextStack(nullptr),
+      mDocumentURL(nullptr)
 {
     if (gRefCnt++ == 0) {
         nsresult rv = CallGetService(kRDFServiceCID, &gRDFService);
@@ -365,9 +332,9 @@ RDFContentSinkImpl::~RDFContentSinkImpl()
         // XXX we should never need to do this, but, we'll write the
         // code all the same. If someone left the content stack dirty,
         // pop all the elements off the stack and release them.
-        PRInt32 i = mContextStack->Length();
+        int32_t i = mContextStack->Length();
         while (0 < i--) {
-            nsIRDFResource* resource = nsnull;
+            nsIRDFResource* resource = nullptr;
             RDFContentSinkState state;
             RDFContentSinkParseMode parseMode;
             PopContext(resource, state, parseMode);
@@ -417,7 +384,7 @@ RDFContentSinkImpl::QueryInterface(REFNSIID iid, void** result)
     if (! result)
         return NS_ERROR_NULL_POINTER;
 
-    *result = nsnull;
+    *result = nullptr;
     if (iid.Equals(kIRDFContentSinkIID) ||
         iid.Equals(kIXMLContentSinkIID) ||
         iid.Equals(kIContentSinkIID) ||
@@ -437,9 +404,9 @@ RDFContentSinkImpl::QueryInterface(REFNSIID iid, void** result)
 NS_IMETHODIMP 
 RDFContentSinkImpl::HandleStartElement(const PRUnichar *aName, 
                                        const PRUnichar **aAtts, 
-                                       PRUint32 aAttsCount, 
-                                       PRInt32 aIndex, 
-                                       PRUint32 aLineNumber)
+                                       uint32_t aAttsCount, 
+                                       int32_t aIndex, 
+                                       uint32_t aLineNumber)
 {
   FlushText();
 
@@ -538,7 +505,7 @@ RDFContentSinkImpl::HandleComment(const PRUnichar *aName)
 
 NS_IMETHODIMP 
 RDFContentSinkImpl::HandleCDataSection(const PRUnichar *aData, 
-                                       PRUint32 aLength)
+                                       uint32_t aLength)
 {
   return aData ?  AddText(aData, aLength) : NS_OK;
 }
@@ -555,7 +522,7 @@ RDFContentSinkImpl::HandleDoctypeDecl(const nsAString & aSubset,
 
 NS_IMETHODIMP 
 RDFContentSinkImpl::HandleCharacterData(const PRUnichar *aData, 
-                                        PRUint32 aLength)
+                                        uint32_t aLength)
 {
   return aData ?  AddText(aData, aLength) : NS_OK;
 }
@@ -570,7 +537,7 @@ RDFContentSinkImpl::HandleProcessingInstruction(const PRUnichar *aTarget,
 NS_IMETHODIMP 
 RDFContentSinkImpl::HandleXMLDeclaration(const PRUnichar *aVersion,
                                          const PRUnichar *aEncoding,
-                                         PRInt32 aStandalone)
+                                         int32_t aStandalone)
 {
     return NS_OK;
 }
@@ -654,7 +621,7 @@ RDFContentSinkImpl::SetParser(nsParserBase* aParser)
 NS_IMETHODIMP
 RDFContentSinkImpl::Init(nsIURI* aURL)
 {
-    NS_PRECONDITION(aURL != nsnull, "null ptr");
+    NS_PRECONDITION(aURL != nullptr, "null ptr");
     if (! aURL)
         return NS_ERROR_NULL_POINTER;
 
@@ -668,9 +635,9 @@ RDFContentSinkImpl::Init(nsIURI* aURL)
 NS_IMETHODIMP
 RDFContentSinkImpl::SetDataSource(nsIRDFDataSource* aDataSource)
 {
-    NS_PRECONDITION(aDataSource != nsnull, "SetDataSource null ptr");
+    NS_PRECONDITION(aDataSource != nullptr, "SetDataSource null ptr");
     mDataSource = aDataSource;
-    NS_ASSERTION(mDataSource != nsnull,"Couldn't QI RDF DataSource");
+    NS_ASSERTION(mDataSource != nullptr,"Couldn't QI RDF DataSource");
     return NS_OK;
 }
 
@@ -687,9 +654,9 @@ RDFContentSinkImpl::GetDataSource(nsIRDFDataSource*& aDataSource)
 // Text buffering
 
 static bool
-rdf_IsDataInBuffer(PRUnichar* buffer, PRInt32 length)
+rdf_IsDataInBuffer(PRUnichar* buffer, int32_t length)
 {
-    for (PRInt32 i = 0; i < length; ++i) {
+    for (int32_t i = 0; i < length; ++i) {
         if (buffer[i] == ' ' ||
             buffer[i] == '\t' ||
             buffer[i] == '\n' ||
@@ -729,8 +696,8 @@ RDFContentSinkImpl::ParseText(nsIRDFNode **aResult)
 
     case eRDFContentSinkParseMode_Int:
         {
-            PRInt32 i, err;
-            i = value.ToInteger(&err);
+            nsresult err;
+            int32_t i = value.ToInteger(&err);
             nsIRDFInt *result;
             gRDFService->GetIntLiteral(i, &result);
             *aResult = result;
@@ -792,7 +759,7 @@ RDFContentSinkImpl::FlushText()
 
 
 nsresult
-RDFContentSinkImpl::AddText(const PRUnichar* aText, PRInt32 aLength)
+RDFContentSinkImpl::AddText(const PRUnichar* aText, int32_t aLength)
 {
     // Create buffer when we first need it
     if (0 == mTextSize) {
@@ -806,12 +773,12 @@ RDFContentSinkImpl::AddText(const PRUnichar* aText, PRInt32 aLength)
     // Copy data from string into our buffer; grow the buffer as needed.
     // It never shrinks, but since the content sink doesn't stick around,
     // this shouldn't be a bloat issue.
-    PRInt32 amount = mTextSize - mTextLength;
+    int32_t amount = mTextSize - mTextLength;
     if (amount < aLength) {
         // Grow the buffer by at least a factor of two to prevent thrashing.
         // Since PR_REALLOC will leave mText intact if the call fails,
         // don't clobber mText or mTextSize until the new mem is allocated.
-        PRInt32 newSize = (2 * mTextSize > (mTextSize + aLength)) ?
+        int32_t newSize = (2 * mTextSize > (mTextSize + aLength)) ?
                           (2 * mTextSize) : (mTextSize + aLength);
         PRUnichar* newText = 
             (PRUnichar *) PR_REALLOC(mText, sizeof(PRUnichar) * newSize);
@@ -995,7 +962,7 @@ RDFContentSinkImpl::GetResourceAttribute(const PRUnichar** aAttributes,
 nsresult
 RDFContentSinkImpl::AddProperties(const PRUnichar** aAttributes,
                                   nsIRDFResource* aSubject,
-                                  PRInt32* aCount)
+                                  int32_t* aCount)
 {
   if (aCount)
       *aCount = 0;
@@ -1097,7 +1064,7 @@ RDFContentSinkImpl::OpenRDF(const PRUnichar* aName)
         return NS_ERROR_UNEXPECTED;
     }
 
-    PushContext(nsnull, mState, mParseMode);
+    PushContext(nullptr, mState, mParseMode);
     mState = eRDFContentSinkState_InDocumentElement;
     return NS_OK;
 }
@@ -1220,7 +1187,7 @@ RDFContentSinkImpl::OpenProperty(const PRUnichar* aName, const PRUnichar** aAttr
         // property. Create an RDF resource for the inline resource
         // URI, add the properties to it, and attach the inline
         // resource to its parent.
-        PRInt32 count;
+        int32_t count;
         rv = AddProperties(aAttributes, target, &count);
         NS_ASSERTION(NS_SUCCEEDED(rv), "problem adding properties");
         if (NS_FAILED(rv)) return rv;
@@ -1297,7 +1264,7 @@ RDFContentSinkImpl::OpenMember(const PRUnichar* aName,
     // The contained element will use nsIRDFContainer::AppendElement() to add
     // the element to the container, which requires only the container
     // and the element to be added.
-    PushContext(nsnull, mState, mParseMode);
+    PushContext(nullptr, mState, mParseMode);
     mState = eRDFContentSinkState_InMemberElement;
     SetParseMode(aAttributes);
 
@@ -1343,7 +1310,7 @@ RDFContentSinkImpl::RegisterNamespaces(const PRUnichar **aAttributes)
         nsDependentSubstring lname(attr, endLocal);
         nsCOMPtr<nsIAtom> preferred = do_GetAtom(lname);
         if (preferred == kXMLNSAtom) {
-            preferred = nsnull;
+            preferred = nullptr;
         }
         sink->AddNameSpace(preferred, nsDependentString(aAttributes[1]));
     }
@@ -1410,7 +1377,7 @@ RDFContentSinkImpl::InitContainer(nsIRDFResource* aContainerType, nsIRDFResource
             rv = ReinitContainer(aContainerType, aContainer);
         }
         else {
-            rv = (gRDFContainerUtils->*(info->mMakeFn))(mDataSource, aContainer, nsnull);
+            rv = (gRDFContainerUtils->*(info->mMakeFn))(mDataSource, aContainer, nullptr);
         }
         return rv;
     }
@@ -1455,18 +1422,18 @@ RDFContentSinkImpl::ReinitContainer(nsIRDFResource* aContainerType, nsIRDFResour
 // Content stack management
 
 nsIRDFResource* 
-RDFContentSinkImpl::GetContextElement(PRInt32 ancestor /* = 0 */)
+RDFContentSinkImpl::GetContextElement(int32_t ancestor /* = 0 */)
 {
-    if ((nsnull == mContextStack) ||
-        (PRUint32(ancestor) >= mContextStack->Length())) {
-        return nsnull;
+    if ((nullptr == mContextStack) ||
+        (uint32_t(ancestor) >= mContextStack->Length())) {
+        return nullptr;
     }
 
     return mContextStack->ElementAt(
            mContextStack->Length()-ancestor-1).mResource;
 }
 
-PRInt32 
+int32_t 
 RDFContentSinkImpl::PushContext(nsIRDFResource         *aResource,
                                 RDFContentSinkState     aState,
                                 RDFContentSinkParseMode aParseMode)
@@ -1493,12 +1460,12 @@ RDFContentSinkImpl::PopContext(nsIRDFResource         *&aResource,
                                RDFContentSinkState     &aState,
                                RDFContentSinkParseMode &aParseMode)
 {
-    if ((nsnull == mContextStack) ||
+    if ((nullptr == mContextStack) ||
         (mContextStack->IsEmpty())) {
         return NS_ERROR_NULL_POINTER;
     }
 
-    PRUint32 i = mContextStack->Length() - 1;
+    uint32_t i = mContextStack->Length() - 1;
     RDFContextStackElement &e = mContextStack->ElementAt(i);
 
     aResource  = e.mResource;
@@ -1516,7 +1483,7 @@ RDFContentSinkImpl::PopContext(nsIRDFResource         *&aResource,
 nsresult
 NS_NewRDFContentSink(nsIRDFContentSink** aResult)
 {
-    NS_PRECONDITION(aResult != nsnull, "null ptr");
+    NS_PRECONDITION(aResult != nullptr, "null ptr");
     if (! aResult)
         return NS_ERROR_NULL_POINTER;
 
