@@ -1,43 +1,9 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
  * vim: set ts=4 sw=4 et tw=80:
  *
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org Code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1999
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Chris Waterson <waterson@netscape.com>
- *   Axel Hecht <axel@pike.org>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsRDFXMLSerializer.h"
 
@@ -51,10 +17,11 @@
 #include "nsTArray.h"
 #include "rdf.h"
 #include "rdfutil.h"
+#include "mozilla/Attributes.h"
 
 #include "rdfIDataSource.h"
 
-PRInt32 nsRDFXMLSerializer::gRefCnt = 0;
+int32_t nsRDFXMLSerializer::gRefCnt = 0;
 nsIRDFContainerUtils* nsRDFXMLSerializer::gRDFC;
 nsIRDFResource* nsRDFXMLSerializer::kRDF_instanceOf;
 nsIRDFResource* nsRDFXMLSerializer::kRDF_type;
@@ -186,13 +153,13 @@ nsRDFXMLSerializer::AddNameSpace(nsIAtom* aPrefix, const nsAString& aURI)
 }
 
 static nsresult
-rdf_BlockingWrite(nsIOutputStream* stream, const char* buf, PRUint32 size)
+rdf_BlockingWrite(nsIOutputStream* stream, const char* buf, uint32_t size)
 {
-    PRUint32 written = 0;
-    PRUint32 remaining = size;
+    uint32_t written = 0;
+    uint32_t remaining = size;
     while (remaining > 0) {
         nsresult rv;
-        PRUint32 cb;
+        uint32_t cb;
 
         if (NS_FAILED(rv = stream->Write(buf + written, remaining, &cb)))
             return rv;
@@ -233,7 +200,7 @@ nsRDFXMLSerializer::EnsureNewPrefix()
             ++iter;
         } 
     } while (!isNewPrefix);
-    nsIAtom* outPrefix = nsnull;
+    nsIAtom* outPrefix = nullptr;
     prefix.swap(outPrefix);
     return outPrefix;
 }
@@ -254,18 +221,20 @@ nsRDFXMLSerializer::RegisterQName(nsIRDFResource* aResource)
         iter->mPrefix->ToUTF8String(qname);
         qname.Append(':');
         qname += StringTail(uri, uri.Length() - iter->mURI.Length());
-        return mQNames.Put(aResource, qname) ? NS_OK : NS_ERROR_FAILURE;
+        mQNames.Put(aResource, qname);
+        return NS_OK;
     }
 
     // Okay, so we don't have it in our map. Try to make one up. This
     // is very bogus.
-    PRInt32 i = uri.RFindChar('#'); // first try a '#'
+    int32_t i = uri.RFindChar('#'); // first try a '#'
     if (i == -1) {
         i = uri.RFindChar('/');
         if (i == -1) {
             // Okay, just punt and assume there is _no_ namespace on
             // this thing...
-            return mQNames.Put(aResource, uri) ? NS_OK : NS_ERROR_FAILURE;
+            mQNames.Put(aResource, uri);
+            return NS_OK;
         }
     }
 
@@ -277,7 +246,8 @@ nsRDFXMLSerializer::RegisterQName(nsIRDFResource* aResource)
     qname.Append(':');
     qname += StringTail(uri, uri.Length() - (i + 1));
 
-    return mQNames.Put(aResource, qname) ? NS_OK : NS_ERROR_FAILURE;
+    mQNames.Put(aResource, qname);
+    return NS_OK;
 }
 
 nsresult
@@ -315,7 +285,7 @@ static const char quot[] = "&quot;";
 static void
 rdf_EscapeAmpersandsAndAngleBrackets(nsCString& s)
 {
-    PRUint32 newLength, origLength;
+    uint32_t newLength, origLength;
     newLength = origLength = s.Length();
 
     // Compute the length of the result string.
@@ -374,7 +344,7 @@ rdf_EscapeAmpersandsAndAngleBrackets(nsCString& s)
 static void
 rdf_EscapeQuotes(nsCString& s)
 {
-    PRInt32 i = 0;
+    int32_t i = 0;
     while ((i = s.FindChar('"', i)) != -1) {
         s.Replace(i, 1, quot, sizeof(quot) - 1);
         i += sizeof(quot) - 2;
@@ -438,7 +408,7 @@ nsRDFXMLSerializer::SerializeChildAssertion(nsIOutputStream* aStream,
     nsCOMPtr<nsIRDFInt> number;
     nsCOMPtr<nsIRDFDate> date;
 
-    if ((resource = do_QueryInterface(aValue)) != nsnull) {
+    if ((resource = do_QueryInterface(aValue)) != nullptr) {
         nsCAutoString uri;
         resource->GetValueUTF8(uri);
 
@@ -456,7 +426,7 @@ nsRDFXMLSerializer::SerializeChildAssertion(nsIOutputStream* aStream,
 
         goto no_close_tag;
     }
-    else if ((literal = do_QueryInterface(aValue)) != nsnull) {
+    else if ((literal = do_QueryInterface(aValue)) != nullptr) {
         const PRUnichar *value;
         literal->GetValueConst(&value);
         NS_ConvertUTF16toUTF8 s(value);
@@ -468,8 +438,8 @@ nsRDFXMLSerializer::SerializeChildAssertion(nsIOutputStream* aStream,
         rv = rdf_BlockingWrite(aStream, s);
         if (NS_FAILED(rv)) return rv;
     }
-    else if ((number = do_QueryInterface(aValue)) != nsnull) {
-        PRInt32 value;
+    else if ((number = do_QueryInterface(aValue)) != nullptr) {
+        int32_t value;
         number->GetValue(&value);
 
         nsCAutoString n;
@@ -481,7 +451,7 @@ nsRDFXMLSerializer::SerializeChildAssertion(nsIOutputStream* aStream,
         rv = rdf_BlockingWrite(aStream, n);
         if (NS_FAILED(rv)) return rv;
     }
-    else if ((date = do_QueryInterface(aValue)) != nsnull) {
+    else if ((date = do_QueryInterface(aValue)) != nullptr) {
         PRTime value;
         date->GetValue(&value);
 
@@ -518,11 +488,11 @@ nsRDFXMLSerializer::SerializeProperty(nsIOutputStream* aStream,
                                       nsIRDFResource* aResource,
                                       nsIRDFResource* aProperty,
                                       bool aInline,
-                                      PRInt32* aSkipped)
+                                      int32_t* aSkipped)
 {
     nsresult rv = NS_OK;
 
-    PRInt32 skipped = 0;
+    int32_t skipped = 0;
 
     nsCOMPtr<nsISimpleEnumerator> assertions;
     mDataSource->GetTargets(aResource, aProperty, true, getter_AddRefs(assertions));
@@ -548,7 +518,7 @@ nsRDFXMLSerializer::SerializeProperty(nsIOutputStream* aStream,
         if (!needsChild) {
             assertions->HasMoreElements(&needsChild);
             if (!needsChild) {
-                const PRUnichar* literalVal = nsnull;
+                const PRUnichar* literalVal = nullptr;
                 literal->GetValueConst(&literalVal);
                 if (literalVal) {
                     for (; *literalVal; literalVal++) {
@@ -641,7 +611,7 @@ nsRDFXMLSerializer::SerializeDescription(nsIOutputStream* aStream,
     // Any value that's a literal we can write out as an inline
     // attribute on the RDF:Description
     nsAutoTArray<nsIRDFResource*, 8> visited;
-    PRInt32 skipped = 0;
+    int32_t skipped = 0;
 
     nsCOMPtr<nsISimpleEnumerator> arcs;
     mDataSource->ArcLabelsOut(aResource, getter_AddRefs(arcs));
@@ -772,7 +742,7 @@ static const char kRDFLIOpen[] = "    <RDF:li";
                                     sizeof(kRDFLIOpen) - 1); 
     if (NS_FAILED(rv)) return rv;
 
-    if ((resource = do_QueryInterface(aMember)) != nsnull) {
+    if ((resource = do_QueryInterface(aMember)) != nullptr) {
         nsCAutoString uri;
         resource->GetValueUTF8(uri);
 
@@ -790,7 +760,7 @@ static const char kRDFLIOpen[] = "    <RDF:li";
 
         goto no_close_tag;
     }
-    else if ((literal = do_QueryInterface(aMember)) != nsnull) {
+    else if ((literal = do_QueryInterface(aMember)) != nullptr) {
         const PRUnichar *value;
         literal->GetValueConst(&value);
 static const char kRDFLIOpenGT[] = ">";
@@ -805,8 +775,8 @@ static const char kRDFLIOpenGT[] = ">";
         rv = rdf_BlockingWrite(aStream, s);
         if (NS_FAILED(rv)) return rv;
     }
-    else if ((number = do_QueryInterface(aMember)) != nsnull) {
-        PRInt32 value;
+    else if ((number = do_QueryInterface(aMember)) != nullptr) {
+        int32_t value;
         number->GetValue(&value);
 
         nsCAutoString n;
@@ -818,7 +788,7 @@ static const char kRDFLIOpenGT[] = ">";
         rv = rdf_BlockingWrite(aStream, n);
         if (NS_FAILED(rv)) return rv;
     }
-    else if ((date = do_QueryInterface(aMember)) != nsnull) {
+    else if ((date = do_QueryInterface(aMember)) != nullptr) {
         PRTime value;
         date->GetValue(&value);
 
@@ -937,7 +907,7 @@ nsRDFXMLSerializer::SerializeContainer(nsIOutputStream* aStream,
             elements->GetNext(getter_AddRefs(isupports));
 
             nsCOMPtr<nsIRDFNode> element = do_QueryInterface(isupports);
-            NS_ASSERTION(element != nsnull, "not an nsIRDFNode");
+            NS_ASSERTION(element != nullptr, "not an nsIRDFNode");
             if (! element)
                 continue;
 
@@ -1039,7 +1009,7 @@ nsRDFXMLSerializer::SerializeEpilogue(nsIOutputStream* aStream)
     return rdf_BlockingWrite(aStream, NS_LITERAL_CSTRING("</RDF:RDF>\n"));
 }
 
-class QNameCollector : public rdfITripleVisitor {
+class QNameCollector MOZ_FINAL : public rdfITripleVisitor {
 public:
     NS_DECL_ISUPPORTS
     NS_DECL_RDFITRIPLEVISITOR
@@ -1061,14 +1031,14 @@ QNameCollector::Visit(nsIRDFNode* aSubject, nsIRDFResource* aPredicate,
             // ignore error
             return NS_OK;
         }
-        if (mParent->mQNames.Get(resType, nsnull)) {
+        if (mParent->mQNames.Get(resType, nullptr)) {
             return NS_OK;
         }
         mParent->RegisterQName(resType);
         return NS_OK;
     }
 
-    if (mParent->mQNames.Get(aPredicate, nsnull)) {
+    if (mParent->mQNames.Get(aPredicate, nullptr)) {
         return NS_OK;
     }
     if (aPredicate == mParent->kRDF_instanceOf ||
