@@ -1,45 +1,8 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
  *
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Brian Ryner <bryner@brianryner.com>
- *   Terry Hayes <thayes@netscape.com>
- *   Kai Engert <kengert@redhat.com>
- *   Petr Kostka <petr.kostka@st.com>
- *   Honza Bambas <honzab@firemni.cz>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 #include "nsNSSComponent.h"
 #include "nsNSSCallbacks.h"
 #include "nsNSSIOLayer.h"
@@ -107,7 +70,7 @@ nsHTTPDownloadEvent::Run()
   NS_ENSURE_STATE(ios);
 
   nsCOMPtr<nsIChannel> chan;
-  ios->NewChannel(mRequestSession->mURL, nsnull, nsnull, getter_AddRefs(chan));
+  ios->NewChannel(mRequestSession->mURL, nullptr, nullptr, getter_AddRefs(chan));
   NS_ENSURE_STATE(chan);
 
   // Disabled because it breaks authentication with a proxy, when such proxy
@@ -164,15 +127,15 @@ nsHTTPDownloadEvent::Run()
                           mListener);
 
   if (NS_SUCCEEDED(rv))
-    rv = hchan->AsyncOpen(mListener->mLoader, nsnull);
+    rv = hchan->AsyncOpen(mListener->mLoader, nullptr);
 
   if (NS_FAILED(rv)) {
     mListener->mResponsibleForDoneSignal = false;
     mResponsibleForDoneSignal = true;
 
     NS_RELEASE(mListener->mLoadGroup);
-    mListener->mLoadGroup = nsnull;
-    mListener->mLoadGroupOwnerThread = nsnull;
+    mListener->mLoadGroup = nullptr;
+    mListener->mLoadGroupOwnerThread = nullptr;
   }
 
   return NS_OK;
@@ -183,13 +146,13 @@ struct nsCancelHTTPDownloadEvent : nsRunnable {
 
   NS_IMETHOD Run() {
     mListener->FreeLoadGroup(true);
-    mListener = nsnull;
+    mListener = nullptr;
     return NS_OK;
   }
 };
 
 SECStatus nsNSSHttpServerSession::createSessionFcn(const char *host,
-                                                   PRUint16 portnum,
+                                                   uint16_t portnum,
                                                    SEC_HTTP_SERVER_SESSION *pSession)
 {
   if (!host || !pSession)
@@ -229,7 +192,7 @@ SECStatus nsNSSHttpRequestSession::createFcn(SEC_HTTP_SERVER_SESSION session,
 
   // Use a maximum timeout value of 10 seconds because of bug 404059.
   // FIXME: Use a better approach once 406120 is ready.
-  PRUint32 maxBug404059Timeout = PR_TicksPerSecond() * 10;
+  uint32_t maxBug404059Timeout = PR_TicksPerSecond() * 10;
   if (timeout > maxBug404059Timeout) {
     rs->mTimeoutInterval = maxBug404059Timeout;
   }
@@ -248,7 +211,7 @@ SECStatus nsNSSHttpRequestSession::createFcn(SEC_HTTP_SERVER_SESSION session,
 }
 
 SECStatus nsNSSHttpRequestSession::setPostDataFcn(const char *http_data, 
-                                                  const PRUint32 http_data_len,
+                                                  const uint32_t http_data_len,
                                                   const char *http_content_type)
 {
   mHasPostData = true;
@@ -275,11 +238,11 @@ SECStatus nsNSSHttpRequestSession::addHeaderFcn(const char *http_header_name,
 }
 
 SECStatus nsNSSHttpRequestSession::trySendAndReceiveFcn(PRPollDesc **pPollDesc,
-                                                        PRUint16 *http_response_code, 
+                                                        uint16_t *http_response_code, 
                                                         const char **http_response_content_type, 
                                                         const char **http_response_headers, 
                                                         const char **http_response_data, 
-                                                        PRUint32 *http_response_data_len)
+                                                        uint32_t *http_response_data_len)
 {
   PR_LOG(gPIPNSSLog, PR_LOG_DEBUG,
          ("nsNSSHttpRequestSession::trySendAndReceiveFcn to %s\n", mURL.get()));
@@ -339,7 +302,7 @@ nsNSSHttpRequestSession::AddRef()
 void
 nsNSSHttpRequestSession::Release()
 {
-  PRInt32 newRefCount = NS_AtomicDecrementRefcnt(mRefCount);
+  int32_t newRefCount = NS_AtomicDecrementRefcnt(mRefCount);
   if (!newRefCount) {
     delete this;
   }
@@ -348,19 +311,19 @@ nsNSSHttpRequestSession::Release()
 SECStatus
 nsNSSHttpRequestSession::internal_send_receive_attempt(bool &retryable_error,
                                                        PRPollDesc **pPollDesc,
-                                                       PRUint16 *http_response_code,
+                                                       uint16_t *http_response_code,
                                                        const char **http_response_content_type,
                                                        const char **http_response_headers,
                                                        const char **http_response_data,
-                                                       PRUint32 *http_response_data_len)
+                                                       uint32_t *http_response_data_len)
 {
-  if (pPollDesc) *pPollDesc = nsnull;
+  if (pPollDesc) *pPollDesc = nullptr;
   if (http_response_code) *http_response_code = 0;
   if (http_response_content_type) *http_response_content_type = 0;
   if (http_response_headers) *http_response_headers = 0;
   if (http_response_data) *http_response_data = 0;
 
-  PRUint32 acceptableResultSize = 0;
+  uint32_t acceptableResultSize = 0;
 
   if (http_response_data_len)
   {
@@ -423,7 +386,7 @@ nsNSSHttpRequestSession::internal_send_receive_attempt(bool &retryable_error,
         // made me aware of this possibility. (kaie)
 
         MutexAutoUnlock unlock(waitLock);
-        NS_ProcessNextEvent(nsnull);
+        NS_ProcessNextEvent(nullptr);
       }
 
       waitCondition.Wait(wait_interval);
@@ -545,18 +508,18 @@ void nsNSSHttpInterface::registerHttpClient()
 
 void nsNSSHttpInterface::unregisterHttpClient()
 {
-  SEC_RegisterDefaultHttpClient(nsnull);
+  SEC_RegisterDefaultHttpClient(nullptr);
 }
 
 nsHTTPListener::nsHTTPListener()
-: mResultData(nsnull),
+: mResultData(nullptr),
   mResultLen(0),
   mLock("nsHTTPListener.mLock"),
   mCondition(mLock, "nsHTTPListener.mCondition"),
   mWaitFlag(true),
   mResponsibleForDoneSignal(false),
-  mLoadGroup(nsnull),
-  mLoadGroupOwnerThread(nsnull)
+  mLoadGroup(nullptr),
+  mLoadGroupOwnerThread(nullptr)
 {
 }
 
@@ -576,7 +539,7 @@ NS_IMPL_THREADSAFE_ISUPPORTS1(nsHTTPListener, nsIStreamLoaderObserver)
 void
 nsHTTPListener::FreeLoadGroup(bool aCancelLoad)
 {
-  nsILoadGroup *lg = nsnull;
+  nsILoadGroup *lg = nullptr;
 
   MutexAutoLock locker(mLock);
 
@@ -587,7 +550,7 @@ nsHTTPListener::FreeLoadGroup(bool aCancelLoad)
     }
     else {
       lg = mLoadGroup;
-      mLoadGroup = nsnull;
+      mLoadGroup = nullptr;
     }
   }
 
@@ -603,8 +566,8 @@ NS_IMETHODIMP
 nsHTTPListener::OnStreamComplete(nsIStreamLoader* aLoader,
                                  nsISupports* aContext,
                                  nsresult aStatus,
-                                 PRUint32 stringLen,
-                                 const PRUint8* string)
+                                 uint32_t stringLen,
+                                 const uint8_t* string)
 {
   mResultCode = aStatus;
 
@@ -668,10 +631,10 @@ ShowProtectedAuthPrompt(PK11SlotInfo* slot, nsIInterfaceRequestor *ir)
 {
   if (!NS_IsMainThread()) {
     NS_ERROR("ShowProtectedAuthPrompt called off the main thread");
-    return nsnull;
+    return nullptr;
   }
 
-  char* protAuthRetVal = nsnull;
+  char* protAuthRetVal = nullptr;
 
   // Get protected auth dialogs
   nsITokenDialogs* dialogs = 0;
@@ -708,7 +671,7 @@ ShowProtectedAuthPrompt(PK11SlotInfo* slot, nsIInterfaceRequestor *ir)
                   protAuthRetVal = ToNewCString(nsDependentCString(PK11_PW_RETRY));
                   break;
               default:
-                  protAuthRetVal = nsnull;
+                  protAuthRetVal = nullptr;
                   break;
               
           }
@@ -729,7 +692,7 @@ class PK11PasswordPromptRunnable : public SyncRunnableBase
 public:
   PK11PasswordPromptRunnable(PK11SlotInfo* slot, 
                              nsIInterfaceRequestor* ir)
-    : mResult(nsnull),
+    : mResult(nullptr),
       mSlot(slot),
       mIR(ir)
   {
@@ -745,14 +708,14 @@ void PK11PasswordPromptRunnable::RunOnTargetThread()
 {
   nsNSSShutDownPreventionLock locker;
   nsresult rv = NS_OK;
-  PRUnichar *password = nsnull;
+  PRUnichar *password = nullptr;
   bool value = false;
   nsCOMPtr<nsIPrompt> prompt;
 
   /* TODO: Retry should generate a different dialog message */
 /*
   if (retry)
-    return nsnull;
+    return nullptr;
 */
 
   if (!mIR)
@@ -762,7 +725,7 @@ void PK11PasswordPromptRunnable::RunOnTargetThread()
   else
   {
     prompt = do_GetInterface(mIR);
-    NS_ASSERTION(prompt != nsnull, "callbacks does not implement nsIPrompt");
+    NS_ASSERTION(prompt != nullptr, "callbacks does not implement nsIPrompt");
   }
 
   if (!prompt)
@@ -799,8 +762,8 @@ void PK11PasswordPromptRunnable::RunOnTargetThread()
       // Although the exact value is ignored, we must not pass invalid
       // bool values through XPConnect.
       bool checkState = false;
-      rv = prompt->PromptPassword(nsnull, promptString.get(),
-                                  &password, nsnull, &checkState, &value);
+      rv = prompt->PromptPassword(nullptr, promptString.get(),
+                                  &password, nullptr, &checkState, &value);
     }
   }
   
@@ -822,25 +785,30 @@ PK11PasswordPrompt(PK11SlotInfo* slot, PRBool retry, void* arg)
 
 void PR_CALLBACK HandshakeCallback(PRFileDesc* fd, void* client_data) {
   nsNSSShutDownPreventionLock locker;
-  PRInt32 sslStatus;
-  char* signer = nsnull;
-  char* cipherName = nsnull;
-  PRInt32 keyLength;
+  int32_t sslStatus;
+  char* signer = nullptr;
+  char* cipherName = nullptr;
+  int32_t keyLength;
   nsresult rv;
-  PRInt32 encryptBits;
+  int32_t encryptBits;
 
   nsNSSSocketInfo* infoObject = (nsNSSSocketInfo*) fd->higher->secret;
 
+  if (infoObject) {
+    // This is the first callback on resumption handshakes
+    infoObject->SetFirstServerHelloReceived();
+  }
+
   // If the handshake completed, then we know the site is TLS tolerant (if this
   // was a TLS connection).
   nsSSLIOLayerHelpers::rememberTolerantSite(infoObject);
 
   if (SECSuccess != SSL_SecurityStatus(fd, &sslStatus, &cipherName, &keyLength,
-                                       &encryptBits, &signer, nsnull)) {
+                                       &encryptBits, &signer, nullptr)) {
     return;
   }
 
-  PRInt32 secStatus;
+  int32_t secStatus;
   if (sslStatus == SSL_SECURITY_STATUS_OFF)
     secStatus = nsIWebProgressListener::STATE_IS_BROKEN;
   else if (encryptBits >= 90)
@@ -877,7 +845,7 @@ void PR_CALLBACK HandshakeCallback(PRFileDesc* fd, void* client_data) {
 
 
   CERTCertificate *peerCert = SSL_PeerCertificate(fd);
-  const char* caName = nsnull; // caName is a pointer only, no ownership
+  const char* caName = nullptr; // caName is a pointer only, no ownership
   char* certOrgName = CERT_GetOrgName(&peerCert->issuer);
   CERT_DestroyCertificate(peerCert);
   caName = certOrgName ? certOrgName : signer;
@@ -910,14 +878,14 @@ void PR_CALLBACK HandshakeCallback(PRFileDesc* fd, void* client_data) {
       infoObject->SetSSLStatus(status);
     }
 
-    nsSSLIOLayerHelpers::mHostsWithCertErrors->LookupCertErrorBits(
-      infoObject, status);
+    RememberCertErrorsTable::GetInstance().LookupCertErrorBits(infoObject,
+                                                               status);
 
     CERTCertificate *serverCert = SSL_PeerCertificate(fd);
     if (serverCert) {
       nsRefPtr<nsNSSCertificate> nssc = nsNSSCertificate::Create(serverCert);
       CERT_DestroyCertificate(serverCert);
-      serverCert = nsnull;
+      serverCert = nullptr;
 
       nsCOMPtr<nsIX509Cert> prevcert;
       infoObject->GetPreviousCert(getter_AddRefs(prevcert));
@@ -964,7 +932,7 @@ void PR_CALLBACK HandshakeCallback(PRFileDesc* fd, void* client_data) {
         state == SSL_NEXT_PROTO_NEGOTIATED)
       infoObject->SetNegotiatedNPN(reinterpret_cast<char *>(npnbuf), npnlen);
     else
-      infoObject->SetNegotiatedNPN(nsnull, 0);
+      infoObject->SetNegotiatedNPN(nullptr, 0);
 
     infoObject->SetHandshakeCompleted();
   }
@@ -986,65 +954,65 @@ static struct OCSPDefaultResponders myDefaultOCSPResponders[] = {
   /* COMODO */
   {
     "CN=AddTrust External CA Root,OU=AddTrust External TTP Network,O=AddTrust AB,C=SE",
-    nsnull, "rb2YejS0Jvf6xCZU7wO94CTLVBo=", nsnull,
+    nullptr, "rb2YejS0Jvf6xCZU7wO94CTLVBo=", nullptr,
     "http://ocsp.comodoca.com"
   },
   {
     "CN=COMODO Certification Authority,O=COMODO CA Limited,L=Salford,ST=Greater Manchester,C=GB",
-    nsnull, "C1jli8ZMFTekQKkwqSG+RzZaVv8=", nsnull,
+    nullptr, "C1jli8ZMFTekQKkwqSG+RzZaVv8=", nullptr,
     "http://ocsp.comodoca.com"
   },
   {
     "CN=COMODO EV SGC CA,O=COMODO CA Limited,L=Salford,ST=Greater Manchester,C=GB",
-    nsnull, "f/ZMNigUrs0eN6/eWvJbw6CsK/4=", nsnull,
+    nullptr, "f/ZMNigUrs0eN6/eWvJbw6CsK/4=", nullptr,
     "http://ocsp.comodoca.com"
   },
   {
     "CN=COMODO EV SSL CA,O=COMODO CA Limited,L=Salford,ST=Greater Manchester,C=GB",
-    nsnull, "aRZJ7LZ1ZFrpAyNgL1RipTRcPuI=", nsnull,
+    nullptr, "aRZJ7LZ1ZFrpAyNgL1RipTRcPuI=", nullptr,
     "http://ocsp.comodoca.com"
   },
   {
     "CN=UTN - DATACorp SGC,OU=http://www.usertrust.com,O=The USERTRUST Network,L=Salt Lake City,ST=UT,C=US",
-    nsnull, "UzLRs89/+uDxoF2FTpLSnkUdtE8=", nsnull,
+    nullptr, "UzLRs89/+uDxoF2FTpLSnkUdtE8=", nullptr,
     "http://ocsp.usertrust.com"
   },
   {
     "CN=UTN-USERFirst-Hardware,OU=http://www.usertrust.com,O=The USERTRUST Network,L=Salt Lake City,ST=UT,C=US",
-    nsnull, "oXJfJhsomEOVXQc31YWWnUvSw0U=", nsnull,
+    nullptr, "oXJfJhsomEOVXQc31YWWnUvSw0U=", nullptr,
     "http://ocsp.usertrust.com"
   },
   /* Network Solutions */
   {
     "CN=Network Solutions Certificate Authority,O=Network Solutions L.L.C.,C=US",
-    nsnull, "ITDJ+wDXTpjah6oq0KcusUAxp0w=", nsnull,
+    nullptr, "ITDJ+wDXTpjah6oq0KcusUAxp0w=", nullptr,
     "http://ocsp.netsolssl.com"
   },
   {
     "CN=Network Solutions EV SSL CA,O=Network Solutions L.L.C.,C=US",
-    nsnull, "tk6FnYQfGx3UUolOB5Yt+d7xj8w=", nsnull,
+    nullptr, "tk6FnYQfGx3UUolOB5Yt+d7xj8w=", nullptr,
     "http://ocsp.netsolssl.com"
   },
   /* GlobalSign */
   {
     "CN=GlobalSign Root CA,OU=Root CA,O=GlobalSign nv-sa,C=BE",
-    nsnull, "YHtmGkUNl8qJUC99BM00qP/8/Us=", nsnull,
+    nullptr, "YHtmGkUNl8qJUC99BM00qP/8/Us=", nullptr,
     "http://ocsp.globalsign.com/ExtendedSSLCACross"
   },
   {
     "CN=GlobalSign,O=GlobalSign,OU=GlobalSign Root CA - R2",
-    nsnull, "m+IHV2ccHsBqBt5ZtJot39wZhi4=", nsnull,
+    nullptr, "m+IHV2ccHsBqBt5ZtJot39wZhi4=", nullptr,
     "http://ocsp.globalsign.com/ExtendedSSLCA"
   },
   {
     "CN=GlobalSign Extended Validation CA,O=GlobalSign,OU=Extended Validation CA",
-    nsnull, "NLH5yYxrNUTMCGkK7uOjuVy/FuA=", nsnull,
+    nullptr, "NLH5yYxrNUTMCGkK7uOjuVy/FuA=", nullptr,
     "http://ocsp.globalsign.com/ExtendedSSL"
   },
   /* Trustwave */
   {
     "CN=SecureTrust CA,O=SecureTrust Corporation,C=US",
-    nsnull, "QjK2FvoE/f5dS3rD/fdMQB1aQ68=", nsnull,
+    nullptr, "QjK2FvoE/f5dS3rD/fdMQB1aQ68=", nullptr,
     "http://ocsp.trustwave.com"
   }
 };
@@ -1052,7 +1020,7 @@ static struct OCSPDefaultResponders myDefaultOCSPResponders[] = {
 static const unsigned int numResponders =
     (sizeof myDefaultOCSPResponders) / (sizeof myDefaultOCSPResponders[0]);
 
-static CERT_StringFromCertFcn oldOCSPAIAInfoCallback = nsnull;
+static CERT_StringFromCertFcn oldOCSPAIAInfoCallback = nullptr;
 
 /*
  * See if we have a hard-coded default responder for this certificate's
@@ -1081,7 +1049,7 @@ char* PR_CALLBACK MyAlternateOCSPAIAInfoCallback(CERTCertificate *cert) {
   if (oldOCSPAIAInfoCallback)
     return (*oldOCSPAIAInfoCallback)(cert);
 
-  return nsnull;
+  return nullptr;
 }
 
 void cleanUpMyDefaultOCSPResponders() {
@@ -1090,11 +1058,11 @@ void cleanUpMyDefaultOCSPResponders() {
   for (i=0; i < numResponders; i++) {
     if (myDefaultOCSPResponders[i].issuerName) {
       CERT_DestroyName(myDefaultOCSPResponders[i].issuerName);
-      myDefaultOCSPResponders[i].issuerName = nsnull;
+      myDefaultOCSPResponders[i].issuerName = nullptr;
     }
     if (myDefaultOCSPResponders[i].issuerKeyID) {
       SECITEM_FreeItem(myDefaultOCSPResponders[i].issuerKeyID, true);
-      myDefaultOCSPResponders[i].issuerKeyID = nsnull;
+      myDefaultOCSPResponders[i].issuerKeyID = nullptr;
     }
   }
 }
@@ -1114,9 +1082,9 @@ SECStatus RegisterMyOCSPAIAInfoCallback() {
     if (!(myDefaultOCSPResponders[i].issuerName))
       goto loser;
     // Create a SECItem from the Base64 authority key identifier keyID.
-    myDefaultOCSPResponders[i].issuerKeyID = NSSBase64_DecodeBuffer(nsnull,
-          nsnull, myDefaultOCSPResponders[i].issuerKeyID_base64,
-          (PRUint32)PORT_Strlen(myDefaultOCSPResponders[i].issuerKeyID_base64));
+    myDefaultOCSPResponders[i].issuerKeyID = NSSBase64_DecodeBuffer(nullptr,
+          nullptr, myDefaultOCSPResponders[i].issuerKeyID_base64,
+          (uint32_t)PORT_Strlen(myDefaultOCSPResponders[i].issuerKeyID_base64));
     if (!(myDefaultOCSPResponders[i].issuerKeyID))
       goto loser;
   }
@@ -1143,12 +1111,12 @@ SECStatus UnregisterMyOCSPAIAInfoCallback() {
 
   // Unregister our alternate OCSP Responder URL lookup function.
   rv = CERT_RegisterAlternateOCSPAIAInfoCallBack(oldOCSPAIAInfoCallback,
-                                                 nsnull);
+                                                 nullptr);
   if (rv != SECSuccess)
     return rv;
 
   // Tidy up.
-  oldOCSPAIAInfoCallback = nsnull;
+  oldOCSPAIAInfoCallback = nullptr;
   cleanUpMyDefaultOCSPResponders();
   return SECSuccess;
 }
