@@ -251,20 +251,6 @@ function SyncTestingInfrastructure(username, password, syncKey) {
   this.fakeCryptoService = new FakeCryptoService();
 }
 
-/*
- * Ensure exceptions from inside callbacks leads to test failures.
- */
-function ensureThrows(func) {
-  return function() {
-    try {
-      func.apply(this, arguments);
-    } catch (ex) {
-      do_throw(ex);
-    }
-  };
-}
-
-
 _("Setting the identity for passphrase");
 Cu.import("resource://services-sync/identity.js");
 
@@ -394,26 +380,3 @@ RotaryEngine.prototype = {
     }
   }
 };
-
-deepCopy: function deepCopy(thing, noSort) {
-  if (typeof(thing) != "object" || thing == null){
-    return thing;
-  }
-  let ret;
-
-  if (Array.isArray(thing)) {
-    ret = [];
-    for (let i = 0; i < thing.length; i++){
-      ret.push(deepCopy(thing[i], noSort));
-	}
-  } else {
-    ret = {};
-    let props = [p for (p in thing)];
-    if (!noSort){
-      props = props.sort();
-    }
-    props.forEach(function(k) ret[k] = deepCopy(thing[k], noSort));
-  }
-
-  return ret;
-};
