@@ -1,41 +1,8 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
  * vim: sw=4 ts=4 et :
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Mozilla Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2009
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Chris Jones <jones.chris.g@gmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /**
  * Note: This file is a copy of xpcom/tests/TestDeadlockDetector.cpp, but all
@@ -123,7 +90,7 @@ class Subprocess
 {
 public:
     // not available until process finishes
-    PRInt32 mExitCode;
+    int32_t mExitCode;
     nsCString mStdout;
     nsCString mStderr;
 
@@ -185,15 +152,15 @@ public:
         PR_DestroyProcessAttr(pattr);
     }
 
-    void RunToCompletion(PRUint32 aWaitMs)
+    void RunToCompletion(uint32_t aWaitMs)
     {
         PR_Close(mStdinfd);
 
         PRPollDesc pollfds[2];
-        PRInt32 nfds;
+        int32_t nfds;
         bool stdoutOpen = true, stderrOpen = true;
         char buf[4096];
-        PRInt32 len;
+        int32_t len;
 
         PRIntervalTime now = PR_IntervalNow();
         PRIntervalTime deadline = now + PR_MillisecondsToInterval(aWaitMs);
@@ -213,7 +180,7 @@ public:
                 ++nfds;
             }
 
-            PRInt32 rv = PR_Poll(pollfds, nfds, deadline - now);
+            int32_t rv = PR_Poll(pollfds, nfds, deadline - now);
             NS_ASSERTION(0 <= rv, PR_ErrorToName(PR_GetError()));
 
             if (0 == rv) {      // timeout
@@ -222,7 +189,7 @@ public:
                 return;
             }
 
-            for (PRInt32 i = 0; i < nfds; ++i) {
+            for (int32_t i = 0; i < nfds; ++i) {
                 if (!pollfds[i].out_flags)
                     continue;
 
@@ -272,7 +239,7 @@ private:
         if (!normalExit) {
             PR_KillProcess(mProc);
             mExitCode = -1;
-            PRInt32 dummy;
+            int32_t dummy;
             PR_WaitProcess(mProc, &dummy);
         }
         else {
@@ -300,7 +267,7 @@ CheckForDeadlock(const char* test, const char* const* findTokens)
     if (0 == proc.mExitCode)
         return false;
 
-    PRInt32 idx = 0;
+    int32_t idx = 0;
     for (const char* const* tp = findTokens; *tp; ++tp) {
         const char* const token = *tp;
 #ifdef MOZILLA_INTERNAL_API
@@ -332,7 +299,7 @@ Sanity_Child()
     TestMutex m1("dd.sanity.m1");
     m1.Lock();
     m1.Lock();
-    return 0;                  // not reached
+    return NS_OK;                  // not reached
 }
 
 nsresult
@@ -362,7 +329,7 @@ Sanity2_Child()
     m1.Lock();
     m2.Lock();
     m1.Lock();
-    return 0;                  // not reached
+    return NS_OK;                  // not reached
 }
 
 nsresult
@@ -404,7 +371,7 @@ Sanity3_Child()
 
     m4.Lock();
     m1.Lock();
-    return 0;
+    return NS_OK;
 }
 
 nsresult
@@ -436,7 +403,7 @@ Sanity4_Child()
     m1.Enter();
     m2.Lock();
     m1.Enter();
-    return 0;
+    return NS_OK;
 }
 
 nsresult
@@ -467,7 +434,7 @@ TestMutex* ttM2;
 static void
 TwoThreads_thread(void* arg)
 {
-    PRInt32 m1First = NS_PTR_TO_INT32(arg);
+    int32_t m1First = NS_PTR_TO_INT32(arg);
     if (m1First) {
         ttM1->Lock();
         ttM2->Lock();
@@ -496,7 +463,7 @@ TwoThreads_Child()
     PRThread* t2 = spawn(TwoThreads_thread, (void*) 1);
     PR_JoinThread(t2);
 
-    return 0;
+    return NS_OK;
 }
 
 nsresult
@@ -520,18 +487,18 @@ TwoThreads()
 
 
 TestMutex* cndMs[4];
-const PRUint32 K = 100000;
+const uint32_t K = 100000;
 
 static void
 ContentionNoDeadlock_thread(void* arg)
 {
-    PRInt32 starti = NS_PTR_TO_INT32(arg);
+    int32_t starti = NS_PTR_TO_INT32(arg);
 
-    for (PRUint32 k = 0; k < K; ++k) {
-        for (PRInt32 i = starti; i < (PRInt32) ArrayLength(cndMs); ++i)
+    for (uint32_t k = 0; k < K; ++k) {
+        for (int32_t i = starti; i < (int32_t) ArrayLength(cndMs); ++i)
             cndMs[i]->Lock();
         // comment out the next two lines for deadlocking fun!
-        for (PRInt32 i = ArrayLength(cndMs) - 1; i >= starti; --i)
+        for (int32_t i = ArrayLength(cndMs) - 1; i >= starti; --i)
             cndMs[i]->Unlock();
 
         starti = (starti + 1) % 3;
@@ -543,19 +510,19 @@ ContentionNoDeadlock_Child()
 {
     PRThread* threads[3];
 
-    for (PRUint32 i = 0; i < ArrayLength(cndMs); ++i)
+    for (uint32_t i = 0; i < ArrayLength(cndMs); ++i)
         cndMs[i] = new TestMutex("dd.cnd.ms");
 
-    for (PRInt32 i = 0; i < (PRInt32) ArrayLength(threads); ++i)
+    for (int32_t i = 0; i < (int32_t) ArrayLength(threads); ++i)
         threads[i] = spawn(ContentionNoDeadlock_thread, NS_INT32_TO_PTR(i));
 
-    for (PRUint32 i = 0; i < ArrayLength(threads); ++i)
+    for (uint32_t i = 0; i < ArrayLength(threads); ++i)
         PR_JoinThread(threads[i]);
 
-    for (PRUint32 i = 0; i < ArrayLength(cndMs); ++i)
+    for (uint32_t i = 0; i < ArrayLength(cndMs); ++i)
         delete cndMs[i];
 
-    return 0;
+    return NS_OK;
 }
 
 nsresult
