@@ -1,41 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Joe Hewitt <hewitt@netscape.com> (Original Author)
- *   Dean Tessman <dean_tessman@hotmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsFormFillController.h"
 
@@ -53,14 +19,12 @@
 #include "nsIContentViewer.h"
 #include "nsIDOMEventTarget.h"
 #include "nsIDOMKeyEvent.h"
-#include "nsIPrivateDOMEvent.h"
 #include "nsIDOMDocument.h"
 #include "nsIDOMElement.h"
 #include "nsIFormControl.h"
 #include "nsIDocument.h"
 #include "nsIContent.h"
 #include "nsIPresShell.h"
-#include "nsIFrame.h"
 #include "nsRect.h"
 #include "nsIDOMHTMLFormElement.h"
 #include "nsILoginManager.h"
@@ -69,7 +33,6 @@
 #include "nsToolkitCompsCID.h"
 #include "nsEmbedCID.h"
 #include "nsIDOMNSEditableElement.h"
-#include "nsIDOMNSEvent.h"
 #include "mozilla/dom/Element.h"
 #include "nsContentUtils.h"
 
@@ -81,9 +44,9 @@ NS_IMPL_ISUPPORTS5(nsFormFillController,
                    nsIMutationObserver)
 
 nsFormFillController::nsFormFillController() :
-  mFocusedInput(nsnull),
-  mFocusedInputNode(nsnull),
-  mListNode(nsnull),
+  mFocusedInput(nullptr),
+  mFocusedInputNode(nullptr),
+  mListNode(nullptr),
   mTimeout(50),
   mMinResultsForPopup(1),
   mMaxRows(0),
@@ -112,20 +75,20 @@ nsFormFillController::~nsFormFillController()
 {
   if (mListNode) {
     mListNode->RemoveMutationObserver(this);
-    mListNode = nsnull;
+    mListNode = nullptr;
   }
   if (mFocusedInputNode) {
     MaybeRemoveMutationObserver(mFocusedInputNode);
-    mFocusedInputNode = nsnull;
-    mFocusedInput = nsnull;
+    mFocusedInputNode = nullptr;
+    mFocusedInput = nullptr;
   }
-  PwmgrInputsEnumData ed(this, nsnull);
+  PwmgrInputsEnumData ed(this, nullptr);
   mPwmgrInputs.Enumerate(RemoveForDocumentEnumerator, &ed);
 
   // Remove ourselves as a focus listener from all cached docShells
-  PRUint32 count;
+  uint32_t count;
   mDocShells->Count(&count);
-  for (PRUint32 i = 0; i < count; ++i) {
+  for (uint32_t i = 0; i < count; ++i) {
     nsCOMPtr<nsIDocShell> docShell;
     mDocShells->GetElementAt(i, getter_AddRefs(docShell));
     nsCOMPtr<nsIDOMWindow> domWindow = GetWindowForDocShell(docShell);
@@ -140,8 +103,8 @@ nsFormFillController::~nsFormFillController()
 void
 nsFormFillController::AttributeChanged(nsIDocument* aDocument,
                                        mozilla::dom::Element* aElement,
-                                       PRInt32 aNameSpaceID,
-                                       nsIAtom* aAttribute, PRInt32 aModType)
+                                       int32_t aNameSpaceID,
+                                       nsIAtom* aAttribute, int32_t aModType)
 {
   if (mListNode && mListNode->Contains(aElement)) {
     RevalidateDataList();
@@ -152,7 +115,7 @@ void
 nsFormFillController::ContentAppended(nsIDocument* aDocument,
                                       nsIContent* aContainer,
                                       nsIContent* aChild,
-                                      PRInt32 aIndexInContainer)
+                                      int32_t aIndexInContainer)
 {
   if (mListNode && mListNode->Contains(aContainer)) {
     RevalidateDataList();
@@ -163,7 +126,7 @@ void
 nsFormFillController::ContentInserted(nsIDocument* aDocument,
                                       nsIContent* aContainer,
                                       nsIContent* aChild,
-                                      PRInt32 aIndexInContainer)
+                                      int32_t aIndexInContainer)
 {
   if (mListNode && mListNode->Contains(aContainer)) {
     RevalidateDataList();
@@ -174,7 +137,7 @@ void
 nsFormFillController::ContentRemoved(nsIDocument* aDocument,
                                      nsIContent* aContainer,
                                      nsIContent* aChild,
-                                     PRInt32 aIndexInContainer,
+                                     int32_t aIndexInContainer,
                                      nsIContent* aPreviousSibling)
 {
   if (mListNode && mListNode->Contains(aContainer)) {
@@ -199,8 +162,8 @@ nsFormFillController::CharacterDataChanged(nsIDocument* aDocument,
 void
 nsFormFillController::AttributeWillChange(nsIDocument* aDocument,
                                           mozilla::dom::Element* aElement,
-                                          PRInt32 aNameSpaceID,
-                                          nsIAtom* aAttribute, PRInt32 aModType)
+                                          int32_t aNameSpaceID,
+                                          nsIAtom* aAttribute, int32_t aModType)
 {
 }
 
@@ -214,11 +177,11 @@ nsFormFillController::NodeWillBeDestroyed(const nsINode* aNode)
 {
   mPwmgrInputs.Remove(aNode);
   if (aNode == mListNode) {
-    mListNode = nsnull;
+    mListNode = nullptr;
     RevalidateDataList();
   } else if (aNode == mFocusedInputNode) {
-    mFocusedInputNode = nsnull;
-    mFocusedInput = nsnull;
+    mFocusedInputNode = nullptr;
+    mFocusedInput = nullptr;
   }
 }
 
@@ -254,7 +217,7 @@ nsFormFillController::AttachToBrowser(nsIDocShell *aDocShell, nsIAutoCompletePop
 NS_IMETHODIMP
 nsFormFillController::DetachFromBrowser(nsIDocShell *aDocShell)
 {
-  PRInt32 index = GetIndexOfDocShell(aDocShell);
+  int32_t index = GetIndexOfDocShell(aDocShell);
   NS_ENSURE_TRUE(index >= 0, NS_ERROR_FAILURE);
 
   // Stop listening for focus events on the domWindow of the docShell
@@ -335,8 +298,12 @@ nsFormFillController::SetPopupOpen(bool aPopupOpen)
       docShell->GetPresShell(getter_AddRefs(presShell));
       NS_ENSURE_STATE(presShell);
       presShell->ScrollContentIntoView(content,
-                                       nsIPresShell::ScrollAxis(),
-                                       nsIPresShell::ScrollAxis(),
+                                       nsIPresShell::ScrollAxis(
+                                         nsIPresShell::SCROLL_MINIMUM,
+                                         nsIPresShell::SCROLL_IF_NOT_VISIBLE),
+                                       nsIPresShell::ScrollAxis(
+                                         nsIPresShell::SCROLL_MINIMUM,
+                                         nsIPresShell::SCROLL_IF_NOT_VISIBLE),
                                        nsIPresShell::SCROLL_OVERFLOW_HIDDEN);
       // mFocusedPopup can be destroyed after ScrollContentIntoView, see bug 420089
       if (mFocusedPopup)
@@ -404,27 +371,27 @@ NS_IMETHODIMP nsFormFillController::SetForceComplete(bool aForceComplete)
 }
 
 NS_IMETHODIMP
-nsFormFillController::GetMinResultsForPopup(PRUint32 *aMinResultsForPopup)
+nsFormFillController::GetMinResultsForPopup(uint32_t *aMinResultsForPopup)
 {
   *aMinResultsForPopup = mMinResultsForPopup;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsFormFillController::SetMinResultsForPopup(PRUint32 aMinResultsForPopup)
+NS_IMETHODIMP nsFormFillController::SetMinResultsForPopup(uint32_t aMinResultsForPopup)
 {
   mMinResultsForPopup = aMinResultsForPopup;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFormFillController::GetMaxRows(PRUint32 *aMaxRows)
+nsFormFillController::GetMaxRows(uint32_t *aMaxRows)
 {
   *aMaxRows = mMaxRows;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFormFillController::SetMaxRows(PRUint32 aMaxRows)
+nsFormFillController::SetMaxRows(uint32_t aMaxRows)
 {
   mMaxRows = aMaxRows;
   return NS_OK;
@@ -456,13 +423,13 @@ NS_IMETHODIMP nsFormFillController::SetShowCommentColumn(bool aShowCommentColumn
 }
 
 NS_IMETHODIMP
-nsFormFillController::GetTimeout(PRUint32 *aTimeout)
+nsFormFillController::GetTimeout(uint32_t *aTimeout)
 {
   *aTimeout = mTimeout;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsFormFillController::SetTimeout(PRUint32 aTimeout)
+NS_IMETHODIMP nsFormFillController::SetTimeout(uint32_t aTimeout)
 {
   mTimeout = aTimeout;
   return NS_OK;
@@ -490,14 +457,14 @@ nsFormFillController::GetSearchParam(nsAString &aSearchParam)
 }
 
 NS_IMETHODIMP
-nsFormFillController::GetSearchCount(PRUint32 *aSearchCount)
+nsFormFillController::GetSearchCount(uint32_t *aSearchCount)
 {
   *aSearchCount = 1;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFormFillController::GetSearchAt(PRUint32 index, nsACString & _retval)
+nsFormFillController::GetSearchAt(uint32_t index, nsACString & _retval)
 {
   _retval.Assign("form-history");
   return NS_OK;
@@ -527,7 +494,7 @@ nsFormFillController::SetTextValue(const nsAString & aTextValue)
 }
 
 NS_IMETHODIMP
-nsFormFillController::GetSelectionStart(PRInt32 *aSelectionStart)
+nsFormFillController::GetSelectionStart(int32_t *aSelectionStart)
 {
   if (mFocusedInput)
     mFocusedInput->GetSelectionStart(aSelectionStart);
@@ -535,7 +502,7 @@ nsFormFillController::GetSelectionStart(PRInt32 *aSelectionStart)
 }
 
 NS_IMETHODIMP
-nsFormFillController::GetSelectionEnd(PRInt32 *aSelectionEnd)
+nsFormFillController::GetSelectionEnd(int32_t *aSelectionEnd)
 {
   if (mFocusedInput)
     mFocusedInput->GetSelectionEnd(aSelectionEnd);
@@ -543,7 +510,7 @@ nsFormFillController::GetSelectionEnd(PRInt32 *aSelectionEnd)
 }
 
 NS_IMETHODIMP
-nsFormFillController::SelectTextRange(PRInt32 aStartIndex, PRInt32 aEndIndex)
+nsFormFillController::SelectTextRange(int32_t aStartIndex, int32_t aEndIndex)
 {
  if (mFocusedInput)
     mFocusedInput->SetSelectionRange(aStartIndex, aEndIndex, EmptyString());
@@ -574,15 +541,14 @@ nsFormFillController::OnTextEntered(bool* aPrevent)
 
   nsCOMPtr<nsIDOMEvent> event;
   domDoc->CreateEvent(NS_LITERAL_STRING("Events"), getter_AddRefs(event));
-  nsCOMPtr<nsIPrivateDOMEvent> privateEvent(do_QueryInterface(event));
-  NS_ENSURE_STATE(privateEvent);
+  NS_ENSURE_STATE(event);
 
   event->InitEvent(NS_LITERAL_STRING("DOMAutoComplete"), true, true);
 
   // XXXjst: We mark this event as a trusted event, it's up to the
   // callers of this to ensure that it's only called from trusted
   // code.
-  privateEvent->SetTrusted(true);
+  event->SetTrusted(true);
 
   nsCOMPtr<nsIDOMEventTarget> targ = do_QueryInterface(mFocusedInput);
 
@@ -666,7 +632,7 @@ nsFormFillController::StartSearch(const nsAString &aSearchString, const nsAStrin
       if (mListNode != node) {
         if (mListNode) {
           mListNode->RemoveMutationObserver(this);
-          mListNode = nsnull;
+          mListNode = nullptr;
         }
         if (node) {
           node->AddMutationObserverUnlessExists(this);
@@ -835,7 +801,7 @@ nsFormFillController::Focus(nsIDOMEvent* aEvent)
 
   nsCOMPtr<nsIDOMHTMLElement> datalist;
   input->GetList(getter_AddRefs(datalist));
-  bool hasList = datalist != nsnull;
+  bool hasList = datalist != nullptr;
 
   bool dummy;
   bool isPwmgrInput = false;
@@ -865,7 +831,7 @@ nsFormFillController::KeyPress(nsIDOMEvent* aEvent)
 
   bool cancel = false;
 
-  PRUint32 k;
+  uint32_t k;
   keyEvent->GetKeyCode(&k);
   switch (k) {
   case nsIDOMKeyEvent::DOM_VK_DELETE:
@@ -938,7 +904,7 @@ nsFormFillController::MouseDown(nsIDOMEvent* aEvent)
   if (!targetInput)
     return NS_OK;
 
-  PRUint16 button;
+  uint16_t button;
   mouseEvent->GetButton(&button);
   if (button != 0)
     return NS_OK;
@@ -979,7 +945,7 @@ nsFormFillController::AddWindowListeners(nsIDOMWindow *aWindow)
     return;
 
   nsCOMPtr<nsPIDOMWindow> privateDOMWindow(do_QueryInterface(aWindow));
-  nsIDOMEventTarget* target = nsnull;
+  nsIDOMEventTarget* target = nullptr;
   if (privateDOMWindow)
     target = privateDOMWindow->GetChromeEventHandler();
 
@@ -1022,7 +988,7 @@ nsFormFillController::RemoveWindowListeners(nsIDOMWindow *aWindow)
   mPwmgrInputs.Enumerate(RemoveForDocumentEnumerator, &ed);
 
   nsCOMPtr<nsPIDOMWindow> privateDOMWindow(do_QueryInterface(aWindow));
-  nsIDOMEventTarget* target = nsnull;
+  nsIDOMEventTarget* target = nullptr;
   if (privateDOMWindow)
     target = privateDOMWindow->GetChromeEventHandler();
 
@@ -1071,7 +1037,7 @@ nsFormFillController::StartControllingInput(nsIDOMHTMLInputElement *aInput)
 
   // Find the currently focused docShell
   nsCOMPtr<nsIDocShell> docShell = GetDocShellForInput(aInput);
-  PRInt32 index = GetIndexOfDocShell(docShell);
+  int32_t index = GetIndexOfDocShell(docShell);
   if (index < 0)
     return;
 
@@ -1108,7 +1074,7 @@ nsFormFillController::StopControllingInput()
 
   if (mListNode) {
     mListNode->RemoveMutationObserver(this);
-    mListNode = nsnull;
+    mListNode = nullptr;
   }
 
   // Reset the controller's input, but not if it has been switched
@@ -1117,14 +1083,14 @@ nsFormFillController::StopControllingInput()
   nsCOMPtr<nsIAutoCompleteInput> input;
   mController->GetInput(getter_AddRefs(input));
   if (input == this)
-    mController->SetInput(nsnull);
+    mController->SetInput(nullptr);
 
   if (mFocusedInputNode) {
     MaybeRemoveMutationObserver(mFocusedInputNode);
-    mFocusedInputNode = nsnull;
-    mFocusedInput = nsnull;
+    mFocusedInputNode = nullptr;
+    mFocusedInput = nullptr;
   }
-  mFocusedPopup = nsnull;
+  mFocusedPopup = nullptr;
 }
 
 nsIDocShell *
@@ -1133,7 +1099,7 @@ nsFormFillController::GetDocShellForInput(nsIDOMHTMLInputElement *aInput)
   nsCOMPtr<nsIDOMDocument> domDoc;
   aInput->GetOwnerDocument(getter_AddRefs(domDoc));
   nsCOMPtr<nsIDocument> doc = do_QueryInterface(domDoc);
-  NS_ENSURE_TRUE(doc, nsnull);
+  NS_ENSURE_TRUE(doc, nullptr);
   nsCOMPtr<nsIWebNavigation> webNav = do_GetInterface(doc->GetWindow());
   nsCOMPtr<nsIDocShell> docShell = do_QueryInterface(webNav);
   return docShell;
@@ -1144,26 +1110,26 @@ nsFormFillController::GetWindowForDocShell(nsIDocShell *aDocShell)
 {
   nsCOMPtr<nsIContentViewer> contentViewer;
   aDocShell->GetContentViewer(getter_AddRefs(contentViewer));
-  NS_ENSURE_TRUE(contentViewer, nsnull);
+  NS_ENSURE_TRUE(contentViewer, nullptr);
 
   nsCOMPtr<nsIDOMDocument> domDoc;
   contentViewer->GetDOMDocument(getter_AddRefs(domDoc));
   nsCOMPtr<nsIDocument> doc = do_QueryInterface(domDoc);
-  NS_ENSURE_TRUE(doc, nsnull);
+  NS_ENSURE_TRUE(doc, nullptr);
 
   return doc->GetWindow();
 }
 
-PRInt32
+int32_t
 nsFormFillController::GetIndexOfDocShell(nsIDocShell *aDocShell)
 {
   if (!aDocShell)
     return -1;
 
   // Loop through our cached docShells looking for the given docShell
-  PRUint32 count;
+  uint32_t count;
   mDocShells->Count(&count);
-  for (PRUint32 i = 0; i < count; ++i) {
+  for (uint32_t i = 0; i < count; ++i) {
     nsCOMPtr<nsIDocShell> docShell;
     mDocShells->GetElementAt(i, getter_AddRefs(docShell));
     if (docShell == aDocShell)
