@@ -21,18 +21,19 @@ function run_test()
   let search = Services.search;
 
   do_test_pending();
-  afterCommit(
-    function()
-    {
-      //Check that search-metadata.json has been created
-      let metadata = gProfD.clone();
-      metadata.append("search-metadata.json");
-      do_check_true(metadata.exists());
 
-      removeMetadata();
-      do_test_finished();
-    }
-  );
+  afterCommit(function commit_complete() {
+    // Check that search-metadata.json has been created
+    let metadata = gProfD.clone();
+    metadata.append("search-metadata.json");
+    do_check_true(metadata.exists());
 
-  search.getEngines();
+    removeMetadata();
+    do_test_finished();
+  });
+
+  search.init(function ss_initialized(rv) {
+    do_check_true(Components.isSuccessCode(rv));
+    search.getEngines();
+  });
 }
