@@ -1,54 +1,14 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code. This file was split
- * from xpfe/appshell/src/nsAppShellService.cpp
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Pierre Phaneuf <pp@ludusdesign.com>
- *   Robert O'Callahan <roc+moz@cs.cmu.edu>
- *   Benjamin Smedberg <bsmedberg@covad.net>
- *   Daniel Brooks <db48x@db48x.net>
- *   Taras Glek <tglek@mozilla.com>
- *   Landry Breuil <landry@openbsd.org>
- *   David Rajchenbach-Teller <dteller@mozilla.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsAppStartup.h"
 
 #include "nsIAppShellService.h"
 #include "nsPIDOMWindow.h"
 #include "nsIInterfaceRequestor.h"
-#include "nsILocalFile.h"
+#include "nsIFile.h"
 #include "nsIObserverService.h"
 #include "nsIPrefBranch.h"
 #include "nsIPrefService.h"
@@ -78,6 +38,8 @@
 #include "nsIXPConnect.h"
 #include "jsapi.h"
 #include "prenv.h"
+#include "nsAppDirectoryServiceDefs.h"
+#include "mozilla/mozPoisonWrite.h"
 
 #if defined(XP_WIN)
 #include <windows.h>
@@ -129,6 +91,10 @@ static NS_DEFINE_CID(kAppShellCID, NS_APPSHELL_CID);
 #define NS_SESSION_STORE_WINDOW_RESTORED_EVENT_CID \
   { 0x917B96B1, 0xECAD, 0x4DAB, \
   { 0xA7, 0x60, 0x8D, 0x49, 0x02, 0x77, 0x48, 0xAE} }
+// {26D1E091-0AE7-4F49-A554-4214445C505C}
+#define NS_XPCOM_SHUTDOWN_EVENT_CID \
+  { 0x26D1E091, 0x0AE7, 0x4F49, \
+  { 0xA5, 0x54, 0x42, 0x14, 0x44, 0x5C, 0x50, 0x5C} }
 
 static NS_DEFINE_CID(kApplicationTracingCID,
   NS_APPLICATION_TRACING_CID);
@@ -136,11 +102,13 @@ static NS_DEFINE_CID(kPlacesInitCompleteCID,
   NS_PLACES_INIT_COMPLETE_EVENT_CID);
 static NS_DEFINE_CID(kSessionStoreWindowRestoredCID,
   NS_SESSION_STORE_WINDOW_RESTORED_EVENT_CID);
+static NS_DEFINE_CID(kXPCOMShutdownCID,
+  NS_XPCOM_SHUTDOWN_EVENT_CID);  
 #endif //defined(XP_WIN)
 
 using namespace mozilla;
 
-PRUint32 gRestartMode = 0;
+uint32_t gRestartMode = 0;
 
 class nsAppExitEvent : public nsRunnable {
 private:
@@ -172,7 +140,9 @@ nsAppStartup::nsAppStartup() :
   mRestart(false),
   mInterrupted(false),
   mIsSafeModeNecessary(false),
-  mStartupCrashTrackingEnded(false)
+  mStartupCrashTrackingEnded(false),
+  mCachedShutdownTime(false),
+  mLastShutdownTime(0)
 { }
 
 
@@ -202,6 +172,7 @@ nsAppStartup::Init()
   os->AddObserver(this, "xul-window-destroyed", true);
 
 #if defined(XP_WIN)
+  os->AddObserver(this, "xpcom-shutdown", true);
   os->AddObserver(this, "places-init-complete", true);
   // This last event is only interesting to us for xperf-based measures
 
@@ -227,6 +198,13 @@ nsAppStartup::Init()
                NS_LITERAL_CSTRING("sessionstore-windows-restored"));
     NS_WARN_IF_FALSE(mSessionWindowRestoredProbe,
                      "Cannot initialize probe 'sessionstore-windows-restored'");
+                     
+    mXPCOMShutdownProbe =
+      mProbesManager->
+      GetProbe(kXPCOMShutdownCID,
+               NS_LITERAL_CSTRING("xpcom-shutdown"));
+    NS_WARN_IF_FALSE(mXPCOMShutdownProbe,
+                     "Cannot initialize probe 'xpcom-shutdown'");
 
     rv = mProbesManager->StartSession();
     NS_WARN_IF_FALSE(NS_SUCCEEDED(rv),
@@ -300,11 +278,88 @@ nsAppStartup::Run(void)
   return mRestart ? NS_SUCCESS_RESTART_APP : NS_OK;
 }
 
+static TimeStamp gRecordedShutdownStartTime;
+static bool gAlreadyFreedShutdownTimeFileName = false;
+static char *gRecordedShutdownTimeFileName = NULL;
+
+static char *
+GetShutdownTimeFileName()
+{
+  if (gAlreadyFreedShutdownTimeFileName) {
+    return NULL;
+  }
+
+  if (!gRecordedShutdownTimeFileName) {
+    nsCOMPtr<nsIFile> mozFile;
+    NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(mozFile));
+    if (!mozFile)
+      return NULL;
+
+    mozFile->AppendNative(NS_LITERAL_CSTRING("Telemetry.ShutdownTime.txt"));
+    nsCAutoString nativePath;
+    nsresult rv = mozFile->GetNativePath(nativePath);
+    if (!NS_SUCCEEDED(rv))
+      return NULL;
+
+    gRecordedShutdownTimeFileName = PL_strdup(nativePath.get());
+  }
+
+  return gRecordedShutdownTimeFileName;
+}
+
+static void
+RecordShutdownStartTimeStamp() {
+  if (!Telemetry::CanRecord())
+    return;
+
+  gRecordedShutdownStartTime = TimeStamp::Now();
+
+  GetShutdownTimeFileName();
+}
+
+namespace mozilla {
+void
+RecordShutdownEndTimeStamp() {
+  if (!gRecordedShutdownTimeFileName || gAlreadyFreedShutdownTimeFileName)
+    return;
+
+  nsCString name(gRecordedShutdownTimeFileName);
+  PL_strfree(gRecordedShutdownTimeFileName);
+  gRecordedShutdownTimeFileName = NULL;
+  gAlreadyFreedShutdownTimeFileName = true;
+
+  nsCString tmpName = name;
+  tmpName += ".tmp";
+  FILE *f = fopen(tmpName.get(), "w");
+  if (!f)
+    return;
+  // On a normal release build this should be called just before
+  // calling _exit, but on a debug build or when the user forces a full
+  // shutdown this is called as late as possible, so we have to
+  // white list this write as write poisoning will be enabled.
+  int fd = fileno(f);
+  MozillaRegisterDebugFD(fd);
+
+  TimeStamp now = TimeStamp::Now();
+  MOZ_ASSERT(now >= gRecordedShutdownStartTime);
+  TimeDuration diff = now - gRecordedShutdownStartTime;
+  uint32_t diff2 = diff.ToMilliseconds();
+  int written = fprintf(f, "%d\n", diff2);
+  MozillaUnRegisterDebugFILE(f);
+  int rv = fclose(f);
+  if (written < 0 || rv != 0) {
+    PR_Delete(tmpName.get());
+    return;
+  }
+  PR_Delete(name.get());
+  PR_Rename(tmpName.get(), name.get());
+}
+}
 
 NS_IMETHODIMP
-nsAppStartup::Quit(PRUint32 aMode)
+nsAppStartup::Quit(uint32_t aMode)
 {
-  PRUint32 ferocity = (aMode & 0xF);
+  uint32_t ferocity = (aMode & 0xF);
 
   // Quit the application. We will asynchronously call the appshell's
   // Exit() method via nsAppExitEvent to allow one last pass
@@ -315,6 +370,8 @@ nsAppStartup::Quit(PRUint32 aMode)
   if (mShuttingDown)
     return NS_OK;
 
+  RecordShutdownStartTimeStamp();
+
   // If we're considering quitting, we will only do so if:
   if (ferocity == eConsiderQuit) {
     if (mConsiderQuitStopper == 0) {
@@ -350,7 +407,7 @@ nsAppStartup::Quit(PRUint32 aMode)
     nsCOMPtr<nsISimpleEnumerator> windowEnumerator;
     nsCOMPtr<nsIWindowMediator> mediator (do_GetService(NS_WINDOWMEDIATOR_CONTRACTID));
     if (mediator) {
-      mediator->GetEnumerator(nsnull, getter_AddRefs(windowEnumerator));
+      mediator->GetEnumerator(nullptr, getter_AddRefs(windowEnumerator));
       if (windowEnumerator) {
         bool more;
         while (windowEnumerator->HasMoreElements(&more), more) {
@@ -373,7 +430,7 @@ nsAppStartup::Quit(PRUint32 aMode)
 
     if (mRestart) {
       // Firefox-restarts reuse the process. Process start-time isn't a useful indicator of startup time
-      PR_SetEnv(PR_smprintf("MOZ_APP_RESTART=%lld", (PRInt64) PR_Now() / PR_USEC_PER_MSEC));
+      PR_SetEnv(PR_smprintf("MOZ_APP_RESTART=%lld", (int64_t) PR_Now() / PR_USEC_PER_MSEC));
     }
 
     obsService = mozilla::services::GetObserverService();
@@ -385,7 +442,7 @@ nsAppStartup::Quit(PRUint32 aMode)
       ExitLastWindowClosingSurvivalArea();
 #endif
       if (obsService)
-        obsService->NotifyObservers(nsnull, "quit-application-granted", nsnull);
+        obsService->NotifyObservers(nullptr, "quit-application-granted", nullptr);
     }
 
     /* Enumerate through each open window and close it. It's important to do
@@ -404,7 +461,7 @@ nsAppStartup::Quit(PRUint32 aMode)
            method hasn't had a chance to wrap itself up yet. So give up.
            We'll return (with eConsiderQuit) as the remaining windows are
            closed. */
-        mediator->GetEnumerator(nsnull, getter_AddRefs(windowEnumerator));
+        mediator->GetEnumerator(nullptr, getter_AddRefs(windowEnumerator));
         if (windowEnumerator) {
           bool more;
           while (windowEnumerator->HasMoreElements(&more), more) {
@@ -436,7 +493,7 @@ nsAppStartup::Quit(PRUint32 aMode)
     if (obsService) {
       NS_NAMED_LITERAL_STRING(shutdownStr, "shutdown");
       NS_NAMED_LITERAL_STRING(restartStr, "restart");
-      obsService->NotifyObservers(nsnull, "quit-application",
+      obsService->NotifyObservers(nullptr, "quit-application",
         mRestart ? restartStr.get() : shutdownStr.get());
     }
 
@@ -474,7 +531,7 @@ nsAppStartup::CloseAllWindows()
 
   nsCOMPtr<nsISimpleEnumerator> windowEnumerator;
 
-  mediator->GetEnumerator(nsnull, getter_AddRefs(windowEnumerator));
+  mediator->GetEnumerator(nullptr, getter_AddRefs(windowEnumerator));
 
   if (!windowEnumerator)
     return;
@@ -512,6 +569,47 @@ nsAppStartup::ExitLastWindowClosingSurvivalArea(void)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsAppStartup::GetLastShutdownDuration(uint32_t *aResult)
+{
+  // We make this check so that GetShutdownTimeFileName() doesn't get
+  // called; calling that function without telemetry enabled violates
+  // assumptions that the write-the-shutdown-timestamp machinery makes.
+  if (!Telemetry::CanRecord()) {
+    *aResult = 0;
+    return NS_OK;
+  }
+
+  if (!mCachedShutdownTime) {
+    const char *filename = GetShutdownTimeFileName();
+
+    if (!filename) {
+      *aResult = 0;
+      return NS_OK;
+    }
+
+    FILE *f = fopen(filename, "r");
+    if (!f) {
+      *aResult = 0;
+      return NS_OK;
+    }
+
+    int shutdownTime;
+    int r = fscanf(f, "%d\n", &shutdownTime);
+    fclose(f);
+    if (r != 1) {
+      *aResult = 0;
+      return NS_OK;
+    }
+
+    mLastShutdownTime = shutdownTime;
+    mCachedShutdownTime = true;
+  }
+
+  *aResult = mLastShutdownTime;
+  return NS_OK;
+}
+
 //
 // nsAppStartup->nsIAppStartup2
 //
@@ -543,7 +641,7 @@ nsAppStartup::GetInterrupted(bool *aInterrupted)
 
 NS_IMETHODIMP
 nsAppStartup::CreateChromeWindow(nsIWebBrowserChrome *aParent,
-                                 PRUint32 aChromeFlags,
+                                 uint32_t aChromeFlags,
                                  nsIWebBrowserChrome **_retval)
 {
   bool cancel;
@@ -557,8 +655,8 @@ nsAppStartup::CreateChromeWindow(nsIWebBrowserChrome *aParent,
 
 NS_IMETHODIMP
 nsAppStartup::CreateChromeWindow2(nsIWebBrowserChrome *aParent,
-                                  PRUint32 aChromeFlags,
-                                  PRUint32 aContextFlags,
+                                  uint32_t aChromeFlags,
+                                  uint32_t aContextFlags,
                                   nsIURI *aURI,
                                   bool *aCancel,
                                   nsIWebBrowserChrome **_retval)
@@ -643,6 +741,10 @@ nsAppStartup::Observe(nsISupports *aSubject,
     if (mPlacesInitCompleteProbe) {
       mPlacesInitCompleteProbe->Trigger();
     }
+  } else if (!strcmp(aTopic, "xpcom-shutdown")) {
+    if (mXPCOMShutdownProbe) {
+      mXPCOMShutdownProbe->Trigger();
+    }
 #endif //defined(XP_WIN)
   } else {
     NS_ERROR("Unexpected observer topic.");
@@ -652,7 +754,7 @@ nsAppStartup::Observe(nsISupports *aSubject,
 }
 
 #if defined(LINUX) || defined(ANDROID)
-static PRUint64 
+static uint64_t 
 JiffiesSinceBoot(const char *file)
 {
   char stat[512];
@@ -665,7 +767,7 @@ JiffiesSinceBoot(const char *file)
     return 0;
   stat[n] = 0;
   
-  long long unsigned starttime = 0; // instead of PRUint64 to keep GCC quiet
+  long long unsigned starttime = 0; // instead of uint64_t to keep GCC quiet
   
   char *s = strrchr(stat, ')');
   if (!s)
@@ -682,6 +784,8 @@ JiffiesSinceBoot(const char *file)
 static void
 ThreadedCalculateProcessCreationTimestamp(void *aClosure)
 {
+  PR_SetCurrentThreadName("Startup Timer");
+
   PRTime now = PR_Now();
   long hz = sysconf(_SC_CLK_TCK);
   if (!hz)
@@ -690,8 +794,8 @@ ThreadedCalculateProcessCreationTimestamp(void *aClosure)
   char thread_stat[40];
   sprintf(thread_stat, "/proc/self/task/%d/stat", (pid_t) syscall(__NR_gettid));
   
-  PRUint64 thread_jiffies = JiffiesSinceBoot(thread_stat);
-  PRUint64 self_jiffies = JiffiesSinceBoot("/proc/self/stat");
+  uint64_t thread_jiffies = JiffiesSinceBoot(thread_stat);
+  uint64_t self_jiffies = JiffiesSinceBoot("/proc/self/stat");
   
   if (!thread_jiffies || !self_jiffies)
     return;
@@ -723,7 +827,7 @@ CalculateProcessCreationTimestamp()
   if (!success)
     return 0;
   // copied from NSPR _PR_FileTimeToPRTime
-  PRUint64 timestamp = 0;
+  uint64_t timestamp = 0;
   CopyMemory(&timestamp, &start, sizeof(PRTime));
 #ifdef __GNUC__
   timestamp = (timestamp - 116444736000000000LL) / 10LL;
@@ -831,8 +935,8 @@ nsAppStartup::GetAutomaticSafeModeNecessary(bool *_retval)
 NS_IMETHODIMP
 nsAppStartup::TrackStartupCrashBegin(bool *aIsSafeModeNecessary)
 {
-  const PRInt32 MAX_TIME_SINCE_STARTUP = 6 * 60 * 60 * 1000;
-  const PRInt32 MAX_STARTUP_BUFFER = 10;
+  const int32_t MAX_TIME_SINCE_STARTUP = 6 * 60 * 60 * 1000;
+  const int32_t MAX_STARTUP_BUFFER = 10;
   nsresult rv;
 
   mStartupCrashTrackingEnded = false;
@@ -853,7 +957,7 @@ nsAppStartup::TrackStartupCrashBegin(bool *aIsSafeModeNecessary)
 
   xr->GetInSafeMode(&inSafeMode);
 
-  PRInt64 replacedLockTime;
+  PRTime replacedLockTime;
   rv = xr->GetReplacedLockTime(&replacedLockTime);
 
   if (NS_FAILED(rv) || !replacedLockTime) {
@@ -864,11 +968,11 @@ nsAppStartup::TrackStartupCrashBegin(bool *aIsSafeModeNecessary)
   }
 
   // check whether safe mode is necessary
-  PRInt32 maxResumedCrashes = -1;
+  int32_t maxResumedCrashes = -1;
   rv = Preferences::GetInt(kPrefMaxResumedCrashes, &maxResumedCrashes);
   NS_ENSURE_SUCCESS(rv, NS_OK);
 
-  PRInt32 recentCrashes = 0;
+  int32_t recentCrashes = 0;
   Preferences::GetInt(kPrefRecentCrashes, &recentCrashes);
   mIsSafeModeNecessary = (recentCrashes > maxResumedCrashes && maxResumedCrashes != -1);
 
@@ -882,11 +986,11 @@ nsAppStartup::TrackStartupCrashBegin(bool *aIsSafeModeNecessary)
   }
 
   // time of last successful startup
-  PRInt32 lastSuccessfulStartup;
+  int32_t lastSuccessfulStartup;
   rv = Preferences::GetInt(kPrefLastSuccess, &lastSuccessfulStartup);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  PRInt32 lockSeconds = (PRInt32)(replacedLockTime / PR_MSEC_PER_SEC);
+  int32_t lockSeconds = (int32_t)(replacedLockTime / PR_MSEC_PER_SEC);
 
   // started close enough to good startup so call it good
   if (lockSeconds <= lastSuccessfulStartup + MAX_STARTUP_BUFFER
@@ -924,7 +1028,7 @@ nsAppStartup::TrackStartupCrashBegin(bool *aIsSafeModeNecessary)
   mIsSafeModeNecessary = (recentCrashes > maxResumedCrashes && maxResumedCrashes != -1);
 
   nsCOMPtr<nsIPrefService> prefs = Preferences::GetService();
-  rv = prefs->SavePrefFile(nsnull); // flush prefs to disk since we are tracking crashes
+  rv = prefs->SavePrefFile(nullptr); // flush prefs to disk since we are tracking crashes
   NS_ENSURE_SUCCESS(rv, rv);
 
   GetAutomaticSafeModeNecessary(aIsSafeModeNecessary);
@@ -953,7 +1057,7 @@ nsAppStartup::TrackStartupCrashEnd()
   if (mainTime <= 0) {
     NS_WARNING("Could not get StartupTimeline::MAIN time.");
   } else {
-    PRInt32 lockFileTime = (PRInt32)(mainTime / PR_USEC_PER_SEC);
+    int32_t lockFileTime = (int32_t)(mainTime / PR_USEC_PER_SEC);
     rv = Preferences::SetInt(kPrefLastSuccess, lockFileTime);
     if (NS_FAILED(rv)) NS_WARNING("Could not set startup crash detection pref.");
   }
@@ -961,8 +1065,8 @@ nsAppStartup::TrackStartupCrashEnd()
   if (inSafeMode && mIsSafeModeNecessary) {
     // On a successful startup in automatic safe mode, allow the user one more crash
     // in regular mode before returning to safe mode.
-    PRInt32 maxResumedCrashes = 0;
-    PRInt32 prefType;
+    int32_t maxResumedCrashes = 0;
+    int32_t prefType;
     rv = Preferences::GetDefaultRootBranch()->GetPrefType(kPrefMaxResumedCrashes, &prefType);
     NS_ENSURE_SUCCESS(rv, rv);
     if (prefType == nsIPrefBranch::PREF_INT) {
@@ -977,13 +1081,13 @@ nsAppStartup::TrackStartupCrashEnd()
     if (NS_FAILED(rv)) NS_WARNING("Could not clear startup crash count.");
   }
   nsCOMPtr<nsIPrefService> prefs = Preferences::GetService();
-  rv = prefs->SavePrefFile(nsnull); // flush prefs to disk since we are tracking crashes
+  rv = prefs->SavePrefFile(nullptr); // flush prefs to disk since we are tracking crashes
 
   return rv;
 }
 
 NS_IMETHODIMP
-nsAppStartup::RestartInSafeMode(PRUint32 aQuitMode)
+nsAppStartup::RestartInSafeMode(uint32_t aQuitMode)
 {
   PR_SetEnv("MOZ_SAFE_MODE_RESTART=1");
   this->Quit(aQuitMode | nsIAppStartup::eRestart);
