@@ -19,22 +19,6 @@ var gProvider;
 var gServer;
 var gAddonInstalled = false;
 
-var channel = "default";
-try {
-  channel = Services.prefs.getCharPref("app.update.channel");
-}
-catch (e) { }
-if (channel != "aurora" &&
-    channel != "beta" &&
-    channel != "release") {
-  var version = "nightly";
-}
-else {
-  version = Services.appinfo.version.replace(/^([^\.]+\.[0-9]+[a-z]*).*/gi, "$1");
-}
-
-const COMPATIBILITY_PREF = "extensions.checkCompatibility." + version;
-
 function test() {
   requestLongerTimeout(2);
   // Turn on searching for this test
@@ -92,8 +76,7 @@ function end_test() {
     installedAddon.uninstall();
 
     AddonManager.getAllInstalls(function(aInstallsList) {
-      for (var i = 0; i < aInstallsList.length; i++) {
-        var install = aInstallsList[i];
+      for (var install of aInstallsList) {
         var sourceURI = install.sourceURI.spec;
         if (sourceURI == REMOTE_INSTALL_URL ||
             sourceURI.match(/^http:\/\/example\.com\/(.+)\.xpi$/) != null)
@@ -177,8 +160,7 @@ function get_actual_results() {
   var rows = list.getElementsByTagName("richlistitem");
 
   var results = [];
-  for (var i = 0; i < rows.length; i++) {
-    var item = rows[i];
+  for (var item of rows) {
 
     // Only consider items that are currently showing
     var style = gManagerWindow.document.defaultView.getComputedStyle(item, "");
@@ -380,8 +362,7 @@ function get_addon_item(aName) {
   var id = aName + "@tests.mozilla.org";
   var list = gManagerWindow.document.getElementById("search-list");
   var rows = list.getElementsByTagName("richlistitem");
-  for (var i = 0; i < rows.length; i++) {
-    var row = rows[i];
+  for (var row of rows) {
     if (row.mAddon && row.mAddon.id == id)
       return row;
   }
@@ -400,8 +381,7 @@ function get_install_item(aName) {
   var sourceURI = "http://example.com/" + aName + ".xpi";
   var list = gManagerWindow.document.getElementById("search-list");
   var rows = list.getElementsByTagName("richlistitem");
-  for (var i = 0; i < rows.length; i++) {
-    var row = rows[i];
+  for (var row of rows) {
     if (row.mInstall && row.mInstall.sourceURI.spec == sourceURI)
       return row;
   }
@@ -441,8 +421,7 @@ add_test(function() {
 
     var list = gManagerWindow.document.getElementById("search-list");
     var results = get_actual_results();
-    for (var i = 0; i < results.length; i++) {
-      var result = results[i];
+    for (var result of results) {
       var installBtn = get_install_button(result.item);
       is(installBtn.hidden, result.name.indexOf("remote") != 0,
          "Install button should only be showing for remote items");
@@ -608,16 +587,16 @@ add_test(function() {
 
 // Tests that incompatible add-ons are shown with a warning if compatibility checking is disabled
 add_test(function() {
-  Services.prefs.setBoolPref(COMPATIBILITY_PREF, false);
+  AddonManager.checkCompatibility = false;
   search("incompatible", false, function() {
     var item = get_addon_item("remote5");
     is_element_visible(item, "Incompatible addon should be visible");
     is(item.getAttribute("notification"), "warning", "Compatibility warning should be shown");
 
-    var item = get_addon_item("remote6");
+    item = get_addon_item("remote6");
     is(item, null, "Addon incompatible with the product should not be visible");
 
-    Services.prefs.clearUserPref(COMPATIBILITY_PREF);
+    AddonManager.checkCompatibility = true;
     run_next_test();
   });
 });
