@@ -1,39 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifndef nsView_h___
 #define nsView_h___
@@ -44,6 +12,7 @@
 #include "nsCRT.h"
 #include "nsIFactory.h"
 #include "nsEvent.h"
+#include "nsIWidgetListener.h"
 #include <stdio.h>
 
 //mmptemp
@@ -51,10 +20,11 @@
 class nsIViewManager;
 class nsViewManager;
 
-class nsView : public nsIView
+class nsView : public nsIView,
+               public nsIWidgetListener
 {
 public:
-  nsView(nsViewManager* aViewManager = nsnull,
+  nsView(nsViewManager* aViewManager = nullptr,
          nsViewVisibility aVisibility = nsViewVisibility_kShow);
 
   NS_DECL_AND_IMPL_ZEROING_OPERATOR_NEW
@@ -93,7 +63,7 @@ public:
    * relative to the view's siblings.
    * @param zindex new z depth
    */
-  void SetZIndex(bool aAuto, PRInt32 aZIndex, bool aTopMost);
+  void SetZIndex(bool aAuto, int32_t aZIndex, bool aTopMost);
 
   /**
    * Set/Get whether the view "floats" above all other views,
@@ -143,7 +113,7 @@ public:
   nsView* GetParent() const { return mParent; }
   nsViewManager* GetViewManager() const { return mViewManager; }
   // These are superseded by a better interface in nsIView
-  PRInt32 GetZIndex() const { return mZIndex; }
+  int32_t GetZIndex() const { return mZIndex; }
   bool GetZIndexIsAuto() const { return (mVFlags & NS_VIEW_FLAG_AUTO_ZINDEX) != 0; }
   // Same as GetBounds but converts to parent appunits if they are different.
   nsRect GetBoundsInParentUnits() const;
@@ -170,8 +140,8 @@ public:
   void SetParent(nsView *aParent) { mParent = aParent; }
   void SetNextSibling(nsView *aSibling) { mNextSibling = aSibling; }
 
-  PRUint32 GetViewFlags() const { return mVFlags; }
-  void SetViewFlags(PRUint32 aFlags) { mVFlags = aFlags; }
+  uint32_t GetViewFlags() const { return mVFlags; }
+  void SetViewFlags(uint32_t aFlags) { mVFlags = aFlags; }
 
   void SetTopMost(bool aTopMost) { aTopMost ? mVFlags |= NS_VIEW_FLAG_TOPMOST : mVFlags &= ~NS_VIEW_FLAG_TOPMOST; }
   bool IsTopMost() { return((mVFlags & NS_VIEW_FLAG_TOPMOST) != 0); }
@@ -187,12 +157,23 @@ public:
   // released if it points to any view in this view hierarchy.
   void InvalidateHierarchy(nsViewManager *aViewManagerParent);
 
+  // nsIWidgetListener
+  virtual nsIPresShell* GetPresShell();
+  virtual nsIView* GetView() { return this; }
+  bool WindowMoved(nsIWidget* aWidget, int32_t x, int32_t y);
+  bool WindowResized(nsIWidget* aWidget, int32_t aWidth, int32_t aHeight);
+  bool RequestWindowClose(nsIWidget* aWidget);
+  void WillPaintWindow(nsIWidget* aWidget, bool aWillSendDidPaint);
+  bool PaintWindow(nsIWidget* aWidget, nsIntRegion aRegion, bool aSentDidPaint, bool aWillSendDidPaint);
+  void DidPaintWindow();
+  nsEventStatus HandleEvent(nsGUIEvent* aEvent, bool aUseAttachedEvents);
+
   virtual ~nsView();
 
   nsPoint GetOffsetTo(const nsView* aOther) const;
   nsIWidget* GetNearestWidget(nsPoint* aOffset) const;
-  nsPoint GetOffsetTo(const nsView* aOther, const PRInt32 aAPD) const;
-  nsIWidget* GetNearestWidget(nsPoint* aOffset, const PRInt32 aAPD) const;
+  nsPoint GetOffsetTo(const nsView* aOther, const int32_t aAPD) const;
+  nsIWidget* GetNearestWidget(nsPoint* aOffset, const int32_t aAPD) const;
 
 protected:
   // Do the actual work of ResetWidgetBounds, unconditionally.  Don't
