@@ -1,39 +1,7 @@
 /* -*- Mode: C++; tab-width: 40; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifndef nsIWidget_h__
 #define nsIWidget_h__
@@ -47,7 +15,6 @@
 #include "nsStringGlue.h"
 
 #include "prthread.h"
-#include "Layers.h"
 #include "nsEvent.h"
 #include "nsCOMPtr.h"
 #include "nsITheme.h"
@@ -55,6 +22,7 @@
 #include "nsWidgetInitData.h"
 #include "nsTArray.h"
 #include "nsXULAppAPI.h"
+#include "LayersTypes.h"
 
 // forward declarations
 class   nsFontMetrics;
@@ -67,12 +35,14 @@ class   imgIContainer;
 class   gfxASurface;
 class   nsIContent;
 class   ViewWrapper;
+class   nsIWidgetListener;
 
 namespace mozilla {
 namespace dom {
-class PBrowserChild;
+class TabChild;
 }
 namespace layers {
+class LayerManager;
 class PLayersChild;
 }
 }
@@ -118,8 +88,8 @@ typedef nsEventStatus (* EVENT_CALLBACK)(nsGUIEvent *event);
 #endif
 
 #define NS_IWIDGET_IID \
-  { 0x7c7ff2ff, 0x61f9, 0x4240, \
-    { 0xaa, 0x58, 0x74, 0xb0, 0xcd, 0xa9, 0xe3, 0x05 } }
+  { 0xb8f43b25, 0x9036, 0x44e7, \
+    { 0xaa, 0xe2, 0x33, 0x76, 0x6c, 0x35, 0x91, 0xfc } }
 
 /*
  * Window shadow styles
@@ -132,13 +102,6 @@ typedef nsEventStatus (* EVENT_CALLBACK)(nsGUIEvent *event);
 #define NS_STYLE_WINDOW_SHADOW_TOOLTIP          3
 #define NS_STYLE_WINDOW_SHADOW_SHEET            4
 
-/**
- * nsIWidget::OnIMEFocusChange should be called during blur,
- * but other OnIME*Change methods should not be called
- */
-#define NS_SUCCESS_IME_NO_UPDATES \
-    NS_ERROR_GENERATE_SUCCESS(NS_ERROR_MODULE_WIDGET, 1)
-
 /**
  * Cursor types.
  */
@@ -194,7 +157,6 @@ enum nsTopLevelWidgetZPlacement { // for PlaceBehind()
   eZPlacementTop          // top of the window stack
 };
 
-
 /**
  * Preference for receiving IME updates
  *
@@ -319,6 +281,9 @@ struct InputContext {
   /* The type of the input if the input is a html input field */
   nsString mHTMLInputType;
 
+  /* The type of the inputmode */
+  nsString mHTMLInputInputmode;
+
   /* A hint for the action that is performed when the input is submitted */
   nsString mActionHint;
 };
@@ -382,6 +347,27 @@ struct InputContextAction {
   }
 };
 
+/**
+ * Size constraints for setting the minimum and maximum size of a widget.
+ * Values are in device pixels.
+ */
+struct SizeConstraints {
+  SizeConstraints()
+    : mMaxSize(NS_MAXSIZE, NS_MAXSIZE)
+  {
+  }
+
+  SizeConstraints(nsIntSize aMinSize,
+                  nsIntSize aMaxSize)
+  : mMinSize(aMinSize),
+    mMaxSize(aMaxSize)
+  {
+  }
+
+  nsIntSize mMinSize;
+  nsIntSize mMaxSize;
+};
+
 } // namespace widget
 } // namespace mozilla
 
@@ -391,24 +377,25 @@ struct InputContextAction {
  */
 class nsIWidget : public nsISupports {
   protected:
-    typedef mozilla::dom::PBrowserChild PBrowserChild;
+    typedef mozilla::dom::TabChild TabChild;
 
   public:
     typedef mozilla::layers::LayerManager LayerManager;
-    typedef LayerManager::LayersBackend LayersBackend;
+    typedef mozilla::layers::LayersBackend LayersBackend;
     typedef mozilla::layers::PLayersChild PLayersChild;
     typedef mozilla::widget::IMEState IMEState;
     typedef mozilla::widget::InputContext InputContext;
     typedef mozilla::widget::InputContextAction InputContextAction;
+    typedef mozilla::widget::SizeConstraints SizeConstraints;
 
     // Used in UpdateThemeGeometries.
     struct ThemeGeometry {
       // The -moz-appearance value for the themed widget
-      PRUint8 mWidgetType;
+      uint8_t mWidgetType;
       // The device-pixel rect within the window for the themed widget
       nsIntRect mRect;
 
-      ThemeGeometry(PRUint8 aWidgetType, const nsIntRect& aRect)
+      ThemeGeometry(uint8_t aWidgetType, const nsIntRect& aRect)
        : mWidgetType(aWidgetType)
        , mRect(aRect)
       { }
@@ -417,8 +404,8 @@ class nsIWidget : public nsISupports {
     NS_DECLARE_STATIC_IID_ACCESSOR(NS_IWIDGET_IID)
 
     nsIWidget()
-      : mLastChild(nsnull)
-      , mPrevSibling(nsnull)
+      : mLastChild(nullptr)
+      , mPrevSibling(nullptr)
     {}
 
         
@@ -444,7 +431,6 @@ class nsIWidget : public nsISupports {
      * @param     aParent       parent nsIWidget
      * @param     aNativeParent native parent widget
      * @param     aRect         the widget dimension
-     * @param     aHandleEventFunction the event handler callback function
      * @param     aContext
      * @param     aInitData     data that is used for widget initialization
      *
@@ -452,9 +438,8 @@ class nsIWidget : public nsISupports {
     NS_IMETHOD Create(nsIWidget        *aParent,
                       nsNativeWidget   aNativeParent,
                       const nsIntRect  &aRect,
-                      EVENT_CALLBACK   aHandleEventFunction,
                       nsDeviceContext *aContext,
-                      nsWidgetInitData *aInitData = nsnull) = 0;
+                      nsWidgetInitData *aInitData = nullptr) = 0;
 
     /**
      * Allocate, initialize, and return a widget that is a child of
@@ -474,48 +459,42 @@ class nsIWidget : public nsISupports {
      */
     virtual already_AddRefed<nsIWidget>
     CreateChild(const nsIntRect  &aRect,
-                EVENT_CALLBACK   aHandleEventFunction,
                 nsDeviceContext  *aContext,
-                nsWidgetInitData *aInitData = nsnull,
+                nsWidgetInitData *aInitData = nullptr,
                 bool             aForceUseIWidgetParent = false) = 0;
 
-    /**
-     * Set the event callback for a widget. If a device context is not
-     * provided then the existing device context will remain, it will
-     * not be nulled out.
-     */
-    NS_IMETHOD SetEventCallback(EVENT_CALLBACK aEventFunction,
-                                nsDeviceContext *aContext) = 0;
-
     /**
      * Attach to a top level widget. 
      *
      * In cases where a top level chrome widget is being used as a content
-     * container, attach a secondary event callback and update the device
-     * context. The primary event callback will continue to be called, so the
-     * owning base window will continue to function.
+     * container, attach a secondary listener and update the device
+     * context. The primary widget listener will continue to be called for
+     * notifications relating to the top-level window, whereas other
+     * notifications such as painting and events will instead be called via
+     * the attached listener. SetAttachedWidgetListener should be used to
+     * assign the attached listener.
      *
-     * aViewEventFunction Event callback that will receive mirrored
-     *                    events.
+     * aUseAttachedEvents if true, events are sent to the attached listener
+     * instead of the normal listener.
      * aContext The new device context for the view
      */
-    NS_IMETHOD AttachViewToTopLevel(EVENT_CALLBACK aViewEventFunction,
+    NS_IMETHOD AttachViewToTopLevel(bool aUseAttachedEvents,
                                     nsDeviceContext *aContext) = 0;
 
     /**
-     * Accessor functions to get and set secondary client data. Used by
+     * Accessor functions to get and set the attached listener. Used by
      * nsIView in connection with AttachViewToTopLevel above.
      */
-    NS_IMETHOD SetAttachedViewPtr(ViewWrapper* aViewWrapper) = 0;
-    virtual ViewWrapper* GetAttachedViewPtr() = 0;
+    virtual void SetAttachedWidgetListener(nsIWidgetListener* aListener) = 0;
+    virtual nsIWidgetListener* GetAttachedWidgetListener() = 0;
 
     /**
-     * Accessor functions to get and set the client data associated with the
-     * widget.
+     * Accessor functions to get and set the listener which handles various
+     * actions for the widget.
      */
     //@{
-    NS_IMETHOD  GetClientData(void*& aClientData) = 0;
-    NS_IMETHOD  SetClientData(void* aClientData) = 0;
+    virtual nsIWidgetListener* GetWidgetListener() = 0;
+    virtual void SetWidgetListener(nsIWidgetListener* alistener) = 0;
     //@}
 
     /**
@@ -539,10 +518,10 @@ class nsIWidget : public nsISupports {
     NS_IMETHOD UnregisterTouchWindow() = 0;
 
     /**
-     * Return the parent Widget of this Widget or nsnull if this is a 
+     * Return the parent Widget of this Widget or nullptr if this is a 
      * top level window
      *
-     * @return the parent widget or nsnull if it does not have a parent
+     * @return the parent widget or nullptr if it does not have a parent
      *
      */
     virtual nsIWidget* GetParent(void) = 0;
@@ -556,10 +535,10 @@ class nsIWidget : public nsISupports {
 
     /**
      * Return the top (non-sheet) parent of this Widget if it's a sheet,
-     * or nsnull if this isn't a sheet (or some other error occurred).
+     * or nullptr if this isn't a sheet (or some other error occurred).
      * Sheets are only supported on some platforms (currently only OS X).
      *
-     * @return the top (non-sheet) parent widget or nsnull
+     * @return the top (non-sheet) parent widget or nullptr
      *
      */
     virtual nsIWidget* GetSheetWindowParent(void) = 0;
@@ -640,7 +619,7 @@ class nsIWidget : public nsISupports {
      * Returns whether the window is visible
      *
      */
-    NS_IMETHOD IsVisible(bool & aState) = 0;
+    virtual bool IsVisible() const = 0;
 
     /**
      * Perform platform-dependent sanity check on a potential window position.
@@ -658,8 +637,8 @@ class nsIWidget : public nsISupports {
      *
      **/
     NS_IMETHOD ConstrainPosition(bool aAllowSlop,
-                                 PRInt32 *aX,
-                                 PRInt32 *aY) = 0;
+                                 int32_t *aX,
+                                 int32_t *aY) = 0;
 
     /**
      * Move this widget.
@@ -671,7 +650,7 @@ class nsIWidget : public nsISupports {
      * @param aY the new y position expressed in the parent's coordinate system
      *
      **/
-    NS_IMETHOD Move(PRInt32 aX, PRInt32 aY) = 0;
+    NS_IMETHOD Move(int32_t aX, int32_t aY) = 0;
 
     /**
      * Reposition this widget so that the client area has the given offset.
@@ -686,22 +665,24 @@ class nsIWidget : public nsISupports {
      *                 screen coordinates)
      *
      **/
-    NS_IMETHOD MoveClient(PRInt32 aX, PRInt32 aY) = 0;
+    NS_IMETHOD MoveClient(int32_t aX, int32_t aY) = 0;
 
     /**
-     * Resize this widget. 
+     * Resize this widget. Any size constraints set for the window by a
+     * previous call to SetSizeConstraints will be applied.
      *
      * @param aWidth  the new width expressed in the parent's coordinate system
      * @param aHeight the new height expressed in the parent's coordinate system
      * @param aRepaint whether the widget should be repainted
      *
      */
-    NS_IMETHOD Resize(PRInt32 aWidth,
-                      PRInt32 aHeight,
+    NS_IMETHOD Resize(int32_t aWidth,
+                      int32_t aHeight,
                       bool     aRepaint) = 0;
 
     /**
-     * Move or resize this widget.
+     * Move or resize this widget. Any size constraints set for the window by
+     * a previous call to SetSizeConstraints will be applied.
      *
      * @param aX       the new x position expressed in the parent's coordinate system
      * @param aY       the new y position expressed in the parent's coordinate system
@@ -710,10 +691,10 @@ class nsIWidget : public nsISupports {
      * @param aRepaint whether the widget should be repainted if the size changes
      *
      */
-    NS_IMETHOD Resize(PRInt32 aX,
-                      PRInt32 aY,
-                      PRInt32 aWidth,
-                      PRInt32 aHeight,
+    NS_IMETHOD Resize(int32_t aX,
+                      int32_t aY,
+                      int32_t aWidth,
+                      int32_t aHeight,
                       bool     aRepaint) = 0;
 
     /**
@@ -724,8 +705,8 @@ class nsIWidget : public nsISupports {
      * @param aRepaint whether the widget should be repainted
      *
      */
-    NS_IMETHOD ResizeClient(PRInt32 aWidth,
-                            PRInt32 aHeight,
+    NS_IMETHOD ResizeClient(int32_t aWidth,
+                            int32_t aHeight,
                             bool  aRepaint) = 0;
 
     /**
@@ -745,21 +726,21 @@ class nsIWidget : public nsISupports {
      * @param aRepaint whether the widget should be repainted
      *
      */
-    NS_IMETHOD ResizeClient(PRInt32 aX,
-                            PRInt32 aY,
-                            PRInt32 aWidth,
-                            PRInt32 aHeight,
+    NS_IMETHOD ResizeClient(int32_t aX,
+                            int32_t aY,
+                            int32_t aWidth,
+                            int32_t aHeight,
                             bool    aRepaint) = 0;
 
     /**
      * Sets the widget's z-index.
      */
-    NS_IMETHOD SetZIndex(PRInt32 aZIndex) = 0;
+    NS_IMETHOD SetZIndex(int32_t aZIndex) = 0;
 
     /**
      * Gets the widget's z-index. 
      */
-    NS_IMETHOD GetZIndex(PRInt32* aZIndex) = 0;
+    NS_IMETHOD GetZIndex(int32_t* aZIndex) = 0;
 
     /**
      * Position this widget just behind the given widget. (Used to
@@ -779,13 +760,13 @@ class nsIWidget : public nsISupports {
      * Minimize, maximize or normalize the window size.
      * Takes a value from nsSizeMode (see nsGUIEvent.h)
      */
-    NS_IMETHOD SetSizeMode(PRInt32 aMode) = 0;
+    NS_IMETHOD SetSizeMode(int32_t aMode) = 0;
 
     /**
      * Return size mode (minimized, maximized, normalized).
      * Returns a value from nsSizeMode (see nsGUIEvent.h)
      */
-    NS_IMETHOD GetSizeMode(PRInt32* aMode) = 0;
+    NS_IMETHOD GetSizeMode(int32_t* aMode) = 0;
 
     /**
      * Enable or disable this Widget
@@ -797,9 +778,8 @@ class nsIWidget : public nsISupports {
 
     /**
      * Ask whether the widget is enabled
-     * @param aState returns true if the widget is enabled
      */
-    NS_IMETHOD IsEnabled(bool *aState) = 0;
+    virtual bool IsEnabled() const = 0;
 
     /**
      * Request activation of this window or give focus to this widget.
@@ -808,9 +788,7 @@ class nsIWidget : public nsISupports {
      *               widget's toplevel window.
      *               If false, the appropriate toplevel window (which in
      *               the case of popups may not be this widget's toplevel
-     *               window) is already active, and this function indicates
-     *               that keyboard events should be reported through the
-     *               aHandleEventFunction provided to this->Create().
+     *               window) is already active.
      */
     NS_IMETHOD SetFocus(bool aRaise = false) = 0;
 
@@ -933,7 +911,7 @@ class nsIWidget : public nsISupports {
      *         supported
      */
     NS_IMETHOD SetCursor(imgIContainer* aCursor,
-                         PRUint32 aHotspotX, PRUint32 aHotspotY) = 0;
+                         uint32_t aHotspotX, uint32_t aHotspotY) = 0;
 
     /** 
      * Get the window type of this widget
@@ -1007,7 +985,7 @@ class nsIWidget : public nsISupports {
      *
      * Ignored on child widgets and on non-Mac platforms.
      */
-    NS_IMETHOD SetWindowShadowStyle(PRInt32 aStyle) = 0;
+    NS_IMETHOD SetWindowShadowStyle(int32_t aStyle) = 0;
 
     /*
      * On Mac OS X, this method shows or hides the pill button in the titlebar
@@ -1070,16 +1048,16 @@ class nsIWidget : public nsISupports {
      * @param aAllowRetaining an outparam that states whether the returned
      * layer manager should be used for retained layers
      */
-    inline LayerManager* GetLayerManager(bool* aAllowRetaining = nsnull)
+    inline LayerManager* GetLayerManager(bool* aAllowRetaining = nullptr)
     {
-        return GetLayerManager(nsnull, LayerManager::LAYERS_NONE,
+        return GetLayerManager(nullptr, mozilla::layers::LAYERS_NONE,
                                LAYER_MANAGER_CURRENT, aAllowRetaining);
     }
 
     inline LayerManager* GetLayerManager(LayerManagerPersistence aPersistence,
-                                         bool* aAllowRetaining = nsnull)
+                                         bool* aAllowRetaining = nullptr)
     {
-        return GetLayerManager(nsnull, LayerManager::LAYERS_NONE,
+        return GetLayerManager(nullptr, mozilla::layers::LAYERS_NONE,
                                aPersistence, aAllowRetaining);
     }
 
@@ -1091,7 +1069,7 @@ class nsIWidget : public nsISupports {
     virtual LayerManager* GetLayerManager(PLayersChild* aShadowManager,
                                           LayersBackend aBackendHint,
                                           LayerManagerPersistence aPersistence = LAYER_MANAGER_CURRENT,
-                                          bool* aAllowRetaining = nsnull) = 0;
+                                          bool* aAllowRetaining = nullptr) = 0;
 
     /**
      * Called before the LayerManager draws the layer tree.
@@ -1128,7 +1106,7 @@ class nsIWidget : public nsISupports {
      *
      * @param aOpaqueRegion the region of the window that is opaque.
      */
-    virtual void UpdateOpaqueRegion(const nsIntRegion &aOpaqueRegion) {};
+    virtual void UpdateOpaqueRegion(const nsIntRegion &aOpaqueRegion) {}
 
     /** 
      * Internal methods
@@ -1137,8 +1115,8 @@ class nsIWidget : public nsISupports {
     //@{
     virtual void AddChild(nsIWidget* aChild) = 0;
     virtual void RemoveChild(nsIWidget* aChild) = 0;
-    virtual void* GetNativeData(PRUint32 aDataType) = 0;
-    virtual void FreeNativeData(void * data, PRUint32 aDataType) = 0;//~~~
+    virtual void* GetNativeData(uint32_t aDataType) = 0;
+    virtual void FreeNativeData(void * data, uint32_t aDataType) = 0;//~~~
 
     // GetDeviceContext returns a weak pointer to this widget's device context
     virtual nsDeviceContext* GetDeviceContext() = 0;
@@ -1226,7 +1204,7 @@ class nsIWidget : public nsISupports {
      *                    conventions. If set to -1, cycles indefinitely until 
      *                    window is brought into the foreground.
      */
-    NS_IMETHOD GetAttention(PRInt32 aCycleCount) = 0;
+    NS_IMETHOD GetAttention(int32_t aCycleCount) = 0;
 
     /**
      * Ask whether there user input events pending.  All input events are
@@ -1307,7 +1285,7 @@ class nsIWidget : public nsISupports {
     /**
      * Begin a window resizing drag, based on the event passed in.
      */
-    NS_IMETHOD BeginResizeDrag(nsGUIEvent* aEvent, PRInt32 aHorizontal, PRInt32 aVertical) = 0;
+    NS_IMETHOD BeginResizeDrag(nsGUIEvent* aEvent, int32_t aHorizontal, int32_t aVertical) = 0;
 
     /**
      * Begin a window moving drag, based on the event passed in.
@@ -1352,9 +1330,9 @@ class nsIWidget : public nsISupports {
      * @return NS_ERROR_NOT_AVAILABLE to indicate that the keyboard
      * layout is not supported and the event was not fired
      */
-    virtual nsresult SynthesizeNativeKeyEvent(PRInt32 aNativeKeyboardLayout,
-                                              PRInt32 aNativeKeyCode,
-                                              PRUint32 aModifierFlags,
+    virtual nsresult SynthesizeNativeKeyEvent(int32_t aNativeKeyboardLayout,
+                                              int32_t aNativeKeyCode,
+                                              uint32_t aModifierFlags,
                                               const nsAString& aCharacters,
                                               const nsAString& aUnmodifiedCharacters) = 0;
 
@@ -1373,8 +1351,8 @@ class nsIWidget : public nsISupports {
      * on Windows)
      */
     virtual nsresult SynthesizeNativeMouseEvent(nsIntPoint aPoint,
-                                                PRUint32 aNativeMessage,
-                                                PRUint32 aModifierFlags) = 0;
+                                                uint32_t aNativeMessage,
+                                                uint32_t aModifierFlags) = 0;
 
     /**
      * A shortcut to SynthesizeNativeMouseEvent, abstracting away the native message.
@@ -1403,12 +1381,12 @@ class nsIWidget : public nsISupports {
      *                          document.
      */
     virtual nsresult SynthesizeNativeMouseScrollEvent(nsIntPoint aPoint,
-                                                      PRUint32 aNativeMessage,
+                                                      uint32_t aNativeMessage,
                                                       double aDeltaX,
                                                       double aDeltaY,
                                                       double aDeltaZ,
-                                                      PRUint32 aModifierFlags,
-                                                      PRUint32 aAdditionalFlags) = 0;
+                                                      uint32_t aModifierFlags,
+                                                      uint32_t aAdditionalFlags) = 0;
 
     /**
      * Activates a native menu item at the position specified by the index
@@ -1487,7 +1465,7 @@ class nsIWidget : public nsISupports {
      * If the platform doesn't support the LED state (or we cannot get the
      * state), this method returns NS_ERROR_NOT_IMPLEMENTED.
      */
-    NS_IMETHOD GetToggledKeyState(PRUint32 aKeyCode, bool* aLEDState) = 0;
+    NS_IMETHOD GetToggledKeyState(uint32_t aKeyCode, bool* aLEDState) = 0;
 
     /*
      * An editable node (i.e. input/textarea/design mode document)
@@ -1509,9 +1487,9 @@ class nsIWidget : public nsISupports {
      * aOldEnd is the ending offset of the change
      * aNewEnd is the caret offset after the change
      */
-    NS_IMETHOD OnIMETextChange(PRUint32 aStart,
-                               PRUint32 aOldEnd,
-                               PRUint32 aNewEnd) = 0;
+    NS_IMETHOD OnIMETextChange(uint32_t aStart,
+                               uint32_t aOldEnd,
+                               uint32_t aNewEnd) = 0;
 
     /*
      * Selection has changed in the focused node
@@ -1548,9 +1526,9 @@ class nsIWidget : public nsISupports {
      * @param aOverriddenDelta The overridden mouse scrolling speed.  This value
      *                         may be same as aOriginalDelta.
      */
-    NS_IMETHOD OverrideSystemMouseScrollSpeed(PRInt32 aOriginalDelta,
+    NS_IMETHOD OverrideSystemMouseScrollSpeed(int32_t aOriginalDelta,
                                               bool aIsHorizontal,
-                                              PRInt32 &aOverriddenDelta) = 0;
+                                              int32_t &aOverriddenDelta) = 0;
 
     /**
      * Return true if this process shouldn't use platform widgets, and
@@ -1575,7 +1553,7 @@ class nsIWidget : public nsISupports {
      * The returned widget must still be nsIWidget::Create()d.
      */
     static already_AddRefed<nsIWidget>
-    CreatePuppetWidget(PBrowserChild *aTabChild);
+    CreatePuppetWidget(TabChild* aTabChild);
 
     /**
      * Reparent this widget's native widget.
@@ -1588,7 +1566,7 @@ class nsIWidget : public nsISupports {
      * Return the internal format of the default framebuffer for this
      * widget.
      */
-    virtual PRUint32 GetGLFrameBufferFormat() { return 0; /*GL_NONE*/ }
+    virtual uint32_t GetGLFrameBufferFormat() { return 0; /*GL_NONE*/ }
 
     /**
      * Return true if widget has it's own GL context
@@ -1602,6 +1580,47 @@ class nsIWidget : public nsISupports {
      */
     virtual bool WidgetPaintsBackground() { return false; }
 
+    virtual bool NeedsPaint() {
+      return true;
+    }
+    /**
+     * Get the natural bounds of this widget.  This method is only
+     * meaningful for widgets for which Gecko implements screen
+     * rotation natively.  When this is the case, GetBounds() returns
+     * the widget bounds taking rotation into account, and
+     * GetNaturalBounds() returns the bounds *not* taking rotation
+     * into account.
+     *
+     * No code outside of the composition pipeline should know or care
+     * about this.  If you're not an agent of the compositor, you
+     * probably shouldn't call this method.
+     */
+    virtual nsIntRect GetNaturalBounds() {
+        nsIntRect bounds;
+        GetBounds(bounds);
+        return bounds;
+    }
+
+    /**
+     * Set size constraints on the window size such that it is never less than
+     * the specified minimum size and never larger than the specified maximum
+     * size. The size constraints are sizes of the outer rectangle including
+     * the window frame and title bar. Use 0 for an unconstrained minimum size
+     * and NS_MAXSIZE for an unconstrained maximum size. Note that this method
+     * does not necessarily change the size of a window to conform to this size,
+     * thus Resize should be called afterwards.
+     *
+     * @param aConstraints: the size constraints in device pixels
+     */
+    virtual void SetSizeConstraints(const SizeConstraints& aConstraints) = 0;
+
+    /**
+     * Return the size constraints currently observed by the widget.
+     *
+     * @return the constraints in device pixels
+     */
+    virtual const SizeConstraints& GetSizeConstraints() const = 0;
+
 protected:
 
     // keep the list of children.  We also keep track of our siblings.
