@@ -1,40 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Ningjie Chen <chenn@email.uc.edu>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include <olectl.h>
 
@@ -61,7 +28,7 @@ nsTextStore*  nsTextStore::sTsfTextStore = NULL;
 UINT nsTextStore::sFlushTIPInputMessage  = 0;
 
 #ifdef PR_LOGGING
-PRLogModuleInfo* sTextStoreLog = nsnull;
+PRLogModuleInfo* sTextStoreLog = nullptr;
 #endif
 
 nsTextStore::nsTextStore()
@@ -69,21 +36,21 @@ nsTextStore::nsTextStore()
   mRefCnt = 1;
   mEditCookie = 0;
   mSinkMask = 0;
-  mWindow = nsnull;
+  mWindow = nullptr;
   mLock = 0;
   mLockQueued = 0;
   mTextChange.acpStart = PR_INT32_MAX;
   mTextChange.acpOldEnd = mTextChange.acpNewEnd = 0;
-  mLastDispatchedTextEvent = nsnull;
+  mLastDispatchedTextEvent = nullptr;
 }
 
 nsTextStore::~nsTextStore()
 {
   if (mCompositionTimer) {
     mCompositionTimer->Cancel();
-    mCompositionTimer = nsnull;
+    mCompositionTimer = nullptr;
   }
-  SaveTextEvent(nsnull);
+  SaveTextEvent(nullptr);
 }
 
 bool
@@ -330,10 +297,10 @@ GetRangeExtent(ITfRange* aRange, LONG* aStart, LONG* aLength)
   return rangeACP->GetExtent(aStart, aLength);
 }
 
-static PRUint32
+static uint32_t
 GetGeckoSelectionValue(TF_DISPLAYATTRIBUTE &aDisplayAttr)
 {
-  PRUint32 result;
+  uint32_t result;
   switch (aDisplayAttr.bAttr) {
     case TF_ATTR_TARGET_CONVERTED:
       result = NS_TEXTRANGE_SELECTEDCONVERTEDTEXT;
@@ -362,17 +329,17 @@ GetLogTextFor(const TF_DA_COLOR &aColor, nsACString &aText)
       break;
     case TF_CT_SYSCOLOR: {
       nsPrintfCString tmp("TF_CT_SYSCOLOR, nIndex:0x%08X",
-                          PRInt32(aColor.nIndex));
+                          int32_t(aColor.nIndex));
       aText += tmp;
       break;
     }
     case TF_CT_COLORREF: {
-      nsPrintfCString tmp("TF_CT_COLORREF, cr:0x%08X", PRInt32(aColor.cr));
+      nsPrintfCString tmp("TF_CT_COLORREF, cr:0x%08X", int32_t(aColor.cr));
       aText += tmp;
       break;
     }
     default: {
-      nsPrintfCString tmp("Unknown(%08X)", PRInt32(aColor.type));
+      nsPrintfCString tmp("Unknown(%08X)", int32_t(aColor.type));
       aText += tmp;
       break;
     }
@@ -399,7 +366,7 @@ GetLogTextFor(TF_DA_LINESTYLE aLineStyle, nsACString &aText)
       aText = "TF_LS_SQUIGGLE";
       break;
     default: {
-      nsPrintfCString tmp("Unknown(%08X)", PRInt32(aLineStyle));
+      nsPrintfCString tmp("Unknown(%08X)", int32_t(aLineStyle));
       aText = tmp;
       break;
     }
@@ -432,7 +399,7 @@ GetLogTextFor(TF_DA_ATTR_INFO aAttr, nsACString &aText)
       aText = "TF_ATTR_OTHER";
       break;
     default: {
-      nsPrintfCString tmp("Unknown(%08X)", PRInt32(aAttr));
+      nsPrintfCString tmp("Unknown(%08X)", int32_t(aAttr));
       aText = tmp;
       break;
     }
@@ -539,17 +506,17 @@ nsTextStore::SaveTextEvent(const nsTextEvent* aEvent)
     if (mLastDispatchedTextEvent->rangeArray)
       delete [] mLastDispatchedTextEvent->rangeArray;
     delete mLastDispatchedTextEvent;
-    mLastDispatchedTextEvent = nsnull;
+    mLastDispatchedTextEvent = nullptr;
   }
   if (!aEvent)
     return S_OK;
 
-  mLastDispatchedTextEvent = new nsTextEvent(true, NS_TEXT_TEXT, nsnull);
+  mLastDispatchedTextEvent = new nsTextEvent(true, NS_TEXT_TEXT, nullptr);
   if (!mLastDispatchedTextEvent)
     return E_OUTOFMEMORY;
   mLastDispatchedTextEvent->rangeCount = aEvent->rangeCount;
   mLastDispatchedTextEvent->theText = aEvent->theText;
-  mLastDispatchedTextEvent->rangeArray = nsnull;
+  mLastDispatchedTextEvent->rangeArray = nullptr;
 
   if (aEvent->rangeCount == 0)
     return S_OK;
@@ -559,7 +526,7 @@ nsTextStore::SaveTextEvent(const nsTextEvent* aEvent)
   mLastDispatchedTextEvent->rangeArray = new nsTextRange[aEvent->rangeCount];
   if (!mLastDispatchedTextEvent->rangeArray) {
     delete mLastDispatchedTextEvent;
-    mLastDispatchedTextEvent = nsnull;
+    mLastDispatchedTextEvent = nullptr;
     return E_OUTOFMEMORY;
   }
   memcpy(mLastDispatchedTextEvent->rangeArray, aEvent->rangeArray,
@@ -644,7 +611,7 @@ GetColor(const TF_DA_COLOR &aTSFColor, nscolor &aResult)
 }
 
 static bool
-GetLineStyle(TF_DA_LINESTYLE aTSFLineStyle, PRUint8 &aTextRangeLineStyle)
+GetLineStyle(TF_DA_LINESTYLE aTSFLineStyle, uint8_t &aTextRangeLineStyle)
 {
   switch (aTSFLineStyle) {
     case TF_LS_NONE:
@@ -718,7 +685,7 @@ nsTextStore::SendTextEventForCompositionString()
       continue;
 
     nsTextRange newRange;
-    newRange.mStartOffset = PRUint32(start - mCompositionStart);
+    newRange.mStartOffset = uint32_t(start - mCompositionStart);
     // The end of the last range in the array is
     // always kept at the end of composition
     newRange.mEndOffset = mCompositionString.Length();
@@ -788,7 +755,7 @@ nsTextStore::SendTextEventForCompositionString()
                               mCompositionSelection.acpEnd);
   caretPosition -= mCompositionStart;
   nsTextRange caretRange;
-  caretRange.mStartOffset = caretRange.mEndOffset = PRUint32(caretPosition);
+  caretRange.mStartOffset = caretRange.mEndOffset = uint32_t(caretPosition);
   caretRange.mRangeType = NS_TEXTRANGE_CARETPOSITION;
   textRanges.AppendElement(caretRange);
 
@@ -833,7 +800,7 @@ nsTextStore::SetSelectionInternal(const TS_SELECTION_ACP* pSelection,
           pSelection->acpStart, pSelection->acpEnd));
   if (mCompositionView) {
     if (aDispatchTextEvent) {
-      HRESULT hr = UpdateCompositionExtent(nsnull);
+      HRESULT hr = UpdateCompositionExtent(nullptr);
       NS_ENSURE_TRUE(SUCCEEDED(hr), hr);
     }
     // Emulate selection during compositions
@@ -848,7 +815,7 @@ nsTextStore::SetSelectionInternal(const TS_SELECTION_ACP* pSelection,
   } else {
     nsSelectionEvent event(true, NS_SELECTION_SET, mWindow);
     event.mOffset = pSelection->acpStart;
-    event.mLength = PRUint32(pSelection->acpEnd - pSelection->acpStart);
+    event.mLength = uint32_t(pSelection->acpEnd - pSelection->acpStart);
     event.mReversed = pSelection->style.ase == TS_AE_START;
     mWindow->InitEvent(event);
     mWindow->DispatchWindowEvent(&event);
@@ -896,7 +863,7 @@ nsTextStore::GetText(LONG acpStart,
     prgRunInfo->uCount = 0;
     prgRunInfo->type = TS_RT_PLAIN;
   }
-  PRUint32 length = -1 == acpEnd ? PR_UINT32_MAX : PRUint32(acpEnd - acpStart);
+  uint32_t length = -1 == acpEnd ? PR_UINT32_MAX : uint32_t(acpEnd - acpStart);
   if (cchPlainReq && cchPlainReq - 1 < length) {
     length = cchPlainReq - 1;
   }
@@ -916,13 +883,13 @@ nsTextStore::GetText(LONG acpStart,
       if (compOldEnd > compNewStart || compNewEnd > compNewStart) {
         NS_ASSERTION(compOldEnd >= mCompositionStart &&
             compNewEnd >= mCompositionStart, "Range end is less than start\n");
-        length = PRUint32(LONG(length) + compOldEnd - compNewEnd);
+        length = uint32_t(LONG(length) + compOldEnd - compNewEnd);
       }
     }
     // Send NS_QUERY_TEXT_CONTENT to get text content
     nsQueryContentEvent event(true, NS_QUERY_TEXT_CONTENT, mWindow);
     mWindow->InitEvent(event);
-    event.InitForQueryTextContent(PRUint32(acpStart), length);
+    event.InitForQueryTextContent(uint32_t(acpStart), length);
     mWindow->DispatchWindowEvent(&event);
     NS_ENSURE_TRUE(event.mSucceeded, E_FAIL);
 
@@ -933,7 +900,7 @@ nsTextStore::GetText(LONG acpStart,
       event.mReply.mString.Replace(compNewStart - acpStart,
           compOldEnd - mCompositionStart, compStrStart,
           compNewEnd - mCompositionStart);
-      length = PRUint32(LONG(length) - compOldEnd + compNewEnd);
+      length = uint32_t(LONG(length) - compOldEnd + compNewEnd);
     }
     NS_ENSURE_TRUE(-1 == acpEnd || event.mReply.mString.Length() == length,
                    TS_E_INVALIDPOS);
@@ -1246,7 +1213,7 @@ nsTextStore::InsertTextAtSelection(DWORD dwFlags,
       // be sent in NS_TEXT_TEXT, not just the inserted part.
       // The actual NS_TEXT_TEXT will be sent in SetSelection or
       // OnUpdateComposition.
-      mCompositionString.Replace(PRUint32(sel.acpStart - mCompositionStart),
+      mCompositionString.Replace(uint32_t(sel.acpStart - mCompositionStart),
                                  sel.acpEnd - sel.acpStart, pchText, cch);
 
       mCompositionSelection.acpStart += cch;
@@ -1322,8 +1289,8 @@ nsTextStore::OnStartCompositionInternal(ITfCompositionView* pComposition,
   // Select composition range so the new composition replaces the range
   nsSelectionEvent selEvent(true, NS_SELECTION_SET, mWindow);
   mWindow->InitEvent(selEvent);
-  selEvent.mOffset = PRUint32(mCompositionStart);
-  selEvent.mLength = PRUint32(mCompositionLength);
+  selEvent.mOffset = uint32_t(mCompositionStart);
+  selEvent.mLength = uint32_t(mCompositionLength);
   selEvent.mReversed = false;
   mWindow->DispatchWindowEvent(&selEvent);
   NS_ENSURE_TRUE(selEvent.mSucceeded, E_FAIL);
@@ -1346,16 +1313,16 @@ nsTextStore::OnStartCompositionInternal(ITfCompositionView* pComposition,
   return S_OK;
 }
 
-static PRUint32
+static uint32_t
 GetLayoutChangeIntervalTime()
 {
-  static PRInt32 sTime = -1;
+  static int32_t sTime = -1;
   if (sTime > 0)
-    return PRUint32(sTime);
+    return uint32_t(sTime);
 
   sTime = NS_MAX(10,
     Preferences::GetInt("intl.tsf.on_layout_change_interval", 100));
-  return PRUint32(sTime);
+  return uint32_t(sTime);
 }
 
 STDMETHODIMP
@@ -1412,11 +1379,11 @@ nsTextStore::OnEndComposition(ITfCompositionView* pComposition)
          ("TSF: OnEndComposition\n"));
 
   // Clear the saved text event
-  SaveTextEvent(nsnull);
+  SaveTextEvent(nullptr);
 
   if (mCompositionTimer) {
     mCompositionTimer->Cancel();
-    mCompositionTimer = nsnull;
+    mCompositionTimer = nullptr;
   }
 
   if (mCompositionString != mLastDispatchedCompositionString) {
@@ -1483,9 +1450,9 @@ nsTextStore::OnFocusChange(bool aFocus,
 }
 
 nsresult
-nsTextStore::OnTextChangeInternal(PRUint32 aStart,
-                                  PRUint32 aOldEnd,
-                                  PRUint32 aNewEnd)
+nsTextStore::OnTextChangeInternal(uint32_t aStart,
+                                  uint32_t aOldEnd,
+                                  uint32_t aNewEnd)
 {
   if (!mLock && mSink && 0 != (mSinkMask & TS_AS_TEXT_CHANGE)) {
     mTextChange.acpStart = NS_MIN(mTextChange.acpStart, LONG(aStart));
@@ -1620,7 +1587,7 @@ void
 nsTextStore::SetInputContextInternal(IMEState::Enabled aState)
 {
   PR_LOG(sTextStoreLog, PR_LOG_ALWAYS,
-         ("TSF: SetInputContext, state=%ld\n", static_cast<PRInt32>(aState)));
+         ("TSF: SetInputContext, state=%ld\n", static_cast<int32_t>(aState)));
 
   VARIANT variant;
   variant.vt = VT_I4;
