@@ -1,41 +1,9 @@
 /* vim: se cin sw=2 ts=2 et : */
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
  *
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2011
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "mozilla/Util.h"
 
@@ -83,12 +51,12 @@ public:
     MOZ_ASSERT(strcmp(aTopic, NS_XPCOM_SHUTDOWN_OBSERVER_ID) == 0);
 
     delete GfxInfoBase::mDriverInfo;
-    GfxInfoBase::mDriverInfo = nsnull;
+    GfxInfoBase::mDriverInfo = nullptr;
 
-    for (PRUint32 i = 0; i < DeviceFamilyMax; i++)
+    for (uint32_t i = 0; i < DeviceFamilyMax; i++)
       delete GfxDriverInfo::mDeviceFamilies[i];
 
-    for (PRUint32 i = 0; i < DeviceVendorMax; i++)
+    for (uint32_t i = 0; i < DeviceVendorMax; i++)
       delete GfxDriverInfo::mDeviceVendors[i];
 
     return NS_OK;
@@ -114,66 +82,6 @@ void InitGfxDriverInfoShutdownObserver()
   observerService->AddObserver(obs, NS_XPCOM_SHUTDOWN_OBSERVER_ID, false);
 }
 
-extern "C" {
-  void StoreSpline(int ax, int ay, int bx, int by, int cx, int cy, int dx, int dy);
-  void CrashSpline(double tolerance, int ax, int ay, int bx, int by, int cx, int cy, int dx, int dy);
-}
-
-static int crash_ax;
-static int crash_ay;
-static int crash_bx;
-static int crash_by;
-static int crash_cx;
-static int crash_cy;
-static int crash_dx;
-static int crash_dy;
-
-void
-StoreSpline(int ax, int ay, int bx, int by, int cx, int cy, int dx, int dy) {
-    crash_ax = ax;
-    crash_ay = ay;
-    crash_bx = bx;
-    crash_by = by;
-    crash_cx = cx;
-    crash_cy = cy;
-    crash_dx = dx;
-    crash_dy = dy;
-}
-
-void
-CrashSpline(double tolerance, int ax, int ay, int bx, int by, int cx, int cy, int dx, int dy) {
-#if defined(MOZ_CRASHREPORTER)
-  static bool annotated;
-
-  if (!annotated) {
-    nsCAutoString note;
-
-    note.AppendPrintf("curve ");
-    note.AppendPrintf("%x ", crash_ax);
-    note.AppendPrintf("%x, ", crash_ay);
-    note.AppendPrintf("%x ", crash_bx);
-    note.AppendPrintf("%x, ", crash_by);
-    note.AppendPrintf("%x ", crash_cx);
-    note.AppendPrintf("%x, ", crash_cy);
-    note.AppendPrintf("%x ", crash_dx);
-    note.AppendPrintf("%x\n", crash_dy);
-    note.AppendPrintf("crv-crash(%f): ", tolerance);
-    note.AppendPrintf("%x ", ax);
-    note.AppendPrintf("%x, ", ay);
-    note.AppendPrintf("%x ", bx);
-    note.AppendPrintf("%x, ", by);
-    note.AppendPrintf("%x ", cx);
-    note.AppendPrintf("%x, ", cy);
-    note.AppendPrintf("%x ", dx);
-    note.AppendPrintf("%x\n", dy);
-
-    CrashReporter::AppendAppNotesToCrashReport(note);
-    annotated = true;
-  }
-#endif
-}
-
-
 using namespace mozilla::widget;
 using namespace mozilla;
 
@@ -184,9 +92,9 @@ NS_IMPL_ISUPPORTS3(GfxInfoBase, nsIGfxInfo, nsIObserver, nsISupportsWeakReferenc
 #define BLACKLIST_ENTRY_TAG_NAME "gfxBlacklistEntry"
 
 static const char*
-GetPrefNameForFeature(PRInt32 aFeature)
+GetPrefNameForFeature(int32_t aFeature)
 {
-  const char* name = nsnull;
+  const char* name = nullptr;
   switch(aFeature) {
     case nsIGfxInfo::FEATURE_DIRECT2D:
       name = BLACKLIST_PREF_BRANCH "direct2d";
@@ -212,6 +120,9 @@ GetPrefNameForFeature(PRInt32 aFeature)
     case nsIGfxInfo::FEATURE_WEBGL_MSAA:
       name = BLACKLIST_PREF_BRANCH "webgl.msaa";
       break;
+    case nsIGfxInfo::FEATURE_STAGEFRIGHT:
+      name = BLACKLIST_PREF_BRANCH "stagefright";
+      break;
     default:
       break;
   };
@@ -222,7 +133,7 @@ GetPrefNameForFeature(PRInt32 aFeature)
 // Returns the value of the pref for the relevant feature in aValue.
 // If the pref doesn't exist, aValue is not touched, and returns false.
 static bool
-GetPrefValueForFeature(PRInt32 aFeature, PRInt32& aValue)
+GetPrefValueForFeature(int32_t aFeature, int32_t& aValue)
 {
   const char *prefname = GetPrefNameForFeature(aFeature);
   if (!prefname)
@@ -233,7 +144,7 @@ GetPrefValueForFeature(PRInt32 aFeature, PRInt32& aValue)
 }
 
 static void
-SetPrefValueForFeature(PRInt32 aFeature, PRInt32 aValue)
+SetPrefValueForFeature(int32_t aFeature, int32_t aValue)
 {
   const char *prefname = GetPrefNameForFeature(aFeature);
   if (!prefname)
@@ -243,7 +154,7 @@ SetPrefValueForFeature(PRInt32 aFeature, PRInt32 aValue)
 }
 
 static void
-RemovePrefForFeature(PRInt32 aFeature)
+RemovePrefForFeature(int32_t aFeature)
 {
   const char *prefname = GetPrefNameForFeature(aFeature);
   if (!prefname)
@@ -296,6 +207,8 @@ BlacklistOSToOperatingSystem(const nsAString& os)
     return DRIVER_OS_WINDOWS_VISTA;
   else if (os == NS_LITERAL_STRING("WINNT 6.1"))
     return DRIVER_OS_WINDOWS_7;
+  else if (os == NS_LITERAL_STRING("WINNT 6.2"))
+    return DRIVER_OS_WINDOWS_8;
   else if (os == NS_LITERAL_STRING("Linux"))
     return DRIVER_OS_LINUX;
   else if (os == NS_LITERAL_STRING("Darwin 9"))
@@ -304,6 +217,8 @@ BlacklistOSToOperatingSystem(const nsAString& os)
     return DRIVER_OS_OS_X_10_6;
   else if (os == NS_LITERAL_STRING("Darwin 11"))
     return DRIVER_OS_OS_X_10_7;
+  else if (os == NS_LITERAL_STRING("Darwin 12"))
+    return DRIVER_OS_OS_X_10_8;
   else if (os == NS_LITERAL_STRING("Android"))
     return DRIVER_OS_ANDROID;
   else if (os == NS_LITERAL_STRING("All"))
@@ -315,15 +230,15 @@ BlacklistOSToOperatingSystem(const nsAString& os)
 static GfxDeviceFamily*
 BlacklistDevicesToDeviceFamily(nsIDOMNodeList* aDevices)
 {
-  PRUint32 length;
+  uint32_t length;
   if (NS_FAILED(aDevices->GetLength(&length)))
-    return nsnull;
+    return nullptr;
 
   // For each <device>, get its device ID, and return a freshly-allocated
   // GfxDeviceFamily with the contents of that array.
   GfxDeviceFamily* deviceIds = new GfxDeviceFamily;
 
-  for (PRUint32 i = 0; i < length; ++i) {
+  for (uint32_t i = 0; i < length; ++i) {
     nsCOMPtr<nsIDOMNode> node;
     if (NS_FAILED(aDevices->Item(i, getter_AddRefs(node))) || !node)
       continue;
@@ -338,7 +253,7 @@ BlacklistDevicesToDeviceFamily(nsIDOMNodeList* aDevices)
   return deviceIds;
 }
 
-static PRInt32
+static int32_t
 BlacklistFeatureToGfxFeature(const nsAString& aFeature)
 {
   if (aFeature == NS_LITERAL_STRING("DIRECT2D"))
@@ -357,11 +272,12 @@ BlacklistFeatureToGfxFeature(const nsAString& aFeature)
     return nsIGfxInfo::FEATURE_WEBGL_ANGLE;
   else if (aFeature == NS_LITERAL_STRING("WEBGL_MSAA"))
     return nsIGfxInfo::FEATURE_WEBGL_MSAA;
-
+  else if (aFeature == NS_LITERAL_STRING("STAGEFRIGHT"))
+    return nsIGfxInfo::FEATURE_STAGEFRIGHT;
   return 0;
 }
 
-static PRInt32
+static int32_t
 BlacklistFeatureStatusToGfxFeatureStatus(const nsAString& aStatus)
 {
   if (aStatus == NS_LITERAL_STRING("NO_INFO"))
@@ -402,7 +318,7 @@ BlacklistComparatorToComparisonOp(const nsAString& op)
   else if (op == NS_LITERAL_STRING("BETWEEN_INCLUSIVE_START"))
     return DRIVER_BETWEEN_INCLUSIVE_START;
 
-  return DRIVER_UNKNOWN_COMPARISON;
+  return DRIVER_COMPARISON_IGNORED;
 }
 
 // Arbitrarily returns the first |tagname| child of |element|.
@@ -466,6 +382,13 @@ BlacklistEntryToDriverInfo(nsIDOMNode* aBlacklistEntry,
     aDriverInfo.mOperatingSystem = BlacklistOSToOperatingSystem(dataValue);
   }
 
+  // <osversion>14</osversion> currently only used for Android
+  if (BlacklistNodeGetChildByName(element, NS_LITERAL_STRING("osversion"),
+                                  getter_AddRefs(dataNode))) {
+    BlacklistNodeToTextValue(dataNode, dataValue);
+    aDriverInfo.mOperatingSystemVersion = strtoul(NS_LossyConvertUTF16toASCII(dataValue).get(), NULL, 10);
+  }
+
   // <vendor>0x8086</vendor>
   if (BlacklistNodeGetChildByName(element, NS_LITERAL_STRING("vendor"),
                                   getter_AddRefs(dataNode))) {
@@ -515,7 +438,7 @@ BlacklistEntryToDriverInfo(nsIDOMNode* aBlacklistEntry,
   if (BlacklistNodeGetChildByName(element, NS_LITERAL_STRING("driverVersion"),
                                   getter_AddRefs(dataNode))) {
     BlacklistNodeToTextValue(dataNode, dataValue);
-    PRUint64 version;
+    uint64_t version;
     if (ParseDriverVersion(dataValue, &version))
       aDriverInfo.mDriverVersion = version;
   }
@@ -527,6 +450,31 @@ BlacklistEntryToDriverInfo(nsIDOMNode* aBlacklistEntry,
     aDriverInfo.mComparisonOp = BlacklistComparatorToComparisonOp(dataValue);
   }
 
+  // <model>foo</model>
+  if (BlacklistNodeGetChildByName(element, NS_LITERAL_STRING("model"),
+                                  getter_AddRefs(dataNode))) {
+    BlacklistNodeToTextValue(dataNode, dataValue);
+    aDriverInfo.mModel = dataValue;
+  }
+  // <product>foo</product>
+  if (BlacklistNodeGetChildByName(element, NS_LITERAL_STRING("product"),
+                                  getter_AddRefs(dataNode))) {
+    BlacklistNodeToTextValue(dataNode, dataValue);
+    aDriverInfo.mProduct = dataValue;
+  }
+  // <manufacturer>foo</manufacturer>
+  if (BlacklistNodeGetChildByName(element, NS_LITERAL_STRING("manufacturer"),
+                                  getter_AddRefs(dataNode))) {
+    BlacklistNodeToTextValue(dataNode, dataValue);
+    aDriverInfo.mManufacturer = dataValue;
+  }
+  // <hardware>foo</hardware>
+  if (BlacklistNodeGetChildByName(element, NS_LITERAL_STRING("hardware"),
+                                  getter_AddRefs(dataNode))) {
+    BlacklistNodeToTextValue(dataNode, dataValue);
+    aDriverInfo.mHardware = dataValue;
+  }
+
   // We explicitly ignore unknown elements.
 
   return true;
@@ -536,13 +484,13 @@ static void
 BlacklistEntriesToDriverInfo(nsIDOMNodeList* aBlacklistEntries,
                              nsTArray<GfxDriverInfo>& aDriverInfo)
 {
-  PRUint32 length;
+  uint32_t length;
   if (NS_FAILED(aBlacklistEntries->GetLength(&length)))
     return;
 
   aDriverInfo.Clear();
   aDriverInfo.SetLength(length);
-  for (PRUint32 i = 0; i < length; ++i) {
+  for (uint32_t i = 0; i < length; ++i) {
     nsCOMPtr<nsIDOMNode> blacklistEntry;
     if (NS_SUCCEEDED(aBlacklistEntries->Item(i,
                                              getter_AddRefs(blacklistEntry))) &&
@@ -603,7 +551,7 @@ GfxInfoBase::Init()
 }
 
 NS_IMETHODIMP
-GfxInfoBase::GetFeatureStatus(PRInt32 aFeature, PRInt32* aStatus NS_OUTPARAM)
+GfxInfoBase::GetFeatureStatus(int32_t aFeature, int32_t* aStatus)
 {
   if (GetPrefValueForFeature(aFeature, *aStatus))
     return NS_OK;
@@ -613,13 +561,13 @@ GfxInfoBase::GetFeatureStatus(PRInt32 aFeature, PRInt32* aStatus NS_OUTPARAM)
   return GetFeatureStatusImpl(aFeature, aStatus, version, driverInfo);
 }
 
-PRInt32
+int32_t
 GfxInfoBase::FindBlocklistedDeviceInList(const nsTArray<GfxDriverInfo>& info,
                                          nsAString& aSuggestedVersion,
-                                         PRInt32 aFeature,
+                                         int32_t aFeature,
                                          OperatingSystem os)
 {
-  PRInt32 status = nsIGfxInfo::FEATURE_STATUS_UNKNOWN;
+  int32_t status = nsIGfxInfo::FEATURE_STATUS_UNKNOWN;
 
   nsAutoString adapterVendorID;
   nsAutoString adapterDeviceID;
@@ -628,13 +576,13 @@ GfxInfoBase::FindBlocklistedDeviceInList(const nsTArray<GfxDriverInfo>& info,
       NS_FAILED(GetAdapterDeviceID(adapterDeviceID)) ||
       NS_FAILED(GetAdapterDriverVersion(adapterDriverVersionString)))
   {
-    return NS_OK;
+    return 0;
   }
 
-  PRUint64 driverVersion;
+  uint64_t driverVersion;
   ParseDriverVersion(adapterDriverVersionString, &driverVersion);
 
-  PRUint32 i = 0;
+  uint32_t i = 0;
   for (; i < info.Length(); i++) {
     if (info[i].mOperatingSystem != DRIVER_OS_ALL &&
         info[i].mOperatingSystem != os)
@@ -642,6 +590,10 @@ GfxInfoBase::FindBlocklistedDeviceInList(const nsTArray<GfxDriverInfo>& info,
       continue;
     }
 
+    if (info[i].mOperatingSystemVersion && info[i].mOperatingSystemVersion != OperatingSystemVersion()) {
+        continue;
+    }
+
     if (!info[i].mAdapterVendor.Equals(GfxDriverInfo::GetDeviceVendor(VendorAll), nsCaseInsensitiveStringComparator()) &&
         !info[i].mAdapterVendor.Equals(adapterVendorID, nsCaseInsensitiveStringComparator())) {
       continue;
@@ -649,7 +601,7 @@ GfxInfoBase::FindBlocklistedDeviceInList(const nsTArray<GfxDriverInfo>& info,
 
     if (info[i].mDevices != GfxDriverInfo::allDevices && info[i].mDevices->Length()) {
         bool deviceMatches = false;
-        for (PRUint32 j = 0; j < info[i].mDevices->Length(); j++) {
+        for (uint32_t j = 0; j < info[i].mDevices->Length(); j++) {
             if ((*info[i].mDevices)[j].Equals(adapterDeviceID, nsCaseInsensitiveStringComparator())) {
                 deviceMatches = true;
                 break;
@@ -663,6 +615,19 @@ GfxInfoBase::FindBlocklistedDeviceInList(const nsTArray<GfxDriverInfo>& info,
 
     bool match = false;
 
+    if (!info[i].mHardware.IsEmpty() && !info[i].mHardware.Equals(Hardware())) {
+        continue;
+    }
+    if (!info[i].mModel.IsEmpty() && !info[i].mModel.Equals(Model())) {
+        continue;
+    }
+    if (!info[i].mProduct.IsEmpty() && !info[i].mProduct.Equals(Product())) {
+        continue;
+    }
+    if (!info[i].mManufacturer.IsEmpty() && !info[i].mManufacturer.Equals(Manufacturer())) {
+        continue;
+    }
+
 #if defined(XP_WIN) || defined(ANDROID)
     switch (info[i].mComparisonOp) {
     case DRIVER_LESS_THAN:
@@ -692,6 +657,10 @@ GfxInfoBase::FindBlocklistedDeviceInList(const nsTArray<GfxDriverInfo>& info,
     case DRIVER_BETWEEN_INCLUSIVE_START:
       match = driverVersion >= info[i].mDriverVersion && driverVersion < info[i].mDriverVersionMax;
       break;
+    case DRIVER_COMPARISON_IGNORED:
+      // We don't have a comparison op, so we match everything.
+      match = true;
+      break;
     default:
       NS_WARNING("Bogus op in GfxDriverInfo");
       break;
@@ -734,11 +703,11 @@ GfxInfoBase::FindBlocklistedDeviceInList(const nsTArray<GfxDriverInfo>& info,
 }
 
 nsresult
-GfxInfoBase::GetFeatureStatusImpl(PRInt32 aFeature,
-                                  PRInt32* aStatus,
+GfxInfoBase::GetFeatureStatusImpl(int32_t aFeature,
+                                  int32_t* aStatus,
                                   nsAString& aSuggestedVersion,
                                   const nsTArray<GfxDriverInfo>& aDriverInfo,
-                                  OperatingSystem* aOS /* = nsnull */)
+                                  OperatingSystem* aOS /* = nullptr */)
 {
   if (*aStatus != nsIGfxInfo::FEATURE_STATUS_UNKNOWN) {
     // Terminate now with the status determined by the derived type (OS-specific
@@ -762,14 +731,14 @@ GfxInfoBase::GetFeatureStatusImpl(PRInt32 aFeature,
     return NS_OK;
   }
 
-  PRUint64 driverVersion;
+  uint64_t driverVersion;
   ParseDriverVersion(adapterDriverVersionString, &driverVersion);
 
   // Check if the device is blocked from the downloaded blocklist. If not, check
   // the static list after that. This order is used so that we can later escape
   // out of static blocks (i.e. if we were wrong or something was patched, we
   // can back out our static block without doing a release).
-  PRInt32 status;
+  int32_t status;
   if (aDriverInfo.Length()) {
     status = FindBlocklistedDeviceInList(aDriverInfo, aSuggestedVersion, aFeature, os);
   } else {
@@ -790,8 +759,8 @@ GfxInfoBase::GetFeatureStatusImpl(PRInt32 aFeature,
 }
 
 NS_IMETHODIMP
-GfxInfoBase::GetFeatureSuggestedDriverVersion(PRInt32 aFeature,
-                                              nsAString& aVersion NS_OUTPARAM)
+GfxInfoBase::GetFeatureSuggestedDriverVersion(int32_t aFeature,
+                                              nsAString& aVersion)
 {
   nsCString version;
   if (GetPrefValueForDriverVersion(version)) {
@@ -799,7 +768,7 @@ GfxInfoBase::GetFeatureSuggestedDriverVersion(PRInt32 aFeature,
     return NS_OK;
   }
 
-  PRInt32 status;
+  int32_t status;
   nsTArray<GfxDriverInfo> driverInfo;
   return GetFeatureStatusImpl(aFeature, &status, aVersion, driverInfo);
 }
@@ -807,7 +776,7 @@ GfxInfoBase::GetFeatureSuggestedDriverVersion(PRInt32 aFeature,
 
 NS_IMETHODIMP
 GfxInfoBase::GetWebGLParameter(const nsAString& aParam,
-                               nsAString& aResult NS_OUTPARAM)
+                               nsAString& aResult)
 {
   return GfxInfoWebGL::GetWebGLParameter(aParam, aResult);
 }
@@ -815,7 +784,7 @@ GfxInfoBase::GetWebGLParameter(const nsAString& aParam,
 void
 GfxInfoBase::EvaluateDownloadedBlacklist(nsTArray<GfxDriverInfo>& aDriverInfo)
 {
-  PRInt32 features[] = {
+  int32_t features[] = {
     nsIGfxInfo::FEATURE_DIRECT2D,
     nsIGfxInfo::FEATURE_DIRECT3D_9_LAYERS,
     nsIGfxInfo::FEATURE_DIRECT3D_10_LAYERS,
@@ -824,6 +793,7 @@ GfxInfoBase::EvaluateDownloadedBlacklist(nsTArray<GfxDriverInfo>& aDriverInfo)
     nsIGfxInfo::FEATURE_WEBGL_OPENGL,
     nsIGfxInfo::FEATURE_WEBGL_ANGLE,
     nsIGfxInfo::FEATURE_WEBGL_MSAA,
+    nsIGfxInfo::FEATURE_STAGEFRIGHT,
     0
   };
 
@@ -833,7 +803,7 @@ GfxInfoBase::EvaluateDownloadedBlacklist(nsTArray<GfxDriverInfo>& aDriverInfo)
   // anywhere permanent.
   int i = 0;
   while (features[i]) {
-    PRInt32 status;
+    int32_t status;
     nsAutoString suggestedVersion;
     if (NS_SUCCEEDED(GetFeatureStatusImpl(features[i], &status,
                                           suggestedVersion,
@@ -881,13 +851,13 @@ GfxInfoBase::LogFailure(const nsACString &failure)
 
 /* void getFailures ([optional] out unsigned long failureCount, [array, size_is (failureCount), retval] out string failures); */
 /* XPConnect method of returning arrays is very ugly. Would not recommend. Fallable nsMemory::Alloc makes things worse */
-NS_IMETHODIMP GfxInfoBase::GetFailures(PRUint32 *failureCount NS_OUTPARAM, char ***failures NS_OUTPARAM)
+NS_IMETHODIMP GfxInfoBase::GetFailures(uint32_t *failureCount, char ***failures)
 {
 
   NS_ENSURE_ARG_POINTER(failureCount);
   NS_ENSURE_ARG_POINTER(failures);
 
-  *failures = nsnull;
+  *failures = nullptr;
   *failureCount = mFailureCount;
 
   if (*failureCount != 0) {
@@ -896,7 +866,7 @@ NS_IMETHODIMP GfxInfoBase::GetFailures(PRUint32 *failureCount NS_OUTPARAM, char
       return NS_ERROR_OUT_OF_MEMORY;
 
     /* copy over the failure messages into the array we just allocated */
-    for (PRUint32 i = 0; i < *failureCount; i++) {
+    for (uint32_t i = 0; i < *failureCount; i++) {
       nsCString& flattenedFailureMessage(mFailures[i]);
       (*failures)[i] = (char*)nsMemory::Clone(flattenedFailureMessage.get(), flattenedFailureMessage.Length() + 1);
 
@@ -925,7 +895,7 @@ nsresult GfxInfoBase::GetInfo(JSContext* aCx, jsval* aResult)
   InitCollectors();
   InfoObject obj(aCx);
 
-  for (PRUint32 i = 0; i < sCollectors->Length(); i++) {
+  for (uint32_t i = 0; i < sCollectors->Length(); i++) {
     (*sCollectors)[i]->GetInfo(obj);
   }
 
@@ -953,7 +923,7 @@ void
 GfxInfoBase::RemoveCollector(GfxInfoCollectorBase* collector)
 {
   InitCollectors();
-  for (PRUint32 i = 0; i < sCollectors->Length(); i++) {
+  for (uint32_t i = 0; i < sCollectors->Length(); i++) {
     if ((*sCollectors)[i] == collector) {
       sCollectors->RemoveElementAt(i);
       break;
@@ -961,7 +931,7 @@ GfxInfoBase::RemoveCollector(GfxInfoCollectorBase* collector)
   }
   if (sCollectors->IsEmpty()) {
     delete sCollectors;
-    sCollectors = nsnull;
+    sCollectors = nullptr;
   }
 }
 
