@@ -1,55 +1,28 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
  * vim: sw=2 ts=8 et :
  */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at:
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Code.
- *
- * The Initial Developer of the Original Code is
- *   The Mozilla Foundation
- * Portions created by the Initial Developer are Copyright (C) 2010
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Chris Jones <jones.chris.g@gmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+
+#include "base/basictypes.h"
 
-#include "mozilla/dom/PBrowserChild.h"
 #include "BasicLayers.h"
+#include "gfxPlatform.h"
 #if defined(MOZ_ENABLE_D3D10_LAYER)
 # include "LayerManagerD3D10.h"
 #endif
-
-#include "gfxPlatform.h"
+#include "mozilla/dom/TabChild.h"
+#include "mozilla/Hal.h"
+#include "mozilla/layers/CompositorChild.h"
+#include "mozilla/layers/PLayersChild.h"
 #include "PuppetWidget.h"
+#include "nsIWidgetListener.h"
 
+using namespace mozilla::dom;
+using namespace mozilla::hal;
 using namespace mozilla::layers;
 using namespace mozilla::widget;
-using namespace mozilla::dom;
 
 static void
 InvalidateRegion(nsIWidget* aWidget, const nsIntRegion& aRegion)
@@ -61,7 +34,7 @@ InvalidateRegion(nsIWidget* aWidget, const nsIntRegion& aRegion)
 }
 
 /*static*/ already_AddRefed<nsIWidget>
-nsIWidget::CreatePuppetWidget(PBrowserChild *aTabChild)
+nsIWidget::CreatePuppetWidget(TabChild* aTabChild)
 {
   NS_ABORT_IF_FALSE(nsIWidget::UsePuppetWidgets(),
                     "PuppetWidgets not allowed in this configuration");
@@ -94,7 +67,7 @@ const size_t PuppetWidget::kMaxDimension = 4000;
 NS_IMPL_ISUPPORTS_INHERITED1(PuppetWidget, nsBaseWidget,
                              nsISupportsWeakReference)
 
-PuppetWidget::PuppetWidget(PBrowserChild *aTabChild)
+PuppetWidget::PuppetWidget(TabChild* aTabChild)
   : mTabChild(aTabChild)
   , mDPI(-1)
 {
@@ -110,13 +83,12 @@ NS_IMETHODIMP
 PuppetWidget::Create(nsIWidget        *aParent,
                      nsNativeWidget   aNativeParent,
                      const nsIntRect  &aRect,
-                     EVENT_CALLBACK   aHandleEventFunction,
                      nsDeviceContext *aContext,
                      nsWidgetInitData *aInitData)
 {
   NS_ABORT_IF_FALSE(!aNativeParent, "got a non-Puppet native parent");
 
-  BaseCreate(nsnull, aRect, aHandleEventFunction, aContext, aInitData);
+  BaseCreate(nullptr, aRect, aContext, aInitData);
 
   mBounds = aRect;
   mEnabled = true;
@@ -128,7 +100,7 @@ PuppetWidget::Create(nsIWidget        *aParent,
 
   mIMEComposing = false;
   if (MightNeedIMEFocus(aInitData)) {
-    PRUint32 chromeSeqno;
+    uint32_t chromeSeqno;
     mTabChild->SendNotifyIMEFocus(false, &mIMEPreference, &chromeSeqno);
     mIMELastBlurSeqno = mIMELastReceivedSeqno = chromeSeqno;
   }
@@ -147,7 +119,6 @@ PuppetWidget::Create(nsIWidget        *aParent,
 
 already_AddRefed<nsIWidget>
 PuppetWidget::CreateChild(const nsIntRect  &aRect,
-                          EVENT_CALLBACK   aHandleEventFunction,
                           nsDeviceContext *aContext,
                           nsWidgetInitData *aInitData,
                           bool             aForceUseIWidgetParent)
@@ -155,10 +126,9 @@ PuppetWidget::CreateChild(const nsIntRect  &aRect,
   bool isPopup = IsPopup(aInitData);
   nsCOMPtr<nsIWidget> widget = nsIWidget::CreatePuppetWidget(mTabChild);
   return ((widget &&
-           NS_SUCCEEDED(widget->Create(isPopup ? nsnull: this, nsnull, aRect,
-                                       aHandleEventFunction,
+           NS_SUCCEEDED(widget->Create(isPopup ? nullptr: this, nullptr, aRect,
                                        aContext, aInitData))) ?
-          widget.forget() : nsnull);
+          widget.forget() : nullptr);
 }
 
 NS_IMETHODIMP
@@ -167,12 +137,12 @@ PuppetWidget::Destroy()
   Base::OnDestroy();
   Base::Destroy();
   mPaintTask.Revoke();
-  mChild = nsnull;
+  mChild = nullptr;
   if (mLayerManager) {
     mLayerManager->Destroy();
   }
-  mLayerManager = nsnull;
-  mTabChild = nsnull;
+  mLayerManager = nullptr;
+  mTabChild = nullptr;
   return NS_OK;
 }
 
@@ -193,8 +163,8 @@ PuppetWidget::Show(bool aState)
 }
 
 NS_IMETHODIMP
-PuppetWidget::Resize(PRInt32 aWidth,
-                     PRInt32 aHeight,
+PuppetWidget::Resize(int32_t aWidth,
+                     int32_t aHeight,
                      bool    aRepaint)
 {
   nsIntRect oldBounds = mBounds;
@@ -212,8 +182,8 @@ PuppetWidget::Resize(PRInt32 aWidth,
     InvalidateRegion(this, dirty);
   }
 
-  if (!oldBounds.IsEqualEdges(mBounds)) {
-    DispatchResizeEvent();
+  if (!oldBounds.IsEqualEdges(mBounds) && mAttachedWidgetListener) {
+    mAttachedWidgetListener->WindowResized(this, mBounds.width, mBounds.height);
   }
 
   return NS_OK;
@@ -232,7 +202,7 @@ PuppetWidget::Invalidate(const nsIntRect& aRect)
 {
 #ifdef DEBUG
   debug_DumpInvalidate(stderr, this, &aRect,
-                       nsCAutoString("PuppetWidget"), nsnull);
+                       nsCAutoString("PuppetWidget"), 0);
 #endif
 
   if (mChild) {
@@ -252,7 +222,7 @@ PuppetWidget::Invalidate(const nsIntRect& aRect)
 void
 PuppetWidget::InitEvent(nsGUIEvent& event, nsIntPoint* aPoint)
 {
-  if (nsnull == aPoint) {
+  if (nullptr == aPoint) {
     event.refPoint.x = 0;
     event.refPoint.y = 0;
   }
@@ -269,7 +239,7 @@ PuppetWidget::DispatchEvent(nsGUIEvent* event, nsEventStatus& aStatus)
 {
 #ifdef DEBUG
   debug_DumpEvent(stdout, event->widget, event,
-                  nsCAutoString("PuppetWidget"), nsnull);
+                  nsCAutoString("PuppetWidget"), 0);
 #endif
 
   NS_ABORT_IF_FALSE(!mChild || mChild->mWindowType == eWindowType_popup,
@@ -277,7 +247,7 @@ PuppetWidget::DispatchEvent(nsGUIEvent* event, nsEventStatus& aStatus)
 
   aStatus = nsEventStatus_eIgnore;
 
-  NS_ABORT_IF_FALSE(mViewCallback, "No view callback!");
+  NS_ABORT_IF_FALSE(mAttachedWidgetListener, "No listener!");
 
   if (event->message == NS_COMPOSITION_START) {
     mIMEComposing = true;
@@ -299,7 +269,8 @@ PuppetWidget::DispatchEvent(nsGUIEvent* event, nsEventStatus& aStatus)
       return NS_OK;
     break;
   }
-  aStatus = (*mViewCallback)(event);
+
+  aStatus = mAttachedWidgetListener->HandleEvent(event, mUseAttachedEvents);
 
   if (event->message == NS_COMPOSITION_END) {
     mIMEComposing = false;
@@ -318,7 +289,7 @@ PuppetWidget::GetLayerManager(PLayersChild* aShadowManager,
     // The backend hint is a temporary placeholder until Azure, when
     // all content-process layer managers will be BasicLayerManagers.
 #if defined(MOZ_ENABLE_D3D10_LAYER)
-    if (LayerManager::LAYERS_D3D10 == aBackendHint) {
+    if (mozilla::layers::LAYERS_D3D10 == aBackendHint) {
       nsRefPtr<LayerManagerD3D10> m = new LayerManagerD3D10(this);
       m->AsShadowForwarder()->SetShadowManager(aShadowManager);
       if (m->Initialize()) {
@@ -348,7 +319,7 @@ PuppetWidget::IMEEndComposition(bool aCancel)
 {
   nsEventStatus status;
   nsTextEvent textEvent(true, NS_TEXT_TEXT, this);
-  InitEvent(textEvent, nsnull);
+  InitEvent(textEvent, nullptr);
   textEvent.seqno = mIMELastReceivedSeqno;
   // SendEndIMEComposition is always called since ResetInputState
   // should always be called even if we aren't composing something.
@@ -363,7 +334,7 @@ PuppetWidget::IMEEndComposition(bool aCancel)
   DispatchEvent(&textEvent, status);
 
   nsCompositionEvent compEvent(true, NS_COMPOSITION_END, this);
-  InitEvent(compEvent, nsnull);
+  InitEvent(compEvent, nullptr);
   compEvent.seqno = mIMELastReceivedSeqno;
   DispatchEvent(&compEvent, status);
   return NS_OK;
@@ -389,12 +360,13 @@ PuppetWidget::SetInputContext(const InputContext& aContext,
     return;
   }
   mTabChild->SendSetInputContext(
-    static_cast<PRInt32>(aContext.mIMEState.mEnabled),
-    static_cast<PRInt32>(aContext.mIMEState.mOpen),
+    static_cast<int32_t>(aContext.mIMEState.mEnabled),
+    static_cast<int32_t>(aContext.mIMEState.mOpen),
     aContext.mHTMLInputType,
+    aContext.mHTMLInputInputmode,
     aContext.mActionHint,
-    static_cast<PRInt32>(aAction.mCause),
-    static_cast<PRInt32>(aAction.mFocusChange));
+    static_cast<int32_t>(aAction.mCause),
+    static_cast<int32_t>(aAction.mFocusChange));
 }
 
 NS_IMETHODIMP_(InputContext)
@@ -402,7 +374,7 @@ PuppetWidget::GetInputContext()
 {
   InputContext context;
   if (mTabChild) {
-    PRInt32 enabled, open;
+    int32_t enabled, open;
     mTabChild->SendGetInputContext(&enabled, &open);
     context.mIMEState.mEnabled = static_cast<IMEState::Enabled>(enabled);
     context.mIMEState.mOpen = static_cast<IMEState::Open>(open);
@@ -419,7 +391,7 @@ PuppetWidget::OnIMEFocusChange(bool aFocus)
   if (aFocus) {
     nsEventStatus status;
     nsQueryContentEvent queryEvent(true, NS_QUERY_TEXT_CONTENT, this);
-    InitEvent(queryEvent, nsnull);
+    InitEvent(queryEvent, nullptr);
     // Query entire content
     queryEvent.InitForQueryTextContent(0, PR_UINT32_MAX);
     DispatchEvent(&queryEvent, status);
@@ -432,7 +404,7 @@ PuppetWidget::OnIMEFocusChange(bool aFocus)
     ResetInputState();
   }
 
-  PRUint32 chromeSeqno;
+  uint32_t chromeSeqno;
   mIMEPreference.mWantUpdates = false;
   mIMEPreference.mWantHints = false;
   if (!mTabChild->SendNotifyIMEFocus(aFocus, &mIMEPreference, &chromeSeqno))
@@ -450,7 +422,7 @@ PuppetWidget::OnIMEFocusChange(bool aFocus)
 }
 
 NS_IMETHODIMP
-PuppetWidget::OnIMETextChange(PRUint32 aStart, PRUint32 aEnd, PRUint32 aNewEnd)
+PuppetWidget::OnIMETextChange(uint32_t aStart, uint32_t aEnd, uint32_t aNewEnd)
 {
   if (!mTabChild)
     return NS_ERROR_FAILURE;
@@ -458,7 +430,7 @@ PuppetWidget::OnIMETextChange(PRUint32 aStart, PRUint32 aEnd, PRUint32 aNewEnd)
   if (mIMEPreference.mWantHints) {
     nsEventStatus status;
     nsQueryContentEvent queryEvent(true, NS_QUERY_TEXT_CONTENT, this);
-    InitEvent(queryEvent, nsnull);
+    InitEvent(queryEvent, nullptr);
     queryEvent.InitForQueryTextContent(0, PR_UINT32_MAX);
     DispatchEvent(&queryEvent, status);
 
@@ -481,7 +453,7 @@ PuppetWidget::OnIMESelectionChange(void)
   if (mIMEPreference.mWantUpdates) {
     nsEventStatus status;
     nsQueryContentEvent queryEvent(true, NS_QUERY_SELECTED_TEXT, this);
-    InitEvent(queryEvent, nsnull);
+    InitEvent(queryEvent, nullptr);
     DispatchEvent(&queryEvent, status);
 
     if (queryEvent.mSucceeded) {
@@ -504,62 +476,43 @@ PuppetWidget::SetCursor(nsCursor aCursor)
 }
 
 nsresult
-PuppetWidget::DispatchPaintEvent()
+PuppetWidget::Paint()
 {
   NS_ABORT_IF_FALSE(!mDirtyRegion.IsEmpty(), "paint event logic messed up");
 
-  nsIntRect dirtyRect = mDirtyRegion.GetBounds();
-  nsPaintEvent event(true, NS_PAINT, this);
-  event.refPoint.x = dirtyRect.x;
-  event.refPoint.y = dirtyRect.y;
-  event.region = mDirtyRegion;
-  event.willSendDidPaint = true;
+  if (!mAttachedWidgetListener)
+    return NS_OK;
+
+  nsIntRegion region = mDirtyRegion;
 
   // reset repaint tracking
   mDirtyRegion.SetEmpty();
   mPaintTask.Revoke();
 
-  nsEventStatus status;
   {
 #ifdef DEBUG
-    debug_DumpPaintEvent(stderr, this, &event,
-                         nsCAutoString("PuppetWidget"), nsnull);
+    debug_DumpPaintEvent(stderr, this, region,
+                         nsCAutoString("PuppetWidget"), 0);
 #endif
 
-    if (LayerManager::LAYERS_D3D10 == mLayerManager->GetBackendType()) {
-      DispatchEvent(&event, status);
+    if (mozilla::layers::LAYERS_D3D10 == mLayerManager->GetBackendType()) {
+      mAttachedWidgetListener->PaintWindow(this, region, false, true);
     } else {
       nsRefPtr<gfxContext> ctx = new gfxContext(mSurface);
       ctx->Rectangle(gfxRect(0,0,0,0));
       ctx->Clip();
       AutoLayerManagerSetup setupLayerManager(this, ctx,
-                                              BasicLayerManager::BUFFER_NONE);
-      DispatchEvent(&event, status);  
+                                              BUFFER_NONE);
+      mAttachedWidgetListener->PaintWindow(this, region, false, true);
+      mTabChild->NotifyPainted();
     }
   }
 
-  nsPaintEvent didPaintEvent(true, NS_DID_PAINT, this);
-  DispatchEvent(&didPaintEvent, status);
+  mAttachedWidgetListener->DidPaintWindow();
 
   return NS_OK;
 }
 
-nsresult
-PuppetWidget::DispatchResizeEvent()
-{
-  nsSizeEvent event(true, NS_SIZE, this);
-
-  nsIntRect rect = mBounds;     // copy in case something messes with it
-  event.windowSize = &rect;
-  event.refPoint.x = rect.x;
-  event.refPoint.y = rect.y;
-  event.mWinWidth = rect.width;
-  event.mWinHeight = rect.height;
-
-  nsEventStatus status;
-  return DispatchEvent(&event, status);
-}
-
 void
 PuppetWidget::SetChild(PuppetWidget* aChild)
 {
@@ -574,7 +527,7 @@ NS_IMETHODIMP
 PuppetWidget::PaintTask::Run()
 {
   if (mWidget) {
-    mWidget->DispatchPaintEvent();
+    mWidget->Paint();
   }
   return NS_OK;
 }
@@ -591,28 +544,134 @@ PuppetWidget::GetDPI()
 }
 
 void*
-PuppetWidget::GetNativeData(PRUint32 aDataType)
+PuppetWidget::GetNativeData(uint32_t aDataType)
 {
-    switch (aDataType) {
-    case NS_NATIVE_SHAREABLE_WINDOW: {
-        NS_ABORT_IF_FALSE(mTabChild, "Need TabChild to get the nativeWindow from!");
-        mozilla::WindowsHandle nativeData = nsnull;
-        mTabChild->SendGetWidgetNativeData(&nativeData);
-        return (void*)nativeData;
-    }
-    case NS_NATIVE_WINDOW:
-    case NS_NATIVE_DISPLAY:
-    case NS_NATIVE_PLUGIN_PORT:
-    case NS_NATIVE_GRAPHIC:
-    case NS_NATIVE_SHELLWIDGET:
-    case NS_NATIVE_WIDGET:
-        NS_WARNING("nsWindow::GetNativeData not implemented for this type");
-        break;
-    default:
-        NS_WARNING("nsWindow::GetNativeData called with bad value");
-        break;
-    }
-    return nsnull;
+  switch (aDataType) {
+  case NS_NATIVE_SHAREABLE_WINDOW: {
+    NS_ABORT_IF_FALSE(mTabChild, "Need TabChild to get the nativeWindow from!");
+    mozilla::WindowsHandle nativeData = 0;
+    mTabChild->SendGetWidgetNativeData(&nativeData);
+    return (void*)nativeData;
+  }
+  case NS_NATIVE_WINDOW:
+  case NS_NATIVE_DISPLAY:
+  case NS_NATIVE_PLUGIN_PORT:
+  case NS_NATIVE_GRAPHIC:
+  case NS_NATIVE_SHELLWIDGET:
+  case NS_NATIVE_WIDGET:
+    NS_WARNING("nsWindow::GetNativeData not implemented for this type");
+    break;
+  default:
+    NS_WARNING("nsWindow::GetNativeData called with bad value");
+    break;
+  }
+  return nullptr;
+}
+
+PuppetScreen::PuppetScreen(void *nativeScreen)
+{
+}
+
+PuppetScreen::~PuppetScreen()
+{
+}
+
+static ScreenConfiguration
+ScreenConfig()
+{
+  ScreenConfiguration config;
+  hal::GetCurrentScreenConfiguration(&config);
+  return config;
+}
+
+NS_IMETHODIMP
+PuppetScreen::GetRect(int32_t *outLeft,  int32_t *outTop,
+                      int32_t *outWidth, int32_t *outHeight)
+{
+  nsIntRect r = ScreenConfig().rect();
+  *outLeft = r.x;
+  *outTop = r.y;
+  *outWidth = r.width;
+  *outHeight = r.height;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+PuppetScreen::GetAvailRect(int32_t *outLeft,  int32_t *outTop,
+                           int32_t *outWidth, int32_t *outHeight)
+{
+  return GetRect(outLeft, outTop, outWidth, outHeight);
+}
+
+
+NS_IMETHODIMP
+PuppetScreen::GetPixelDepth(int32_t *aPixelDepth)
+{
+  *aPixelDepth = ScreenConfig().pixelDepth();
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+PuppetScreen::GetColorDepth(int32_t *aColorDepth)
+{
+  *aColorDepth = ScreenConfig().colorDepth();
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+PuppetScreen::GetRotation(uint32_t* aRotation)
+{
+  NS_WARNING("Attempt to get screen rotation through nsIScreen::GetRotation().  Nothing should know or care this in sandboxed contexts.  If you want *orientation*, use hal.");
+  return NS_ERROR_NOT_AVAILABLE;
+}
+
+NS_IMETHODIMP
+PuppetScreen::SetRotation(uint32_t aRotation)
+{
+  NS_WARNING("Attempt to set screen rotation through nsIScreen::GetRotation().  Nothing should know or care this in sandboxed contexts.  If you want *orientation*, use hal.");
+  return NS_ERROR_NOT_AVAILABLE;
+}
+
+NS_IMPL_ISUPPORTS1(PuppetScreenManager, nsIScreenManager)
+
+PuppetScreenManager::PuppetScreenManager()
+{
+    mOneScreen = new PuppetScreen(nullptr);
+}
+
+PuppetScreenManager::~PuppetScreenManager()
+{
+}
+
+NS_IMETHODIMP
+PuppetScreenManager::GetPrimaryScreen(nsIScreen** outScreen)
+{
+  NS_IF_ADDREF(*outScreen = mOneScreen.get());
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+PuppetScreenManager::ScreenForRect(int32_t inLeft,
+                                   int32_t inTop,
+                                   int32_t inWidth,
+                                   int32_t inHeight,
+                                   nsIScreen** outScreen)
+{
+  return GetPrimaryScreen(outScreen);
+}
+
+NS_IMETHODIMP
+PuppetScreenManager::ScreenForNativeWidget(void* aWidget,
+                                           nsIScreen** outScreen)
+{
+  return GetPrimaryScreen(outScreen);
+}
+
+NS_IMETHODIMP
+PuppetScreenManager::GetNumberOfScreens(uint32_t* aNumberOfScreens)
+{
+  *aNumberOfScreens = 1;
+  return NS_OK;
 }
 
 }  // namespace widget
