@@ -1,39 +1,7 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2000
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsPrintSettingsImpl.h"
 #include "nsReadableUtils.h"
@@ -81,7 +49,7 @@ nsPrintSettings::nsPrintSettings() :
 {
 
   /* member initializers and constructor code */
-  PRInt32 marginWidth = NS_INCHES_TO_INT_TWIPS(DEFAULT_MARGIN_WIDTH);
+  int32_t marginWidth = NS_INCHES_TO_INT_TWIPS(DEFAULT_MARGIN_WIDTH);
   mMargin.SizeTo(marginWidth, marginWidth, marginWidth, marginWidth);
   mEdge.SizeTo(0, 0, 0, 0);
   mUnwriteableMargin.SizeTo(0,0,0,0);
@@ -117,7 +85,7 @@ nsPrintSettings::~nsPrintSettings()
 NS_IMETHODIMP nsPrintSettings::GetPrintSession(nsIPrintSession **aPrintSession)
 {
   NS_ENSURE_ARG_POINTER(aPrintSession);
-  *aPrintSession = nsnull;
+  *aPrintSession = nullptr;
   
   nsCOMPtr<nsIPrintSession> session = do_QueryReferent(mSession);
   if (!session)
@@ -143,26 +111,26 @@ NS_IMETHODIMP nsPrintSettings::SetPrintSession(nsIPrintSession *aPrintSession)
 }
 
 /* attribute long startPageRange; */
-NS_IMETHODIMP nsPrintSettings::GetStartPageRange(PRInt32 *aStartPageRange)
+NS_IMETHODIMP nsPrintSettings::GetStartPageRange(int32_t *aStartPageRange)
 {
   //NS_ENSURE_ARG_POINTER(aStartPageRange);
   *aStartPageRange = mStartPageNum;
   return NS_OK;
 }
-NS_IMETHODIMP nsPrintSettings::SetStartPageRange(PRInt32 aStartPageRange)
+NS_IMETHODIMP nsPrintSettings::SetStartPageRange(int32_t aStartPageRange)
 {
   mStartPageNum = aStartPageRange;
   return NS_OK;
 }
 
 /* attribute long endPageRange; */
-NS_IMETHODIMP nsPrintSettings::GetEndPageRange(PRInt32 *aEndPageRange)
+NS_IMETHODIMP nsPrintSettings::GetEndPageRange(int32_t *aEndPageRange)
 {
   //NS_ENSURE_ARG_POINTER(aEndPageRange);
   *aEndPageRange = mEndPageNum;
   return NS_OK;
 }
-NS_IMETHODIMP nsPrintSettings::SetEndPageRange(PRInt32 aEndPageRange)
+NS_IMETHODIMP nsPrintSettings::SetEndPageRange(int32_t aEndPageRange)
 {
   mEndPageNum = aEndPageRange;
   return NS_OK;
@@ -195,13 +163,13 @@ NS_IMETHODIMP nsPrintSettings::SetPrintInColor(bool aPrintInColor)
 }
 
 /* attribute short orientation; */
-NS_IMETHODIMP nsPrintSettings::GetOrientation(PRInt32 *aOrientation)
+NS_IMETHODIMP nsPrintSettings::GetOrientation(int32_t *aOrientation)
 {
   NS_ENSURE_ARG_POINTER(aOrientation);
   *aOrientation = mOrientation;
   return NS_OK;
 }
-NS_IMETHODIMP nsPrintSettings::SetOrientation(PRInt32 aOrientation)
+NS_IMETHODIMP nsPrintSettings::SetOrientation(int32_t aOrientation)
 {
   mOrientation = aOrientation;
   return NS_OK;
@@ -214,7 +182,7 @@ NS_IMETHODIMP nsPrintSettings::GetColorspace(PRUnichar * *aColorspace)
   if (!mColorspace.IsEmpty()) {
     *aColorspace = ToNewUnicode(mColorspace);
   } else {
-    *aColorspace = nsnull;
+    *aColorspace = nullptr;
   }
   return NS_OK;
 }
@@ -235,7 +203,7 @@ NS_IMETHODIMP nsPrintSettings::GetResolutionName(PRUnichar * *aResolutionName)
   if (!mResolutionName.IsEmpty()) {
     *aResolutionName = ToNewUnicode(mResolutionName);
   } else {
-    *aResolutionName = nsnull;
+    *aResolutionName = nullptr;
   }
   return NS_OK;
 }
@@ -285,13 +253,13 @@ NS_IMETHODIMP nsPrintSettings::SetPrinterName(const PRUnichar * aPrinter)
 }
 
 /* attribute long numCopies; */
-NS_IMETHODIMP nsPrintSettings::GetNumCopies(PRInt32 *aNumCopies)
+NS_IMETHODIMP nsPrintSettings::GetNumCopies(int32_t *aNumCopies)
 {
   NS_ENSURE_ARG_POINTER(aNumCopies);
   *aNumCopies = mNumCopies;
   return NS_OK;
 }
-NS_IMETHODIMP nsPrintSettings::SetNumCopies(PRInt32 aNumCopies)
+NS_IMETHODIMP nsPrintSettings::SetNumCopies(int32_t aNumCopies)
 {
   mNumCopies = aNumCopies;
   return NS_OK;
@@ -345,25 +313,25 @@ NS_IMETHODIMP nsPrintSettings::SetToFileName(const PRUnichar * aToFileName)
 }
 
 /* attribute short outputFormat; */
-NS_IMETHODIMP nsPrintSettings::GetOutputFormat(PRInt16 *aOutputFormat)
+NS_IMETHODIMP nsPrintSettings::GetOutputFormat(int16_t *aOutputFormat)
 {
   NS_ENSURE_ARG_POINTER(aOutputFormat);
   *aOutputFormat = mOutputFormat;
   return NS_OK;
 }
-NS_IMETHODIMP nsPrintSettings::SetOutputFormat(PRInt16 aOutputFormat)
+NS_IMETHODIMP nsPrintSettings::SetOutputFormat(int16_t aOutputFormat)
 {
   mOutputFormat = aOutputFormat;
   return NS_OK;
 }
 
 /* attribute long printPageDelay; */
-NS_IMETHODIMP nsPrintSettings::GetPrintPageDelay(PRInt32 *aPrintPageDelay)
+NS_IMETHODIMP nsPrintSettings::GetPrintPageDelay(int32_t *aPrintPageDelay)
 {
   *aPrintPageDelay = mPrintPageDelay;
   return NS_OK;
 }
-NS_IMETHODIMP nsPrintSettings::SetPrintPageDelay(PRInt32 aPrintPageDelay)
+NS_IMETHODIMP nsPrintSettings::SetPrintPageDelay(int32_t aPrintPageDelay)
 {
   mPrintPageDelay = aPrintPageDelay;
   return NS_OK;
@@ -600,13 +568,13 @@ NS_IMETHODIMP nsPrintSettings::SetPrintBGImages(bool aPrintBGImages)
 }
 
 /* attribute long printRange; */
-NS_IMETHODIMP nsPrintSettings::GetPrintRange(PRInt16 *aPrintRange)
+NS_IMETHODIMP nsPrintSettings::GetPrintRange(int16_t *aPrintRange)
 {
   NS_ENSURE_ARG_POINTER(aPrintRange);
   *aPrintRange = mPrintRange;
   return NS_OK;
 }
-NS_IMETHODIMP nsPrintSettings::SetPrintRange(PRInt16 aPrintRange)
+NS_IMETHODIMP nsPrintSettings::SetPrintRange(int16_t aPrintRange)
 {
   mPrintRange = aPrintRange;
   return NS_OK;
@@ -619,7 +587,7 @@ NS_IMETHODIMP nsPrintSettings::GetTitle(PRUnichar * *aTitle)
   if (!mTitle.IsEmpty()) {
     *aTitle = ToNewUnicode(mTitle);
   } else {
-    *aTitle = nsnull;
+    *aTitle = nullptr;
   }
   return NS_OK;
 }
@@ -640,7 +608,7 @@ NS_IMETHODIMP nsPrintSettings::GetDocURL(PRUnichar * *aDocURL)
   if (!mURL.IsEmpty()) {
     *aDocURL = ToNewUnicode(mURL);
   } else {
-    *aDocURL = nsnull;
+    *aDocURL = nullptr;
   }
   return NS_OK;
 }
@@ -659,7 +627,7 @@ NS_IMETHODIMP nsPrintSettings::SetDocURL(const PRUnichar * aDocURL)
  *	@update 1/12/01 rods
  */
 NS_IMETHODIMP 
-nsPrintSettings::GetPrintOptions(PRInt32 aType, bool *aTurnOnOff)
+nsPrintSettings::GetPrintOptions(int32_t aType, bool *aTurnOnOff)
 {
   NS_ENSURE_ARG_POINTER(aTurnOnOff);
   *aTurnOnOff = mPrintOptions & aType ? true : false;
@@ -670,7 +638,7 @@ nsPrintSettings::GetPrintOptions(PRInt32 aType, bool *aTurnOnOff)
  *	@update 1/12/01 rods
  */
 NS_IMETHODIMP 
-nsPrintSettings::SetPrintOptions(PRInt32 aType, bool aTurnOnOff)
+nsPrintSettings::SetPrintOptions(int32_t aType, bool aTurnOnOff)
 {
   if (aTurnOnOff) {
     mPrintOptions |=  aType;
@@ -685,7 +653,7 @@ nsPrintSettings::SetPrintOptions(PRInt32 aType, bool aTurnOnOff)
  *	@update 1/12/01 rods
  */
 NS_IMETHODIMP 
-nsPrintSettings::GetPrintOptionsBits(PRInt32 *aBits)
+nsPrintSettings::GetPrintOptionsBits(int32_t *aBits)
 {
   NS_ENSURE_ARG_POINTER(aBits);
   *aBits = mPrintOptions;
@@ -696,10 +664,10 @@ nsPrintSettings::GetPrintOptionsBits(PRInt32 *aBits)
 nsresult 
 nsPrintSettings::GetMarginStrs(PRUnichar * *aTitle, 
                               nsHeaderFooterEnum aType, 
-                              PRInt16 aJust)
+                              int16_t aJust)
 {
   NS_ENSURE_ARG_POINTER(aTitle);
-  *aTitle = nsnull;
+  *aTitle = nullptr;
   if (aType == eHeader) {
     switch (aJust) {
       case kJustLeft:   *aTitle = ToNewUnicode(mHeaderStrs[0]);break;
@@ -719,7 +687,7 @@ nsPrintSettings::GetMarginStrs(PRUnichar * *aTitle,
 nsresult
 nsPrintSettings::SetMarginStrs(const PRUnichar * aTitle, 
                               nsHeaderFooterEnum aType, 
-                              PRInt16 aJust)
+                              int16_t aJust)
 {
   NS_ENSURE_ARG_POINTER(aTitle);
   if (aType == eHeader) {
@@ -800,26 +768,26 @@ NS_IMETHODIMP nsPrintSettings::SetFooterStrRight(const PRUnichar * aTitle)
 }
 
 /* attribute short printFrameTypeUsage; */
-NS_IMETHODIMP nsPrintSettings::GetPrintFrameTypeUsage(PRInt16 *aPrintFrameTypeUsage)
+NS_IMETHODIMP nsPrintSettings::GetPrintFrameTypeUsage(int16_t *aPrintFrameTypeUsage)
 {
   NS_ENSURE_ARG_POINTER(aPrintFrameTypeUsage);
   *aPrintFrameTypeUsage = mPrintFrameTypeUsage;
   return NS_OK;
 }
-NS_IMETHODIMP nsPrintSettings::SetPrintFrameTypeUsage(PRInt16 aPrintFrameTypeUsage)
+NS_IMETHODIMP nsPrintSettings::SetPrintFrameTypeUsage(int16_t aPrintFrameTypeUsage)
 {
   mPrintFrameTypeUsage = aPrintFrameTypeUsage;
   return NS_OK;
 }
 
 /* attribute long printFrameType; */
-NS_IMETHODIMP nsPrintSettings::GetPrintFrameType(PRInt16 *aPrintFrameType)
+NS_IMETHODIMP nsPrintSettings::GetPrintFrameType(int16_t *aPrintFrameType)
 {
   NS_ENSURE_ARG_POINTER(aPrintFrameType);
-  *aPrintFrameType = (PRInt32)mPrintFrameType;
+  *aPrintFrameType = (int32_t)mPrintFrameType;
   return NS_OK;
 }
-NS_IMETHODIMP nsPrintSettings::SetPrintFrameType(PRInt16 aPrintFrameType)
+NS_IMETHODIMP nsPrintSettings::SetPrintFrameType(int16_t aPrintFrameType)
 {
   mPrintFrameType = aPrintFrameType;
   return NS_OK;
@@ -871,7 +839,7 @@ NS_IMETHODIMP nsPrintSettings::GetPaperName(PRUnichar * *aPaperName)
   if (!mPaperName.IsEmpty()) {
     *aPaperName = ToNewUnicode(mPaperName);
   } else {
-    *aPaperName = nsnull;
+    *aPaperName = nullptr;
   }
   return NS_OK;
 }
@@ -892,7 +860,7 @@ NS_IMETHODIMP nsPrintSettings::GetPlexName(PRUnichar * *aPlexName)
   if (!mPlexName.IsEmpty()) {
     *aPlexName = ToNewUnicode(mPlexName);
   } else {
-    *aPlexName = nsnull;
+    *aPlexName = nullptr;
   }
   return NS_OK;
 }
@@ -907,13 +875,13 @@ NS_IMETHODIMP nsPrintSettings::SetPlexName(const PRUnichar * aPlexName)
 }
 
 /* attribute boolean howToEnableFrameUI; */
-NS_IMETHODIMP nsPrintSettings::GetHowToEnableFrameUI(PRInt16 *aHowToEnableFrameUI)
+NS_IMETHODIMP nsPrintSettings::GetHowToEnableFrameUI(int16_t *aHowToEnableFrameUI)
 {
   NS_ENSURE_ARG_POINTER(aHowToEnableFrameUI);
   *aHowToEnableFrameUI = mHowToEnableFrameUI;
   return NS_OK;
 }
-NS_IMETHODIMP nsPrintSettings::SetHowToEnableFrameUI(PRInt16 aHowToEnableFrameUI)
+NS_IMETHODIMP nsPrintSettings::SetHowToEnableFrameUI(int16_t aHowToEnableFrameUI)
 {
   mHowToEnableFrameUI = aHowToEnableFrameUI;
   return NS_OK;
@@ -959,39 +927,39 @@ NS_IMETHODIMP nsPrintSettings::SetPaperHeight(double aPaperHeight)
 }
 
 /* attribute short PaperSizeUnit; */
-NS_IMETHODIMP nsPrintSettings::GetPaperSizeUnit(PRInt16 *aPaperSizeUnit)
+NS_IMETHODIMP nsPrintSettings::GetPaperSizeUnit(int16_t *aPaperSizeUnit)
 {
   NS_ENSURE_ARG_POINTER(aPaperSizeUnit);
   *aPaperSizeUnit = mPaperSizeUnit;
   return NS_OK;
 }
-NS_IMETHODIMP nsPrintSettings::SetPaperSizeUnit(PRInt16 aPaperSizeUnit)
+NS_IMETHODIMP nsPrintSettings::SetPaperSizeUnit(int16_t aPaperSizeUnit)
 {
   mPaperSizeUnit = aPaperSizeUnit;
   return NS_OK;
 }
 
 /* attribute short PaperSizeType; */
-NS_IMETHODIMP nsPrintSettings::GetPaperSizeType(PRInt16 *aPaperSizeType)
+NS_IMETHODIMP nsPrintSettings::GetPaperSizeType(int16_t *aPaperSizeType)
 {
   NS_ENSURE_ARG_POINTER(aPaperSizeType);
   *aPaperSizeType = mPaperSizeType;
   return NS_OK;
 }
-NS_IMETHODIMP nsPrintSettings::SetPaperSizeType(PRInt16 aPaperSizeType)
+NS_IMETHODIMP nsPrintSettings::SetPaperSizeType(int16_t aPaperSizeType)
 {
   mPaperSizeType = aPaperSizeType;
   return NS_OK;
 }
 
 /* attribute short PaperData; */
-NS_IMETHODIMP nsPrintSettings::GetPaperData(PRInt16 *aPaperData)
+NS_IMETHODIMP nsPrintSettings::GetPaperData(int16_t *aPaperData)
 {
   NS_ENSURE_ARG_POINTER(aPaperData);
   *aPaperData = mPaperData;
   return NS_OK;
 }
-NS_IMETHODIMP nsPrintSettings::SetPaperData(PRInt16 aPaperData)
+NS_IMETHODIMP nsPrintSettings::SetPaperData(int16_t aPaperData)
 {
   mPaperData = aPaperData;
   return NS_OK;
@@ -1093,7 +1061,7 @@ nsPrintSettings::GetEffectivePageSize(double *aWidth, double *aHeight)
 }
 
 NS_IMETHODIMP
-nsPrintSettings::GetPageRanges(nsTArray<PRInt32> &aPages)
+nsPrintSettings::GetPageRanges(nsTArray<int32_t> &aPages)
 {
   aPages.Clear();
   return NS_OK;
@@ -1174,7 +1142,7 @@ nsPrintSettings& nsPrintSettings::operator=(const nsPrintSettings& rhs)
   mOutputFormat        = rhs.mOutputFormat;
   mPrintPageDelay      = rhs.mPrintPageDelay;
 
-  for (PRInt32 i=0;i<NUM_HEAD_FOOT;i++) {
+  for (int32_t i=0;i<NUM_HEAD_FOOT;i++) {
     mHeaderStrs[i] = rhs.mHeaderStrs[i];
     mFooterStrs[i] = rhs.mFooterStrs[i];
   }
