@@ -1,41 +1,8 @@
 /* -*- Mode: C++; tab-width: 50; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* vim:set ts=2 sw=2 sts=2 ci et: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla.org code.
- *
- * The Initial Developer of the Original Code is the Mozilla Foundation.
- *
- * Portions created by the Initial Developer are Copyright (C) 2011
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Justin Lebar <justin.lebar@gmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "mozilla/Util.h"
 
@@ -45,6 +12,7 @@
 #include "nsCOMPtr.h"
 #include "nsTHashtable.h"
 #include "nsHashKeys.h"
+#include "mozilla/Attributes.h"
 #include <stdio.h>
 
 namespace mozilla {
@@ -82,7 +50,7 @@ namespace {
 
 bool EndsWithLiteral(const nsCString &aHaystack, const char *aNeedle)
 {
-  PRInt32 idx = aHaystack.RFind(aNeedle);
+  int32_t idx = aHaystack.RFind(aNeedle);
   if (idx == -1) {
     return false;
   }
@@ -92,7 +60,7 @@ bool EndsWithLiteral(const nsCString &aHaystack, const char *aNeedle)
 
 void GetDirname(const nsCString &aPath, nsACString &aOut)
 {
-  PRInt32 idx = aPath.RFind("/");
+  int32_t idx = aPath.RFind("/");
   if (idx == -1) {
     aOut.Truncate();
   }
@@ -104,7 +72,7 @@ void GetDirname(const nsCString &aPath, nsACString &aOut)
 void GetBasename(const nsCString &aPath, nsACString &aOut)
 {
   nsCString out;
-  PRInt32 idx = aPath.RFind("/");
+  int32_t idx = aPath.RFind("/");
   if (idx == -1) {
     out.Assign(aPath);
   }
@@ -124,26 +92,25 @@ void GetBasename(const nsCString &aPath, nsACString &aOut)
 }
 
 // MapsReporter::CollectReports uses this stuct to keep track of whether it's
-// seen a mapping under 'smaps/resident', 'smaps/pss', 'smaps/vsize', and
-// 'smaps/swap'.
+// seen a mapping under 'rss', 'pss', 'size', and 'swap'.
 struct CategoriesSeen {
   CategoriesSeen() :
-    mSeenResident(false),
+    mSeenRss(false),
     mSeenPss(false),
-    mSeenVsize(false),
+    mSeenSize(false),
     mSeenSwap(false)
   {
   }
 
-  bool mSeenResident;
+  bool mSeenRss;
   bool mSeenPss;
-  bool mSeenVsize;
+  bool mSeenSize;
   bool mSeenSwap;
 };
 
 } // anonymous namespace
 
-class MapsReporter : public nsIMemoryMultiReporter
+class MapsReporter MOZ_FINAL : public nsIMemoryMultiReporter
 {
 public:
   MapsReporter();
@@ -161,7 +128,7 @@ public:
                  nsISupports *aClosure);
 
   NS_IMETHOD
-  GetExplicitNonHeap(PRInt64 *aAmount) {
+  GetExplicitNonHeap(int64_t *aAmount) {
     // This reporter doesn't do any "explicit" measurements.
     *aAmount = 0;
     return NS_OK;
@@ -202,9 +169,9 @@ NS_IMPL_THREADSAFE_ISUPPORTS1(MapsReporter, nsIMemoryMultiReporter)
 MapsReporter::MapsReporter()
   : mSearchedForLibxul(false)
 {
-  const PRUint32 len = ArrayLength(mozillaLibraries);
+  const uint32_t len = ArrayLength(mozillaLibraries);
   mMozillaLibraries.Init(len);
-  for (PRUint32 i = 0; i < len; i++) {
+  for (uint32_t i = 0; i < len; i++) {
     nsCAutoString str;
     str.Assign(mozillaLibraries[i]);
     mMozillaLibraries.PutEntry(str);
@@ -229,17 +196,19 @@ MapsReporter::CollectReports(nsIMemoryMultiReporterCallback *aCb,
 
   fclose(f);
 
-  // For sure we should have created some node under 'smaps/resident' and
-  // 'smaps/vsize'; otherwise we're probably not reading smaps correctly.  If we
-  // didn't create a node under 'smaps/swap', create one here so about:memory
-  // knows to create an empty 'smaps/swap' tree.  See also bug 682735.
+  // For sure we should have created some node under 'rss' and
+  // 'size'; otherwise we're probably not reading smaps correctly.  If we
+  // didn't create a node under 'swap', create one here so about:memory
+  // knows to create an empty 'swap' tree;  it needs a 'total' child because
+  // about:memory expects at least one report whose path begins with 'swap/'.
 
-  NS_ASSERTION(categoriesSeen.mSeenVsize, "Didn't create a vsize node?");
-  NS_ASSERTION(categoriesSeen.mSeenVsize, "Didn't create a resident node?");
+  NS_ASSERTION(categoriesSeen.mSeenSize, "Didn't create a size node?");
+  NS_ASSERTION(categoriesSeen.mSeenRss, "Didn't create a rss node?");
+  NS_ASSERTION(categoriesSeen.mSeenPss, "Didn't create a pss node?");
   if (!categoriesSeen.mSeenSwap) {
     nsresult rv;
     rv = aCb->Callback(NS_LITERAL_CSTRING(""),
-                       NS_LITERAL_CSTRING("smaps/swap/total"),
+                       NS_LITERAL_CSTRING("swap/total"),
                        nsIMemoryReporter::KIND_NONHEAP,
                        nsIMemoryReporter::UNITS_BYTES,
                        0,
@@ -301,10 +270,10 @@ MapsReporter::ParseMapping(
 {
   // We need to use native types in order to get good warnings from fscanf, so
   // let's make sure that the native types have the sizes we expect.
-  MOZ_STATIC_ASSERT(sizeof(long long) == sizeof(PRInt64),
-                    "size of (long long) is expected to match (PRInt64)");
-  MOZ_STATIC_ASSERT(sizeof(int) == sizeof(PRInt32),
-                    "size of (int) is expected to match (PRInt32)");
+  MOZ_STATIC_ASSERT(sizeof(long long) == sizeof(int64_t),
+                    "size of (long long) is expected to match (int64_t)");
+  MOZ_STATIC_ASSERT(sizeof(int) == sizeof(int32_t),
+                    "size of (int) is expected to match (int32_t)");
 
   // Don't bail if FindLibxul fails.  We can still gather meaningful stats
   // here.
@@ -321,8 +290,11 @@ MapsReporter::ParseMapping(
   unsigned long long addrStart, addrEnd;
   char perms[5];
   unsigned long long offset;
-  char devMajor[3];
-  char devMinor[3];
+  // The 2.6 and 3.0 kernels allocate 12 bits for the major device number and
+  // 20 bits for the minor device number.  Future kernels might allocate more.
+  // 64 bits ought to be enough for anybody.
+  char devMajor[17];
+  char devMinor[17];
   unsigned int inode;
   char path[1025];
 
@@ -334,7 +306,9 @@ MapsReporter::ParseMapping(
   // with or without a path, but we don't want to look to a new line for the
   // path.  Thus we have %u%1024[^\n] at the end of the pattern.  This will
   // capture into the path some leading whitespace, which we'll later trim off.
-  int numRead = fscanf(aFile, "%llx-%llx %4s %llx %2s:%2s %u%1024[^\n]",
+  int numRead = fscanf(aFile,
+                       "%llx-%llx %4s %llx "
+                       "%16[0-9a-fA-F]:%16[0-9a-fA-F] %u%1024[^\n]",
                        &addrStart, &addrEnd, perms, &offset, devMajor,
                        devMinor, &inode, path);
 
@@ -484,8 +458,8 @@ MapsReporter::ParseMapBody(
   nsISupports *aClosure,
   CategoriesSeen *aCategoriesSeen)
 {
-  MOZ_STATIC_ASSERT(sizeof(long long) == sizeof(PRInt64),
-                    "size of (long long) is expected to match (PRInt64)");
+  MOZ_STATIC_ASSERT(sizeof(long long) == sizeof(int64_t),
+                    "size of (long long) is expected to match (int64_t)");
 
   const int argCount = 2;
 
@@ -502,12 +476,12 @@ MapsReporter::ParseMapBody(
 
   const char* category;
   if (strcmp(desc, "Size") == 0) {
-    category = "vsize";
-    aCategoriesSeen->mSeenVsize = true;
+    category = "size";
+    aCategoriesSeen->mSeenSize = true;
   }
   else if (strcmp(desc, "Rss") == 0) {
-    category = "resident";
-    aCategoriesSeen->mSeenResident = true;
+    category = "rss";
+    aCategoriesSeen->mSeenRss = true;
   }
   else if (strcmp(desc, "Pss") == 0) {
     category = "pss";
@@ -523,7 +497,6 @@ MapsReporter::ParseMapBody(
   }
 
   nsCAutoString path;
-  path.Append("smaps/");
   path.Append(category);
   path.Append("/");
   path.Append(aName);
@@ -533,7 +506,7 @@ MapsReporter::ParseMapBody(
                      path,
                      nsIMemoryReporter::KIND_NONHEAP,
                      nsIMemoryReporter::UNITS_BYTES,
-                     PRInt64(size) * 1024, // convert from kB to bytes
+                     int64_t(size) * 1024, // convert from kB to bytes
                      aDescription, aClosure);
   NS_ENSURE_SUCCESS(rv, rv);
 
