@@ -1,41 +1,7 @@
 /* -*- Mode: C++; tab-width: 50; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * mozilla.org
- * Portions created by the Initial Developer are Copyright (C) 2008
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Vladimir Vukicevic <vladimir@pobox.com> (original author)
- *   Nicholas Nethercote <nnethercote@mozilla.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsAtomTable.h"
 #include "nsAutoPtr.h"
@@ -45,22 +11,10 @@
 #include "nsArrayEnumerator.h"
 #include "nsISimpleEnumerator.h"
 #include "mozilla/Telemetry.h"
+#include "mozilla/Attributes.h"
 
 using namespace mozilla;
 
-static PRInt64 GetExplicit()
-{
-    nsCOMPtr<nsIMemoryReporterManager> mgr = do_GetService("@mozilla.org/memory-reporter-manager;1");
-    if (mgr == nsnull)
-        return (PRInt64)-1;
-
-    PRInt64 n;
-    nsresult rv = mgr->GetExplicit(&n);
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    return n;
-}
-
 #if defined(MOZ_MEMORY)
 #  define HAVE_JEMALLOC_STATS 1
 #  include "jemalloc.h"
@@ -71,57 +25,62 @@ static PRInt64 GetExplicit()
 #include <sys/time.h>
 #include <sys/resource.h>
 
-static PRInt64 GetHardPageFaults()
+#define HAVE_PAGE_FAULT_REPORTERS 1
+static nsresult GetHardPageFaults(int64_t *n)
 {
-  struct rusage usage;
-  int err = getrusage(RUSAGE_SELF, &usage);
-  if (err != 0) {
-    return PRInt64(-1);
-  }
-
-  return usage.ru_majflt;
+    struct rusage usage;
+    int err = getrusage(RUSAGE_SELF, &usage);
+    if (err != 0) {
+        return NS_ERROR_FAILURE;
+    }
+    *n = usage.ru_majflt;
+    return NS_OK;
 }
 
-static PRInt64 GetSoftPageFaults()
+static nsresult GetSoftPageFaults(int64_t *n)
 {
-  struct rusage usage;
-  int err = getrusage(RUSAGE_SELF, &usage);
-  if (err != 0) {
-    return PRInt64(-1);
-  }
-
-  return usage.ru_minflt;
+    struct rusage usage;
+    int err = getrusage(RUSAGE_SELF, &usage);
+    if (err != 0) {
+        return NS_ERROR_FAILURE;
+    }
+    *n = usage.ru_minflt;
+    return NS_OK;
 }
 
-#endif
+#endif  // HAVE_PAGE_FAULT_REPORTERS
 
 #if defined(XP_LINUX)
 
 #include <unistd.h>
-static PRInt64 GetProcSelfStatmField(int n)
+static nsresult GetProcSelfStatmField(int field, int64_t *n)
 {
     // There are more than two fields, but we're only interested in the first
     // two.
     static const int MAX_FIELD = 2;
     size_t fields[MAX_FIELD];
-    NS_ASSERTION(n < MAX_FIELD, "bad field number");
+    MOZ_ASSERT(field < MAX_FIELD, "bad field number");
     FILE *f = fopen("/proc/self/statm", "r");
     if (f) {
         int nread = fscanf(f, "%zu %zu", &fields[0], &fields[1]);
         fclose(f);
-        return (PRInt64) ((nread == MAX_FIELD) ? fields[n]*getpagesize() : -1);
+        if (nread == MAX_FIELD) {
+            *n = fields[field] * getpagesize();
+            return NS_OK;
+        } 
     }
-    return (PRInt64) -1;
+    return NS_ERROR_FAILURE;
 }
 
-static PRInt64 GetVsize()
+#define HAVE_VSIZE_AND_RESIDENT_REPORTERS 1
+static nsresult GetVsize(int64_t *n)
 {
-    return GetProcSelfStatmField(0);
+    return GetProcSelfStatmField(0, n);
 }
 
-static PRInt64 GetResident()
+static nsresult GetResident(int64_t *n)
 {
-    return GetProcSelfStatmField(1);
+    return GetProcSelfStatmField(1, n);
 }
 
 #elif defined(SOLARIS)
@@ -130,10 +89,10 @@ static PRInt64 GetResident()
 #include <fcntl.h>
 #include <unistd.h>
 
-static void XMappingIter(PRInt64& Vsize, PRInt64& Resident)
+static void XMappingIter(int64_t& vsize, int64_t& resident)
 {
-    Vsize = -1;
-    Resident = -1;
+    vsize = -1;
+    resident = -1;
     int mapfd = open("/proc/self/xmap", O_RDONLY);
     struct stat st;
     prxmap_t *prmapp = NULL;
@@ -155,11 +114,11 @@ static void XMappingIter(PRInt64& Vsize, PRInt64& Resident)
                     break;
                 }
                 if (nmap >= n / sizeof (prxmap_t)) {
-                    Vsize = 0;
-                    Resident = 0;
+                    vsize = 0;
+                    resident = 0;
                     for (int i = 0; i < n / sizeof (prxmap_t); i++) {
-                        Vsize += prmapp[i].pr_size;
-                        Resident += prmapp[i].pr_rss * prmapp[i].pr_pagesize;
+                        vsize += prmapp[i].pr_size;
+                        resident += prmapp[i].pr_rss * prmapp[i].pr_pagesize;
                     }
                     break;
                 }
@@ -171,18 +130,27 @@ static void XMappingIter(PRInt64& Vsize, PRInt64& Resident)
     }
 }
 
-static PRInt64 GetVsize()
+#define HAVE_VSIZE_AND_RESIDENT_REPORTERS 1
+static nsresult GetVsize(int64_t *n)
 {
-    PRInt64 Vsize, Resident;
-    XMappingIter(Vsize, Resident);
-    return Vsize;
+    int64_t vsize, resident;
+    XMappingIter(vsize, resident);
+    if (vsize == -1) {
+        return NS_ERROR_FAILURE;
+    }
+    *n = vsize;
+    return NS_OK;
 }
 
-static PRInt64 GetResident()
+static nsresult GetResident(int64_t *n)
 {
-    PRInt64 Vsize, Resident;
-    XMappingIter(Vsize, Resident);
-    return Resident;
+    int64_t vsize, resident;
+    XMappingIter(vsize, resident);
+    if (resident == -1) {
+        return NS_ERROR_FAILURE;
+    }
+    *n = resident;
+    return NS_OK;
 }
 
 #elif defined(XP_MACOSX)
@@ -201,13 +169,18 @@ static bool GetTaskBasicInfo(struct task_basic_info *ti)
 // The VSIZE figure on Mac includes huge amounts of shared memory and is always
 // absurdly high, eg. 2GB+ even at start-up.  But both 'top' and 'ps' report
 // it, so we might as well too.
-static PRInt64 GetVsize()
+#define HAVE_VSIZE_AND_RESIDENT_REPORTERS 1
+static nsresult GetVsize(int64_t *n)
 {
     task_basic_info ti;
-    return (PRInt64) (GetTaskBasicInfo(&ti) ? ti.virtual_size : -1);
+    if (!GetTaskBasicInfo(&ti))
+        return NS_ERROR_FAILURE;
+
+    *n = ti.virtual_size;
+    return NS_OK;
 }
 
-static PRInt64 GetResident()
+static nsresult GetResident(int64_t *n)
 {
 #ifdef HAVE_JEMALLOC_STATS
     // If we're using jemalloc on Mac, we need to instruct jemalloc to purge
@@ -224,7 +197,11 @@ static PRInt64 GetResident()
 #endif
 
     task_basic_info ti;
-    return (PRInt64) (GetTaskBasicInfo(&ti) ? ti.resident_size : -1);
+    if (!GetTaskBasicInfo(&ti))
+        return NS_ERROR_FAILURE;
+
+    *n = ti.resident_size;
+    return NS_OK;
 }
 
 #elif defined(XP_WIN)
@@ -232,31 +209,50 @@ static PRInt64 GetResident()
 #include <windows.h>
 #include <psapi.h>
 
-static PRInt64 GetVsize()
+#define HAVE_VSIZE_AND_RESIDENT_REPORTERS 1
+static nsresult GetVsize(int64_t *n)
 {
-  MEMORYSTATUSEX s;
-  s.dwLength = sizeof(s);
+    MEMORYSTATUSEX s;
+    s.dwLength = sizeof(s);
 
-  bool success = GlobalMemoryStatusEx(&s);
-  if (!success)
-    return -1;
+    if (!GlobalMemoryStatusEx(&s)) {
+        return NS_ERROR_FAILURE;
+    }
 
-  return s.ullTotalVirtual - s.ullAvailVirtual;
+    *n = s.ullTotalVirtual - s.ullAvailVirtual;
+    return NS_OK;
 }
 
-static PRInt64 GetPrivate()
+static nsresult GetResident(int64_t *n)
+{
+    PROCESS_MEMORY_COUNTERS pmc;
+    pmc.cb = sizeof(PROCESS_MEMORY_COUNTERS);
+
+    if (!GetProcessMemoryInfo(GetCurrentProcess(), &pmc, sizeof(pmc))) {
+        return NS_ERROR_FAILURE;
+    }
+
+    *n = pmc.WorkingSetSize;
+    return NS_OK;
+}
+
+#define HAVE_PRIVATE_REPORTER
+static nsresult GetPrivate(int64_t *n)
 {
     PROCESS_MEMORY_COUNTERS_EX pmcex;
     pmcex.cb = sizeof(PROCESS_MEMORY_COUNTERS_EX);
 
     if (!GetProcessMemoryInfo(GetCurrentProcess(),
                               (PPROCESS_MEMORY_COUNTERS) &pmcex, sizeof(pmcex)))
-    return (PRInt64) -1;
+    {
+        return NS_ERROR_FAILURE;
+    }
 
-    return pmcex.PrivateUsage;
+    *n = pmcex.PrivateUsage;
+    return NS_OK;
 }
 
-NS_MEMORY_REPORTER_IMPLEMENT(Private,
+NS_FALLIBLE_MEMORY_REPORTER_IMPLEMENT(Private,
     "private",
     KIND_OTHER,
     UNITS_BYTES,
@@ -265,28 +261,10 @@ NS_MEMORY_REPORTER_IMPLEMENT(Private,
     "is committed and marked MEM_PRIVATE, data that is not mapped, and "
     "executable pages that have been written to.")
 
-static PRInt64 GetResident()
-{
-  PROCESS_MEMORY_COUNTERS pmc;
-  pmc.cb = sizeof(PROCESS_MEMORY_COUNTERS);
+#endif  // XP_<PLATFORM>
 
-  if (!GetProcessMemoryInfo(GetCurrentProcess(), &pmc, sizeof(pmc)))
-      return (PRInt64) -1;
-
-  return pmc.WorkingSetSize;
-}
-
-#else
-
-static PRInt64 GetResident()
-{
-    return (PRInt64) -1;
-}
-
-#endif
-
-#if defined(XP_LINUX) || defined(XP_MACOSX) || defined(XP_WIN) || defined(SOLARIS)
-NS_MEMORY_REPORTER_IMPLEMENT(Vsize,
+#ifdef HAVE_VSIZE_AND_RESIDENT_REPORTERS
+NS_FALLIBLE_MEMORY_REPORTER_IMPLEMENT(Vsize,
     "vsize",
     KIND_OTHER,
     UNITS_BYTES,
@@ -298,10 +276,22 @@ NS_MEMORY_REPORTER_IMPLEMENT(Vsize,
     "limited use on Mac, where processes share huge amounts of memory with one "
     "another.  But even on other operating systems, 'resident' is a much better "
     "measure of the memory resources used by the process.")
-#endif
 
-#if defined(XP_LINUX) || defined(XP_MACOSX) || defined(SOLARIS)
-NS_MEMORY_REPORTER_IMPLEMENT(PageFaultsSoft,
+NS_FALLIBLE_MEMORY_REPORTER_IMPLEMENT(Resident,
+    "resident",
+    KIND_OTHER,
+    UNITS_BYTES,
+    GetResident,
+    "Memory mapped by the process that is present in physical memory, "
+    "also known as the resident set size (RSS).  This is the best single "
+    "figure to use when considering the memory resources used by the process, "
+    "but it depends both on other processes being run and details of the OS "
+    "kernel and so is best used for comparing the memory usage of a single "
+    "process at different points in time.")
+#endif  // HAVE_VSIZE_AND_RESIDENT_REPORTERS
+
+#ifdef HAVE_PAGE_FAULT_REPORTERS
+NS_FALLIBLE_MEMORY_REPORTER_IMPLEMENT(PageFaultsSoft,
     "page-faults-soft",
     KIND_OTHER,
     UNITS_COUNT_CUMULATIVE,
@@ -316,7 +306,7 @@ NS_MEMORY_REPORTER_IMPLEMENT(PageFaultsSoft,
     "and because the OS services a soft page fault without accessing the disk, "
     "they impact performance much less than hard page faults.")
 
-NS_MEMORY_REPORTER_IMPLEMENT(PageFaultsHard,
+NS_FALLIBLE_MEMORY_REPORTER_IMPLEMENT(PageFaultsHard,
     "page-faults-hard",
     KIND_OTHER,
     UNITS_COUNT_CUMULATIVE,
@@ -331,28 +321,7 @@ NS_MEMORY_REPORTER_IMPLEMENT(PageFaultsHard,
     "accessing the disk is up to a million times slower than accessing RAM, "
     "the program may run very slowly when it is experiencing more than 100 or "
     "so hard page faults a second.")
-#endif
-
-NS_MEMORY_REPORTER_IMPLEMENT(Explicit,
-    "explicit",
-    KIND_OTHER,
-    UNITS_BYTES,
-    GetExplicit,
-    "This is the same measurement as the root of the 'explicit' tree.  "
-    "However, it is measured at a different time and so gives slightly "
-    "different results.")
-
-NS_MEMORY_REPORTER_IMPLEMENT(Resident,
-    "resident",
-    KIND_OTHER,
-    UNITS_BYTES,
-    GetResident,
-    "Memory mapped by the process that is present in physical memory, "
-    "also known as the resident set size (RSS).  This is the best single "
-    "figure to use when considering the memory resources used by the process, "
-    "but it depends both on other processes being run and details of the OS "
-    "kernel and so is best used for comparing the memory usage of a single "
-    "process at different points in time.")
+#endif  // HAVE_PAGE_FAULT_REPORTERS
 
 /**
  ** memory reporter implementation for jemalloc and OSX malloc,
@@ -362,39 +331,49 @@ NS_MEMORY_REPORTER_IMPLEMENT(Resident,
 
 #if HAVE_JEMALLOC_STATS
 
-static PRInt64 GetHeapUnallocated()
+#define HAVE_HEAP_ALLOCATED_REPORTERS 1
+
+static int64_t GetHeapUnused()
 {
     jemalloc_stats_t stats;
     jemalloc_stats(&stats);
-    return (PRInt64) stats.mapped - stats.allocated;
+    return (int64_t) (stats.mapped - stats.allocated);
 }
 
-static PRInt64 GetHeapAllocated()
+static int64_t GetHeapAllocated()
 {
     jemalloc_stats_t stats;
     jemalloc_stats(&stats);
-    return (PRInt64) stats.allocated;
+    return (int64_t) stats.allocated;
 }
 
-static PRInt64 GetHeapCommitted()
+static int64_t GetHeapCommitted()
 {
     jemalloc_stats_t stats;
     jemalloc_stats(&stats);
-    return (PRInt64) stats.committed;
+    return (int64_t) stats.committed;
 }
 
-static PRInt64 GetHeapCommittedFragmentation()
+static int64_t GetHeapCommittedUnused()
 {
     jemalloc_stats_t stats;
     jemalloc_stats(&stats);
-    return (PRInt64) 10000 * (1 - stats.allocated / (double)stats.committed);
+    return stats.committed - stats.allocated;
 }
 
-static PRInt64 GetHeapDirty()
+static int64_t GetHeapCommittedUnusedRatio()
 {
     jemalloc_stats_t stats;
     jemalloc_stats(&stats);
-    return (PRInt64) stats.dirty;
+    return (int64_t) 10000 * (stats.committed - stats.allocated) /
+                              ((double)stats.allocated);
+}
+
+static int64_t GetHeapDirty()
+{
+    jemalloc_stats_t stats;
+    jemalloc_stats(&stats);
+    return (int64_t) stats.dirty;
 }
 
 NS_MEMORY_REPORTER_IMPLEMENT(HeapCommitted,
@@ -409,15 +388,24 @@ NS_MEMORY_REPORTER_IMPLEMENT(HeapCommitted,
     "memory and is unable to decommit it because a small part of that block is "
     "currently in use.")
 
-NS_MEMORY_REPORTER_IMPLEMENT(HeapCommittedFragmentation,
-    "heap-committed-fragmentation",
+NS_MEMORY_REPORTER_IMPLEMENT(HeapCommittedUnused,
+    "heap-committed-unused",
+    KIND_OTHER,
+    UNITS_BYTES,
+    GetHeapCommittedUnused,
+    "Committed bytes which do not correspond to an active allocation; i.e., "
+    "'heap-committed' - 'heap-allocated'.  Although the allocator will waste some "
+    "space under any circumstances, a large value here may indicate that the "
+    "heap is highly fragmented.")
+
+NS_MEMORY_REPORTER_IMPLEMENT(HeapCommittedUnusedRatio,
+    "heap-committed-unused-ratio",
     KIND_OTHER,
     UNITS_PERCENTAGE,
-    GetHeapCommittedFragmentation,
-    "Fraction of committed bytes which do not correspond to an active "
-    "allocation; i.e., 1 - (heap-allocated / heap-committed).  Although the "
-    "allocator will waste some space under any circumstances, a large value here "
-    "may indicate that the heap is highly fragmented.")
+    GetHeapCommittedUnusedRatio,
+    "Ratio of committed, unused bytes to allocated bytes; i.e., "
+    "'heap-committed-unused' / 'heap-allocated'.  This measures the overhead "
+    "of the heap allocator relative to amount of memory allocated.")
 
 NS_MEMORY_REPORTER_IMPLEMENT(HeapDirty,
     "heap-dirty",
@@ -432,42 +420,37 @@ NS_MEMORY_REPORTER_IMPLEMENT(HeapDirty,
 #elif defined(XP_MACOSX) && !defined(MOZ_MEMORY)
 #include <malloc/malloc.h>
 
-static PRInt64 GetHeapUnallocated()
+#define HAVE_HEAP_ALLOCATED_REPORTERS 1
+
+static int64_t GetHeapUnused()
 {
     struct mstats stats = mstats();
-    return (PRInt64) (stats.bytes_total - stats.bytes_used);
+    return stats.bytes_total - stats.bytes_used;
 }
 
-static PRInt64 GetHeapAllocated()
+static int64_t GetHeapAllocated()
 {
     struct mstats stats = mstats();
-    return (PRInt64) stats.bytes_used;
+    return stats.bytes_used;
 }
 
-static PRInt64 GetHeapZone0Committed()
+// malloc_zone_statistics() crashes when run under DMD because Valgrind doesn't
+// intercept it.  This measurement isn't important for DMD, so don't even try
+// to get it.
+#ifndef MOZ_DMD
+#define HAVE_HEAP_ZONE0_REPORTERS 1
+static int64_t GetHeapZone0Committed()
 {
-#ifdef MOZ_DMD
-    // malloc_zone_statistics() crashes when run under DMD because Valgrind
-    // doesn't intercept it.  This measurement isn't important for DMD, so
-    // don't even try.
-    return (PRInt64) -1;
-#else
     malloc_statistics_t stats;
     malloc_zone_statistics(malloc_default_zone(), &stats);
     return stats.size_in_use;
-#endif
 }
 
-static PRInt64 GetHeapZone0Used()
+static int64_t GetHeapZone0Used()
 {
-#ifdef MOZ_DMD
-    // See comment in GetHeapZone0Committed above.
-    return (PRInt64) -1;
-#else
     malloc_statistics_t stats;
     malloc_zone_statistics(malloc_default_zone(), &stats);
     return stats.size_allocated;
-#endif
 }
 
 NS_MEMORY_REPORTER_IMPLEMENT(HeapZone0Committed,
@@ -485,26 +468,16 @@ NS_MEMORY_REPORTER_IMPLEMENT(HeapZone0Used,
     GetHeapZone0Used,
     "Memory mapped by the heap allocator in the default zone that is "
     "allocated to the application.")
-
-#else
-
-static PRInt64 GetHeapAllocated()
-{
-    return (PRInt64) -1;
-}
-
-static PRInt64 GetHeapUnallocated()
-{
-  return (PRInt64) -1;
-}
+#endif  // MOZ_DMD
 
 #endif
 
-NS_MEMORY_REPORTER_IMPLEMENT(HeapUnallocated,
-    "heap-unallocated",
+#ifdef HAVE_HEAP_ALLOCATED_REPORTERS
+NS_MEMORY_REPORTER_IMPLEMENT(HeapUnused,
+    "heap-unused",
     KIND_OTHER,
     UNITS_BYTES,
-    GetHeapUnallocated,
+    GetHeapUnused,
     "Memory mapped by the heap allocator that is not part of an active "
     "allocation. Much of this memory may be uncommitted -- that is, it does not "
     "take up space in physical memory or in the swap file.")
@@ -519,10 +492,32 @@ NS_MEMORY_REPORTER_IMPLEMENT(HeapAllocated,
     "application because the allocator regularly rounds up request sizes. (The "
     "exact amount requested is not recorded.)")
 
+// The computation of "explicit" fails if "heap-allocated" isn't available,
+// which is why this is depends on HAVE_HEAP_ALLOCATED_AND_EXPLICIT_REPORTERS.
+#define HAVE_EXPLICIT_REPORTER 1
+static nsresult GetExplicit(int64_t *n)
+{
+    nsCOMPtr<nsIMemoryReporterManager> mgr = do_GetService("@mozilla.org/memory-reporter-manager;1");
+    if (mgr == nullptr)
+        return NS_ERROR_FAILURE;
+
+    return mgr->GetExplicit(n);
+}
+
+NS_FALLIBLE_MEMORY_REPORTER_IMPLEMENT(Explicit,
+    "explicit",
+    KIND_OTHER,
+    UNITS_BYTES,
+    GetExplicit,
+    "This is the same measurement as the root of the 'explicit' tree.  "
+    "However, it is measured at a different time and so gives slightly "
+    "different results.")
+#endif  // HAVE_HEAP_ALLOCATED_REPORTERS
+
 NS_MEMORY_REPORTER_MALLOC_SIZEOF_FUN(AtomTableMallocSizeOf, "atom-table")
 
-static PRInt64 GetAtomTableSize() {
-  return NS_SizeOfAtomTableIncludingThis(AtomTableMallocSizeOf);
+static int64_t GetAtomTableSize() {
+  return NS_SizeOfAtomTablesIncludingThis(AtomTableMallocSizeOf);
 }
 
 // Why is this here?  At first glance, you'd think it could be defined and
@@ -531,11 +526,11 @@ static PRInt64 GetAtomTableSize() {
 // and that happens before XPCOM components are initialized, which means the
 // NS_RegisterMemoryReporter call fails.  So instead we do it here.
 NS_MEMORY_REPORTER_IMPLEMENT(AtomTable,
-    "explicit/atom-table",
+    "explicit/atom-tables",
     KIND_HEAP,
     UNITS_BYTES,
     GetAtomTableSize,
-    "Memory used by the atoms table.")
+    "Memory used by the dynamic and static atoms tables.")
 
 /**
  ** nsMemoryReporterManager implementation
@@ -553,29 +548,35 @@ nsMemoryReporterManager::Init()
 
 #define REGISTER(_x)  RegisterReporter(new NS_MEMORY_REPORTER_NAME(_x))
 
+#ifdef HAVE_HEAP_ALLOCATED_REPORTERS
     REGISTER(HeapAllocated);
-    REGISTER(HeapUnallocated);
-    REGISTER(Explicit);
-    REGISTER(Resident);
-
-#if defined(XP_LINUX) || defined(XP_MACOSX) || defined(XP_WIN) || defined(SOLARIS)
-    REGISTER(Vsize);
+    REGISTER(HeapUnused);
 #endif
 
-#if defined(XP_LINUX) || defined(XP_MACOSX) || defined(SOLARIS)
+#ifdef HAVE_EXPLICIT_REPORTER
+    REGISTER(Explicit);
+#endif
+
+#ifdef HAVE_VSIZE_AND_RESIDENT_REPORTERS
+    REGISTER(Vsize);
+    REGISTER(Resident);
+#endif
+
+#ifdef HAVE_PAGE_FAULT_REPORTERS
     REGISTER(PageFaultsSoft);
     REGISTER(PageFaultsHard);
 #endif
 
-#if defined(XP_WIN)
+#ifdef HAVE_PRIVATE_REPORTER
     REGISTER(Private);
 #endif
 
 #if defined(HAVE_JEMALLOC_STATS)
     REGISTER(HeapCommitted);
-    REGISTER(HeapCommittedFragmentation);
+    REGISTER(HeapCommittedUnused);
+    REGISTER(HeapCommittedUnusedRatio);
     REGISTER(HeapDirty);
-#elif defined(XP_MACOSX) && !defined(MOZ_MEMORY)
+#elif defined(HAVE_HEAP_ZONE0_REPORTERS)
     REGISTER(HeapZone0Committed);
     REGISTER(HeapZone0Used);
 #endif
@@ -655,14 +656,18 @@ nsMemoryReporterManager::UnregisterMultiReporter(nsIMemoryMultiReporter *reporte
 }
 
 NS_IMETHODIMP
-nsMemoryReporterManager::GetResident(PRInt64 *aResident)
+nsMemoryReporterManager::GetResident(int64_t *aResident)
 {
-    *aResident = ::GetResident();
-    return NS_OK;
+#if HAVE_VSIZE_AND_RESIDENT_REPORTERS
+    return ::GetResident(aResident);
+#else
+    *aResident = 0;
+    return NS_ERROR_NOT_AVAILABLE;
+#endif
 }
 
 struct MemoryReport {
-    MemoryReport(const nsACString &path, PRInt64 amount) 
+    MemoryReport(const nsACString &path, int64_t amount) 
     : path(path), amount(amount)
     {
         MOZ_COUNT_CTOR(MemoryReport);
@@ -677,33 +682,33 @@ struct MemoryReport {
         MOZ_COUNT_DTOR(MemoryReport);
     }
     const nsCString path;
-    PRInt64 amount;
+    int64_t amount;
 };
 
 #ifdef DEBUG
-// This is just a wrapper for PRInt64 that implements nsISupports, so it can be
+// This is just a wrapper for int64_t that implements nsISupports, so it can be
 // passed to nsIMemoryMultiReporter::CollectReports.
-class PRInt64Wrapper : public nsISupports {
+class PRInt64Wrapper MOZ_FINAL : public nsISupports {
 public:
     NS_DECL_ISUPPORTS
     PRInt64Wrapper() : mValue(0) { }
-    PRInt64 mValue;
+    int64_t mValue;
 };
 NS_IMPL_ISUPPORTS0(PRInt64Wrapper)
 
-class ExplicitNonHeapCountingCallback : public nsIMemoryMultiReporterCallback
+class ExplicitNonHeapCountingCallback MOZ_FINAL : public nsIMemoryMultiReporterCallback
 {
 public:
     NS_DECL_ISUPPORTS
 
     NS_IMETHOD Callback(const nsACString &aProcess, const nsACString &aPath,
-                        PRInt32 aKind, PRInt32 aUnits, PRInt64 aAmount,
+                        int32_t aKind, int32_t aUnits, int64_t aAmount,
                         const nsACString &aDescription,
                         nsISupports *aWrappedExplicitNonHeap)
     {
         if (aKind == nsIMemoryReporter::KIND_NONHEAP &&
             PromiseFlatCString(aPath).Find("explicit") == 0 &&
-            aAmount != PRInt64(-1))
+            aAmount != int64_t(-1))
         {
             PRInt64Wrapper *wrappedPRInt64 =
                 static_cast<PRInt64Wrapper *>(aWrappedExplicitNonHeap);
@@ -719,25 +724,27 @@ NS_IMPL_ISUPPORTS1(
 #endif
 
 NS_IMETHODIMP
-nsMemoryReporterManager::GetExplicit(PRInt64 *aExplicit)
+nsMemoryReporterManager::GetExplicit(int64_t *aExplicit)
 {
     NS_ENSURE_ARG_POINTER(aExplicit);
     *aExplicit = 0;
-
+#ifndef HAVE_EXPLICIT_REPORTER
+    return NS_ERROR_NOT_AVAILABLE;
+#else
     nsresult rv;
     bool more;
 
     // Get "heap-allocated" and all the KIND_NONHEAP measurements from normal
     // (i.e. non-multi) "explicit" reporters.
-    PRInt64 heapAllocated = PRInt64(-1);
-    PRInt64 explicitNonHeapNormalSize = 0;
+    int64_t heapAllocated = int64_t(-1);
+    int64_t explicitNonHeapNormalSize = 0;
     nsCOMPtr<nsISimpleEnumerator> e;
     EnumerateReporters(getter_AddRefs(e));
     while (NS_SUCCEEDED(e->HasMoreElements(&more)) && more) {
         nsCOMPtr<nsIMemoryReporter> r;
         e->GetNext(getter_AddRefs(r));
 
-        PRInt32 kind;
+        int32_t kind;
         rv = r->GetKind(&kind);
         NS_ENSURE_SUCCESS(rv, rv);
 
@@ -750,25 +757,18 @@ nsMemoryReporterManager::GetExplicit(PRInt64 *aExplicit)
         if (kind == nsIMemoryReporter::KIND_NONHEAP &&
             path.Find("explicit") == 0)
         {
-            PRInt64 amount;
-            rv = r->GetAmount(&amount);
-            NS_ENSURE_SUCCESS(rv, rv);
-
             // Just skip any NONHEAP reporters that fail, because
             // "heap-allocated" is the most important one.
-            if (amount != PRInt64(-1)) {
+            int64_t amount;
+            rv = r->GetAmount(&amount);
+            if (NS_SUCCEEDED(rv)) {
                 explicitNonHeapNormalSize += amount;
             }
         } else if (path.Equals("heap-allocated")) {
+            // If we don't have "heap-allocated", give up, because the result
+            // would be horribly inaccurate.
             rv = r->GetAmount(&heapAllocated);
             NS_ENSURE_SUCCESS(rv, rv);
-
-            // If we don't have "heap-allocated", give up, because the result would be
-            // horribly inaccurate.
-            if (heapAllocated == PRInt64(-1)) {
-                *aExplicit = PRInt64(-1);
-                return NS_OK;
-            }
         }
     }
 
@@ -782,13 +782,13 @@ nsMemoryReporterManager::GetExplicit(PRInt64 *aExplicit)
     // guarantees the two measurement paths are equivalent.  This is wise
     // because it's easy for memory reporters to have bugs.)
 
-    PRInt64 explicitNonHeapMultiSize = 0;
+    int64_t explicitNonHeapMultiSize = 0;
     nsCOMPtr<nsISimpleEnumerator> e2;
     EnumerateMultiReporters(getter_AddRefs(e2));
     while (NS_SUCCEEDED(e2->HasMoreElements(&more)) && more) {
       nsCOMPtr<nsIMemoryMultiReporter> r;
       e2->GetNext(getter_AddRefs(r));
-      PRInt64 n;
+      int64_t n;
       rv = r->GetExplicitNonHeap(&n);
       NS_ENSURE_SUCCESS(rv, rv);
       explicitNonHeapMultiSize += n;
@@ -806,7 +806,7 @@ nsMemoryReporterManager::GetExplicit(PRInt64 *aExplicit)
       e3->GetNext(getter_AddRefs(r));
       r->CollectReports(cb, wrappedExplicitNonHeapMultiSize2);
     }
-    PRInt64 explicitNonHeapMultiSize2 = wrappedExplicitNonHeapMultiSize2->mValue;
+    int64_t explicitNonHeapMultiSize2 = wrappedExplicitNonHeapMultiSize2->mValue;
 
     // Check the two measurements give the same result.  This was an
     // NS_ASSERTION but they occasionally don't match due to races (bug
@@ -819,10 +819,11 @@ nsMemoryReporterManager::GetExplicit(PRInt64 *aExplicit)
         NS_WARNING(msg);
         PR_smprintf_free(msg);
     }
-#endif
+#endif  // DEBUG
 
     *aExplicit = heapAllocated + explicitNonHeapNormalSize + explicitNonHeapMultiSize;
     return NS_OK;
+#endif // HAVE_HEAP_ALLOCATED_AND_EXPLICIT_REPORTERS
 }
 
 NS_IMETHODIMP
@@ -842,9 +843,9 @@ NS_IMPL_ISUPPORTS1(nsMemoryReporter, nsIMemoryReporter)
 
 nsMemoryReporter::nsMemoryReporter(nsACString& process,
                                    nsACString& path,
-                                   PRInt32 kind,
-                                   PRInt32 units,
-                                   PRInt64 amount,
+                                   int32_t kind,
+                                   int32_t units,
+                                   int64_t amount,
                                    nsACString& desc)
 : mProcess(process)
 , mPath(path)
@@ -871,19 +872,19 @@ NS_IMETHODIMP nsMemoryReporter::GetPath(nsACString &aPath)
     return NS_OK;
 }
 
-NS_IMETHODIMP nsMemoryReporter::GetKind(PRInt32 *aKind)
+NS_IMETHODIMP nsMemoryReporter::GetKind(int32_t *aKind)
 {
     *aKind = mKind;
     return NS_OK;
 }
 
-NS_IMETHODIMP nsMemoryReporter::GetUnits(PRInt32 *aUnits)
+NS_IMETHODIMP nsMemoryReporter::GetUnits(int32_t *aUnits)
 {
   *aUnits = mUnits;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMemoryReporter::GetAmount(PRInt64 *aAmount)
+NS_IMETHODIMP nsMemoryReporter::GetAmount(int64_t *aAmount)
 {
     *aAmount = mAmount;
     return NS_OK;
@@ -899,7 +900,7 @@ nsresult
 NS_RegisterMemoryReporter (nsIMemoryReporter *reporter)
 {
     nsCOMPtr<nsIMemoryReporterManager> mgr = do_GetService("@mozilla.org/memory-reporter-manager;1");
-    if (mgr == nsnull)
+    if (mgr == nullptr)
         return NS_ERROR_FAILURE;
     return mgr->RegisterReporter(reporter);
 }
@@ -908,7 +909,7 @@ nsresult
 NS_RegisterMemoryMultiReporter (nsIMemoryMultiReporter *reporter)
 {
     nsCOMPtr<nsIMemoryReporterManager> mgr = do_GetService("@mozilla.org/memory-reporter-manager;1");
-    if (mgr == nsnull)
+    if (mgr == nullptr)
         return NS_ERROR_FAILURE;
     return mgr->RegisterMultiReporter(reporter);
 }
@@ -917,7 +918,7 @@ nsresult
 NS_UnregisterMemoryReporter (nsIMemoryReporter *reporter)
 {
     nsCOMPtr<nsIMemoryReporterManager> mgr = do_GetService("@mozilla.org/memory-reporter-manager;1");
-    if (mgr == nsnull)
+    if (mgr == nullptr)
         return NS_ERROR_FAILURE;
     return mgr->UnregisterReporter(reporter);
 }
@@ -926,7 +927,7 @@ nsresult
 NS_UnregisterMemoryMultiReporter (nsIMemoryMultiReporter *reporter)
 {
     nsCOMPtr<nsIMemoryReporterManager> mgr = do_GetService("@mozilla.org/memory-reporter-manager;1");
-    if (mgr == nsnull)
+    if (mgr == nullptr)
         return NS_ERROR_FAILURE;
     return mgr->UnregisterMultiReporter(reporter);
 }
@@ -941,7 +942,7 @@ public:
     NS_DECL_ISUPPORTS
 
     NS_IMETHOD Callback(const nsACString &aProcess, const nsACString &aPath,
-                        PRInt32 aKind, PRInt32 aUnits, PRInt64 aAmount,
+                        int32_t aKind, int32_t aUnits, int64_t aAmount,
                         const nsACString &aDescription,
                         nsISupports *aData)
     {
@@ -969,7 +970,7 @@ DMDCheckAndDump()
         e->GetNext(getter_AddRefs(r));
 
         // Just getting the amount is enough for the reporter to report to DMD.
-        PRInt64 amount;
+        int64_t amount;
         (void)r->GetAmount(&amount);
     }
 
@@ -980,7 +981,7 @@ DMDCheckAndDump()
     while (NS_SUCCEEDED(e2->HasMoreElements(&more)) && more) {
       nsCOMPtr<nsIMemoryMultiReporter> r;
       e2->GetNext(getter_AddRefs(r));
-      r->CollectReports(cb, nsnull);
+      r->CollectReports(cb, nullptr);
     }
 
     VALGRIND_DMD_CHECK_REPORTING;
