@@ -1,42 +1,8 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
  * vim: set shiftwidth=4 tabstop=8 autoindent cindent expandtab: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla stack walking code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2000
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Michael Judge, 20-December-2000
- *   L. David Baron <dbaron@dbaron.org>, Mozilla Corporation
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /* API for getting a stack trace of the C/C++ stack on the current thread */
 
@@ -85,7 +51,7 @@ malloc_logger_t(uint32_t type, uintptr_t arg1, uintptr_t arg2, uintptr_t arg3,
 extern malloc_logger_t *malloc_logger;
 
 static void
-stack_callback(void *pc, void *closure)
+stack_callback(void *pc, void *sp, void *closure)
 {
   const char *name = reinterpret_cast<char *>(closure);
   Dl_info info;
@@ -103,9 +69,9 @@ stack_callback(void *pc, void *closure)
 #define MAC_OS_X_VERSION_10_7_HEX 0x00001070
 #define MAC_OS_X_VERSION_10_6_HEX 0x00001060
 
-static PRInt32 OSXVersion()
+static int32_t OSXVersion()
 {
-  static PRInt32 gOSXVersion = 0x0;
+  static int32_t gOSXVersion = 0x0;
   if (gOSXVersion == 0x0) {
     OSErr err = ::Gestalt(gestaltSystemVersion, (SInt32*)&gOSXVersion);
     MOZ_ASSERT(err == noErr);
@@ -207,6 +173,7 @@ StackWalkInitCriticalAddress()
 #include <windows.h>
 #include <process.h>
 #include <stdio.h>
+#include <malloc.h>
 #include "plstr.h"
 #include "mozilla/FunctionTimer.h"
 
@@ -234,15 +201,18 @@ bool EnsureSymInitialized();
 bool EnsureImageHlpInitialized();
 
 struct WalkStackData {
-  PRUint32 skipFrames;
+  uint32_t skipFrames;
   HANDLE thread;
   bool walkCallingThread;
   HANDLE process;
   HANDLE eventStart;
   HANDLE eventEnd;
   void **pcs;
-  PRUint32 pc_size;
-  PRUint32 pc_count;
+  uint32_t pc_size;
+  uint32_t pc_count;
+  void **sps;
+  uint32_t sp_size;
+  uint32_t sp_count;
 };
 
 void PrintError(char *prefix, WalkStackData* data);
@@ -321,6 +291,7 @@ WalkStackMain64(struct WalkStackData* data)
     HANDLE myProcess = data->process;
     HANDLE myThread = data->thread;
     DWORD64 addr;
+    DWORD64 spaddr;
     STACKFRAME64 frame64;
     // skip our own stack walking frames
     int skip = (data->walkCallingThread ? 3 : 0) + data->skipFrames;
@@ -382,10 +353,12 @@ WalkStackMain64(struct WalkStackData* data)
         );
         LeaveCriticalSection(&gDbgHelpCS);
 
-        if (ok)
+        if (ok) {
             addr = frame64.AddrPC.Offset;
-        else {
+            spaddr = frame64.AddrStack.Offset;
+         } else {
             addr = 0;
+            spaddr = 0;
             PrintError("WalkStack64");
         }
 
@@ -401,6 +374,10 @@ WalkStackMain64(struct WalkStackData* data)
             data->pcs[data->pc_count] = (void*)addr;
         ++data->pc_count;
 
+        if (data->sp_count < data->sp_size)
+            data->sps[data->sp_count] = (void*)spaddr;
+        ++data->sp_count;
+
         if (frame64.AddrReturn.Offset == 0)
             break;
     }
@@ -469,11 +446,12 @@ WalkStackThread(void* aData)
  */
 
 EXPORT_XPCOM_API(nsresult)
-NS_StackWalk(NS_WalkStackCallback aCallback, PRUint32 aSkipFrames,
+NS_StackWalk(NS_WalkStackCallback aCallback, uint32_t aSkipFrames,
              void *aClosure, uintptr_t aThread)
 {
     MOZ_ASSERT(gCriticalAddress.mInit);
-    HANDLE myProcess, myThread;
+    static HANDLE myProcess = NULL;
+    HANDLE myThread;
     DWORD walkerReturn;
     struct WalkStackData data;
 
@@ -490,13 +468,15 @@ NS_StackWalk(NS_WalkStackCallback aCallback, PRUint32 aSkipFrames,
     }
 
     // Have to duplicate handle to get a real handle.
-    if (!::DuplicateHandle(::GetCurrentProcess(),
-                           ::GetCurrentProcess(),
-                           ::GetCurrentProcess(),
-                           &myProcess,
-                           PROCESS_ALL_ACCESS, FALSE, 0)) {
-        PrintError("DuplicateHandle (process)");
-        return NS_ERROR_FAILURE;
+    if (!myProcess) {
+        if (!::DuplicateHandle(::GetCurrentProcess(),
+                               ::GetCurrentProcess(),
+                               ::GetCurrentProcess(),
+                               &myProcess,
+                               PROCESS_ALL_ACCESS, FALSE, 0)) {
+            PrintError("DuplicateHandle (process)");
+            return NS_ERROR_FAILURE;
+        }
     }
     if (!::DuplicateHandle(::GetCurrentProcess(),
                            targetThread,
@@ -504,7 +484,6 @@ NS_StackWalk(NS_WalkStackCallback aCallback, PRUint32 aSkipFrames,
                            &myThread,
                            THREAD_ALL_ACCESS, FALSE, 0)) {
         PrintError("DuplicateHandle (thread)");
-        ::CloseHandle(myProcess);
         return NS_ERROR_FAILURE;
     }
 
@@ -515,11 +494,25 @@ NS_StackWalk(NS_WalkStackCallback aCallback, PRUint32 aSkipFrames,
     data.pcs = local_pcs;
     data.pc_count = 0;
     data.pc_size = ArrayLength(local_pcs);
+    void *local_sps[1024];
+    data.sps = local_sps;
+    data.sp_count = 0;
+    data.sp_size = ArrayLength(local_pcs);
 
     if (aThread) {
         // If we're walking the stack of another thread, we don't need to
         // use a separate walker thread.
         WalkStackMain64(&data);
+
+        if (data.pc_count > data.pc_size) {
+            data.pcs = (void**) _alloca(data.pc_count * sizeof(void*));
+            data.pc_size = data.pc_count;
+            data.pc_count = 0;
+            data.sps = (void**) _alloca(data.sp_count * sizeof(void*));
+            data.sp_size = data.sp_count;
+            data.sp_count = 0;
+            WalkStackMain64(&data);
+        }
     } else {
         data.eventStart = ::CreateEvent(NULL, FALSE /* auto-reset*/,
                               FALSE /* initially non-signaled */, NULL);
@@ -533,9 +526,12 @@ NS_StackWalk(NS_WalkStackCallback aCallback, PRUint32 aSkipFrames,
         if (walkerReturn != WAIT_OBJECT_0)
             PrintError("SignalObjectAndWait (1)");
         if (data.pc_count > data.pc_size) {
-            data.pcs = (void**) malloc(data.pc_count * sizeof(void*));
+            data.pcs = (void**) _alloca(data.pc_count * sizeof(void*));
             data.pc_size = data.pc_count;
             data.pc_count = 0;
+            data.sps = (void**) _alloca(data.sp_count * sizeof(void*));
+            data.sp_size = data.sp_count;
+            data.sp_count = 0;
             ::PostThreadMessage(gStackWalkThread, WM_USER, 0, (LPARAM)&data);
             walkerReturn = ::SignalObjectAndWait(data.eventStart,
                                data.eventEnd, INFINITE, FALSE);
@@ -548,13 +544,9 @@ NS_StackWalk(NS_WalkStackCallback aCallback, PRUint32 aSkipFrames,
     }
 
     ::CloseHandle(myThread);
-    ::CloseHandle(myProcess);
 
-    for (PRUint32 i = 0; i < data.pc_count; ++i)
-        (*aCallback)(data.pcs[i], aClosure);
-
-    if (data.pc_size > ArrayLength(local_pcs))
-        free(data.pcs);
+    for (uint32_t i = 0; i < data.pc_count; ++i)
+        (*aCallback)(data.pcs[i], data.sps[i], aClosure);
 
     return NS_OK;
 }
@@ -624,7 +616,7 @@ BOOL SymGetModuleInfoEspecial64(HANDLE aProcess, DWORD64 aAddr, PIMAGEHLP_MODULE
      * Init the vars if we have em.
      */
     aModuleInfo->SizeOfStruct = NS_IMAGEHLP_MODULE64_SIZE;
-    if (nsnull != aLineInfo) {
+    if (nullptr != aLineInfo) {
         aLineInfo->SizeOfStruct = sizeof(IMAGEHLP_LINE64);
     }
 
@@ -663,7 +655,7 @@ BOOL SymGetModuleInfoEspecial64(HANDLE aProcess, DWORD64 aAddr, PIMAGEHLP_MODULE
      * If we got module info, we may attempt line info as well.
      * We will not report failure if this does not work.
      */
-    if (FALSE != retval && nsnull != aLineInfo) {
+    if (FALSE != retval && nullptr != aLineInfo) {
         DWORD displacement = 0;
         BOOL lineRes = FALSE;
         lineRes = SymGetLineFromAddr64(aProcess, aAddr, &displacement, aLineInfo);
@@ -765,7 +757,7 @@ NS_DescribeCodeAddress(void *aPC, nsCodeAddressDetails *aDetails)
 
 EXPORT_XPCOM_API(nsresult)
 NS_FormatCodeAddressDetails(void *aPC, const nsCodeAddressDetails *aDetails,
-                            char *aBuffer, PRUint32 aBufferSize)
+                            char *aBuffer, uint32_t aBufferSize)
 {
     if (aDetails->function[0])
         _snprintf(aBuffer, aBufferSize, "%s!%s+0x%016lX",
@@ -775,7 +767,7 @@ NS_FormatCodeAddressDetails(void *aPC, const nsCodeAddressDetails *aDetails,
 
     aBuffer[aBufferSize - 1] = '\0';
 
-    PRUint32 len = strlen(aBuffer);
+    uint32_t len = strlen(aBuffer);
     if (aDetails->filename[0]) {
         _snprintf(aBuffer + len, aBufferSize - len, " (%s, line %d)\n",
                   aDetails->filename, aDetails->lineno);
@@ -851,9 +843,9 @@ static struct bucket * newbucket ( void * pc );
 static struct frame * cs_getmyframeptr ( void );
 static void   cs_walk_stack ( void * (*read_func)(char * address),
                               struct frame * fp,
-                              int (*operate_func)(void *, void *),
+                              int (*operate_func)(void *, void *, void *),
                               void * usrarg );
-static void   cs_operate ( void (*operate_func)(void *, void *),
+static void   cs_operate ( void (*operate_func)(void *, void *, void *),
                            void * usrarg );
 
 #ifndef STACK_BIAS
@@ -885,7 +877,7 @@ struct bucket {
 
 struct my_user_args {
     NS_WalkStackCallback callback;
-    PRUint32 skipFrames;
+    uint32_t skipFrames;
     void *closure;
 };
 
@@ -981,13 +973,13 @@ csgetframeptr()
 
 
 static void
-cswalkstack(struct frame *fp, int (*operate_func)(void *, void *),
+cswalkstack(struct frame *fp, int (*operate_func)(void *, void *, void *),
     void *usrarg)
 {
 
     while (fp != 0 && fp->fr_savpc != 0) {
 
-        if (operate_func((void *)fp->fr_savpc, usrarg) != 0)
+        if (operate_func((void *)fp->fr_savpc, NULL, usrarg) != 0)
             break;
         /*
          * watch out - libthread stacks look funny at the top
@@ -1001,13 +993,13 @@ cswalkstack(struct frame *fp, int (*operate_func)(void *, void *),
 
 
 static void
-cs_operate(int (*operate_func)(void *, void *), void * usrarg)
+cs_operate(int (*operate_func)(void *, void *, void *), void * usrarg)
 {
     cswalkstack(csgetframeptr(), operate_func, usrarg);
 }
 
 EXPORT_XPCOM_API(nsresult)
-NS_StackWalk(NS_WalkStackCallback aCallback, PRUint32 aSkipFrames,
+NS_StackWalk(NS_WalkStackCallback aCallback, uint32_t aSkipFrames,
              void *aClosure, uintptr_t aThread)
 {
     MOZ_ASSERT(gCriticalAddress.mInit);
@@ -1062,7 +1054,7 @@ NS_DescribeCodeAddress(void *aPC, nsCodeAddressDetails *aDetails)
 
 EXPORT_XPCOM_API(nsresult)
 NS_FormatCodeAddressDetails(void *aPC, const nsCodeAddressDetails *aDetails,
-                            char *aBuffer, PRUint32 aBufferSize)
+                            char *aBuffer, uint32_t aBufferSize)
 {
     snprintf(aBuffer, aBufferSize, "%p %s:%s+0x%lx\n",
              aPC,
@@ -1085,7 +1077,7 @@ extern void *__libc_stack_end; // from ld-linux.so
 #endif
 namespace mozilla {
 nsresult
-FramePointerStackWalk(NS_WalkStackCallback aCallback, PRUint32 aSkipFrames,
+FramePointerStackWalk(NS_WalkStackCallback aCallback, uint32_t aSkipFrames,
                       void *aClosure, void **bp, void *aStackEnd)
 {
   // Stack walking code courtesy Kipp's "leaky".
@@ -1107,15 +1099,21 @@ FramePointerStackWalk(NS_WalkStackCallback aCallback, PRUint32 aSkipFrames,
 #if (defined(__ppc__) && defined(XP_MACOSX)) || defined(__powerpc64__)
     // ppc mac or powerpc64 linux
     void *pc = *(bp+2);
+    bp += 3;
 #else // i386 or powerpc32 linux
     void *pc = *(bp+1);
+    bp += 2;
 #endif
     if (IsCriticalAddress(pc)) {
       printf("Aborting stack trace, PC is critical\n");
       return NS_ERROR_UNEXPECTED;
     }
     if (--skip < 0) {
-      (*aCallback)(pc, aClosure);
+      // Assume that the SP points to the BP of the function
+      // it called. We can't know the exact location of the SP
+      // but this should be sufficient for our use the SP
+      // to order elements on the stack.
+      (*aCallback)(pc, bp, aClosure);
     }
     bp = next;
   }
@@ -1128,7 +1126,7 @@ FramePointerStackWalk(NS_WalkStackCallback aCallback, PRUint32 aSkipFrames,
 #if X86_OR_PPC && (NSSTACKWALK_SUPPORTS_MACOSX || NSSTACKWALK_SUPPORTS_LINUX) // i386 or PPC Linux or Mac stackwalking code
 
 EXPORT_XPCOM_API(nsresult)
-NS_StackWalk(NS_WalkStackCallback aCallback, PRUint32 aSkipFrames,
+NS_StackWalk(NS_WalkStackCallback aCallback, uint32_t aSkipFrames,
              void *aClosure, uintptr_t aThread)
 {
   MOZ_ASSERT(gCriticalAddress.mInit);
@@ -1159,6 +1157,7 @@ NS_StackWalk(NS_WalkStackCallback aCallback, PRUint32 aSkipFrames,
 #elif defined(HAVE__UNWIND_BACKTRACE)
 
 // libgcc_s.so symbols _Unwind_Backtrace@@GCC_3.3 and _Unwind_GetIP@@GCC_3.0
+#define _GNU_SOURCE
 #include <unwind.h>
 
 struct unwind_info {
@@ -1172,6 +1171,7 @@ unwind_callback (struct _Unwind_Context *context, void *closure)
 {
     unwind_info *info = static_cast<unwind_info *>(closure);
     void *pc = reinterpret_cast<void *>(_Unwind_GetIP(context));
+    // TODO Use something like '_Unwind_GetGR()' to get the stack pointer.
     if (IsCriticalAddress(pc)) {
         printf("Aborting stack trace, PC is critical\n");
         /* We just want to stop the walk, so any error code will do.
@@ -1180,12 +1180,12 @@ unwind_callback (struct _Unwind_Context *context, void *closure)
         return _URC_FOREIGN_EXCEPTION_CAUGHT;
     }
     if (--info->skip < 0)
-        (*info->callback)(pc, info->closure);
+        (*info->callback)(pc, NULL, info->closure);
     return _URC_NO_REASON;
 }
 
 EXPORT_XPCOM_API(nsresult)
-NS_StackWalk(NS_WalkStackCallback aCallback, PRUint32 aSkipFrames,
+NS_StackWalk(NS_WalkStackCallback aCallback, uint32_t aSkipFrames,
              void *aClosure, uintptr_t aThread)
 {
     MOZ_ASSERT(gCriticalAddress.mInit);
@@ -1244,7 +1244,7 @@ NS_DescribeCodeAddress(void *aPC, nsCodeAddressDetails *aDetails)
 
 EXPORT_XPCOM_API(nsresult)
 NS_FormatCodeAddressDetails(void *aPC, const nsCodeAddressDetails *aDetails,
-                            char *aBuffer, PRUint32 aBufferSize)
+                            char *aBuffer, uint32_t aBufferSize)
 {
   if (!aDetails->library[0]) {
     snprintf(aBuffer, aBufferSize, "UNKNOWN %p\n", aPC);
@@ -1264,7 +1264,7 @@ NS_FormatCodeAddressDetails(void *aPC, const nsCodeAddressDetails *aDetails,
 #else // unsupported platform.
 
 EXPORT_XPCOM_API(nsresult)
-NS_StackWalk(NS_WalkStackCallback aCallback, PRUint32 aSkipFrames,
+NS_StackWalk(NS_WalkStackCallback aCallback, uint32_t aSkipFrames,
              void *aClosure, uintptr_t aThread)
 {
     MOZ_ASSERT(gCriticalAddress.mInit);
@@ -1274,7 +1274,7 @@ NS_StackWalk(NS_WalkStackCallback aCallback, PRUint32 aSkipFrames,
 
 namespace mozilla {
 nsresult
-FramePointerStackWalk(NS_WalkStackCallback aCallback, PRUint32 aSkipFrames,
+FramePointerStackWalk(NS_WalkStackCallback aCallback, uint32_t aSkipFrames,
                       void *aClosure, void **bp)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
@@ -1295,7 +1295,7 @@ NS_DescribeCodeAddress(void *aPC, nsCodeAddressDetails *aDetails)
 
 EXPORT_XPCOM_API(nsresult)
 NS_FormatCodeAddressDetails(void *aPC, const nsCodeAddressDetails *aDetails,
-                            char *aBuffer, PRUint32 aBufferSize)
+                            char *aBuffer, uint32_t aBufferSize)
 {
     aBuffer[0] = '\0';
     return NS_ERROR_NOT_IMPLEMENTED;
