@@ -1,42 +1,7 @@
 /* vim:set ts=2 sw=2 et cindent: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla.
- *
- * The Initial Developer of the Original Code is IBM Corporation.
- * Portions created by IBM Corporation are Copyright (C) 2003
- * IBM Corporation.  All Rights Reserved.
- *
- * Contributor(s):
- *   Darin Fisher <darin@meer.net>
- *   Benjamin Smedberg <benjamin@smedbergs.us>
- *   Ben Turner <mozilla@songbirdnest.com>
- *   Prasad Sunkari <prasad@medhas.org>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /**
  * This header provides wrapper classes around the frozen string API
@@ -60,33 +25,33 @@
 /**
  * Comparison function for use with nsACString::Equals
  */
-NS_HIDDEN_(PRInt32)
+NS_HIDDEN_(int32_t)
 CaseInsensitiveCompare(const char *a, const char *b,
-                       PRUint32 length);
+                       uint32_t length);
 
 class nsAString
 {
 public:
   typedef PRUnichar  char_type;
   typedef nsAString  self_type;
-  typedef PRUint32   size_type;
-  typedef PRUint32   index_type;
+  typedef uint32_t   size_type;
+  typedef uint32_t   index_type;
 
   /**
    * Returns the length, beginning, and end of a string in one operation.
    */
-  NS_HIDDEN_(PRUint32) BeginReading(const char_type **begin,
-                                    const char_type **end = nsnull) const;
+  NS_HIDDEN_(uint32_t) BeginReading(const char_type **begin,
+                                    const char_type **end = nullptr) const;
 
   NS_HIDDEN_(const char_type*) BeginReading() const;
   NS_HIDDEN_(const char_type*) EndReading() const;
 
-  NS_HIDDEN_(char_type) CharAt(PRUint32 aPos) const
+  NS_HIDDEN_(char_type) CharAt(uint32_t aPos) const
   {
     NS_ASSERTION(aPos < Length(), "Out of bounds");
     return BeginReading()[aPos];
   }
-  NS_HIDDEN_(char_type) operator [](PRUint32 aPos) const
+  NS_HIDDEN_(char_type) operator [](uint32_t aPos) const
   {
     return CharAt(aPos);
   }
@@ -97,7 +62,7 @@ public:
   NS_HIDDEN_(char_type) Last() const
   {
     const char_type* data;
-    PRUint32 dataLen = NS_StringGetData(*this, &data);
+    uint32_t dataLen = NS_StringGetData(*this, &data);
     return data[dataLen - 1];
   }
 
@@ -109,14 +74,14 @@ public:
    *                  to leave the length unchanged.
    * @return  The new length of the string, or 0 if resizing failed.
    */
-  NS_HIDDEN_(PRUint32) BeginWriting(char_type **begin,
-                                    char_type **end = nsnull,
-                                    PRUint32 newSize = PR_UINT32_MAX);
+  NS_HIDDEN_(uint32_t) BeginWriting(char_type **begin,
+                                    char_type **end = nullptr,
+                                    uint32_t newSize = PR_UINT32_MAX);
 
-  NS_HIDDEN_(char_type*) BeginWriting(PRUint32 = PR_UINT32_MAX);
+  NS_HIDDEN_(char_type*) BeginWriting(uint32_t = PR_UINT32_MAX);
   NS_HIDDEN_(char_type*) EndWriting();
 
-  NS_HIDDEN_(bool) SetLength(PRUint32 aLen);
+  NS_HIDDEN_(bool) SetLength(uint32_t aLen);
 
   NS_HIDDEN_(size_type) Length() const
   {
@@ -169,7 +134,7 @@ public:
   NS_HIDDEN_(void) Replace( index_type cutStart, size_type cutLength, const self_type& readable )
   {
     const char_type* data;
-    PRUint32 dataLen = NS_StringGetData(readable, &data);
+    uint32_t dataLen = NS_StringGetData(readable, &data);
     NS_StringSetDataRange(*this, cutStart, cutLength, data, dataLen);
   }
   NS_HIDDEN_(void) SetCharAt( char_type c, index_type pos )
@@ -189,7 +154,7 @@ public:
   NS_HIDDEN_(void) Insert( const char_type* data, index_type pos, size_type length = size_type(-1) )  { Replace(pos, 0, data, length); }
   NS_HIDDEN_(void) Insert( const self_type& readable, index_type pos )                                { Replace(pos, 0, readable); }
 
-  NS_HIDDEN_(void) Cut( index_type cutStart, size_type cutLength )                                    { Replace(cutStart, cutLength, nsnull, 0); }
+  NS_HIDDEN_(void) Cut( index_type cutStart, size_type cutLength )                                    { Replace(cutStart, cutLength, nullptr, 0); }
 
   NS_HIDDEN_(void) Truncate() { SetLength(0); }
 
@@ -209,18 +174,18 @@ public:
   /**
    * Compare strings of characters. Return 0 if the characters are equal,
    */
-  typedef PRInt32 (*ComparatorFunc)(const char_type *a,
+  typedef int32_t (*ComparatorFunc)(const char_type *a,
                                     const char_type *b,
-                                    PRUint32 length);
+                                    uint32_t length);
 
-  static NS_HIDDEN_(PRInt32) DefaultComparator(const char_type *a,
+  static NS_HIDDEN_(int32_t) DefaultComparator(const char_type *a,
                                                const char_type *b,
-                                               PRUint32 length);
+                                               uint32_t length);
 
-  NS_HIDDEN_(PRInt32) Compare( const char_type *other,
+  NS_HIDDEN_(int32_t) Compare( const char_type *other,
                                ComparatorFunc c = DefaultComparator ) const;
 
-  NS_HIDDEN_(PRInt32) Compare( const self_type &other,
+  NS_HIDDEN_(int32_t) Compare( const self_type &other,
                                ComparatorFunc c = DefaultComparator ) const;
 
   NS_HIDDEN_(bool) Equals( const char_type *other,
@@ -299,7 +264,7 @@ public:
    *
    * @return the offset of aStr, or -1 if not found
    */
-  NS_HIDDEN_(PRInt32) Find(const self_type& aStr,
+  NS_HIDDEN_(int32_t) Find(const self_type& aStr,
                            ComparatorFunc c = DefaultComparator) const
   { return Find(aStr, 0, c); }
 
@@ -308,7 +273,7 @@ public:
    *
    * @return the offset of aStr, or -1 if not found
    */
-  NS_HIDDEN_(PRInt32) Find(const self_type& aStr, PRUint32 aOffset,
+  NS_HIDDEN_(int32_t) Find(const self_type& aStr, uint32_t aOffset,
                            ComparatorFunc c = DefaultComparator) const;
 
   /**
@@ -316,10 +281,10 @@ public:
    *
    * @return the offset of aStr, or -1 if not found.
    */
-  NS_HIDDEN_(PRInt32) Find(const char *aStr, bool aIgnoreCase = false) const
+  NS_HIDDEN_(int32_t) Find(const char *aStr, bool aIgnoreCase = false) const
   { return Find(aStr, 0, aIgnoreCase); }
 
-  NS_HIDDEN_(PRInt32) Find(const char *aStr, PRUint32 aOffset, bool aIgnoreCase = false) const;
+  NS_HIDDEN_(int32_t) Find(const char *aStr, uint32_t aOffset, bool aIgnoreCase = false) const;
 
   /**
    * Find the last occurrence of aStr in this string.
@@ -327,7 +292,7 @@ public:
    * @return The offset of aStr from the beginning of the string,
    *         or -1 if not found.
    */
-  NS_HIDDEN_(PRInt32) RFind(const self_type& aStr,
+  NS_HIDDEN_(int32_t) RFind(const self_type& aStr,
                             ComparatorFunc c = DefaultComparator) const
   { return RFind(aStr, -1, c); }
 
@@ -339,7 +304,7 @@ public:
    * @return The offset of aStr from the beginning of the string,
    *         or -1 if not found.
    */
-  NS_HIDDEN_(PRInt32) RFind(const self_type& aStr, PRInt32 aOffset,
+  NS_HIDDEN_(int32_t) RFind(const self_type& aStr, int32_t aOffset,
                             ComparatorFunc c = DefaultComparator) const;
 
   /**
@@ -348,7 +313,7 @@ public:
    * @return The offset of aStr from the beginning of the string,
    *         or -1 if not found.
    */
-  NS_HIDDEN_(PRInt32) RFind(const char *aStr, bool aIgnoreCase = false) const
+  NS_HIDDEN_(int32_t) RFind(const char *aStr, bool aIgnoreCase = false) const
   { return RFind(aStr, -1, aIgnoreCase); }
 
   /**
@@ -359,7 +324,7 @@ public:
    * @return The offset of aStr from the beginning of the string,
    *         or -1 if not found.
    */
-  NS_HIDDEN_(PRInt32) RFind(const char *aStr, PRInt32 aOffset, bool aIgnoreCase) const;
+  NS_HIDDEN_(int32_t) RFind(const char *aStr, int32_t aOffset, bool aIgnoreCase) const;
 
   /**
    * Search for the offset of the first occurrence of a character in a
@@ -370,7 +335,7 @@ public:
    * @return The offset of the character from the beginning of the string,
    *         or -1 if not found.
    */
-  NS_HIDDEN_(PRInt32) FindChar(char_type aChar, PRUint32 aOffset = 0) const;
+  NS_HIDDEN_(int32_t) FindChar(char_type aChar, uint32_t aOffset = 0) const;
 
   /**
    * Search for the offset of the last occurrence of a character in a
@@ -379,12 +344,12 @@ public:
    * @return The offset of the character from the beginning of the string,
    *         or -1 if not found.
    */
-  NS_HIDDEN_(PRInt32) RFindChar(char_type aChar) const;
+  NS_HIDDEN_(int32_t) RFindChar(char_type aChar) const;
 
   /**
    * Append a string representation of a number.
    */
-  NS_HIDDEN_(void) AppendInt(int aInt, PRInt32 aRadix = 10);
+  NS_HIDDEN_(void) AppendInt(int aInt, int32_t aRadix = 10);
 
 #ifndef XPCOM_GLUE_AVOID_NSPR
   /**
@@ -393,8 +358,8 @@ public:
    * @param aErrorCode pointer to contain result code.
    * @param aRadix must be 10 or 16
    */
-  NS_HIDDEN_(PRInt32) ToInteger(nsresult* aErrorCode,
-                                PRUint32 aRadix = 10) const;
+  NS_HIDDEN_(int32_t) ToInteger(nsresult* aErrorCode,
+                                uint32_t aRadix = 10) const;
 #endif // XPCOM_GLUE_AVOID_NSPR
 
 protected:
@@ -407,24 +372,24 @@ class nsACString
 public:
   typedef char       char_type;
   typedef nsACString self_type;
-  typedef PRUint32   size_type;
-  typedef PRUint32   index_type;
+  typedef uint32_t   size_type;
+  typedef uint32_t   index_type;
 
   /**
    * Returns the length, beginning, and end of a string in one operation.
    */
-  NS_HIDDEN_(PRUint32) BeginReading(const char_type **begin,
-                                    const char_type **end = nsnull) const;
+  NS_HIDDEN_(uint32_t) BeginReading(const char_type **begin,
+                                    const char_type **end = nullptr) const;
 
   NS_HIDDEN_(const char_type*) BeginReading() const;
   NS_HIDDEN_(const char_type*) EndReading() const;
 
-  NS_HIDDEN_(char_type) CharAt(PRUint32 aPos) const
+  NS_HIDDEN_(char_type) CharAt(uint32_t aPos) const
   {
     NS_ASSERTION(aPos < Length(), "Out of bounds");
     return BeginReading()[aPos];
   }
-  NS_HIDDEN_(char_type) operator [](PRUint32 aPos) const
+  NS_HIDDEN_(char_type) operator [](uint32_t aPos) const
   {
     return CharAt(aPos);
   }
@@ -435,7 +400,7 @@ public:
   NS_HIDDEN_(char_type) Last() const
   {
     const char_type* data;
-    PRUint32 dataLen = NS_CStringGetData(*this, &data);
+    uint32_t dataLen = NS_CStringGetData(*this, &data);
     return data[dataLen - 1];
   }
 
@@ -447,14 +412,14 @@ public:
    *                  to leave the length unchanged.
    * @return  The new length of the string, or 0 if resizing failed.
    */
-  NS_HIDDEN_(PRUint32) BeginWriting(char_type **begin,
-                                    char_type **end = nsnull,
-                                    PRUint32 newSize = PR_UINT32_MAX);
+  NS_HIDDEN_(uint32_t) BeginWriting(char_type **begin,
+                                    char_type **end = nullptr,
+                                    uint32_t newSize = PR_UINT32_MAX);
 
-  NS_HIDDEN_(char_type*) BeginWriting(PRUint32 aLen = PR_UINT32_MAX);
+  NS_HIDDEN_(char_type*) BeginWriting(uint32_t aLen = PR_UINT32_MAX);
   NS_HIDDEN_(char_type*) EndWriting();
 
-  NS_HIDDEN_(bool) SetLength(PRUint32 aLen);
+  NS_HIDDEN_(bool) SetLength(uint32_t aLen);
 
   NS_HIDDEN_(size_type) Length() const
   {
@@ -512,7 +477,7 @@ public:
   NS_HIDDEN_(void) Replace( index_type cutStart, size_type cutLength, const self_type& readable )
   {
     const char_type* data;
-    PRUint32 dataLen = NS_CStringGetData(readable, &data);
+    uint32_t dataLen = NS_CStringGetData(readable, &data);
     NS_CStringSetDataRange(*this, cutStart, cutLength, data, dataLen);
   }
   NS_HIDDEN_(void) SetCharAt( char_type c, index_type pos )
@@ -532,7 +497,7 @@ public:
   NS_HIDDEN_(void) Insert( const char_type* data, index_type pos, size_type length = size_type(-1) )  { Replace(pos, 0, data, length); }
   NS_HIDDEN_(void) Insert( const self_type& readable, index_type pos )                                { Replace(pos, 0, readable); }
 
-  NS_HIDDEN_(void) Cut( index_type cutStart, size_type cutLength )                                    { Replace(cutStart, cutLength, nsnull, 0); }
+  NS_HIDDEN_(void) Cut( index_type cutStart, size_type cutLength )                                    { Replace(cutStart, cutLength, nullptr, 0); }
 
   NS_HIDDEN_(void) Truncate() { SetLength(0); }
 
@@ -552,18 +517,18 @@ public:
   /**
    * Compare strings of characters. Return 0 if the characters are equal,
    */
-  typedef PRInt32 (*ComparatorFunc)(const char_type *a,
+  typedef int32_t (*ComparatorFunc)(const char_type *a,
                                     const char_type *b,
-                                    PRUint32 length);
+                                    uint32_t length);
 
-  static NS_HIDDEN_(PRInt32) DefaultComparator(const char_type *a,
+  static NS_HIDDEN_(int32_t) DefaultComparator(const char_type *a,
                                                const char_type *b,
-                                               PRUint32 length);
+                                               uint32_t length);
 
-  NS_HIDDEN_(PRInt32) Compare( const char_type *other,
+  NS_HIDDEN_(int32_t) Compare( const char_type *other,
                                ComparatorFunc c = DefaultComparator ) const;
 
-  NS_HIDDEN_(PRInt32) Compare( const self_type &other,
+  NS_HIDDEN_(int32_t) Compare( const self_type &other,
                                ComparatorFunc c = DefaultComparator ) const;
 
   NS_HIDDEN_(bool) Equals( const char_type *other,
@@ -648,7 +613,7 @@ public:
    *
    * @return the offset of aStr, or -1 if not found
    */
-  NS_HIDDEN_(PRInt32) Find(const self_type& aStr,
+  NS_HIDDEN_(int32_t) Find(const self_type& aStr,
                            ComparatorFunc c = DefaultComparator) const
   { return Find(aStr, 0, c); }
 
@@ -657,7 +622,7 @@ public:
    *
    * @return the offset of aStr, or -1 if not found
    */
-  NS_HIDDEN_(PRInt32) Find(const self_type& aStr, PRUint32 aOffset,
+  NS_HIDDEN_(int32_t) Find(const self_type& aStr, uint32_t aOffset,
                            ComparatorFunc c = DefaultComparator) const;
 
   /**
@@ -665,10 +630,10 @@ public:
    *
    * @return the offset of aStr, or -1 if not found
    */
-  NS_HIDDEN_(PRInt32) Find(const char_type *aStr,
+  NS_HIDDEN_(int32_t) Find(const char_type *aStr,
                            ComparatorFunc c = DefaultComparator) const;
 
-  NS_HIDDEN_(PRInt32) Find(const char_type *aStr, PRUint32 aLen,
+  NS_HIDDEN_(int32_t) Find(const char_type *aStr, uint32_t aLen,
                            ComparatorFunc c = DefaultComparator) const;
 
   /**
@@ -677,7 +642,7 @@ public:
    * @return The offset of the character from the beginning of the string,
    *         or -1 if not found.
    */
-  NS_HIDDEN_(PRInt32) RFind(const self_type& aStr,
+  NS_HIDDEN_(int32_t) RFind(const self_type& aStr,
                             ComparatorFunc c = DefaultComparator) const
   { return RFind(aStr, -1, c); }
 
@@ -689,7 +654,7 @@ public:
    * @return The offset of aStr from the beginning of the string,
    *         or -1 if not found.
    */
-  NS_HIDDEN_(PRInt32) RFind(const self_type& aStr, PRInt32 aOffset,
+  NS_HIDDEN_(int32_t) RFind(const self_type& aStr, int32_t aOffset,
                             ComparatorFunc c = DefaultComparator) const;
 
   /**
@@ -698,7 +663,7 @@ public:
    * @return The offset of aStr from the beginning of the string,
    *         or -1 if not found.
    */
-  NS_HIDDEN_(PRInt32) RFind(const char_type *aStr,
+  NS_HIDDEN_(int32_t) RFind(const char_type *aStr,
                             ComparatorFunc c = DefaultComparator) const;
 
   /**
@@ -709,7 +674,7 @@ public:
    * @return The offset of aStr from the beginning of the string,
    *         or -1 if not found.
    */
-  NS_HIDDEN_(PRInt32) RFind(const char_type *aStr, PRInt32 aLen,
+  NS_HIDDEN_(int32_t) RFind(const char_type *aStr, int32_t aLen,
                             ComparatorFunc c = DefaultComparator) const;
 
   /**
@@ -721,7 +686,7 @@ public:
    * @return The offset of the character from the beginning of the string,
    *         or -1 if not found.
    */
-  NS_HIDDEN_(PRInt32) FindChar(char_type aChar, PRUint32 aOffset = 0) const;
+  NS_HIDDEN_(int32_t) FindChar(char_type aChar, uint32_t aOffset = 0) const;
 
   /**
    * Search for the offset of the last occurrence of a character in a
@@ -730,12 +695,12 @@ public:
    * @return The offset of the character from the beginning of the string,
    *         or -1 if not found.
    */
-  NS_HIDDEN_(PRInt32) RFindChar(char_type aChar) const;
+  NS_HIDDEN_(int32_t) RFindChar(char_type aChar) const;
 
   /**
    * Append a string representation of a number.
    */
-  NS_HIDDEN_(void) AppendInt(int aInt, PRInt32 aRadix = 10);
+  NS_HIDDEN_(void) AppendInt(int aInt, int32_t aRadix = 10);
 
 #ifndef XPCOM_GLUE_AVOID_NSPR
   /**
@@ -744,8 +709,8 @@ public:
    * @param aErrorCode pointer to contain result code.
    * @param aRadix must be 10 or 16
    */
-  NS_HIDDEN_(PRInt32) ToInteger(nsresult* aErrorCode,
-                                PRUint32 aRadix = 10) const;
+  NS_HIDDEN_(int32_t) ToInteger(nsresult* aErrorCode,
+                                uint32_t aRadix = 10) const;
 #endif // XPCOM_GLUE_AVOID_NSPR
 
 protected:
@@ -851,7 +816,7 @@ public:
 
 protected:
   
-  nsString(const char_type* aData, size_type aLength, PRUint32 aFlags)
+  nsString(const char_type* aData, size_type aLength, uint32_t aFlags)
   {
     NS_StringContainerInit2(*this, aData, aLength, aFlags);
   }
@@ -912,7 +877,7 @@ public:
 
 protected:
   
-  nsCString(const char_type* aData, size_type aLength, PRUint32 aFlags)
+  nsCString(const char_type* aData, size_type aLength, uint32_t aFlags)
   {
     NS_CStringContainerInit2(*this, aData, aLength, aFlags);
   }
@@ -1013,7 +978,7 @@ public:
   }
 
   explicit
-  NS_ConvertASCIItoUTF16(const char* aData, PRUint32 aLength = PR_UINT32_MAX)
+  NS_ConvertASCIItoUTF16(const char* aData, uint32_t aLength = PR_UINT32_MAX)
   {
     NS_CStringToUTF16(nsDependentCString(aData, aLength),
                       NS_CSTRING_ENCODING_ASCII, *this);
@@ -1035,7 +1000,7 @@ public:
   }
 
   explicit
-  NS_ConvertUTF8toUTF16(const char* aData, PRUint32 aLength = PR_UINT32_MAX)
+  NS_ConvertUTF8toUTF16(const char* aData, uint32_t aLength = PR_UINT32_MAX)
   {
     NS_CStringToUTF16(nsDependentCString(aData, aLength),
                       NS_CSTRING_ENCODING_UTF8, *this);
@@ -1057,7 +1022,7 @@ public:
   }
 
   explicit
-  NS_ConvertUTF16toUTF8(const PRUnichar* aData, PRUint32 aLength = PR_UINT32_MAX)
+  NS_ConvertUTF16toUTF8(const PRUnichar* aData, uint32_t aLength = PR_UINT32_MAX)
   {
     NS_UTF16ToCString(nsDependentString(aData, aLength),
                       NS_CSTRING_ENCODING_UTF8, *this);
@@ -1079,7 +1044,7 @@ public:
   }
 
   explicit
-  NS_LossyConvertUTF16toASCII(const PRUnichar* aData, PRUint32 aLength = PR_UINT32_MAX)
+  NS_LossyConvertUTF16toASCII(const PRUnichar* aData, uint32_t aLength = PR_UINT32_MAX)
   {
     NS_UTF16ToCString(nsDependentString(aData, aLength),
                       NS_CSTRING_ENCODING_ASCII, *this);
@@ -1106,15 +1071,15 @@ private:
   MOZ_STATIC_ASSERT(sizeof(wchar_t) == 2, "size of wchar_t must be 2");
   #define NS_LL(s)                                L##s
 #endif
-  #define NS_MULTILINE_LITERAL_STRING(s)          nsDependentString(reinterpret_cast<const nsAString::char_type*>(s), PRUint32((sizeof(s)/2)-1))
-  #define NS_MULTILINE_LITERAL_STRING_INIT(n,s)   n(reinterpret_cast<const nsAString::char_type*>(s), PRUint32((sizeof(s)/2)-1))
-  #define NS_NAMED_MULTILINE_LITERAL_STRING(n,s)  const nsDependentString n(reinterpret_cast<const nsAString::char_type*>(s), PRUint32((sizeof(s)/2)-1))
+  #define NS_MULTILINE_LITERAL_STRING(s)          nsDependentString(reinterpret_cast<const nsAString::char_type*>(s), uint32_t((sizeof(s)/2)-1))
+  #define NS_MULTILINE_LITERAL_STRING_INIT(n,s)   n(reinterpret_cast<const nsAString::char_type*>(s), uint32_t((sizeof(s)/2)-1))
+  #define NS_NAMED_MULTILINE_LITERAL_STRING(n,s)  const nsDependentString n(reinterpret_cast<const nsAString::char_type*>(s), uint32_t((sizeof(s)/2)-1))
   typedef nsDependentString nsLiteralString;
 #else
   #define NS_LL(s)                                s
-  #define NS_MULTILINE_LITERAL_STRING(s)          NS_ConvertASCIItoUTF16(s, PRUint32(sizeof(s)-1))
-  #define NS_MULTILINE_LITERAL_STRING_INIT(n,s)   n(s, PRUint32(sizeof(s)-1))
-  #define NS_NAMED_MULTILINE_LITERAL_STRING(n,s)  const NS_ConvertASCIItoUTF16 n(s, PRUint32(sizeof(s)-1))
+  #define NS_MULTILINE_LITERAL_STRING(s)          NS_ConvertASCIItoUTF16(s, uint32_t(sizeof(s)-1))
+  #define NS_MULTILINE_LITERAL_STRING_INIT(n,s)   n(s, uint32_t(sizeof(s)-1))
+  #define NS_NAMED_MULTILINE_LITERAL_STRING(n,s)  const NS_ConvertASCIItoUTF16 n(s, uint32_t(sizeof(s)-1))
   typedef NS_ConvertASCIItoUTF16 nsLiteralString;
 #endif
 
@@ -1136,9 +1101,9 @@ MOZ_STATIC_ASSERT(PRUnichar(-1) > PRUnichar(0), "PRUnichar is by definition an u
 #define NS_LITERAL_STRING_INIT(n,s)               NS_MULTILINE_LITERAL_STRING_INIT(n, NS_LL(s))
 #define NS_NAMED_LITERAL_STRING(n,s)              NS_NAMED_MULTILINE_LITERAL_STRING(n, NS_LL(s))
 
-#define NS_LITERAL_CSTRING(s)                     static_cast<const nsDependentCString&>(nsDependentCString(s, PRUint32(sizeof(s)-1)))
-#define NS_LITERAL_CSTRING_INIT(n,s)              n(s, PRUint32(sizeof(s)-1))
-#define NS_NAMED_LITERAL_CSTRING(n,s)             const nsDependentCString n(s, PRUint32(sizeof(s)-1))
+#define NS_LITERAL_CSTRING(s)                     static_cast<const nsDependentCString&>(nsDependentCString(s, uint32_t(sizeof(s)-1)))
+#define NS_LITERAL_CSTRING_INIT(n,s)              n(s, uint32_t(sizeof(s)-1))
+#define NS_NAMED_LITERAL_CSTRING(n,s)             const nsDependentCString n(s, uint32_t(sizeof(s)-1))
 
 typedef nsDependentCString nsLiteralCString;
 
@@ -1162,7 +1127,7 @@ public:
   typedef PRUnichar char_type;
 
   nsGetterCopies(nsString& aStr)
-    : mString(aStr), mData(nsnull)
+    : mString(aStr), mData(nullptr)
   {}
 
   ~nsGetterCopies()
@@ -1192,7 +1157,7 @@ public:
   typedef char char_type;
 
   nsCGetterCopies(nsCString& aStr)
-    : mString(aStr), mData(nsnull)
+    : mString(aStr), mData(nullptr)
   {}
 
   ~nsCGetterCopies()
@@ -1237,7 +1202,7 @@ public:
     NS_StringContainerInit(*this);
   }
 
-  nsDependentSubstring(const char_type *aStart, PRUint32 aLength)
+  nsDependentSubstring(const char_type *aStart, uint32_t aLength)
   {
     NS_StringContainerInit2(*this, aStart, aLength,
                             NS_STRING_CONTAINER_INIT_DEPEND |
@@ -1245,11 +1210,11 @@ public:
   }
 
   nsDependentSubstring(const abstract_string_type& aStr,
-                       PRUint32 aStartPos);
+                       uint32_t aStartPos);
   nsDependentSubstring(const abstract_string_type& aStr,
-                       PRUint32 aStartPos, PRUint32 aLength);
+                       uint32_t aStartPos, uint32_t aLength);
 
-  void Rebind(const char_type *aStart, PRUint32 aLength)
+  void Rebind(const char_type *aStart, uint32_t aLength)
   {
     NS_StringContainerFinish(*this);
     NS_StringContainerInit2(*this, aStart, aLength,
@@ -1277,7 +1242,7 @@ public:
     NS_CStringContainerInit(*this);
   }
 
-  nsDependentCSubstring(const char_type *aStart, PRUint32 aLength)
+  nsDependentCSubstring(const char_type *aStart, uint32_t aLength)
   {
     NS_CStringContainerInit2(*this, aStart, aLength,
                              NS_CSTRING_CONTAINER_INIT_DEPEND |
@@ -1285,11 +1250,11 @@ public:
   }
 
   nsDependentCSubstring(const abstract_string_type& aStr,
-                        PRUint32 aStartPos);
+                        uint32_t aStartPos);
   nsDependentCSubstring(const abstract_string_type& aStr,
-                        PRUint32 aStartPos, PRUint32 aLength);
+                        uint32_t aStartPos, uint32_t aLength);
 
-  void Rebind(const char_type *aStart, PRUint32 aLength)
+  void Rebind(const char_type *aStart, uint32_t aLength)
   {
     NS_CStringContainerFinish(*this);
     NS_CStringContainerInit2(*this, aStart, aLength,
@@ -1308,13 +1273,13 @@ private:
 
 // PRUnichar
 inline const nsDependentSubstring
-Substring( const nsAString& str, PRUint32 startPos )
+Substring( const nsAString& str, uint32_t startPos )
 {
   return nsDependentSubstring(str, startPos);
 }
 
 inline const nsDependentSubstring
-Substring( const nsAString& str, PRUint32 startPos, PRUint32 length )
+Substring( const nsAString& str, uint32_t startPos, uint32_t length )
 {
   return nsDependentSubstring(str, startPos, length);
 }
@@ -1322,37 +1287,37 @@ Substring( const nsAString& str, PRUint32 startPos, PRUint32 length )
 inline const nsDependentSubstring
 Substring( const PRUnichar* start, const PRUnichar* end )
 {
-  NS_ABORT_IF_FALSE(PRUint32(end - start) == uintptr_t(end - start), "string too long");
-  return nsDependentSubstring(start, PRUint32(end - start));
+  NS_ABORT_IF_FALSE(uint32_t(end - start) == uintptr_t(end - start), "string too long");
+  return nsDependentSubstring(start, uint32_t(end - start));
 }
 
 inline const nsDependentSubstring
-Substring( const PRUnichar* start, PRUint32 length )
+Substring( const PRUnichar* start, uint32_t length )
 {
   return nsDependentSubstring(start, length);
 }
 
 inline const nsDependentSubstring
-StringHead( const nsAString& str, PRUint32 count )
+StringHead( const nsAString& str, uint32_t count )
 {
   return nsDependentSubstring(str, 0, count);
 }
 
 inline const nsDependentSubstring
-StringTail( const nsAString& str, PRUint32 count )
+StringTail( const nsAString& str, uint32_t count )
 {
   return nsDependentSubstring(str, str.Length() - count, count);
 }
 
 // char
 inline const nsDependentCSubstring
-Substring( const nsACString& str, PRUint32 startPos )
+Substring( const nsACString& str, uint32_t startPos )
 {
   return nsDependentCSubstring(str, startPos);
 }
 
 inline const nsDependentCSubstring
-Substring( const nsACString& str, PRUint32 startPos, PRUint32 length )
+Substring( const nsACString& str, uint32_t startPos, uint32_t length )
 {
   return nsDependentCSubstring(str, startPos, length);
 }
@@ -1361,25 +1326,25 @@ inline
 const nsDependentCSubstring
 Substring( const char* start, const char* end )
 {
-  NS_ABORT_IF_FALSE(PRUint32(end - start) == uintptr_t(end - start), "string too long");
-  return nsDependentCSubstring(start, PRUint32(end - start));
+  NS_ABORT_IF_FALSE(uint32_t(end - start) == uintptr_t(end - start), "string too long");
+  return nsDependentCSubstring(start, uint32_t(end - start));
 }
 
 inline
 const nsDependentCSubstring
-Substring( const char* start, PRUint32 length )
+Substring( const char* start, uint32_t length )
 {
   return nsDependentCSubstring(start, length);
 }
 
 inline const nsDependentCSubstring
-StringHead( const nsACString& str, PRUint32 count )
+StringHead( const nsACString& str, uint32_t count )
 {
   return nsDependentCSubstring(str, 0, count);
 }
 
 inline const nsDependentCSubstring
-StringTail( const nsACString& str, PRUint32 count )
+StringTail( const nsACString& str, uint32_t count )
 {
   return nsDependentCSubstring(str, str.Length() - count, count);
 }
@@ -1431,16 +1396,16 @@ CompressWhitespace(nsAString& aString);
  * Convert an ASCII string to all upper/lowercase (a-z,A-Z only). As a bonus,
  * returns the string length.
  */
-NS_HIDDEN_(PRUint32)
+NS_HIDDEN_(uint32_t)
 ToLowerCase(nsACString& aStr);
 
-NS_HIDDEN_(PRUint32)
+NS_HIDDEN_(uint32_t)
 ToUpperCase(nsACString& aStr);
 
-NS_HIDDEN_(PRUint32)
+NS_HIDDEN_(uint32_t)
 ToLowerCase(const nsACString& aSrc, nsACString& aDest);
 
-NS_HIDDEN_(PRUint32)
+NS_HIDDEN_(uint32_t)
 ToUpperCase(const nsACString& aSrc, nsACString& aDest);
 
 /**
