@@ -1,42 +1,9 @@
 #!/usr/bin/env python
 # header.py - Generate C++ header files from IDL.
 #
-# ***** BEGIN LICENSE BLOCK *****
-# Version: MPL 1.1/GPL 2.0/LGPL 2.1
-#
-# The contents of this file are subject to the Mozilla Public License Version
-# 1.1 (the "License"); you may not use this file except in compliance with
-# the License. You may obtain a copy of the License at
-# http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS IS" basis,
-# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
-# for the specific language governing rights and limitations under the
-# License.
-#
-# The Original Code is mozilla.org code.
-#
-# The Initial Developer of the Original Code is
-#   Mozilla Foundation.
-# Portions created by the Initial Developer are Copyright (C) 2008
-# the Initial Developer. All Rights Reserved.
-#
-# Contributor(s):
-#   Benjamin Smedberg <benjamin@smedbergs.us>
-#
-# Alternatively, the contents of this file may be used under the terms of
-# either of the GNU General Public License Version 2 or later (the "GPL"),
-# or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
-# in which case the provisions of the GPL or the LGPL are applicable instead
-# of those above. If you wish to allow use of your version of this file only
-# under the terms of either the GPL or the LGPL, and not to allow others to
-# use your version of this file under the terms of the MPL, indicate your
-# decision by deleting the provisions above and replace them with the notice
-# and other provisions required by the GPL or the LGPL. If you do not delete
-# the provisions above, a recipient may use your version of this file under
-# the terms of any one of the MPL, the GPL or the LGPL.
-#
-# ***** END LICENSE BLOCK *****
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 """Print a C++ header file for the IDL files specified on the command line"""
 
@@ -84,14 +51,12 @@ def attributeParamlist(a, getter):
     return ", ".join(l)
 
 def attributeAsNative(a, getter):
-        scriptable = a.isScriptable() and "NS_SCRIPTABLE " or ""
         deprecated = a.deprecated and "NS_DEPRECATED " or ""
-        params = {'scriptable': scriptable,
-                  'deprecated': deprecated,
+        params = {'deprecated': deprecated,
                   'returntype': attributeReturnType(a, 'NS_IMETHOD'),
                   'binaryname': attributeNativeName(a, getter),
                   'paramlist': attributeParamlist(a, getter)}
-        return "%(deprecated)s%(scriptable)s%(returntype)s %(binaryname)s(%(paramlist)s)" % params
+        return "%(deprecated)s%(returntype)s %(binaryname)s(%(paramlist)s)" % params
 
 def methodNativeName(m):
     return m.binaryname is not None and m.binaryname or firstCap(m.name)
@@ -109,12 +74,9 @@ def methodReturnType(m, macro):
         return macro
 
 def methodAsNative(m):
-    scriptable = m.isScriptable() and "NS_SCRIPTABLE " or ""
-
-    return "%s%s %s(%s)" % (scriptable,
-                            methodReturnType(m, 'NS_IMETHOD'),
-                            methodNativeName(m),
-                            paramlistAsNative(m))
+    return "%s %s(%s)" % (methodReturnType(m, 'NS_IMETHOD'),
+                          methodNativeName(m),
+                          paramlistAsNative(m))
 
 def paramlistAsNative(m, empty='void'):
     l = [paramAsNative(p) for p in m.params]
@@ -123,7 +85,7 @@ def paramlistAsNative(m, empty='void'):
         l.append("JSContext* cx")
 
     if m.optional_argc:
-        l.append('PRUint8 _argc')
+        l.append('uint8_t _argc')
 
     if not m.notxpcom and m.realtype.name != 'void':
         l.append(paramAsNative(xpidl.Param(paramtype='out',
@@ -139,14 +101,8 @@ def paramlistAsNative(m, empty='void'):
     return ", ".join(l)
 
 def paramAsNative(p):
-    if p.paramtype == 'in':
-        typeannotate = ''
-    else:
-        typeannotate = ' NS_%sPARAM' % p.paramtype.upper()
-
-    return "%s%s%s" % (p.nativeType(),
-                       p.name,
-                       typeannotate)
+    return "%s%s" % (p.nativeType(),
+                     p.name)
 
 def paramlistNames(m):
     names = [p.name for p in m.params]
@@ -182,6 +138,11 @@ jspubtd_include = """
 #include "jspubtd.h"
 """
 
+infallible_includes = """
+#include "mozilla/Assertions.h"
+#include "mozilla/Util.h"
+"""
+
 header_end = """/* For IDL files that don't want to include root IDL files. */
 #ifndef NS_NO_VTABLE
 #define NS_NO_VTABLE
@@ -214,6 +175,13 @@ def print_header(idl, fd, filename):
     if idl.needsJSTypes():
         fd.write(jspubtd_include)
 
+    # Include some extra files if any attributes are infallible.
+    for iface in [p for p in idl.productions if p.kind == 'interface']:
+        for attr in [m for m in iface.members if isinstance(m, xpidl.Attribute)]:
+            if attr.infallible:
+                fd.write(infallible_includes)
+                break
+
     fd.write('\n')
     fd.write(header_end)
 
@@ -324,6 +292,16 @@ iface_template_epilog = """/* End of implementation class template. */
 
 """
 
+attr_infallible_tmpl = """\
+  inline %(realtype)s%(nativename)s(%(args)s)
+  {
+    %(realtype)sresult;
+    mozilla::DebugOnly<nsresult> rv = %(nativename)s(%(argnames)s&result);
+    MOZ_ASSERT(NS_SUCCEEDED(rv));
+    return result;
+  }
+"""
+
 def write_interface(iface, fd):
     if iface.namemap is None:
         raise Exception("Interface was not resolved.")
@@ -354,6 +332,13 @@ def write_interface(iface, fd):
         fd.write("  /* %s */\n" % a.toIDL());
 
         fd.write("  %s = 0;\n" % attributeAsNative(a, True))
+        if a.infallible:
+            fd.write(attr_infallible_tmpl %
+                     {'realtype': a.realtype.nativeType('in'),
+                      'nativename': attributeNativeName(a, getter=True),
+                      'args': '' if not a.implicit_jscontext else 'JSContext* cx',
+                      'argnames': '' if not a.implicit_jscontext else 'cx, '})
+
         if not a.readonly:
             fd.write("  %s = 0;\n" % attributeAsNative(a, False))
         fd.write("\n")
@@ -390,8 +375,6 @@ def write_interface(iface, fd):
     if not foundcdata:
         fd.write("NS_NO_VTABLE ")
 
-    if iface.attributes.scriptable:
-        fd.write("NS_SCRIPTABLE ")
     if iface.attributes.deprecated:
         fd.write("MOZ_DEPRECATED ")
     fd.write(iface.name)
@@ -505,10 +488,9 @@ if __name__ == '__main__':
             print >>sys.stderr, "--regen useless without --cachedir"
         # Delete the lex/yacc files.  Ply is too stupid to regenerate them
         # properly
-        deadfiles = os.path.join(options.cachedir, "*.py*")
-        for filename in glob.glob(deadfiles):
-            print filename
-            os.remove(filename)
+        for fileglobs in [os.path.join(options.cachedir, f) for f in ["xpidllex.py*", "xpidlyacc.py*"]]:
+            for filename in glob.glob(fileglobs):
+                os.remove(filename)
 
     # Instantiate the parser.
     p = xpidl.IDLParser(outputdir=options.cachedir)
