@@ -1,44 +1,6 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2002
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Darin Fisher <darin@netscape.com>
- *   Brian Stell <bstell@ix.netcom.com>
- *   Frank Tang <ftang@netscape.com>
- *   Brendan Eich <brendan@mozilla.org>
- *   Sergei Dolgov <sergei_d@fi.fi.tartu.ee>
- *   Jungshik Shin <jshin@i18nl10n.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "xpcom-private.h"
 
@@ -105,7 +67,7 @@ using namespace mozilla;
 #endif
 
 static void
-isolatin1_to_utf16(const char **input, PRUint32 *inputLeft, PRUnichar **output, PRUint32 *outputLeft)
+isolatin1_to_utf16(const char **input, uint32_t *inputLeft, PRUnichar **output, uint32_t *outputLeft)
 {
     while (*inputLeft && *outputLeft) {
         **output = (unsigned char) **input;
@@ -117,7 +79,7 @@ isolatin1_to_utf16(const char **input, PRUint32 *inputLeft, PRUnichar **output,
 }
 
 static void
-utf16_to_isolatin1(const PRUnichar **input, PRUint32 *inputLeft, char **output, PRUint32 *outputLeft)
+utf16_to_isolatin1(const PRUnichar **input, uint32_t *inputLeft, char **output, uint32_t *outputLeft)
 {
     while (*inputLeft && *outputLeft) {
         **output = (unsigned char) **input;
@@ -291,10 +253,10 @@ public:
     nsNativeCharsetConverter();
    ~nsNativeCharsetConverter();
 
-    nsresult NativeToUnicode(const char      **input , PRUint32 *inputLeft,
-                             PRUnichar       **output, PRUint32 *outputLeft);
-    nsresult UnicodeToNative(const PRUnichar **input , PRUint32 *inputLeft,
-                             char            **output, PRUint32 *outputLeft);
+    nsresult NativeToUnicode(const char      **input , uint32_t *inputLeft,
+                             PRUnichar       **output, uint32_t *outputLeft);
+    nsresult UnicodeToNative(const PRUnichar **input , uint32_t *inputLeft,
+                             char            **output, uint32_t *outputLeft);
 
     static void GlobalInit();
     static void GlobalShutdown();
@@ -327,7 +289,7 @@ iconv_t nsNativeCharsetConverter::gUTF8ToNative    = INVALID_ICONV_T;
 iconv_t nsNativeCharsetConverter::gUnicodeToUTF8   = INVALID_ICONV_T;
 iconv_t nsNativeCharsetConverter::gUTF8ToUnicode   = INVALID_ICONV_T;
 #endif
-Mutex  *nsNativeCharsetConverter::gLock            = nsnull;
+Mutex  *nsNativeCharsetConverter::gLock            = nullptr;
 bool    nsNativeCharsetConverter::gInitialized     = false;
 bool    nsNativeCharsetConverter::gIsNativeUTF8    = false;
 
@@ -337,7 +299,7 @@ nsNativeCharsetConverter::LazyInit()
     const char  *blank_list[] = { "", NULL };
     const char **native_charset_list = blank_list;
     const char  *native_charset = nl_langinfo(CODESET);
-    if (native_charset == nsnull) {
+    if (native_charset == nullptr) {
         NS_ERROR("native charset is unknown");
         // fallback to ISO-8859-1
         native_charset_list = ISO_8859_1_NAMES;
@@ -417,7 +379,7 @@ nsNativeCharsetConverter::GlobalShutdown()
 {
     if (gLock) {
         delete gLock;
-        gLock = nsnull;
+        gLock = nullptr;
     }
 
     if (gNativeToUnicode != INVALID_ICONV_T) {
@@ -481,9 +443,9 @@ nsNativeCharsetConverter::~nsNativeCharsetConverter()
 
 nsresult
 nsNativeCharsetConverter::NativeToUnicode(const char **input,
-                                          PRUint32    *inputLeft,
+                                          uint32_t    *inputLeft,
                                           PRUnichar  **output,
-                                          PRUint32    *outputLeft)
+                                          uint32_t    *outputLeft)
 {
     size_t res = 0;
     size_t inLeft = (size_t) *inputLeft;
@@ -553,9 +515,9 @@ nsNativeCharsetConverter::NativeToUnicode(const char **input,
 
 nsresult
 nsNativeCharsetConverter::UnicodeToNative(const PRUnichar **input,
-                                          PRUint32         *inputLeft,
+                                          uint32_t         *inputLeft,
                                           char            **output,
-                                          PRUint32         *outputLeft)
+                                          uint32_t         *outputLeft)
 {
     size_t res = 0;
     size_t inLeft = (size_t) *inputLeft * 2;
@@ -654,10 +616,10 @@ class nsNativeCharsetConverter
 public:
     nsNativeCharsetConverter();
 
-    nsresult NativeToUnicode(const char      **input , PRUint32 *inputLeft,
-                             PRUnichar       **output, PRUint32 *outputLeft);
-    nsresult UnicodeToNative(const PRUnichar **input , PRUint32 *inputLeft,
-                             char            **output, PRUint32 *outputLeft);
+    nsresult NativeToUnicode(const char      **input , uint32_t *inputLeft,
+                             PRUnichar       **output, uint32_t *outputLeft);
+    nsresult UnicodeToNative(const PRUnichar **input , uint32_t *inputLeft,
+                             char            **output, uint32_t *outputLeft);
 
     static void GlobalInit();
     static void GlobalShutdown() { }
@@ -713,9 +675,9 @@ nsNativeCharsetConverter::GlobalInit()
 
 nsresult
 nsNativeCharsetConverter::NativeToUnicode(const char **input,
-                                          PRUint32    *inputLeft,
+                                          uint32_t    *inputLeft,
                                           PRUnichar  **output,
-                                          PRUint32    *outputLeft)
+                                          uint32_t    *outputLeft)
 {
     if (gWCharIsUnicode) {
         int incr;
@@ -754,9 +716,9 @@ nsNativeCharsetConverter::NativeToUnicode(const char **input,
 
 nsresult
 nsNativeCharsetConverter::UnicodeToNative(const PRUnichar **input,
-                                          PRUint32         *inputLeft,
+                                          uint32_t         *inputLeft,
                                           char            **output,
-                                          PRUint32         *outputLeft)
+                                          uint32_t         *outputLeft)
 {
     if (gWCharIsUnicode) {
         int incr;
@@ -774,7 +736,7 @@ nsNativeCharsetConverter::UnicodeToNative(const PRUnichar **input,
                 incr = 1;
             }
             // most likely we're dead anyways if this assertion should fire
-            NS_ASSERTION(PRUint32(incr) <= *outputLeft, "wrote beyond end of string");
+            NS_ASSERTION(uint32_t(incr) <= *outputLeft, "wrote beyond end of string");
             (*output) += incr;
             (*outputLeft) -= incr;
             (*input)++;
@@ -808,7 +770,7 @@ NS_CopyNativeToUnicode(const nsACString &input, nsAString &output)
 {
     output.Truncate();
 
-    PRUint32 inputLen = input.Length();
+    uint32_t inputLen = input.Length();
 
     nsACString::const_iterator iter;
     input.BeginReading(iter);
@@ -826,10 +788,10 @@ NS_CopyNativeToUnicode(const nsACString &input, nsAString &output)
     output.BeginWriting(out_iter);
 
     PRUnichar *result = out_iter.get();
-    PRUint32 resultLeft = inputLen;
+    uint32_t resultLeft = inputLen;
 
     const char *buf = iter.get();
-    PRUint32 bufLeft = inputLen;
+    uint32_t bufLeft = inputLen;
 
     nsNativeCharsetConverter conv;
     nsresult rv = conv.NativeToUnicode(&buf, &bufLeft, &result, &resultLeft);
@@ -855,10 +817,10 @@ NS_CopyUnicodeToNative(const nsAString &input, nsACString &output)
     nsNativeCharsetConverter conv;
 
     const PRUnichar *buf = iter.get();
-    PRUint32 bufLeft = Distance(iter, end);
+    uint32_t bufLeft = Distance(iter, end);
     while (bufLeft) {
         char *p = temp;
-        PRUint32 tempLeft = sizeof(temp);
+        uint32_t tempLeft = sizeof(temp);
 
         nsresult rv = conv.UnicodeToNative(&buf, &bufLeft, &p, &tempLeft);
         if (NS_FAILED(rv)) return rv;
@@ -909,7 +871,7 @@ NS_ShutdownNativeCharsetUtils()
 nsresult
 NS_CopyNativeToUnicode(const nsACString &input, nsAString &output)
 {
-    PRUint32 inputLen = input.Length();
+    uint32_t inputLen = input.Length();
 
     nsACString::const_iterator iter;
     input.BeginReading(iter);
@@ -917,7 +879,7 @@ NS_CopyNativeToUnicode(const nsACString &input, nsAString &output)
     const char *buf = iter.get();
 
     // determine length of result
-    PRUint32 resultLen = 0;
+    uint32_t resultLen = 0;
     int n = ::MultiByteToWideChar(CP_ACP, 0, buf, inputLen, NULL, 0);
     if (n > 0)
         resultLen += n;
@@ -939,7 +901,7 @@ NS_CopyNativeToUnicode(const nsACString &input, nsAString &output)
 nsresult
 NS_CopyUnicodeToNative(const nsAString  &input, nsACString &output)
 {
-    PRUint32 inputLen = input.Length();
+    uint32_t inputLen = input.Length();
 
     nsAString::const_iterator iter;
     input.BeginReading(iter);
@@ -947,7 +909,7 @@ NS_CopyUnicodeToNative(const nsAString  &input, nsACString &output)
     const PRUnichar *buf = iter.get();
 
     // determine length of result
-    PRUint32 resultLen = 0;
+    uint32_t resultLen = 0;
 
     int n = ::WideCharToMultiByte(CP_ACP, 0, buf, inputLen, NULL, 0, NULL, NULL);
     if (n > 0)
@@ -973,13 +935,13 @@ NS_CopyUnicodeToNative(const nsAString  &input, nsACString &output)
 }
 
 // moved from widget/windows/nsToolkit.cpp
-PRInt32 
+int32_t 
 NS_ConvertAtoW(const char *aStrInA, int aBufferSize, PRUnichar *aStrOutW)
 {
     return MultiByteToWideChar(CP_ACP, 0, aStrInA, -1, aStrOutW, aBufferSize);
 }
 
-PRInt32 
+int32_t 
 NS_ConvertWtoA(const PRUnichar *aStrInW, int aBufferSizeOut,
                char *aStrOutA, const char *aDefault)
 {
@@ -1026,14 +988,14 @@ static UconvObject UnicodeConverter = NULL;
 nsresult
 NS_CopyNativeToUnicode(const nsACString &input, nsAString  &output)
 {
-    PRUint32 inputLen = input.Length();
+    uint32_t inputLen = input.Length();
 
     nsACString::const_iterator iter;
     input.BeginReading(iter);
     const char *inputStr = iter.get();
 
     // determine length of result
-    PRUint32 resultLen = inputLen;
+    uint32_t resultLen = inputLen;
     if (!EnsureStringLength(output, resultLen))
         return NS_ERROR_OUT_OF_MEMORY;
 
