@@ -1,39 +1,7 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 200
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsWindowDataSource.h"
 #include "nsIXULWindow.h"
@@ -50,15 +18,15 @@
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsIDocShell.h"
 
-PRUint32 nsWindowDataSource::windowCount = 0;
+uint32_t nsWindowDataSource::windowCount = 0;
 
-nsIRDFResource* nsWindowDataSource::kNC_Name = nsnull;
-nsIRDFResource* nsWindowDataSource::kNC_WindowRoot = nsnull;
-nsIRDFResource* nsWindowDataSource::kNC_KeyIndex = nsnull;
+nsIRDFResource* nsWindowDataSource::kNC_Name = nullptr;
+nsIRDFResource* nsWindowDataSource::kNC_WindowRoot = nullptr;
+nsIRDFResource* nsWindowDataSource::kNC_KeyIndex = nullptr;
 
-nsIRDFService*  nsWindowDataSource::gRDFService = nsnull;
+nsIRDFService*  nsWindowDataSource::gRDFService = nullptr;
 
-PRUint32 nsWindowDataSource::gRefCnt = 0;
+uint32_t nsWindowDataSource::gRefCnt = 0;
 
 static const char kURINC_WindowRoot[] = "NC:WindowMediatorRoot";
 
@@ -121,8 +89,8 @@ nsWindowDataSource::Observe(nsISupports *aSubject, const char* aTopic, const PRU
     if (strcmp(aTopic, NS_XPCOM_SHUTDOWN_OBSERVER_ID) == 0) {
         // release these objects so that they release their reference
         // to us
-        mContainer = nsnull;
-        mInner = nsnull;
+        mContainer = nullptr;
+        mInner = nullptr;
     }
 
     return NS_OK;
@@ -245,7 +213,7 @@ nsWindowDataSource::OnCloseWindow(nsIXULWindow *window)
     
     // update RDF and keyindex - from this point forward we'll ignore
     // errors, because they just indicate some kind of RDF inconsistency
-    PRInt32 winIndex = -1;
+    int32_t winIndex = -1;
     rv = mContainer->IndexOf(resource, &winIndex);
         
     if (NS_FAILED(rv))
@@ -271,7 +239,7 @@ nsWindowDataSource::OnCloseWindow(nsIXULWindow *window)
         if (NS_FAILED(rv))
             continue;
 
-        PRInt32 currentIndex = -1;
+        int32_t currentIndex = -1;
         mContainer->IndexOf(windowResource, &currentIndex);
 
         // can skip updating windows with lower indexes
@@ -340,7 +308,7 @@ nsWindowDataSource::GetWindowForResource(const char *aResourceString,
                              getter_AddRefs(windowResource));
 
     // now reverse-lookup in the hashtable
-    findWindowClosure closure = { windowResource.get(), nsnull };
+    findWindowClosure closure = { windowResource.get(), nullptr };
     mWindowResources.Enumerate(findWindow, (void*)&closure);
     if (closure.resultWindow) {
 
@@ -392,7 +360,7 @@ NS_IMETHODIMP nsWindowDataSource::GetTarget(nsIRDFResource *aSource, nsIRDFResou
     // special case kNC_KeyIndex before we forward to mInner
     if (aProperty == kNC_KeyIndex) {
 
-        PRInt32 theIndex = 0;
+        int32_t theIndex = 0;
         nsresult rv = mContainer->IndexOf(aSource, &theIndex);
         if (NS_FAILED(rv)) return rv;
 
